dnl If no prefix command line option was given add the default prefix to the
dnl configure command line options, so that it is propagated to configure
dnl scripts in subdirectories. 
AC_DEFUN(AC_PROPAGATE_PREFIX_DEFAULT,
[
if test "${prefix}" = NONE && test -n "$ac_default_prefix"; then
  ac_configure_args="--prefix=${ac_default_prefix} ${ac_configure_args}"
fi
])

dnl ------------------------------------------------------------------------
dnl find a file or one of more files in a list of directories
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(AC_FIND_FILE,
[
$3=NO
for i in $2;
do
  for j in $1;
  do
    if test -r "$i/$j"; then
      $3=$i
      break 2
    fi
  done
done
])

dnl ------------------------------------------------------------------------
dnl Check C compiler
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(AC_CHECK_COMPILERS,
[
  dnl this is somehow a fat lie, but prevents other macros from double checking
  AC_PROVIDE([AC_PROG_CC])
  AC_PROVIDE([AC_PROG_CPP])
  AC_ARG_ENABLE(debug,[  --enable-debug          creates debugging code [default=no]],
  [ 
   if test $enableval = "no"; dnl 
     then ac_use_debug_code="no"
     else ac_use_debug_code="yes"
   fi
  ], [ac_use_debug_code="no"])

  AC_ARG_ENABLE(strict,[  --enable-strict         compiles with strict compiler options (may not work!)],
   [ 
    if test $enableval = "no"; then 
         ac_use_strict_options="no"
       else 
         ac_use_strict_options="yes"
    fi
   ], [ac_use_strict_options="no"])

dnl this was AC_PROG_CC. I had to include it manualy, since I had to patch it
  AC_MSG_CHECKING(for a C-Compiler)
  dnl if there is one, print out. if not, don't matter
  AC_MSG_RESULT($CC) 
 
  if test -z "$CC"; then AC_CHECK_PROG(CC, gcc, gcc) fi
  if test -z "$CC"; then AC_CHECK_PROG(CC, cc, cc, , , /usr/ucb/cc) fi
  if test -z "$CC"; then AC_CHECK_PROG(CC, xlc, xlc) fi
  test -z "$CC" && AC_MSG_ERROR([no acceptable cc found in \$PATH])

  AC_PROG_CC_WORKS
  AC_PROG_CC_GNU

  if test $ac_cv_prog_gcc = yes; then
    GCC=yes
  else
    GCC=
  fi

  if test -z "$CFLAGS"; then
    if test "$ac_use_debug_code" = "yes"; then
      AC_PROG_CC_G
      if test $ac_cv_prog_cc_g = yes; then
        CFLAGS="-g"
      fi
    else
      if test "$GCC" = "yes"; then
        CFLAGS="-O2"
      else
        CFLAGS=""
      fi
    fi

    if test "$GCC" = "yes"; then
     if test "$ac_use_strict_options" = "yes"; then
	CFLAGS="$CFLAGS -W -ansi -pedantic"     
     fi
    fi

  fi

  case "$host" in 
  *-*-sysv4.2uw*) CFLAGS="$CFLAGS -D_UNIXWARE";;
  esac

  if test -z "$LDFLAGS" && test "$ac_use_debug_code" = "no" && test "$GCC" = "yes"; then
     LDFLAGS="-s"
  fi


dnl this is AC_PROG_CPP. I had to include it here, since autoconf checks
dnl dependecies between AC_PROG_CPP and AC_PROG_CC (or is it automake?)

  AC_MSG_CHECKING(how to run the C preprocessor)
  # On Suns, sometimes $CPP names a directory.
  if test -n "$CPP" && test -d "$CPP"; then
    CPP=
  fi
  if test -z "$CPP"; then
  AC_CACHE_VAL(ac_cv_prog_CPP,
  [ # This must be in double quotes, not single quotes, because CPP may get
    # substituted into the Makefile and "${CC-cc}" will confuse make.
    CPP="${CC-cc} -E"
    # On the NeXT, cc -E runs the code through the compiler's parser,
    # not just through cpp.
    dnl Use a header file that comes with gcc, so configuring glibc    
    dnl with a fresh cross-compiler works.
    AC_TRY_CPP([#include <assert.h>
    Syntax Error], ,
    CPP="${CC-cc} -E -traditional-cpp"
    AC_TRY_CPP([#include <assert.h>
    Syntax Error], , CPP=/lib/cpp))
    ac_cv_prog_CPP="$CPP"])dnl
    CPP="$ac_cv_prog_CPP"
  else
    ac_cv_prog_CPP="$CPP"
  fi
  AC_MSG_RESULT($CPP)
  AC_SUBST(CPP)dnl
    
])

dnl ------------------------------------------------------------------------
dnl Check C compiler flags
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(AC_CHECK_C_COMPILER_FLAG,
[
AC_REQUIRE([AC_CHECK_COMPILERS])
AC_MSG_CHECKING(whether $CC supports -$1)
flag_name=`echo $1 | sed 'y%.=/+-%___p_%'`
AC_CACHE_VAL(ac_cv_prog_cc_$flag_name,
[
echo 'int main() { return 0; }' >conftest.c
eval "ac_cv_prog_cc_$flag_name=no"
if test -z "`$CC -$1 -c conftest.c 2>&1`"; then
  if test -z "`$CC -$1 -o conftest conftest.o 2>&1`"; then
    eval "ac_cv_prog_cc_$flag_name=yes"
  fi
fi
rm -f conftest*
])
if eval "test \"`echo '$ac_cv_prog_cc_'$flag_name`\" = yes"; then
 AC_MSG_RESULT(yes)
 :
 $2
else
 AC_MSG_RESULT(no)
 :
 $3
fi
])

dnl ------------------------------------------------------------------------
dnl Check the linker
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(AC_CHECK_LINKER_GCC,
[
AC_REQUIRE([AC_CHECK_COMPILERS])

if test "$ac_cv_prog_gcc" = yes; then
  AM_PROG_LD
else
  ac_cv_prog_gnu_ld="no"
fi
  
])

dnl ------------------------------------------------------------------------
dnl Check if the linker supports rpath
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(AC_CHECK_CC_RPATH,
[
AC_MSG_CHECKING([if the linker accepts run time library search paths])

AC_REQUIRE([AC_CANONICAL_HOST])
AC_REQUIRE([AC_CANONICAL_BUILD])
AC_REQUIRE([AC_CHECK_LINKER_GCC])

AC_CACHE_VAL(ac_cv_prog_cc_rpath,
[
ac_ldflags_rpath=
ac_ldflags_rpath_with_ifs="yes"

dnl Check if linker supports the rpath feature
if test "$ac_cv_prog_gnu_ld" = yes; then
  have_cc_rpath=yes
  ac_ldflags_rpath="-rpath"
else
  case "${host}" in
    sparc-sun-solaris*) have_cc_rpath=yes
                        ac_ldflags_rpath="-R"
                        ac_ldflags_rpath_with_ifs="no";;

    hppa*-hp-hpux*)     have_cc_rpath=yes
                        ac_ldflags_rpath="+b";;

    *)                  have_cc_rpath=no;;
  esac
fi

if test "$have_cc_rpath" = yes; then
  ac_cflags_ld=
  ac_cflags_ld_ifs=","

  dnl Set compiler option that passes options to the linker
  if test "$ac_cv_prog_gcc" = yes; then
    ac_cflags_ld="-Wl"
  else
    case "${host}" in
      sparc-sun-solaris*) ac_cflags_ld=""
                          ac_cflags_ld_ifs="";;

      *)                  ac_cflags_ld="-Wl";;
    esac
  fi

  ac_ldflags_rpath="$ac_cflags_ld$ac_cflags_ld_ifs$ac_ldflags_rpath"
else
 ac_cv_prog_cc_rpath="have_cc_rpath=no"
fi
])

eval "$ac_cv_prog_cc_rpath"

if test "$have_cc_rpath" = yes; then
  ac_cv_prog_cc_rpath="have_cc_rpath=$have_cc_rpath \
    ac_ldflags_rpath_with_ifs=$ac_ldflags_rpath_with_ifs \
    ac_ldflags_rpath=$ac_ldflags_rpath"
fi

AC_MSG_RESULT($have_cc_rpath)

])

dnl ------------------------------------------------------------------------
dnl Set compiler rpath option
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(AC_SET_CC_RPATH,
[
AC_REQUIRE([AC_CHECK_CC_RPATH])

$1=""

if test "$have_cc_rpath" = yes; then
  if test "$ac_ldflags_rpath_with_ifs" = yes; then
    $1="$ac_ldflags_rpath $2"
  else
    $1="$ac_ldflags_rpath$2"
  fi
fi

])

dnl ------------------------------------------------------------------------
dnl Check for ar
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(AC_PROG_AR,
[
AC_PROVIDE([AC_PROG_AR])

AC_CHECK_PROG(AR, ar, ar, NONE)
if test "$AR" = NONE; then
  AC_MSG_ERROR([Cannot find \'ar\'])
fi
])

dnl The macros AM_PROG_LD and AM_PROG_LD_GNU have been copied from libtool.m4
dnl to avoid the definition of the the configure variable LIBTOOL in the
dnl Makefile.in. Otherwise make install would fail since it expects libtool
dnl to be setup properly. But there is actually no need to use libtool here.
dnl
AC_DEFUN(AM_PROG_LD,
[AC_ARG_WITH(gnu-ld,
[  --with-gnu-ld           assume the C compiler uses GNU ld [default=no]],
test "$withval" = no || with_gnu_ld=yes, with_gnu_ld=no)
AC_REQUIRE([AC_PROG_CC])dnl
AC_REQUIRE([AC_CANONICAL_HOST])dnl
AC_REQUIRE([AC_CANONICAL_BUILD])dnl
ac_prog=ld
if test "$ac_cv_prog_gcc" = yes; then
  # Check if gcc -print-prog-name=ld gives a path.
  AC_MSG_CHECKING([for ld used by GCC])
  ac_prog=`($CC -print-prog-name=ld) 2>&5`
  case "$ac_prog" in
    # Accept absolute paths.
changequote(,)dnl
    /* | [A-Za-z]:/*)
      # Canonicalize the path of ld
      re_direlt='/[^/][^/]*/\.\./'
      sub_uncdrive='s%^\([A-Za-z]\):/%//\1/%'
changequote([,])dnl
      while echo $ac_prog | grep "$re_direlt" > /dev/null 2>&1; do
        ac_prog=`echo $ac_prog| sed "s%$re_direlt%/%"`
      done
      case "$host_os" in
      cygwin*)
        # Convert to a UNC path for cygwin
        test -z "$LD" && LD=`echo X$ac_prog | $Xsed -e "$sub_uncdrive"`
        ;;
      *)
        test -z "$LD" && LD="$ac_prog"
        ;;
      esac
      ;;
    ##
    ## FIXME:  The code fails later on if we try to use an $LD with
    ##         '\\' path separators.
    ##
changequote(,)dnl
    [A-Za-z]:[\\]*)
      # Canonicalize the path of ld
      re_direlt='\\[^\\][^\\]*\\\.\.\(\\\)'
      sub_uncdrive='s%^\([A-Za-z]\):\\%//\1/%'
changequote([,])dnl
      sub_uncdir='s%\\%/%g'
      while echo $ac_prog | grep "$re_direlt" > /dev/null 2>&1; do
        ac_prog=`echo $ac_prog| sed "s%$re_direlt%\1%"`
      done
      case "$host_os" in
      cygwin*)
        # Convert to a UNC path for cygwin
        test -z "$LD" && LD=`echo X$ac_prog | sed -e 's%^X%%' -e "$sub_uncdrive"
 -e "$sub_uncdir"`
        ;;
      *)
        test -z "$LD" && LD="$ac_prog"
        ;;
      esac
      ;;
  "")
    # If it fails, then pretend we aren't using GCC.
    ac_prog=ld
    ;;
  *)
    # If it is relative, then search for the first ld in PATH.
    with_gnu_ld=unknown
    ;;
  esac
elif test "$with_gnu_ld" = yes; then
  AC_MSG_CHECKING([for GNU ld])
else
  AC_MSG_CHECKING([for non-GNU ld])
fi
AC_CACHE_VAL(ac_cv_path_LD,
[if test -z "$LD"; then
  IFS="${IFS=   }"; ac_save_ifs="$IFS"; IFS="${IFS}:"
  for ac_dir in $PATH; do
    test -z "$ac_dir" && ac_dir=.
    if test -f "$ac_dir/$ac_prog"; then
      ac_cv_path_LD="$ac_dir/$ac_prog"
      # Check to see if the program is GNU ld.  I'd rather use --version,
      # but apparently some GNU ld's only accept -v.
      # Break only if it was the GNU/non-GNU ld that we prefer.
      if "$ac_cv_path_LD" -v 2>&1 < /dev/null | egrep '(GNU|with BFD)' > /dev/null; then
        test "$with_gnu_ld" != no && break
      else
        test "$with_gnu_ld" != yes && break
      fi
    fi
  done
  IFS="$ac_save_ifs"
else
  ac_cv_path_LD="$LD" # Let the user override the test with a path.
fi])
LD="$ac_cv_path_LD"
if test -n "$LD"; then
  AC_MSG_RESULT($LD)
else
  AC_MSG_RESULT(no)
fi
test -z "$LD" && AC_MSG_ERROR([no acceptable ld found in \$PATH])
AC_SUBST(LD)
AM_PROG_LD_GNU
])

AC_DEFUN(AM_PROG_LD_GNU,
[AC_CACHE_CHECK([if the linker ($LD) is GNU ld], ac_cv_prog_gnu_ld,
[# I'd rather use --version here, but apparently some GNU ld's only accept -v.
if $LD -v 2>&1 </dev/null | egrep '(GNU|with BFD)' 1>&5; then
  ac_cv_prog_gnu_ld=yes
else
  ac_cv_prog_gnu_ld=no
fi])
])

dnl ------------------------------------------------------------------------
dnl Determine a Fortran 77 compiler to use. Order is: f77 g77 f2c !!!
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(AC_PIPE_F77,
[AC_BEFORE([$0], [AC_PROG_CPP])dnl
if test -z "$F77"; then
  AC_CHECK_PROGS(F77, f77 g77 f2c)
    test -z "$F77" && AC_MSG_ERROR([no acceptable Fortran 77 compiler
found in \$PATH])
fi

AC_PROG_F77_WORKS
AC_PROG_F77_GNU

if test $ac_cv_prog_g77 = yes; then
  G77=yes
dnl Check whether -g works, even if FFLAGS is set, in case the package
dnl plays around with FFLAGS (such as to build both debugging and
dnl normal versions of a library), tasteless as that idea is.
  ac_test_FFLAGS="${FFLAGS+set}"
  ac_save_FFLAGS="$FFLAGS"
  FFLAGS=
  AC_PROG_F77_G
  if test "$ac_test_FFLAGS" = set; then
    FFLAGS="$ac_save_FFLAGS"
  elif test $ac_cv_prog_f77_g = yes; then
    FFLAGS="-g -O2"
  else
    FFLAGS="-O2"
  fi
else
  G77=
  test "${FFLAGS+set}" = set || FFLAGS="-O"
fi
])

dnl ------------------------------------------------------------------------
dnl check for directories
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(PIPE_FIND_PATH,
[
   AC_MSG_CHECKING([for $1])
   AC_CACHE_VAL(pipe_cv_path_$1,
   [
     pipe_cv_path_$1="NONE"
     if test -n "$$2"; then
        pipe_cv_path_$1="$$2";
     else
        dirs="$3"
        pipe_save_IFS=$IFS
        IFS=':'
        for dir in $PATH; do
          dirs="$dirs $dir"
        done
        IFS=$pipe_save_IFS

        for dir in $dirs; do
          if test -x "$dir/$1"; then
            if test -n "$5"
            then
              evalstr="$dir/$1 $5 2>&1 "
              if eval $evalstr; then
                pipe_cv_path_$1="$dir/$1"
                break
              fi
            else
                pipe_cv_path_$1="$dir/$1"
                break
            fi
          fi
        done

     fi
         
   ])

   if test -z "$pipe_cv_path_$1" || test "$pipe_cv_path_$1" = "NONE"; then
      AC_MSG_RESULT(not found)
      $4
   else 
     AC_MSG_RESULT($pipe_cv_path_$1)
     $2=$pipe_cv_path_$1
   fi
])

dnl ------------------------------------------------------------------------
dnl Create library aliases
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(PIPE_CREATE_LIBS_ALIASES,
[
  LIB_PIPECCD='-lccd'
  LIB_PIPEREBIN='-lrebin'
  LIB_PIPEFORS='-lfors'
  LIB_PIPEALL='$(LIB_PIPECCD) $(LIB_PIPEREBIN) $(LIB_PIPEFORS)'

  AC_SUBST(LIB_PIPECCD)
  AC_SUBST(LIB_PIPEREBIN)
  AC_SUBST(LIB_PIPEFORS)
  AC_SUBST(LIB_PIPEALL)
])

dnl ------------------------------------------------------------------------
dnl Build default prefix for PIPE pipeline installation
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(PIPE_PREFIX_DEFAULT,
[
AC_PREFIX_DEFAULT($1)

if test "${prefix}" = NONE; then
  if test -n "$2"; then
    ac_default_prefix=${ac_default_prefix}/$2
  fi
fi
])

dnl ------------------------------------------------------------------------
dnl Setup PIPE pipeline basic directory tree
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(PIPE_BASE_PATH,
[
AC_MSG_CHECKING([for instrument pipeline])

if test "${prefix}" != NONE; then
  pipe_includes=${prefix}/include
  ac_pipe_includes=$prefix/include

  if test "${exec_prefix}" != NONE; then
    pipe_libraries=${exec_prefix}/lib
    ac_pipe_libraries=$exec_prefix/lib
  else
    pipe_libraries=${prefix}/lib
    ac_pipe_libraries=$prefix/lib
  fi
else
  ac_pipe_includes=
  ac_pipe_libraries=
  pipe_includes=""
  pipe_libraries=""
fi

AC_CACHE_VAL(ac_cv_have_pipe,
[
if test -z "$1"; then
  pipe_incdirs=""
  test -n "$PIPE_HOME" && \
       pipe_incdirs="$PIPE_HOME/fors/include $pipe_incdirs"
  pipe_incdirs="$ac_pipe_includes $pipe_incdirs"

  AC_FIND_FILE(rebin.h, $pipe_incdirs, pipe_incdir)
  ac_pipe_includes="$pipe_incdir"

  if test -n "$ac_pipe_includes" && test ! -r "$ac_pipe_includes/rebin.h"; then
    AC_MSG_ERROR([No headers were found beneath the given directory prefix. Please check!])
  fi

  pipe_libdirs=""
  test -n "$PIPE_HOME" && pipe_libdirs="$PIPE_HOME/fors/lib $pipe_libdirs"
  pipe_libdirs="$ac_pipe_libraries $pipe_libdirs"

  AC_FIND_FILE(libfors.a, $pipe_libdirs, pipe_libdir)
  ac_pipe_libraries="$pipe_libdir"

  if test -n "$ac_pipe_libraries" && test ! -r "$ac_pipe_libraries/libfors.a"; then
    AC_MSG_ERROR([No libraries were found beneath the given directory prefix. Please check!])
  fi

  if test "$ac_pipe_includes" = NO || test "$ac_pipe_libraries" = NO; then
    ac_cv_have_pipe="have_pipe=no"
  else
    ac_cv_have_pipe="have_pipe=yes ac_pipe_includes=$ac_pipe_includes \
      ac_pipe_libraries=$ac_pipe_libraries"
  fi
else
  ac_cv_have_pipe="have_pipe=no"
fi
])

eval "$ac_cv_have_pipe"

if test "$have_pipe" != "yes"; then
  if test "${prefix}" = NONE; then
    ac_pipe_prefix="$ac_default_prefix"
  else
    ac_pipe_prefix="$prefix"
  fi

  if test "$exec_prefix" = NONE; then
    ac_pipe_exec_prefix="$ac_pipe_prefix"
    AC_MSG_RESULT([will be installed in $ac_pipe_prefix])
  else
    ac_pipe_exec_prefix="$exec_prefix"
    AC_MSG_RESULT([will be installed in $ac_pipe_prefix and $ac_pipe_exec_prefix])
  fi

  pipe_includes="${ac_pipe_prefix}/include"
  pipe_libraries="${ac_pipe_exec_prefix}/lib"

  PIPE_INCLUDES='-I$(top_srcdir)/pipe/incl'
  PIPE_LDFLAGS='-L$(top_srcdir)/pipe/libsrc'
  XMOS_INCLUDES='-I$(top_srcdir)/xmos/incl -I$(top_srcdir)/xmos/fors/incl'
  XMOS_LDFLAGS='-L$(top_srcdir)/xmos/libsrc -L$(top_srcdir)/xmos/fors/libsrc'
else
  ac_cv_have_pipe="have_pipe=yes ac_pipe_includes=$ac_pipe_includes \
    ac_pipe_libraries=$ac_pipe_libraries"
  AC_MSG_RESULT([headers $ac_pipe_includes, libraries $ac_pipe_libraries])

  pipe_includes="$ac_pipe_includes"
  pipe_libraries="$ac_pipe_libraries"

  PIPE_INCLUDES="-I$pipe_includes"
  PIPE_LDFLAGS="-L$pipe_libraries"
fi

AC_SUBST(pipe_includes)
AC_SUBST(pipe_libraries)

AC_SUBST(PIPE_INCLUDES)
AC_SUBST(PIPE_LDFLAGS)

AC_SUBST(XMOS_INCLUDES)
AC_SUBST(XMOS_LDFLAGS)

all_includes="$PIPE_INCLUDES $all_includes"
all_libraries="$PIPE_LDFLAGS $all_libraries"

AC_REQUIRE([PIPE_CHECK_EXTRA_LIBS])

AC_SUBST(all_includes)
AC_SUBST(all_libraries)

])

dnl ------------------------------------------------------------------------
dnl Create pipeline auxiliary directory tree
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(PIPE_PACKAGE_PATH,
[
AC_MSG_CHECKING([for PIPELINE paths])
pipe_result=""

AC_CACHE_VAL(ac_cv_all_paths,
[
  if test -z "$pipe_bindir"; then
    pipe_bindir='\${prefix}/exec'
  fi

  if test -z "$pipe_extrabindir"; then
    pipe_extrabindir='\${prefix}/bin'
  fi

  if test -z "$pipe_contextdir"; then
    pipe_contextdir='\${prefix}/context'
  fi

  if test -z "$pipe_libdir"; then
    pipe_libdir='\${prefix}/lib'
  fi

  if test -z "$pipe_procdir"; then
    pipe_procdir='\${prefix}/proc'
  fi

  if test "$1" = "default"; then
    pipe_result=defaults
  else 
    if test ! -d "$pipe_bindir"; then
      pipe_bindir=""
    fi

    if test ! -d "$pipe_extrabindir"; then
      pipe_extrabindir=""
    fi

    if test ! -d "$pipe_contextdir"; then
      pipe_contextdir=""
    fi

    if test ! -d "$pipe_libdir"; then
      pipe_libdir=""
    fi

    if test ! -d "$pipe_procdir"; then
      pipe_procdir=""
    fi

    if test -z "$pipe_bindir" || test -z "$pipe_extrabindir" || 
       test -z "$pipe_contextdir" || test -z "$pipe_libdir" || 
       test -z "$pipe_procdir"; then
      pipe_result=problems
    else
      pipe_result=done
    fi
  fi

  ac_cv_all_paths="pipe_have_all_paths=yes \
    pipe_bindir=\"$pipe_bindir\" \
    pipe_extrabindir=\"$pipe_extrabindir\" \
    pipe_contextdir=\"$pipe_contextdir\" \
    pipe_libdir=\"$pipe_libdir\" \
    pipe_procdir=\"$pipe_procdir\" \
    pipe_result=$pipe_result"

])

eval "$ac_cv_all_paths"

if test -z "$pipe_bindir" || test -z "$pipe_extrabindir" || 
   test -z "$pipe_contextdir" || test -z "$pipe_libdir" || 
   test -z "$pipe_procdir" || test "$pipe_have_all_paths" != yes; then
  pipe_have_all_paths=no
  AC_MSG_ERROR([No correct PIPELINE directory tree could be found.])
else
  AC_MSG_RESULT($pipe_result)
fi

bindir=$pipe_bindir
extrabindir=$pipe_extrabindir
contextdir=$pipe_contextdir
libdir=$pipe_libdir
procdir=$pipe_procdir

AC_SUBST(pipe_bindir)
AC_SUBST(pipe_extrabindir)
AC_SUBST(pipe_contextdir)
AC_SUBST(pipe_libdir)
AC_SUBST(pipe_procdir)

AC_SUBST(bindir)
AC_SUBST(extrabindir)
AC_SUBST(contextdir)
AC_SUBST(libdir)
AC_SUBST(procdir)
])

dnl ------------------------------------------------------------------------
dnl Check for non-standard headers and libraries
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(PIPE_CHECK_EXTRA_LIBS,
[
  AC_MSG_CHECKING(for extra includes)
  AC_ARG_WITH(extra-includes, [  --with-extra-includes=DIR  adds non standard include paths], 
    pipe_use_extra_includes="$withval",
    pipe_use_extra_includes=NONE)

  if test -n "$pipe_use_extra_includes" && \
    test "$pipe_use_extra_includes" != "NONE"; then

    ac_save_ifs=$IFS
    IFS=':'
    for dir in $pipe_use_extra_includes; do
      all_includes="$all_includes -I$dir"
      USER_INCLUDES="$USER_INCLUDES -I$dir"
    done
    IFS=$ac_save_ifs
    pipe_use_extra_includes="added" 
  else
    pipe_use_extra_includes="no"
  fi

  AC_MSG_RESULT($pipe_use_extra_includes)

  AC_MSG_CHECKING(for extra libs)
  AC_ARG_WITH(extra-libs, [  --with-extra-libs=DIR   adds non standard library paths], 
    pipe_use_extra_libs=$withval,
    pipe_use_extra_libs=NONE)

  if test -n "$pipe_use_extra_libs" && \
    test "$pipe_use_extra_libs" != "NONE"; then

    ac_save_ifs=$IFS
    IFS=':'
    for dir in $pipe_use_extra_libs; do
      all_libraries="$all_libraries -L$dir"
      USER_LDFLAGS="$USER_LDFLAGS -L$dir"
    done
    IFS=$ac_save_ifs
    pipe_use_extra_libs="added"
  else
    pipe_use_extra_libs="no"
  fi

  AC_MSG_RESULT($pipe_use_extra_libs)

])

dnl ------------------------------------------------------------------------
dnl Check for special functions
dnl ------------------------------------------------------------------------
AC_DEFUN(PIPE_CHECK_STRDUP,
[
AC_MSG_CHECKING([for strdup])
AC_CACHE_VAL(pipe_cv_have_strdup,
[
  AC_LANG_C
  save_CFLAGS="$CFLAGS"

  if test "$GCC" = "yes"; then
    CFLAGS="$CFLAGS -pedantic-errors"
  fi

  AC_TRY_COMPILE([
  #include <string.h>],[
  char *s, buffer[] = "Hello!";
  s = strdup(buffer);
  ], pipe_cv_have_strdup=yes, pipe_cv_have_strdup=no)

  CFLAGS="$save_CFLAGS"
])
AC_MSG_RESULT($pipe_cv_have_strdup)

if test "$pipe_cv_have_strdup" = "yes"; then
  AC_DEFINE_UNQUOTED(HAVE_STRDUP)
fi
])

AC_DEFUN(PIPE_CHECK_GETUID,
[
AC_MSG_CHECKING([for getuid])
AC_CACHE_VAL(pipe_cv_have_getuid,
[
  AC_LANG_C
  save_CFLAGS="$CFLAGS"

  if test "$GCC" = "yes"; then
    CFLAGS="$CFLAGS -pedantic-errors"
  fi

  AC_TRY_COMPILE([
  #include <unistd.h>
  #include <sys/types.h>
  ],
  [
  uid_t u;
  u = getuid();
  ],
  pipe_cv_have_getuid=yes,
  pipe_cv_have_getuid=no)

  CFLAGS="$save_CFLAGS"
])
AC_MSG_RESULT($pipe_cv_have_getuid)

if test "$pipe_cv_have_getuid" = "yes"; then
  AC_DEFINE_UNQUOTED(HAVE_GETUID)
fi
])

AC_DEFUN(PIPE_CHECK_GETPWUID,
[
AC_MSG_CHECKING([for getpwuid])
AC_CACHE_VAL(pipe_cv_have_getpwuid,
[
  AC_LANG_C
  save_CFLAGS="$CFLAGS"

  if test "$GCC" = "yes"; then
    CFLAGS="$CFLAGS -pedantic-errors"
  fi

  AC_TRY_COMPILE([
  #include <pwd.h>
  #include <sys/types.h>
  ],
  [
  struct passwd *p;
  uid_t u;
  p = getpwuid(u);
  ],
  pipe_cv_have_getpwuid=yes,
  pipe_cv_have_getpwuid=no)

  CFLAGS="$save_CFLAGS"
])
AC_MSG_RESULT($pipe_cv_have_getpwuid)

if test "$pipe_cv_have_getpwuid" = "yes"; then
  AC_DEFINE_UNQUOTED(HAVE_GETPWUID)
fi
])

dnl ------------------------------------------------------------------------
dnl check for MIDAS
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(PIPE_CHECK_MIDAS_PATH,
[
AC_MSG_CHECKING([for ESO MIDAS])
AC_REQUIRE([AC_CHECK_CC_RPATH])

ac_midas_includes=NO
ac_midas_libraries=NO
midas_includes=""
midas_libraries=""
AC_ARG_WITH(midas-dir,
    [  --with-midas-dir=DIR       where MIDAS is installed, including version],
    [  ac_midas_includes="$withval"/incl
       ac_midas_libraries="$withval"/lib
    ])

AC_ARG_WITH(midas-includes,
    [  --with-midas-includes=DIR  where MIDAS includes are installed],
    [  ac_midas_includes="$withval"
    ])

AC_ARG_WITH(midas-libraries,
    [  --with-midas-libraries=DIR  where MIDAS libraries are installed],
    [  ac_midas_libraries="$withval"
    ])

AC_CACHE_VAL(ac_cv_have_midas,
[# try to guess MIDAS location

midas_incdirs="$ac_midas_includes"
test -n "$MIDASHOME" && test -n "$MIDVERS" && midas_incdirs="$MIDASHOME/$MIDVERS/incl $midas_incdirs"
AC_FIND_FILE(midas_def.h, $midas_incdirs, midas_incdir)
ac_midas_includes="$midas_incdir"

if test ! "$ac_midas_libraries" = "NO"; then
  midas_libdirs="$ac_midas_libraries"
fi

midas_libdirs="$midas_libdirs"
test -n "$MIDASHOME" && test -n "$MIDVERS" && midas_libdirs="$MIDASHOME/$MIDVERS/lib $midas_libdirs"

test=NONE
midas_libdir=NONE
for dir in $midas_libdirs; do
  try="ls -1 $dir/libmidas*"
  if test=`eval $try 2> /dev/null`; then
    midas_libdir=$dir
    break
  else
    echo "tried $dir" >&AC_FD_CC
  fi
done

ac_midas_libraries="$midas_libdir"

ac_cflags_save="$CFLAGS"
ac_ldflags_save="$LDFLAGS"
ac_libs_save="$LIBS"

CFLAGS="$CFLAGS -I$midas_incdir $all_includes"
LDFLAGS="-L$midas_libdir $all_libraries"

LIBMIDAS="-lmidas -lm $LIBSOCKET"
LIBS="$LIBS $LIBMIDAS"

PIPE_PRINT_MIDAS_PROGRAM

if AC_TRY_EVAL(ac_link) && test -s conftest; then
  rm -f conftest*
else
  echo "configure: failed program was:" >&AC_FD_CC
  cat conftest.$ac_ext >&AC_FD_CC
  ac_midas_libraries="NO"
fi
rm -f conftest*
CFLAGS="$ac_cflags_save"
LDFLAGS="$ac_ldflags_save"
LIBS="$ac_libs_save"

if test "$ac_midas_includes" = NO || test "$ac_midas_libraries" = NO; then
  ac_cv_have_midas="have_midas=no"
  ac_midas_notfound=""
  if test "$ac_midas_includes" = NO; then
    if test "$ac_midas_libraries" = NO; then
      ac_midas_notfound="(headers and libraries)";
    else
      ac_midas_notfound="(headers)";
    fi
  else
    ac_midas_notfound="(libraries)";
  fi

  AC_MSG_ERROR([MIDAS $ac_midas_notfound not found. Please check your MIDAS environment!]);
else
  have_midas="yes"
fi
])

eval "$ac_cv_have_midas"

if test "$have_midas" != yes; then
  AC_MSG_RESULT([$have_midas]);
else
  ac_cv_have_midas="have_midas=yes \
    ac_midas_includes=$ac_midas_includes ac_midas_libraries=$ac_midas_libraries"
  AC_MSG_RESULT([libraries $ac_midas_libraries, headers $ac_midas_includes])
  
  midas_libraries="$ac_midas_libraries"
  midas_includes="$ac_midas_includes"
fi

AC_SUBST(midas_libraries)
AC_SUBST(midas_includes)

if test -z "$midas_includes"; then
  MIDAS_INCLUDES=""
else
  MIDAS_INCLUDES="-I$midas_includes"
  all_includes="$MIDAS_INCLUDES $all_includes"
fi

if test -z "$midas_libraries"; then
  MIDAS_LDFLAGS=""
else
  MIDAS_LDFLAGS="-L$midas_libraries"
  all_libraries="$MIDAS_LDFLAGS $all_libraries"
fi

AC_SET_CC_RPATH(midas_rpath, "$midas_libraries")
if test -z "$midas_rpath"; then
  MIDAS_RPATH=""
  AC_MSG_WARN([The rpath feature is not available. Make sure that the MIDAS shared libraries have been added to the library search path.])
else
  MIDAS_RPATH="$midas_rpath"
fi

AC_SUBST(MIDAS_INCLUDES)
AC_SUBST(MIDAS_LDFLAGS)
AC_SUBST(MIDAS_RPATH)

LIB_MIDAS='-lmidas $(LIBM) $(LIBSOCKET)'
AC_SUBST(LIB_MIDAS)

])

dnl ------------------------------------------------------------------------
dnl MIDAS test code
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(PIPE_PRINT_MIDAS_PROGRAM,
[
AC_LANG_C
cat > conftest.$ac_ext <<EOF
#include "confdefs.h"
#ifdef VERSION
#undef VERSION
#endif
#include <midas_def.h>

int main() {
    return 0;
}
EOF
])

dnl ------------------------------------------------------------------------
dnl Setup installation program for MIDAS procedures
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(PIPE_PROC_INSTALL,
[
if test -z "$INSTALL"; then
  AC_PROG_INSTALL
fi

dnl Override INSTALL_SCRIPT value
INSTALL_SCRIPT="${INSTALL_DATA}"
AC_SUBST(INSTALL_SCRIPT)
])

dnl ------------------------------------------------------------------------
dnl Define QC dictionary version
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(PIPE_QCDIC_VERSION,
[
QCDIC_VERSION=[$1]
AC_SUBST(QCDIC_VERSION)
])
