! $Id: forsinit.prg,v 1.184 2001/06/20 08:06:48 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
!.IDENT      $RCSfile: forsinit.prg,v $
!
!.KEYWORDS   FORS pipeline, environment setup
!
!.USAGE      init/fors  [session]
!
!.INPUT      session      (P1)    Session name (file name prefix of keyword
!                                 file of a previous session)
!
!.OUTPUT     Session table forstab.tbl
!
!.RETURNS    The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!            updated with the exit status. 
!
!.PURPOSE    This procedure creates and initializes the FORS pipeline
!            environment (MIDAS FORS context). Note that apart from a few 
!            exceptions all keywords are read-only to inhibit any accidential
!            change of their values. Some of theses keywords may be modified
!            via dedicated commands.
!
!.ENVIRON    None.
!
!.LANGUAGE   MIDAS command language 98NOV.
!
!.AUTHOR     R. Palsa, ESO-DMO
!
!.VERSION    Changes:
!            $Log: forsinit.prg,v $
!            Revision 1.184  2001/06/20 08:06:48  sbogun
!            Keyword qc1log_file added.
!
!            Revision 1.183  2001/06/15 14:13:31  sbogun
!            Modifications according to new QC dictionary 1.8.
!
!            Revision 1.182  2001/05/17 09:01:50  sbogun
!            Added keyword DATAMD5.
!
!            Revision 1.181  2001/05/08 16:15:55  sbogun
!            Modifications according to the new product header
!            creation scheme.
!
!
!------------------------------------------------------------------------------

crossref session

define/param p1  +   c   "Enter session name:"
define/maxpar 1


! Procedure identification
define/local procID/c/1/80      "ForsInit"


! Local variables
define/local i/i/1/1          0
define/local msgflg/i/1/1     0
define/local msglev/i/1/1     0
define/local skplck/c/1/200   " "  all  +lower


! Setup session
if p1(1:1) .eq. "+" then
  write/out " "
  write/out "Initializing FORS environment"
  write/out " "
  register/session fors {PIPE_PROC} forsinit.prg forstab.tbl

! Remove keyword protection
  protect/fors fors 0


! Status of message keyword
  info/keyword mess_level
  if mid$info(1) .ge. 1 then
    if mid$info(1) .eq. 1 then
      msglev = {mess_level}
      msgflg = 1
    endif
    delete/keyw mess_level
  endif


!*** NEVER CHANGE THE FOLLOWING LINE ***
!Begin Session List
!***************************************
!
! Message Keyword
  write/key MESS_LEVEL/i/1/1     3                    ! Defines chattiness of the pipeline
! Recipe index. May be determined elsewhere in a later version
  write/key reci/c/1/4          "REC1"
! Pipeline internal logging facility keywords
  write/key SLOG_FILE/c/1/20    "shortlog.log"        ! Shortlog file name
  write/key LLOG_FILE/c/1/20    "longlog.log"         ! Longlog file name
  write/key LLOG_TABLE/c/1/20   "llog.tbl"            ! Longlog table name
  write/key QC1LOG_TABLE/c/1/20 "qc1log.tbl"          ! QC1 Log cache table name
  write/key QC1LOG_FILE/c/1/20  "qc1log.paf"          ! QC1 Log PAF file name
  write/key PROHEAD_FILE/c/1/20 "prohead.txt"         ! product header temp. file name
! Display Keywords
  write/key NICE_CREA/c/1/1      "n"                  ! Display flag
  write/key NICE_DPOS/c/1/30     " " all              ! Position of win.title
  write/key NICE_DLAB/c/1/30     " " all              ! Display window title
  write/key NICE_GLAB/c/1/30     " " all              ! Graphics window title (mm)
  write/key NICE_GLABW/c/1/40    " " all              ! Graphics window title (world coords.)
  write/key NICE_DSCR/i/1/2      0   all              ! Screen center
  write/key NICE_DDIM/i/1/2      0   all              ! Screen size (display)
  write/key NICE_GDIM/i/1/2      0   all              ! Screen size (graph.)
  write/key NICE_DIDS/i/1/10     0   all              ! Available display windows
  write/key NICE_GIDS/i/1/10     0   all              ! Available graphic windows
  write/key NICE_HCOPY/c/1/3     NO                   ! Do hardcopies YES or NO
  write/key NICE_DISP/c/1/30     "?"                  ! "host:display[.screen]"
  write/key RBINS_OLD/c/1/20     "new"                ! Instrument name
! Environment Keywords
  write/key FORS_LOCK/i/1/1      13012119             ! Lock number used to protect keywords
  write/key FORS_IDSTRNG/c/1/20  "DO_CLASSIFICATION"  ! Descriptor name that holds the frame identifier string
  write/key FORS_NOCLASS/c/1/20  "No classification"  ! FORS_IDSTRNG entry for frames that cannot be classified.
  write/key FORS_INSMODE/c/1/10  " " all   ! Instrument mode identification, i.e. IMG, LSS, or MOS
  write/key FORS_MAXFILT/i/1/1      4      ! Maximum number of filters
  write/key FORS_CCD_RDDIR/c/1/20   "X"    ! Readout direction
  write/key FORS_MAXOUT/i/1/1       4      ! Maximum number of readout ports
  write/key FORS_PORTS/c/1/4        " " all  ! Names of readout ports
  write/key FORS_MAXLAMP/i/1/1      6      ! Maximum number of lamps
  write/key FORS_LMAXTYP/i/1/1      4      ! Number of different lamp types
  write/key FORS_MAXSLITLET/i/1/1   19     ! Maximum number of MOS slitlets
  write/key FORS_G150_WSTRT/r/1/1   3100.  ! Start wavel.[A] f.rebin. (grism150I)
  write/key FORS_G150_WEND/r/1/1    11600. ! End   wavel.[A] f.rebin. (grism150I)
  write/key FORS_G200_WSTRT/r/1/1   3100.  ! Start wavel.[A] f.rebin. (grism200I)
  write/key FORS_G200_WEND/r/1/1    11600. ! End   wavel.[A] f.rebin. (grism200I)
  write/key FORS_MINRESPFAC/r/1/1   0.05   ! min.resp.ratio = FAC * max.resp.ratio
  write/key FORS_NPRESCAN/i/1/1     16     ! Number of prescan pixels for FORS1/2
  write/key SEAITER/r/1/8           1.5,2.,4.,6.5,9.5,13.,17.,21.5  ! line search: replaces setup var. SEAPAR(1) in iterations
! Pipeline control keywords
  write/key p_version/c/1/100      " "  all    ! Pipeline version number
  write/key p_qcdic_version/c/1/100 " "  all   ! QC-dictionary version number
  out_a = m$lower(rbinstrument) // "/FORS_VERSION"
  write/key p_version_file/c/1/100 "PIPE_HOME:{out_a}"
                                               ! Pipeline version file name
  write/key p_ctxdir/c/1/100       " "  all    ! Pipeline context directory. All contexts are loaded from here.
  write/key p_bindir/c/1/100       " "  all    ! Executables are found here.
  write/key p_ctxdel/i/1/1         0      l    ! Delete flag for auxiliary contexts in super mode
  write/key p_mainonly/i/1/1       0           ! Pipeline product delivery flag
  write/key p_nmodes/i/1/1         5           ! Total number of supported instrument modes
  write/key p_supported/c*15/1/5   " "  all    ! List of supported instrument modes
  write/key p_modes/c*3/1/5        " "  all    ! Internal names of instrument modes
  write/key p_maxfrm/i/1/2         300,100     ! Maximum number of frames allowed in input and reference sets.
  write/key p_dspold/i/1/1         0           ! Display status from previous recipe execution
  write/key p_sizold/i/1/1         0           ! Display size from previous recipe execution
  write/key p_dspmode/i/1/1        0           ! Colour mode of display:
                                               ! 0 =  8bit: PseudoColour
                                               ! 1 = 24bit: RGB
                                               ! 2 = 24bit: Pseudo on DirectColour
                                               ! 3 = 24bit: Pseudo on TrueColour
  write/key p_hdrpurge/c/1/100     " "  all    ! Table with header keyword cleanup information
  write/key p_startdate/c/1/80     " "  all    ! For start date of recipe execution
! Instrument names
  write/key TC_NAME/c/1/10     "VLTTC"  ! Test camera instrument name
  write/key FORS_NAME/c/1/10   "FORS"   ! FORS1 instrument name
! Instrument setup keywords
  write/key kbase_outi/c/1/20      "ESO.DET.OUT"   ! Base name of readout port header keyords
  write/key ksuff_outi/c/1/20      ".NAME"           ! Name suffix of readout port header keywords
  write/key kbase_filti/c/1/20     "ESO.INS.FILT"  ! Base name of the filter subsystem keywords
  write/key ksuff_filti/c/1/20     ".NAME"         ! Name suffix of the filter subsystem keywords
  write/key kbase_lampi/c/1/20     "ESO.INS.LAMP"  ! Base name of the filter subsystem keywords
  write/key ksuff_lampi/c/1/20     ".NAME"         ! Name suffix of the filter subsystem keywords
  write/key kbase_mosi/c/1/20      "ESO.INS.MOS"   ! Base name of the MOS subsystem keywords
  write/key ksuff_mospos/c/1/20    ".POS"          ! Name suffix of the MOS slit position keywords
  write/key ksuff_moswid/c/1/20    ".WID"          ! Name suffix of the MOS slitlet width keywords
  write/key fors_nout/i/1/1        0               ! Total number of active readout ports
  write/key fors_outi/i/1/4        0    all        ! Identification numbers of active readout ports
  write/key fors_filti/c*10/1/4    " "  all        ! Names of used filters
  write/key fors_nfilt/i/1/1       0               ! Total number of filters used
  write/key fors_nlamp/i/1/1       0               ! Total number of active lamps
  write/key fors_lampi/c*10/1/6    " "  all        ! Names of active lamps
  write/key fors_ltypes/c*10/1/4   " "  all        ! Types of calibration lamps
  write/key fors_nolamp/c/1/10     "None"          ! Name of dummy calibration lamp
  write/key fors_nofilt/c/1/10     "None"          ! Name of dummy filter
  write/key fors_fcam/r/1/1        280.0           ! FORS camera focal length in mm
  write/key fors_fcoll/r/1/2       1233.0,616.5    ! FORS collimato focal length in mm (SR,HR)
  write/key fors_maxlin/r/1/1      65534.          ! Linearity limit of detector (ADU)
  write/key fors_qcstatwin/i/1/1   100             ! Size of window for QC1
! Pipeline processing levels and product types
  write/key d_empty/c/1/60    "EMPTY"              ! Dummy string for empty reduction levels
  write/key d_tmp/c/1/60      "TEMPORARY"          ! Type string of temporary products
  write/key d_pre/c/1/60      "PREPROCESSED"       ! Type string of preliminary products
  write/key d_red/c/1/60      "REDUCED"            ! Type string of fully reduced products
  write/key d_qcp/c/1/60      "QCPARAM"            ! Type string for quality control parameters
  write/key d_det/c/1/60      "DETECTOR"           ! Product level for (partly) removed detector effects
  write/key d_geo/c/1/60      "GEOMETRIC"          ! Product level for removed instrumental effects
  write/key d_pho/c/1/60      "PHOTOMETRIC"        ! Product level for calibrated products
  write/key d_uchk/c/1/60     "UNCHECKED"          ! Initial certification level of raw frames
! Map frame descriptors names to keywords
! General
  write/key h_airmass/c/1/60  "O_AIRM"             ! Airmass descriptor
  write/key h_instrume/c/1/60 "INSTRUME"           ! Name of instrument in use
  write/key h_origname/c/1/60 "ORIGFILE"           ! Original name of file
  write/key h_arcfile/c/1/60  "ARCFILE"            ! Archive name of file
  write/key h_pipefile/c/1/60 "PIPEFILE"           ! Pipeline name of file
  write/key h_filename/c/1/60 "DO_FILENAME"        ! Actual name of file
  write/key h_ident/c/1/60    "IDENT"              ! Identifier
  write/key h_observer/c/1/60 "OBSERVER"           ! Name of observer
! Data product and data processing keywords
  write/key h_dprcatg/c/1/60   "ESO.DPR.CATG"         ! Data product category                   
  write/key h_dprtype/c/1/60   "ESO.DPR.TYPE"	      ! Data product type                       
  write/key h_dprtech/c/1/60   "ESO.DPR.TECH"	      ! Data product technique                  
  write/key h_protype/c/1/60   "ESO.PRO.TYPE"	      ! Pipeline product type
  write/key h_prolevel/c/1/60  "ESO.PRO.REDLEVEL"     ! Reduction level of data product
  write/key h_procatg/c/1/60   "ESO.PRO.CATG"	      ! Category of pipeline product frame
  write/key h_prostat/c/1/60   "ESO.PRO.{reci}.STATUS" ! Pipeline status of product frame
  write/key h_prodmd5/c/1/60   "DATAMD5"              ! Pipeline status of product frame
  write/key h_datamin/c/1/60   "DATAMIN"              ! Minimun of frame pixel values
  write/key h_datamax/c/1/60   "DATAMAX"              ! Maximum of frame pixel values
  write/key h_promean/c/1/60   "ESO.PRO.DATAAVG"      ! Mean of frame pixel values
  write/key h_promedi/c/1/60   "ESO.PRO.DATAMED"      ! Standard deviation of frame pixel values
  write/key h_prosigm/c/1/60   "ESO.PRO.DATARMS"      ! Median of pixel values
  write/key h_proncom/c/1/60   "ESO.PRO.DATANCOM"     ! Number of frames combined
  write/key h_proexpt/c/1/60   "ESO.PRO.EXPTTOT"      ! Total integration time
  write/key h_proqcsts/c/1/60  "ESO.QC.STATUS"        ! Quality control level of product frame
  write/key h_proqccert/c/1/60 "ESO.QC.CERTIF"        ! Certification level of frame
  write/key h_proqcdid/c/1/60  "ESO.QC.DID"           ! Data dictionary for QC
  write/key h_proqcovex/c/1/60 "ESO.QC.OVEREXPO"      ! Percentage of overexposed pixels
  write/key h_proqcsmea/c/1/60 "ESO.QC.SKYAVG"        ! Mean of sky background (mag/arcs^2)
  write/key h_proqcssig/c/1/60 "ESO.QC.SKYRMS"        ! Standard deviation of sky background (mag/arcs^2)
  write/key h_proqcsmed/c/1/60 "ESO.QC.SKYMED"        ! Median of sky background (mag/arcs^2)
  write/key h_proqcfwhm/c/1/60 "ESO.QC.IMGQU"         ! Seeing (FWHM) determined from image (arcs)
  write/key h_proqcfwsg/c/1/60 "ESO.QC.IMGQUERR"      ! Uncert. of seeing (FWHM) determ. from img. (arcs)
  write/key h_proqcstel/c/1/60 "ESO.QC.STELLAVG"      ! Mean stellarity from img.
  write/key h_proqczpoi/c/1/60 "ESO.QC.ZPOINT"        ! Photometric night zeropoint (mag)
  write/key h_proqczpsg/c/1/60 "ESO.QC.ZPOINTRMS"     ! Photometric night zeropoint (mag)
  write/key h_proqcextc/c/1/60 "ESO.QC.EXTCOEFF"      ! Atmospheric extinction coefficient
  write/key h_proqcobia/c/1/60 "ESO.QC.OVBIAS"        ! Bias level in pre-/overscan
  write/key h_proqconoi/c/1/60 "ESO.QC.OVNOISE"       ! Noise level in pre-/overscan (e-)
  write/key h_prqcobi/c*60/1/4 " " all                ! Bias level in individual pre-/overscan
  write/key h_prqcono/c*60/1/4 " " all                ! Noise level in individual pre-/overscan (e-)
  write/key h_proqclimg/c/1/60 "ESO.QC.LIMAG"         ! Limiting magnitude (for S/N=5)
  write/key h_qcbiameda/c/1/60 "ESO.QC.OUT1.DATAMED"  ! Median of bias across port A
  write/key h_qcbiastra/c/1/60 "ESO.QC.OUT1.STRUCT"   ! Structure of bias in port A
  write/key h_qcbiadfad/c/1/60 "ESO.QC.OUT1.OUT4.DIFF" ! Port diff. between ports A/D
  write/key h_qcbiadfcd/c/1/60 "ESO.QC.OUT3.OUT4.DIFF" ! Port diff. between ports C/D
  write/key h_qcbianoma/c/1/60 "ESO.QC.OUT1.RONAVG"   ! Readout noise measured in port A
  write/key h_qcbianoea/c/1/60 "ESO.QC.OUT1.RONEXP"   ! Readout noise expected in port A
  write/key h_qcbiansga/c/1/60 "ESO.QC.OUT1.RONRMS"   ! Error of readout noise in port A
  write/key h_qcflteffa/c/1/60 "ESO.QC.OUT1.EFF"      ! Efficiency measured in port A
  write/key h_qcfltnoph/c/1/60 "ESO.QC.PHN"           ! Photon noise measured in a flatfield
  write/key h_qcfltnofp/c/1/60 "ESO.QC.FPN"           ! Fix-pattern noise measured in a flatfield
  write/key h_qcfltdfab/c/1/60 "ESO.QC.OUT1.OUT2.DIFF" ! Port diff. between ports A/B
  write/key h_qcfltcont/c/1/60 "ESO.QC.CONT"          ! Contamination measured in a flatfield
  write/key h_qcfltcnva/c/1/60 "ESO.QC.OUT1.CONADAVG" ! Conversion factor measured in a flatfield in port A
  write/key h_qcfltcnvb/c/1/60 "ESO.QC.OUT2.CONADAVG" ! Conversion factor measured in a flatfield in port B
  write/key h_qcfltcnvc/c/1/60 "ESO.QC.OUT3.CONADAVG" ! Conversion factor measured in a flatfield in port C
  write/key h_qcfltcnvd/c/1/60 "ESO.QC.OUT4.CONADAVG" ! Conversion factor measured in a flatfield in port D
  write/key h_qcfltcsga/c/1/60 "ESO.QC.OUT1.CONADRMS" ! error for conversion factor in port A
  write/key h_qcfltcsgb/c/1/60 "ESO.QC.OUT2.CONADRMS" ! error for conversion factor in port B
  write/key h_qcfltcsgc/c/1/60 "ESO.QC.OUT3.CONADRMS" ! error for conversion factor in port C
  write/key h_qcfltcsgd/c/1/60 "ESO.QC.OUT4.CONADRMS" ! error for conversion factor in port D
  write/key h_ovexmask/c/1/60  "OVEREXPO_MASK"        ! Name of overexposure mask
  write/key h_dicolamb/c/1/60  "COEFSLIT"             ! Prefix f. tbl. desc. w. dispers. coef. (i.e., without "xx" no.)
  write/key h_dicodpix/c/1/60  "CENTERXY"             ! Tbl. desc. w. reference detect. coord. f. dispers. coef.
  write/key h_bsmean/c/1/60    "BIAS_MEAN"            ! Bias values of overscan regions
! Observation block  keywords			      
  write/key h_obsname/c/1/60  "ESO.OBS.NAME"	      ! Observation block name                  
  write/key h_obsid/c/1/60    "ESO.OBS.ID"	          ! Observation block ID
  write/key h_obstpno/c/1/60  "ESO.OBS.TPLNO"	      ! Number of templates within OB           
! Template keywords				      
  write/key h_tpid/c/1/60     "ESO.TPL.ID"	          ! Template ID
  write/key h_tpnexp/c/1/60   "ESO.TPL.NEXP"	      ! Number of exposures within template     
  write/key h_tpexpno/c/1/60  "ESO.TPL.EXPNO"	      ! Exposure sequence number within template
! Telescope keywords
  write/key h_telid/c/1/60    "ESO.TEL.ID"	          ! Telescope ID
  write/key h_geolon/c/1/60   "ESO.TEL.GEOLON"        ! Telescope geographic longitude (+ = North)
  write/key h_geolat/c/1/60   "ESO.TEL.GEOLAT"        ! Telescope geographic latitude (+ = East)
  write/key h_seestrt/c/1/60  "ESO.TEL.AMBI.FWHM.START"  ! Seeing from ambient server at start of exposure
  write/key h_seeend/c/1/60   "ESO.TEL.AMBI.FWHM.END"    ! Seeing from ambient server at end of exposure
! Instrument keywords				      
  write/key h_insmode/c/1/60  "ESO.INS.MODE"          ! Instrument mode               
  write/key h_coll/c/1/60     "ESO.INS.COLL.NAME"     ! Collimator name               
  write/key h_pixscale/c/1/60 "ESO.INS.PIXSCALE"      ! Pixel scale                   
  write/key h_focpos/c/1/60   "ESO.INS.FOCU.POS"      ! Focus position
  write/key h_filti/c*60/1/4  " "  all                ! Names of used filters
  write/key h_grism1/c/1/60   "ESO.INS.GRIS1.NAME"    ! Name of grism  #1             
  write/key h_gidno1/c/1/60   "ESO.INS.GRIS1.ID"      ! ID number of grism  #1  (ECH: x.grism)
  write/key h_wcenter/c/1/60  "ESO.INS.GRIS1.WLEN"    ! Central wavelength of grism #1
  write/key h_avdisp/c/1/60   "ESO.INS.GRIS1.DISP"    ! Average dispersion of grism #1
  write/key h_grism2/c/1/60   "ESO.INS.GRIS2.NAME"    ! Name of grism  #2  (ECH: e.grism)
  write/key h_slit/c/1/60     "ESO.INS.SLIT.NAME"     ! Name of slit                  
  write/key h_slwidth/c/1/60  "ESO.INS.SLIT.WID"      ! LSS slit width in arcsec
  write/key h_slpos/c/1/60    "ESO.INS.SLIT.POS"      ! LSS slit offset from center in mm
  write/key h_mosra/c*60/1/{FORS_MAXSLITLET}    " "  all  ! RA of MOS slits in deg
  write/key h_mosdec/c*60/1/{FORS_MAXSLITLET}   " "  all  ! DEC of MOS slits in deg
  write/key h_moswid/c*60/1/{FORS_MAXSLITLET}   " "  all  ! Width of MOS slits in arcsec
  write/key h_moslen/c*60/1/{FORS_MAXSLITLET}   " "  all  ! Length of MOS slits in arcsec
  write/key h_mospos/c*60/1/{FORS_MAXSLITLET}   " "  all  ! Position of MOS slits in mm
  write/key h_moswidth/c*60/1/{FORS_MAXSLITLET} " "  all  ! Width of MOS slits in mm
! Detector keywords
  write/key h_detid/c/1/60    "ESO.DET.ID"	          ! Detector ID
  write/key h_detector/c/1/60 "ESO.DET.NAME"          ! Detector name              
  write/key h_xpixsize/c/1/60 "ESO.DET.CHIP1.PSZX"    ! Size of pixels along x-axis
  write/key h_nout/c/1/60     "ESO.DET.OUTPUTS"	      ! Total number of active readout ports
  write/key h_outmode/c/1/60  "ESO.DET.READ.MODE"     ! Readout mode
  write/key h_outclock/c/1/60 "ESO.DET.READ.CLOCK"    ! Readout mode description
  write/key h_confac/c*60/1/4 " "  all                ! Conversion factor electrons to ADU
  write/key h_ron/c*60/1/4    " "  all                ! Detector readout noise
  write/key h_pscan/c*60/1/4  " "  all		          ! Prescan along x-axis       
  write/key h_oscan/c*60/1/4  " "  all		          ! Overscan along x-axis      
  write/key h_pscany/c*60/1/4  " "  all		          ! Prescan along y-axis       
  write/key h_oscany/c*60/1/4  " "  all		          ! Overscan along y-axis      
  write/key h_nwin/c/1/60     "ESO.DET.WINDOWS"	      ! Number of readout windows
  write/key h_xwinstr/c/1/60  "ESO.DET.WIN1.STRX"     ! start pixel on x-axis
  write/key h_ywinstr/c/1/60  "ESO.DET.WIN1.STRY"     ! start pixel on y-axis
  write/key h_xwinn/c/1/60    "ESO.DET.WIN1.NX"       ! number of pixels on x.
  write/key h_ywinn/c/1/60    "ESO.DET.WIN1.NY"       ! number of pixels on y.
  write/key h_xwinbin/c/1/60  "ESO.DET.WIN1.BINX"     ! Binning size along x-axis  
  write/key h_ywinbin/c/1/60  "ESO.DET.WIN1.BINY"     ! Binning size along y-axis
  write/key h_dit1/c/1/60     "ESO.DET.WIN1.DIT1"     ! Actual integration time
  write/key h_chipnx/c/1/60   "ESO.DET.CHIP1.NX"      ! Number of pixels along X axis of chip
  write/key h_chipny/c/1/60   "ESO.DET.CHIP1.NY"      ! Number of pixels along Y axis of chip
! Comments for header keywords (Note that session manager limit
! for keyword defaults is 50 characters.
  write/key i_protype/c/1/48  "Product type"                         ! Comment for keyword h_protype
  write/key i_prolevel/c/1/48 "Reduction level of data product"      ! Comment for keyword h_prolevel
  write/key i_procatg/c/1/48  "Category of pipeline product frame"   ! Comment for keyword h_procatg
  write/key i_prostat/c/1/48  "Pipeline status of frame"             ! Comment for keyword h_prostat
  write/key i_prodmd5/c/1/48  "MD5 data signature"                   ! Comment for keyword h_prodmd5
  write/key i_datamin/c/1/60  "Minimum pixel value"                  ! Comment for keyword h_datamin
  write/key i_datamax/c/1/60  "Maximum pixel value"                  ! Comment for keyword h_datamax
  write/key i_promean/c/1/60  "Mean of pixel values"                 ! Comment for keyword h_promean
  write/key i_promedi/c/1/60   "Median of pixel values"              ! Comment for keyword h_promedi
  write/key i_prosigm/c/1/60   "Standard deviation of pixel values"  ! Comment for keyword h_prosigm
  write/key i_proncom/c/1/60   "Number of frames combined"           ! Comment for keyword h_proncom
  write/key i_proexpt/c/1/60   "Total exposure time of all frames combined"        ! Comment for keyword h_proexpt
  write/key i_proqcsts/c/1/48  "QC status of frame"                                ! Comment for keyword h_proqcsts
  write/key i_proqccert/c/1/48 "Certification level of frame"                      ! Comment for keyword h_proqccert
  write/key i_proqcdid/c/1/48  "Data dictionary for QC"                            ! Comment for keyword h_proqcdid
  write/key i_proqcovex/c/1/48 "Percentage of overexposed pixels (%)"              ! Comment for keyword h_proqcovex
  write/key i_proqcsmea/c/1/48 "Mean of sky background (mag/arcsec^2)"             ! Comment for keyword h_proqcsmea
  write/key i_proqcssig/c/1/48 "Std.dev.of sky backgr. (mag/arcsec^2)"             ! Comment for keyword h_proqcssig
  write/key i_proqcsmed/c/1/48 "Median of sky backgr. (mag/arcsec^2)"              ! Comment for keyword h_proqcsmed
  write/key i_proqcfwhm/c/1/48 "Image quality determined from image (arcsec)"      ! Comment for keyword h_proqcfwhm
  write/key i_proqcfwsg/c/1/48 "Uncertainty of image quality (arcsec)"             ! Comment for keyword h_proqcstel
  write/key i_proqcstel/c/1/48 "Mean stellarity index"                             ! Comment for keyword h_proqfwsgc
  write/key i_proqczpoi/c/1/48 "Photometric night zeropoint (mag)"                 ! Comment for keyword h_proqczpoi
  write/key i_proqczpsg/c/1/48 "Uncertainty of photometric night zeropoint (mag)"  ! Comment for keyword h_proqczpoi
  write/key i_proqcextc/c/1/48 "Atm. extinction coefficient (mag)"                 ! Comment for keyword h_proqcextc
  write/key i_proqcobia/c/1/48 "Bias level in pre-/overscan"                       ! Comment for keyword h_proqcobia
  write/key i_proqconoi/c/1/48 "Noise level in pre-/overscan (e-)"                 ! Comment for keyword h_proqconoi
  write/key i_prqcobi/c/1/48   "Bias level in pre-/overscan"                       ! Comment for keyword h_prqcobi
  write/key i_prqcono/c/1/48   "Noise level in pre-/overscan (e-)"                 ! Comment for keyword h_prqcono
  write/key i_proqclimg/c/1/48 "Limiting magnitude (mag) (S/N=5)"                  ! Comment for keyword h_proqclimg
  write/key i_filti/c/1/48     "Filter i name"                                     ! Comment for keyword h_filti
  write/key i_qcbiameda/c/1/48 "Median of bias across port A (ADU)"                ! Comment for keyword h_qcbiameda
  write/key i_qcbiastra/c/1/48 "Structure of bias in port A (ADU)"                 ! Comment for keyword h_qcbiastra
  write/key i_qcbiadfad/c/1/48 "Fract.port diff.between ports A and D (%)"         ! Comment for keyword h_qcbiadfad
  write/key i_qcbiadfcd/c/1/48 "Fract.port diff.between ports C and D (%)"         ! Comment for keyword h_qcbiadfcd
  write/key i_qcbianoma/c/1/48 "Measured readout noise in port A (ADU)"            ! Comment for keyword h_qcbianoma
  write/key i_qcbianoea/c/1/48 "Expected readout noise in port A (ADU)"            ! Comment for keyword h_qcbianoea
  write/key i_qcbiansga/c/1/48 "Error of readout noise in port A (ADU)"            ! Comment for keyword h_qcbiansga
  write/key i_qcflteffa/c/1/48 "Efficiency measured in port A (ADU/s)"             ! Comment for keyword h_qcflteffa
  write/key i_qcfltnoph/c/1/48 "Photon noise (ADU)"                                ! Comment for keyword h_qcfltnoph
  write/key i_qcfltnofp/c/1/48 "Fixed-pattern noise (ADU)"                         ! Comment for keyword h_qcfltnofp
  write/key i_qcfltdfab/c/1/48 "Fract.port diff.between ports A and B (%)"         ! Comment for keyword h_qcfltdfab
  write/key i_qcfltcont/c/1/48 "Contamination (%)"                                 ! Comment for keyword h_qcfltcont
  write/key i_qcfltcnva/c/1/48 "Conversion factor port A (e-/ADU)"                 ! Comment for keyword h_qcfltcnva
  write/key i_qcfltcnvb/c/1/48 "Conversion factor port B (e-/ADU)"                 ! Comment for keyword h_qcfltcnvb
  write/key i_qcfltcnvc/c/1/48 "Conversion factor port C (e-/ADU)"                 ! Comment for keyword h_qcfltcnvc
  write/key i_qcfltcnvd/c/1/48 "Conversion factor port D (e-/ADU)"                 ! Comment for keyword h_qcfltcnvd
  write/key i_qcfltcsga/c/1/48 "Error for conversion factor in port A (e-/ADU)"    ! Comment for keyword h_qcfltcsga
  write/key i_qcfltcsgb/c/1/48 "Error for conversion factor in port B (e-/ADU)"    ! Comment for keyword h_qcfltcsgb
  write/key i_qcfltcsgc/c/1/48 "Error for conversion factor in port C (e-/ADU)"    ! Comment for keyword h_qcfltcsgc
  write/key i_qcfltcsgd/c/1/48 "Error for conversion factor in port D (e-/ADU)"    ! Comment for keyword h_qcfltcsgd
! Column names for calibration tables
  write/key c_slsno/c/1/20    "SLIT"        ! Slit number column label in slit position table
  write/key c_slsname/c/1/20  "SLIT_NAME"   ! Slit name column label in slit position table
  write/key c_slcoll/c/1/20   "COLLIMATOR"  ! Collimator column label in slit position table
  write/key c_sloffset/c/1/20 "XOFFSET"     ! Slit offset column label in slit position table
  write/key c_slystart/c/1/20 "YSTART"      ! Column label for Y start pixel coordinate
  write/key c_slyend/c/1/20   "YEND"        ! Column label for Y end pixel coordinate
  write/key c_ltxpos/c/1/20   "X"           ! X position column label in line position table
  write/key c_ltypos/c/1/20   "Y"           ! Y position column label in line position table
  write/key c_ltslit/c/1/20   "SLIT"        ! Slit number column label in line position table
  write/key c_ltwave/c/1/20   "WAVE"        ! Identified lines column label in line position table
  write/key c_ltwavec/c/1/20  "WAVEC"       ! Computed lines column label in line position table
  write/key c_ltresid/c/1/20  "RESIDUAL"    ! Fit residuals column label in line position table
  write/key c_ltreject/c/1/20 "REJECT"      ! Rejection flag column label in line position table
  write/key c_gdname/c/1/20   "NAME"        ! Grism name column label in grism parameter table
  write/key c_gdidno/c/1/20   "IDNO"        ! Grism ID number column label in grism parameter table
  write/key c_lcgrism/c/1/20  "GRISM_NAME"  ! Grism name column label in master line list
  write/key c_lcgidno/c/1/20  "GRISM_ID"    ! Grism ID column label in master line list
  write/key c_lcswid/c/1/20   "SLIT_WIDTH"  ! Slit width column label in master line list
  write/key c_lcsname/c/1/20  "SLIT_NAME"   ! Slit name column label in master line list
  write/key c_lcnline/c/1/20  "NLINES"      ! Label for number of lines column in line list catalog
  write/key c_lcwave/c/1/20   "WAVE"        ! Label for wavelength column in master line list
  write/key c_lcion/c/1/20    "ION"         ! Ion column label in master line list
  write/key c_lcintens/c/1/20 "INTENSITY"   ! Intensity column label in master line list
  write/key c_lclamp/c/1/20   "LAMP"        ! Lamp column label in master line list
  write/key c_lccomm/c/1/20   "COMMENTS"    ! Comment column label in master line list
  write/key c_grgrism/c/1/20  "GRISM_NAME"  ! Grism name column label in grism bandpass table
  write/key c_grfilt/c/1/20   "FILTER_NAME" ! Filter name column label in grism bandpass table
  write/key c_grstart/c/1/20  "W_START"     ! Start wavelength column label in grism bandpass table
  write/key c_grend/c/1/20    "W_END"       ! End wavelength column label in grism bandpass table
  write/key c_grstep/c/1/20   "W_STEP"      ! Wavelength step column label in grism bandpass table
  write/key c_fxoname/c/1/20  "OBJECT"      ! Object name column label in standard flux database
  write/key c_fxndata/c/1/20  "NDATA"       ! Label for column storing number of data items in flux database
  write/key c_fxwave/c/1/20   "LAMBDA"      ! Column label of wavelength array column in object spectrum table
  write/key c_fxflux/c/1/20   "F_LAMBDA"    ! Column label of flux array column in object spectrum table
  write/key c_fxbwidth/c/1/20 "BIN_WIDTH"   ! Column label of wavelength bin width array column in object spectrum table
  write/key c_woslit/c/1/20   "OBJ_SLIT"    ! Object region number column label in MOS window table
  write/key c_wostart/c/1/20  "OBJ_STRT"    ! Object region start column label in MOS window table
  write/key c_woend/c/1/20    "OBJ_END"     ! Object region end column label in MOS window table
  write/key c_sostart/c/1/20  "W_START"     ! Start wavelength column label in object spectrum table
  write/key c_sostep/c/1/20   "W_STEP"      ! Wavelength step column label in object spectrum table
  write/key c_sondata/c/1/20  "NDATA"       ! Label of column with number of data values per spectrum in object spectrum table
  write/key c_sowave/c/1/20   "LAMBDA"      ! Column label of wavelength array column in object spectrum table
  write/key c_soflux/c/1/20   "FLUX"        ! Column label of flux array column in object spectrum table
  write/key c_soferr/c/1/20   "FLUX_ERR"    ! Column label of flux error array column in object spectrum table
  write/key c_sorefid/c/1/20  "FXREFID"     ! Entry number of standard star in flux database
  write/key c_rswave/c/1/20   "WAVE"        ! Label for wavelength column in the response table
  write/key c_rsratio/c/1/20  "RATIO"       ! Label for response value column in the response table
  write/key c_xtstart/c/1/20  "W_START"     ! Start wavelength column label in extinction data table
  write/key c_xtstep/c/1/20   "W_STEP"      ! Wavelength step column label in extinction data table
  write/key c_xtndata/c/1/20  "NDATA"       ! Label of column with number of data values per spectrum in extinction data table
  write/key c_xtratio/c/1/20  "RATIO"       ! Column label of flux ratio column (mag) in extinction data table.
  write/key c_xtadev/c/1/20   "ABSDEV"      ! Column label for mean absolute deviation of extinction coeff. fit
  write/key c_exwav/c/1/20    "LAMBDA"      ! Column name for wavelength in extinction coeff. table
  write/key c_izerop/c/1/20   "z_"          ! IMG: FIXED column prefix for zero point (filter is appended)
  write/key c_iextco/c/1/20   "kext_"       ! IMG: FIXED column prefix for extinction coefficient (filter is appended)
  write/key c_icolco/c/1/20   "kcol_"       ! IMG: FIXED column prefix for colour coefficient (filter is appended)
  write/key c_szerop/c/1/20   "ZEROPOINT"   ! LSS/MOS: Column name for extinction zero points
  write/key c_sextco/c/1/20   "EXTCOEF"     ! LSS/MOS: Column name for extinction coefficients
  write/key c_mmjd/c/1/20     "MEAN_MJD"    ! Column prefix for mean MJD
  write/key c_xpix/c/1/20     "X_PIX"       ! Standard star x position in pixel
  write/key c_ypix/c/1/20     "Y_PIX"       ! Standard star y position in pixel
  write/key c_dxpix/c/1/20    "DX_PIX"      ! Standard star x position error in pixel
  write/key c_dypix/c/1/20    "DY_PIX"      ! Standard star y position error in pixel
  write/key c_rect/c/1/20     "RA"          ! Standard star rectascension
  write/key c_decl/c/1/20     "DEC"         ! Standard star declination
  write/key c_drect/c/1/20    "DRA"         ! Standard star rectascension (acc.)
  write/key c_ddecl/c/1/20    "DDEC"        ! Standard star declination (accura.)
  write/key c_radeg/c/1/20    "RA_DEG"      ! Standard star rectascension (degrees)
  write/key c_decdeg/c/1/20   "DEC_DEG"     ! Standard star declination (degrees)
  write/key c_dradeg/c/1/20   "DRA_DEG"     ! Standard star rectascension uncertainty (degrees)
  write/key c_ddecdeg/c/1/20  "DDEC_DEG"    ! Standard star declination uncertainty (degrees)
  write/key c_airm/c/1/20     "AIRMASS"     ! Airmass
  write/key c_mjd/c/1/20      "MJD"         ! MJD
  write/key c_flx/c/1/20      "FLUX"        ! star  [e-]
  write/key c_mag/c/1/20      "MAG"         ! star  [mag]
  write/key c_sky/c/1/20      "SKY"         ! sky  [e-]
  write/key c_skymag/c/1/20   "SKY_MAG"     ! sky  [mag]
  write/key c_magsky/c/1/20   "MAG_SKY"     ! raw-sky  [mag]
  write/key c_s2n/c/1/20      "S2N"         ! signal to noise
  write/key c_seeing/c/1/20   "SEEING"      ! Seeing
  write/key c_aptobj/c/1/20   "OBJECT"      ! Names of stars in aligned photometry table
  write/key c_aptmeaU/c/1/20   "MAG_U"      ! measured mag. in U filter in aligned photometry table
  write/key c_aptmeaB/c/1/20   "MAG_B"      ! measured mag. in B filter in aligned photometry table
  write/key c_aptmeaV/c/1/20   "MAG_V"      ! measured mag. in V filter in aligned photometry table
  write/key c_aptmeaR/c/1/20   "MAG_R"      ! measured mag. in R filter in aligned photometry table
  write/key c_aptmeaI/c/1/20   "MAG_I"      ! measured mag. in I filter in aligned photometry table
  write/key c_aptair/c/1/20    "AIRMASS"    ! airmass in aligned photometry table
  write/key c_aptmjd/c/1/20    "MJD"        ! modified julian date in aligned photometry table
  write/key c_aptflg/c/1/20    "FLAGS"      ! object quality flags in aligned photometry table
  write/key c_aptstdU/c/1/20   "U"          ! standard mag. in U filter in aligned photometry table
  write/key c_aptstdB/c/1/20   "B"          ! standard mag. in B filter in aligned photometry table
  write/key c_aptstdV/c/1/20   "V"          ! standard mag. in V filter in aligned photometry table
  write/key c_aptstdR/c/1/20   "R"          ! standard mag. in R filter in aligned photometry table
  write/key c_aptstdI/c/1/20   "I"          ! standard mag. in I filter in aligned photometry table
  write/key c_aptNstd/c/1/20   "N"          ! no. of measurements used to derive std.mag. in ali.phot.tbl.
! Global frame identifier keyword
  write/key fors_id/c/1/25     " "          ! Frame category of the current frame is stored here (e.g. SCREEN_FLAT)


! Map DO classifications to pipeline internal variables.
! Common and mode dependent frames require a suffix indicating the observing mode.
! This is done in 'fors_cfgenv.prg', which has to be consistent with the definitions below.
! DON'T FORGET to update the lists of header keyword names and comments below,
! as well as 'fors_proheadinit.prg' !!!

! Mode independent frames
  write/key BPIXF_ID/c/1/25      "BADPIX"	  ! Bad pixel flat fields
  write/key BPIXM_ID/c/1/25      "BAD_PIXEL_MAP"  ! Bad pixel map
  write/key BPIXT_ID/c/1/25      "BAD_PIXEL_TABLE"  ! Bad pixel table
  write/key BIAS_ID/c/1/25       "BIAS"           ! Bias frame
  write/key RBIAS_ID/c/1/25      "REDUCED_BIAS"   ! Reduced bias frame
  write/key MBIAS_ID/c/1/25      "MASTER_BIAS"    ! Master bias frame
  write/key DARK_ID/c/1/25       "DARK"           ! Dark frame
  write/key RDARK_ID/c/1/25      "REDUCED_DARK"   ! Reduced dark frame
  write/key MDARK_ID/c/1/25      "MASTER_DARK"    ! Master dark frame
  write/key PHOTT_ID/c/1/25      "PHOTOMETRY_TABLE"  ! Photometry table
  write/key PHOTTALI_ID/c/1/25   "ALIGNED_PHOTOMETRY_TABLE"  ! Aligned photometry table
  write/key PHOTTCAL_ID/c/1/25   "PHOTOMETRY_FLXCAL_TABLE"   ! Flux-calibrated photometry table
  write/key OVEXPT_ID/c/1/25     "OVEREXPO_TABLE"  ! Statistics of (over)exposures
  write/key OVEXPM_ID/c/1/25     "OVEREXPO_MASK"   ! Mask with overexposed pixels
  write/key OVSCAN_ID/c/1/25     "OVERSCAN_TABLE"  ! Overscan output table
  write/key CHKCALT_ID/c/1/25    "CHKCAL_TABLE"    ! Statistics of frame consistency with master
  write/key COLTRNST_ID/c/1/25   "COL_TRANS_FORM"  ! Color transformation table
  write/key SEXINPAR_ID/c/1/25   "PHOTOMETRY_INPUT_PAR"  ! Sextractor input parameters
  write/key SEXCONVOL_ID/c/1/25  "PHOTOMETRY_CONVOL_MASK"  ! Sextractor convolution mask
  write/key SEXOUTPAR_ID/c/1/25  "PHOTOMETRY_OUTPUT_PAR"  ! Sextractor output parameters
  write/key SEXNEUPAR_ID/c/1/25  "PHOTOMETRY_NEURON_PAR"  ! Neuronal network weights
  write/key GRISMS_ID/c/1/25     "GRISM_PARAMETERS"  ! Physical parameters of grisms
  write/key FCAMERA_ID/c/1/25    "CAMERA_FLENGTH_TABLE"  ! Camera focal length table. Focal length vs. color
  write/key DISPQCT_ID/c/1/25    "DISPQUAL_TABLE"  ! Quality control table for dispersion solution
  write/key RESOLT_ID/c/1/25     "RESOL_TABLE"     ! Quality control table for lamp frames
  write/key CHKOBST_ID/c/1/25    "CHKOBS_TABLE"    ! Quality control table for spec. std. stars
  write/key CCDLINT_ID/c/1/25    "LIN_GAIN_TABLE"  ! CCD linearity table
  write/key CCDCTET_ID/c/1/25    "CCDCTE_TABLE"  ! CCD transfer efficiency table
  write/key STATISTT_ID/c/1/25   "STATISTICS_TABLE"  ! Statistics table of std./sci. frames
  write/key TMASTERT_ID/c/1/25   "TSTMASTER_TABLE"   ! Statistics of master/reference comparison
  write/key FOCT_ID/c/1/25       "FOCUS_TABLE"       ! Focal alignment table
  write/key PURGET_ID/c/1/25     "DESC_PURGE_TABLE"  ! Purging information for all product frame types
  write/key TEST_ID/c/1/25       "TEST"              ! General test frame
! Frames common for all modes
  write/key SCIENCE_ID/c/1/25   "SCIENCE_"          ! Science frame
  write/key RSCIENCE_ID/c/1/25  "SCIENCE_REDUCED_"  ! Reduced science frame
  write/key FSCIENCE_ID/c/1/25  "SCIENCE_FLXCAL_"  ! Flux calibrated science frame
  write/key ESCI_ID/c/1/25      "SCIENCE_EXTRACTED_" ! List of extracted science spectra
  write/key SCIBKG_ID/c/1/25      "SCIENCE_SKY_" ! Fitted sky background frame
  write/key SCISRC_ID/c/1/25      "SCIENCE_SOURCE_" ! Sky subtracted spectrum framei
  write/key STD_ID/c/1/25       "STANDARD_"          ! Standard star frame
  write/key RSTD_ID/c/1/25      "STANDARD_REDUCED_"  ! Reduced standard star frame
  write/key FLAT_ID/c/1/25      "SCREEN_FLAT_"       ! Screen flat field
  write/key RFLAT_ID/c/1/25     "REDUCED_SCREEN_FLAT_"  ! Reduced screen flat field
  write/key MFLAT_ID/c/1/25     "MASTER_SCREEN_FLAT_"   ! Master screen flat field
  write/key STDFLX_ID/c/1/25    "FLX_STD_"          ! Catalog of standard star fluxes/spectra
  write/key DRSRC_ID/c/1/25     "DRS_SETUP_"        ! Data reduction system setup file
! Frames for direct imaging
  write/key SKY_ID/c/1/25       "SKY_FLAT_"  ! Sky flat field
  write/key RSKY_ID/c/1/25      "REDUCED_SKY_FLAT_"  ! Reduced sky flat field
  write/key MSKY_ID/c/1/25      "MASTER_SKY_FLAT_"  ! Master sky flat field
  write/key SSKY_ID/c/1/25      "SUPER_SKY_FLAT_"  ! Super sky flat field
  write/key MNIGHT_ID/c/1/25    "MASTER_NIGHT_FLAT_"  ! Master night sky flat field
  write/key SNIGHT_ID/c/1/25    "SUPER_NIGHT_FLAT_"  ! Super night sky flat field
  write/key MASTERFF_ID/c/1/25  "MASTER_FLAT_"  ! Combined master screen and sky flat field
  write/key SKYBACK_ID/c/1/25   "PHOT_BACKGROUND_"  ! Sky background frame
  write/key SKYSUB_ID/c/1/25    "PHOT_BACKSUBTR_"  ! Sky subtracted frame
  write/key ZEROPOINT_ID/c/1/25 "ZEROPOINT_TABLE_"   ! Magnitude zero points
  write/key EXTCOEFF_ID/c/1/25 "EXTCOEFF_TABLE_"     ! Extinction coefficients
  write/key COLCOEFF_ID/c/1/25 "COLCOEFF_TABLE_"     ! Colour coefficients
  write/key FOCUS_ID/c/1/25     "FOCUS_"  ! Focus frame
  write/key CCDLIN_ID/c/1/25    "CCDLIN_"  ! Screen flatfield frame for linearity tests
  write/key CCDCTE_ID/c/1/25    "CCDCTE_"  ! Screen flatfield frame for transfer tests
! Frames for LSS, MOS and ECH
  write/key MNFLAT_ID/c/1/25    "MASTER_NORM_FLAT_"  ! Normalized master screen flat
  write/key NFLAT_ID/c/1/25     "NORM_FLAT_"  ! Normalized master screen flat
  write/key WLC_ID/c/1/25       "LAMP_"  ! Wavelength calibration frame
  write/key RWLC_ID/c/1/25      "REDUCED_LAMP_"  ! Reducend wavelength calibration frame
  write/key LINCAT_ID/c/1/25    "MASTER_LINECAT_"  ! Emission line catalog of calibration lamps
  write/key LINPOS_ID/c/1/25    "LINEPOS_"  ! Table of detected line positions
  write/key DISPREL_ID/c/1/25   "DISP_COEFF_"  ! Table of dispersion coefficients
  write/key ATMEXT_ID/c/1/25    "EXTCOEFF_TABLE_"   ! Atmospheric extinction coefficients (LSS,MOS)
  write/key ESTD_ID/c/1/25      "STANDARD_EXTRACTED_" ! List of extracted standard star spectra
  write/key ASTD_ID/c/1/25      "STANDARD_ALIGNED_" ! List of identified standard star spectra
  write/key SBKG_ID/c/1/25      "STANDARD_SKY_" ! Fitted sky background frame
  write/key SSRC_ID/c/1/25      "STANDARD_SOURCE_" ! Sky subtracted spectrum frame
  write/key RESP_ID/c/1/25      "INSTR_RESPONSE_"  ! Wavelength dependent instrument response
  write/key SLITS_ID/c/1/25     "SLIT_POSITIONS_"  ! Table of slit positions
! Frames for ECH
  write/key ORDER_ID/c/1/25     "ORDER_POS_"       ! Table with echelle order positions
  write/key BACK_ID/c/1/25      "BACKGROUND_POS_"  ! Table w. ech. interorder background positions

!
!*** NEVER CHANGE THE FOLLOWING LINE ***
!End Session List
!***************************************

  write/key inputi/i/1/10 0  ALL  ! keyword for integer input
  write/key inputr/r/1/10 0. ALL  ! keyword for real input
  write/key inputd/d/1/10 0. ALL  ! keyword for double input

  write/key SLITIDX/I/1/1 0       ! index of active slitlet with representative slitwidth


! Pipeline context directory
  p_ctxdir = m$symbol("PIPE_HOME") // "/" // m$lower(rbinstrument) // "/context"

! Pipeline binary directory
  p_bindir = m$symbol("PIPE_HOME") // "/" // m$lower(rbinstrument) // "/bin"


! Set pipeline delivery flag
  if m$existk("PIPELINE") .eq. 1 then
    p_mainonly = pipeline(9)
  else
    p_mainonly = -1
  endif


! Default values for array keywords
  p_supported(1) = "IMAGE"
  p_supported(2) = "SPECTRUM"
  p_supported(3) = "MOS"
  p_supported(4) = "INS-THROUGH"
  p_supported(5) = "ECHELLE"

  p_modes(1) = "IMG"
  p_modes(2) = "LSS"
  p_modes(3) = "MOS"
  p_modes(4) = "IMG"
  p_modes(5) = "ECH"

  fors_ltypes(1) = "HgCd"
  fors_ltypes(2) = "He"
  fors_ltypes(3) = "Ne"
  fors_ltypes(4) = "Ar"

  h_filti(1) = "ESO.INS.FILT1.NAME"
  h_filti(2) = "ESO.INS.FILT2.NAME"
  h_filti(3) = "ESO.INS.FILT3.NAME" ! at most 3 filter wheels
  h_filti(4) = "ESO.INS.FILT.NAME"  ! for TC

  FORS_PORTS(1:4) = "ABCD"

  h_pscan(1) = "ESO.DET.OUT1.PRSCX"
  h_pscan(2) = "ESO.DET.OUT2.PRSCX"
  h_pscan(3) = "ESO.DET.OUT3.PRSCX"
  h_pscan(4) = "ESO.DET.OUT4.PRSCX"

  h_oscan(1) = "ESO.DET.OUT1.OVSCX"
  h_oscan(2) = "ESO.DET.OUT2.OVSCX"
  h_oscan(3) = "ESO.DET.OUT3.OVSCX"
  h_oscan(4) = "ESO.DET.OUT4.OVSCX"

  h_pscany(1) = "ESO.DET.OUT1.PRSCY"
  h_pscany(2) = "ESO.DET.OUT2.PRSCY"
  h_pscany(3) = "ESO.DET.OUT3.PRSCY"
  h_pscany(4) = "ESO.DET.OUT4.PRSCY"

  h_oscany(1) = "ESO.DET.OUT1.OVSCY"
  h_oscany(2) = "ESO.DET.OUT2.OVSCY"
  h_oscany(3) = "ESO.DET.OUT3.OVSCY"
  h_oscany(4) = "ESO.DET.OUT4.OVSCY"

  h_confac(1) = "ESO.DET.OUT1.CONAD"
  h_confac(2) = "ESO.DET.OUT2.CONAD"
  h_confac(3) = "ESO.DET.OUT3.CONAD"
  h_confac(4) = "ESO.DET.OUT4.CONAD"

  h_ron(1) = "ESO.DET.OUT1.RON"
  h_ron(2) = "ESO.DET.OUT2.RON"
  h_ron(3) = "ESO.DET.OUT3.RON"
  h_ron(4) = "ESO.DET.OUT4.RON"

  h_prqcobi(1) = "ESO.QC.OUT1.OVBIAS"
  h_prqcobi(2) = "ESO.QC.OUT2.OVBIAS"
  h_prqcobi(3) = "ESO.QC.OUT3.OVBIAS"
  h_prqcobi(4) = "ESO.QC.OUT4.OVBIAS"

  h_prqcono(1) = "ESO.QC.OUT1.OVNOISE"
  h_prqcono(2) = "ESO.QC.OUT2.OVNOISE"
  h_prqcono(3) = "ESO.QC.OUT3.OVNOISE"
  h_prqcono(4) = "ESO.QC.OUT4.OVNOISE"

  set/format i1 
  do i = 1 {FORS_MAXSLITLET} 1
    h_mosra({i})    = "ESO.INS.MOS{i}.RA"
    h_mosdec({i})   = "ESO.INS.MOS{i}.DEC"
    h_moswid({i})   = "ESO.INS.MOS{i}.WID"
    h_moslen({i})   = "ESO.INS.MOS{i}.LEN"
    h_mospos({i})   = "ESO.INS.MOS{i}.POS"
    h_moswidth({i}) = "ESO.INS.MOS{i}.WIDTH"
  enddo
  set/format




else

! Remove keyword protection
  protect/fors fors 0
  savinit/fors {p1}KEYS.tbl read

endif


! Reset message level
if msgflg .eq. 1 then
  mess_level = msglev
endif


! Protect keywords
skplck = "MESS_LEVEL,FORS_INSMODE,fors_id" // -
         ",nice_crea,nice_dpos,nice_dlab,nice_glab,nice_dscr,nice_ddim"
skplck = skplck // -
         ",nice_gdim,nice_dids,nice_gids,nice_glabw,nice_hcopy,nice_disp"
skplck = skplck // -
         ",rbins_old,p_dspold,p_sizold,p_dspmode"
skplck = skplck // -
         ",slog_file,llog_file,llog_table,qc1log_file,qc1log_table,prohead_file"

protect/fors fors {FORS_LOCK}

return 0
