/* $Id: locate.c,v 1.1 1999/03/04 11:39:19 rpalsa Exp $   
 *
 *.IDENT      $RCSfile: locate.c,v $
 *
 *.KEYWORDS   returns the position of the bin within a data grid
 *
 *.USAGE      unsigned long locate(double s, double *t, unsigned long n)
 *
 *.INPUT      s      value to look for in the array
 *            t      array to be searched
 *            n      Number of array elements in t
 *
 *.OUTPUT     None
 *
 *.RETURNS    The function returns the position of the array element, i.e. 
 *            the bin number (see description below). If the search value is
 *            smaller than the first element of the array -1 is returned and
 *            -2 if the search value is bigger than the last array element.
 *
 *.PURPOSE    Performs a bisection of an array until two values t[i] and 
 *            t[i+1] are found in the grid so that for the a given value s
 *            the relation
 *                               t[i] <= s < t[i+1]
 *            is true. If s lies outside the array boundaries this is indicated
 *            by a negative return value. That is -1 if the search value is too
 *            small and -2 if the search value is too big.
 *
 *            Before locate is called the user has to be sure that the input
 *            array is monotonic!
 *
 *.ENVIRON     None
 *
 *.LANGUAGE    ANSI C/KR C
 *
 *.AUTHOR      R. Palsa, Univ.-Sternwarte Muenchen
 *
 *.VERSION     0.1 1996/03/21: Creation (RP)
 *             0.2 1999/02/28: Support KR C, Header changed to MIDAS
 *                             standard (RP)
 *
 *-----------------------------------------------------------------------------
 */

#ifdef __STDC__
unsigned long locate(double s, double *t, unsigned long n)
#else
unsigned long locate(s, t, n)
     double s, *t;
     unsigned long n;
#endif
{

  register int ascnd, low, mid, high;


  /*
   * test if the input array t is ascendent or descendent
   */

  ascnd = *(t + n - 1) > *t;


  /*
   * check if search value is out of range
   */

  if (ascnd) {
    if (s < *t)
      return -1;

    if (s >= *(t + n - 1))
      return -2;
  }
  else {
    if (s >= *t)
      return -2;

    if (s < *(t + n - 1))
      return -1;
  }


  /*
   * bisection
   */

  low = 0;
  high = n + 1;

  while (high - low > 1) {
    mid = (high + low) >> 1;
    if ((s >= *(t + mid - 1)) == ascnd)
      low = mid;
    else
      high = mid;
  }

  return low - 1;

}
