! $Id: fors_ad2xy.prg,v 1.6 1999/11/25 11:23:19 sbogun Exp $   
!+++ FORS_AD2XY +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_ad2xy.prg,v $
!
!.KEYWORDS      convert alpha,delta to x,y pixel position
!
!.USAGE         @p fors_ad2xy image
!
!.INPUT         image   (P1): name of image file
!               ra      (P2): RA        (alpha) decimal format [degrees]
!               dec     (P3): DEC       (delta) decimal format [degrees]
!
!.OUTPUT        OUTPUTI(1),OUTPUTI(2) frame pixel coordinates
!
!.RETURN        Q1:  0: successful return (valid pixel coordinates)
!                   -1: a successful return failed
!                    1: returned pixel coordinates are outside the frame!
!
!.DEFAULT
!
!.PURPOSE       Determine x,y-position from world coordinates ra,dec
!
!.ALGORITHM     Compute an X and Y position given the RA and DEC
!               A tangent (gnomonic) projection is computed directly.
!
!.ENVIRON       
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/12/01  Creation
!
!.COMMENT
!------------------------------------------------------------------------------

define/param P1 ? c "Enter name of image file (measurement):"
define/param P2 ? n "Enter RA: "
define/param P3 ? n "Enter DEC:"


define/local RADEG/d/1/1 0.0

define/local h/d/1/1 0.0
define/local ix/i/1/1 0.0
define/local iy/i/1/1 0.0
define/local x/d/1/1 0.0
define/local y/d/1/1 0.0
define/local ra/d/1/1 0.0
define/local dec/d/1/1 0.0
define/local radif/d/1/1 0.0
define/local xsi/d/1/1 0.0
define/local eta/d/1/1 0.0
define/local cdelt/d/1/2 0.0 ALL
define/local crpix/d/1/2 0.0 ALL
define/local crval/d/1/2 0.0 ALL
define/local cdmat/d/1/4 0.0 ALL +lower_levels
define/local cdinv/d/1/4 0.0 ALL
define/local coorsys/d/1/13 0.0 ALL
define/local ctype/c/1/80 " " ALL
define/local npix/i/1/2 0 ALL

ra  = {P2}
dec = {P3}

RADEG = 180.0/3.14159265358979323846            !Double precision RADEG

@p fors_extast {P1}

copy/keyw outputd/d/1/13 coorsys/d/1/13
ctype = outputc

if m$index(ctype,"TAN") .gt. 0 then
  cdelt(1) = coorsys(5)
  cdelt(2) = coorsys(6)

  crval(1) = coorsys(9) * RADEG                 !it has to be degrees
  crval(2) = coorsys(10) * RADEG                !it has to be degrees

  radif = ra - crval(1)

  h     = m$sin(dec)*m$sin(crval(2)) + m$cos(dec)*m$cos(crval(2))*m$cos(radif)

  xsi = m$cos(dec)*m$sin(radif)/h
  eta = (m$sin(dec)*m$cos(crval(2)) - m$cos(dec)*m$sin(crval(2))*m$cos(radif))
  eta = eta / h

  xsi = xsi*RADEG
  eta = eta*RADEG

else
  write/out "ERROR: image has no RA-TAN,DEC-TAN world coordinate system !"
  return -1
endif

xsi = xsi/cdelt(1)
eta = eta/cdelt(2)

crpix(1) = coorsys(7)
crpix(2) = coorsys(8)

cdmat(1) = coorsys(1)
cdmat(2) = coorsys(2)
cdmat(3) = coorsys(3)
cdmat(4) = coorsys(4)

@p fors_invert cdmat 2 2
copy/keyw outputd/d/1/4 cdinv/d/1/4

x = cdinv(1)*xsi + cdinv(2)*eta + crpix(1)
y = cdinv(3)*xsi + cdinv(4)*eta + crpix(2)

outputi(1) = m$nint(x)
outputi(2) = m$nint(y)

set/format I1 F12.5
write/out "world coordinates entered: {ra}, {dec}"
write/out "frame pixel determined:    {outputi(1)}, {outputi(2)}"
set/format

copy/dk {p1} NPIX npix
if outputi(1) .lt. 1 .OR. outputi(1) .gt. npix(1) return 1
if outputi(2) .lt. 1 .OR. outputi(2) .gt. npix(2) return 1
return 0
