! $Id: fors_addexttbl.prg,v 1.2 1999/03/16 05:56:06 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_addexttbl.prg,v $
!
!.KEYWORDS    Add data set to extinction correction factor table
!
!.USAGE       add/exttbl extfrm index exttbl
!
!.INPUT       extfrm      (P1)    1d image of wavelength dependent extinction
!                                 correction factors.
!             index       (P2)    Row number of correction factor table where
!                                 extinction data will be stored.
!
!.OUTPUT      exttbl      (P3)    Output extinction correction factor table.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The procedure writes the extinction correction factors
!             stored in the 1d input frame into a table element of the
!             3d output table. The table row has to be given by the index
!             parameter. The output table elements must be large enough
!             to store the extinction data.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/12/02:  Creation
!
!------------------------------------------------------------------------------

crossref  extfrm index exttbl

define/param p1 ? c "Extinction correction factor frame:"
define/param p2 ? c "Table row number:"
define/param p3 ? c "Extinction correction factor table:"
define/maxpar 3


! Procedure identification
define/local procID/c/1/20      "AddExttbl"


! Local variables
define/local row/i/1/1       0
define/local ndata/i/1/1     0
define/local extfrm/c/1/100  {p1}
define/local exttbl/c/1/100  {p3}



! Process parameters
if m$tstno(p2) .ne. 1 then
  mess/out 1 {procID} "Invalid table row!"
  return 1
else
  row = {p2}
  if row .lt. 1 then
    mess/out 1 {procID} "Table row is out of range!"
    return 1
  endif
endif

if m$value({extfrm},naxis) .gt. 1 then
  mess/out 1 {procID} "Invalid input extinction data frame!"
  return 1
endif


! Check existence of output table and columns START, STEP and NDATA in
! extinction data table.
if m$exist(exttbl) .ne. 1 then
  mess/out 1 {proID} "Output table >{exttbl}< does not exist!"
  return 1
else
  if m$existc(exttbl,":{c_xtstart}") .lt. 0 then
    mess/out 1 {procID} "Column :{c_xtstart} not found in {exttbl}!"
    return 1
  else if m$existc(exttbl,":{c_xtstep}") .lt. 0 then
    mess/out 1 {procID} "Column :{c_xtstep} not found in {exttbl}!"
    return 1
  else if m$existc(exttbl,":{c_xtndata}") .lt. 0 then
    mess/out 1 {procID} "Column :{c_xtndata} not found in {exttbl}!"
    return 1
  endif
endif


! Write extinction data to output table element
{exttbl},:{c_xtstart},{row} = m$value({extfrm},start(1))
{exttbl},:{c_xtstep},{row} = m$value({extfrm},step(1))

ndata = m$value({extfrm},npix(1))
{exttbl},:{c_xtndata},{row} = {ndata}

copy/t3d {extfrm} 1 1,{ndata} {exttbl} :{c_xtratio} {row}
if {q1} .ne. 0 then
  mess/out 1 {procID} "Cannot write extinction data to output table element!"
  return 1
endif

return 0
