! $Id: fors_adjustsea.prg,v 1.7 2001/01/19 14:37:33 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_adjustsea.prg,v $
!
!.KEYWORDS    Setup detection parameters for emission line search,
!             wavelength calibration, spectroscopy
!
!.USAGE       adjust/search wfrm [spar] [slits]
!
!.INPUT       wfrm        (P1)    Wavelength calibration frame
!             spar        (P2)    Search parameters: search threshold
!                                 factor and search window in terms of
!                                 slit width.
!             slits       (P3)    Input slit position table
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. The obtained threshold and
!             window width is returned as OUTPUTR(1) and OUTPUTR(2)
!             respectively.
!
!.PURPOSE     The procedure scales the first mode of the normalized calibration 
!             frame by the search threshold factor and determines the search 
!             window in pixel.
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/02/28:  Creation
!
!------------------------------------------------------------------------------

crossref  wfrm spar slits

define/param p1 ? c "Enter calibration frame:"
define/param p2 + c "Enter search parameters:"
define/param p3 + c "Enter input slit table:"

define/maxpar 3


! Procedure identification
define/local procID/c/1/80	"AdjustSearch"


! Local constants
define/local smin/r/1/1   5.
define/local TMPSLIT/c/1/14  "midslitpos.tbl"
define/local TMPSTAT/c/1/14  "midstat.tbl"


! Local variables
define/local pos1/i/1/1     0
define/local pos2/i/1/1     0
define/local sthresh/r/1/1  0.
define/local swindow/r/1/1  0.
define/local stab/c/1/80    " "   all
define/local wfrm/c/1/100   {p1}
define/local tmp_h_slwidth/c/1/60   " "
define/local obsmode/c/1/10 " "   all
define/local starty/r/1/1   0.
define/local stepy/r/1/1    0.



! Process parameters
pos1 = m$index(seapar,",") - 1

if p2(1:1) .eq. "+" then
  if pos1 .le. 0 then
    mess/out 1 {procID} "Invalid search parameters in session keyword!"
    return 1
  else
    sthresh = {seapar(1:{pos1})}
    pos1 = pos1 + 2
    swindow = {seapar({pos1}:)}
  endif
else
  pos2 = m$index(p2,",") - 1
  if pos2 .lt. 0 then
    sthresh = {p2}
    pos1 = pos1 + 2
    swindow = {seapar({pos1}:)}
  elseif pos2 .eq. 0 then
    sthresh = {seapar(1:{pos1})}
    swindow = {p2(2:)}
  else
    sthresh = {p2(1:{pos2})}
    pos2 = pos2 + 2
    swindow = {p2({pos2}:)}
  endif
endif

if p3 .ne. "+" then
  stab = p3
else
  stab = mos
endif


obsmode = m$value({wfrm},{h_insmode})
if obsmode .EQ. p_modes(2) then               
! if "LSS" ....
  tmp_h_slwidth = h_slwidth
else if obsmode .EQ. p_modes(3) then          
! if "MOS" ....
  set/format i1 
  tmp_h_slwidth = h_moswid({slitidx})          ! slitwidth in arcsec
											   ! slitidx defined by define/position
else
  mess/out 1 {procID} "Unexpected instrument mode in {wlc}: {obsmode} !" 
  return 1
endif


! Scale threshold value by first mode value
! (within active slits, mean of all first modes):
-copy {stab} {TMPSLIT}

create/column {TMPSLIT} :XSTART "World" F7.1 R*4
create/column {TMPSLIT} :XEND   "World" F7.1 R*4
@a getcoord {wfrm} @1,@1 >Null
compute/table {TMPSLIT} :XSTART = {outputd(1)}
@a getcoord {wfrm} @{{wfrm},NPIX(1)},@1 >Null
compute/table {TMPSLIT} :XEND   = {outputd(1)}

statistics/image {wfrm} {TMPSLIT} #256 ? fnnn {TMPSTAT} >Null
statistics/table {TMPSTAT} :SMAL_MODE >Null
sthresh = sthresh * outputr(3)
if sthresh .le. 0. then
  set/format i1 f1.2
  mess/out 1 {procID} "Invalid search threshold {sthresh} obtained!"
  set/format
  return 1
endif
-delete {TMPSLIT}
-delete {TMPSTAT}


! Convert search window
swindow = swindow * m$value({wfrm},{tmp_h_slwidth}) / m$value({wfrm},{h_pixscale})
if swindow .lt. smin then
  set/format i1 f1.2
  mess/out 2 {procID} "Search window is too small ({swindow} pxl)."
  mess/out 2 {procID} "Search window adjusted to minimum value {smin} pxl!"
  set/format
  swindow = smin
endif

outputr(1) = sthresh
outputr(2) = swindow

return 0


