! $Id: fors_annotatespe.prg,v 1.4 1999/09/17 16:19:04 jreetz Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_annotatespe.prg,v $
!
!.KEYWORDS    Annotate 2-d object spectrum
!
!.USAGE       annotate/spectrum  obj dwnd title top
!
!.INPUT       obj      (P1)   Object spectrum frame 
!             dwnd     (P2)   Display window ID to be used.
!             title    (P3)   Title string
!             tmargin  (P4)   Top margin for setup information text
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Annotate object spectrum frame with some instrument setup
!             information. The title string may contain spaces (must be
!             enclosed by double quotes). 
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/01/05:  Creation
!
!------------------------------------------------------------------------------

crossref obj dwnd tmargin


define/param p1 ?       c  "Enter name of calibration frame:"
define/param p2 0       c  "Enter display window ID:"
define/param p3 +       c  "Enter title string:"
define/param p4 +       c  "Enter top margin:"
define/maxpar 4


! Procedure identification
define/local procID/c/1/80      "AnnotateSpec"


! Local constants
define/local loffset/i/1/1   20


! Local variables
define/local i/i/1/1        0
define/local n/i/1/1        0
define/local didx/i/1/1     0
define/local lypos/i/1/1    0
define/local lfont/i/1/1    0
define/local lcolor/c/1/10  " "   all
define/local label/c/1/100  " "   all
define/local frm/c/1/100    {p1}
define/local title/c/1/100  {p3}
define/local top/c/1/100    {p4}



! Process parameters
if m$tstno(p2) .eq. 1 then
  didx = {p2}
endif


! Check availability of display window
if nice_dids({didx}) .ne. 1 then
  mess/out 2 {procID} "Requested display window is not active!"
  return 0
endif


! Get display y-size
lypos = ididev(3)


! Get current label font and color
config/label
if {q1} .ne. 0 then
  mess/out 2 {procID} -
           "Cannot update label font and color. Using defaults!"
endif
lfont = outputi(1)
lcolor = outputc


! Display title string
if title(1:1) .ne. "+" then
  if top(1:1) .eq. "+" then
    lypos = ididev(3) - loffset
  else
    lypos = ididev(3) - m$nint({top})
  endif
  label/display "{title}" {lypos},5 O {lcolor} {lfont}
else
  if top(1:1) .ne. "+" then
    lypos = ididev(3) - m$nint({top})
  endif
endif


! Display grism, slit and filter setup at top of screen. Additionally
! the lamp setup is displayed if a lamp is indicated by fors_nlamp > 0.
lypos = lypos - loffset
label = m$value({frm},{h_grism1})
label/display "Grism: {label}" {lypos},5 O {lcolor} {lfont}

if m$existd(frm,h_slit) .eq. 1 then
  lypos = lypos - loffset
  label = m$value({frm},{h_slit})
  label/display "Slit: {label}" {lypos},5 O {lcolor} {lfont}
endif

if fors_nlamp .gt. 0 then
  lypos = lypos - loffset
  label = "Lamp: "
  n = fors_nlamp - 1
  if n .gt. 0 then
    do i = 1 n
      label = label // fors_lampi({i}) // ","
    enddo
  endif
  label = label // fors_lampi({fors_nlamp})
  label/display "{label}" {lypos},5 O {lcolor} {lfont}
endif

lypos = lypos - loffset
if fors_nfilt .gt. 0 then
  label = "Filter: "
  n = fors_nfilt - 1
  if n .gt. 0 then
    do i = 1 n
      label = label // fors_filti({i}) // ","
    enddo
  endif
  label = label // fors_filti({fors_nfilt})
else
  label = "Filter: None"
endif
label/display "{label}" {lypos},5 O {lcolor} {lfont}

return 0
