! $Id: fors_applycalib.prg,v 1.6 2000/08/01 12:47:52 nrainer Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 2000  European Southern Observatory (ESO)
!
! .IDENT      $RCSfile: fors_applycalib.prg,v $
!
! .KEYWORDS   Wavelength calibration
!
! .PURPOSE    Calculate the wavelength for a certain xy-pixel on the CCD.
! __________
!
! .USAGE      @p fors_applycalib pixel mos [linfit] [show]
!
! .INPUT      pixel  (P1): x- AND y-CCD-pixel value, separated by a comma;
!                            default: none
!
!             mos    (P2): table with slit positions;
!                            default: none
!
!             linfit (P3): table with dispersion solution;
!                            default: fors_coerbr.tbl
!
!             show   (P4): flag for result output on terminal: <y>:show, <n>o;
!                            default: <y>
!
! .OUTPUT     OUTPUTI(1): resulting slit number for entered y-CCD-pixel value
!             OUTPUTR(1): resulting wavelength for entered x-,y-CCD-pixel values
!
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  The input pixel values are normalized as it was done for
!             the computation of the dispersion solution and then
!             inserted into the dispersion equation.
!
! .COMMENT    * The dispersion solution must have been computed with
!               'calibrate/grism' (="moscalifors.prg").
!             * The slit position table must have been selected with
!               'define/position' (="fors_defpositions.prg").
!
! .ENVIRON    MOS context for 'calibrate/grism'.
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.5  00/06/29  Creation
!                  00/       Release
!
!------------------------------------------------------------------------------
!
!
crossref pixel mos linfit show
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "applycalib"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      *****************************************************************"
  w/o "      pixel  (P1): x- AND y-CCD-pixel value, separated by a comma;"
  w/o "                     default: none"
  w/o " "
  w/o "      mos    (P2): table with slit positions;"
  w/o "                     default: none"
  w/o " "
  w/o "      linfit (P3): table with dispersion solution;"
  w/o "                     default: fors_coerbr.tbl"
  w/o " "
  w/o "      show   (P4): flag for result output on terminal: <y>:show, <n>o;"
  w/o "                     default: <y>"
  w/o "      *****************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(pixel):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter x-pixel,y-pixel [def.:none]: "
outputi = m$parse(p1,"pix")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P1(pixel) !"
  goto error
endif
if m$tstno(pix01) .ne. 1 then
        write/out " "
        mess/out 1 {PROID} "1st element of P1(x-pixel) is no number !"
  goto error
endif
if m$tstno(pix02) .ne. 1 then
        write/out " "
        mess/out 1 {PROID} "2nd element of P1(y-pixel) is no number !"
  goto error
endif
define/local xpix/r/1/1 {pix01}
define/local ypix/r/1/1 {pix02}
if xpix .le. 0. then
        write/out " "
        mess/out 1 {PROID} "1st element of P1(x-pixel) is <= 0. !"
  goto error
endif
if ypix .le. 0. then
        write/out " "
        mess/out 1 {PROID} "2nd element of P1(y-pixel) is <= 0. !"
  goto error
endif
!
!
!P2(mos):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter table with slit positions [def.:none]: "
define/local mos/c/1/64 "{p2}"
if m$index(mos,".tbl") .eq. 0 mos = mos//".tbl"
if m$exist(mos) .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered tbl. w. slit positions is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({mos})"
  goto error
endif
if m$existc(mos,":SLIT") .eq. -1 then
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({mos},:SLIT)"
  goto error
elseif m$existc(mos,":YSTART") .eq. -1 then
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({mos},:YSTART)"
  goto error
elseif m$existc(mos,":YEND") .eq. -1 then
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({mos},:YEND)"
  goto error
endif
!
!
!P3(linfit):
!----------
define/parameter p3 fors_coerbr.tbl ? -
  "        Enter table with dispersion solution [def.:fors_coerbr.tbl]: "
define/local linfit/c/1/64 "{p3}"
if m$index(linfit,".tbl") .eq. 0 linfit = linfit//".tbl"
if m$exist(linfit) .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered tbl. w. dispers. solution is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({linfit})"
  goto error
endif
if m$existd(linfit,"NORMCOORD") .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered desc. w. fit degrees is not present"
        mess/out 1 {PROID} "in tbl. w. dispers. solution !"
        mess/out 1 {PROID} "({linfit},NORMCOORD)"
  goto error
endif
if m$existd(linfit,"CENTERXY") .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered desc. w. norm. constants is not present"
        mess/out 1 {PROID} "in tbl. w. dispers. solution !"
        mess/out 1 {PROID} "({linfit},CENTERXY)"
  goto error
endif
!
!
!P4(show):
!--------
define/parameter p4 y ? -
  "        Flag for result output on terminal [def.:<y>|<n>]: "
define/local show/c/1/1 "{p4}"
!
!
!
!          DETERMINATION OF SLIT BELONGING
!          TO ENTERED Y-POSITION (MOS) :
!          *******************************
!
define/local mid$selidx/i/1/1 0 ? +low
select/table {mos} :YSTART.le.{ypix}.and.:YEND.ge.{ypix} >Null
if outputi .eq. 0 then
        set/format F7.2
        write/out "      "
        mess/out 1 {PROID} "Entered y-pixel value ({ypix})"
        mess/out 1 {PROID} "not within active slit !"
        set/format {FMTDEF}
  goto error
elseif outputi .gt. 1 then
        write/out "      "
        mess/out 1 {PROID} "Tbl. w. dispers. solution not properly selected -"
        mess/out 1 {PROID} "must be done with <define/position> before !"
        mess/out 1 {PROID} "({mos})"
  goto error
endif
define/local slitno/i/1/1 0
slitno = {{mos},:SLIT,@{mid$selidx}}
select/table {mos} all
!
if m$existd(linfit,"COEFSLIT{slitno}") .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered desc. w. dispers. solution is not present"
        mess/out 1 {PROID} "in tbl. w. dispers. solution !"
        mess/out 1 {PROID} "({linfit},COEFSLIT{slitno})"
  goto error
endif
!
!
!          READING OF DESCRIPTORS :
!          **********************
!
define/local xdeg/i/1/1 0
                  xdeg = {{linfit},NORMCOORD(2)}-1
if xdeg .lt. 1 then
        write/out "      "
        mess/out 1 {PROID} "x-order <1 ! ({linfit},NORMCOORD(2)-1)"
  goto error
endif
define/local ydeg/i/1/1  {{linfit},NORMCOORD(4)}
if ydeg .lt. 1 then
        write/out "      "
        mess/out 1 {PROID} "y-order <1 ! ({linfit},NORMCOORD(4))"
  goto error
endif
define/local xydeg/i/1/1 {{linfit},NORMCOORD(3)}
if xydeg .lt. 1 then
        write/out "      "
        mess/out 1 {PROID} "mixed order <1 ! ({linfit},NORMCOORD(3))"
  goto error
endif
!
define/local xref/r/1/1  {{linfit},CENTERXY(1)}
define/local yref/r/1/1  {{linfit},CENTERXY(2)}
define/local norm/r/1/1  {{linfit},CENTERXY(3)}
!
define/local ncoef/i/1/1 0
                  ncoef = 1+xdeg+ydeg+xydeg
define/local coef/r/1/{ncoef} 0. all
                  copy/dk {linfit} COEFSLIT{slitno} coef
!
!
!          NORMALIZING OF INPUT PIXEL VALUES :
!          *********************************
!
define/local xnorm/r/1/1 0.
                  xnorm = (xpix-xref)/norm
define/local ynorm/r/1/1 0.
                  ynorm = (ypix-yref)/norm
!
!
!          APPLYING DISPERSION SOLUTION :
!          ****************************
!
define/local degloop/i/1/1 0
define/local coefloop/i/1/1 0
define/local lambda/r/1/1 0.
  lambda = coef(1)
do degloop = 1 xdeg
  coefloop = 1+degloop
  lambda = lambda +coef({coefloop})*(xnorm**degloop)
enddo
do degloop = 1 ydeg
  coefloop = 1+xdeg+degloop
  lambda = lambda +coef({coefloop})*(ynorm**degloop)
enddo
do degloop = 1 xydeg
  coefloop = 1+xdeg+ydeg+degloop
  lambda = lambda +coef({coefloop})*(xnorm*ynorm**degloop)
enddo
!
!
!          OUTPUT :
!          ******
!
if lambda .ge. 0. then
  if show .ne. "n" then
        set/format I2 T7.2,T8.2
        outputd = lambda
        mess/out 3 {PROID} "  x-pixel y-pixel => slit#  wavelength"
        mess/out 3 {PROID} "  {xpix} {ypix}       {slitno}  {outputd} A"
        set/format {FMTDEF}
  endif
else
        set/format I2 T7.2,T8.2
        outputd = lambda
        mess/out 3 {PROID} "  x-pixel y-pixel => slit#  wavelength"
        mess/out 3 {PROID} "  {xpix} {ypix}       {slitno}  {outputd} A"
        mess/out 2 {PROID} "  Negative wavelength for this pixel !"
        set/format {FMTDEF}
endif
write/keyword outputi 0 all
write/keyword outputr 0. all
write/keyword outputd 0. all
outputi(1) = slitno
outputr(1) = lambda
!
!
set/format
!
return 0
!
error:
  return 1
!
!
!------------------------------------------------------------------------------
