! $Id: fors_ascii2fits.prg,v 1.19 2001/04/10 09:29:03 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_ascii2fits.prg,v $
!
! .KEYWORDS   Format conversion
!
! .PURPOSE    Transform SExtractor ASCII files to FITS files for CalibDb.
! __________
!
! .USAGE      @p fors_ascii2fits asciifile procatg
!
! .INPUT      asciifile (P1): ASCII table from SExtractor;
!                              default: none
!
!             procatg   (P2): descriptor update, shortcuts:
!                             i=PHOTOMETRY_INPUT_PAR,
!                             c=PHOTOMETRY_CONVOL_MASK,
!                             o=PHOTOMETRY_OUTPUT_PAR
!                             n=PHOTOMETRY_NEURON_PAR
!
! .OUTPUT     <P1>.tfits: FITS table
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    -
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/06  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref asciifile procatg
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "ascii2fits"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      asciifile (P1): ASCII table from SExtractor (def.ext. ".dat");"
  w/o "                       default: none"
  w/o " "
  w/o "      procatg   (P2): descriptor update, shortcuts:"
  w/o "                      i=PHOTOMETRY_INPUT_PAR,"
  w/o "                      c=PHOTOMETRY_CONVOL_MASK,"
  w/o "                      o=PHOTOMETRY_OUTPUT_PAR"
  w/o "                      n=PHOTOMETRY_NEURON_PAR"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
        if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? "        Enter ASCII file (def.ext. ".dat"): "
define/local asciifile/c/1/64 "{p1}"
if m$indexb(asciifile,".") .eq. 0 asciifile = asciifile//".dat"
!
if m$exist(asciifile) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file is not present !"
        mess/out 1 {PROID} "({asciifile})"
  goto error
endif
!
if m$filtyp(asciifile," ") .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file is no ascii file !"
        mess/out 1 {PROID} "({asciifile})"
  goto error
endif
!
!
if p2 .eq. "?" then
  mess/out 2 {PROID} "Enter value of FITS desc. `PRO.CATG' ..."
  mess/out 2 {PROID} "        shortcuts: <i> = `PHOTOMETRY_INPUT_PAR'"
  mess/out 2 {PROID} "                   <c> = `PHOTOMETRY_CONVOL_MASK'"
  mess/out 2 {PROID} "                   <o> = `PHOTOMETRY_OUTPUT_PAR'"
  mess/out 2 {PROID} "                   <n> = `PHOTOMETRY_NEURON_PAR'"
endif
define/parameter p2 ? ? "                        ... : "
define/local procatg/c/1/30 "{p2}"
if procatg .eq. "i" procatg = "PHOTOMETRY_INPUT_PAR"
if procatg .eq. "c" procatg = "PHOTOMETRY_CONVOL_MASK"
if procatg .eq. "o" procatg = "PHOTOMETRY_OUTPUT_PAR"
if procatg .eq. "n" procatg = "PHOTOMETRY_NEURON_PAR"
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local char/i/1/1 0
  define/local fitsfile/c/1/64 " " all
  define/local formcode/i/1/2 0 all
!
!
!          ASCII -> FITS :
!          *************
!
  char = m$indexb(asciifile,".")-1
  fitsfile = asciifile(1:{char})//".tfits"
  !
        mess/out 4 {PROID} "converting: {asciifile} -> {fitsfile}"
!
! ASCII -> TBL :
open/file sex.fmt write formcode
write/file {formcode(1)} define/field 1 160 C*161 A161 :sex_file " "
close/file {formcode(1)}
create/table tmp.tbl 1 1000 {asciifile} sex.fmt
-delete sex.fmt
!
create/image empty.bdf 0,0 0.,0. nodata +
write/desc empty.bdf ESO.DPR.TECH/c/1/8 "IMAGE"
write/desc empty.bdf ESO.PRO.CATG/c/1/30 "{procatg}"
write/desc empty.bdf INSTRUME/c/1/11 "FORS1"
!
! TBL -> FITS :
outdisk/sfits empty.bdf,tmp.tbl {fitsfile} onn
-delete empty.bdf
-delete tmp.tbl
!
!
set/format
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------








