! $Id: fors_bindif.prg,v 1.16 1999/11/25 11:23:47 sbogun Exp $   
!+++ FORS_BINDIF ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_bindif.prg,v $
!
!.KEYWORDS      Determine mean and sigma image of input frame within bins
!
!.USAGE         @p fors_bindif ref single binx biny kappa init plot
!
!.INPUT         ref     (P1): reference image (or just image to be binned)
!               single  (P2): image which should be compared with reference,
!                             set to + if binning should be done only on P1
!               binx    (P3): binning in x direction:
!                               >0: number of bins in x-direction
!                               <0: size of bin in x-direction
!               biny    (P4): binning in y direction:
!                               >0: number of bins in y-direction
!                               <0: size of bin in y-direction
!               kappa   (P5): kappa of kappa-sigma clipping
!               init    (P6): set to 1 to create output images on disk
!                             set to 0 in case output images can be 'refilled'
!                               (that makes sense as soon as a series of input
!                                frames will be processed then only for the
!                                first input frame init has to be set to 1)
!               scale1  (P7): scale first image by 1.0/scale1
!               scale2  (P8): scale second image by 1.0/scale2
!
!.OUTPUT        out_a: name of output frame: means per bin
!               out_b: name of output frame: sigmas per bin
!               outputd: mean of each quadrant
!
!.RETURN        Q1: >=0: successful return: number of outlayers found
!                    <0: a successful return failed
!
!.DEFAULT       P3      64
!               P4      64
!               P5      7
!               P6      0
!               P7      1
!               P8      1
!
!.PURPOSE       Do a binning of a scaled difference frame or of first input
!               frame P1 in case P2 is set to +.
!
!.ALGORITHM     . Create a dummy output frame of required size (bin-sizes) as
!               soon as the initialization flag is set (P6=1).
!               . In case STATISTIC does not exist in P1 then MIDAS command
!               STATISTIC/IMAGE will be called with exact median calculation.
!               . In case CCDSCALE does not exist in P2 then CCDSCALE is set
!               to 1.0.
!               
!               . Call program fors_bindif.exe which calculates the mean and
!               its sigma of each bin. This will be stored in the already
!               created frames. Furthermore, the mean of each quadrant will
!               be determined and passed to this procedure through OUTPUTD.
!               Moreover, a kappa-sigma clipping allows to remove cosmics. It
!               is enabled if a kappa>0 is specified. The sigma is ment as the
!               median sigma of the median of each bin.
!                       Before all these calculations are made, the scaled
!               difference frame will be computed if P2 is not set to +:
!                       diff = P1 / SCALE1 - P2 / SCALE2
!
!               . Finally the two output images will be displayed if P7=1.
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/09/16  Creation
!
!.COMMENT
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name of master frame:"
define/param p2 ? c "Enter name of single frame:"
define/param p3 64 n "Enter NBINX:"
define/param p4 64 n "Enter NBINY:"
define/param p5 7 n "KAPPA of kappa-sigma clipping:"
define/param p6 0 n "Enter 1 for initialisation: "
define/param p7 1.0 n "Scale factor of first image:"
define/param p8 1.0 n "Scale factor of second image:"

define/local MDIF/r/1/1 1.0
define/local SCALA/r/1/1 1.0
define/local SCALB/r/1/1 1.0
define/local MASTERM/r/1/1 1.0
define/local DEGREE/i/1/1 2
define/local DEG1/i/1/1 2
define/local KAPPA/i/1/1 0
define/local IMSEC/i/1/10
define/local IM_SEC/c/1/100 " " ALL
define/local i1/i/1/1 0
define/local j1/i/1/1 0
define/local k1/i/1/1 0
define/local nx/i/1/1 0
define/local ny/i/1/1 0
define/local nrow/i/1/1 0
define/local nbinx/i/1/1 {P3}
define/local nbiny/i/1/1 {P4}
define/local nout/i/1/1 0
define/local dx/r/1/1 0
define/local dy/r/1/1 0
define/local x1/r/1/1 0
define/local y1/r/1/1 0
define/local x2/r/1/1 0
define/local y2/r/1/1 0
define/local quadrant/d/1/4 0.0 ALL

configure/ports {P1}
image/section {P1} X
copy/keyw OUTPUTI/i/1/10 IMSEC/i/1/10
IM_SEC = OUTPUTC

nx = OUTPUTI(3) - OUTPUTI(1) + 1        !size of whole used image section:
ny = OUTPUTI(4) - OUTPUTI(2) + 1        !  nx x ny

if nbinx .lt. 0 .AND. nbiny .lt. 0 then
  dx = -nbinx
  dy = -nbiny
  nbinx = nx / dx
  nbiny = ny / dy
else
  dx = nx / nbinx
  dy = nx / nbiny
endif
if dx .lt. 2 .OR. dy .lt. 2 then
  mess/out 1 bindif -
        "Probably wrong binning selected.\nWidth of bins are less than 2!"
  return -2
endif

if {P6} .eq. 1 then
  mess/out 4 bindif "    create output images: mean.bdf, sigma.bdf"
  create/image mean.bdf 2,{nbinx},{nbiny} 1,1,{dx},{dy} POLY 0,0
  mean.bdf,STEP(1) = m$value({P1},STEP(1)) * m$value(mean.bdf,STEP(1))
  mean.bdf,STEP(2) = m$value({P1},STEP(2)) * m$value(mean.bdf,STEP(2))
  mean.bdf,START(1) = IMSEC(5) * IMSEC(9)  + 1 + 0.5*m$value(mean.bdf,STEP(1))
  mean.bdf,START(2) = IMSEC(7) * IMSEC(10) + 1 + 0.5*m$value(mean.bdf,STEP(2))
  mean.bdf,IDENT = "BINNED_FRAME                   "
  -copy mean.bdf medi.bdf
  -copy mean.bdf sigma.bdf
endif

SCALA = {P7}
if SCALA .eq. 0 then
  mess/out 1 bindif "BAD mean of master frame `{P1}`: {SCALA}"
  return -1
endif

SCALB = {P8}
if SCALB .eq. 0 then
  mess/out 1 bindif "BAD scale of single frame `{P2}`: {SCALB}"
  return -1
endif

if P2(1:1) .ne. "+" then
  mess/out 4 bindif "    perform binning on difference frame ..."
else
  mess/out 4 bindif "    perform binning on input frame: `{P1}` ..."
endif

IN_A = "{P1}"
IN_B = "{P2}"
OUT_A = "mean.bdf"
OUT_B = "sigma.bdf"
INPUTI(1) = fors_nout
KAPPA = {P5}

run PIPE_EXE:fors_bindif.exe
nout = outputi(1)                               !number of outlayers
if nout .lt. 0 return -1                        !error occured
copy/keyw outputd/d/1/4 quadrant/d/1/4          !mean in each quadrant



out_a = "mean.bdf"
out_b = "sigma.bdf"
return {nout}
