! $Id: fors_calcoeff_check.prg,v 1.12 2000/12/01 16:25:02 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 2000  European Southern Observatory (ESO)
!
! .IDENT      fors_calcoeff_check.prg
!
! .KEYWORDS   Calibration coefficients
!
! .PURPOSE    Checking of thresholds and indexing of stars.
! __________
!
! .USAGE      @p fors_calcoeff_check type
!
! .INPUT      Parameter:
!             ---------
!             type (P1): <e>xt: check all but colour related values,
!                        <c>ol: check only colour related values;
!                           default: none
!
!             "+lower_level" variables from "fors_calcoeff.prg":
!             -------------------------------------------------
!             ncoef
!
!             hiqualtbl
!             tblfilt
!             if ncoef=0 :
!               outidx
!               colour
!
!             min_stdmea
!             airbinsize
!             min_noairs
!             min_airrng
!             min_nstars
!             if ncoef=0 :
!               colbinsize
!               min_nocols
!               min_colrng
!
!             if ncoef=0 :
!               COLACC
!
! .OUTPUT     {hiqualtbl}: - column :IDX is added
!                          - stars not fulfilling the thresholds are indexed
!                            with a negative number
!
!             Q2: 0: table is O.K.
!                 1: table should be skipped because
!                    - either too few stars are left after checking per star
!                        | whether too few measurements had been used to derive
!                          the std.mags.
!                        | whether too few different airmasses are present
!                        | whether the range of the airmasses is too low
!                    - or too few colours are present
!                    - or the range of the colours is too low
!
!             ONLY FOR type(P1)=e :
!             Q3: number of left stars fulfilling the thresholds
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  For the calculation of the number of different airmasses and
!             different colours bins are used ({airbinsize},{colbinsize}).
!
! .COMMENT    Special procedure for calib.coeffs. - not to be used generally !
!
!             Used UNIX command:
!             $ /bin/echo (### as long as mess/out is still too slow)
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.5  00/01/19  Creation
!                  00/       Release
!
!------------------------------------------------------------------------------
!
!
crossref type
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "calcoeff_check"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
write/keyword q3 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      Parameter:"
  w/o "      ---------"
  w/o "      type (P1): <e>xt: check all but colour related values,"
  w/o "                 <c>ol: check only colour related values;"
  w/o "                    default: none"
  w/o " "
  w/o "      "+lower_level" variables from "fors_calcoeff.prg":"
  w/o "      -------------------------------------------------"
  w/o "      ncoef"
  w/o " "
  w/o "      hiqualtbl"
  w/o "      tblfilt"
  w/o "      if ncoef=0 :"
  w/o "        outidx"
  w/o "        colour"
  w/o " "
  w/o "      min_stdmea"
  w/o "      airbinsize"
  w/o "      min_noairs"
  w/o "      min_airrng"
  w/o "      min_nstars"
  w/o "      if ncoef=0 :"
  w/o "        colbinsize"
  w/o "        min_nocols"
  w/o "        min_colrng"
  w/o " "
  w/o "      if ncoef=0 :"
  w/o "        COLACC"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT (PARAMETER) :
!          *****************
!
!P1(type):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Type of checks [<e>xt.|<c>ol.]: "
define/local type/c/1/1 "{p1}"
!
if type .ne. "e" .and. type .ne. "c" then
        write/out "      "
        mess/out 1 {PROID} "Entered check type does not exist !"
        mess/out 1 {PROID} "({p1})"
  goto error
endif
!
!
!          INPUT (USED "+LOWER_LEVEL" VARIABLES) :
!          *************************************
!
! See header of this procedure.
!
!
!
!          DEFINITION OF FURTHER VARIABLES :
!          *******************************
!
  define/local chkloop/i/1/1 0
  define/local allrows/i/1/1 0
  allrows = {{hiqualtbl},TBLCONTR(4)}
  !
  !   "select one object" :
  define/local selrows/i/1/1 0
  define/local lastsel/i/1/1 0
  !
  !   "bright. from enough meas." :
  define/local badflg/i/1/1 0
  define/local badthrcnt/i/1/3 0 all
  !
  !   "different airmasses" :
  define/local loop1/i/1/1 0
  define/local airint/i/1/{allrows} 0 all
  define/local nair/i/1/1 0
  define/local airflag/i/1/{allrows} 0 all
  define/local loop2/i/1/1 0
  !
  !   "range of airmasses" :
  define/local airrng/r/1/1 0.
  !
  !   "indexing object" :
  define/local idxloop/i/1/1 0
  define/local allcnt/i/1/1 0
  define/local badcnt/i/1/1 0
!
!     "enough objects" :
  define/local messtype/i/1/1 0
  define/local skip/i/1/1 0
  !
  !   "different colours" :
  define/local totselrows/i/1/1 0
  define/local colint/i/1/{allrows} 0 all
  define/local ncol/i/1/1 0
  define/local colflag/i/1/{allrows} 0 all
  !
  !   "range of colours" :
  define/local colrng/r/1/1 0.
!
!
!
!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
if type .eq. "c" goto check_col
!>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!
!
!          CHECKING EXTINCTION THRESHOLDS /
!          INDEXING OBJECTS IN MERGED TABLES :
!          *********************************
!
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} -
          "  identifying different objects :"
!
sort/table {hiqualtbl} :OBJECT
select/table {hiqualtbl} all
copy/table {hiqualtbl} tmp_c.tbl
-rename tmp_c.tbl {hiqualtbl}
!
do chkloop = 1 allrows
  !
  ! SELECT ONE OBJECT :
  ! -----------------
  write/keyword inputc {{hiqualtbl},:OBJECT,@{chkloop}}
  select/table {hiqualtbl} :OBJECT.eq."{inputc}" >Null
  selrows = outputi
  lastsel = chkloop+selrows-1
  !
  ! BRIGHTNESS IN STD.CAT. DERIVED FROM ENOUGH MEASUREMENTS ?
  ! -------------------------------------------------------
  badflg = 0
  if {{hiqualtbl},:NSTD,@{chkloop}} .lt. min_stdmea then
    badflg = 1
    badthrcnt(1) = badthrcnt(1)+1
  endif
  !
  ! ENOUGH DIFFERENT AIRMASSES PRESENT (RELATED TO ENTERED AIR. BINSIZE) ?
  ! --------------------------------------------------------------------
  inputi = 0
  do loop1 = chkloop lastsel
    inputi = inputi+1
    airint({inputi}) = m$nint({{hiqualtbl},:AIRMASS,@{loop1}}/airbinsize)
  enddo
  nair = selrows
  do loop1 = 1 selrows
    if airflag({loop1}) .eq. 0 then
      do loop2 = 1 selrows
        if loop2 .ne. loop1 then
          if airint({loop2}) .eq. airint({loop1}) then
            nair = nair-1
            airflag({loop2}) = 1
            airflag({loop1}) = 1
          endif
        endif
      enddo
    endif
  enddo
  if nair .lt. min_noairs then
    badflg = 1
    badthrcnt(2) = badthrcnt(2)+1
  endif
  !
  ! RANGE OF AIRMASSES HIGH ENOUGH ?
  ! ------------------------------
  statistics/table {hiqualtbl} :AIRMASS >Null
  airrng = outputr(2)-outputr(1)
  if airrng .lt. min_airrng then
    badflg = 1
    badthrcnt(3) = badthrcnt(3)+1
  endif
  !
  ! INDEXING CURRENT OBJECT:
  ! -----------------------
  if badflg .eq. 0 then
    idxloop = idxloop+1
    allcnt = idxloop+badcnt
    write/table {hiqualtbl} :IDX @{chkloop}..{lastsel} {idxloop}
  else
    badcnt = badcnt+1
    allcnt = idxloop+badcnt
    write/table {hiqualtbl} :IDX @{chkloop}..{lastsel} -{badcnt}
  endif
        ! ### mess/out still too slow !
        ! mess/out -4 {PROID} -
        !   "    value {chkloop}-{lastsel} / {allrows}  =  object {allcnt}\r"
        if mess_level .gt. 3 -
          $ /bin/echo -
            "INFO    \0133{PROID}\0135:\tvalue {chkloop}-{lastsel} / {allrows}  =  object {allcnt}\r\c"
  !
  chkloop = chkloop+selrows-1
  !
enddo  ! "chkloop = 1 allrows"
        ! ### mess/out 4 {PROID}
        if mess_level .gt. 3 $ /bin/echo "INFO"
!
!
! ENOUGH OBJECTS PRESENT ?
! ----------------------
        mess/out 3 {PROID} -
          "  - found {allcnt}"
!
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} -
          "  checking conditions for computation of k_ext :"
if idxloop .ge. min_nstars then
  messtype = 3
else
  messtype = 2
endif
        set/format J3 T6.3
        mess/out {messtype} {PROID} -
     "  - {tblfilt}_lit computed from <{min_stdmea}meas. : {badthrcnt(1)}"
        mess/out {messtype} {PROID} -
     "  - <{min_noairs}diff. airmasses available : {badthrcnt(2)}"
        mess/out {messtype} {PROID} -
     "    (airmass binsize:{airbinsize})"
        mess/out {messtype} {PROID} -
     "  - airmass range <{min_airrng}          : {badthrcnt(3)}"
        mess/out {messtype} {PROID} -
    "  => remaining objects             : {idxloop} (min.{min_nstars}\b)"
        set/format {FMTDEF}
if idxloop .ge. min_nstars then
  select/table {hiqualtbl} :IDX.gt.0 >Null
else
  select/table {hiqualtbl} all
        mess/out 2 {PROID} -
     "  Too few good ones - skipping filter <{tblfilt}> !"
  warn = 1
  skip = 1
endif
!
!
!>>>>>>>>>>>>>>>>>
goto skipped_entry
check_col:
!>>>>>>>>>>>>>>>>>
!
!
!          CHECKING COLOUR THRESHOLDS :
!          **************************
!
if ncoef .eq. 0 then
  !
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} -
          "  checking conditions for computation of k_col / z :"
  !
  select/table {hiqualtbl} :IDX.gt.0 >Null
  statistics/table {hiqualtbl} :IDX >Null
  if outputr(2) .lt. min_nstars then
        mess/out 2 {PROID} -
     "  Too few good obj. left - skipping filter <{tblfilt}> !"
    warn = 1
    skip = 1
    goto skipped_entry
  endif
  copy/table {hiqualtbl} tmp_d.tbl  ! DO NOT RENAME tmp_d.tbl TO {hiqualtbl} -
  !                                 ! ELSE FURTHER CALCULATIONS CAN FAIL !!!
  !
  ! ENOUGH DIFFERENT COLOURS PRESENT (RELATED TO ENTERED COL. BINSIZE) ?
  ! ------------------------------------------------------------------
  totselrows = {tmp_d.tbl,TBLCONTR(4)}
  do loop1 = 1 totselrows
    colint({loop1}) = -
    m$nint({tmp_d.tbl,:{colour({outidx})},@{loop1}}/colbinsize)
  enddo
  ncol = totselrows
  do loop1 = 1 totselrows
    if colflag({loop1}) .eq. 0 then
      do loop2 = 1 totselrows
        if loop2 .ne. loop1 then
          if colint({loop2}) .eq. colint({loop1}) then
            ncol = ncol-1
            colflag({loop2}) = 1
            colflag({loop1}) = 1
          endif
        endif
      enddo
    endif
  enddo
  set/format F5.{COLACC},
  if ncol .ge. min_nocols then
        mess/out 3 {PROID} -
     "  - different colour values: {ncol} (min.{min_nocols})"
        mess/out 3 {PROID} -
     "    (colour binsize: {colbinsize})"
  else
        mess/out 2 {PROID} -
     "  - different colour values: {ncol} (min.{min_nocols})"
        mess/out 2 {PROID} -
     "    (colour binsize: {colbinsize})"
        mess/out 2 {PROID} -
     "  Too few diff.colours - skipping filter <{tblfilt}> !"
    warn = 1
    skip = 1
    goto skipped_entry
  endif
  !
  ! RANGE OF COLOURS HIGH ENOUGH ?
  ! ----------------------------
  statistics/table tmp_d.tbl :{colour({outidx})} >Null
  colrng = outputr(2)-outputr(1)
  if colrng .ge. min_colrng then
        mess/out 3 {PROID} -
     "  - colour range           :  {colrng} (min.{min_colrng})"
        set/format {FMTDEF}
  else
        mess/out 2 {PROID} -
     "  - colour range           :  {colrng} (min.{min_colrng})"
        mess/out 2 {PROID} -
     "  Colour range too low - skipping filter <{tblfilt}> !"
        set/format {FMTDEF}
    warn = 1
    skip = 1
    goto skipped_entry
  endif
  !
  -delete tmp_d.tbl
  !
endif  ! "ncoef .eq. 0"
!
!
!>>>>>>>>>>
idxloop = 0
!>>>>>>>>>>
!
!
skipped_entry:
  if warn .eq. 0 then
    return 0 {skip} {idxloop}
  else
    return -1 {skip} {idxloop}
  endif
!
error:
  return 1 -99 -99
!
!
!------------------------------------------------------------------------------
