! $Id: fors_calcoeff_plt.prg,v 1.35 2000/09/13 13:36:17 nrainer Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 2000  European Southern Observatory (ESO)
!
! .IDENT      fors_calcoeff_plt.prg
!
! .KEYWORDS   Special table plotting
!
! .PURPOSE    Plotting of different zero points against airmass and colour.
! __________
!
! .USAGE      @p fors_calcoeff_plt plot
!
! .INPUT      Parameter:
!             ---------
!             plot  (P1): plot number flag:
!                         <1> : extinction dependence,
!                         <21>: colour depend. of ext.correc. z.p. - good obj.,
!                         <22>: colour depend. of ext.correc. z.p. - bad obj.,
!                         <3> : colour depend. of ext.+col.correc. z.p.,
!                         <4> : missing or skipped filters,
!                         <5> : saving of all plots;
!                           default: none
!
!             "+lower_level" variables from "fors_calcoeff.prg":
!             -------------------------------------------------
!             TOTFILT
!             ALLFILT
!
!             ncoef
!
!             hiqualtbl
!             loqualtbl
!             loqualobjno
!             tblfilt
!             alltblfilt
!             goodtblfilt
!             outidx
!             colour
!             mjdmin
!             mjdmax
!
!             kext{tblfilt}
!             zcol{tblfilt}
!             kextmea
!             kextsig
!             kcolmea
!             kcolsig
!             zmea
!             zsig
!
!             if present:
!               RBINSTRUMENT
!
! .OUTPUT     -
!
! .RETURN     Q1:  0: successful return
!                  1: an error occured
!             Q2:  Name of plot file which was created
!                  (-99 on failure)
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for calib. coeffs. - not to be used generally !
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.5  00/02/09  Creation
!                  00/       Release
!
!------------------------------------------------------------------------------
!
!
crossref plot
!
!
define/local XSCL/r/1/1 -26.6
define/local YSCL/r/1/1 -25.
define/local XOFF/r/1/5 16,58.8,101.6,144.4,187.2
define/local YOFF/r/1/3 96.,50.,10.
define/local instrument/c/1/20 " "
if m$existk("RBINSTRUMENT") .eq. 1 then  ! written by pipeline environment
  in_a = " {RBINSTRUMENT}"
  instrument = "-" // m$upper(in_a)
endif
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "calcoeff_plt"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  Plot file
define/local PLOT_FILE/c/1/40 "calcoeff.plt"
!
!  For return values:
write/keyword q1 0
write/keyword q2 0
!
if p1 .eq. "??" then
  w/o "      *****************************************************************"
  w/o "      Parameter:"
  w/o "      ---------"
  w/o "      plot  (P1): plot number flag:"
  w/o "                  <1> : extinction dependence,"
  w/o "                  <21>: colour depend. of ext.correc. z.p. - good obj.,"
  w/o "                  <22>: colour depend. of ext.correc. z.p. - bad obj.,"
  w/o "                  <3> : colour depend. of ext.+col.correc. z.p.,"
  w/o "                  <4> : missing or skipped filters,"
  w/o "                  <5> : saving of all plots;"
  w/o "                    default: none"
  w/o " "
  w/o "      "+lower_level" variables from "fors_calcoeff.prg":"
  w/o "      -------------------------------------------------"
  w/o "      TOTFILT"
  w/o "      ALLFILT"
  w/o " "
  w/o "      ncoef"
  w/o " "
  w/o "      hiqualtbl"
  w/o "      loqualtbl"
  w/o "      loqualobjno"
  w/o "      tblfilt"
  w/o "      alltblfilt"
  w/o "      goodtblfilt"
  w/o "      outidx"
  w/o "      colour"
  w/o "      mjdmin"
  w/o "      mjdmax"
  w/o " "
  w/o "      kext{tblfilt}"
  w/o "      zcol{tblfilt}"
  w/o "      kextmea"
  w/o "      kextsig"
  w/o "      kcolmea"
  w/o "      kcolsig"
  w/o "      zmea"
  w/o "      zsig"
  w/o " "
  w/o "      if present:"
  w/o "        RBINSTRUMENT"
  w/o "      *****************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(plot):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? n -
  "        Enter plot number (see proc. header) [def.:none]: "
define/local plotnumber/i/1/1 {p1}
inputi = 1
if plotnumber .eq. 1 inputi = 0
if plotnumber .eq. 21 inputi = 0
if plotnumber .eq. 22 inputi = 0
if plotnumber .eq. 3 inputi = 0
if plotnumber .eq. 4 inputi = 0
if plotnumber .eq. 5 inputi = 0
if inputi .eq. 1 then
        write/out "      "
        mess/out 1 {PROID} "Entered plot number does not exist !"
        mess/out 1 {PROID} "(<{plotnumber}>; #plot=[1|21|22|3|4|5]] !)"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!      "first plot" :
  define/local airmin/r/1/1 0.
  define/local airmax/r/1/1 0.
  define/local gdobjno/i/1/1 0
  define/local objloop/i/1/1 0
  define/local std_ins_0/r/1/1 0.
  define/local std_ins_1/r/1/1 0.
!
!      "third plot" :
  define/local median/r/1/1 0.
!
!      "missing labels" :
  define/local outi/i/1/1 0
  define/local filter/c/1/1 " "
  define/local loopidx/i/1/1 0
  define/local datemin/c/1/11 "---------- " ? +low
  define/local datemax/c/1/11 "---------- " ? +low
!
!      "saving plot" :
  define/local char/i/1/1 0
!
!
!
!          PLOT CHOICE :
!          ***********
!
branch plotnumber 1,21,22,3,4,5 first,secondgood,secondbad,third,miss_skip,save
!
!
!          FIRST PLOT :
!          **********
!
first:
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} "  plotting extinction dependence"
!
assign/graphic g,{FIXGRAPHNO} >Null
set/graphic default
set/graphic pmode=0 font=1 cleargra=off
set/graphic xscale={XSCL} yscale={YSCL} -
  xoffset={XOFF({outidx})} yoffset={YOFF(1)}
statistics/table {hiqualtbl} :AIRMASS >Null
airmin = outputr(1)
airmax = outputr(2)
inputr(1) = m$nint(airmax-0.5)    ! int
inputr(2) = m$nint(airmax)        ! nint
if inputr(1) .eq. inputr(2) then  ! int=nint true for [i.0;i.5[
  inputr(3) = m$nint(airmax)+0.6  !   => use i.5 as x-axis end
else                              ! int!=nint true for [i.5;++i.0[
  inputr(3) = m$nint(airmax)+0.1  !   => use ++i.0 as x-axis end
endif
set/graphic xaxis=0.9,{inputr(3)},0.5,0.1 yaxis=auto
set/graphic ssize=3
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! ncoef=[0|1]:
! The table is still selected from the main proc. - ":IDX.gt.0". This means,
! that bad objects regarding the threshold tests are excluded (:IDX <0) and
! and also objects with negative extinction coefficient (:IDX =0):
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
regress/general {hiqualtbl} AIRMASS,STD_INS -
  slope,{kextmea({outidx})} avg {FIXGRAPHNO},w n n n,n,n
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} -
          "Regression w.const.slope failed - :STD_INS(:AIRMASS) !"
  goto error
endif
!
if tblfilt .eq. "U" then
  copy/keyword plrgrap/r/1/4 ux
  copy/keyword plrgrap/r/5/4 uy
endif
!
if ncoef .eq. 0 .or. ncoef .eq. 1 then
  !
  select/table {hiqualtbl} :IDX.ge.0 >Null  ! ".ge.": "0"=pos.ext.coef. -
  statistics/table {hiqualtbl} :IDX >Null   ! such values are also stored in
  gdobjno = outputr(2)                      ! kext{tblfilt}(1..{gdobjno})
  set/graphic colour=1
  do objloop = 1 gdobjno
    if kext{tblfilt}({objloop}) .lt. 0. then
      std_ins_0 = zcol{tblfilt}({objloop})+kext{tblfilt}({objloop})*airmin
      std_ins_1 = zcol{tblfilt}({objloop})+kext{tblfilt}({objloop})*airmax
      overplot/line 3 {airmin},{std_ins_0} {airmax},{std_ins_1}
    endif
  enddo
  !
  select/table {hiqualtbl} :IDX.le.0 >Null
  if outputi .gt. 0 then
    set/graphic stype=3 ssize=1 colour=2           ! Open squares:
    overplot/table {hiqualtbl} :AIRMASS :STD_INS   ! bad bec.of thresh./
  endif                                            ! pos.ext.coef.
  !
  ! - - - - - - - - - - - - - - - - -
  select/table {hiqualtbl} :IDX.gt.0 >Null
  ! - - - - - - - - - - - - - - - - -
  !
  set/graphic colour=1
  overplot/line 5 1.,{plrgrap(5)} 1.,{plrgrap(6)}  ! AIRMASS=1
  !
endif
!
if {{loqualtbl},TBLCONTR(4)} .gt. 0 then
  set/graphic stype=2 ssize=1 colour=2             ! Open circles:
  overplot/table {loqualtbl} :AIRMASS :STD_INS     ! bad bec.of init.select.
endif
!
set/graphic colour=1
label/graphic "{tblfilt}" 11.5,38,mm ? 2.5 1
if tblfilt .eq. "U" then
  label/graphic "CALIBRATION COEFFICIENTS {instrument}" 0,43,mm ? 2.5 1
  set/graphic colour=2
  label/graphic -
   "(Mean) ext.coef. (of all obj.) / all zero points Z_COL (ext. correction)" -
    0,31,mm ? 2.2 1
endif
set/graphic colour=1
set/format T4.2,T5.2
label/graphic "k_ext= {kextmea({outidx})} +-{kextsig({outidx})}" -
0,27,mm ? 1.5 1
set/format {FMTDEF}
!
goto done
!
!
!          SECOND PLOT - GOOD OBJECTS :
!          **************************
!
secondgood:
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} -
          "  plot. col.depend. of ext.correc. z.p. - good obj."
!
set/graphic default
set/graphic pmode=0 font=1 cleargra=off
set/graphic xscale={XSCL} yscale={YSCL} -
  xoffset={XOFF({outidx})} yoffset={YOFF(2)}
set/graphic xaxis=-1.2,2.2 yaxis=auto
set/graphic ssize=3
!
goto done
!
!
!          SECOND PLOT - BAD OBJECTS :
!          *************************
!
secondbad:
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} -
          "  plot. col.depend. of ext.correc. z.p. - bad obj."
!
if ncoef .eq. 0 .or. ncoef .eq. 1 then
  select/table {hiqualtbl} :IDX.le.0 >Null
  if outputi .gt. 0 then
    set/graphic stype=3 ssize=1 colour=2             ! Open squares:
    overplot/table {hiqualtbl} :{colour({outidx})} :Z_COL
  endif                                              ! bad (thresh./pos.ex.co.)
  select/table {hiqualtbl} :IDX.gt.0 >Null
endif
!
if loqualobjno .gt. 0 then
  set/graphic stype=2 ssize=1 colour=2               ! Open circles:
  overplot/table {loqualtbl} :{colour({outidx})} :Z_COL
endif                                                ! bad (initial selection)
!
if tblfilt .eq. "U" then
  set/graphic colour=2
  label/graphic -
    "Col.coef. / mean zero point Z (ext. corrected, col. correction)" -
    0,34,mm ? 2.2 1
endif
set/graphic colour=1
if kcolmea({outidx}) .ge. 0. then
  set/format ,T4.2
  inputc = "+{kcolmea({outidx})}"
else
  set/format ,T5.2
  inputc = "{kcolmea({outidx})}"
endif
set/format T4.2,
label/graphic "k_col= {inputc} +-{kcolsig({outidx})}" 0,30,mm ? 1.5 1
set/format ,T5.2
label/graphic "z= {zmea({outidx})} +-{zsig({outidx})}" 5.5,27,mm ? 1.5 1
set/format {FMTDEF}
!
goto done
!
!
!          THIRD PLOT :
!          **********
!
third:
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} "  plotting colour depend. of ext.+col.correc. z.p."
!
set/graphic default
set/graphic pmode=0 font=1 cleargra=off
set/graphic xscale={XSCL} yscale={YSCL} -
  xoffset={XOFF({outidx})} yoffset={YOFF(3)}
statistics/table {hiqualtbl} :MJD >Null
inputr(1) = m$nint(outputr(1))-0.6
inputr(2) = m$nint(outputr(2))+0.6
set/graphic xaxis={inputr(1)},{inputr(2)},0.5,0.1
!
if outputr(1) .lt. mjdmin mjdmin = outputr(1)
if outputr(2) .gt. mjdmax mjdmax = outputr(2)
!
copy/ti {hiqualtbl} midtmp.bdf
set/refc {hiqualtbl} :Z
inputi = {{hiqualtbl},TBLCONTR(5)}
statistics/image midtmp.bdf [<,@{inputi}:>,@{inputi}] ? ? xn >Null
set/refc {hiqualtbl} seq
-delete midtmp.bdf
inputr(1) = outputr(1)-plrgrap(7)  ! plrgrap from 2nd plot
inputr(2) = outputr(2)+plrgrap(7)
median = outputr(8)
set/graphic yaxis={inputr(1)},{inputr(2)}
!
set/graphic stype=1 ssize=3 ltype=0 bcolour=8 colour=1
plot/table {hiqualtbl} :MJD :Z >Null
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
set/graphic lwidth=2 colour=2
overplot/line 1 {plrgrap(1)},{median} {plrgrap(2)},{median}
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!
if ncoef .eq. 0 .or. ncoef .eq. 1 then
  select/table {hiqualtbl} :IDX.le.0 >Null
  if outputi .gt. 0 then
    set/graphic stype=3 ssize=1 colour=2  ! Open squares:
    overplot/table {hiqualtbl} :MJD :Z    ! bad (thresh./pos.ex.co.)
  endif
  select/table {hiqualtbl} :IDX.gt.0 >Null
endif
!
if {{loqualtbl},TBLCONTR(4)} .gt. 0 then
  set/graphic stype=2 ssize=1 colour=2    ! Open circles:
  overplot/table {loqualtbl} :MJD :Z      ! bad (initial selection)
endif
!
if tblfilt .eq. "U" then
  set/graphic colour=2
  label/graphic -
    "All zero points Z (ext. and col. corrected)" -
    0,28,mm ? 2.2 1
endif
set/graphic colour=1
!
goto done
!
!
!          PLOTTING OF MISSING LABELS DUE
!          TO MISSING OR SKIPPED FILTERS
!          AND PLOTTING OF DATE RANGES :
!          ******************************
!
miss_skip:
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} "marking missing or skipped filters"
!
set/graphic default
set/graphic pmode=0 font=1 cleargra=off
set/graphic xaxis=auto yaxis=auto
do outi = 1 TOTFILT
  filter = ALLFILT({outi}:{outi})
  loopidx = m$index(ALLFILT,filter)
  if m$index(goodtblfilt,filter) .eq. 0 then
    !
    !------ 1ST PLOT ----------------------------------------------------------
    set/graphic colour=0
    plot/axes ? ? {XSCL},{YSCL},{XOFF({loopidx})},{YOFF(1)} >Null
      ! CLEANING OF POSSIBLE PLOT CREATED BEFORE SKIPPING
      if m$index(alltblfilt,filter) .gt. 0 then
        set/graphic colour=8 lwidth=4
        l$l$l$r = -13.
        do l$l$l$i = 1 45
          overplot/line 1 {l$l$l$r},-6.,mm {l$l$l$r},28.,mm
          l$l$l$r = l$l$l$r+0.9
        enddo
      endif
      ! END OF CLEANING
    set/graphic colour=1
    label/graphic "{filter}" 11.5,38,mm ? 2.5 1
    !
    if filter .eq. "U" then
      !
      if mjdmin .lt. 500000. .and. mjdmax .gt. 0 then  ! "500000."<f._calcoeff
        @p fors_calcoeff_plt,date
      endif
      !
      label/graphic -
        "CALIBRATION COEFFICIENTS {instrument} (from {datemin} to {datemax})" -
        0,43,mm ? 2.5 1
      set/graphic colour=2
      label/graphic -
   "(Mean) ext.coef. (of all obj.) / all zero points Z_COL (ext. correction)" -
        0,31,mm ? 2.2 1
    endif
    !
    set/graphic colour=1
    if m$index(alltblfilt,filter) .gt. 0 then
      label/graphic "failed" 7,12.5,mm ? 2.0 1
    else
      label/graphic "no data" 7,12.5,mm ? 2.0 1
    endif
    !
    !------ 2ND PLOT ----------------------------------------------------------
    set/graphic colour=0
    plot/axes ? ? {XSCL},{YSCL},{XOFF({loopidx})},{YOFF(2)} >Null
    !
    if filter .eq. "U" then
      set/graphic colour=2
      label/graphic -
        "Col.coef. / mean zero point Z (ext. corrected, col. correction)" -
        0,34,mm ? 2.2 1
    endif
    !
    set/graphic colour=1
    if m$index(alltblfilt,filter) .gt. 0 then
      label/graphic "failed" 7,12.5,mm ? 2.0 1
    else
      label/graphic "no data" 7,12.5,mm ? 2.0 1
    endif
    !
    !------ 3RD PLOT ----------------------------------------------------------
    set/graphic colour=0
    plot/axes ? ? {XSCL},{YSCL},{XOFF({loopidx})},{YOFF(3)} >Null
    !
    if filter .eq. "U" then
      set/graphic colour=2
      label/graphic -
        "All zero points Z (ext. and col. corrected)" -
        0,28,mm ? 2.2 1
    endif
    !
    set/graphic colour=1
    if m$index(alltblfilt,filter) .gt. 0 then
      label/graphic "failed" 7,12.5,mm ? 2.0 1
    else
      label/graphic "no data" 7,12.5,mm ? 2.0 1
    endif
    !--------------------------------------------------------------------------
    !
  elseif filter .eq. "U" then
    !
    if mjdmin .lt. 500000. .and. mjdmax .gt. 0 then  ! "500000."<f._calcoeff
      @p fors_calcoeff_plt,date
    endif
    !
    set/graphic colour=1
    plot/axes {ux(1)},{ux(2)},{ux(3)},{ux(4)} {uy(1)},{uy(2)},{uy(3)},{uy(4)} -
      {XSCL},{YSCL},{XOFF({loopidx})},{YOFF(1)} >Null
    label/graphic -
      "CALIBRATION COEFFICIENTS {instrument} (from {datemin} to {datemax})" -
      0,43,mm ? 2.5 1
    !
  endif
enddo
!
goto done
!
!
!          SAVING PLOT FILE :
!          ****************
!
save:
!
set/graphic default
!
in_a = " "
in_a(1:9) = m$lower(mid$plot(1:9))
in_a(10:10) = mid$sess(7:7)
in_a(11:12) = mid$sess(11:12)
in_a(13:16) = ".plt"
if m$exist(in_a) .eq. 0 then
      write/out " "
      mess/out 1 {PROID} -
        "No plot file for assigned graph. window {in_a(10:10)} present !"
  goto error
endif
!
        mess/out 4 {PROID} "saving colour plot to <{PLOT_FILE}>"
!
-copy {in_a} {PLOT_FILE}

!
goto done
!
!
done:
  return 0 {PLOT_FILE}
!
error:
  return 1 -99
!
!
!------------------------------------------------------------------------------
!
entry date
!
! MJD -> CIVIL DATE :
!
  define/local jdmin/d/1/1 0.
  define/local jdmax/d/1/1 0.
  !
  jdmin = mjdmin+2400000.5
  jdmax = mjdmax+2400000.5
  !
  create/table date.tbl 1 2 null
  create/column date.tbl :MINMAX " " T10.0 R*8
  date.tbl,:MINMAX,@1 = {jdmin}
  date.tbl,:MINMAX,@2 = {jdmax}
  !
  open/file date.fmt write inputi
  write/file {inputi} define/field 1 10 R*8 T10.0 :MINMAX " "
  close/file {inputi}
  !
  assign/print file date.dat
  print/table date.tbl @1 date.fmt n
  open/file date.dat read inputi
  read/file {inputi} datemin
  close/file {inputi}
  print/table date.tbl @2 date.fmt n
  open/file date.dat read inputi
  read/file {inputi} datemax
  close/file {inputi}
  !
  assign/print terminal
  -delete date.tbl date.fmt date.dat
!
!------------------------------------------------------------------------------
