! $Id: fors_ccdcte.prg,v 1.43 2001/04/06 16:19:24 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_ccdcte.prg,v $
!
! .KEYWORDS   Detector: transfer
!
! .PURPOSE    Determine horizontal CCD charge transfer efficiency
!             (vertical not possible because of nonexistent overscan pixels).
! __________
!
! .USAGE      @p fors_ccdcte icat [scan] [disp]
!
! .INPUT      icat (P1): image catalogue with RAW high level screen flatfield
!                        frames WITH OVERSCAN AREA;
!                          default: none
!
!             scan (P2): number of pre- AND overscan columns, separated by
!                        a comma - MOST IMPORTANT PARAMETER;
!                          default: 16,16
!
!             disp (P3): display selection: <p>:pipeline mode:read global key
!                        NICE_CREA [y|n], <n>:none, <l>:local, or
!                        host:display[.screen] - with the latter option
!                        you can specify the name or IP-address of a display;
!                          default: <l>
!
! .OUTPUT     "{icat}_cte.tbl": charge transfer efficiencies
!
!             OUTPUTR(1): mean charge transfer efficiency
!             OUTPUTR(2): standard deviation thereof
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Extended Pixel Edge Response test (Janesick 1987)
!
! .COMMENT    Modified version from context "ccdtest" (Author: R.Warmels).
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/16  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref icat scan disp
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "ccdcte"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For `longlog/out':
write/keyword out_a "given command/qualif not defined ..."
show/command longlog/out | write/keyword in_a
-delete Mid{mid$sess(11:12)}1Pipe
-delete midlis{mid$sess(11:12)}.prg
if in_a .eq. out_a create/command longlog/out "write/out"
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      icat (P1): image catalogue with RAW high level screen flatfield"
  w/o "                 frames WITH OVERSCAN AREA;"
  w/o "                   default: none"
  w/o " "
  w/o "      scan (P2): number of pre- AND overscan columns, separated by"
  w/o "                 a comma - MOST IMPORTANT PARAMETER;"
  w/o "                   default: 16,16"
  w/o " "
  w/o "      disp (P3): display selection: <p>:pipeline mode:read global key"
  w/o "                 NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                 host:display[.screen] - with the latter option"
  w/o "                 you can specify the name or IP-address of a display;"
  w/o "                   default: <l>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!
!
!          INPUT :
!          *****
!
!P1(icat):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of catalogue with images to be tested: "
define/local icat/c/1/64 "{p1}"
if m$index(icat,".cat") .eq. 0 icat = icat//".cat"
!
if m$exist(icat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
show/icat {icat} n
if {outputi(1)} .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is empty !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
define/local catal/i/1/1 0
inloop:
  store/frame inputc {icat} 1 end_inloop
  if m$exist(inputc) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "At least one catalogue entry is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
    goto error
  endif
  goto inloop
end_inloop:
!
!
!P2(scan):
!--------
define/parameter p2 16,16 ? -
  "        Enter pre-/overscan values [def.:16,16]: "
outputi = m$parse(p2,"scan")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local prescan/i/1/1 {scan01}
define/local ovescan/i/1/1 {scan02}
!
if prescan .lt. 0 .or. ovescan .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible pre- or overscan value !"
  goto error
endif
!
!
!P3(disp):
!--------
define/parameter p3 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p3}"
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Preparing loop" :
  define/local char/i/1/1 0
  define/local ctetable/c/1/64 " " all
!
!     "Computing" :
  define/local topicima/c/1/64 " " all
  define/local lopcount/i/1/1 0
  define/local xpix/i/1/1 0
  define/local ovfirst/i/1/1 0
  define/local imalast/i/1/1 0
  define/local imaxpix/i/1/1 0
  define/local hcte/d/1/1 0.
!
!     "Results" :
  define/local meancte/r/1/1 0.
  define/local sigmcte/r/1/1 0.
!
!
!
!          MANAGING DISPLAYS :
!          *****************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_gids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 1 is missing !"
    goto error
  endif
  !
  if nice_dids(1) .eq. 1 then
    assign/display d,1 >Null
    clear/display
    display/lut off
  endif
  if nice_dids(2) .eq. 1 then
    assign/display d,2 >Null
    clear/display
    display/lut off
  endif
  if nice_dids(3) .eq. 1 then
    assign/display d,3 >Null
    clear/display
    display/lut off
  endif
  assign/graphic g,1 >Null
  clear/graphic
  if nice_gids(2) .eq. 1 then
    assign/graphic g,2 >Null
    clear/graphic
  endif
  set/graphic default
  set/graphic pmode=0 bcolour=1
  set/graphic stype=0 ltype=1 colour=3
  !
endif
!
!
!          PREPARING LOOP :
!          **************
!
        mess/out 4 {PROID} "preparing output"
!
  char = m$index(icat,".cat")-1
  ctetable = icat(1:{char})//"_cte.tbl"
show/icat {icat} n
outputi(1) = outputi(1)+2  ! empty line + result line
create/table {ctetable} 4 {outputi(1)} null
!
create/column {ctetable} :FRAME      " "   A28   C*64
create/column {ctetable} :oscanfirst "ADU" G12.6 R*4
create/column {ctetable} :imseclast  "ADU" G13.6 R*4
create/column {ctetable} :HCTE       " "   D12.7 R*8
!
!
!          COMPUTING EFFICIENCY :
!          ********************
!
        mess/out 3 {PROID} " "
!
catal = 0
catloop:
  store/frame topicima {icat} 1 end_catloop
  lopcount = lopcount+1
  show/icat {icat} n
        mess/out 3 {PROID} "image {lopcount} / {outputi(1)} : {topicima}"
  !
  average/row avrow.bdf = {topicima} <,>
  !
  if nice_crea .eq. "y" then
    !
    assign/graphic g,1 >Null
    plot/row avrow.bdf @1 <,> >Null
    !***************
    @p fors_nicegrap
    !***************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of plot failed !"
      goto error
    endif
    label/graphic "{topicima}" {nice_glabw}
    !
  endif
  !
  xpix = {{topicima},NPIX(1)}
  ovfirst = xpix-ovescan+1
  imalast = xpix-ovescan
  imaxpix = xpix-ovescan-prescan
  !
  if {avrow.bdf[@{imalast}]} .le. 0 then
        write/out " "
        mess/out 1 {PROID} "No computation of C.T.E. possible - average"
        mess/out 1 {PROID} "count rate of last image section pixels is"
        mess/out 1 {PROID} "lower than or equal zero !"
    goto error
  endif
  !
  hcte = 1 - {avrow.bdf[@{ovfirst}]} / {avrow.bdf[@{imalast}]} / {imaxpix}
  !
        set/format ,F8.7
        mess/out 3 {PROID} "  horizontal CTE: {hcte}"
        set/format {FMTDEF}
  !
  {ctetable},:FRAME,{lopcount} = "{topicima}"
  {ctetable},:oscanfirst,{lopcount} = {avrow.bdf[@{ovfirst}]}
  {ctetable},:imseclast,{lopcount} = {avrow.bdf[@{imalast}]}
  {ctetable},:HCTE,{lopcount} = hcte
  !
  goto catloop
end_catloop:
!
!
-delete avrow.bdf
!
!
!          RESULTS :
!          *******
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} "calculating statistics"
!
statistics/table {ctetable} :HCTE >Null
meancte = outputr(3)
sigmcte = outputr(4)
!
show/table {ctetable} >Null
outputi(2) = outputi(2)+2
write/table {ctetable} :FRAME @{outputi(2)} -
  "                        MEAN"  ! 24 blanks
write/table {ctetable} :HCTE @{outputi(2)} {meancte}
outputi(2) = outputi(2)+1
write/table {ctetable} :FRAME @{outputi(2)} -
  "                       SIGMA"  ! 23 blanks
write/table {ctetable} :HCTE @{outputi(2)} {sigmcte}
!
!
!          OUTPUT :
!          ******
!
outputr(1) = meancte
outputr(2) = sigmcte
!
if mess_level .ge. 3 then
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "Results:"
        write/out " "
  read/table {ctetable}
        if mess_level .eq. 4 write/out " "
        mess/out 4 {PROID} ">>> Efficiencies are in <{ctetable}> <<<"
        mess/out 4 {PROID} ">>> OUTPUTR(1): mean charge transfer effi. <<<"
        mess/out 4 {PROID} ">>> OUTPUTR(2): standard deviation thereof <<<"
endif
!
        set/format T9.7,
        longlog/out 1 "     mean charge transfer effi.: {outputr(1)}" >Null
        longlog/out 1 "     standard deviation thereof: {outputr(2)}" >Null
        set/format {FMTDEF}
!
!
set/format
if nice_crea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" set/graphic default
  endif
  return 1
!
!
!-------------------------------------------------------------------------------
