! $Id: fors_ccdlin.prg,v 1.44 2001/04/06 16:19:24 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_ccdlin.prg,v $
!
! .KEYWORDS   Detector: linearity, transfer
!
! .PURPOSE    Compute CCD linearity (relationship between illumination and
!             output of amplifiers) and transfer values (electrons to ADU
!             conversion factor and readout noise) for 1-port readout or
!             as average of all ports.
! __________
!
! .USAGE      @p fors_ccdlin icat bias [scan] [sel] [int] [disp] [ps]
!
! .INPUT      icat (P1): image catalogue with at least {MINPAIR} (see below)
!                        pairs of input screen flatfield frames (per pair same
!                        exposure time) with exposure levels from above bias to
!                        nearly saturation;
!                          default: none
!
!             bias (P2): name of master bias frame;
!                          default: none
!
!             scan (P3): number of pre- AND overscan columns, separated by
!                        a comma;
!                          default: 16,16
!
!             sel  (P4): threshold for low exposure times [sec] to be excluded
!                        for better statistics;
!                          default: 5.000
!
!             int  (P5): time interval [sec] to fit the shutter offset,
!                        separated by a comma;
!                          default: -2.,2.
!
!             disp (P6): display selection: <p>:pipeline mode:read global key
!                        NICE_CREA [y|n], <n>:none, <l>:local, or
!                        host:display[.screen] - with the latter option
!                        you can specify the name or IP-address of a display;
!                          default: <l>
!
!             ps   (P7): flag for creation of plot postscript files: <>:do not
!                        create, <y>es,create;
!                          default: <>
!                          hint: ignored, if P6=n
!
! .OUTPUT     "{icat}_lin.tbl": linearity statistics and conversion factor
!
!             OUTPUTR(1): conversion factor [e-/ADU]
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Modified version from context "ccdtest" (Author: R.Warmels).
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/09/18  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref icat bias scan sel int disp ps
!
!
!  MINPAIR is "+lower_levels" in fors_gen_cal_ccdlin.prg
if m$existk("MINPAIR") .eq. 0 define/local MINPAIR/i/1/1 3 ? +lower_levels !>=3!
outputi(1) = MINPAIR*2
define/local MINFRAM/i/1/1 {outputi(1)}
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "ccdlin"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For `longlog/out':
write/keyword out_a "given command/qualif not defined ..."
show/command longlog/out | write/keyword in_a
-delete Mid{mid$sess(11:12)}1Pipe
-delete midlis{mid$sess(11:12)}.prg
if in_a .eq. out_a create/command longlog/out "write/out"
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  set/format I1
  w/o "      ******************************************************************"
  w/o "      icat (P1): image catalogue with at least {MINPAIR} pairs of input"
  w/o "                 screen flatfield frames (per pair same exposure time)"
  w/o "                 with exposure levels from above bias to nearly"
  w/o "                 saturation;"
  w/o "                   default: none"
  w/o " "
  w/o "      bias (P2): name of master bias frame;"
  w/o "                   default: none"
  w/o " "
  w/o "      scan (P3): number of pre- AND overscan columns, separated by"
  w/o "                 a comma;"
  w/o "                   default: 16,16"
  w/o " "
  w/o "      sel  (P4): threshold for low exposure times [sec] to be excluded"
  w/o "                 for better statistics;"
  w/o "                   default: 5.000"
  w/o " "
  w/o "      int  (P5): time interval [sec] to fit the shutter offset,"
  w/o "                 separated by a comma;"
  w/o "                   default: -2.,2."
  w/o " "
  w/o "      disp (P6): display selection: <p>:pipeline mode:read global key"
  w/o "                 NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                 host:display[.screen] - with the latter option"
  w/o "                 you can specify the name or IP-address of a display;"
  w/o "                   default: <l>"
  w/o " "
  w/o "      ps   (P7): flag for creation of plot postscript files: <>:do not"
  w/o "                 create, <y>es,create;"
  w/o "                   default: <>"
  w/o "                   hint: ignored, if P6=n"
  w/o "      ******************************************************************"
  set/format
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(icat):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of catalogue with images to be tested: "
define/local icat/c/1/64 "{p1}"
if m$index(icat,".cat") .eq. 0 icat = icat//".cat"
!
if m$exist(icat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
show/icat {icat} n
if outputi(1) .lt. MINFRAM then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue has less than {MINPAIR} pairs !"
        mess/out 1 {PROID} "({icat})"
  goto error
elseif m$nint(outputi(1)/2.)-outputi(1)/2. .ne. 0. then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue has odd number of entries !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
define/local catal/i/1/1 0
inloop:
  store/frame inputc {icat} 1 end_inloop
  if m$exist(inputc) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "At least one catalogue entry is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
    goto error
  endif
  goto inloop
end_inloop:
!
!
!P2(bias):
!--------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? i -
  "        Enter name of master bias frame: "
define/local mbias/c/1/64 "{p2}"
if m$index(mbias,".bdf") .eq. 0 mbias = mbias//".bdf"
if m$exist(mbias) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered master bias frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({mbias})"
  goto error
endif
!
!
!P3(scan):
!--------
define/parameter p3 16,16 ? -
  "        Enter pre-/overscan values [def.:16,16]: "
outputi = m$parse(p3,"scan")
define/local prescan/i/1/1 {scan01}
define/local ovescan/i/1/1 {scan02}
!
if prescan .lt. 0 .or. ovescan .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible pre- or overscan value !"
  goto error
endif
!
!
!P4(sel):
!-------
define/parameter p4 5. n -
  "        Enter thresh for low exp.times to be excluded [def:5.]"
define/local sel/r/1/1 {p4}
!
if sel .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible low thresh. for exposure times !"
  goto error
endif
!
!
!P5(int):
!-------
define/parameter p5 -2.,2. ? -
  "        Enter time interval to fit the shutter offset [def.:-2.,2.]: "
outputi = m$parse(p5,"intv")
define/local intlow/r/1/1 {intv01}
define/local inthigh/r/1/1 {intv02}
!
if intlow .ge. inthigh then
        write/out " "
        mess/out 1 {PROID} "Impossible time interval for fitting !"
  goto error
endif
!
!
!P6(disp):
!--------
define/parameter p6 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p6}"
!
!
!P7(ps):
!------
define/parameter p7 + ? -
  "        Enter flag for PS files of plots [def.:n|<y>es,create]: "
define/local ps/c/1/1 "{p7}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Comp...lin...con.fact" :
  define/local bsubxstart/i/1/1 0
  define/local bsubxend/i/1/1 0
  define/local bval/r/1/1 0.
  define/local char/i/1/1 0
  define/local lintable/c/1/64 " " all
  define/local noconad/i/1/1 0
  define/local conad/r/1/1 0.
!
!     "Results" :
  define/local totrowno/i/1/1 0
  define/local maxnonlin/r/1/2 0. all
!
!     "Output" :
  define/local linplot/c/1/64 " " all
  define/local traplot/c/1/64 " " all
!
!
!
!          MANAGING DISPLAYS :
!          *****************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_gids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 1 is missing !"
    goto error
  endif
  if nice_gids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 2 is missing !"
    goto error
  endif
  !
  if nice_dids(1) .eq. 1 then
    assign/display d,1 >Null
    clear/display
    display/lut off
  endif
  if nice_dids(2) .eq. 1 then
    assign/display d,2 >Null
    clear/display
    display/lut off
  endif
  if nice_dids(3) .eq. 1 then
    assign/display d,3 >Null
    clear/display
    display/lut off
  endif
  assign/graphic g,1 >Null
  clear/graphic
  assign/graphic g,2 >Null
  clear/graphic
  set/graphic default
  !
endif
!
!
!          COMPUTATION OF LINEARITY AND CONVERSION FACTOR :
!          **********************************************
!
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "calculating bias mean value"
!
bsubxstart = prescan+1
bsubxend = {{mbias},NPIX(1)}-ovescan
statistics/image {mbias} [@{bsubxstart},<:@{bsubxend},>] ? ? sn >Null
bval = outputr(3)  ! mean
        set/format F6.2,
        mess/out 4 {PROID} "  using bias value: {bval}"
        set/format {FMTDEF}
!
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "determining flatfield statistics"
!
!******************************************************
@p fors_ccdlin_crea {icat} {bval} {prescan},{ovescan} n
!******************************************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Flatfield statistics failed !"
  goto error
endif
!
!
  char = m$index(icat,".cat")-1
  lintable = icat(1:{char})//"_lin.tbl"
select/table {lintable} :Exp_tim1.gt.{sel} >Null
if outputi(1) .lt. MINPAIR then
        write/out " "
        set/format F5.3
        mess/out 1 {PROID} "No computation of linearity or conversion factor"
        mess/out 1 {PROID} "possible - less than {MINFRAM} frames left after"
        mess/out 1 {PROID} "exclusion of exp.times < {sel} sec !"
  goto error
endif
select/table {lintable} all
!
!
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "computing CCD linearity"
!
!**************************************************************************
@p fors_ccdlin_fitlin {lintable} {sel} {intlow},{inthigh} {dispflag} n {ps}
!**************************************************************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Computation of linearity failed !"
  goto error
endif
!
!
select/table {lintable} :Variance.eq.0. >Null
if outputi(1) .gt. 0 then
  select/table {lintable} all >Null
        mess/out 2 {PROID} " "
        mess/out 2 {PROID} "No computation of conversion factor possible -"
        mess/out 2 {PROID} "the frames of at least one pair are identical !"
  noconad = 1
else
  select/table {lintable} all >Null
  !
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "computing average conversion factor"
  !
  !*******************************************************
  @p fors_ccdlin_fittra {lintable} {sel} {dispflag} n {ps}
  !*******************************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Computation of conversion factor failed !"
    goto error
  endif
  noconad = 0
  conad = outputr(1)
  !
endif
!
!
!          RESULTS :
!          *******
!
show/table {lintable} >Null
totrowno = outputi(2)
!
! from fitlin:
compute/table {lintable} :scratch = abs(:Lin_res1)
select/table {lintable} :Exp_tim1.gt.{sel} >Null
statistics/table {lintable} :scratch >Null
maxnonlin(1) = outputr(2)
outputi(2) = totrowno+2
write/table {lintable} :Lin_res1 {outputi(2)} {maxnonlin(1)}
!
compute/table {lintable} :scratch = abs(:Lin_res2)
select/table {lintable} :Exp_tim2.gt.{sel} >Null
statistics/table {lintable} :scratch >Null
maxnonlin(2) = outputr(2)
outputi(2) = totrowno+2
  set/format T5.3,
  write/table {lintable} :Frame2 {outputi(2)} -
  "Max.Non-Linearities > {sel}sec [%]"
  set/format {FMTDEF}
write/table {lintable} :Lin_res2 {outputi(2)} {maxnonlin(2)}
select/table {lintable} all
!
! from fittra:
create/column {lintable} :ConAD_avg "e-/ADU" F9.4 R*4
if noconad .ne. 1 write/table {lintable} :ConAD_avg @1..{totrowno} {conad}
!
delete/column {lintable} :scratch
!
if mess_level .ge. 3 then
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "Results:"
        write/out " "
  read/table {lintable} -
  :Frame1,Frame2,Exp_tim1,Exp_tim2,Med_cnt1,Med_cnt2,Lin_res1,Lin_res2
  if noconad .ne. 1 then
    set/format T6.3,
    write/out " "
    write/out "          ConAD_avg: {conad} e-/ADU"  ! 10 leading blanks
    set/format {FMTDEF}
  endif
endif
!
!
!          OUTPUT :
!          ******
!
outputr(1) = maxnonlin(1)
outputr(2) = maxnonlin(2)
if noconad .eq. 0 then
  outputr(3) = conad
else
  outputr(3) = -99.
endif
!
        if mess_level .eq. 4 write/out " "
        set/format T5.3,
        mess/out 4 {PROID} ">>> Statistics are in <{lintable}>. <<<"
        mess/out 4 {PROID} ">>> OUTPUTR(1): max.non-lin.frm1. >{sel}sec [%] <<<"
        mess/out 4 {PROID} ">>> OUTPUTR(2): max.non-lin.frm2. >{sel}sec [%] <<<"
      if noconad .eq. 0 then
        mess/out 4 {PROID} ">>> OUTPUTR(3): conversion factor      [e-/ADU] <<<"
      endif
!
        longlog/out 1 "     max.non-lin.frm1. >{sel}sec [%]: {outputr(1)}" >Null
        longlog/out 1 "     max.non-lin.frm2. >{sel}sec [%]: {outputr(2)}" >Null
      if noconad .eq. 0 then
        longlog/out 1 "     conversion factor      [e-/ADU]: {outputr(3)}" >Null
      endif
        set/format {FMTDEF}
!
if nice_crea .eq. "y" .and. ps .eq. "y" then
          char = m$index(lintable,".tbl")-1
          linplot = lintable(1:{char})//".ps"
        mess/out 4 {PROID} ">>> PS files of the plots are <{linplot}>"
          char = m$index(lintable,"_lin.tbl")-1
          traplot = lintable(1:{char})//"_tra.ps"
        mess/out 4 {PROID} "                          and <{traplot}>. <<<"
endif
!
!
set/format
if nice_crea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  if m$existk("lintable") .eq. 1 then
    if m$exist(lintable) .eq. 1 select/table {lintable} all
  endif
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" set/graphic default
  endif
  return 1
!
!
!-------------------------------------------------------------------------------
