! $Id: fors_ccdlin_crea.prg,v 1.23 1999/11/25 11:23:48 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_ccdlin_crea.prg,v $
!
! .KEYWORDS   Detector: linearity, transfer
!
! .PURPOSE    Determine statistics for computation of CCD linearity (relation-
!             ship between illumination and output of amplifiers) and transfer
!             values (electrons to ADU conversion factor and readout noise).
! __________
!
! .USAGE      @p fors_ccdlin_crea icat bval [scan] [show]
!
! .INPUT      icat (P1): image catalogue with at least {MINPAIR} (see below)
!                        pairs of input screen flatfield frames (per pair same
!                        exposure time) with exposure levels from above bias to
!                        nearly saturation.
!                          default: none
!
!             bval (P2): bias value;
!                          default: none
!
!             scan (P3): number of pre- AND overscan columns, separated by
!                        a comma;
!                          default: 16,16
!
!             show (P4): flag for terminal output: <>:show, <n>o;
!                          default: <>
!
! .OUTPUT     "{icat}_lin.tbl": statistics of input frames - needed for
!                               fors_ccdlin_fitlin.prg and
!                               fors_ccdlin_fittra.prg.
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Modified version from context "ccdtest" (Author: R.Warmels).
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/09/16  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref icat bval scan show
!
!
!  MINPAIR is "+lower_levels" in fors_ccdlin.prg
if m$existk("MINPAIR") .eq. 0 define/local MINPAIR/i/1/1 3 ? +lower_levels !>=3!
outputi(1) = MINPAIR*2
define/local MINFRAM/i/1/1 {outputi(1)}
!
!  Threshold for max. diff. of exp.times:
define/local THR_TDIFF/r/1/1 0.02
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "ccdlin_crea"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  set/format I1
  w/o "      ******************************************************************"
  w/o "      icat (P1): image catalogue with at least {MINPAIR} pairs of input"
  w/o "                 screen flatfield frames (per pair same exposure time)"
  w/o "                 with exposure levels from above bias to nearly"
  w/o "                 saturation."
  w/o "                   default: none"
  w/o " "
  w/o "      bval (P2): bias value;"
  w/o "                   default: none"
  w/o " "
  w/o "      scan (P3): number of pre- AND overscan columns, separated by"
  w/o "                 a comma;"
  w/o "                   default: 16,16"
  w/o " "
  w/o "      show (P4): flag for terminal output: <>:show, <n>o;"
  w/o "                   default: <>"
  w/o "      ******************************************************************"
  set/format
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(icat):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of catalogue with images to be tested: "
define/local icat/c/1/64 "{p1}"
if m$index(icat,".cat") .eq. 0 icat = icat//".cat"
!
if m$exist(icat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
show/icat {icat} n
if {outputi(1)} .lt. MINFRAM then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue has less than {MINPAIR} pairs !"
        mess/out 1 {PROID} "({icat})"
  goto error
elseif m$nint(outputi(1)/2.)-outputi(1)/2. .ne. 0. then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue has odd number of entries !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
define/local catal/i/1/1 0
inloop:
  store/frame inputc {icat} 1 end_inloop
  if m$exist(inputc) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "At least one catalogue entry is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
    goto error
  endif
  goto inloop
end_inloop:
!
!
!P2(bval):
!--------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? n -
  "        Enter bias value: "
define/local bval/r/1/1 {p2}
!
if bval .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible bias value !"
  goto error
endif
!
!
!P3(scan):
!--------
define/parameter p3 16,16 ? -
  "        Enter pre-/overscan values [def.:16,16]: "
outputi = m$parse(p3,"scan")
define/local prescan/i/1/1 {scan01}
define/local ovescan/i/1/1 {scan02}
!
if prescan .lt. 0 .or. ovescan .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible pre- or overscan value !"
  goto error
endif
!
!
!P4(show):
!--------
define/parameter p4 y ? -
  "        Enter flag for terminal output [def.:show|<n>o]: "
define/local show/c/1/1 "{p4}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Preparing loop" :
!
  define/local char/i/1/1 0
  define/local lintable/c/1/64 " " all
!
!     "Computing statistics" :
!
  define/local ima1/c/1/64 " " all
  define/local ima2/c/1/64 " " all
  define/local imacount/i/1/1 0
  !
  define/local timediff/r/1/1 0.
  define/local tblcount/i/1/1 0
  define/local subxstart/i/1/1 0
  define/local subxend/i/1/1 0
  define/local variance/r/1/1 0.
!
!
!
!          PREPARING LOOP :
!          **************
!
        mess/out 4 {PROID} "  preparing statistics"
!
  char = m$index(icat,".cat")-1
  lintable = icat(1:{char})//"_lin.tbl"
show/icat {icat} n
create/table {lintable} 10 {outputi(1)} null  ! "10": 3 cols. added in
!                                             !       fors_ccdlin_fitlin.prg
create/column {lintable} :Frame1   " "   A34  C*64
create/column {lintable} :Frame2   " "   A34  C*64
create/column {lintable} :Exp_tim1 "sec" F9.3 R*4
create/column {lintable} :Exp_tim2 "sec" F9.3 R*4
create/column {lintable} :Med_cnt1 "ADU" F9.4 R*4
create/column {lintable} :Med_cnt2 "ADU" F9.4 R*4
create/column {lintable} :Variance "ADU" F9.4 R*4
!
!
!          COMPUTING STATISTICS :
!          ********************
!
catal = 0
catloop:
  store/frame ima1 {icat} 1 end_catloop
  store/frame ima2 {icat} 1 end_catloop
  imacount = imacount+1
  show/icat {icat} n
  outputi(1) = outputi(1)/2
        mess/out 3 {PROID} -
          "  pair of images {imacount} / {outputi(1)} :"
        mess/out 3 {PROID} "    {ima1}"
        mess/out 3 {PROID} "    {ima2}"
  !
  timediff = m$abs({{ima1},O_TIME(7)}-{{ima2},O_TIME(7)})
  !
  if timediff .gt. THR_TDIFF then
    set/format F5.3
        mess/out 2 {PROID} "  Exposure times of current frame pair differ"
        mess/out 2 {PROID} "  more than {THR_TDIFF}sec (threshold) -"
        mess/out 2 {PROID} "  skipping {ima1}"
        mess/out 2 {PROID} "       and {ima2} !"
    set/format {FMTDEF}
    warn = 1
  else
    tblcount = tblcount+1
    !
    {lintable},:Frame1,{tblcount} = "{ima1}"
    {lintable},:Frame2,{tblcount} = "{ima2}"
    !
    {lintable},:Exp_tim1,{tblcount} = {{ima1},O_TIME(7)}
    {lintable},:Exp_tim2,{tblcount} = {{ima2},O_TIME(7)}
    !
        mess/out 4 {PROID} "    computing statistics (1st frame)"
    subxstart = prescan+1
    subxend = {{ima1},NPIX(1)}-ovescan
    statistics/image {ima1} [@{subxstart},<:@{subxend},>] ? ? xn >Null
    {lintable},:Med_cnt1,{tblcount} = outputr(8)-bval
    !
        mess/out 4 {PROID} "    computing statistics (2nd frame)"
    subxend = {{ima2},NPIX(1)}-ovescan
    statistics/image {ima2} [@{subxstart},<:@{subxend},>] ? ? xn >Null
    {lintable},:Med_cnt2,{tblcount} = outputr(8)-bval
    !
        mess/out 4 {PROID} "    creating difference frame"
    compute/pixel diff.bdf = "{ima1}" - "{ima2}"
        mess/out 4 {PROID} "    computing statistics (diff.frame)"
    statistics/image diff.bdf [@{subxstart},<:@{subxend},>] ? ? sn >Null
    variance = outputr(4)*outputr(4)
    {lintable},:Variance,{tblcount} = variance
    !
  endif
  !
  goto catloop
end_catloop:
!
!
-delete diff.bdf
!
!
!          OUTPUT :
!          ******
!
if show .ne. "n" .and. mess_level .ge. 3 then
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "Statistical results:"
        write/out " "
  read/table {lintable}
        write/out " "
        mess/out 4 {PROID} ">>> Statistics are in <{lintable}> <<<"
endif
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------
