! $Id: fors_ccdlin_fitlin.prg,v 1.34 2001/04/06 16:19:25 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_ccdlin_fitlin.prg,v $
!
! .KEYWORDS   Detector: linearity
!
! .PURPOSE    Compute CCD linearity (relationship between illumination and
!             output of amplifiers).
! __________
!
! .USAGE      @p fors_ccdlin_fitlin tbl [sel] [int] [disp] [show] [ps]
!
! .INPUT      tbl  (P1): table with statistics - created by fors_ccdlin_crea.prg
!                          default: none
!
!             sel  (P2): threshold for low exposure times [sec] to be excluded
!                        for better statistics;
!                          default: 5.
!
!             int  (P3): time interval [sec] to fit the shutter offset,
!                        separated by a comma;
!                          default: -2.,2.
!
!             disp (P4): display selection: <p>:pipeline mode:read global key
!                        NICE_CREA [y|n], <n>:none, <l>:local, or
!                        host:display[.screen] - with the latter option
!                        you can specify the name or IP-address of a display;
!                          default: <l>
!
!             show (P5): flag for result output on terminal: <>:show, <n>o;
!                          default: <>
!
!             ps   (P6): flag for creation of plot postscript file: <>:do not
!                        create, <y>es,create;
!                          default: <>
!                          hint: ignored, if P4=n
!
! .OUTPUT     "{tbl}" from fors_ccdlin_crea.prg is extended by columns with the
!             fractional counts.
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for linearity computation - not to be used
!             generally.
!
!             Modified version from context "ccdtest" (Author: R.Warmels).
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/09/17  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref tbl sel int disp show ps
!
!
!  Threshold for step size in shut.err. comp.
define/local THR_STPSZ/r/1/1 0.1
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "ccdlin_fitlin"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      tbl  (P1): table with statistics - created by fors_ccdlin_crea.prg"
  w/o "                   default: none"
  w/o " "
  w/o "      sel  (P2): threshold for low exposure times [sec] to be excluded"
  w/o "                 for better statistics;"
  w/o "                   default: 5."
  w/o " "
  w/o "      int  (P3): time interval [sec] to fit the shutter offset,"
  w/o "                 separated by a comma;"
  w/o "                   default: -2.,2."
  w/o " "
  w/o "      disp (P4): display selection: <p>:pipeline mode:read global key"
  w/o "                 NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                 host:display[.screen] - with the latter option"
  w/o "                 you can specify the name or IP-address of a display;"
  w/o "                   default: <l>"
  w/o " "
  w/o "      show (P5): flag for result output on terminal: <>:show, <n>o;"
  w/o "                   default: <>"
  w/o " "
  w/o "      ps   (P6): flag for creation of plot postscript file: <>:do not"
  w/o "                 create, <y>es,create;"
  w/o "                   default: <>"
  w/o "                   hint: ignored, if P4=n"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(tbl):
!-------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? t -
  "        Enter name of table created by fors_ccdlin_crea.prg: "
define/local lintable/c/1/64 "{p1}"
if m$index(lintable,".tbl") .eq. 0 lintable = lintable//".tbl"
!
if m$exist(lintable) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lintable})"
  goto error
endif
!
show/table {lintable} >Null
if {outputi(2)} .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({lintable})"
  goto error
endif
!
!
!P2(sel):
!-------
define/parameter p2 5. n -
  "        Enter thresh for low exp.times to be excluded [def:5.]"
define/local sel/r/1/1 {p2}
!
if sel .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible low thresh. for exposure times !"
  goto error
endif
!
!
!P3(int):
!-------
define/parameter p3 -2.,2. ? -
  "        Enter time interval to fit the shutter offset [def.:-2.,2.]: "
outputi = m$parse(p3,"intv")
define/local intlow/r/1/1 {intv01}
define/local inthigh/r/1/1 {intv02}
!
if intlow .ge. inthigh then
        write/out " "
        mess/out 1 {PROID} "Impossible time interval for fitting !"
  goto error
endif
!
!
!P4(disp):
!--------
define/parameter p4 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p4}"
!
!
!P5(show):
!--------
define/parameter p5 y ? -
  "        Enter flag for result output on terminal [def.:show|<n>o]: "
define/local show/c/1/1 "{p5}"
!
!
!P6(ps):
!------
define/parameter p6 + ? -
  "        Enter flag for PS files of plots [def.:n|<y>es,create]: "
define/local ps/c/1/1 "{p6}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "MED_ADU <> EXP.TIME" :
!
  define/local xlab/r/1/1 0.
  define/local ylab/r/1/1 0.
  define/local mnadu/r/1/1 0.
  define/local sherr/r/1/1 0.
!
!     "MED_ADU/EXP.TIME <> MED_ADU" :
!
  define/local shval/r/1/3 0.,0.,0.
  define/local nstep/i/1/1 0
  define/local shoff1/r/1/1 0.
  define/local loop/i/1/1 0
  define/local shut/r/1/1 0.
  define/local shoff2/r/1/1 0.
  define/local iter/i/1/1 0
!
!     "Results" :
!
  define/local char/i/1/1 0
  define/local linplot/c/1/64 " " all
!
!
!
!          MANAGING DISPLAYS :
!          *****************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_gids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 1 is missing !"
    goto error
  endif
  !
  assign/graphic g,1 >Null
  clear/graphic
  set/graphic default
  set/graphic pmode=0 clear=off ltype=2
  !
endif
!
!
!          PREPARING LINEARITY COMPUTATIONS :
!          ********************************
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} "  preparing computations"
!
if m$existc(lintable,"Lin_res1") .eq. -1 -
  create/column {lintable} :Lin_res1 "%" F9.4 R*4
if m$existc(lintable,"Lin_res2") .eq. -1 -
  create/column {lintable} :Lin_res2 "%" F9.4 R*4
if m$existc(lintable,":scratch") .eq. -1 -
  create/column {lintable} :scratch " " F9.4 R*4
!
!
!          LINEAR REGRESSIONS MED_ADU VERSUS EXP.TIME :
!          ******************************************
!...............................................................................
!
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "  fitting medians of frames versus exp.times"
        set/format T5.3,
        mess/out 3 {PROID} "  (excluding exp.times < {sel} sec)"
        set/format {FMTDEF}
        mess/out 3 {PROID} "    with 1st frames of the pairs"
  !
  select/table {lintable} :Exp_tim1.gt.{sel} >Null
  !
  if nice_crea .eq. "y" then
    assign/graphic g,1 >Null
    set/graphic stype=5 xformat=auto yformat=E6.2
    plot/table {lintable} :Exp_tim1 :Med_cnt1 -60,-40,25,75  ! ### ### ###
    xlab = plrgrap(1) + (plrgrap(2)-plrgrap(1))/10
    ylab = plrgrap(6) - (plrgrap(6)-plrgrap(5))/10
    label/graphic "Linearity Curve (1)" {xlab},{ylab} ? ? 1
  endif
  !
  regression/linear {lintable} :Med_cnt1 :Exp_tim1 >Null
  mnadu = outputd(2)
  sherr = outputd(1)/outputd(2)
  !
  if nice_crea .eq. "y" then
    save/regression {lintable} regress
    compute/regression {lintable} :scratch = regress
    set/graphic stype=0
    overplot/table {lintable} :Exp_tim1 :scratch
  endif
  !
        set/format T9.3,T9.3
        mess/out 3 {PROID} -
          "      Mean (expo.norm.counts): {mnadu} ADU/sec"
        mess/out 3 {PROID} -
          "      Expo.offset(+slow-fast): {sherr} sec" 
        set/format {FMTDEF}
!
! No further use of these values => only for terminal output (and plotting) !
!
!...............................................................................
!
        mess/out 3 {PROID} "    with 2nd frames of the pairs"
  !
  select/table {lintable} :Exp_tim2.gt.{sel} >Null
  !
  if nice_crea .eq. "y" then
    set/graphic stype=5
    plot/table {lintable} :Exp_tim2 :Med_cnt2 -60,-40,115,75  ! ### ### ###
    xlab = plrgrap(1) + (plrgrap(2)-plrgrap(1))/10
    ylab = plrgrap(6) - (plrgrap(6)-plrgrap(5))/10
    label/graphic "Linearity Curve (2)" {xlab},{ylab} ? ? 1
  endif
  !
  regression/linear {lintable} :Med_cnt2 :Exp_tim2 >Null
  mnadu = outputd(2)
  sherr = outputd(1)/outputd(2)
  !
  if nice_crea .eq. "y" then
    save/regression {lintable} regress
    compute/regression {lintable} :scratch = regress
    set/graphic stype=0
    overplot/table {lintable} :Exp_tim2 :scratch
  endif
  !
        set/format T9.3,T9.3
        mess/out 3 {PROID} -
          "      Mean (expo.norm.counts): {mnadu} ADU/sec"
        mess/out 3 {PROID} -
          "      Expo.offset(+slow-fast): {sherr} sec" 
        set/format {FMTDEF}
!
! No further use of these values => only for terminal output (and plotting) !
!
!...............................................................................
!
!
!          LINEAR REGRESSIONS MED_ADU/EXP.TIME VERSUS MED_ADU :
!          **************************************************
!...............................................................................
!
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "  fitting medians/exp.time versus medians"
        mess/out 4 {PROID} "  (determ. shutter offset from fract. ADU)"
        mess/out 3 {PROID} "    with 1st frames of the pairs"
!
shval(1) = intlow
shval(2) = inthigh
shval(3) = THR_STPSZ
nstep = (shval(2)-shval(1))/shval(3)
!
write/descriptor {lintable} HISTORY_UPDA/i/1/1 0
ITER1:
shoff1 = 999.
do loop = 1 nstep
  shut = shval(1) + (loop-1)*shval(3)
  compute/table {lintable} :Lin_res1 = :Med_cnt1/(:Exp_tim1+{shut})
  select/table {lintable} :Exp_tim1.gt.{sel} >Null
  regression/linear {lintable} :Lin_res1 :Exp_tim1 >Null
  shoff2 = outputr(4)
  if shoff2 .gt. shoff1 .and. iter .eq. 0 then
    shval(1) = shval(1) + (loop-3)*shval(3)
    shval(3) = shval(3)/10
    nstep = 20
    iter  = 1
    goto ITER1
  else if shoff2 .gt. shoff1 .and. iter .eq. 1 then
    shut = shval(1) + (loop-2)*shval(3)
    goto DONE1
  else
    shoff1 = shoff2
  endif
enddo
DONE1:
{lintable},HISTORY_UPDA = 1
!
        set/format F6.3
        mess/out 3 {PROID} "      Shutter offset: {shut} sec"
        set/format {FMTDEF}
        mess/out 4 {PROID} "      improving comp. of fractional counts"
!
compute/table {lintable} :Lin_res1 = :Med_cnt1/(:Exp_tim1+{shut})
select/table {lintable} :Exp_tim1.gt.{sel} >Null
statistics/table {lintable} :Lin_res1 >Null
compute/table {lintable} :Lin_res1 = (1.-:Lin_res1/{outputr(3)})*100.
  !
  if nice_crea .eq. "y" then
    !
    select/table {lintable} :Exp_tim1.gt.{sel} >Null
    !
    set/graphic stype=5 xformat=E6.0 yformat=F6.4
    plot/table {lintable} :Med_cnt1 :Lin_res1 -60,-40,25,15  ! ### ### ###
    xlab = plrgrap(1) + (plrgrap(2)-plrgrap(1))/10
    ylab = plrgrap(6) - (plrgrap(6)-plrgrap(5))/10
    label/graphic "Linearity Residual (1)" {xlab},{ylab} ? ? 1
    !
    regression/linear {lintable} :Lin_res1 :Med_cnt1 >Null
    !
    save/regression {lintable} regress
    compute/regression {lintable} :scratch = regress
    set/graphic stype=0
    overplot/table {lintable} :Med_cnt1 :scratch
    !
  endif
!
!...............................................................................
!
        mess/out 3 {PROID} "    with 2nd frames of the pairs"
!
shval(1) = intlow
shval(2) = inthigh
shval(3) = THR_STPSZ
nstep = (shval(2)-shval(1))/shval(3)
iter  = 0
!
{lintable},HISTORY_UPDA = 0
ITER2:
shoff1 = 999.
do loop = 1 nstep
  shut = shval(1) + (loop-1)*shval(3)
  compute/table {lintable} :Lin_res2 = :Med_cnt2/(:Exp_tim2+{shut})
  select/table {lintable} :Exp_tim2.gt.{sel} >Null
  regression/linear {lintable} :Lin_res2 :Exp_tim2 >Null
  shoff2 = outputr(4)
  if shoff2 .gt. shoff1 .and. iter .eq. 0 then
    shval(1) = shval(1) + (loop-3)*shval(3)
    shval(3) = shval(3)/10
    nstep = 20
    iter  = 1
    goto ITER2
  else if shoff2 .gt. shoff1 .and. iter .eq. 1 then
    shut = shval(1) + (loop-2)*shval(3)
    goto DONE2
  else
    shoff1 = shoff2
  endif
enddo
DONE2:
{lintable},HISTORY_UPDA = 1
!
        set/format F6.3
        mess/out 3 {PROID} "      Shutter offset: {shut} sec"
        set/format {FMTDEF}
        mess/out 4 {PROID} "      improving comp. of fractional counts"
!
compute/table {lintable} :Lin_res2 = :Med_cnt2/(:Exp_tim2+{shut})
select/table {lintable} :Exp_tim2.gt.{sel} >Null
statistics/table {lintable} :Lin_res2 >Null
compute/table {lintable} :Lin_res2 = (1.-:Lin_res2/{outputr(3)})*100.
  !
  if nice_crea .eq. "y" then
    !
    select/table {lintable} :Exp_tim2.gt.{sel} >Null
    !
    set/graphic stype=5
    plot/table {lintable} :Med_cnt2 :Lin_res2 -60,-40,115,15  ! ### ### ###
    xlab = plrgrap(1) + (plrgrap(2)-plrgrap(1))/10
    ylab = plrgrap(6) - (plrgrap(6)-plrgrap(5))/10
    label/graphic "Linearity Residual (2)" {xlab},{ylab} ? ? 1
    !
    regression/linear {lintable} :Lin_res2 :Med_cnt2 >Null
    !
    save/regression {lintable} regress
    compute/regression {lintable} :scratch = regress
    set/graphic stype=0
    overplot/table {lintable} :Med_cnt2 :scratch
    !
  endif
!
!...............................................................................
!
!
!          RESULTS :
!          *******
!
if mess_level .ge. 3 then
        mess/out 3 {PROID} " "
        set/format T5.3,
        mess/out 3 {PROID} "  Max. Non-Linearities >{sel} sec :"
        set/format F9.4,
  compute/table {lintable} :scratch = abs(:Lin_res1)
  select/table {lintable} :Exp_tim1.gt.{sel} >Null
  statistics/table {lintable} :scratch >Null
        mess/out 3 {PROID} "    Frames 1: {outputr(2)} %"
  compute/table {lintable} :scratch = abs(:Lin_res2)
  select/table {lintable} :Exp_tim2.gt.{sel} >Null
  statistics/table {lintable} :scratch >Null
        mess/out 3 {PROID} "    Frames 2: {outputr(2)} %"
        set/format {FMTDEF}
endif
!
select/table {lintable} all
  !
  if nice_crea .eq. "y" then
    if ps .eq. "y" then
      !
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} "  creating postscript file of plots"
      !
      assign/graphic g,1 >Null
      copy/graphic postscript
        char = m$index(lintable,".tbl")-1
        linplot = lintable(1:{char})//".ps"
      -rename postscript.ps {linplot}
    endif
  endif
!
!
!          OUTPUT :
!          ******
!
if show .ne. "n" then
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} ">>> Statistics are in <{lintable}> <<<"
  if nice_crea .eq. "y" .and. ps .eq. "y" then
        mess/out 4 {PROID} ">>> PS file of the plots is <{linplot}> <<<"
  endif
endif
!
!
set/format
if nice_crea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  if m$existk("lintable") .eq. 1 then
    if m$exist(lintable) .eq. 1 select/table {lintable} all
  endif
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" set/graphic default
  endif
  return 1
!
!
!-------------------------------------------------------------------------------
