! $Id: fors_ccdlin_fittra.prg,v 1.33 2001/04/06 16:19:23 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_ccdlin_fittra.prg,v $
!
! .KEYWORDS   Detector: transfer
!
! .PURPOSE    Compute electrons to ADU conversion factor.
! __________
!
! .USAGE      @p fors_ccdlin_fittra tbl [sel] [disp] [show] [ps]
!
! .INPUT      tbl  (P1): table with statistics - created by fors_ccdlin_crea.prg
!                          default: none
!
!             sel  (P2): threshold for low exposure times [sec] to be excluded
!                        for better statistics;
!                          default: 5.
!
!             disp (P3): display selection: <p>:pipeline mode:read global key
!                        NICErea_CREA [y|n], <n>:none, <l>:local, or
!                        host:display[.screen] - with the latter option
!                        you can specify the name or IP-address of a display;
!                          default: <l>
!
!             show (P4): flag for result output on terminal: <>:show, <n>o;
!                          default: <>
!
!             ps   (P5): flag for creation of plot postscript file: <>:do not
!                        create, <y>es,create;
!                          default: <>
!                          hint: ignored, if P3=n
!
! .OUTPUT     OUTPUTR(1): conversion factor [e-/ADU]
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for transfer computations - not to be used
!             generally.
!
!             Modified version from context "ccdtest" (Author: R.Warmels).
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/09/18  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref tbl sel disp show ps
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "ccdlin_fittra"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      tbl  (P1): table with statistics - created by fors_ccdlin_crea.prg"
  w/o "                   default: none"
  w/o " "
  w/o "      sel  (P2): threshold for low exposure times [sec] to be excluded"
  w/o "                 for better statistics;"
  w/o "                   default: 5."
  w/o " "
  w/o "      disp (P3): display selection: <p>:pipeline mode:read global key"
  w/o "                 NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                 host:display[.screen] - with the latter option"
  w/o "                 you can specify the name or IP-address of a display;"
  w/o "                   default: <l>"
  w/o " "
  w/o "      show (P4): flag for result output on terminal: <>:show, <n>o;"
  w/o "                   default: <>"
  w/o " "
  w/o "      ps   (P5): flag for creation of plot postscript file: <>:do not"
  w/o "                 create, <y>es,create;"
  w/o "                   default: <>"
  w/o "                   hint: ignored, if P4=n"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(tbl):
!-------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? t -
  "        Enter name of table created by fors_ccdlin_crea.prg: "
define/local lintable/c/1/64 "{p1}"
if m$index(lintable,".tbl") .eq. 0 lintable = lintable//".tbl"
!
if m$exist(lintable) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lintable})"
  goto error
endif
!
show/table {lintable} >Null
if {outputi(2)} .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({lintable})"
  goto error
endif
!
!
!P2(sel):
!-------
define/parameter p2 5. n -
  "        Enter thresh for low exp.times to be excluded [def:5.]"
define/local sel/r/1/1 {p2}
!
if sel .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible low thresh. for exposure times !"
  goto error
endif
!
!
!P3(disp):
!--------
define/parameter p3 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p3}"
!
!
!P4(show):
!--------
define/parameter p4 y ? -
  "        Enter flag for result output on terminal [def.:show|<n>o]: "
define/local show/c/1/1 "{p4}"
!
!
!P5(ps):
!------
define/parameter p5 + ? -
  "        Enter flag for PS files of plots [def.:n|<y>es,create]: "
define/local ps/c/1/1 "{p5}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "VARIANCE(DIFF) <> MED_ADU(SINGLE)" :
!
  define/local xlab/r/1/1 0.
  define/local ylab/r/1/1 0.
  define/local aducf1/r/1/1 0.
  define/local aducf2/r/1/1 0.
!
!     "Results" :
!
  define/local aducf/r/1/1 0.
  define/local char/i/1/1 0
  define/local traplot/c/1/64 " " all
!
!
!
!          MANAGING DISPLAYS :
!          *****************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_gids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 2 is missing !"
    goto error
  endif
  !
  assign/graphic g,2 >Null
  clear/graphic
  set/graphic default
  set/graphic pmode=0 clear=off xformat=E6.0 yformat=E6.2
  !
endif
!
!
!          PREPARING LINEARITY COMPUTATIONS :
!          ********************************
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} "  preparing computations"
!
if m$existc(lintable,"Lin_res1") .eq. -1 -
  create/column {lintable} :Lin_res1 "%" F9.4 R*4 ! f."fitlin" => same order
if m$existc(lintable,"Lin_res2") .eq. -1 -
  create/column {lintable} :Lin_res2 "%" F9.4 R*4 ! f."fitlin" => same order
if m$existc(lintable,":scratch") .eq. -1 -
  create/column {lintable} :scratch " " F9.4 R*4
!
!
!          LINEAR REGRESSIONS VARIANCE (DIFF.FRAMES)
!              VERSUS MED_ADU (SINGLE FRAMES) :
!          *****************************************
!...............................................................................
!
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "  fitting variances of difference frames"
        mess/out 3 {PROID} "  versus medians of single frames"
        mess/out 3 {PROID} "    with 1st frames of the pairs"
  !
select/table {lintable} :Exp_tim1.gt.{sel} >Null
  !
  if nice_crea .eq. "y" then
    assign/graphic g,2 >Null
    set/graphic stype=5
    plot/table {lintable} :Med_cnt1 :Variance -80,-40,40,75  ! ### ### ###
    xlab = plrgrap(1) + (plrgrap(2)-plrgrap(1))/10
    ylab = plrgrap(6) - (plrgrap(6)-plrgrap(5))/10
    label/graphic "Transfer curve (1), edited" {xlab},{ylab} ? ? 1
  endif
  !
regression/linear {lintable} :Variance :Med_cnt1 >Null
aducf1 = 1/outputd(2)
  !
  if nice_crea .eq. "y" then
    save/regression {lintable} regress
    compute/regression {lintable} :scratch = regress
    set/graphic stype=0
    overplot/table {lintable} :Med_cnt1 :scratch
  endif
  !
        set/format T6.3,
        mess/out 3 {PROID} "      conversion fact.: {aducf1} e-/ADU"
        set/format {FMTDEF}
!
!...............................................................................
!
        mess/out 3 {PROID} "    with 2nd frames of the pairs"
  !
select/table {lintable} :Exp_tim2.gt.{sel} >Null
  !
  if nice_crea .eq. "y" then
    set/graphic stype=5
    plot/table {lintable} :Med_cnt2 :Variance -80,-40,40,15  ! ### ### ###
    xlab = plrgrap(1) + (plrgrap(2)-plrgrap(1))/10
    ylab = plrgrap(6) - (plrgrap(6)-plrgrap(5))/10
    label/graphic "Transfer curve (2), edited" {xlab},{ylab} ? ? 1
  endif
  !
regression/linear {lintable} :Variance :Med_cnt2 >Null
aducf2 = 1/outputd(2)
  !
  if nice_crea .eq. "y" then
    save/regression {lintable} regress
    compute/regression {lintable} :scratch = regress
    set/graphic stype=0
    overplot/table {lintable} :Med_cnt2 :scratch
  endif
  !
        set/format T6.3,
        mess/out 3 {PROID} "      conversion fact.: {aducf2} e-/ADU"
        set/format {FMTDEF}
  !
select/table {lintable} all
!
!...............................................................................
!
!
!          RESULTS :
!          *******
!
aducf = (aducf1+aducf2)/2.
!
        mess/out 3 {PROID} " "
        set/format T6.3,
        mess/out 3 {PROID} "  Avg.conversion fact.: {aducf} e-/ADU"
        set/format {FMTDEF}
!
write/keyword outputr 0. all
outputr(1) = aducf
  !
  if nice_crea .eq. "y" then
    if ps .eq. "y" then
      !
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} "  creating postscript file of plots"
      !
      assign/graphic g,2 >Null
      copy/graphic postscript
        char = m$index(lintable,"_lin.tbl")-1
        traplot = lintable(1:{char})//"_tra.ps"
      -rename postscript.ps {traplot}
    endif
  endif
!
!
!          OUTPUT :
!          ******
!
if show .ne. "n" then
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} ">>> OUTPUTR(1): conversion factor [e-/ADU] <<<"
  if nice_crea .eq. "y" .and. ps .eq. "y" then
        mess/out 4 {PROID} ">>> PS file of the plots is <{traplot}> <<<"
  endif
endif
!
!
set/format
if nice_crea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  if m$existk("lintable") .eq. 1 then
    if m$exist(lintable) .eq. 1 select/table {lintable} all
  endif
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" set/graphic default
  endif
  return 1
!
!
!-------------------------------------------------------------------------------
