! $Id: fors_cfgenv.prg,v 1.19 2001/03/21 17:21:10 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_cfgenv.prg,v $
!
!.KEYWORDS    Pipeline environment setup
!
!.USAGE       config/env  obsmode
!
!.INPUT       obsmode     (P1)    Observation mode (LSS,MOS,IMG,ECH)
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The procedure does the final setup of global keywords that serve
!             as a translation table. The contents of these keywords depends
!             on the current pipeline mode. The frame identifier strings
!             should only be accessed via these translations to keep the
!             procedures independent of any specific Data Organizer category
!             name scheme!
!   
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/07/10:  Creation
!
!------------------------------------------------------------------------------

crossref obsmode

define/param p1  ?   c   "Enter observing mode:"
define/maxpar 1


! Procedure identification
define/local procID/c/1/80      "ForsCfgEnv"


! Local variables
define/local obsmode/c/1/3  " "  all



obsmode = m$upper(p1)

! indentifiers common to all modes
set/obsmod SCIENCE_ID  {obsmode}
set/obsmod RSCIENCE_ID {obsmode}
set/obsmod FSCIENCE_ID {obsmode}
set/obsmod STD_ID      {obsmode}
set/obsmod RSTD_ID     {obsmode}
set/obsmod FLAT_ID     {obsmode}
set/obsmod RFLAT_ID    {obsmode}
set/obsmod MFLAT_ID    {obsmode}
set/obsmod STDFLX_ID   {obsmode}
set/obsmod DRSRC_ID    {obsmode}


! Frame identifications specific to direct imaging
if obsmode(1:3) .eq. "IMG" then
  set/obsmod SKY_ID       {obsmode}
  set/obsmod RSKY_ID      {obsmode}
  set/obsmod MSKY_ID      {obsmode}
  set/obsmod SSKY_ID      {obsmode}
  set/obsmod MNIGHT_ID    {obsmode}
  set/obsmod SNIGHT_ID    {obsmode}
  set/obsmod MASTERFF_ID  {obsmode}
  set/obsmod SKYBACK_ID   {obsmode}
  set/obsmod SKYSUB_ID    {obsmode}
  set/obsmod ZEROPOINT_ID {obsmode}
  set/obsmod EXTCOEFF_ID  {obsmode}
  set/obsmod COLCOEFF_ID  {obsmode}
  set/obsmod FOCUS_ID     {obsmode}
  set/obsmod CCDLIN_ID    {obsmode}
  set/obsmod CCDCTE_ID    {obsmode}
endif


! Frame identifications specific to longslit,
! multi object and echelle spectroscopy
inputi = 0
if obsmode(1:3) .eq. "LSS" inputi = 1
if obsmode(1:3) .eq. "MOS" inputi = 1
if obsmode(1:3) .eq. "ECH" inputi = 1
if inputi(1) .eq. 1 then
  set/obsmod MNFLAT_ID  {obsmode}
  set/obsmod NFLAT_ID   {obsmode}
  set/obsmod WLC_ID     {obsmode}
  set/obsmod RWLC_ID    {obsmode}
  set/obsmod DISPREL_ID {obsmode}
  set/obsmod ATMEXT_ID  {obsmode}
  set/obsmod RESP_ID    {obsmode}
  set/obsmod SLITS_ID   {obsmode}
  set/obsmod LINCAT_ID  {obsmode}
  set/obsmod LINPOS_ID  {obsmode}
  set/obsmod ESTD_ID    {obsmode}
  set/obsmod ASTD_ID    {obsmode}
  set/obsmod SBKG_ID    {obsmode}
  set/obsmod SSRC_ID    {obsmode}
  set/obsmod ESCI_ID    {obsmode}
  set/obsmod SCIBKG_ID  {obsmode}
  set/obsmod SCISRC_ID  {obsmode}
endif


! Frame identifications specific to echelle spectroscopy
if obsmode(1:3) .eq. "ECH" then
  set/obsmod ORDER_ID {obsmode}
  set/obsmod BACK_ID  {obsmode}
endif


return 0
