! $Id: fors_chck_iset.prg,v 1.36 1999/11/30 10:40:07 sbogun Exp $   
!+++ FORS_CHCK_ISET +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         $RCSfile: fors_chck_iset.prg,v $
!
!.KEYWORDS      Checks for completness of input set of frames
!
!.USAGE         @p (IDENT) sof_cat expotyp outcat
!
!.INPUT         sof_cat (P1): set of input frames (MIDAS catalog)
!               expotyp (P2): exposure type of CCD_IN (frame to be processed)
!               outcat  (P3): name of output catalog (cal.set)
!               scale   (P4): Master bias scaling method (allowed values:
!                             "MED", "OVE", "NO" (see fors_img_obs_reduce.prg).
!.OUTPUT        
!
!.RETURN        Q1:  0: successful return
!                   -1: specified frame does not exist
!                   -2: desired (dpr.type == P2) input frame NOT specified
!                   -3: flatfield missing (only for science and std-*)
!                    1: calibration frames necessary for reduction are missing!
!
!.DEFAULT       P2      SCI
!               P3      cal.cat
!               P4      NO
!
!.PURPOSE       Check input data set for calling program and set CCDRED
!               keywords:
!                  CCD_IN, SC_BSFRM, SC_DKFRM, SC_FFFRM, SC_ILFRM, SC_FRFRM,
!               as well as
!                  SC_BSCOR, SC_DKCOR, SC_FFCOR, SC_ILCOR, SC_FRCOR
!
!.ALGORITHM     Reset all SC_... keywords (no action).
!               Step through the whole list of input frames. Check their
!               existance, exposure type and analyse if a complete set of input
!               frames exists (at minimum science and flatfield).
!               Do the appropriate update of the CCDRED keywords.
!               If parameter P2 is set to SKY or FF then the program will check
!               for a sky or flatfield frame instead of a science frame! This
!               case is needed in order to check the set of input frames for
!               illumination correction purposes, too
!               At the end CCD_IN will be set to the frame whose exposure type
!               has been indicated by P2 (e.g. P2=FF -> CCD_IN == flatfield !).
!               The flatfields for the science/standard star reduction have
!               the following priority:
!               1. SUPER_NIGHT_FLAT
!               2. MASTER_NIGHT_FLAT
!               3. SUPER_SKY_FLAT
!               4. MASTER_SKY_FLAT
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1997/12/18  Creation
!               1.0 1998/12/23  EXTRACT/IMAGE changed to EXTRACT/FRAME
!                               due to refpix-bug in EXTRACT/IMAGE
!               1.2 1999/02/16  screen flat detection if P2 is a MASTER_SKY-
!                               FLAT or MASTER_NIGHT_FLAT (SUPER-flat crea.)
!                   1999/03/10  check for flatfield if P2 is science or std-*
!
!.COMMENT       Caution:
!               . At the moment only at maximum ONE of each frame-type is
!                 assumed to be in the frame catalog. If this would not be
!                 the case then the last occurence of a specific frame will
!                 be used !!
!               . For a list of actual exposure types see:
!                 DICB (GEN-SPE-ESO-19400-794)
!               . SUPER_NIGHT_FLAT has highest priority! Follwed by
!                 MASTER_NIGHT_FLAT, SUPER_SKY_FLAT, MASTER_SKY_FLAT.
!                 MASTER_SCREEN_FLAT will never be used, they are only
!                 used for the SUPER-flat creation.
!
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name of MIDAS-catalog of input set:"
define/param p2 SCI c "requested exposure type of CCD_IN: "
define/param p3 cal.cat c "name of calibration catalog:"
define/param p4 NO c "Use scaled master bias (MED/OVE/NO):"

define/local cnt1/i/1/1 0
define/local iilc/i/1/1 0
define/local ifrc/i/1/1 0
define/local ibia/i/1/1 0
define/local idar/i/1/1 0
define/local ifla/i/1/1 0
define/local indx/i/1/1 0
define/local catal/i/1/1 0
define/local fflag/i/1/1 0
define/local ImSc/i/1/4 0 ALL +lower_levels
define/local bs_scl/c/1/3 " " ALL
define/local SC_BSTPL/c/1/100 " " ALL
define/local expt/c/1/25 " "
define/local nused/c/1/15 "not_used"
define/local rfrm/c/1/80 " "
define/local warn0/c/1/80 " "
define/local warn1/c/1/80 " "
define/local warn2/c/1/80 " "
define/local warn3/c/1/80 " "
define/local warn4/c/1/80 " "
warn1 = "frame {rfrm} does not exist!"
warn2 = "NO {p2}-frame specified or could not be found!"
warn3 = "NO flatfield-frame specified or could not be found!"
warn4 = "Select NEXT frame of detected frame type, OLD: "

bs_scl = m$upper(p4(1:3))

create/icat {P3} NULL {FORS_IDSTRNG} >Null      !create output catalog


! initialize coordinate descriptors:
! initialize correction flags:
SC_SCAN = "no"          !Oversc. corr.  
SC_TRIM = "no"          !Trim the frame 
SC_FXPIX = "no"         !Fix bad pixel  
SC_BSCOR = "no"         !Bias correct.  
SC_DKCOR = "no"         !Dark correct.  
SC_FFCOR = "no"         !Flat correct.  
SC_ILCOR = "no"         !Illum correct. 
SC_FRCOR = "no"         !Fringe correct.
SC_ILFRM = nused        !Illum correct. frame
SC_FRFRM = nused        !Fringe correct. frame
SC_FFFRM = nused        !Flatfield frame
!SC_DKFRM = nused       !dark frame
!SC_BSFRM = nused       !bias frame
FX_TABLE = nused        !Bad pixel table
CCD_IN   = " "          !Science frame





ima_loop:
  store/frame rfrm {p1} 1 finish 
  if m$exist(rfrm) .eq. 1 then                  !frame exists
    if m$existd(rfrm,"SKYCOR") .eq. 1 .OR. -
       m$existd(rfrm,"ILLCOR") .eq. 1 then      !frame specified
       cnt1 = m$index(SC_ILFRM,nused)
       if cnt1 .le. 0 mess/out 2 chck_iset "1:{warn4} {SC_ILFRM}"
       iilc = 1
       SC_ILCOR = "yes"
       SC_ILFRM = rfrm
       swap/units {rfrm} PIXEL                  !we have to work on PIXEL syst.
       add/icat {P3} {rfrm} >Null
       goto ima_loop    !next frame, should not be identified as SCIENCE frame!
    endif
    if m$existd(rfrm,"FRICOR") .eq. 1 then      !fringe correction frame
       cnt1 = m$index(SC_FRFRM,nused)
       if cnt1 .le. 0 mess/out 2 chck_iset "2:{warn4} {SC_FRFRM}"
       ifrc = 1
       SC_FRCOR = "yes"
       SC_FRFRM = rfrm
       swap/units {rfrm} PIXEL                  !we have to work on PIXEL syst.
       add/icat {P3} {rfrm} >Null
       goto ima_loop
    endif
! 
! The exposure descriptor keyword EXP_DESC is defined in ESO_DESCR.PRG and
! finally set in FORS_INITFORS.PRG to DO_CLASSIFICATION.
! As EXP_DESC has to be reset due to identification of the MASTER frames
! within ccdredfrm (BRANCH command) the EXP_DESC will be set to a NEW DESCR.:
! EXPO_TYPE
    WRITE/KEYW   EXP_DESC/C/1/20  "EXPO_TYPE"   ! exposure type descr.

    expt = m$value({rfrm},{FORS_IDSTRNG})
!    write/out "EXPT: {expt} ({P2})"    
    if expt .eq. "{p2}" then
       cnt1 = M$LEN(CCD_IN)
       if cnt1 .gt. 0 mess/out 2 chck_iset "3:{warn4} {CCD_IN}"
       write/descr {rfrm} {EXP_DESC}/c/1/4 OBJE
       SC_TYP = "{{rfrm},{EXP_DESC}}"
       CCD_IN = rfrm                            !CCD_IN does not allow
       indx = m$indexb(rfrm,".bdf") - 1         !any extension [.bdf]
       if indx .gt. 0 CCD_IN = rfrm(1:{indx})
    else if expt .eq. "{SNIGHT_ID}" then
       cnt1 = m$index(SC_FFFRM,nused)           !SUPER NIGHT FLAT
       if cnt1 .le. 0 mess/out 2 chck_iset "4:{warn4} {SC_FFFRM}"
       ifla = 1
       SC_FFCOR = "yes"
       SC_FFFRM = rfrm
       write/descr {rfrm} {EXP_DESC}/c/1/4 FLAT
       FF_TYP = "{{rfrm},{EXP_DESC}}"
       fflag = 4                                !flat.flag controls priority
       swap/units {rfrm} PIXEL                  !we have to work on PIXEL syst.
       add/icat {P3} {rfrm} >Null
    else if expt .eq. "{MNIGHT_ID}" .AND. fflag .lt. 4 then
       cnt1 = m$index(SC_FFFRM,nused)           !MASTER NIGHT FLAT
       if cnt1 .le. 0 mess/out 2 chck_iset "5:{warn4} {SC_FFFRM}"
       ifla = 1
       SC_FFCOR = "yes"
       SC_FFFRM = rfrm
       write/descr {rfrm} {EXP_DESC}/c/1/4 FLAT
       FF_TYP = "{{rfrm},{EXP_DESC}}"
       fflag = 3                                !flatfield flag
       swap/units {rfrm} PIXEL                  !we have to work on PIXEL syst.
       add/icat {P3} {rfrm} >Null
    else if expt .eq. "{SSKY_ID}" .AND. fflag .lt. 3 then
       cnt1 = m$index(SC_FFFRM,nused)           !MASTER SKY FLAT
       if cnt1 .le. 0 mess/out 2 chck_iset "5:{warn4} {SC_FFFRM}"
       ifla = 1
       SC_FFCOR = "yes"
       SC_FFFRM = rfrm
       write/descr {rfrm} {EXP_DESC}/c/1/4 FLAT
       FF_TYP = "{{rfrm},{EXP_DESC}}"
       fflag = 2                                !flatfield flag
       swap/units {rfrm} PIXEL                  !we have to work on PIXEL syst.
       add/icat {P3} {rfrm} >Null
    else if expt .eq. "{MSKY_ID}" .AND. fflag .lt. 2 then
       cnt1 = m$index(SC_FFFRM,nused)           !MASTER SKY FLAT
       if cnt1 .le. 0 mess/out 2 chck_iset "5:{warn4} {SC_FFFRM}"
       ifla = 1
       SC_FFCOR = "yes"
       SC_FFFRM = rfrm
       write/descr {rfrm} {EXP_DESC}/c/1/4 FLAT
       FF_TYP = "{{rfrm},{EXP_DESC}}"
       fflag = 1
       swap/units {rfrm} PIXEL                  !we have to work on PIXEL syst.
       add/icat {P3} {rfrm} >Null
    else if expt .eq. "{MFLAT_ID}" .AND. p2(1:6) .eq. "MASTER" then
       cnt1 = m$index(SC_FFFRM,nused)           !MASTER SKY FLAT
       if cnt1 .le. 0 mess/out 2 chck_iset "5:{warn4} {SC_FFFRM}"
       ifla = 1
       SC_FFCOR = "yes"
       SC_FFFRM = rfrm
       write/descr {rfrm} {EXP_DESC}/c/1/4 FLAT
       FF_TYP = "{{rfrm},{EXP_DESC}}"
       fflag = 1
       swap/units {rfrm} PIXEL                  !we have to work on PIXEL syst.
       add/icat {P3} {rfrm} >Null
    else if expt .eq. "{MDARK_ID}" then         !MASTER DARK
       idar = 1
       SC_DKCOR = "yes"
       SC_DKFRM = rfrm
       write/descr {rfrm} {EXP_DESC}/c/1/4 DARK
       DK_TYP = "{{rfrm},{EXP_DESC}}"
       add/icat {P3} {rfrm} >Null
    else if expt .eq. "{MBIAS_ID}" then         !MASTER BIAS
       ibia = 1
       SC_BSCOR = "yes"
       SC_BSFRM = rfrm

       if bs_scl .eq. "MED" .or. bs_scl .eq. "OVE" then
         SC_BSTPL = "TPL" // SC_BSFRM
         normalize/mbias {SC_BSFRM} {SC_BSTPL} {bs_scl}
         if {q1} .ne. 0 then
           mess/out 1 chck_iset "Normalization of master bias failed !"
           return 1
         endif
       endif

       write/descr {rfrm} {EXP_DESC}/c/1/4 BIAS
       BS_TYP = "{{rfrm},{EXP_DESC}}"

       if bs_scl .eq. "NO" then
         add/icat {P3} {rfrm} >Null
       endif
    else if m$index(expt,"{BPIXT_ID}") .gt. 0 then
       FX_TABLE = rfrm                          !BAD PIXEL TABLE
       SC_FXPIX = "no "
    endif

  else                                          !frame does not exist

    mess/out 1 chck_iset "{warn1}"
    return -1
  endif
goto ima_loop
finish:

!selected exposure type of frame to be reduced == FLATFIELD:

!if m$index(p2,"{FLAT_ID}") .gt. 0 SC_FFFRM = "{CCD_IN}.bdf"
if m$index(p2,"{FLAT_ID}") .eq. 1 -
   .OR. m$index(p2,"{SKY_ID}") .eq. 1 then
  SC_FFFRM = nused                              !NO FF-correction allowed
  SC_FFCOR = "no"
endif
if m$index(p2,"{DARK_ID}") .gt. 0 then
  SC_DKFRM = nused                              !wether FF nor DARK 
  SC_DKCOR = "no"                               !correction allowed.
  SC_FFFRM = nused
  SC_FFCOR = "no"
endif

cnt1 = M$LEN(CCD_IN)
if cnt1 .eq. 0 then
  mess/out 1 chck_iset "{warn2}"        !to be processed frame type ..
  return -2                             !is missing !
endif

append/out "INPUT frame:\n"
append/out " {SC_TYP}: {CCD_IN}.bdf [{{CCD_IN},{FORS_IDSTRNG}}]\n\n"
append/out "CALIBRATIONS applied:\n"
if SC_BSFRM(1:8) .ne. nused then
  if m$tstno(SC_BSFRM) .ne. 1 then

! Use scaled bias template if requested.
    if {q1} .ne. 0 then
      mess/out 1 chck_iset "Bias values cannot be determined from overscan!"
      return 1
    endif
    if SC_BSCOR .eq. "yes" .and. bs_scl .ne. "NO" then
      SC_BSFRM = "SCL" // SC_BSTPL(4:)
      scale/mbias {CCD_IN}.bdf {SC_BSTPL} {SC_BSFRM}
      if {Q1} .ne. 0 then 
        mess/out 1 chck_iset "Master bias could not be scaled!"
        return 1
      endif
      add/icat {P3} {SC_BSFRM} >Null
    endif

    append/out " BIAS: {SC_BSFRM} [{{SC_BSFRM},{FORS_IDSTRNG}}]\n"
  else if {SC_BSFRM} .ne. 0 then
    SC_BSCOR = "yes"
    append/out " constant BIAS value subtracted: {SC_BSFRM}\n"
  endif
endif
if SC_DKFRM(1:8) .ne. nused then
  if m$tstno(SC_DKFRM) .ne. 1 then
    append/out " DARK: {SC_DKFRM} [{{SC_DKFRM},{FORS_IDSTRNG}}]\n"
  else if {SC_DKFRM} .ne. 0 then
    SC_DKCOR = "yes"
    append/out " constant DARK value subtracted: {SC_DKFRM}\n"
  endif
endif
if SC_FFFRM(1:8) .ne. nused then
  append/out " FLAT: {SC_FFFRM} [{{SC_FFFRM},{FORS_IDSTRNG}}]\n"
endif
if SC_ILFRM(1:8) .ne. nused then
  append/out " ILLC: {SC_ILFRM} [{{SC_ILFRM},{FORS_IDSTRNG}}]\n"
endif
if SC_FRFRM(1:8) .ne. nused then
  append/out " FRIC: {SC_FRFRM} [{{SC_FRFRM},{FORS_IDSTRNG}}]\n"
endif
if FX_TABLE(1:8) .ne. nused then
  append/out " BPIX: {FX_TABLE} [{{FX_TABLE},{FORS_IDSTRNG}}]\n"
endif
mess/out 4 chck_iset




branch P2 {STD_ID},{SCIENCE_ID},{FLAT_ID},{SKY_ID},{DARK_ID},{BIAS_ID} -
          SCI,SCI,FLA,FLA,DAR,BIA
return 0

SCI:
     ibia = ibia + idar + ifla

     if ifla .eq. 0 then
       mess/out 1 chck_iset "No flatfield specified !"
       return -3
     endif

     @p fors_getwin {CCD_IN}
     if {Q1} .eq. 0 then        !no window readout
       mess/out 4 chck_iset "Extract {P2} image ..."
       image/section {CCD_IN}.bdf
       ImSc(1) = outputi(1)
       ImSc(2) = outputi(2)
       extract/frame {CCD_IN}.bdf middummx.bdf {out_b}
       -rename middummx.bdf {CCD_IN}.bdf
     else
       ImSc(1) = outputi(1)
       ImSc(2) = outputi(2)
     endif
     !
     ! The science frame has been extracted now, the world coordinate system
     ! has been adjusted. REDUCE/CCD uses COMPUTE/IMAGE which only works on
     ! frames having the same coordinate system.
     ! As all calibration frames and bad pixel tables are based on PIXEL coord.
     ! system the science frame has to be *temporarily* transferred from WORLD
     ! coord.system to the PIXEL system.
     ! Usually, the procedure doing this job: SWAP/UNITS assumes to find
     ! the corresponding PIXEL coordinates in the ESO.DET.WIN1.STR? descriptor
     ! but after the EXTRACTION (cutting the overscan regions) these descriptor
     ! should not be updated. Thus the start pixel will be passed to the
     ! procedure by the global key ImSc.
     !
     mess/out 4 chck_iset "Set coordinate system({P2}) to PIXEL."
     swap/units {CCD_IN}.bdf PIXEL ImSc !we have to work on PIXEL syst.
     goto DAR
FLA: 
     ibia = ibia + idar
DAR: 
     if ibia .eq. 0 then
       mess/out 2 chck_iset "NO calibration frames for reduction specified!"
       return 1
     endif
BIA:

return 0
