! $Id: fors_check_overl.prg,v 1.6 2000/09/28 12:32:01 nrainer Exp $   
!+++ FORS_CHECK_OVERL ++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! Author: Sebastian Wolf @ESO                               Date:   97-12-18
! Purpose:
!   Check overlap of areas defined in a specified table file
! Algorithm:
!   The table files have to have a certain structure. It must contain 5
!   columns labeled by: :XSTART, :XEND, :YSTART, :YEND, and :VALUE1.
!   The column :VALUE1 has to be non zero whereas the other four define
!   the areas. The second table has to be empty and will be filled by the
!   combination of overlapping regions and non-overlapping regions.
!   There are two modes to check the overlapping areas depending on how
!   many areas have been found in total. If the number of areas is less
!   than 33 then all areas may be checked in one step otherwise this procedure
!   has to be called iteratively:
!   The first table will become a copy of the second in order to be able
!   to iteratively determine re-overlapped regions (produced by this
!   combination of areas) by re-calling this procedure.
!
!   At first an image will be created by the area-definition table (P2).
!   This will be done in the ADD mode of the WRITE/IMAGE command. As each
!   area has a datavalue of 2^n, overlapping areas within the new image will
!   be indicated by values > 2! Thus a simple FIND/PIXEL command will create
!   an overlap-table (local: overl). Step through each of the detected pixels
!   and comparing it with the areas of table P2 each pixel may be registered
!   as a member of the current area or not.
! Context:
!
! Input:
!   P1: name of table defining all areas
!   P2: name of table defining smallest combination of overlapping areas
! Output:
!   table defining smallest combination of overlapping areas
!---------------------------------------------------------------------------

define/param p1 ? c "Name of area table: "
define/param p2 ? c "Name of result area table: "

define/local STEPX/i/1/1 1
define/local NDIMX/i/1/1 2080


IN_A = "{P1}.tbl"                    	!area definition table
OUT_A = "{P2}.tbl"                   	!badpixel table
STEPX = {P3}
NDIMX = {P4}

run PIPE_EXE:fors_check_overl.exe

return 0				!finish -> RETURN to calling program
