! $Id: fors_checknlines.prg,v 1.9 2001/03/13 10:44:43 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_checknlines.prg,v $
!
!.KEYWORDS    Simple check of dispersion solution.
!
!.USAGE       check/nlines frame [order] [slits]
!
!.INPUT       frame       (P1)    Wavelength calibration frame.
!             order       (P2)    Degree of fit polynomial.
!             slits       (P3)    Slit positions table.
!
!.OUTPUT      None
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. The keyword OUTPUTI(1) is 0 if
!             there are enough line detections, otherwise it is set to 1.
!
!.PURPOSE     The procedure checks if the mean number of detected
!             lines per slit is large enough to perform a polynomial fit
!             of the line positions with the selected order of the fit.
!             The check fails if the number of lines is smaller than
!             the number of polynom coefficients plus 1 so that it is
!             not only possible to fit the data but also to calculate the
!             residuals.
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/03/10:  Creation
!
!------------------------------------------------------------------------------

crossref  frm order slits

define/param p1 ? c "Enter calibration frame:"
define/param p2 + c "Enter order of fit polynom:"
define/param p3 ? c "Enter slit positions table:"
define/maxpar 3


! Procedure identification
define/local procID/c/1/20      "CheckNlines"


! Local variables
define/local i/i/1/1         0
define/local order/i/1/1     0
define/local nslt/i/1/1      0
define/local act_slit/i/1/1  0
define/local nl/i/1/1        0
define/local limit/i/1/1     0
define/local failed/i/1/1    0
define/local frm/c/1/100     {p1}
define/local spos/c/1/100    {p3}


! Process parameters
if p2(1:1) .eq. "+" then
  order = wlcord(1)
else
  if m$tstno(p2) .ne. 1 then
    mess/out 1 {procID} "Invalid order of fit polynom!"
    return 1
  else
    order = {p2}
  endif
endif

! Get number of active slitlets from slit position table
nslt = m$value({spos},tblcontr(4))
if nslt .ne. nslit then
  mess/out 1 {procID} "Invalid slit position table encountered!"
  return 1
endif


! Check if line detection descriptor exists
if m$existd(frm,"NLINES") .ne. 1 then
  mess/out 1 {procID} "Line detection descriptor is missing!"
  return 1
endif

! Check if the number of detected lines is below limit for any slitlet
! Increase number by one for safty.
limit = order + 2 + 1

if m$existc(spos,":CHK") .eq. -1 create/column {spos} :CHK " " I3 I*1
compute/table {spos} :CHK = 0

do i = 1 nslt
  nl = m$value({frm},NLINES({i}))
  act_slit = {{spos},:{c_slsno},@{i}}   ! slitlet index may differ from loop
                                        ! counter
  set/format i1
  if nl .lt. limit then
    mess/out 2 {procID} "Not enough line detections for slit #{act_slit}!"
    mess/out 2 {procID} "  Selected order of polynomial fit: {order}."
    mess/out 2 {procID} "  Number of detected lines: {nl}."
    mess/out 2 {procID} "  Slit will not be used."
    failed = failed+1
    NSLIT = NSLIT-1       ! filled by def/pos, later also used by check/disp
    {spos},:CHK,@{i} = 1
  else
    mess/out 3 {procID} "Number of detected lines in slit #{act_slit}: {nl}."
  endif
  set/format
enddo

! Remove descriptor NLINES from frame
delete/descr {frm} NLINES

select/table {spos} :CHK.eq.0 >Null  ! only these slits have enough lines
copy/table {spos} tmp_{spos}
-rename tmp_{spos} {spos}
delete/column {spos} :CHK
!
if failed .lt. nslt then
  outputi(1) = 0
else
  outputi(1) = 1
endif

return 0
