! $Id: fors_chkobs_crea.prg,v 1.8 1999/11/25 11:23:49 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_chkobs_crea.prg,v $
!
! .KEYWORDS   Spectroscopical quality
!
! .PURPOSE    Create table with y-range of topical object
!             and x-coordinates of wavelength bins.
! __________
!
! .USAGE      @p fors_chkobs_crea subima ylohi [binsz] [binmd] outtbl
!
! .INPUT      subima (P1): sky-subtracted standard (or science) frame;
!                            default: none
!
!             ylohi  (P2): lower y-world-coordinate (arcsec) of object AND
!                          higher y-world-coordinate of object, separated by a
!                          comma;
!                            default: none
! 
!             binsz  (P3): number of x-pixels per wavelength bin;
!                            default: 27
!
!             binmd  (P4): flag for mode of binning: <b>:normal binning,
!                          <o>:overlapping bins in steps of 1 pixel;
!                            default: <b>
!
!             outtbl (P5): name of output table with bin coordinates (will be
!                          created);
!                            default: none
!
! .OUTPUT     "<P5>": table with bin coordinates
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for spectroscopy - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/02/09  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref subima ylohi binsz binmd outtbl
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "chkobs_crea"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      subima (P1): sky-subtracted standard (or science) frame;"
  w/o "                     default: none"
  w/o " "
  w/o "      ylohi  (P2): lower y-world-coordinate (arcsec) of object AND"
  w/o "                   higher y-world-coordinate of object, separated by a"
  w/o "                   comma;"
  w/o "                     default: none"
  w/o " "
  w/o "      binsz  (P3): number of x-pixels per wavelength bin;"
  w/o "                     default: 27"
  w/o " "
  w/o "      binmd  (P4): flag for mode of binning: <b>:normal binning,"
  w/o "                   <o>:overlapping bins in steps of 1 pixel;"
  w/o "                     default: <b>"
  w/o " "
  w/o "      outtbl (P5): name of output table with bin coordinates (will be"
  w/o "                   created);"
  w/o "                     default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(subima):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of sky subtracted frame: "
define/local subima/c/1/64 "{p1}"
if m$index(subima,".bdf") .eq. 0 subima = subima//".bdf"
!
if m$exist(subima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({subima})"
  goto error
endif
!
!
!P2(ylohi):
!---------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter y-range of object (ylow,yhigh): "
outputi = m$parse(p2,"yrange")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local ylow/r/1/1 {yrange01}
define/local yhigh/r/1/1 {yrange02}
!
!
!P3(binsz):
!---------
define/parameter p3 27 n -
  "        Enter binsize [def.:27(pixels);>=10!!]: "
define/local binsize/i/1/1 {p3}
!
outputi(1) = {{subima},NPIX(1)}
if binsize .lt. 3 .or. binsize .gt. outputi(1) then
        write/out " "
        mess/out 1 {PROID} "Binsize out of range !"
        mess/out 1 {PROID} "({binsize} ; 3<=binsize<={outputi(1)})"
  goto error
endif
!
!
!P4(binmd):
!---------
define/parameter p4 b ? -
  "        Enter flag for binmode [def.:<b>ins|<o>verlapping]: "
define/local binmode/c/1/1 "{p4}"
!
!
!P5(outtbl):
!----------
if p5 .eq. "?" mess/out 2 {PROID} " "
define/parameter p5 ? ? -
  "        Enter name of output table: "
define/local bintbl/c/1/64 "{p5}"
if m$index(bintbl,".tbl") .eq. 0 bintbl = bintbl//".tbl"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local xpix/i/1/1 0
  define/local bins/i/1/1 0
  define/local xpixs/i/1/1 0
  define/local xstart/d/1/1 0.
  define/local xstep/d/1/1 0.
  define/local binloop/i/1/1 0
!
!
!
!          CREATING TABLE WITH BIN POSITIONS :
!          *********************************
!
xpix = {{subima},NPIX(1)}
outputr(1) = binsize
bins = m$nint(xpix/outputr(1)-0.5)
xpixs = xpix-(binsize-1)
xstart = {{subima},START(1)}
xstep = {{subima},STEP(1)}
!
if binmode .ne. "o" then  ! Bins:
  !
  create/table {bintbl} 15 {bins} null  ! 11 cols. added later
  create/column {bintbl} :XSTART "angstrom" F9.2 R*4
  create/column {bintbl} :XEND   "angstrom" F9.2 R*4
  create/column {bintbl} :YSTART "arcsec"   F7.2 R*4
  create/column {bintbl} :YEND   "arcsec"   F7.2 R*4
  !
        mess/out 3 {PROID} "    binsize: {binsize}  bins: {bins}"
  !
  do binloop = 1 bins
    {bintbl},:XSTART,{binloop} = xstart+xstep*((binloop-1)*binsize)
    {bintbl},:XEND,{binloop} = xstart+xstep*((binloop-1)*binsize+(binsize-1))
    {bintbl},:YSTART,{binloop} = ylow
    {bintbl},:YEND,{binloop} = yhigh
  enddo
  !
else                      ! Overlapping:
  !
  create/table {bintbl} 15 {xpixs} null  ! 11 cols. added later
  create/column {bintbl} :XSTART "angstrom" F9.2 R*4
  create/column {bintbl} :XEND   "angstrom" F9.2 R*4
  create/column {bintbl} :YSTART "arcsec"   F7.2 R*4
  create/column {bintbl} :YEND   "arcsec"   F7.2 R*4
  !
        mess/out 3 {PROID} "    binsize: {binsize}  overlapping bins: {xpixs}"
  !
  do binloop = 1 xpixs
    {bintbl},:XSTART,{binloop} = xstart+xstep*(binloop-1)
    {bintbl},:XEND,{binloop} = xstart+xstep*((binloop-1)+(binsize-1))
    {bintbl},:YSTART,{binloop} = ylow
    {bintbl},:YEND,{binloop} = yhigh
  enddo
  !
endif
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------
