! $Id: fors_chkobs_ex3d.prg,v 1.11 2000/02/23 14:49:30 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_chkobs_ex3d.prg,v $
!
! .KEYWORDS   Spectroscopical quality
!
! .PURPOSE    Extract two-dimensional table with spec. values of one object
!             from three-dimensional table with input values of all objects.
! __________
!
! .USAGE      @p fors_chkobs_ex3d dim3tbl row subima outtbl
!
! .INPUT      dim3tbl (P1): three-dimensional input spectroscopy table;
!                             default: none
!
!             row     (P2): row (object) number of P1;
!                             default: none
!
!             subima  (P3): sky-subtracted standard (or science) frame;
!                             default: none
!
!             outtbl  (P4): name of two-dimensional output table (will be
!                           created);
!                             default: none
!
! .OUTPUT     "<P4>": two-dimensional spectroscopy table (one object extracted
!                     per invocation)
!
!             Q2: y-coords.(start,end) of obj. in sky subtr. frame [pixel]
!
!             Q3: y-coords.(start,end) of obj. in sky subtr. frame [arcsec]
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for spectroscopy - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/02/19  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref dim3tbl row subima outtbl
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "chkobs_ex3d"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
write/keyword q3 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      dim3tbl (P1): three-dimensional spectroscopy table;"
  w/o "                      default: none"
  w/o " "
  w/o "      row     (P2): row (object) number of P1;"
  w/o "                      default: none"
  w/o " "
  w/o "      subima  (P3): sky-subtracted standard (or science) frame;"
  w/o "                      default: none"
  w/o " "
  w/o "      outtbl  (P4): name of two-dimensional output table;"
  w/o "                      default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(dim3tbl):
!-----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? t -
  "        Enter 3D input table: "
define/local dim3tbl/c/1/64 "{p1}"
if m$index(dim3tbl,".tbl") .eq. 0 dim3tbl = dim3tbl//".tbl"
!
if m$existc(dim3tbl,":{cslit}") .eq. -2 then  ! col. names are "+lower_levels"
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({dim3tbl})"
  goto error
elseif m$existc(dim3tbl,":{cslit}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({dim3tbl},:{cslit})"
  goto error
elseif m$existc(dim3tbl,":{cra}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({dim3tbl},:{cra})"
  goto error
elseif m$existc(dim3tbl,":{cdec}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({dim3tbl},:{cdec})"
  goto error
elseif m$existc(dim3tbl,":{clambda}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({dim3tbl},:{clambda})"
  goto error
elseif m$existc(dim3tbl,":{cflux}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({dim3tbl},:{cflux})"
  goto error
elseif m$existc(dim3tbl,":{cystart}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({dim3tbl},:{cystart})"
  goto error
elseif m$existc(dim3tbl,":{cyend}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({dim3tbl},:{cyend})"
  goto error
endif
!
show/table {dim3tbl} >Null
define/local dim3rows/i/1/1 {outputi(2)}
if dim3rows .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({dim3tbl})"
  goto error
endif
!
!
!P2(row):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? n -
  "        Enter row number: "
define/local rowloop/i/1/1 {p2}
!
if rowloop .lt. 1 .or. rowloop .gt. dim3rows then
        write/out " "
        mess/out 1 {PROID} "Row number out of range !"
        mess/out 1 {PROID} "({rowloop} ; 1<=row<={dim3rows})"
  goto error
endif
!
!
!P3(subima):
!----------
if p3 .eq. "?" mess/out 2 {PROID} " "
define/parameter p3 ? i -
  "        Enter name of sky subtracted frame: "
define/local subima/c/1/64 "{p3}"
if m$index(subima,".bdf") .eq. 0 subima = subima//".bdf"
!
if m$exist(subima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({subima})"
  goto error
endif
!
!
!P4(outtbl):
!----------
if p4 .eq. "?" mess/out 2 {PROID} " "
define/parameter p4 ? ? -
  "        Enter name of 2D output table: "
define/local dim2tbl/c/1/64 "{p4}"
if m$index(dim2tbl,".tbl") .eq. 0 dim2tbl = dim2tbl//".tbl"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local xpix/i/1/1 0
  define/local slit/i/1/1 0
  define/local ra/r/1/1 0.
  define/local dec/r/1/1 0.
  define/local waveloop/i/1/1 0
  define/local ystart/d/1/1 0.
  define/local ystep/d/1/1 0.
  define/local ystartpix/r/1/1 0.
  define/local yendpix/r/1/1 0.
  define/local maxbinsize/i/1/1 0
  define/local ylow/r/1/1 0.
  define/local yhigh/r/1/1 0.
!
!                                                _  array (lam./pos./flx.-vals.;
!                                                /| depth = x-ax. pix.)
!          EXTRACTING OBJECT PLANE              /
!          FROM 3D INPUT TABLE :               +-------> cols.(lambda,pos.,flux)
!          ***********************             |
!                                              V rows (objects)
!
xpix = {{subima},NPIX(1)}
create/table {dim2tbl} 12 {xpix} null  ! 5 cols. added later
create/column {dim2tbl} :SLIT " "   I4    I*4
create/column {dim2tbl} :RA   "deg" F11.5 R*8
create/column {dim2tbl} :DEC  "deg" F10.5 R*8
!
        mess/out 4 {PROID} "    filling object values into 2D table"
!
slit = {{dim3tbl},:{cslit},{rowloop}}
ra = {{dim3tbl},:{cra},{rowloop}}
dec = {{dim3tbl},:{cdec},{rowloop}}
do waveloop = 1 xpix  ! copying of following values from first to all rows
  {dim2tbl},:SLIT,{waveloop} = slit
  {dim2tbl},:RA,{waveloop} = ra
  {dim2tbl},:DEC,{waveloop} = dec
enddo
!
!***************************************************************
@p fors_t3dextract {dim3tbl} {clambda} {rowloop} {dim2tbl} tmp +
  if {q1} .eq. 1 goto extfail
  create/column    {dim2tbl} :LAMBDA "angstrom" F9.2 R*4
  copy/tt          {dim2tbl} :tmp {dim2tbl} :LAMBDA
  delete/column    {dim2tbl} :tmp
!***************************************************************
!
create/column {dim2tbl} :YSTART "arcsec" F7.2 R*4
create/column {dim2tbl} :YEND   "arcsec" F7.2 R*4
!
ystart = {{subima},START(2)}
ystep = {{subima},STEP(2)}
ystartpix = {{dim3tbl},:{cystart},{rowloop}}  ! -> q2
yendpix = {{dim3tbl},:{cyend},{rowloop}}      ! -> q2
if ystartpix .ge. yendpix then                ! possible error in input table !
  ! mess/out in fors_chkobs.prg because of return values.
  warn = 1
  goto skipped
endif
!
outputr(1) = yendpix-ystartpix+1
maxbinsize = m$nint(MAXFITNPIX/outputr(1)-0.5)
if binsize .gt. maxbinsize then
        write/out " "
        mess/out 1 {PROID} "Binsize too high !"
        mess/out 1 {PROID} "({binsize} ; {MINFITXPIX}<=binsize<={maxbinsize})"
  goto error
endif
!
ylow = ystart+ystep*(ystartpix-1.)  ! -> q3
yhigh = ystart+ystep*(yendpix-1.)   ! -> q3
!
compute/table {dim2tbl} :YSTART = {ylow}
compute/table {dim2tbl} :YEND   = {yhigh}
!
!*************************************************************
@p fors_t3dextract {dim3tbl} {cflux} {rowloop} {dim2tbl} tmp +
  if {q1} .eq. 1 goto extfail
  create/column    {dim2tbl} :FLUX "ADU" G13.6 R*4
  copy/tt          {dim2tbl} :tmp {dim2tbl} :FLUX
  delete/column    {dim2tbl} :tmp
!*************************************************************
goto end_extfail
extfail:
        write/out " "
        mess/out 1 {PROID} "Extraction of column failed !"
        mess/out 1 {PROID} "({dim3tbl} -> {dim2tbl})"
  goto error
end_extfail:
!
!
skipped:
!
!
set/format
!
!
if warn .eq. 0 then
  return 0 {ystartpix},{yendpix} {ylow},{yhigh}
else
  return -1 {ystartpix},{yendpix} {ylow},{yhigh}
endif
!
error:
  set/format
  return 1 -99 -99
!
!
!-------------------------------------------------------------------------------
