! $Id: fors_chkobs_fwhm.prg,v 1.10 1999/11/25 11:23:50 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_chkobs_fwhm.prg,v $
!
! .KEYWORDS   Spectroscopical quality
!
! .PURPOSE    Compute FWHM values of current object (per wavelength bin,
!             perpendicular to dispersion direction).
! __________
!
! .USAGE      @p fors_chkobs_fwhm subima bintbl objtbl objval
!
! .INPUT      subima (P1): sky-subtracted standard (or science) frame;
!                            default: none
!
!             bintbl (P2): table with bin coordinates;
!                            default: none
!
!             objtbl (P3): two-dimensional object table;
!                            default: none
!
!             objval (P4): current row (object) number of three-dimensional
!                          spectroscopy table AND lower y-pixel-coordinate of
!                          object AND total number of objects, separated by
!                          commas;
!                            default: none
!
! .OUTPUT     "<P3>": columns SKY(*),LAM_CEN,STATUS,FWHM,S2N(*) computed
!                     ("*": empty, for later)
!
!             Q2:  0: object need not to be skipped
!                  1: object must be skipped
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for spectroscopy - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/02/23  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref subima bintbl objtbl objval
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "chkobs_fwhm"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      subima (P1): sky-subtracted standard (or science) frame;"
  w/o "                     default: none"
  w/o " "
  w/o "      bintbl (P2): table with bin coordinates;"
  w/o "                     default: none"
  w/o " "
  w/o "      objtbl (P3): two-dimensional object table;"
  w/o "                     default: none"
  w/o " "
  w/o "      objval (P4): current row (object) number of three-dimensional"
  w/o "                   spectroscopy table AND lower y-pixel-coordinate of"
  w/o "                   object AND total number of objects, separated by"
  w/o "                   commas;"
  w/o "                     default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(subima):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of sky subtracted frame: "
define/local subima/c/1/64 "{p1}"
if m$index(subima,".bdf") .eq. 0 subima = subima//".bdf"
!
if m$exist(subima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({subima})"
  goto error
endif
!
!
!P2(bintbl):
!----------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter name of table with bin positions: "
define/local bintbl/c/1/64 "{p2}"
if m$index(bintbl,".tbl") .eq. 0 bintbl = bintbl//".tbl"
!
if m$existc(bintbl,"XSTART") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({bintbl})"
  goto error
elseif m$existc(bintbl,"XSTART") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({bintbl},:XSTART)"
  goto error
elseif m$existc(bintbl,"XEND") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({bintbl},:XEND)"
  goto error
elseif m$existc(bintbl,"YSTART") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({bintbl},:YSTART)"
  goto error
elseif m$existc(bintbl,"YEND") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({bintbl},:YEND)"
  goto error
endif
!
show/table {bintbl} >Null
define/local bins/i/1/1 {outputi(2)}
if bins .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({bintbl})"
  goto error
endif
!
!
!P3(objtbl):
!----------
if p3 .eq. "?" mess/out 2 {PROID} " "
define/parameter p3 ? t -
  "        Enter 2D table with object values: "
define/local dim2tbl/c/1/64 "{p3}"
if m$index(dim2tbl,".tbl") .eq. 0 dim2tbl = dim2tbl//".tbl"
!
if m$exist(dim2tbl) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({dim2tbl})"
  goto error
endif
!
!
!P4(objval):
!----------
if p4 .eq. "?" mess/out 2 {PROID} " "
define/parameter p4 ? ? -
  "        Enter curr.obj.no.,obj.-ylowpix,tot.obj.no.: "
outputi = m$parse(p4,"objval")
if outputi(1) .ne. 3 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P4 !"
  goto error
endif
define/local rowloop/i/1/1 {objval01}
define/local ystartpix/i/1/1 {objval02}
define/local totobjno/i/1/1 {objval03}
!
if rowloop .lt. 1 .or. rowloop .gt. totobjno then
        write/out " "
        mess/out 1 {PROID} "Row number out of range !"
        mess/out 1 {PROID} "({rowloop} ; 1<=row<={totobjno})"
  goto error
endif
!
outputi(1) = {{subima},NPIX(2)}
if ystartpix .lt. 1 .or. ystartpix .gt. outputi(1) then
        write/out " "
        mess/out 1 {PROID} "Lower object end out of range !"
        mess/out 1 {PROID} "({ystartpix} ; 1<=ypix<={outputi(1)})"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Drawing":
  define/local goodbins/i/1/1 0
  define/local labxl/i/1/1 0
  define/local labxm/i/1/3 0 all
  define/local labxr/i/1/1 0
  define/local scry0/i/1/1 0
  define/local frmy/i/1/1 0
  define/local frmy0/i/1/1 0
  define/local dscly/i/1/1 0
  define/local scaly/r/1/1 0.
  define/local scry/i/1/1 0
  define/local objslit/i/1/1 0
  define/local labobjxl/i/1/1 0
  define/local labobjxr/i/1/1 0
  define/local moreskip/i/1/1 0
!
!     "Spreading":
  define/local binloop/i/1/1 0
  define/local waveloop/i/1/1 0
  define/local waverow/i/1/2 0 all
!
!
!
!          APPLYING GAUSS FIT
!          PERPENDICULAR TO DISPERSION DIRECTION
!          TO EACH BIN OF THE OBJECT :
!          *************************************
!
create/column {bintbl} :XCEN   "angstrom" F9.2  R*4
create/column {bintbl} :YCEN   "arcsec"   F7.2  R*4
create/column {bintbl} :STATUS " "        I6    I*4
create/column {bintbl} :XERR   "angstrom" F9.2  R*4
create/column {bintbl} :YERR   "arcsec"   F7.2  R*4
create/column {bintbl} :XSIG   "angstrom" F9.2  R*4
create/column {bintbl} :YSIG   "arcsec"   F7.2  R*4
create/column {bintbl} :XFWHM  "angstrom" F9.2  R*4
create/column {bintbl} :YFWHM  "arcsec"   F7.2  R*4
create/column {bintbl} :ICENT  "ADU"      G13.6 R*4
create/column {bintbl} :IDENT  " "        A8    C*8
!
        mess/out 4 {PROID} "    fitting each bin perpend. to dispers. direc."
!
center/gauss {subima},{bintbl} {bintbl} >Null
!
delete/column {bintbl} :YCEN :XERR :YERR :XSIG :YSIG :XFWHM :ICENT
delete/column {bintbl} :IDENT
name/column {bintbl} :XCEN :LAM_CEN
name/column {bintbl} :YFWHM :FWHM
!
!
!          DRAWING BIN QUALITY (RESULTED FROM FIT) IN DISPLAY :
!          **************************************************
!
select/table {bintbl} :STATUS.eq.0 >Null
goodbins = outputi(1)
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "    displaying bin quality"
  !
  if rowloop .eq. 1 then
    assign/display d,3 >Null
    labxl  = 6
    label/display "OBJ" 10,{labxl} ? green 1
    outputi = labxl+3
    label/display "^" 23,{outputi(1)} ? green 1
    if nice_ddim(1) .ge. 285 then
      labxm(1) = nice_dscr(1)-88
      labxm(2) = labxm(1)+54
      labxm(3) = labxm(2)+72
      !
      label/display "good" 10,{labxm(1)} ? cyan 1
      label/display "<BINS>" 10,{labxm(2)} ? green 1
      label/display "failed" 10,{labxm(3)} ? red 1
    else
      labxm(1) = nice_dscr(1)-28
      labxm(3) = labxm(1)+33
      !
      label/display "gd" 10,{labxm(1)} ? cyan 1
      label/display "bd" 10,{labxm(3)} ? red 1
    endif
    labxr  = nice_dscr(1)*2-46
    label/display "SLIT" 10,{labxr} ? green 1
    outputi = labxr+29
    label/display "^" 23,{outputi(1)} ? green 1
  endif
  !
  ! ----------------------------------------------------------------
  ! scry - scry0 = nint[ (frmy - frmy0) /scaly ] ,
  !   with  scry0 = nice_dscr(2)     (screen pixel in window centre)
  !   and   frmy0 = DISPLAY_DATA(4)  (frame  pixel in window centre
  !                                   after `load/ima ... c,c')
  !   and   scaly =     DISPLAY_DATA(2), if positive
  !                 -1./DISPLAY_DATA(2), if negative
  ! ----------------------------------------------------------------
  !
  scry0 = nice_dscr(2)
  frmy  = ystartpix
  frmy0 = {{subima},DISPLAY_DATA(4)}
  dscly = {{subima},DISPLAY_DATA(2)}
  if dscly .ge. 1 then  ! 1,2,3,...
    scaly = dscly
  else                  ! ...,-3,-2 ; 0 not possible, -1 doesn't exist
    scaly = -1./dscly
  endif
  scry = m$nint((frmy-frmy0)/scaly-0.5)+scry0
  labobjxl = 6
  if rowloop .lt. 10 then
    label/display "  {rowloop}" {scry},{labobjxl} ? green 1
  else
    label/display "{rowloop}" {scry},{labobjxl} ? green 1
  endif
  !
  objslit = {{dim2tbl},:SLIT,1}
  labobjxr = nice_dscr(1)*2-26
  if objslit .lt. 10 then
    label/display "  {objslit}" {scry},{labobjxr} ? green 1
  else
    label/display "{objslit}" {scry},{labobjxr} ? green 1
  endif
  !
  draw/rectangle {bintbl} f n cyan,0 >Null  ! (display 3 is assigned)
  select/table {bintbl} :STATUS.ne.0 >Null
  draw/rectangle {bintbl} f n red,0 >Null
  !
endif
!
select/table {bintbl} all
if goodbins .eq. 0 then
        mess/out 2 {PROID} "    No good bin found - skipping object !"
  skip = skip+1  ! skip is "+lower_levels" (f._chkobs)
  moreskip = 1
  warn = 1
  goto skipped
endif
!
!
!          SPREADING BINNED FWHM VALUES
!          TO NON-BINNED OBJECT TABLE :
!          ****************************
!
        mess/out 4 {PROID} "    adding FWHM values to object table"
!
create/column {dim2tbl} :SKY     "ADU"      G12.6 R*4  ! for S/N
create/column {dim2tbl} :LAM_CEN "angstrom" F9.2  R*4
create/column {dim2tbl} :STATUS  " "        I6    I*4
create/column {dim2tbl} :FWHM    "arcsec"   F7.2  R*4
create/column {dim2tbl} :S2N     " "        F10.3 R*4  ! for S/N
!
do binloop = 1 bins
  !
  waverow(1) = (binloop-1)*binsize+1  ! binsize is "+lower_levels" (f._chkobs)
  waverow(2) = binloop*binsize
  do waveloop = waverow(1) waverow(2)
    {dim2tbl},:LAM_CEN,{waveloop} = {{bintbl},:LAM_CEN,{binloop}}
    {dim2tbl},:FWHM,{waveloop}    = {{bintbl},:FWHM,{binloop}}
    {dim2tbl},:STATUS,{waveloop}  = {{bintbl},:STATUS,{binloop}}
  enddo
  !
enddo
!
!
!          PLOTTING FWHMS OF WELL-FITTED BINS :
!          **********************************
!
! output table instead of bin table => negative fluxes can be omitted
!                                   => bin width can be shown
!
select/table {dim2tbl} :STATUS.eq.0.and.:FLUX.gt.0. >Null
!
if outputi(1) .eq. 0 then
        mess/out 2 {PROID} "    No proper wavelength found - skipping object !"
  skip = skip+1
  moreskip = 1
  warn = 1
  goto skipped
endif
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "    plotting FWHMs of well-fitted bins"
  !
  assign/graphic g,1 >Null
  set/graphic stype=1 ltype=0 colour=3
  !
  plot/table {dim2tbl} :LAMBDA :FWHM >Null
  !
  !***************
  @p fors_nicegrap
  !***************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of plot failed !"
    goto error
  endif
  label/graphic "OBJ {rowloop} / {totobjno}" {nice_glabw}
  !
endif
!
select/table {dim2tbl} all
!
!
skipped:
!
!
set/format
!
!
if warn .eq. 0 then
  return 0 {moreskip}
else
  return -1 {moreskip}
endif
!
error:
  set/format
  return 1 -99
!
!
!-------------------------------------------------------------------------------
