! $Id: fors_chkobs_in3d.prg,v 1.7 2000/02/23 14:49:30 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_chkobs_in3d.prg,v $
!
! .KEYWORDS   Spectroscopical quality
!
! .PURPOSE    Insert two-dimensional table with values of one object into
!             three-dimensional output table with result values of all objects.
! __________
!
! .USAGE      @p fors_chkobs_in3d dim2tbl totobj outtbl row
!
! .INPUT      dim2tbl (P1): two-dimensional result object table;
!                             default: none
!
!             totobj  (P2): total number of objects;
!                             default: none
!
!             outtbl  (P3): name of three-dimensional output table (will be
!                           created);
!                             default: none
!
!             row     (P4): row (object) number to be used in P3;
!                             default: none
!
! .OUTPUT     "<P3>": three-dimensional result table (one object inserted per
!                     invocation)
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for spectroscopy - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/02/22  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref dim2tbl totobj outtbl row
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "chkobs_in3d"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      dim2tbl (P1): two-dimensional result object table;"
  w/o "                      default: none"
  w/o " "
  w/o "      totobj  (P2): total number of objects;"
  w/o "                      default: none"
  w/o " "
  w/o "      outtbl  (P3): name of three-dimensional output table (will be"
  w/o "                    created);"
  w/o "                      default: none"
  w/o " "
  w/o "      row     (P4): row (object) number to be used in P3;"
  w/o "                      default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(dim2tbl):
!-----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? t -
  "        Enter 2D input table: "
define/local qc2dtbl/c/1/64 "{p1}"
if m$index(qc2dtbl,".tbl") .eq. 0 qc2dtbl = qc2dtbl//".tbl"
!
if m$existc(qc2dtbl,":SLIT") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({qc2dtbl})"
  goto error
elseif m$existc(qc2dtbl,":SLIT") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:SLIT)"
  goto error
elseif m$existc(qc2dtbl,"RA") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:RA)"
  goto error
elseif m$existc(qc2dtbl,"DEC") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:DEC)"
  goto error
elseif m$existc(qc2dtbl,"LAMBDA") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:LAMBDA)"
  goto error
elseif m$existc(qc2dtbl,"YSTART") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:YSTART)"
  goto error
elseif m$existc(qc2dtbl,"YEND") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:YEND)"
  goto error
elseif m$existc(qc2dtbl,"FLUX") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:FLUX)"
  goto error
elseif m$existc(qc2dtbl,":SKY") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:SKY)"
  goto error
elseif m$existc(qc2dtbl,"LAM_CEN") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:LAM_CEN)"
  goto error
elseif m$existc(qc2dtbl,":STATUS") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:STATUS)"
  goto error
elseif m$existc(qc2dtbl,"FWHM") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:FWHM)"
  goto error
elseif m$existc(qc2dtbl,":S2N") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc2dtbl},:S2N)"
  goto error
endif
!
show/table {qc2dtbl} >Null
define/local dim2rows/i/1/1 {outputi(2)}
if dim2rows .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({qc2dtbl})"
  goto error
endif
!
!
!P2(totobj):
!----------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? n -
  "        Enter total number of objects: "
define/local totobjno/i/1/1 {p2}
!
!
!P3(outtbl):
!----------
if p3 .eq. "?" mess/out 2 {PROID} " "
define/parameter p3 ? ? -
  "        Enter name of 3D output table: "
define/local qc3dtbl/c/1/64 "{p3}"
if m$index(qc3dtbl,".tbl") .eq. 0 qc3dtbl = qc3dtbl//".tbl"
!
!
!P4(row):
!-------
if p4 .eq. "?" mess/out 2 {PROID} " "
define/parameter p4 ? n -
  "        Enter row number for output table: "
define/local rowloop/i/1/1 {p4}
!
if rowloop .lt. 1 .or. rowloop .gt. totobjno then
        write/out " "
        mess/out 1 {PROID} "Row number out of range !"
        mess/out 1 {PROID} "({rowloop} ; 1<=row<={totobjno})"
  goto error
endif
!
!
!Further tests:
!-------------
if rowloop .gt. 1 then
  !
  if m$existc(qc3dtbl,":SLIT") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Output table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({qc3dtbl})"
    goto error
  elseif m$existc(qc3dtbl,":SLIT") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:SLIT)"
    goto error
  elseif m$existc(qc3dtbl,"RA") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:RA)"
    goto error
  elseif m$existc(qc3dtbl,"DEC") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:DEC)"
    goto error
  elseif m$existc(qc3dtbl,"LAMBDA") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:LAMBDA)"
    goto error
  elseif m$existc(qc3dtbl,"YSTART") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:YSTART)"
    goto error
  elseif m$existc(qc3dtbl,"YEND") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:YEND)"
    goto error
  elseif m$existc(qc3dtbl,"FLUX") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:FLUX)"
    goto error
  elseif m$existc(qc3dtbl,":SKY") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:SKY)"
    goto error
  elseif m$existc(qc3dtbl,"LAM_CEN") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:LAM_CEN)"
    goto error
  elseif m$existc(qc3dtbl,":STATUS") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:STATUS)"
    goto error
  elseif m$existc(qc3dtbl,"FWHM") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:FWHM)"
    goto error
  elseif m$existc(qc3dtbl,":S2N") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({qc3dtbl},:S2N)"
    goto error
  endif
  !
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
! none
!
!                                                _  array (.../fwhm/s2n-vals.;
!                                                /| depth = x-ax. pix.)
!          ADDING RESULTS                       /
!          TO 3D OUTPUT TABLE :                +-------> cols.(.../fwhm/s2n)
!          ******************                  |
!                                              V rows (objects)
!
if rowloop .eq. 1 then
  create/table {qc3dtbl} 12 {totobjno} null
  create/column {qc3dtbl} :SLIT    " "        I4    I*4({dim2rows})
  create/column {qc3dtbl} :RA      "deg"      F11.5 R*8({dim2rows})
  create/column {qc3dtbl} :DEC     "deg"      F10.5 R*8({dim2rows})
  create/column {qc3dtbl} :LAMBDA  "angstrom" F9.2  R*4({dim2rows})
  create/column {qc3dtbl} :YSTART  "arcsec"   F7.2  R*4({dim2rows})
  create/column {qc3dtbl} :YEND    "arcsec"   F7.2  R*4({dim2rows})
  create/column {qc3dtbl} :FLUX    "ADU"      G13.6 R*4({dim2rows})
  create/column {qc3dtbl} :SKY     "ADU"      G12.6 R*4({dim2rows})
  create/column {qc3dtbl} :LAM_CEN "angstrom" F9.2  R*4({dim2rows})
  create/column {qc3dtbl} :STATUS  " "        I6    I*4({dim2rows})
  create/column {qc3dtbl} :FWHM    "arcsec"   F7.2  R*4({dim2rows})
  create/column {qc3dtbl} :S2N     " "        F10.3 R*4({dim2rows})
endif
!
        mess/out 4 {PROID} "    filling object results into 3D table"
!
!******************************************************************
@p fors_t3dinsert {qc2dtbl} :SLIT    + {qc3dtbl} :SLIT    {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :RA      + {qc3dtbl} :RA      {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :DEC     + {qc3dtbl} :DEC     {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :LAMBDA  + {qc3dtbl} :LAMBDA  {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :YSTART  + {qc3dtbl} :YSTART  {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :YEND    + {qc3dtbl} :YEND    {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :FLUX    + {qc3dtbl} :FLUX    {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :SKY     + {qc3dtbl} :SKY     {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :LAM_CEN + {qc3dtbl} :LAM_CEN {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :STATUS  + {qc3dtbl} :STATUS  {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :FWHM    + {qc3dtbl} :FWHM    {rowloop}
  if {q1} .eq. 1 goto insfail
@p fors_t3dinsert {qc2dtbl} :S2N     + {qc3dtbl} :S2N     {rowloop}
  if {q1} .eq. 1 goto insfail
!******************************************************************
goto end_insfail
insfail:
        write/out " "
        mess/out 1 {PROID} "Insertion of column failed !"
        mess/out 1 {PROID} "({qc2dtbl} -> {qc3dtbl})"
  goto error
end_insfail:
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------
