! $Id: fors_chkobs_s2n.prg,v 1.8 2000/02/23 14:49:30 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_chkobs_s2n.prg,v $
!
! .KEYWORDS   Spectroscopical quality
!
! .PURPOSE    Compute signal-to-noise values of current object (per wavelength).
! __________
!
! .USAGE      @p fors_chkobs_s2n skyima objtbl yplohi roncon objval
!
! .INPUT      skyima (P1): input sky background frame;
!                            default: none
!
!             objtbl (P2): two-dimensional object table;
!                            default: none
!
!             yplohi (P3): lower y-pixel-coordinate of object AND higher
!                          y-pixel-coordinate of object, separated by a comma;
!                            default: none
!
!             roncon (P4): values of readout noise AND electrons to ADU
!                          conversion factor, separated by a comma;
!                            default: none
!
!             objval (P5): current row (object) number of three-dimensional
!                          spectroscopy table AND total number of objects,
!                          separated by a comma;
!                            default: none
!
! .OUTPUT     "<P3>": columns SKY and S2N computed
!
!               The signal-to-noise values are calculated as follows:
!
!                 All fluxes and readout noise in e- , w. original expo. time ,
!                 the object and sky fluxes are integrated (per wavelength,
!                 perpendicular to dispersion direction):
!
!                                     flux_obj.
!                 ----------------------------------------------------
!                 sqrt( flux_obj. + flux_sky + #ypix./wave.*(RON**2) )
!
!             Q2:  0: object need not to be skipped
!                  1: object must be skipped
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for spectroscopy - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/02/10  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref skyima objtbl yplohi roncon objval
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "chkobs_s2n"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      skyima (P1): input sky background frame;"
  w/o "                     default: none"
  w/o " "
  w/o "      objtbl (P2): two-dimensional object table;"
  w/o "                     default: none"
  w/o " "
  w/o "      yplohi (P3): lower y-pixel-coordinate of object AND higher"
  w/o "                   y-pixel-coordinate of object, separated by a comma;"
  w/o "                     default: none"
  w/o " "
  w/o "      roncon (P4): values of readout noise AND electrons to ADU"
  w/o "                   conversion factor, separated by a comma;"
  w/o "                     default: none"
  w/o " "
  w/o "      objval (P5): current row (object) number of three-dimensional"
  w/o "                   spectroscopy table AND total number of objects,"
  w/o "                   separated by a comma;"
  w/o "                     default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(skyima):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of background frame: "
define/local backima/c/1/64 "{p1}"
if m$index(backima,".bdf") .eq. 0 backima = backima//".bdf"
!
if m$exist(backima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({backima})"
  goto error
endif
!
!
!P2(objtbl):
!----------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? t -
  "        Enter 2D table with object values: "
define/local dim2tbl/c/1/64 "{p2}"
if m$index(dim2tbl,".tbl") .eq. 0 dim2tbl = dim2tbl//".tbl"
!
if m$existc(dim2tbl,"FLUX") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({dim2tbl})"
  goto error
elseif m$existc(dim2tbl,"FLUX") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({dim2tbl},:FLUX)"
  goto error
elseif m$existc(dim2tbl,":SKY") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({dim2tbl},:SKY)"
  goto error
elseif m$existc(dim2tbl,":S2N") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({dim2tbl},:S2N)"
  goto error
endif
!
show/table {dim2tbl} >Null
if outputi(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({dim2tbl})"
  goto error
endif
!
!
!P3(yplohi):
!----------
if p3 .eq. "?" mess/out 2 {PROID} " "
define/parameter p3 ? ? -
  "        Enter y-range of object (ypixlow,ypixhigh): "
outputi = m$parse(p3,"ypxrng")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P3 !"
  goto error
endif
define/local ypixlow/r/1/1 {ypxrng01}
define/local ypixhigh/r/1/1 {ypxrng02}
!
!
!P4(roncon):
!----------
if p4 .eq. "?" mess/out 2 {PROID} " "
define/parameter p4 ? ? -
  "        Enter readout_noise,conversion_factor values: "
outputi = m$parse(p4,"roncf")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P4 !"
  goto error
endif
define/local eron/r/1/1 {roncf01}
define/local cfadu/r/1/1 {roncf02}
!
if eron .le. 0. then
        write/out " "
        mess/out 1 {PROID} "Impossible readout noise value ({eron}) !"
  goto error
endif
if cfadu .le. 0. then
        write/out " "
        mess/out 1 {PROID} "Impossible conversion factor value ({cfadu}) !"
  goto error
endif
!
!
!P5(objval):
!----------
if p5 .eq. "?" mess/out 2 {PROID} " "
define/parameter p5 ? ? -
  "        Enter curr.obj.no.,tot.obj.no.: "
outputi = m$parse(p5,"objval")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P5 !"
  goto error
endif
define/local rowloop/i/1/1 {objval01}
define/local totobjno/i/1/1 {objval02}
!
if rowloop .lt. 1 .or. rowloop .gt. totobjno then
        write/out " "
        mess/out 1 {PROID} "Row number out of range !"
        mess/out 1 {PROID} "({rowloop} ; 1<=row<={totobjno})"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local char/i/1/1 0
  define/local ypix/i/1/1 0
  define/local ronsqr/r/1/1 0.
  define/local moreskip/i/1/1 0
!
!
!
!          COMPUTING SIGNAL-TO-NOISE VALUES :
!          ********************************
!
        mess/out 4 {PROID} "    integrating sky per wavelength"
!
! SKY:
average/row skyint.bdf = {backima} @{ypixlow},@{ypixhigh} sum
copy/it skyint.bdf skyint.tbl
copy/tt skyint.tbl :LAB001 {dim2tbl} :SKY
-delete skyint.bdf
-delete skyint.tbl
!
! RON:
ypix = m$abs(ypixhigh-ypixlow)
ronsqr = (eron*cfadu)**2
!
        mess/out 4 {PROID} "    calculating S/N values"
!
! S/N:
compute/table {dim2tbl} :S2N = :FLUX*{cfadu} +:SKY*{cfadu} +{ypix}*{ronsqr}
compute/table {dim2tbl} :S2N = :FLUX*{cfadu} / sqrt(:S2N)
!
!
!          PLOTTING SPECTRAL S/N DISTRIBUTION :
!          **********************************
!
select/table {dim2tbl} :FLUX.gt.0..and.:SKY.gt.0. >Null
!
if outputi(1) .eq. 0 then
        mess/out 2 {PROID} "    No proper wavelength found - skipping object !"
  skip = skip+1  ! skip is "+lower_levels" (f._chkobs)
  moreskip = 1
  warn = 1
  goto skipped
endif
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "    plotting spectral S/N distribution"
  !
  assign/graphic g,2 >Null
  plot/table {dim2tbl} :LAMBDA :S2N >Null
  !
  !***************
  @p fors_nicegrap
  !***************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of plot failed !"
    goto error
  endif
  label/graphic "OBJ {rowloop} / {totobjno}" {nice_glabw}
  !
endif
!
select/table {dim2tbl} all
!
!
skipped:
!
!
set/format
!
!
if warn .eq. 0 then
  return 0 {moreskip}
else
  return -1 {moreskip}
endif
!
error:
  set/format
  return 1 -99
!
!
!-------------------------------------------------------------------------------
