! $Id: fors_comb_setup.prg,v 1.15 1999/11/25 11:23:51 sbogun Exp $   
!+++ FORS_COMB_SETUP +++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_comb_setup.prg,v $
!
!.KEYWORDS      Initialize combine/ccd setup table depending on exposure type
!
!.USAGE         @p fors_comb_setup sof_in out sof_ref [optpar]
!
!.INPUT         expotyp (P1): exposure type of to be combined frames
!               info    (P6): switch: Y yes show info; N: no info.
!
!.OUTPUT        updated combine/ccd setup
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       The MIDAS combine/ccd command uses a setup to figure out how
!               to combine a series of input frames (see MIDAS manual). The
!               appropriate setup will be selected depending on the exposure
!               type passed to this routine.
!
!.ALGORITHM     Check the exposure type and branch to the appropriate setup.
!               The following setups are implemented:
!               BIAS:   do a median averaging.
!               DARK:   do a median averaging, scale each frame by its exposure
!                       time before averaging.
!               FLAT:   do a median averaging, scale each frame by its median
!                       devided by the sum of all medians before averaging.
!
!               (SKYFLATS are handled in the same way as SCREENFLATS.)
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1998/02/18  Creation.
!               1.0 1998/12/02  update comments.
!                   1999/02/01  preparations for preselection test implemented.
!                   1999/02/11  show scaling factors if MESS_LEVEL==4
!                   1999/03/12  default P2=+ -> skip preselection prep.
!
!.COMMENT
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter exposure type (BS, DK, FF): "
define/param p2 + c "Enter name of image catalog:"
define/param p3 0 n "Enter number of input frames:"
define/param p4 + c "Enter name of array of means of reduced frames:"
define/param p5 + c "Enter name of array of medians of reduced frames:"
define/param p6 N c "Enter Yes or No for information on terminal: "
!
! local variable declaration
!
define/local i1/i/1/1 0
define/local err1/c/1/80 " " ALL
err1 = "Unknown exposure type: {P1}"

CCD_IN = "?"
{P1}_SIG = "no"                 !no SIGMA frame will be created
{P1}_DEL = "no"                 !do not DELETE anny frames after combining
{P1}_OFF = "no"                 !add no OFFSET to the data
{P1}_EXP = "no"                 !do not scale by EXPOSURE time (reset below)
{P1}_RAN(1) = -9.99990E+04      !data RANGE to be used
{P1}_RAN(2) = 9.99990E+04
{P1}_CLP(1) = 3.00000E+00       !clip window in order to be able to exclude
{P1}_CLP(2) = 3.00000E+00       !data (depends on the METHOD keyword)
{P1}_NUL = "0.00000E+00"        !value for rejections

{P1}_MET = "median"             !combining METHOD: median
{P1}_SEC = IM_SEC               !image SECTION to be used
{P1}_WEI = "no"                 !do a WEIGHTED average

BRANCH P1(1:1) B,D,F LBS,LDK,LFF
mess/out 1 comb_setup {err1}
goto fini

LFF:
  {P1}_SCA = "yes"
  {P1}_STA = "median"
  @p fors_comb_setup,SCALE {p2} {p3} {p5} 3
  if P6(1:1) .eq. "Y" show/ccd {P1}
  goto fini
LDK:
  {P1}_SCA = "no"
  {P1}_EXP = "yes"
  @p fors_comb_setup,SCALE {p2} {p3} {p5} 2
  if P6(1:1) .eq. "Y" show/ccd {P1}
  goto fini
LBS:
!  {P1}_SCA = "yes"             !old
  {P1}_SCA = "no "
  {P1}_STA = "median"
  @p fors_comb_setup,SCALE {p2} {p3} {p5} 1
  if P6(1:1) .eq. "Y" show/ccd {P1}

fini:

if P2(1:1) .eq. "+" return 1

!determine: n_j = meanred_j / scale_j   ;scale_j scale factor for comb/ccd
!                                       ;defined in subroutine SCALE.
do i1 = 1 {p3}
  mess/out 4 comb_setup "Mean(red.frame_{i1}): {{p4}({i1})}\nScale factor(frame_{i1}): {{p5}({i1})}"
  {p5}({i1}) = {p4}({i1}) / {p5}({i1})
enddo
return 1


!=== SUBROUTINE ==========================================================

ENTRY SCALE
define/param p1 + c "Enter name of image catalog:"
define/param p2 + c "Enter number of elements to use:"
define/param p3 + c "Enter name of array:"
define/param p4 + c "Enter mode:"

define/local i1/i/1/1 0
define/local i2/i/1/1 0

if P1(1:1) .eq. "+" return 0

i2 = {p2} + 1
if {p4} .eq. 3 then begin
  do i1 = 1 {p2}
    {p3}({i1}) = {p3}({i1}) / {p3}({i2})        !array will now be replaced
  enddo
else if {p4} .eq. 2 then begin
  do i1 = 1 {p2}
    {p3}({i1}) = {#{i1}_{p1},O_TIME(7)}         !array will now be replaced
  enddo
else if {p4} .eq. 1 then begin
  do i1 = 1 {p2}
    {p3}({i1}) = 1.0                            !array will now be replaced
  enddo
endif
