! $Id: fors_corr_badpix.prg,v 1.4 1998/08/28 11:44:44 forsp Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! Synopsis:
!   @p fors_corr_badpix  infrm outfrm pixtab method
!
!  Inputs:
!   infrm   (P1)	Input frame to be cleaned
!
!   pixtab  (P3)	Table with locations of bad pixels on the CCD
!
!   method  (P4)        Selector for method used to remove bad pixels
!
!  Outputs:
!   outfrm  (P2)	cleaned frame
!
!  Returns:
!   none
!
! Description:
!   The recipe is similar to that of the FIXPIX/CCD command of the CCDRED
!   package.
!
! Keywords:
!   none
!
! Pipeline version:
!   v0.5
!
! Author:
!   R. Palsa (@rp)
!
!-----------------------------------------------------------------------------~

define/param p1 ?            ima  "Enter input frame:"
define/param p2 ?            ima  "Enter cleaned frame:"
define/param p3 {BP_TABLE}   tbl  "Enter bad pixel table:"
define/param p4 {BP_METHOD}  c    "Select method of bad pixel correction:"

! Procedure identification
define/local procID/c/1/80      "CorrBadPix"


! Check if correction was already been applied
if m$existd(p1,"BP_STATUS") .eq. 0 then
  mess/out 2 {procID} "Frame status descriptor not present!"
  write/descr {p1} BP_STATUS/c/1/5 "DIRTY"
else
  if m$index({{p1},BP_STATUS},"CLEAN") .gt. 0 then
    mess/out 2 {procID} "Bad pixel correction already applied to {p1}!"
    return 0
  endif
endif

! Check for the presence of bad pixel lookup table
if m$exist(BP_TABLE) .eq. 0 then
  mess/out 3 {procID} "Cannot access bad pixel table!"
  return 1
endif

! Modify bad pixel entries according to selected method
if p4(1:1) .eq. "A" then
  if m$existc(BP_TABLE,":xstart") .eq. -1 then
    mess/out 3 {procID} "Invalid bad pixel table for method {p4}!"
    return 1
  else
    modify/area {p1},{p3} {p2} {BP_AUX(1)} {BP_NOISE}
  endif
elseif p4(1:1) .eq. "P" then
  if m$existc(BP_TABLE,":xstart") .eq. -1 then
    mess/out 3 {procID} "Invalid bad pixel table for method {p4}!"
    return 1
  else
    modify/pixel {p1},{p3} {p2} {BP_AUX(1)} {BP_AUX(2:4)} ? {BP_NOISE}
  endif
elseif p4(1:1) .eq. "C" then
  if m$existc(BP_TABLE,":x") .eq. -1 then
    mess/out 3 {procID} "Invalid bad pixel table for method {p4}!"
    return 1
  else
    modify/column {p1},{p3} {p2} V
  endif
elseif p4(1:1) .eq. "R" then
  if m$existc(BP_TABLE,":y") .eq. -1 then
    mess/out 3 {procID}: "Invalid bad pixel table for method {p4}!"
    return 1
  else
    modify/row {p1},{p3} {p2} V
  endif
else
 mess/out 3 {procID} "Invalid correction method selected!"
 return 1
endif

! Perform some statistics
if verbose(1:1) .gt. 0 then
  statistic/image {p2} {frm_section} ? ? ff
else
  statistic/image {p2} {frm_section} ? ? fx
endif

! Update frame descriptors
{p2},BP_STATUS = "clean"

return 0
