! $Id: fors_corr_bias.prg,v 1.10 1999/11/30 10:39:56 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_corr_bias.prg,v $
!
!.KEYWORDS    Basic CCD reduction, bias correction
!
!.USAGE       @p fors_corr_bias  raw_frm bs_frm red_frm trim
!
!.INPUT       raw_frm	  (P1)	  Input frame to be corrected.
!             bs_frm	  (P2)	  Bias frame or value to be corrected for.
!             trim	      (P4)	  Frame coordinates for extracting useful
!                                 image parts.
!             bscale      (P5)    Bias scaling method ("MED", "OVE" or "NO").
!.OUTPUT      red_frm	  (P3)	  Corrected frame.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The recipe does a straight forward bias correction, i.e. the
!             bias (frame or value) is subtracted from the science frame.
!             If an output frame name is given the result frame is stored
!             using this name, otherwise the input frame is replaced by the
!             corrected version. Optionally a window can be specified for the
!             bias frame so that it matches the size of the input frame.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!
!------------------------------------------------------------------------------

crossref  input bias output window bscale

define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter name of bias frame/value:"
define/param p3 + c "Enter name of reduced frame:"
define/param p4 + c "Enter window coordinates:"
define/param p5 + c "Enable bias scaling (MED/OVE/NO):"
define/maxpar 5


! Procedure identification
define/local procID/c/1/80	"CorrBias"


! Local constants
define/local tmp0/c/1/12     "middumma.bdf"
define/local tmp1/c/1/12     "middummb.bdf"
define/local tmp2/c/1/12     "middummc.bdf"
define/local tmp3/c/1/12     "middummd.bdf"


! Local variables
define/local bscale/c/1/3    " "
define/local raw_frm/c/1/80  {p1}
define/local bs_frm/c/1/80   {p2}
define/local red_frm/c/1/80  {p3}
define/local nport/i/1/1     0


bscale = m$upper(p5(1:3))

if red_frm(1:1) .eq. "+" then
  red_frm = raw_frm
  -rename {raw_frm} {tmp0}
  raw_frm = tmp0
endif


! Get number of readout ports
nport = m$value({raw_frm},{h_nout})
if nport .ne. 1 .and. nport .ne. 4 then
  mess/out 1 redinfo "Invalid readout port setup!"
  return 1
endif


! Do the correction
if m$tstno(bs_frm) .eq. 0 then
  if bscale .ne. "NO" then
    normalize/mbias {bs_frm} {tmp2} {bscale}
    if {Q1} .ne. 0 return 1
    scale/mbias {raw_frm} {tmp2} {tmp3}
    if {Q1} .ne. 0 return 1
    bs_frm = tmp3
  endif

  if p4(1:1) .ne. "+" then
    extract/image {tmp1} = {bs_frm} {p4}
    bs_frm = tmp1
  endif
endif

compute/image {red_frm} = {raw_frm} - {bs_frm}


! Create log messages
if m$tstno(bs_frm) .ne. 1 then
  longlog/out 1 "     Master Bias frame subtracted: {bs_frm}"
  if bscale .eq. "MED" then
    if nport .eq. 4 then
      longlog/out 1 "       (each quadrant divided by its median and multiplied by the"
      longlog/out 1 "       corresponding pre- or overscan value of the input frame)"
    else
      longlog/out 1 "       (divided by its median and multiplied by the average of pre-"
      longlog/out 1 "       and overscan values of the input frame)"
    endif
  else if bscale .eq. "OVE" then
    if nport .eq. 4 then
      longlog/out 1 "       (each quadrant divided by its corresponding pre- or overscan"
      longlog/out 1 "       value and multiplied by the corresponding pre- or overscan value"
      longlog/out 1 "       of the input frame)"
    else
      longlog/out 1 "       (divided by the average of its pre- and overscan values and multiplied"
      longlog/out 1 "       by the average of pre- and overscan values of the input frame)"
    endif
  endif
else
  longlog/out 1 "     Constant BIAS value subtracted: {bs_frm}"
endif
shortlog/out 1 "Bias correction done."


! Cleanup
-delete {tmp0}
-delete {tmp1}

if bscale .ne. "NO" then
  -delete {tmp2}
  -delete {tmp3}
endif


return 0
