! $Id: fors_corr_bpxl.prg,v 1.36 1999/11/25 11:23:52 sbogun Exp $   
!+++ FORS_CORR_BPXL ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_corr_bpxl.prg,v $
!
!.KEYWORDS      Correct images for bad pixels
!
!.USAGE         @p fors_corr_bpxl badpix in_frm noise csyst
!
!.INPUT         badpix  (P1): bad pixel table
!           in_cat  (P2): input image catalogue
!           noise   (P3): add noise Yes/No
!           csyst   (P4): coordinate system: WORLD or PIXEL
!
!.OUTPUT        bad pixel corrected frame, INPUT frame will be changed!
!
!.RETURN
!
!.DEFAULT
!
!.PURPOSE       Do the bad pixel correction using MIDAS command MODIFY/PIXEL
!
!.ALGORITHM     Loop through all input frames of the CCDRED context (CCDIN,
!           SC_BSFRM, SC_DKFRM, SC_FFFRM).
!           First check if the input frame is just one number (bias, or
!           dark). In that case no bad pixel correction will be necessary!
!           Otherwise the correction will be performed by MODIFY/PIXEL or
!           MODIFY/COL depending on the existance of non-zero values of
!           the X-column within the bad pixel table.
!           NEGATIVE column numbers have special meanings:
!                   -1:  indicates a single bad pixel. Those may not treated in
!                the usual way.
!                This singularity will be replaced by the mean of its 8
!                neighbours.
!                   -2:  indicates a bad pixel area at the edge of the image
!                frame.
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1998/01/13  Creation
!           1.0 1998/08/20  Add noise to modified pixel areas
!                 99/08/23  Enable catalogue input (NR)
!
!.COMMENT
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name of bad pixel MIDAS table: "
define/param p2 auto c "Enter name of INPUT catalogue: "
define/param p3 N c "noise flag (N=NO, Y=YES): "
define/param p4 W c "coordinate system: WORLD or PIXEL: "
!
! local keywords
!
define/local pid/c/1/20 "corr_bpxl" ? +lower_levels
define/local i1/i/1/1 0                         !do-counter
define/local j1/i/1/1 0                         !do-counter
define/local ilo/i/1/1 1
define/local ihi/i/1/1 1
define/local num/i/1/1 4                        !number of INPUT frames
define/local nrw/i/1/1 0                        !number of rows in badpixel.tbl
define/local nar/i/1/1 0                        !number of bad pixel areas
define/local nco/i/1/1 4                        !number of bad columns
define/local sta/i/1/2 0,0
define/local bnd/i/1/4 0.0 ALL                  !boundary array of an area
define/local XOFF/i/1/1 0                       !x-offset [pixel]
define/local YOFF/i/1/1 0                       !x-offset [pixel]
set/format I1
if p2 .ne. "auto" then
  show/icat {p2} n
  define/local totimano/i/1/1 {outputi(1)}
  if totimano .eq. 0 then
    mess/out 1 {pid} "Empty input catalogue !"
    return 1
  endif
  define/local img/c*100/1/{totimano} " " all
else
  define/local img/c*100/1/4 " " all
endif
define/local XDIM2/i/1/1 0                      !

define/local ISEC/c/1/80 " " ALL
define/local fnam/c/1/80 " " ALL
define/local warn/c/1/80 " " ALL

define/local catal/i/1/1 0
define/local nsl1/i/1/1 0 ? +lower_levels               !number of selected rows
define/local nsl2/i/1/1 0 ? +lower_levels               !number of selected rows


set/format I1
if m$index(P1,"not_used") .gt. 0 return         !no badpixel table specified




fnam = p1                                       !BDF-file name of bad pixel tbl


select/table {P1} ALL
show/tab {P1} >Null                             !number of bad pixel areas
nrw = outputi(2)
if nrw .eq. 0 then
  mess/out 4 {pid} -
    "Bad pixel table is empty.\nNo corrections have to be done!"
  return
else
  define/local mid$selidx/i/1/{nrw} 0 ALL +lower_levels
endif



if m$index(P2,"auto") .gt. 0 then               !process set of cal.frames
  img(1) = "{CCD_IN}.bdf"
  img(2) = SC_BSFRM
  img(3) = SC_DKFRM
  img(4) = SC_FFFRM
  ilo = 1
  ihi = 4
else                                            !process input catalogue
  num    = {totimano}
  ilo    = 1
  ihi    = {totimano}
  catal = 0
  do i1 = ilo ihi
    store/frame inputc {p2}
    img({i1}) = inputc
  enddo
endif



image/section {img(1)} X                        !get image section on CCD
if {Q1} .gt. 0 return 1

if P4(1:1) .eq. "W" then
!  ISEC = outputc                               !in world coordinates
  ISEC = out_b                                  !in pixel coordinates
  XOFF = 0
  YOFF = 0
else
  ISEC = out_b                                  !in frame-pixel coordinates
  XOFF = m$value({img(1)},{h_xwinstr}) - 1      !first pixel in image area
  YOFF = m$value({img(1)},{h_ywinstr}) - 1      !first pixel in image area
  if XOFF .gt. 0 @p fors_corr_bpxl,SHIFT {P1} {XOFF} X
  if YOFF .gt. 0 @p fors_corr_bpxl,SHIFT {P1} {YOFF} Y
endif
XDIM2 = outputi(3) - outputi(1) + 1 - 2



do i1 = ilo ihi

  fnam = img({i1})

  if m$tstno(fnam) .eq. 1 goto NXT              !is a constant (e.g. bias) ?
  if m$index(fnam,"not_used") .gt. 0 goto NXT
  if m$existd(fnam,FORS_IDSTRNG) .eq. 1 then
    if m$index("{{fnam},{FORS_IDSTRNG}}","MASTER") .gt. 0 .or. -
       m$index("{{fnam},{FORS_IDSTRNG}}","SUPER") .gt. 0 goto NXT
  endif

  if m$existd(fnam,"CCDSTAT") .lt. 1 -
    write/desc {fnam} CCDSTAT/i/1/8 0 A !used by CCDRED-package
  write/descr {fnam} HISTORY/c/-1/80 "Processed by  @p fors_corr_bpxl {P1}"
  write/descr {fnam} BPXLCORR/C/1/15 fors_corr_bpxl.prg



!
! Start doing the corrections:
! a) bad column correction
!    . using MODIFY/COLUMN: columns will be replaced by a linear least squares
!      fit based on the 2 adjacent columns on the left and right. Thus, the
!      to be corrected columns have to be in a range of 2 < col < NPIX-2.
!
!    . using COMPUTE/COLUMN: in order to take account for the columns at the
!      border of the frame col < 3 AND col > NPIX-3.
! b) bad pixel areas
!    will be corrected using the MODIFY/PIXEL command. It performs a 2-dim. fit
!    based on the surrounding of the to be corrected area. The surrounding area
!    operates only on the image section of the frame which may be specified.
!    This is essential especially in the case of frames still having their pre-
!    and postscan regions -- the fits would be disturbed.
!

!  outputi(1) = 0               
  select/table {p1} :X.gt.0 >Null               !start doing the correction
  nco = outputi(1)                              !bad columns
  if nco .gt. 0 then
    @p fors_corr_bpxl,BCOL {P1} {fnam} {XDIM2}
  endif


  select/table {p1} :X.eq.0.OR.:X.eq.-1 >Null   !check for bad pixel areas
  nar = outputi(1)                              !number of bad pixel areas
  if nar .gt. 0 then                            !bad pixel areas
    @p fors_corr_bpxl,BAREA {P1} {fnam} {P3} {P4} {ISEC} {nar}
  endif



  select/table {P1} ALL                 !reset selection
  {fnam},CCDSTAT(3) = 1                 !indicates bad pixel corr. DONE

NXT:
enddo

if num .gt. 1 then
  if m$existk("FX_TABLE") .eq. 1 FX_TABLE = P1
  if m$existk("SC_FXPIX") .eq. 1 SC_FXPIX = "no"
endif
if XOFF .gt. 0 @p fors_corr_bpxl,SHIFT {P1} -{XOFF} X
if YOFF .gt. 0 @p fors_corr_bpxl,SHIFT {P1} -{YOFF} Y

!*** OUTPUT: ***
  mess/out 3 {pid}   "Bad pixel correction applied on"
  longlog/out 1 "     Bad pixel correction applied on"
do i1 = ilo ihi
  if m$tstno(img({i1})) .eq. 1 goto not_corrected ! if a constant (e.g. bias)
  if m$index(img({i1}),"not_used") .gt. 0 goto not_corrected
  if m$existd(img({i1}),FORS_IDSTRNG) .eq. 1 then
    if m$index("{{img({i1})},{FORS_IDSTRNG}}","MASTER") .gt. 0 .or. -
       m$index("{{img({i1})},{FORS_IDSTRNG}}","SUPER") .gt. 0 goto not_corrected
  endif
  mess/out 3 {pid}   "  {img({i1})}"
  longlog/out 1 "       {img({i1})}"
  not_corrected:
enddo
  mess/out 3 {pid}   "Bad pixel map used: {p1}"
  longlog/out 1 "     Bad pixel map used: {p1}"
  set/format J3
if nco .gt. 0 then
  mess/out 4 {pid}   "  bad columns: {nco}"
  longlog/out 1 "       bad columns: {nco}"
endif
if nar .gt. 0 then
  mess/out 4 {pid}   "  bad areas:   {nar}"
  longlog/out 1 "       bad areas:   {nar}"
  longlog/out 1 "       corrected normal areas: {nsl1}"
  longlog/out 1 "       corrected big areas:    {nsl2}"
endif
shortlog/out 1  "Bad pixel correction done."
!***************

set/format
return 0

!=== SUBROUTINES ==============================================================

ENTRY BCOL

define/param p1 ? c "Enter name of bad pixel MIDAS table: "
define/param p2 ? c "Enter name of INPUT frame: "
define/param p3 ? c "Enter right edge of frame: "

define/local nc/i/1/1 0
define/local ic/i/1/1 0
define/local ic1/i/1/1 0
define/local ic2/i/1/1 0
define/local i1/i/1/1 0
define/local j1/i/1/1 0

!outputi(1) = 0
select/table {p1} :X.gt.0.AND.:XSTARTPIX.gt.2.AND.:XENDPIX.lt.{p3} >Null
!write/out "DEBUG-select/table: {outputi(1)} {P3}"
if outputi(1) .gt. 0 then
  modify/column {p2},{p1} {p2} V >Null
else
  do j1 = 1 2
    ic  = {P3}
    ic2 = {P3} + 2
    if j1 .eq. 1 then
!    outputi(1) = 0
    select/table {p1} :X.gt.0.AND.(:XSTARTPIX.ge.0.AND.:XSTARTPIX.le.2) >Null
    else
    select/table {p1} :X.gt.0.AND.(:XENDPIX.ge.{ic}.AND.:XENDPIX.le.{ic2}) >Null
    endif
!    write/out "DEBUG-select/table: {j1} {outputi(1)} {P3}"
    nc = outputi(1)
    if nc .gt. 0 then
      do i1 = 1 {nc}
        ic = m$value({p1},:XSTARTPIX,{mid$selidx({i1})}) + 1
        if ic .gt. 2 then
          ic1 = ic - 1
          ic2 = ic - 2
        else
          ic1 = ic + 1
          ic2 = ic + 2
        endif
        write/key inputi/i/1/3 2,{ic1},{ic2}

        @p fors_replcol {p2} {ic}
    !    compute/column {p2}.C{ic} = (C{ic1}+C{ic2})/2.0
      enddo
    endif
  enddo
endif



ENTRY BAREA

define/param p1 ? c "Enter name of bad pixel MIDAS table: "
define/param p2 ? c "Enter name of INPUT frame: "
define/param p3 N c "Noise flag (N=NO, Y=YES): "
define/param p4 W c "Coordinate system: WORLD or PIXEL: "
define/param p5 ? c "Image section: "
define/param p6 ? c "Number of bad pixel areas: "

define/local num/i/1/1 0                        !number of rows
define/local xf/r/1/1 0.0                       !factors used to define ..
define/local yf/r/1/1 0.0                       !surr. area in MODI/PIX
define/local xwmax/r/1/1 0.0                    !maximum x-width of region
define/local ywmax/r/1/1 0.0                    !maximum y-width of region
define/local MAXXW/r/1/1 320.                   !allowed maximum x-width
define/local MAXYW/r/1/1 320.                   !allowed maximum y-width
define/local MSZ/i/1/1 102400                   !max. size 320*320 (MODI/PIXEL)

set/format I1 F8.5
compute/table {P1} :SZ = :XW * :YW * 4          !if XW and YW is 160 the
                                                !maximum for the surrounding
                                                !area 160*2x160*2 = 320*320
                                                !is reached. Min. scale factor
                                                !P3 in MODIF/PIX should be 2.
num = {p6}
!outputi(1) = 0
inputc = ":X.gt.-2.AND.:X.le.0.AND.:MAXW.le.80.AND.:SZ.le.{MSZ}"
select/table {P1} {inputc} >Null
nsl1 = outputi(1)
if nsl1 .gt. 0 then
  mess/out 4 {pid} "{nsl1} normal areas will be corrected ..."
  modify/pixel {p2},{p1},{p4} p3=4,4 p4=3,3,1 p6={p3} p7={p5} >Null
  num = num - nsl1
endif


!outputi(1) = 0
inputc = ":X.gt.-2.AND.:X.le.0.AND.:MAXW.gt.80.AND.:SZ.le.{MSZ}"
select/table {p1} {inputc} >Null
nsl2 = outputi(1)
if nsl2 .gt. 0 then
  mess/out 4 {pid} "{nsl2} BIG areas will be corrected ..."
  statist/table {p1} :XW >Null
  xwmax = outputr(2)
  xf = MAXXW / xwmax
  if xf .lt. 1 xf = 2
  if xf .gt. 4 xf = 3

  statist/table {p1} :YW >Null
  ywmax = outputr(2)
  yf = MAXYW / ywmax
  if yf .lt. 1 yf = 2
  if yf .gt. 4 yf = 3

  mess/out 4 {pid} "Scaling factors to get surr.area: {xf},{yf}"
  modify/pixel {p2},{p1},{p4}  p3={xf},{yf} p4=3,3,1 p6={p3} p7={p5} >Null
  num = num - nsl2
endif

if num .gt. 0 then
  append/out "{num} area(s) are too big !\n"
  append/out "Bad pixel areas GREATER than 25600 pixels\n"
  append/out "may NOT be corrected !"
  mess/out 2 {pid}
  inputc = ":X.gt.-2.AND.:X.le.0.AND.:MAXW.gt.80.AND.:SZ.gt.{MSZ}"
  select/tab {p1} {inputc} >Null
  read/tab {p1} :XSTART,:YSTART,:XEND,:YEND
endif

set/format
compute/table {P1} :SZ = :XW * :YW                      !reset



ENTRY SHIFT

define/param p1 c ? "Bad pixel table: "
define/param p2 n ? "Offset (pixel [integer]): "
define/param p3 c ? "X or Y:"

mess/out 4 {pid} "Shift table by {P2}"
compute/table {P1} :{P3}STARTPIX = :{P3}STARTPIX - {P2}
compute/table {P1} :{P3}ENDPIX = :{P3}ENDPIX - {P2}
