! $Id: fors_corr_dark.prg,v 1.8 1999/09/08 14:06:03 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_corr_dark.prg,v $
!
!.KEYWORDS    Basic CCD reduction, dark correction
!
!.USAGE       @p fors_corr_dark  raw_frm dk_frm red_frm trim
!
!.INPUT       raw_frm	  (P1)	  Science frame to be corrected.
!             dk_frm	  (P2)	  Dark frame or value to be corrected for.
!             trim	  (P4)	  Frame coordinates for extracting useful
!                                 image parts.
!
!.OUTPUT      red_frm     (P3)    Corrected frame.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The recipe does a straight forward dark correction, i.e. the
!             dark frame or value is subtracted from the science frame. Note
!             that the dark correction is exposure time dependent, i.e. the
!             dark frame has to be normalized to unit time and must be scaled
!             by the exposure time of the input raw frame. If a dark value is 
!             given instead of a frame, it is assumed that the number is in
!             units of 1/sec and has only to be scaled by the exposure time
!             of the input raw frame.
!             If an output frame name is given the result frame is stored
!             using this name, otherwise the input frame is replaced by the
!             corrected version. Optionally a window can be specified for the
!             dark frame so that it matches the size of the input frame.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!
!------------------------------------------------------------------------------

crossref  input dark output window

define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter name of dark frame/value:"
define/param p3 + c "Enter name of reduced frame:"
define/param p4 + c "Enter extract region coordinates:"
define/maxpar 4


! Procedure identification
define/local procID/c/1/80	"CorrDark"


! Local constants
define/local tmp0/c/1/12     "middumma.bdf"
define/local tmp1/c/1/12     "middummb.bdf"


! Local variables
define/local dk_tim/d/1/1    0.
define/local exp_tim/d/1/1   0.
define/local raw_frm/c/1/80  {p1}
define/local dk_frm/c/1/80   {p2}
define/local red_frm/c/1/80  {p3}



if red_frm(1:1) .eq. "+" then
  red_frm = raw_frm
  -rename {raw_frm} {tmp0}
  raw_frm = tmp0
endif

if m$existd(raw_frm,"O_TIME") .eq. 0 then
  mess/out 1 {procID} "Descriptor O_TIME not present in {raw_frm}!"
  return 1
else
  exp_tim = m$value({raw_frm},O_TIME(7))
endif 

if m$tstno(dk_frm) .eq. 0 then
  if m$existd(dk_frm,"O_TIME") .eq. 0 then
    mess/out 1 {procID} "Descriptor O_TIME not present in {dk_frm}!"
    return 1
  else
    dk_tim = m$value({dk_frm},O_TIME(7))
    if dk_tim .le. 0. then
      mess/out 1 {procID} "Invalid exposure time entry in dark frame!"
      return 1
    endif
  endif
else
  dk_tim = 1.
  exp_tim = 1. !!! no scaling when constant is used (SB) !!!
endif 

! Do the correction
if m$tstno(dk_frm) .eq. 0 .and. p4(1:1) .ne. "+" then
  extract/image {tmp1} = {dk_frm} {p4}
  dk_frm = tmp1
endif

compute/image {red_frm} = {raw_frm} - ({exp_tim}/{dk_tim})*{dk_frm}


! Create log messages
if m$tstno(p2) .ne. 1 then
  longlog/out 1 "     Master Dark frame subtracted: {p2}"
  longlog/out 1 "     (scaled to exposure time of raw frame)"
else
  longlog/out 1 "     Constant DARK value subtracted: {p2}"
endif
shortlog/out 1 "Dark correction done."


! Cleanup
-delete {tmp0}
-delete {tmp1}


return 0
