! $Id: fors_corr_nflat.prg,v 1.8 1999/09/08 14:06:02 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_corr_nflat.prg,v $
!
!.KEYWORDS    Basic CCD reduction, flat field correction
!
!.USAGE       @p fors_corr_nflat  raw_frm nf_frm red_frm trim
!
!.INPUT       raw_frm     (P1)    Science frame to be corrected.
!             nf_frm      (P2)    Normalized flat field frame used for
!                                 correction. 
!             trim        (P4)    Frame coordinates for extracting useful
!                                 image parts.
!
!.OUTPUT      red_frm     (P3)    Corrected frame.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The procedure does a straight forward flat field correction
!             dividing an input frame by a normalized flat field frame.
!             If an output frame name is given the result frame is stored
!             using this name, otherwise the input frame is replaced by the
!             corrected version. Optionally a window can be specified for the
!             normalized flat field frame so that it matches the size of the
!             input frame. 
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!
!------------------------------------------------------------------------------

crossref  input nflat output window

define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter name of normalized flat field frame:"
define/param p3 + c "Enter name of reduced frame:"
define/param p4 + c "Enter window coordinates:"
define/maxpar 4


! Procedure identification
define/local procID/c/1/80	"CorrNflat"


! Local constants
define/local tmp0/c/1/12     "middumma.bdf"
define/local tmp1/c/1/12     "middummb.bdf"


! Local variables
define/local raw_frm/c/1/80  {p1}
define/local nf_frm/c/1/80   {p2}
define/local red_frm/c/1/80  {p3}



if red_frm(1:1) .eq. "+" then
  red_frm = raw_frm
  -rename {raw_frm} {tmp0}
  raw_frm = tmp0
endif

! Do the correction
if p4(1:1) .ne. "+" then
  extract/image {tmp1} = {nf_frm} {p4}
  nf_frm = tmp1
endif

compute/image {red_frm} = {raw_frm} / {nf_frm} >Null


! Create log messages
longlog/out 1 "     High spatial frequencies from {{p2},{FORS_IDSTRNG}}"
longlog/out 1 "     Master Normflat frame used: {p2}"
longlog/out 1 "     (raw frame was divided by normalized flat.)"
shortlog/out 1 "Flat field correction done."


! Cleanup
-delete {tmp0}
-delete {tmp1}

return 0
