! $Id: fors_corr_wcs.prg,v 1.3 2001/06/21 09:38:36 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 2000  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_corr_wcs.prg,v $
!
!.KEYWORDS    Corrects incorrect WCS descriptors for FORS frames
!
!.USAGE       @p fors_corr_wcs catalog
!
!.INPUT       catalog (P1)      a catalog containing files (images and tables)
!
!.OUTPUT      All images in the catalog get correct WCS descriptors (START and 
!             REFPIX).
!
!.RETURNS     The routine returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status.
!
!.PURPOSE     The routine corrects the WCS keywords START and REFPIX for
!             FORS images. The location of the window is used to
!             determine correct values for these keywords. Currently only
!             implemented for binning 1x1.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 99NOV.
!
!.AUTHOR      S.Bogun, ESO-DMO
!
!.VERSION     1.0  2000/01/18:  Creation
!
!------------------------------------------------------------------------------

define/param p1  ?      c    "Frame catalog:"


! Procedure identification
define/local procID/c/1/80	"fors_corr_wcs"


! Local variables
define/local i/i/1/1             0
define/local catal/i/1/1         0
define/local catalog/c/1/80      {p1}
define/local nframes/i/1/1       0
define/local refpix/d/1/2        0.            all
define/local winstr/d/1/2        0.            all
define/local start/d/1/2         0.            all
define/local bin/i/1/2           0             all
define/local cat_entry/c/1/80    " "           all
define/local i1/i/1/1 17
define/local i2/i/1/1 32
define/local j1/i/1/1 33
define/local j2/i/1/1 48


show/icat {catalog} >Null
nframes = outputi(1)

if nframes .eq. 0 then
  mess/out 1 {procID} "Catalog {catalog} is empty."
  return 1
endif


do i = 1 nframes 1
  store/frame cat_entry {catalog} 1
  if m$filtyp(cat_entry,".bdf") .eq. 1 .and. m$existd(cat_entry,"CUNIT") -
     .ne. 0 then
    if m$index("{{cat_entry},CUNIT({i1}:{i2})}","PIXEL") .gt. 0 .AND. -
       m$index("{{cat_entry},CUNIT({j1}:{j2})}","PIXEL") .gt. 0 then

      bin(1) = m$value({cat_entry},{h_xwinbin})
      bin(2) = m$value({cat_entry},{h_ywinbin})  

      copy/dk {cat_entry} REFPIX refpix
      copy/dk {cat_entry} START  start

      start(1) = m$nint(start(1))
      start(2) = m$nint(start(2))

      @p fors_getwin {cat_entry}

      if {q1} .eq. 1 then
        if bin(1) .eq. 1 .and. bin(2) .eq. 1 then
          winstr(1) = outputi(1)
          winstr(2) = outputi(2)

          outputd(1) = start(1) - ( fors_nprescan + winstr(1) )
          outputd(2) = start(2) - winstr(2)

          start(1) = fors_nprescan + winstr(1)
          start(2) = winstr(2)

          refpix(1) = refpix(1) + outputd(1)
          refpix(2) = refpix(2) + outputd(2)

        else
          mess/out 2 {procID} "WCS of binned frames only partially implemented!"
        endif
      endif          

      copy/kd refpix {cat_entry} REFPIX
      copy/kd start  {cat_entry} START

      mess/out 3 {procID} "WCS of frame {cat_entry} corrected."

    endif
  endif
enddo


return 0
