! $Id: fors_crea_tab.prg,v 1.6 1999/11/25 11:23:52 sbogun Exp $
!+++ FORS_CREA_TAB ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_crea_tab.prg,v $
!
!.KEYWORDS      Create a table using a reference table
!
!.USAGE         @p fors_crea_tab newtab reftab histo mode
!
!.INPUT         newtab  (P1): name of new table
!               reftab  (P2): name of reference table
!               histo   (P3): history string to be added
!               mode    (P4): allowed values:
!                               0: do not delete old HISTORY
!                               1: delete old HISTORY
!                               2: project reference table onto new table
!                                  (see MIDAS command PROJ/TAB)
!                               3: same as 2 but delete all rows
!
!.OUTPUT        new table having the same FITS header as the reference
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Create a table with the same descriptors as the reference
!
!.ALGORITHM     Create a copy of the reference table (P2) and delete all
!               columns. As a result, the new table (P1) will have the same
!               descriptors as (P2).
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.9 1998/03/09  Creation
!
!.COMMENT
!------------------------------------------------------------------------------
!
! Check parameters
!
define/para p1 ? c "Enter name of new table: "
define/para p2 ? c "Name of reference table (its header will be copied to new)"
define/para p3 ? c "History string:"
define/para p4 0 n "1: Delete old HISTORY descriptor, 0: do not delete!"
!
! Define local keywords
!
define/local i1/i/1/1 1
define/local ncol/i/1/1 1
define/local nrow/i/1/1 1


show/tab {P2} >Null
ncol = outputi(1)
nrow = outputi(2)

BRANCH P4 0,1,2,3 CREA,CREA,PROJ,PROJ

CREA:
  copy/tab {P2} {P1}
  do i1 = ncol 1 -1
    del/col {P1} #{i1}
  enddo
  if {P4} .eq. 1 del/descr {P1} HISTORY
  wr/desc {P1} HISTORY/c/1/100 {P3}
goto endbra

PROJ:
  -copy {P2} wrk.tbl
  del/row wrk.tbl @1..{nrow}

  set/format I3
  do i1 = 1 ncol 1
    copy/tt wrk.tbl #{i1} {P1} :{wrk.tbl,TLABL{i1}(1:10)}
  enddo
  set/format
  wr/desc {P1} HISTORY/c/1/100 {P3}
endbra:

show/tab {P1} >Null
return {outputi(1)} {outputi(2)}
