! $Id: fors_creadisp.prg,v 1.4 1999/11/25 11:23:52 sbogun Exp $   
!+++ FORS_CREADISP ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_creadisp.prg,v $
!
!.KEYWORDS      Creates image and graphic displays needed for pipeline
!
!.USAGE         @p fors_creadisp sof_in out sof_ref [optpar]
!
!.INPUT         use_X   (P1): used X-screen-dimension
!               use_Y   (P2): used Y-screen-dimension
!               max_X   (P3): maximum X-screen-dimension (hardware)
!               max_Y   (P4): maximum Y-screen-dimension (hardware)
!
!.OUTPUT        terminal output
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT       P1      1600
!               P2      1200
!               P3      1600
!               P4      1200
!
!.PURPOSE       Create 3 image displays on top of the screen. Below these
!               2 graphic windows will be created.
!               Two global keys will be created holding the X- and Y-size
!               all equi-sized windows: FORS_XWSZ, FORS_YWSZ
!
!.ALGORITHM     Calls standard MIDAS commands
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/08/24  Creation
!
!.COMMENT
!------------------------------------------------------------------------------
define/param P1 1600 n "used X-dimension:"
define/param P2 1200 n "used Y-dimension:"
define/param P3 1600 n "maximum allowed X-dimension [hardware]:"
define/param P4 1200 n "maximum allowed Y-dimension [hardware]:"

define/local i1/i/1/1 0
define/local xdim/i/1/1 0
define/local ydim/i/1/1 0
define/local xoff/i/1/1 0
define/local yoff/i/1/1 0
define/local off/i/1/1 6
define/local dd/r/1/1 0.0

if {P1} .gt. {P3} .OR. {P1} .lt. 0 then
  mess/out 1 creadisp "Bad X-size: {P1} (<={P3} and should be positive!)"
  return -1
endif
if {P2} .gt. {P4} .OR. {P2} .lt. 0 then
  mess/out 1 creadisp "Bad Y-size: {P2} (<={P4} and should be positive!)"
  return -2
endif

 
dd = ({P1} - 3 * off) / 3.0
xdim = m$nint(dd)
dd = ({P2} - 120) / 2.0
ydim = m$nint(dd)

wr/key FORS_XWSZ/i/1/1 {xdim}
wr/key FORS_YWSZ/i/1/1 {ydim}

xoff = xdim + off
yoff = {P4} - 2 * ydim - 56
crea/gra 1 {xdim},{ydim},{xoff},{yoff} >Null
xoff = xoff + xdim + off
crea/gra 2 {xdim},{ydim},{xoff},{yoff} >Null


xoff = -xdim - off
yoff = {P4} - ydim
do i1 = 1 3
  xoff = xoff + xdim + off
  creat/disp {i1} {xdim},{ydim},{xoff},{yoff} >Null
enddo


return 0
