! $Id: fors_creaexttbl.prg,v 1.1 1999/03/09 08:46:26 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_creaexttbl.prg,v $
!
!.KEYWORDS    Create extinction data table
!
!.USAGE       create/exttbl exttbl seqno size
!
!.INPUT       exttbl      (P1)    Name or base name of 3d extinction 
!                                 correction factor table.
!             seqno       (P2)    Sequence number to append to table name.
!             size        (P3)    Table element array size.
!
!.OUTPUT      None
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. The keyword OUTPUTC is filled
!             with the complete name of the created extinction correction
!             factor table.
!
!.PURPOSE     The procedure creates an empty 3d table suitable to store
!             multiple, spectroscopic extinction correction factors. The
!             table is created with the table element size provided by the
!             size parameter. If no sequence number is given, the provided
!             table name is assumed to be complete. Otherwise it is taken 
!             as a name prefix and the sequence number is appended to obtain
!             the complete table name.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/12/02:  Creation
!
!------------------------------------------------------------------------------

crossref  exttbl seqno size

define/param p1 ? c "Extinction correction factor table (base)name:"
define/param p2 + c "Table sequence number:"
define/param p3 ? c "Table element size:"
define/maxpar 3


! Procedure identification
define/local procID/c/1/20      "CreaExttbl"


! Local variables
define/local size/i/1/1    0
define/local seqno/i/1/1   0
define/local name/c/1/100  {p1}



! Process parameters
if m$indexb(name,".tbl") .gt. 1 then
  if m$tstno(p2) .eq. 1 then
    mess/out 1 {procID} -
        "Table name already complete! Cannot append sequence number!"
    return 1
  endif
else
  if m$tstno(p2) .eq. 1 then
    seqno = {p2}
    set/format i4
    name = name // "{seqno}" // ".tbl"
    set/format
  else
    name = name // ".tbl"
  endif
endif

if m$tstno(p3) .ne. 1 then
  mess/out 1 {proID} "Invalid table element size!"
  return 1
else
  size = {p3}
  if size .lt. 1 then
    mess/out 1 {proID} "Invalid table element size!"
    return 1
  endif
endif


! Create extinction correction factor table
create/table {name} 1 1 NULL
create/column {name} :{c_xtstart} "Angstrom" F10.4 R*4
create/column {name} :{c_xtstep} "Angstrom" F10.4 R*4
create/column {name} :{c_xtndata} "None" I6 I*4
create/column {name} :{c_xtratio} "mag" E12.6 R*4({size})

outputc = name

return 0
