! $Id: fors_ctxdel.prg,v 1.2 1999/02/11 14:12:18 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_ctxdel.prg,v $
!
!.KEYWORDS    Contexts, administration, pipeline
!
!.USAGE       set/ctxdel [reset]
!
!.INPUT       reset       (P1)   Reset flag
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. The keyword P_CTXDEL is used
!             to indicate the delete status.
!
!.PURPOSE     The procedure sets or removes the delete flag for auxiliary
!             contexts used by recipes. A context is regarded as auxiliary
!             when it is used by an individual recipe within a super recipe
!             and was not already loaded as main context. The lock status
!             of the keyword is preserved.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/02/10:  Creation
!
!------------------------------------------------------------------------------

crossref reset

define/param p1  +   c     "Enter delete flag:"


! Procedure identification
define/local procID/c/1/80      "ForsCtxDel"


! Local constants
define/local keyname/c/1/16  "p_ctxdel"


! Local variables
define/local lockno/i/1/1    0
define/local flag/i/1/1      0
define/local state/c/1/100   {p1}



state = m$upper(state)
if state(1:1) .eq. "R" then
  flag = 0
else
  flag = 1
endif

! Check if keyword is locked
info/keyw {keyname}
lockno = mid$info(4)

if lockno .ne. 0 then
  unlock/key {keyname} {lockno}
endif

if m$existk(keyname) .eq. 1 then
  write/key {keyname}/i/1/1 {flag}
else
  mess/out 1 {procID} "Keyword {keyname} to be modified does not exist!"
  return 1
endif

if lockno .ne. 0 then
  lock/key {keyname} {lockno}
endif
  
return 0


