! $Id: fors_defbandpass.prg,v 1.2 2000/03/24 15:31:42 nrainer Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_defbandpass.prg,v $
!
!.KEYWORDS    Spectroscopy, spectrum rebinning
!
!.USAGE       define/bandpass  frame gtable
!
!.INPUT       frame    (P1)   Input spectrum to be rebinned.
!             gtable   (P2)   Table of grism wavelength ranges for different
!                             instrument setups.
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Select the grism rebinning parameters according to the
!             instrument grism and filter setup. The parameters are
!             stored in the XMOS session keywords REBSTRT, REBEND and
!             REBSTP.
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/03/22:  Creation
!
!------------------------------------------------------------------------------

crossref  frame gtable 

define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter grism range table:"
define/maxpar 2


! Procedure identification
define/local procID/c/1/80      "DefBandpass"


! Local variables
define/local i/i/1/1                       0
define/local nrow/i/1/1                    0
define/local nsetup/i/1/1                  0
define/local isetup/i/1/1                  0
define/local gname/c/1/80                  " "  all
define/local filter/c/1/80                 " "  all
define/local frm/c/1/100                   {p1}
define/local gtbl/c/1/100                  {p2}



! Get active filter (filters must already be configured)
if fors_nfilt .gt. 1 then
  mess/out 1 {procID} "Cannot handle multi filter exposures!"
  return 1
else if fors_nfilt .eq. 0 then
  filter = FORS_NOFILT
else
  filter = fors_filti(1)
endif


! Get current grism name
gname = m$value({frm},{h_grism1})


! Get number of instrument setups in catalog
nrow = m$value({gtbl},tblcontr(4))


! Select appropriate grism bandpass from catalog
nsetup = 0

do i = 1 nrow
  in_a = "{{gtbl},:{c_grgrism},{i}}"  ! removes trailing blanks
  in_b = "{{gtbl},:{c_grfilt},{i}}"   ! removes trailing blanks
  if gname .eq. in_a .and. filter .eq. in_b then
    isetup = i
    nsetup = nsetup + 1
  endif
enddo


! Check setup selection
if nsetup .eq. 0 then
  mess/out 1 {procID} "Cannot set wavelength range for rebinning!"
  mess/out 1 {procID} -
      "No appropriate instrument setup found in grism range database!"
  return 1
else if nsetup .gt. 1 then
  mess/out 1 {procID} "Cannot set wavelength range for rebinning!"
  mess/out 1 {procID} -
      "No unique instrument setup found in grism range database!"
  return 1
endif


! Setup rebin session keywords
rebstrt = m$value({gtbl},:{c_grstart},{isetup})
rebend = m$value({gtbl},:{c_grend},{isetup})
rebstp = m$value({gtbl},:{c_grstep},{isetup})

mess/out 3 {procID} "Using rebin parameter setup for"
mess/out 3 {procID} "grism >{gname}< and filter >{filter}<."

return 0
