! $Id: fors_defgrism.prg,v 1.3 1998/09/29 16:24:22 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_defgrism.prg,v $
!
!.KEYWORDS    Spectroscopy, Wavelength calibration, FORS grisms
!
!.USAGE       define/grism  frame gristbl
!
!.INPUT       frame    (P1)   Calibration frame from which the instrument
!                             grism setup is extracted.
!             gristbl  (P2)   table of grism parameters for all grisms 
!                             built in the instrument.
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Select the physical parameters from the grism parameter table
!             for the grism used for the exposure of the wavelength
!             calibration frame. 
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/09/21:  Creation
!
!------------------------------------------------------------------------------

crossref  frame gristbl

define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter name of grism table:"
define/maxpar 2


! Procedure identification
define/local procID/c/1/80      "DefGrism"


! Local variables
define/local i/i/1/1                       0
define/local nsel/i/1/1                    0
define/local nrow/i/1/1                    0
define/local grow/i/1/1                    0
define/local frame/c/1/100                 {p1}
define/local gristbl/c/1/100               {p2}



! Look for current grism in grism data table
nrow = m$value({gristbl},tblcontr(4))

do i = 1 nrow 1
  if m$index("{{gristbl},:{c_gdname},{i}}","{{frame},{h_grism1}}") .ne. 0 then
    nsel = nsel + 1
    grow = i
  endif
enddo

! Check number of selections
if nsel .ne. 1 then
  mess/out 1 {procID} "Unknown grism encountered!"
  return 1
else
  grism = m$value({gristbl},:{c_gdidno},{grow})
endif

return 0
