! $Id: fors_definerange.prg,v 1.5 2000/09/26 16:59:26 nrainer Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 2000  European Southern Observatory (ESO)
!
! .IDENT      $RCSfile: fors_definerange.prg,v $
!
! .KEYWORDS   Wavelength calibration
!
! .PURPOSE    Calculate actually existing wavelength range on CCD.
! __________
!
! .USAGE      @p fors_definerange spec mos [linfit] [show]
!
! .INPUT      spec   (P1): either reduced lamp or standard or science frame;
!                            default: none
!
!             mos    (P2): table with slit positions;
!                            default: none
!
!             linfit (P3): table with dispersion solution;
!                            default: fors_coerbr.tbl
!
!             show   (P4): flag for result output on terminal: <y>:show, <n>o
!                          (only valid for MESS_LEVEL=3, else always <y>);
!                            default: <y>
!
! .OUTPUT     OUTPUTR(1): start wavelength
!             OUTPUTR(2): end wavelength
!             OUTPUTI(1): slit used for determination of start wavelength
!             OUTPUTI(2): slit used for determination of end wavelength
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  The lines look like this (exaggerated):
!               +---------+
!               |\     \  |
!               | \     \ |
!               L  |     ||
!               | /     / |
!               |/     /  |
!               +---------H
!             Therefore at the left x-margin the reference pixel in the
!             y-middle is used and at the right x-margin the first pixel
!             at y-start in order to get the lowest (L) and highest (H)
!             possible wavelengths at the CCD.
!             It is tested (for MOS) whether (L) and (H) are within active
!             slits - if not, they are shifted to the first y-pixel in the
!             nearest slit so that a valid dispersion solution can be applied.
!
! .COMMENT    * The dispersion solution must have been computed with
!               'calibrate/grism' (="moscalifors.prg").
!             * The slit position table must have been selected with
!               'define/position' (="fors_defpositions.prg").
!
! .ENVIRON    MOS context for programmes mentioned in .COMMENT
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.5  00/07/07  Creation
!                  00/       Release
!
!------------------------------------------------------------------------------
!
!
crossref spec mos linfit show
!
!
if m$existk("h_grism1") .eq. 0 -
  write/keyword h_grism1/c/1/60 "ESO.INS.GRIS1.NAME"
if m$existk("FORS_G150_WSTRT") .eq. 0 write/keyword FORS_G150_WSTRT/r/1/1 3100.
if m$existk("FORS_G150_WEND")  .eq. 0 write/keyword FORS_G150_WEND/r/1/1 11600.
if m$existk("FORS_G200_WSTRT") .eq. 0 write/keyword FORS_G200_WSTRT/r/1/1 3100.
if m$existk("FORS_G200_WEND")  .eq. 0 write/keyword FORS_G200_WEND/r/1/1 11600.
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "definerange"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      *****************************************************************"
  w/o "      spec   (P1): either reduced lamp or standard or science frame;"
  w/o "                     default: none"
  w/o " "
  w/o "      mos    (P2): table with slit positions;"
  w/o "                     default: none"
  w/o " "
  w/o "      linfit (P3): table with dispersion solution;"
  w/o "                     default: fors_coerbr.tbl"
  w/o " "
  w/o "      show   (P4): flag for result output on terminal: <y>:show, <n>o"
  w/o "                   (only valid for MESS_LEVEL=3, else always <y>);"
  w/o "                     default: <y>"
  w/o "      *****************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(spec):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter frame with reduced spectrum [def.:none]: "
define/local spec/c/1/64 "{p1}"
if m$index(spec,".bdf") .eq. 0 spec = spec//".bdf"
if m$exist(spec) .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered frame with reduced spectrum is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({spec})"
  goto error
endif
!
!
!P2(mos):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter table with slit offset [def.:none]: "
define/local mos/c/1/64 "{p2}"
if m$index(mos,".tbl") .eq. 0 mos = mos//".tbl"
if m$exist(mos) .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered tbl. w. slit position(s) is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({mos})"
  goto error
endif
if m$existc(mos,":YSTART") .eq. -1 then
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({mos},:YSTART)"
  goto error
elseif m$existc(mos,":YEND") .eq. -1 then
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({mos},:YEND)"
  goto error
elseif m$existc(mos,":SLIT") .eq. -1 then  ! for apply/calib
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({mos},:SLIT)"
  goto error
endif
!
!
!P3(linfit):                               ! for apply/calib
!----------
define/parameter p3 fors_coerbr.tbl ? -
  "        Enter table with dispersion solution [def.:fors_coerbr.tbl]: "
define/local linfit/c/1/64 "{p3}"
if m$index(linfit,".tbl") .eq. 0 linfit = linfit//".tbl"
if m$exist(linfit) .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered tbl. w. dispers. solution is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({linfit})"
  goto error
endif
if m$existd(linfit,"NORMCOORD") .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered desc. w. fit degrees is not present"
        mess/out 1 {PROID} "in tbl. w. dispers. solution !"
        mess/out 1 {PROID} "({linfit},NORMCOORD)"
  goto error
endif
if m$existd(linfit,"CENTERXY") .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered desc. w. norm. constants is not present"
        mess/out 1 {PROID} "in tbl. w. dispers. solution !"
        mess/out 1 {PROID} "({linfit},CENTERXY)"
  goto error
endif
!
!
!P4(show):
!--------
define/parameter p4 y ? -
  "        Flag for result output on terminal [def.:<y>|<n>]: "
define/local show/c/1/1 "{p4}"
!
!
!
!          DEFINITION OF FURTHER VARIABLES :
!          *******************************
!
define/local wstart/r/1/1 0.
define/local sstart/i/1/1 0
define/local wend/r/1/1 0.
define/local send/i/1/1 0
define/local centy/r/1/1 {{linfit},CENTERXY(2)}
define/local ydiff/r/1/2 0. all
define/local ypix/r/1/1 0.
define/local showcal/c/1/1 " "
define/local noslitdisp/i/1/1 0
!
!
!
!          COMPUTATION OF WAVELENGTH RANGE ON CCD :
!          **************************************
!
if mess_level .lt. 4 then
  showcal = "n"
else
  showcal = "y"
endif
!
!- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
! Start wavelength:
select/table {mos} :YSTART.le.{centy}.and.:YEND.ge.{centy} >Null
if outputi .eq. 0 then
  ! CENTERXY(2) not within active slit - find first y-pixel in nearest slit
  select/table {mos} :YSTART.gt.{centy} >Null
  if outputi .eq. 0 then  ! CENTERXY(2) above all active slits
    ydiff(1) = {{spec},NPIX(2)}*9999
  else
    statistics/table {mos} :YSTART >Null
    ydiff(1) = outputr(1)-centy
  endif
  select/table {mos} :YEND.lt.{centy} >Null
  if outputi .eq. 0 then  ! CENTERXY(2) below all active slits
    ydiff(2) = {{spec},NPIX(2)}*9999
  else
    statistics/table {mos} :YEND >Null
    ydiff(2) = centy-outputr(2)
  endif
  ypix = ydiff(1)+centy ; if ydiff(2) .lt. ydiff(1) ypix = centy-ydiff(2)
elseif outputi .gt. 1 then
        write/out "      "
        mess/out 1 {PROID} "Tbl. w. dispers. solution not properly selected -"
        mess/out 1 {PROID} "must be done with <define/position> before !"
        mess/out 1 {PROID} "({mos})"
  goto error
else
  ypix = centy
endif
apply/calib 1.,{ypix} {mos} {linfit} {showcal}
if {q1} .eq. 1 then
        write/out "      "
        mess/out 1 {PROID} "Determination of start wavelength failed !"
  goto error
endif
sstart = outputi(1)
wstart = m$nint(outputr(1)-0.5)
select/table {mos} all
!
!- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
! End wavelength:
statistics/table {mos} :YSTART >Null  ! look for minimum
apply/calib {{spec},NPIX(1)},{outputr(1)} {mos} {linfit} {showcal}
if {q1} .eq. 1 then
        write/out "      "
        mess/out 1 {PROID} "Determination of end wavelength failed !"
  goto error
endif
send = outputi(1)
wend = m$nint(outputr(1)+0.4999999)
!
!- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
! Overwriting of computed values with defaults -
! THE COMPUTATION ABOVE MUST NEVERTHELESS BE DONE,
! BECAUSE {sstart} AND {send} ARE USED BY DIFFERENT RECIPES !
if "{{spec},{h_grism1}}" .eq. "GRIS_150I" then
  wstart = FORS_G150_WSTRT
  wend = FORS_G150_WEND
        set/format F7.2
        mess/out 4 {PROID} "  GRIS_150I: no use of calc. wavelength ranges -"
        mess/out 4 {PROID} "             using default values !"
        set/format {FMTDEF}
  noslitdisp = 1
  warn = 1
elseif "{{spec},{h_grism1}}" .eq. "GRIS_200I" then
  wstart = FORS_G200_WSTRT
  wend = FORS_G200_WEND
        set/format F7.2
        mess/out 4 {PROID} "  GRIS_200I: no use of calc. wavelength ranges -"
        mess/out 4 {PROID} "             using default values !"
        set/format {FMTDEF}
  noslitdisp = 1
  warn = 1
endif
!
!
!          OUTPUT :
!          ******
!
if show .ne. "n" .or. mess_level .eq. 4 then
        set/format I2 T5.0,
      if noslitdisp .eq. 0 then
        mess/out 3 {PROID} "  start wavelength:  {wstart} A  (slit #{sstart})"
        mess/out 3 {PROID} "  end   wavelength:  {wend} A  (slit #{send})"
      else
        mess/out 3 {PROID} "  start wavelength:  {wstart} A"
        mess/out 3 {PROID} "  end   wavelength:  {wend} A"
      endif
        set/format {FMTDEF}
endif
write/keyword outputi 0 all
write/keyword outputr 0. all
write/keyword outputd 0. all
outputi(1) = sstart
outputi(2) = send
outputr(1) = wstart
outputr(2) = wend
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!------------------------------------------------------------------------------
