! $Id: fors_deflines.prg,v 1.13 2000/07/17 16:13:08 nrainer Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_deflines.prg,v $
!
!.KEYWORDS    Spectroscopy, Wavelength calibration, line lists
!
!.USAGE       define/linecat  frame mlist linecat
!
!.INPUT       frame    (P1)   Calibration frame from which the instrument
!                             lamp setup is extracted.
!             mlist    (P2)   Table of emission line catalog for all
!                             instrument setups.
!             linecat  (P3)   Master line list of emission lines for all
!                             calibration lamps used by the instrument.
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Select and extract an emission line list from the line list
!             database. Then, all emission lines compatible with the setup
!             of calibration lamps used for the exposure of the wavelength
!             calibration frame are selected from the extracted line list.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/09/21:  Creation
!                  1999/10/07: changes for MOS context (JKR)
!             1.5  2000/05/19: adding of dummy lines to 2D master line
!                              catalogue after selection of grism, slit width
!                              and lamps (due to error in a low level function
!                              used by moscalifors.c).
!
!------------------------------------------------------------------------------

crossref  frame linecat

define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter line list catalog:"
define/param p3 ? c "Enter name of master line list:"
define/param p4 ? c "Enter name of header keyword for slitwidth :"
define/maxpar 4


! Procedure identification
define/local procID/c/1/80      "DefLineCat"


! Local constants
define/local minlin/i/1/1  2


! Local variables
define/local i/i/1/1                       0
define/local j/i/1/1                       0
define/local pos/i/1/1                     0
define/local nsel/i/1/1                    0
define/local nlamp/i/1/1                   0
define/local swid/r/1/1                    0.
define/local frm/c/1/100                   {p1}
define/local mlist/c/1/100                 {p2}
define/local lcat/c/1/100                  {p3}
define/local hkeyswid/c/1/100              {p4}
define/local gname/c/1/80                  " "  all
define/local dscname/c/1/80                " "  all
define/local lamp/c/1/10                   " "  all
define/local lidno/i/1/{FORS_MAXLAMP}      0    all



! Get active lamps
config/lamps {frm}
nlamp = fors_nlamp

do i = 1 fors_nlamp 1
  lamp = fors_lampi({i})
  pos = m$indexb(lamp,"+") - 1
  if pos .gt. 0 then
    j = 1

    ltype_loop:

    if lamp(1:{pos}) .ne. fors_ltypes({j}) then
      if j .gt. FORS_LMAXTYP then
        mess/out 3 {procID} "Invalid calibration lamp name encountered!"
        return 1
      else if lamp(1:{pos}) .eq. fors_nolamp then
        mess/out 4 {procID} "Skipping dummy lamp >{lamp}<."
        nlamp = nlamp - 1
      else
        j = j + 1
        goto ltype_loop
      endif
    else
      lidno({i}) = j
    endif
  else
    mess/out 1 {procID} "Invalid calibration lamp identifier encountered!"
    return 1
  endif
enddo

if nlamp .le. 0 then
  mess/out 1 {procID} "No active lamps found in {frm}!"
  return 1
endif



! Get line list for current grism and slit combination

if m$existd(frm,hkeyswid) .ne. 1 then
  mess/out 1 {procID} -
       "Undetermined slitwidth, descriptor {hkeyswid} is missing in {frm}!"
  return 1
else
  swid  = m$value({frm},{hkeyswid})
endif

if m$existd(frm,h_grism1) .ne. 1 then
  mess/out 1 {procID} -
       "Undetermined grism, descriptor {h_grism1} is missing in {frm}!"
  return 1
else
  gname = m$value({frm},{h_grism1})
endif

get/linecat {mlist} {lcat} {gname} {swid}
if {q1} .ne. 0 then  
  mess/out 1 {procID} "Invalid line list catalog encountered!"
  return 1
endif

! Select appropriate emission lines from catalog
select/tab {lcat} :{c_lclamp} .eq. {lidno(1)} >Null
if nlamp .gt. 1 then
    do i = 2 nlamp 1 
      select/tab {lcat} sel .or. :{c_lclamp} .eq. {lidno({i})} >Null
    enddo
endif


! Check number of selections
nsel = m$value({lcat},tblcontr(10))
if nsel .lt. minlin then
  mess/out 1 {procID} "Improper line list for current calibration lamp setup!"
  return 1
else
  set/format i1
  mess/out 3 {procID} "{nsel} lines selected from {lcat}."
  longlog/out 1 "     {nsel} lines selected from {lcat}."
  set/format
endif

return 0
