! $Id: fors_echwave_dis.prg,v 1.14 2001/04/06 16:19:23 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_echwave_dis.prg,v $
!
! .KEYWORDS   Dispersion coefficients
!
! .PURPOSE    Use a lamp frame and it's line table for the following
!             computations:
!               Make a copy of the line table for the modifications.
!               Ident_1: Use a 1st guess solution of the dispersion coeffi-
!                        cients, which was obtained before with <fors_echwave_
!                        ide.prg> and <fors_echwave_gue.prg>, for a comparison
!                        of the line positions.
!               Ident_2: Identify the calibrated lines with the help of a cata-
!                        logue with laboratory wavelengths.
!               Coeff_1: Apply a two-dimensional fit to the identified lines
!                        for final calculation of the dispers. coefficients.
!               Coeff_2: Convert the 2D to 1D coefficients (i.e., per order),
!                        calculate the start and end wavelenghts per order and
!                        identify the lines per order.
!               Coeff_3: Calculate the pixel sizes from the 1D coefficients.
!               Coeff_4: Store order informations, fit values and the minimal
!                        pixel size in descriptors destined for this purpose.
!                        These descriptors are later used for the wavelength
!                        calibration of standard and science spectra.
!             The identified wavelengths, residuals (identified minus calcula-
!             ted wavelengths) and pixel sizes can be displayed.
! __________
!
! .USAGE      @p fors_echwave_dis lampima [extimg] [wavtbl] guetbl wavcat -
!                                 [degree] [disp]
!
! .INPUT      lampima (P1): lamp frame;
!                             default: none
!
!             extimg  (P2): order frame (extracted from the lamp frame);
!                             default: <P1>_ext.bdf
!
!             wavtbl  (P3): line table from <fors_echwave_sea.prg>;
!                             default: <P1>_wav.tbl
!
!             guetbl  (P4): 1st guess reference table (created by <fors_echwave
!                           _gue.prg>);
!                             default: none
!
!             wavcat  (P5): table with labaratory wavelengths;
!                             default: none
!
!             degree  (P6): degree for fitting both x-pixels and absol. orders;
!                             default: 3
!
!             disp    (P7): display selection:<p>:pipeline mode:read global key
!                           NICE_CREA [y|n], <n>:none, <l>:local, or
!                           host:display[.screen] - with the latter option
!                           you can specify the name or IP-address of a display;
!                             default: <l>
!
! .OUTPUT     Q2: table with order/fit/pix.size infos in modified columns and
!                 updated descriptors: "dispcoef.tbl"
!
!             Q3: minimal pixel size [angstroem]
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Ident_1: The absolute order numbers of the 1st guess solution are
!                      checked against those in the line table. Possible shifts
!                      of the x-positions between the two tables are calculated
!                      by cross-correlation per order. The shifts are used to
!                      calculate improved dispersion coefficients.
!             Ident_2: Calculation of minimal residuals (catalogue minus calcu-
!                      lated wavelengths) and re-calculation of dispersion co-
!                      efficients is done iteratively.
!             Coeff_1: Polynomial regression with entered degrees. The disper-
!                      sion coefficients are calculated both for the extracted
!                      frame (descriptors REGR*) and for the original lamp frame
!                      (descriptors RORD*).
!             Coeff_2: Per order insertion of absolute orders into polynome and
!                      devision of result by absolute order. The 1D coefficients
!                      are used for the rebinning of the standard and science
!                      spectra (descriptors COEF*).
!             Coeff_3: Using 1D dispersion coefficients and number of x-pixels.
!
! .COMMENT    THE MIDAS ECHELLE CONTEXT IS NEEDED - THIS CAN CHANGE IN FUTURE !
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/10/16  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref lampima extimg wavtbl guetbl wavcat degree disp
!
!
define/local MAXDEGREE/i/1/1 6     ! ecaxtly 7 elem.p.ord. in desc. COEFD re-
!                                  ! served - DON'T CHANGE THE "6" !!!
define/local DISPTBL/c/1/20 "dispcoef.tbl"
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "echwave_dis"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
write/keyword q3 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      lampima (P1): lamp frame;"
  w/o "                      default: none"
  w/o " "
  w/o "      extimg  (P2): order frame (extracted from the lamp frame);"
  w/o "                      default: <P1>_ext.bdf"
  w/o " "
  w/o "      wavtbl  (P3): line table from <fors_echwave_sea.prg>;"
  w/o "                      default: <P1>_wav.tbl"
  w/o " "
  w/o "      guetbl  (P4): 1st guess reference table (created by <fors_echwave"
  w/o "                    _gue.prg>);"
  w/o "                      default: none"
  w/o " "
  w/o "      wavcat  (P5): table with labaratory wavelengths;"
  w/o "                      default: none"
  w/o " "
  w/o "      degree  (P6): degree for fitting both x-pixels and absol. orders;"
  w/o "                      default: 3"
  w/o " "
  w/o "      disp    (P7): display selection:<p>:pipeline mode:read global key"
  w/o "                    NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                    host:display[.screen] - with the latter option"
  w/o "                    you can specify the name or IP-address of a display;"
  w/o "                      default: <l>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(lampima):
!-----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of lamp frame: "
define/local lampima/c/1/64 "{p1}"
if m$index(lampima,".bdf") .eq. 0 lampima = lampima//".bdf"
!
if m$exist(lampima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lampima})"
  goto error
endif
!
!
!P2(extimg):
!----------
define/local char/i/1/1 0
define/local extimg/c/1/64 " " all
char = m$indexb(lampima,".")-1
extimg = lampima(1:{char})//"_ext.bdf"
define/parameter p2 {extimg} i -
  "        Enter name of order frame: "
extimg = p2
if m$index(extimg,".bdf") .eq. 0 extimg = extimg//".bdf"
!
if m$exist(extimg) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({extimg})"
  goto error
endif
!
!
!P3(wavtbl):
!----------
define/local wavtbl/c/1/60 " " all
wavtbl = lampima(1:{char})//"_wav.tbl"
define/parameter p3 {wavtbl} ? -
  "        Enter line table: "
wavtbl = p3
if m$index(wavtbl,".tbl") .eq. 0 wavtbl = wavtbl //".tbl"
!
if m$existc(wavtbl,":X") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({wavtbl})"
  goto error
elseif m$existc(wavtbl,":X") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({wavtbl},:X)"
  goto error
elseif m$existc(wavtbl,":Y") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({wavtbl},:Y)"
  goto error
elseif m$existc(wavtbl,":IDENT") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({wavtbl},:IDENT)"
  goto error
elseif m$existc(wavtbl,":YNEW") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({wavtbl},:YNEW)"
  goto error
endif
!
if {{wavtbl},TBLCONTR(4)} .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered line table is empty !"
        mess/out 1 {PROID} "({wavtbl})"
  goto error
endif
!
!
!P4(guetbl):
!----------
if p4 .eq. "?" mess/out 2 {PROID} " "
define/parameter p4 ? ? -
  "        Enter guess table: "
define/local guetbl/c/1/60 "{p4}"
if m$index(guetbl,".tbl") .eq. 0 guetbl = guetbl //".tbl"
!
if m$existc(guetbl,":X") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({guetbl})"
  goto error
elseif m$existc(guetbl,":X") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({guetbl},:X)"
  goto error
elseif m$existc(guetbl,":YNEW") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({guetbl},:YNEW)"
  goto error
elseif m$existc(guetbl,":ORDER") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({guetbl},:ORDER)"
  goto error
endif
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
if m$existd(guetbl,"PIXEL") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({guetbl},PIXEL)"
  goto error
endif
outputi = 0
if m$existd(guetbl,"RORDC") .eq. 0 outputi = 1
if m$existd(guetbl,"RORDI") .eq. 0 outputi = 1
if m$existd(guetbl,"RORDR") .eq. 0 outputi = 1
if m$existd(guetbl,"RORDD") .eq. 0 outputi = 1
if outputi(1) .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({guetbl},RORD[ C/I/R/D ])"
  goto error
endif
if m$existd(guetbl,"ORDER") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({guetbl},ORDER)"
  goto error
endif
show/descriptor {guetbl} ORDER h
if outputi(3) .lt. 2 then
        write/out " "
        mess/out 1 {PROID} "Descriptor has too few elements"
        mess/out 1 {PROID} "({guetbl},ORDER) - 2 needed !"
  goto error
endif
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
if {{guetbl},TBLCONTR(4)} .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered guess table is empty !"
        mess/out 1 {PROID} "({guetbl})"
  goto error
endif
!
statistics/table {guetbl} :ORDER >Null
define/local absord/i/1/2 {outputr(2)},{outputr(1)}
!
if absord(1) .eq. absord(2) then
        write/out " "
        mess/out 1 {PROID} "Highest abs.ord.no. ({absord(1)}) =="
        mess/out 1 {PROID} " lowest abs.ord.no. ({absord(2)}) !"
        mess/out 1 {PROID} " ({guetbl},:ORDER)"
  goto error
endif
!
!
!P5(wavcat):
!----------
if p5 .eq. "?" mess/out 2 {PROID} " "
define/parameter p5 ? ? -
  "        Enter line catalogue: "
define/local wavcat/c/1/60 "{p5}"
if m$index(wavcat,".tbl") .eq. 0 wavcat = wavcat //".tbl"
!
if m$existc(wavcat,":WAVE") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({wavcat})"
  goto error
elseif m$existc(wavcat,":WAVE") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({wavcat},:WAVE)"
  goto error
endif
!
inputi = absord(1)+1-absord(2)
define/local totordno/i/1/1 {inputi(1)}  ! also for below
inputi(1) = totordno*2
inputi(2) = {{wavcat},TBLCONTR(4)}
if inputi(2) .lt. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "Too few lines present in line catalogue !"
        mess/out 1 {PROID} "({inputi(2)} ; >= {inputi(1)} (2 per order) needed)"
  goto error
endif
!
!
!P6(degree):
!----------
define/parameter p6 3 ? -
  "        Enter fit degree for x-pixels and abs.orders: "
define/local x_ord_deg/i/1/1 {p6}
!
if x_ord_deg .gt. MAXDEGREE then
        write/out " "
        mess/out 1 {PROID} "Fit degree for x-pixels and abs.orders too high !"
        mess/out 1 {PROID} "({x_ord_deg} ; degree<={MAXDEGREE})"
  goto error
endif
!
inputi(1) = (x_ord_deg+1)**2
inputi(2) = {{wavtbl},TBLCONTR(4)}
if inputi(2) .le. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "Too few lines present in line table !"
        mess/out 1 {PROID} -
        "({inputi(2)} ; > {inputi(1)} needed - X^{x_ord_deg},ORDER^{x_ord_deg})"
  goto error
endif
!
!
!P7(disp):
!--------
define/parameter p7 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p7}"
!
!
!          DEFINITION OF FURTHER VARIABLES :
!          *******************************
!
define/local ctxstart1/c/1/28 "====== BEGIN ECHELLE CONTEXT" all
define/local ctxstart2/c/1/50 "=" all
define/local ctxend1/c/1/26   "====== END ECHELLE CONTEXT" all
define/local ctxend2/c/1/52   "=" all
define/local pszmin/d/1/1 0.
!
!
!
!          MANAGING DISPLAYS / ITT / LUT :
!          *****************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_dids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 1 is missing !"
    goto error
  endif
  if nice_gids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 1 is missing !"
    goto error
  endif
  if nice_gids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 2 is missing !"
    goto error
  endif
  !
  !assign/display d,1 >Null
  !clear/display
  !display/lut off
  !label/display "LAMP" {nice_dlab}
  !if nice_dids(2) .eq. 1 then
  !  assign/display d,2 >Null
  !  clear/display
  !  display/lut off
  !endif
  if nice_dids(3) .eq. 1 then
    assign/display d,3 >Null
    clear/display
    display/lut off
  endif
  assign/graphic g,1 >Null
  clear/graphic
  assign/graphic g,2 >Null
  clear/graphic
  set/graphic default
  set/graphic pmode=0
  !
endif
!
!
!          PREPARATION OF LINE TABLE :
!          *************************
!
        mess/out 4 {PROID} "  preparing output table"
!
-copy {wavtbl} {DISPTBL}
if m$existc(DISPTBL,":ORDER") .eq. -1 then     ! "wav" is correct - not "gue" !
  create/column {DISPTBL} :ORDER " " F5.0 R*4  ! I->F f. neciden.prg !!!
  compute/table {DISPTBL} :ORDER = {absord(1)}+1 - :Y
endif
!
!
if nice_crea .eq. "y" then
  !
  !
  !        DISPLAYING OF LAMP FRAME :
  !        ************************
  !
  assign/display d,1 >Null
  show/channel 0 >Null
  if idimemc .eq. lampima then
    !
        mess/out 4 {PROID} "  lamp frame already loaded"
  else
        mess/out 4 {PROID} "  displaying lamp frame"
    !
    !***********************************************
    @p fors_nicedisp {lampima} ? n 0,0 ? ? 0.1,0.1 n
    !***********************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of lamp frame failed !"
        mess/out 1 {PROID} "({lampima})"
      goto error
    endif
    !
        mess/out 4 {PROID} "    loading lamp image into display 1"
    assign/display d,1 >Null
    load/image {lampima} ? {outputi(1)},avg c,c
    label/display "LAMP" {nice_dlab}  ! removed by special load/image
    !
  endif
  !
endif
!
!
!          Ident_1: COMPARISON OF LINE TABLE WITH 1ST GUESS SOLUTION,
!          Ident_2: IDENTIFICATION OF LINES VIA LINE CATALOGUE,
!          Coeff_1: FITTING OF IDENTIFICATIONS => 2D DISP.COEF.,
!          Coeff_2: CONVERTING 2D TO 1D DISP.COEF. => DISP.COEF. PER ORDER,
!          Coeff_3: COPYING OF MINIMAL PIXEL SIZE FROM GUESS TABLE,
!          Coeff_4: STORING OF ORDER INFORMATIONS AND FIT VALUES IN DESCS. :
!          ***************************************************************
!
        mess/out 3 {PROID} "  comparing line table with 1st guess solution,"
        mess/out 3 {PROID} "  identifying lines via line catalogue,"
        mess/out 3 {PROID} "  2D fitting of identified lines,"
        mess/out 3 {PROID} "  computing 1D dispersion coefficients:"
!
!====== BEGIN CONTEXT =========================================================
!
        write/out " "
        write/out "{ctxstart1} {ctxstart2}"
!
show/context echelle
if outputi(5) .eq. 0 then
        mess/out 2 {PROID} "  Echelle context is not set - setting now ..."
  set/context echelle >Null
  warn = 1
endif
!
!------------------------------------------------------------------------------
!
write/keyw wlc/c/1/80 "{lampima}"         ! new lamp frame
!
write/keyw lintab/c/1/60 "{DISPTBL}"      ! new line table
!
! Necessary for "@s neciden":
! --------------------------
write/keyw guess/c/1/60 "GUESS"           ! prfx.f.cp. lintbl.w.1st-gue.di.co.
  outputc = guess//"LINE.tbl"             ! "LINE" literal
  if guetbl .ne. outputc -copy {guetbl} {outputc}
!
! Only because of following bugfix:
! --------------------------------
  if wavcat .ne. "LINECATAL.tbl" -copy {wavcat} LINECATAL.tbl
write/keyw lincat/c/1/80 "LINECATAL.tbl"  ! line catalogue
  !
  ! Following lines because of bug in "@s neciden":
  create/row {lincat} @0 3
  create/row {lincat} @{{lincat},TBLCONTR(4)} 3
  {lincat},:WAVE,@1 = {{lincat},:WAVE,@4}/8.
  {lincat},:WAVE,@2 = {{lincat},:WAVE,@4}/4.
  {lincat},:WAVE,@3 = {{lincat},:WAVE,@4}/2.
  {lincat},:ION,@1 = "BgFx"
  {lincat},:ION,@2 = "BgFx"
  {lincat},:ION,@3 = "BgFx"
  inputi(3) = {{lincat},TBLCONTR(4)}-3
  inputi(2) = {{lincat},TBLCONTR(4)}-2
  inputi(1) = {{lincat},TBLCONTR(4)}-1
  {lincat},:WAVE,@{inputi(2)} = {{lincat},:WAVE,@{inputi(3)}}*2.
  {lincat},:WAVE,@{inputi(1)} = {{lincat},:WAVE,@{inputi(3)}}*4.
  {lincat},:WAVE,@{{lincat},TBLCONTR(4)} = {{lincat},:WAVE,@{inputi(2)}}*8.
  {lincat},:ION,@{inputi(2)} = "BgFx"
  {lincat},:ION,@{inputi(1)} = "BgFx"
  {lincat},:ION,@{{lincat},TBLCONTR(4)} = "BgFx"
!
!------------------------------------------------------------------------------
!
! For verify/echelle:
! ------------------
write/keyw  imsize/i/1/2     {{wlc},NPIX(1)},{{wlc},NPIX(2)}  ! i.(1) f.id.loop
write/keyw  scan/i/1/2       1,{{wlc},NPIX(2)}
!
! Wavelength calibration:
! ----------------------
write/keyw  wlcmtd/c/1/10    "guess"      ! cal.meth.[,xpix-shift] (DEF.:pair)
write/keyw  wlcopt/c/1/60    "2D"         ! "* ord." (2D) or per ord. (1D,DEF.)
write/keyw  echord/i/1/6     {totordno}   ! # of orders
write/keyw  echord/i/2/1     {absord(2)}  ! 1st abs.ord.no.
write/keyw  echord/i/3/1     {absord(1)}  ! last abs.ord.no.
write/keyw  echord/i/4/3     0,0,0        ! [filled later]
write/keyw  dc/i/1/1         {x_ord_deg}  ! degree for fit (DEF.:4)
write/keyw  ccdbin/r/1/2     1.,1.        ! x,y bin.fact. (DEF.:1.,1.)
!
! Identification loop:
! -------------------
write/keyw  wlcreg/c/1/12    "standard"   ! regress.meth.: std.(DEF.),robust(HT)
write/keyw  wlciter/r/1/5    1.           ! ini.acc.[px] (DEF.:1.).
!                                         !   ### IF<=(Sig.1stDisRel) =>INQ/KEY!
write/keyw  wlciter/r/2/1    0.3          ! next neighb. (>0.;DEF.:0.3);
!                                         !   ">=1.": everyth.is identified.
!                                         !   IF*(Cal-CatNxt)<=(Cal-Cat) =>NoId!
write/keyw  wlciter/r/3/1    20.          ! max.err.[px] (DEF.:20.).
!                                         !   ### IF<(AnySig.) =>LOOP-ABORT!
write/keyw  wlciter/r/4/2    -99.,-99.    ! [not used]
write/keyw  wlcniter/i/1/2   2,10         ! min.,max. # of iterat. (DEF.:3,20)
!
! Rejection loop:
! --------------
write/keyw  tol/r/1/1        1.           ! "* (pix.sz.[angs.])" (DEF.:0.2[px]);
!                                         !   "0.": auto.3sigma(residuals).
!                                         !   ### IF<=(Sig.1stEchRel) =>INQ/KEY!
!
! Output:
! ------
write/keyw  sessoutv/c/1/24  ">>Null"     ! cr/col,sel/t,reg/p,sta/t,cor/l,loa/t
write/keyw  outmode/c/1/10   "silent"     ! show params.: silent,normal(DEF.)
write/keyw  wlcvisu/c/1/3    "no"         ! disp.wlc+ident.+resid.: no,yes(DEF.)
!
!
!*********
@s neciden
!*********
!
!
-delete dat.dat middumm* nectab.tbl
!
        write/out "{ctxend1} {ctxend2}"
        write/out " "
!
!====== END CONTEXT ===========================================================
!
!
!          VISUALIZATION OF IDENTIFIED LINES :
!          *********************************
!
if nice_crea .eq. "y" then
  !
  select/table {DISPTBL} :IDENT.ne.0. >Null
  !
  assign/display d,1 >Null
  clear/channel overlay
  label/display "LAMP" {nice_dlab}
  load/table {DISPTBL} :X :YNEW ? 101 3 8
  load/table {DISPTBL} :X :YNEW ? 4 2 5
  label/display "Ident. lines" 10,10 ? cyan 1
  !
  !*******************************************
  @p fors_echwave_reg_plt {extimg} {DISPTBL} i
  !*******************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Plotting of wavelengths / residuals failed !"
        mess/out 1 {PROID} "({DISPTBL})"
    goto error
  endif
  !
  select/table {DISPTBL} all
  !
endif
!
!
!          Coeff_3: RE-CALCULATION OF PIXEL SIZES (IN CONTEXT ABOVE MIN.
!                   PIXEL SIZE IS ONLY COPIED FROM GUESS TABLE DESC.),
!          Coeff_4: STORING OF MINIMAL PIXEL SIZE IN DESC. :
!          *************************************************************
!
        mess/out 3 {PROID} "  calculating pixel sizes"
!
!*****************************************
@p fors_echwave_reg_psz {extimg} {DISPTBL}
!*****************************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Calculation of pixel sizes failed !"
        mess/out 1 {PROID} "({DISPTBL})"
  goto error
endif
pszmin = {q2}
!
if m$existd(DISPTBL,"PIXEL") .eq. 1 -
  {DISPTBL},PIXEL(1) = {pszmin}  ! only for further usage of the context
!
!
if warn .eq. 0 then
  return 0 {DISPTBL} {pszmin}
else
  return -1 {DISPTBL} {pszmin}
endif
!
error:
  set/format
  write/out " "
  return 1 -99 -99
!
!
!------------------------------------------------------------------------------
