! $Id: fors_echwave_gue.prg,v 1.6 2001/04/06 16:19:24 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_echwave_gue.prg,v $
!
! .KEYWORDS   Dispersion coefficients
!
! .PURPOSE    Use the output table from "fors_echwave_ide.prg" (MANUAL line
!             identifications) for the following computations:
!               0. Make a copy of the table for the modifications.
!               1. Apply a two-dimensional fit to the identified lines
!                  for calculation of the dispersion coefficients.
!               2. Convert the 2D to 1D coefficients (i.e., per order)
!                  and calculate the start and end wavelenghts per order.
!               3. Calculate the pixel sizes from the 1D coefficients.
!               4. Store order informations, fit values and the minimal
!                  pixel size in descriptors destined for this purpose.
!                  These descriptors are later used by "fors_echwave_dis.prg"
!                  for the re-computation of dispersion coefficients of a
!                  new lamp frame.
!             The calculated wavelengths, residuals (MANUALLY identified
!             minus calculated wavelengths) and pixel sizes can be plotted.
!             SEE .COMMENT !!!
! __________
!
! .USAGE      @p fors_echwave_gue extimg idetbl absord [degrees] [disp]
!
! .INPUT      extimg  (P1): order frame (extracted from the lamp frame);
!                             default: none
!
!             idetbl  (P2): output table from "fors_echwave_ide.prg";
!                             default: none
!
!             absord  (P3): highest(1st) and lowest(2nd) absolute order number,
!                           separated by a comma;
!                             default: none
!
!             degrees (P4): degrees for fitting x-pixels and absolute orders,
!                           separated by a comma;
!                             default: 3,3
!
!             disp    (P5): display selection:<p>:pipeline mode:read global key
!                           NICE_CREA [y|n], <n>:none, <l>:local, or
!                           host:display[.screen] - with the latter option
!                           you can specify the name or IP-address of a display;
!                             default: <l>
!
! .OUTPUT     Q2: table with order/fit/pix.size infos: "*[_ide->_gue].tbl"
!
!             Q3: minimal pixel size [angstroem]
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  1. 2D fitting: Polynomial regression with entered degrees.
!             2. 2D->1D conversion: Per order insertion of absolute orders into
!                polynome and devision of result by absolute order.
!             3. Pixel sizes: Using 1D dispersion coefficients and number of
!                x-pixels.
!
! .COMMENT    THIS PROCEDURE SHOULD NOT BE USED WITHIN THE PIPELINE -
!             SEE .PURPOSE !!!
!
!             AFTER APPLICATION OF THIS PROCEDURE:
!               Store the output table as 1st guess reference
!               for the dispersion coefficients (in CalibDb) !
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/10/15  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref extimg idetbl absord degrees disp
!
!
define/local MAXDEGREE/i/1/1 6     ! ecaxtly 7 elem.p.ord. in desc. COEFD re-
!                                  ! served - DON'T CHANGE THE "6" !!!
define/local A2MICRO/r/1/1 0.0001  ! angstroem -> micrometer
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "echwave_gue"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
write/keyword q3 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      extimg  (P1): order frame (extracted from the lamp frame);"
  w/o "                      default: none"
  w/o " "
  w/o "      idetbl  (P2): output table from "fors_echwave_ide.prg";"
  w/o "                      default: none"
  w/o " "
  w/o "      absord  (P3): highest(1st) and lowest(2nd) absolute order number,"
  w/o "                    separated by a comma;"
  w/o "                      default: none"
  w/o " "
  w/o "      degrees (P4): degrees for fitting x-pixels and absolute orders,"
  w/o "                    separated by a comma;"
  w/o "                      default: 3,3"
  w/o " "
  w/o "      disp    (P5): display selection:<p>:pipeline mode:read global key"
  w/o "                    NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                    host:display[.screen] - with the latter option"
  w/o "                    you can specify the name or IP-address of a display;"
  w/o "                      default: <l>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(extimg):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of order frame: "
define/local extimg/c/1/64 "{p1}"
if m$index(extimg,".bdf") .eq. 0 extimg = extimg//".bdf"
!
if m$exist(extimg) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({extimg})"
  goto error
endif
!
!
!P2(idetbl):
!----------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter line table: "
define/local idetbl/c/1/60 "{p2}"
if m$index(idetbl,".tbl") .eq. 0 idetbl = idetbl //".tbl"
!
if m$indexb(idetbl,"_ide") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Table is no output from "fors_echwave_ide.prg" -"
        mess/out 1 {PROID} "missing suffix "_ide.tbl" !"
  goto error
endif
!
if m$existc(idetbl,":X") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({idetbl})"
  goto error
elseif m$existc(idetbl,":X") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({idetbl},:X)"
  goto error
elseif m$existc(idetbl,":Y") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({idetbl},:Y)"
  goto error
elseif m$existc(idetbl,":IDENT") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({idetbl},:IDENT)"
  goto error
elseif m$existc(idetbl,":YNEW") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({idetbl},:YNEW)"
  goto error
endif
!
if {{idetbl},TBLCONTR(4)} .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered line table is empty !"
        mess/out 1 {PROID} "({idetbl})"
  goto error
endif
!
!
!P3(absord):
!----------
if p3 .eq. "?" mess/out 2 {PROID} " "
define/parameter p3 ? ? -
  "        Enter highest,lowest absolute order number: "
outputi = m$parse(p3,"order")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P3 !"
  goto error
endif
define/local absord/i/1/2 {order01},{order02}
!
if absord(1) .le. absord(2) then
        write/out " "
        mess/out 1 {PROID} "Highest abs.ord.no. ({absord(1)}) <="
        mess/out 1 {PROID} " lowest abs.ord.no. ({absord(2)}) !"
  goto error
endif
!
inputi = absord(1)+1-absord(2)
define/local totordno/i/1/1 {inputi(1)}  ! for below
!
!
!P4(degrees):
!-----------
define/parameter p4 3,3 ? -
  "        Enter fit degree for x-pixels,abs.orders: "
outputi = m$parse(p4,"degree")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P4 !"
  goto error
endif
define/local xdeg/i/1/1 {degree01}
define/local orderdeg/i/1/1 {degree02}
!
if xdeg .gt. MAXDEGREE then
        write/out " "
        mess/out 1 {PROID} "Fit degree for x-pixels too high !"
        mess/out 1 {PROID} "({xdeg} ; x-degree<={MAXDEGREE})"
  goto error
endif
if orderdeg .gt. MAXDEGREE then
        write/out " "
        mess/out 1 {PROID} "Fit degree for abs.orders too high !"
        mess/out 1 {PROID} "({orderdeg} ; order-degree<={MAXDEGREE})"
  goto error
endif
!
inputi = xdeg+1
define/local totcoefno/i/1/1 {inputi(1)}  ! for below
inputi = (xdeg+1)*(orderdeg+1)
define/local totelemno/i/1/1 {inputi(1)}  ! for below
!
inputi = {{idetbl},TBLCONTR(4)}
if inputi(1) .le. totelemno then
        write/out " "
        mess/out 1 {PROID} "Too few lines present !"
        mess/out 1 {PROID} -
          "({inputi(1)} ; > {totelemno} needed - X^{xdeg},ORDER^{orderdeg})"
  goto error
endif
!
!
!P5(disp):
!--------
define/parameter p5 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p5}"
!
!
!          DEFINITION OF FURTHER VARIABLES :
!          *******************************
!
!   "Preparation" :
define/local char/i/1/1 0
define/local guetbl/c/1/64 " " all
!
!   "Fitting" :
define/local regc/c/1/20         " " all +lower_levels  ! if (xdeg+1)*(o.d.+1)
define/local regi/i/1/13          0  all +lower_levels  !   >20, outputd does
define/local regr/r/1/5           0. all +lower_levels  !   not work for regr/
define/local regd/d/1/{totelemno} 0. all +lower_levels  !   poly => {totelemno}
!
!   "Converting" :
define/local pixstart/i/1/1 0
define/local pixend/i/1/1 0
define/local pixcent/i/1/1 0
define/local wldstart/r/1/1 0.
define/local wldend/r/1/1 0.
define/local wldcent/r/1/1 0.
define/local microloop/i/1/1 0
define/local relord/i/1/1 0
define/local absordloop/i/1/1 0
define/local xdegloop/i/1/1 0
define/local dstart/i/1/1 0
define/local dend/i/1/1 0
define/local dkeyloop/i/1/1 0
define/local orderdegloop/i/1/1 0
define/local dsum/d/1/{totcoefno} 0. all
define/local element/i/1/1 0
define/local lambstart/d/1/1 0.
define/local lambcent/d/1/1 0.
define/local lambend/d/1/1 0.
!
!   "Pixel sizes" :
define/local pszmin/d/1/1 0.
!
!
!
!          MANAGING DISPLAYS / ITT / LUT :
!          *****************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_gids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 1 is missing !"
    goto error
  endif
  if nice_gids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 2 is missing !"
    goto error
  endif
  !
  if nice_dids(1) .eq. 1 then
    assign/display d,1 >Null
    clear/display
    display/lut off
  endif
  if nice_dids(2) .eq. 1 then
    assign/display d,2 >Null
    clear/display
    display/lut off
  endif
  if nice_dids(3) .eq. 1 then
    assign/display d,3 >Null
    clear/display
    display/lut off
  endif
  assign/graphic g,1 >Null
  clear/graphic
  assign/graphic g,2 >Null
  clear/graphic
  set/graphic default
  set/graphic pmode=0
  !
endif
!
!
!          PREPARATION OF OUTPUT TABLE :
!          ***************************
!
        mess/out 4 {PROID} "  preparing output table"
!
  char = m$indexb(idetbl,"_ide")-1
  guetbl = idetbl(1:{char})//"_gue.tbl"
-copy {idetbl} tmp.tbl
select/table tmp.tbl :IDENT.ne.Null >Null
copy/table tmp.tbl {guetbl}
-delete tmp.tbl
create/column {guetbl} :ORDER    " "        F5.0  R*4  ! I->F f. "_dis.prg" !!!
create/column {guetbl} :AUX      " "        F12.4 R*8  !\
create/column {guetbl} :WAVEC    "angstrom" F8.2  R*8  ! > "*8" necessary
create/column {guetbl} :RESIDUAL "angstrom" F8.2  R*8  !/     f. "_dis.prg" !!!
!
!
!          FITTING OF IDENTIFICATIONS => 2D DISP.COEF. :
!          *******************************************
!
        mess/out 3 {PROID} "  2D fitting of initially identified lines"
!
compute/table {guetbl} :ORDER = {absord(1)}+1 - :Y
!
! FOR ORIGINAL IMAGE:
regression/poly {guetbl} :ORDER :X,:YNEW {xdeg},{orderdeg} reg >Null
save/regression {guetbl} RORD reg
!
! FOR EXTRACTED IMAGE:
compute/table {guetbl} :AUX = :ORDER*:IDENT
regression/poly {guetbl} :AUX :X,:ORDER {xdeg},{orderdeg} reg >Null
save/regression {guetbl} REGR reg
compute/regression {guetbl} :WAVEC = REGR
compute/table {guetbl} :WAVEC = :WAVEC/:ORDER
compute/table {guetbl} :RESIDUAL = :WAVEC-:IDENT
delete/column {guetbl} :AUX
!
if nice_crea .eq. "y" then
  !
  !******************************************
  @p fors_echwave_reg_plt {extimg} {guetbl} c
  !******************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Plotting of wavelengths / residuals failed !"
        mess/out 1 {PROID} "({guetbl})"
    goto error
  endif
  !
endif
!
!
!          CONVERTING 2D TO 1D DISP.COEF. => DISP.COEF. PER ORDER :
!          ******************************************************
!
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "  computing 1D dispersion coefficients"
!
pixstart = 1
pixend = {{extimg},NPIX(1)}
pixcent = m$nint((pixstart+pixend)/2.)
wldstart = {{extimg},START(1)}
wldend = wldstart+{{extimg},STEP(1)}*(pixend-1)
wldcent = (wldstart+wldend)/2.
!
inputi = (MAXDEGREE+1)*totordno
write/descriptor {guetbl} COEFD/d/1/{inputi(1)} 0. all
!
do microloop = 1 totelemno                       ! angstroem -> micrometer
  regd({microloop}) = regd({microloop})*A2MICRO  ! (needed for pixel size calc.
enddo                                            ! below and for rebinning)
!
! -------------------------------------------------------------
! regression/poly line.tbl :I :X,:O {p},{q} reg
! fits the function
! i = r(1)           +r(2)          *x +... +r(p+1)        *x^p
!    +r(1+(p+1))*o   +r(2+(p+1))*o  *x +... +r(2*(p+1))*o  *x^p
!    +...
!    +r(1+(p+1))*o^q +r(2+(p+1))*o^q*x +... +r(2*(p+1))*o^q*x^p
! as explained in the following table (r=regd):
!
!        | x^0          x^1          ... x^p
!    - - + - - - - - - - - - - - - - - - - - - - - - - -
!    o^0 | r(1)         r(2)         ... r(p+1)
!
!    o^1 | r(1+(p+1))   r(2+(p+1))   ... r(2*(p+1))
!
!    ... |
!
!    o^q | r(1+q*(p+1)) r(2+q*(p+1)) ... r((q+1)*(p+1))
! -------------------------------------------------------------
!
relord = 0
do absordloop = absord(1) absord(2) -1
  !
        set/format J2
        mess/out 3 {PROID} "    abs.ord. {absordloop}:"
        set/format {FMTDEF}
  !
  relord = relord+1
  do xdegloop = 0 xdeg
    dstart = 1+xdegloop
    dend = dstart+orderdeg*totcoefno
    orderdegloop = -1
    dsum({dstart}) = 0
    do dkeyloop = dstart dend totcoefno  ! index of elements in "regd"
      orderdegloop = orderdegloop+1
      dsum({dstart}) = dsum({dstart}) -
                       +regd({dkeyloop})*(absordloop**orderdegloop)
    enddo
    dsum({dstart}) = dsum({dstart})/absordloop
    element = (relord-1)*(MAXDEGREE+1)+dstart
    {guetbl},COEFD({element}) = {dsum({dstart})}
    inputr(1) = xdegloop
    !
        set/format F1.0,T17.11
        inputd = dsum({dstart})/A2MICRO
        mess/out 3 {PROID} "      disp.coef.(x^{inputr(1)})={inputd(1)}"
        set/format {FMTDEF}
    !
  enddo
  lambstart = 0.
  lambcent = 0.
  lambend = 0.
  do xdegloop = 0 xdeg
    dstart = 1+xdegloop
    lambstart = lambstart +dsum({dstart})/A2MICRO*(wldstart**xdegloop)
    lambcent = lambcent +dsum({dstart})/A2MICRO*(wldcent**xdegloop)
    lambend = lambend +dsum({dstart})/A2MICRO*(wldend**xdegloop)
  enddo
  !
  if lambstart .lt. 0. then
        mess/out 2 {PROID} -
          "  Bad fit (abs.order {absordloop}: lowest wavel. <0.) !"
  endif
  !
  inputr(1) = pixstart
  inputr(2) = pixcent
  inputr(3) = pixend
  !
        set/format F4.0,T8.2
        mess/out 3 {PROID} -
"      @{inputr(1)}={lambstart}  @{inputr(2)}={lambcent}  @{inputr(3)}={lambend}"
        set/format {FMTDEF}
  !
enddo
!
!
!          CALCULATION OF PIXEL SIZES :
!          **************************
!
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "  calculating pixel sizes"
!
!****************************************
@p fors_echwave_reg_psz {extimg} {guetbl}
!****************************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Calculation of pixel sizes failed !"
        mess/out 1 {PROID} "({guetbl})"
  goto error
endif
pszmin = {q2}
!
!
!          FURTHER NECESSARY OUTPUT :
!          ************************
!
        mess/out 3 {PROID} " "
        mess/out 4 {PROID} "  updating descriptors of output table"
!
write/descriptor {guetbl} COEFS/i/1/2 {xdeg},{totordno}
write/descriptor {guetbl} COEFI/i/1/{totordno} {totcoefno} all
write/descriptor {guetbl} ORDER/i/1/2 {absord(1)},{absord(2)}
write/descriptor {guetbl} PIXEL/r/1/2 {pszmin},-99.  ! pszmin in angstroem
!
!
if warn .eq. 0 then
  return 0 {guetbl} {pszmin}
else
  return -1 {guetbl} {pszmin}
endif
!
error:
  set/format
  write/out " "
  return 1 -99 -99
!
!
!------------------------------------------------------------------------------
