! $Id: fors_echwave_ide.prg,v 1.3 2001/04/06 16:19:25 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_echwave_ide.prg,v $
!
! .KEYWORDS   Line identification
!
! .PURPOSE    Store manually identified wavelengths - SEE .COMMENT !!!
! __________
!
! .USAGE      @p fors_echwave_ide lintbl idemod
!
! .INPUT      lintbl (P1): output table from "fors_echwave_sea.prg";
!                            default: none
!
!             idemod (P2): mode flag: either <0>:creation of new table template
!                          or <1>:processing of manually filled template;
!                            default: none
!
! .OUTPUT     Q2: table with identified lines: "<P1>[_wav->_ide].tbl"
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    THIS PROCEDURE MUST NEITHER BE USED WITHIN THE PIPELINE NOR
!             SHOULD IT BE INVOKED DIRECTLY - IT'S PURPOSE IS THE FOLLOWING:
!
!               1. Use "fors_echwave.prg" with mode "manu0" :
!                    Then this procedure is invoked with P2=0,
!                    it uses the output table from "fors_echwave_sea.prg",
!                    it copies it to "_ide.tbl",
!                    --------------------------------------------------------
!                    >>> in a programme break you have to insert MANUALLY <<<
!                    >>> identified wavelengths into a text file.         <<<
!                    --------------------------------------------------------
!               2. Afterwards use "fors_echwave.prg" with mode "manu1" :
!                    Then this procedure is invoked with P2=1,
!                    it stores the manually identified wavelenghts in a table
!                    suitable for the further procedures,
!                    these further procedures then calculate the dispersion
!                    coefficients.
!               3. Store the output table from "fors_echwave.prg"
!                    as 1st guess reference for the dispersion coeffi-
!                    cients (in CalibDb) !
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/10/13  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref lintbl idemod
!
!
!  For `mess/out':
define/local PROID/c/1/40 "echwave_ide"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      lintbl (P1): output table from "fors_echwave_sea.prg";"
  w/o "                     default: none"
  w/o " "
  w/o "      idemod (P2): mode flag: either <0>:creation of new table template"
  w/o "                   or <1>:processing of manually filled template;"
  w/o "                     default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          INPUT :
!          *****
!
!P1(lintbl):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter line table: "
define/local lintbl/c/1/60 "{p1}"
if m$index(lintbl,".tbl") .eq. 0 lintbl = lintbl //".tbl"
!
if m$indexb(lintbl,"_wav") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Table is no output from "fors_echwave_sea.prg" -"
        mess/out 1 {PROID} "missing suffix "_wav.tbl" !"
  goto error
endif
!
if m$existc(lintbl,":X") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lintbl})"
  goto error
elseif m$existc(lintbl,":X") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({lintbl},:X)"
  goto error
elseif m$existc(lintbl,":Y") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({lintbl},:Y)"
  goto error
elseif m$existc(lintbl,":IDENT") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({lintbl},:IDENT)"
  goto error
elseif m$existc(lintbl,":YNEW") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({lintbl},:YNEW)"
  goto error
endif
!
if {{lintbl},TBLCONTR(4)} .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered line table is empty !"
        mess/out 1 {PROID} "({lintbl})"
  goto error
endif
!
!
!P2(idemod):
!----------
define/parameter p2 ? ? -
  "        Enter mode (0:create/1:process): "
define/local idemod/c/1/1 "{p2}"
!
if idemod .ne. "0" .and. idemod .ne. "1" then
        write/out " "
        mess/out 1 {PROID} "Undefined programme mode - "{idemod}" -"
        mess/out 1 {PROID} "enter "0"(creation) or "1"(processing) !"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
define/local char/i/1/1 0
define/local idetbl/c/1/64 " " all
define/local idetxt/c/1/64 " " all
define/local width/i/1/1 79
define/local formcode/i/1/2 0 all
!
!
char = m$indexb(lintbl,"_wav")-1
idetbl = lintbl(1:{char})//"_ide.tbl"
char = m$indexb(lintbl,".")-1
idetxt = idetbl(1:{char})//".txt"
!
!
!
if idemod .eq. "0" then
  !
  !
  !        MANUAL LINE IDENTIFICATION AND INPUT :
  !        ************************************
  !
  !
        mess/out 3 {PROID} "  copying line table and converting to text file"
        mess/out 4 {PROID} "    ({lintbl} -> {idetxt})"
  !
  -copy {lintbl} {idetbl}
  !
  assign/print file {idetxt}
  print/table {idetbl} {width}
  assign/print terminal
  !
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "  Now edit  {idetxt}"
        mess/out 3 {PROID} "    and insert as many manually identified wave-"
        mess/out 3 {PROID} "    lenghts as possible somewhere in the IDENT"
        mess/out 3 {PROID} "    column (Y=extracted frame, YNEW=lamp frame) !"
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "  Afterwards compute the dispersion coefficients"
        mess/out 3 {PROID} "    by using <fors_echwave.prg> with mode "manu1" !"
        mess/out 3 {PROID} " "
  !
  return/exit  ! DON'T REMOVE !!!
  !
else
  !
  if m$exist(idetxt) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Edited ascii file is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({idetxt})"
    goto error
  endif
  !
  !        STORAGE OF MANUALLY FOUND LINES :
  !        *******************************
  !
        mess/out 3 {PROID} "  storing line identifications"
  !
  -copy {idetxt} tmp.dat
  $ wc -l tmp.dat |cut -f 1 -d "t" | write/keyword outputi/i/1/1
  -delete Mid{mid$sess(11:12)}1Pipe
  -delete midlis{mid$sess(11:12)}.prg
  $ sed "1,3d;{outputi(1)}d" tmp.dat >{idetxt}
  -delete tmp.dat
  !
  !
  !        CONVERSION TO MIDAS TABLE :
  !        *************************
  !
        mess/out 4 {PROID} "  converting to MIDAS table"
  !
  open/file dico.fmt write formcode
  write/file {formcode(1)} define/field 11 20 R*4 F10.2 :X "World Coord"
  write/file {formcode(1)} define/field 22 31 R*4 F10.2 :Y "PIXEL"
  write/file {formcode(1)} define/field 33 44 R*4 E12.3 :PEAK "DN"
  write/file {formcode(1)} define/field 46 56 R*8 F11.4 :IDENT " "
  write/file {formcode(1)} define/field 58 67 R*4 F10.2 :YNEW "PIXEL"
  close/file {formcode(1)}
  !
  create/table {idetbl} {{lintbl},TBLCONTR(1)} {{lintbl},TBLCONTR(2)} -
    {idetxt} dico.fmt
  copy/dd {lintbl} *,3 {idetbl}
  -delete dico.fmt
  !
  !
endif
!
!
if warn .eq. 0 then
  return 0 {idetbl}
else
  return -1 {idetbl}
endif
!
error:
  write/out " "
  return 1 -99
!
!
!------------------------------------------------------------------------------
