! $Id: fors_echwave_reg_plt.prg,v 1.8 1999/11/25 14:05:50 nrainer Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_echwave_reg_plt.prg,v $
!
! .KEYWORDS   Special image / table plotting
!
! .PURPOSE    Plots per order: Fitted wavelengths and residuals.
! __________
!
! .USAGE      @p fors_echwave_reg_plt extimg regresstbl [lambdaflag]
!
! .INPUT      extimg     (P1): order frame (extracted from the lamp frame);
!                                default: none
!
!             regresstbl (P2): output table either from "fors_echwave_gue.prg"
!                              or from "fors_echwave_dis.prg";
!                                default: none
!
!             lambdaflag (P3): flag for wavelength column: <c>:use calculated
!                              lines (:WAVEC), <i>:use identif. lines (:IDENT);
!                                default: <c>
!
! .OUTPUT     -
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for line identif. - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/10/14  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref extimg regresstbl lambdaflag
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "echwave_reg_plt"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      extimg     (P1): order frame (extracted from the lamp frame);"
  w/o "                         default: none"
  w/o " "
  w/o "      regresstbl (P2): output table either from "fors_echwave_gue.prg" "
  w/o "                       or from "fors_echwave_dis.prg";"
  w/o "                         default: none"
  w/o " "
  w/o "      lambdaflag (P3): flag for wavelength column: <c>:use calculated"
  w/o "                       lines (:WAVEC), <i>:use identif. lines (:IDENT);"
  w/o "                         default: <c>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(extimg):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of order frame: "
define/local extimg/c/1/64 "{p1}"
if m$index(extimg,".bdf") .eq. 0 extimg = extimg//".bdf"
!
if m$exist(extimg) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({extimg})"
  goto error
endif
!
!
!P2(regresstbl):
!--------------
define/parameter p2 ? ? -
  "        Enter regression table: "
define/local regresstbl/c/1/60 "{p2}"
if m$index(regresstbl,".tbl") .eq. 0 regresstbl = regresstbl //".tbl"
!
if m$existc(regresstbl,":X") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({regresstbl})"
  goto error
elseif m$existc(regresstbl,":X") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({regresstbl},:X)"
  goto error
elseif m$existc(regresstbl,":Y") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({regresstbl},:Y)"
  goto error
elseif m$existc(regresstbl,":RESIDUAL") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({regresstbl},:RESIDUAL)"
  goto error
endif
!
statistics/table {regresstbl} :Y >Null
define/local totordno/i/1/1 {outputr(2)}  ! for below
!
!
!P3(lambdaflag):
!--------------
define/parameter p3 c ? -
  "        Enter flag for line col. [def.:<c>alc.:WAVEC|<i>:IDENT]: "
define/local lambdaflag/c/1/1 "{p3}"
if lambdaflag .eq. "i" then
  define/local lambdacol/c/1/20 "IDENT"
else
  define/local lambdacol/c/1/20 "WAVEC"
endif
!
if m$existc(regresstbl,":{lambdacol}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({regresstbl},:{lambdacol})"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
define/local ordloop/i/1/1 0
!
!
!
!          PREPARING TABLE :
!          ***************
!
!if lambdaflag .eq. "i" then
!  select/table {regresstbl} :IDENT.ne.0. >Null
!  copy/table {regresstbl} ident.tbl
!  regresstbl = "ident.tbl"
!endif
!
!
!          PLOTTING FITTED LINES INTO
!          EXTRACTED SPECTRUM (PER ORDER) :
!          ******************************
!
        mess/out 4 {PROID} "  plotting fitted wavelengths per order"
!
assign/graphic g,1 >Null
set/graphic default
set/graphic pmode=0
set/graphic yformat=F5.2
do ordloop = 1 totordno
  select/table {regresstbl} :Y.eq.{ordloop} >Null
  plot/row {extimg} @{ordloop}
  set/graphic colour=4
  if ordloop .eq. 1 then
    !***************
    @p fors_nicegrap
    !***************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of plot failed !"
      goto error
    endif
  endif
  if lambdaflag .eq. "c" then
    label/graphic "{lambdacol} - rel.ord. {ordloop}" {nice_glabw}
  else
    label/graphic "{lambdacol}-r.ord.{ordloop} (0.:not.ident.)" {nice_glabw}
  endif
  !
  p1 = "{regresstbl}"
  p2 = ":X"            ! x-position
  p3 = ":{lambdacol}"  ! label with identification
  p4 = "bottom"
  pcount(1) = 4
  !***********************
  run STD_EXE:overiden.exe
  !***********************
  plcdata(1:60) = "{regresstbl}"
  plcdata(61:80) = "table"
  copy/graphic
  !
  set/graphic colour=1
enddo
select/table {regresstbl} all
!
!
!          PLOTTING RESIDUALS OF
!          FIT (PER ORDER) :
!          *********************
!
        mess/out 4 {PROID} "  plotting residuals of fit"
!
!if lambdaflag .eq. "c" then
  select/table {regresstbl} :IDENT.ne.0. >Null
  copy/table {regresstbl} ident.tbl
  regresstbl = "ident.tbl"
        mess/out 4 {PROID} "  (only from identified lines)"
!endif
!
!assign/graphic g,1 >Null
set/graphic default
set/graphic pmode=0 colour=8
plot/table {regresstbl} :{lambdacol} :RESIDUAL
set/graphic yformat=F5.2 colour=1
plot/axes {plrgrap(1)},{plrgrap(2)} {plrgrap(5)},{plrgrap(6)} ? -
  "{lambdacol} (angstrom)" "RESID. (WAVEC-IDENT)"
set/graphic colour=4
!***************
@p fors_nicegrap
!***************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of plot failed !"
  goto error
endif
label/graphic "Rel. orders" {nice_glabw}
do ordloop = 1 totordno
  inputi = 49+ordloop  ! "49": ascii code
  set/graphic stype={inputi(1)}
  select/table {regresstbl} :Y.eq.{ordloop} >Null
  overplot/table {regresstbl} :{lambdacol} :RESIDUAL
enddo
select/table {regresstbl} all
!
!
-delete ident.tbl
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!------------------------------------------------------------------------------
