! $Id: fors_echwave_reg_psz.prg,v 1.6 1999/11/25 11:24:14 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_echwave_reg_psz.prg,v $
!
! .KEYWORDS   Pixel sizes in Angstroem
!
! .PURPOSE    Calculate pixel sizes. They can be plotted per order.
! __________
!
! .USAGE      @p fors_echwave_reg_psz extimg regresstbl
!
! .INPUT      extimg     (P1): order frame (extracted from the lamp frame);
!                                default: none
!
!             regresstbl (P2): output table either from "fors_echwave_gue.prg"
!                              or from "fors_echwave_dis.prg";
!                                default: none
!
! .OUTPUT     Q2: minimal pixel size [angstroem]
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Using 1D dispersion coefficients and number of x-pixels.
!
! .COMMENT    Special proc. f. calc. of disp.coef. - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/10/15  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref extimg regresstbl
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "echwave_reg_psz"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      extimg     (P1): order frame (extracted from the lamp frame);"
  w/o "                         default: none"
  w/o " "
  w/o "      regresstbl (P2): output table either from "fors_echwave_gue.prg" "
  w/o "                       or from "fors_echwave_dis.prg";"
  w/o "                         default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(extimg):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of order frame: "
define/local extimg/c/1/64 "{p1}"
if m$index(extimg,".bdf") .eq. 0 extimg = extimg//".bdf"
!
if m$exist(extimg) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({extimg})"
  goto error
endif
!
!
!P2(regresstbl):
!--------------
define/parameter p2 ? ? -
  "        Enter regression table: "
define/local regresstbl/c/1/60 "{p2}"
if m$index(regresstbl,".tbl") .eq. 0 regresstbl = regresstbl //".tbl"
!
if m$existc(regresstbl,":Y") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({regresstbl})"
  goto error
elseif m$existc(regresstbl,":Y") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({regresstbl},:Y)"
  goto error
endif
if m$existd(regresstbl,"COEFD") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Descriptor with 1D disp.coeff.s is missing !"
        mess/out 1 {PROID} "({regresstbl},COEFD)"
  goto error
endif
!
statistics/table {regresstbl} :Y >Null
define/local totordno/i/1/1 {outputr(2)}  ! for below
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
define/local imsize/i/1/1 0         ! variable name literal
define/local lintab/c/1/60 " " all  ! variable name literal
define/local pszmin/d/1/1 0.
!
!
!
!          CALCULATION (AND PLOTTING) OF PIXEL SIZES :
!          *****************************************
!
imsize = {{extimg},NPIX(1)}
lintab = "{regresstbl}"
inputi(1) = 1
inputi(2) = totordno
inputi(3) = 0  ! no plot
!
if nice_crea .eq. "y" then
        mess/out 4 {PROID} "  plotting pixel sizes"
  set/graphic default
  set/graphic pmode=0
  assign/graphic g,2
  inputi(3) = 1  ! plot
endif
!
write/key outputd 0. all
!***********************
run STD_EXE:necompix.exe
!***********************
if outputd(1) .le. 0. then
        write/out " "
        mess/out 1 {PROID} "Fit failed (min. pix.size <=0.) !"
  goto error
endif
if outputd(2) .le. 0. then
        write/out " "
        mess/out 1 {PROID} "Fit failed (max. pix.size <=0.) !"
  goto error
endif
outputd(3) = (outputd(1)+outputd(2))/2.  ! avg. pix.size
        set/format ,T7.4
        mess/out 3 {PROID} "    pix.size:    min.   | avg.   | max."
        mess/out 3 {PROID} "                 -------+--------+-------"
        mess/out 3 {PROID} -
          "                {outputd(1)} |{outputd(3)} |{outputd(2)}   angs."
        set/format {FMTDEF}
pszmin = outputd(1)  ! min. instead of avg. better for further calculations
!
if nice_crea .eq. "y" then
  set/graphic colour=4
  select/table {regresstbl} :IDENT.ne.0. >Null
  statistics/table {regresstbl} :IDENT >Null
  overplot/line 5 {outputr(1)},{plrgrap(5)} {outputr(1)},{plrgrap(6)}
  overplot/line 5 {outputr(2)},{plrgrap(5)} {outputr(2)},{plrgrap(6)}
  !***************
  @p fors_nicegrap
  !***************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of plot failed !"
    goto error
  endif
  label/graphic "dash: ident.range" {nice_glabw}
  select/table {regresstbl} all
  -delete compixel.plt
  set/graphic default
endif
!
!
if warn .eq. 0 then
  return 0 {pszmin}
else
  return -1 {pszmin}
endif
!
error:
  set/format
  return 1 -99
!
!
!------------------------------------------------------------------------------
