! $Id: fors_echwave_sea.prg,v 1.5 2001/04/06 16:19:24 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_echwave_sea.prg,v $
!
! .KEYWORDS   Line search
!
! .PURPOSE    Search for emission lines on a frame with orders (extracted from
!             a lamp frame).
!             The found lines can be displayed.
! __________
!
! .USAGE      @p fors_echwave_sea lampima [extima] [ordtbl] [thresh] [disp]
!
! .INPUT      lampima (P1): lamp frame;
!                             default: none
!
!             extima  (P2): order frame (extracted from the lamp frame);
!                             default: <P1>_ext.bdf
!
!             ordtbl  (P3): output table from "fors_findorder.prg";
!                             default: <ORDDEF> (see below)
!
!             thresh  (P4): approx. line width in dispersion direction [pixels,
!                           >0.] AND detecting threshold above(!) local back-
!                           ground [counts,>0.], separated by a comma;
!                             default: 12.,10.
!
!             disp    (P5): display selection:<p>:pipeline mode:read global key
!                           NICE_CREA [y|n], <n>:none, <l>:local, or
!                           host:display[.screen] - with the latter option
!                           you can specify the name or IP-address of a display;
!                             default: <l>
!
! .OUTPUT     Q2: line table "<P1>_wav.tbl"
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  1. Detection: In a running window (height: 1 pixel, width: P4[1])
!                intensities are sorted to form a local histogramme. An emission
!                line is found when the central value in the window is greater
!                than the median value minus the local background plus P4[2].
!             2. Centering: A gaussian fit is centred to the line.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/08/04  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref lampima extima ordtbl thresh disp
!
!
define/local ORDDEF/c/1/20 "order.tbl"
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "echwave_sea"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      lampima (P1): lamp frame;"
  w/o "                      default: none"
  w/o " "
  w/o "      extima  (P2): order frame (extracted from the lamp frame);"
  w/o "                      default: <P1>_ext.bdf"
  w/o " "
  w/o "      ordtbl  (P3): output table from "fors_findorder.prg";"
  w/o "                      default: {ORDDEF}"
  w/o " "
  w/o "      thresh  (P4): approx. line width in dispersion direction [pixels,"
  w/o "                    >0.] AND detecting threshold above(!) local back-"
  w/o "                    ground [counts,>0.], separated by a comma;"
  w/o "                      default: 12.,10."
  w/o " "
  w/o "      disp    (P5): display selection:<p>:pipeline mode:read global key"
  w/o "                    NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                    host:display[.screen] - with the latter option"
  w/o "                    you can specify the name or IP-address of a display;"
  w/o "                      default: <l>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(lampima):
!-----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of lamp frame: "
define/local lampima/c/1/64 "{p1}"
if m$index(lampima,".bdf") .eq. 0 lampima = lampima//".bdf"
!
if m$exist(lampima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lampima})"
  goto error
endif
!
!
!P2(extima):
!----------
define/local char/i/1/1 0
define/local extima/c/1/64 " " all
char = m$indexb(lampima,".")-1
extima = lampima(1:{char})//"_ext.bdf"
define/parameter p2 {extima} i -
  "        Enter name of order frame: "
extima = p2
if m$index(extima,".bdf") .eq. 0 extima = extima//".bdf"
!
if m$exist(extima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({extima})"
  goto error
endif
!
define/local xpix/i/1/1 {{extima},NPIX(1)}
statistics/image {extima} + ? ? mn
define/local intmax/r/1/1 {outputr(2)}
!
!
!P3(ordtbl):
!----------
define/parameter p3 {ORDDEF} ? -
  "        Enter order positions table [def.:{ORDDEF}]: "
define/local ordtbl/c/1/60 "{p3}"
if m$index(ordtbl,".tbl") .eq. 0 ordtbl = ordtbl //".tbl"
!
if m$exist(ordtbl) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({ordtbl})"
  goto error
endif
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
outputi = 0
if m$existd(ordtbl,"REGRESSC") .eq. 0 outputi = 1
if m$existd(ordtbl,"REGRESSI") .eq. 0 outputi = 1
if m$existd(ordtbl,"REGRESSR") .eq. 0 outputi = 1
if m$existd(ordtbl,"REGRESSD") .eq. 0 outputi = 1
if outputi(1) .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({ordtbl},REGRESS[ C/I/R/D ])"
  goto error
endif
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
show/table {ordtbl} >Null
if outputi(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({ordtbl})"
  goto error
endif
!
!
!P4(thresh):
!----------
define/parameter p4 12.,10. ? -
  "        Enter line width,detect.thresh. [def.:12.,10.]: "
outputi = m$parse(p4,"thresh")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P4 !"
  goto error
endif
define/local linwid/r/1/1 {thresh01}
define/local level/r/1/1 {thresh02}
!
define/local threshbad/i/1/1 0
inputi(1) = m$nint(xpix/2.-1.-0.5)
if linwid .le. 0. .or. linwid .gt. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "Line width out of range !"
        mess/out 1 {PROID} "({linwid} ; 1.<=line_width<<{inputi(1)}.)"
  threshbad = 1
endif
if level .le. 0. then
        set/format F7.1
        write/out " "
        mess/out 1 {PROID} "Detection threshold <= 0. !"
        mess/out 1 {PROID} "({level} ; 1.<=det.thresh.<<{intmax})"
        set/format {FMTDEF}
  threshbad = 1
endif
if level .ge. intmax then
        set/format F7.1
        write/out " "
        mess/out 1 {PROID} "Detection threshold >= max.intens.(order frame) !"
        mess/out 1 {PROID} "({level} ; 1.<=det.thresh.<<{intmax})"
        set/format {FMTDEF}
  threshbad = 1
endif
if threshbad .eq. 1 goto error
!
!
!P5(disp):
!--------
define/parameter p5 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p5}"
!
!
!          DEFINITION OF FURTHER VARIABLES :
!          *******************************
!
define/local lintbl/c/1/64 " " all
!
!
!
!          MANAGING DISPLAYS / ITT / LUT :
!          *****************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_dids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 1 is missing !"
    goto error
  endif
  if nice_dids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 2 is missing !"
    goto error
  endif
  !
  !assign/display d,1 >Null
  !clear/display
  !display/lut off
  !label/display "LAMP" {nice_dlab}
  !assign/display d,2 >Null
  !clear/display
  !display/lut off
  !label/display "EXTRACTED" {nice_dlab}
  if nice_dids(3) .eq. 1 then
    assign/display d,3 >Null
    clear/display
    display/lut off
  endif
  load/lut heat
  load/itt ramp
  if nice_gids(1) .eq. 1 then
    assign/graphic g,1 >Null
    clear/graphic
  endif
  if nice_gids(2) .eq. 1 then
    assign/graphic g,2 >Null
    clear/graphic
  endif
  !
endif
!
!
if nice_crea .eq. "y" then
  !
  !
  !        DISPLAYING OF LAMP FRAME :
  !        ************************
  !
  assign/display d,1 >Null
  show/channel 0 >Null
  if idimemc .eq. lampima then
    !
        mess/out 4 {PROID} "  lamp frame already loaded"
  else
        mess/out 4 {PROID} "  displaying lamp frame"
    !
    !***********************************************
    @p fors_nicedisp {lampima} ? n 0,0 ? ? 0.1,0.1 n
    !***********************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of lamp frame failed !"
        mess/out 1 {PROID} "({lampima})"
      goto error
    endif
    !
        mess/out 4 {PROID} "    loading lamp image into display 1"
    load/image {lampima} ? {outputi(1)},avg c,c
    label/display "LAMP" {nice_dlab}  ! removed by special load/image
    !
  endif
  !
endif
!
!
if nice_crea .eq. "y" then
  !
  !
  !        DISPLAYING OF EXTRACTED FRAME :
  !        *****************************
  !
  assign/display d,2 >Null
  show/channel 0 >Null
  if idimemc .eq. extima then
    !
        mess/out 4 {PROID} "  extracted frame already loaded"
  else
        mess/out 4 {PROID} "  displaying extracted frame"
    !
    !*********************************************
    @p fors_nicedisp {extima} ? n 0,0 ? ? 0.,0.2 n
    !*********************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of extracted frame failed !"
        mess/out 1 {PROID} "({extima})"
      goto error
    endif
    !
    ! MAGNIFY IMAGE IN ORDER DIRECTION:
    inputr(2) = {{extima},NPIX(2)}
    inputi(2) = nice_dscr(2)/2  ! use only half the height of the display
    if inputr(2) .ge. inputi(2) then
      outputi(2) = -m$nint((inputr(2)-1.)/(inputi(2)*2.)+0.5)
    else
      outputi(2) = m$nint((inputi(2)*2.)/inputr(2)-0.5)
    endif
    if outputi(2) .le. -1 .or. outputi(2) .ge. 2 then
      {extima},DISPLAY_DATA(2) = -1*{outputi(2)}
    else  ! 0 not possible, -1 equivalent to 1
      {extima},DISPLAY_DATA(2) = 1
    endif
    !
        mess/out 4 {PROID} "    loading extracted image into display 2"
    assign/display d,2 >Null
    load/image {extima} ? {outputi(1)},{outputi(2)} c,c  !### 2)},max not psbl.
                                                         !### bec. of Mid. error
    label/display "EXTRACTED" {nice_dlab}  ! removed by special load/image
    label/display "(y-averaged)" 10,10 ? green 1
    !
  endif
  !
endif
!
!
!          DETECTING AND CENTERING EMISSION LINES :
!          **************************************
!
        mess/out 4 {PROID} "  detecting / centering emission lines"
!
char = m$indexb(lampima,".")-1
lintbl = lampima(1:{char})//"_wav.tbl"
!
p1 = extima
write/keyword inputr {linwid},{level},0.  ! "0.": order averaging already done
p3 = lintbl
p4 = "gaussian"
p5 = "emission"
action = "xx"
!
!******************************
run STD_EXE:spesearch.exe >Null
!******************************
delete/descriptor {lintbl} PIXEL
!
        mess/out 3 {PROID} "    found  < {{lintbl},TBLCONTR(4)} >  lines"
!
if nice_crea .eq. "y" then
  !
  assign/display d,2 >Null
  clear/channel overlay
  label/display "EXTRACTED" {nice_dlab}
  label/display "(y-averaged)" 10,10 ? green 1
  load/table {lintbl} :X :Y ? 101 3 8
  load/table {lintbl} :X :Y ? 4 2 5
  !
endif
!
!
!          PREPARING LINE IDENTIFICATION :
!          *****************************
!
create/column {lintbl} :IDENT " " F11.4 R*8
!
!
!          COMPUTING LINE POSITIONS
!          FOR NOT EXTRACTED FRAME :
!          ************************
!
create/column {lintbl} :YNEW "PIXEL" F10.2 R*4
copy/dd {ordtbl} REGRESS* {lintbl}
{lintbl},REGRESSI(4) = 1  ! # of 1st independent column (:X)
{lintbl},REGRESSI(5) = 2  ! # of 2nd independent column (:Y = order)
compute/regression {lintbl} :YNEW = REGRESS
!
if nice_crea .eq. "y" then
  !
  assign/display d,1 >Null
  clear/channel overlay
  label/display "LAMP" {nice_dlab}
  load/table {lintbl} :X :YNEW ? 101 3 8
  load/table {lintbl} :X :YNEW ? 4 2 5
  !
endif
!
!
if warn .eq. 0 then
  return 0 {lintbl}
else
  return -1 {lintbl}
endif
!
error:
  set/format
  return 1 -99
!
!
!------------------------------------------------------------------------------
