! $Id: fors_echwave_sel.prg,v 1.3 1999/11/25 11:24:15 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_echwave_sel.prg,v $
!
! .KEYWORDS   Master line catalogue
!
! .PURPOSE    Select wavelengths that match the used grism combination and used
!             lamps.
! __________
!
! .USAGE      @p fors_echwave_sel lampima linecat
!
! .INPUT      lampima (P1): lamp frame;
!                             default: none
!
!             linecat (P5): table with laboratory wavelengths;
!                             default: none
!
! .OUTPUT     Q2: extracted 2D line catalogue for further usage: "<P2>_sel.tbl"
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for echelle - not to be used generally.
!
! .ENVIRON    FORS context
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.5  99/11/02  Creation
!                  99/       Release
!
!------------------------------------------------------------------------------
!
!
crossref lampima linecat
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "echwave_sel"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      lampima (P1): lamp frame;"
  w/o "                      default: none"
  w/o " "
  w/o "      linecat (P5): table with laboratory wavelengths;"
  w/o "                      default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!
!P1(lampima):
!-----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of lamp frame: "
define/local lampima/c/1/64 "{p1}"
if m$index(lampima,".bdf") .eq. 0 lampima = lampima//".bdf"
!
if m$exist(lampima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lampima})"
  goto error
endif
if m$existd(lampima,h_grism2) .eq. 0 then
        mess/out 1 {PROID} "Descriptor w. ech. grism name missing !"
        mess/out 1 {PROID} "({lampima},{h_grism2}) !"
  goto error
endif
if m$existd(lampima,h_grism1) .eq. 0 then
        mess/out 1 {PROID} "Descriptor w. cross disperser grism name missing !"
        mess/out 1 {PROID} "({lampima},{h_grism1}) !"
  goto error
endif
!
!
!P2(linecat):
!-----------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? t -
  "        Enter line catalogue: "
define/local cat3d/c/1/64 "{p2}"
if m$index(cat3d,".tbl") .eq. 0 cat3d = cat3d //".tbl"
if m$existc(cat3d,":EGRISM") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({cat3d})"
  goto error
elseif m$existc(cat3d,":EGRISM") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({cat3d},:EGRISM)"
  goto error
elseif m$existc(cat3d,":XGRISM") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({cat3d},:XGRISM)"
  goto error
elseif m$existc(cat3d,":LAMP") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({cat3d},:LAMP)"
  goto error
elseif m$existc(cat3d,":WAVE") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({cat3d},:WAVE)"
  goto error
endif
set/refcol {cat3d} :LAMP
show/table {cat3d} >Null
set/format I3
inputc = "{{cat3d},TLABL{outputi(4)}}"
outputc = inputc(33:33)
set/format {FMTDEF}
if outputc .ne. "I" then
        write/out " "
        mess/out 1 {PROID} "Column :LAMP has wrong type !"
        mess/out 1 {PROID} "(<{outputc}> - must be <I>)"
  goto error
endif
set/refcol {cat3d} seq
!
!
!
!          DEFINITION OF FURTHER VARIABLES :
!          *******************************
!
define/local egrism/c/1/25  " " all
define/local xgrism/c/1/25  " " all
define/local tottypno/i/1/1 0
define/local lamploop/i/1/1 0
define/local typeloop/i/1/1 0
define/local cat2d/c/1/76   " " all
!
!
!
!          SELECTION OF GRISM COMBINATION (3D->2D table) :
!          *********************************************
!
egrism = "{{lampima},{h_grism2}}"
xgrism = "{{lampima},{h_grism1}}"
!
!-------------------------------------------------------------------------
select/table {cat3d} :EGRISM.eq."{egrism}".and.:XGRISM.eq."{xgrism}" >Null
!-------------------------------------------------------------------------
!
if outputi .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "GRISM combination not in line catalogue !"
        mess/out 1 {PROID} "(EGRISM: {egrism}, XGRISM: {xgrism})"
  goto error
elseif outputi .gt. 1 then
        write/out " "
        mess/out 1 {PROID} -
          "GRISM combination  {outputi(1)} times  in line catalogue !"
        mess/out 1 {PROID} "(EGRISM: {egrism}, XGRISM: {xgrism})"
  goto error
endif
copy/table {cat3d} tmp3d.tbl
select/table {cat3d} all
!
create/table tmp2d.tbl 2 100 null
@p fors_t3dextract tmp3d.tbl :LAMP 1 tmp2d.tbl tmp +
  create/column    tmp2d.tbl :LAMP " "        I5   I*4
  copy/tt          tmp2d.tbl :tmp tmp2d.tbl :LAMP
  delete/column    tmp2d.tbl :tmp
@p fors_t3dextract tmp3d.tbl :WAVE 1 tmp2d.tbl tmp +
  create/column    tmp2d.tbl :WAVE "angstrom" F8.2 R*8
  copy/tt          tmp2d.tbl :tmp tmp2d.tbl :WAVE
  delete/column    tmp2d.tbl :tmp
-delete tmp3d.tbl
!
!
!          SELECTION OF LAMP(S) (rows are selected) :
!          ****************************************
!
select/table tmp2d.tbl all
select/table tmp2d.tbl .not.sel >Null
!
!*************************
@p fors_getlamps {lampima}
!*************************
if fors_nlamp .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No lamps in header !"
        mess/out 1 {PROID} "({lampima},{kbase_lampi}i{ksuff_lampi})"
  goto error
elseif fors_nlamp .eq. 1 then
        mess/out 4 {PROID} "Following lamp has been used:"
else
        mess/out 4 {PROID} "Following {fors_nlamp} lamps have been used:"
endif
tottypno = 0
do lamploop = 1 fors_nlamp
        mess/out 4 {PROID} "  {fors_lampi({lamploop})}"
  do typeloop = 1 FORS_LMAXTYP
    outputi = m$index("{fors_lampi({lamploop})}","{fors_ltypes({typeloop})}")
    if outputi .ne. 0 then
      tottypno = tottypno+1
      !
      !-----------------------------------------------------------------------
      !select/table tmp2d.tbl sel.or.:LAMP.eq."{fors_ltypes({typeloop})}" >Null
      ! NOT POSSIBLE DUE TO MIDAS ERROR: no 3D char. columns possible !!
      !
      select/table tmp2d.tbl sel.or.:LAMP.eq.{typeloop} >Null
      !-----------------------------------------------------------------------
      !
    endif
  enddo
enddo
if tottypno .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "None of the known lamp types"
  do typeloop = 1 FORS_LMAXTYP
        mess/out 1 {PROID} "  {fors_ltypes({typeloop})}"
  enddo
        mess/out 1 {PROID} "in header ({lampima},{kbase_lampi}i{ksuff_lampi}) !"
  goto error
endif
if {tmp2d.tbl,TBLCONTR(10)} .eq. 0 then
        write/out " "
  if fors_nlamp .eq. 1 then
        mess/out 1 {PROID} "Lamp type in header"
  else
        mess/out 1 {PROID} "None of the lamp types in header"
  endif
  do lamploop = 1 fors_nlamp
        mess/out 1 {PROID} "  {fors_lampi({lamploop})}"
  enddo
  if fors_nlamp .eq. 1 then
        mess/out 1 {PROID} "is not in line catalogue (as index):"
  else
        mess/out 1 {PROID} "are in line catalogue (as index):"
  endif
  do lamploop = 1 fors_nlamp
        mess/out 1 {PROID} "  {lamploop} = {fors_lampi({lamploop})}"
  enddo
  goto error
endif
!
inputi = m$indexb(cat3d,".")-1
cat2d = cat3d(1:{inputi})//"_sel.tbl"
copy/table tmp2d.tbl {cat2d}
-delete tmp2d.tbl
!
!
set/format
!
!
if warn .eq. 0 then
  return 0 {cat2d}
else
  return -1 {cat2d}
endif
!
error:
  set/format
  return 1 -99
!
!
!------------------------------------------------------------------------------
