! $Id: fors_expfini.prg,v 1.13 1999/11/25 11:23:52 sbogun Exp $   
!+++ FORS_EXPFINI +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_expfini.prg,v $
!
!.KEYWORDS      Determine initial parameter set for later use with fors_nonlfit
!
!.USAGE         @p fors_expfini image table partbl
!
!.INPUT         image   (P1): name of data image
!
!.OUTPUT        table   (P2): name of data table
!               partbl  (P3): name of parameter table
!
!               updated parameter and data table
!
!.RETURN        
!
!.DEFAULT
!
!.PURPOSE       Determine initial parameter set for later use with fors_nonlfit
!
!.ALGORITHM     nonlfit performs a non-linear least squares fit.
!               The functions to be fitted are user-defined functions.
!
!               Here the initialization shall be done for a special case:
!                          a*exp(-b*x) + c
!
!               The parameters a, b, and c will be determined by the first
!               guesses:
!
!               c = MEDIAN(y), a = y_min - c, b = -ln(D)/(x_max - x_min);
!               D = |(y_max - c)/(y_min - c)|
!
!               (x[],y[]) are the x, and y-coordintes of the data which have
!               to be fitted.
!               These parameters will be stored in a table which will be
!               created by this procedure. This parameter-table will be the
!               input to the call of nonlfit.
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.9 1998/02/04  Creation
!               1.0 1998/11/25  Add 1.0 to column :X (should start at 1.0)
!.COMMENT
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name of data image: "
define/param p2 ? c "Enter name of data table (will be created): "
define/param p3 ? c "Enter name of parameter table: "
!
! local constant
!
define/local EPS/r/1/1 1e-8
!
! local variable declaration
!
define/local num/i/1/1 0
define/local dd/r/1/1 0.0
define/local x1/r/1/1 0.0
define/local xn/r/1/1 0.0
define/local y1/r/1/1 0.0
define/local yn/r/1/1 0.0
define/local ym/r/1/1 0.0
define/local acc/r/1/1 0.001
!
! Create data table
!
copy/it {P1} {P2} :X
name/col {P2} #2 :Y
creat/col {P2} :SIG " "
creat/col {P2} :FIT " " 
creat/col {P2} :RESIDUAL " "
dd = {{P2},#1,1}
comp/tab {P2} :X = :X - {dd} + 1.0
comp/tab {P2} :SIG = 1.0
comp/tab {P2} :FIT = 0.0
comp/tab {P2} :RESIDUAL = 0.0

!
! Create parameter table
!
crea/tab {P3} 3,3
crea/col {P3} :PARAM " "
crea/col {P3} :FIXED " "
crea/col {P3} :RESULT " "

statistic/ima {P1} ? ? ? GN >Null
ym = outputr(8)                         !MEDIAN
y1 = outputr(1)                         !MINIMUM
yn = outputr(2)                         !MAXIMUM
inputr = m$abs(y1 - yn)
if inputr(1) .lt. EPS then
  {P3},:PARAM,3 = ym
  {P3},:PARAM,1 = 0.0
  {P3},:PARAM,2 = 1e10
else
  x1 = m$value({P2},:X,{outputi(2)})    !X_min
  xn = m$value({P2},:X,{outputi(3)})    !X_max
  y1 = m$value({P2},:Y,{outputi(2)})    !Y_min
  yn = m$value({P2},:Y,{outputi(3)})    !Y_max
endif

{P3},:PARAM,3 = ym
{P3},:PARAM,1 = y1 - ym
{P3},:PARAM,2 = -m$ln( m$abs(yn - ym) / (y1 - ym) ) / (xn - x1)

compute/tab {P3} :FIXED = 0.0           !no parameter is fixed by default
compute/tab {P3} :RESULT = 0.0
read/tab {P3}

write/keyw INPUTI/i/1/8 1,2,3,4,5,1,2,3
return 0

