! $Id: fors_extast.prg,v 1.7 1999/11/25 11:23:53 sbogun Exp $   
!+++ FORS_EXTAST ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_extast.prg,v $
!
!.KEYWORDS      Extract astronomy keywords (world coordinate system WCS)
!
!.USAGE         @p fors_extast image info
!
!.INPUT         image   (P1): name of image file
!               info    (P2): info flag, 1: show info, 0: no info
!
!.OUTPUT        OUTPUTD
!               OUTPUTD(1)
!               OUTPUTD(2)      coordinate description matrix (2x2 matrix)
!               OUTPUTD(3)
!               OUTPUTD(4)
!               OUTPUTD(5)      step size in RA
!               OUTPUTD(6)      step size in DEC
!               OUTPUTD(7)      reference pixel (x-coordinate)
!               OUTPUTD(8)      reference pixel (y-coordinate)
!               OUTPUTD(9)      RA and DEC ...
!               OUTPUTD(10)     at reference pixel
!               OUTPUTD(11)     longpole
!               OUTPUTD(12)     projection p1
!               OUTPUTD(13)     projection p2
!
!               OUTPUTC         coordinate type (CUNIT)
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT       P2: 0
!
!.PURPOSE       Extract astronomy keywords (world coordinate system WCS)
!
!.ALGORITHM     Extract astrometry parameters from a BDF image header.
!               The astrometry in the header can be in either CD (Coordinate
!               description) format, or CROTA and CDELT (AIPS-type) format.
!
!.ENVIRON       
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/12/01  Creation
!
!.COMMENT
!------------------------------------------------------------------------------
define/param P1 ? c "Enter name of BDF-image file:"
define/param P2 0 n "Enter 1 to get info-output: "


define/local RADEG/d/1/1 0.0

define/local longpole/d/1/1 -1.0
define/local projp1/d/1/1 -1.0
define/local projp2/d/1/1 -1.0
define/local cd/d/1/4 0.0 ALL
define/local cdelt/d/1/2 0.0 ALL
define/local crpix/d/1/2 0.0 ALL
define/local crval/d/1/2 0.0 ALL
define/local crota/d/1/2 0.0 ALL
define/local ctype/c/1/80 " " ALL
define/local projection/c/1/3 " " ALL


RADEG = 180.0D0/3.14159265358979323846

ctype = "{{P1},CUNIT}"

if m$existd(P1,"CRVAL") .gt. 0 then
  crval(1) = m$value({P1},CRVAL(1)) / RADEG
  crval(2) = m$value({P1},CRVAL(2)) / RADEG
else
  return -2
endif
  
if m$existd(P1,"REFPIX") .gt. 0 then
  crpix(1) = m$value({P1},REFPIX(1))
  crpix(2) = m$value({P1},REFPIX(2))
else
  return -3
endif


if m$existd(P1,"CD001001") .gt. 0 then
  CD(1) = m$value({P1},CD001001)                        !CD11
  CD(2) = m$value({P1},CD001002)                        !CD12
  CD(3) = m$value({P1},CD002001)                        !CD21
  CD(4) = m$value({P1},CD002002)                        !CD22

! Get CDELT values.   If no CDELT keywords found, then assume that they
! are incorporated in the CD00n00m values, and convert to CDn_m

  cdelt(1) = m$value({P1},STEP(1))
  cdelt(2) = m$value({P1},STEP(2))
else
  if m$existd(P1,"CD1_1") .gt. 0 then
    CD(1) = m$value({P1},CD1_1)                         !CD11
    CD(2) = m$value({P1},CD1_2)                         !CD12
    CD(3) = m$value({P1},CD2_1)                         !CD21
    CD(4) = m$value({P1},CD2_2)                         !CD22
!    cdelt(1) = 1.0
!    cdelt(2) = 1.0
    cdelt(1) = m$value({P1},STEP(1))
    cdelt(2) = m$value({P1},STEP(2))
  else

! Now get rotation, first try CROTA2, if not found try CROTA1, if that
! not found assume North-up.   Then convert to CD matrix
    cdelt(1) = m$value({P1},STEP(1))
    cdelt(2) = m$value({P1},STEP(2))

    if m$existd(P1,"ROTA") .gt. 0 then
      crota(1) = m$value({P1},ROTA(1)) / RADEG
      crota(2) = m$value({P1},ROTA(2)) / RADEG
    else
      crota(1) = 0.0
      crota(2) = 0.0
    endif
    CD(1) = m$cos(crota(1))                             !CD11
    CD(2) = m$sin(crota(1))                             !CD12
    CD(3) = -m$sin(crota(1))                            !CD21
    CD(4) = m$cos(crota(1))                             !CD22

  endif
endif
!
!     ( CD11    CD21 )
! cd =(              )
!     ( CD12    CD22 )
!

if m$index(ctype,"TAN") .gt. 0 then
  projection = "TAN"
  longpole = 180.0
  projp1 = -1.
  projp2 = -2.
else
  if m$existd(P1,"LONGPOLE") .gt. 0 then
    longpole = m$value({P1},LONGPOLE)
  else
    longpole = 180.0
  endif
  if m$existd(P1,"PROJP1") .gt. 0 projp1 = m$value({P1},PROJP1)
  if m$existd(P1,"PROJP2") .gt. 0 projp2 = m$value({P1},PROJP2)
endif

! Note that the dimensions and datatype of each tag must be explicit, so that
! there is no conflict with structure definitions from different FITS headers

outputd(1)  = cd(1)
outputd(2)  = cd(2)
outputd(3)  = cd(3)
outputd(4)  = cd(4)
outputd(5)  = cdelt(1)
outputd(6)  = cdelt(2)
outputd(7)  = crpix(1)
outputd(8)  = crpix(2)
outputd(9)  = crval(1)
outputd(10) = crval(2)
outputd(11) = longpole
outputd(12) = projp1
outputd(13) = projp2

outputc     = ctype

if {P2} .eq. 1 then
  write/out "Matrix:"
  write/out "CD = [[{outputd(1)},{outputd(2)}],[{outputd(3)},{outputd(4)}]]"
  write/out "Coordinate Step size (CRDELT):"
  write/out "({outputd(5)},{outputd(6)})"
  write/out "Reference pixel position (CRPIX):"
  write/out "({outputd(7)},{outputd(8)})"
  write/out "RA and DEC at ref pixel (CRVAL):"
  write/out "({outputd(9)},{outputd(10)})"
endif


return 0
