! $Id: fors_extorder.prg,v 1.9 2001/04/06 16:19:23 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_extorder.prg,v $
!
! .KEYWORDS   Order extraction
!
! .PURPOSE    Extract orders on a lamp frame.
!             The extracted orders can be displayed.
! __________
!
! .USAGE      @p fors_extorder lampima [ordtbl] [disp]
!
! .INPUT      lampima (P1): lamp frame;
!                             default: none
!
!             ordtbl  (P2): output table from "fors_findorder.prg";
!                             default: <ORDDEF> (see below)
!
!             disp    (P3): display selection:<p>:pipeline mode:read global key
!                           NICE_CREA [y|n], <n>:none, <l>:local, or
!                           host:display[.screen] - with the latter option
!                           you can specify the name or IP-address of a display;
!                             default: <l>
!
! .OUTPUT     Q2: frame with extracted orders "<P1>_ext.bdf"
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Pixel values within the orders (positions defined by REGRESS*
!             descriptors) are averaged perpendicular to the dispersion
!             direction. Per order a row in the output image is created.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/08/02  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref lampima ordtbl disp
!
!
define/local ORDDEF/c/1/20 "order.tbl"
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "extorder"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      lampima (P1): lamp frame;"
  w/o "                      default: none"
  w/o " "
  w/o "      ordtbl  (P2): output table from "fors_findorder.prg";"
  w/o "                      default: {ORDDEF}"
  w/o " "
  w/o "      disp    (P3): display selection:<p>:pipeline mode:read global key"
  w/o "                    NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                    host:display[.screen] - with the latter option"
  w/o "                    you can specify the name or IP-address of a display;"
  w/o "                      default: <l>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(lampima):
!-----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of lamp frame: "
define/local lampima/c/1/64 "{p1}"
if m$index(lampima,".bdf") .eq. 0 lampima = lampima//".bdf"
!
if m$exist(lampima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lampima})"
  goto error
endif
!
!
!P2(ordtbl):
!----------
define/parameter p2 {ORDDEF} ? -
  "        Enter order positions table [def.:{ORDDEF}]: "
define/local ordtbl/c/1/60 "{p2}"
if m$index(ordtbl,".tbl") .eq. 0 ordtbl = ordtbl //".tbl"
!
if m$exist(ordtbl) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({ordtbl})"
  goto error
endif
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
outputi = 0
if m$existd(ordtbl,"REGRESSC") .eq. 0 outputi = 1
if m$existd(ordtbl,"REGRESSI") .eq. 0 outputi = 1
if m$existd(ordtbl,"REGRESSR") .eq. 0 outputi = 1
if m$existd(ordtbl,"REGRESSD") .eq. 0 outputi = 1
if outputi(1) .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({ordtbl},REGRESS[ C/I/R/D ])"
  goto error
endif
if m$existd(ordtbl,"ORDWIDTH") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({ordtbl},ORDWIDTH)"
  goto error
endif
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
show/table {ordtbl} >Null
define/local ndata/i/1/1 {outputi(2)}  ! needed for below
if ndata .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({ordtbl})"
  goto error
endif
!
define/local ORDWID/i/1/1 {{ordtbl},ORDWIDTH}
!
!
!P3(disp):
!--------
define/parameter p3 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p3}"
!
!
!          DEFINITION OF FURTHER VARIABLES :
!          *******************************
!
define/local foundords/i/1/1 0
define/local ordloop/i/1/1 0
define/local char/i/1/1 0
define/local extima/c/1/64 " " all
define/local scry0/i/1/1 0
define/local frmy0/r/1/1 0.
define/local dscly/i/1/1 0
define/local scaly/r/1/1 0.
define/local labobjxl/i/1/1 0
define/local frmy/r/1/1 0.
define/local scry/i/1/1 0
!
!
!
!          MANAGING DISPLAYS / ITT / LUT :
!          *****************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_dids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 1 is missing !"
    goto error
  endif
  if nice_dids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 2 is missing !"
    goto error
  endif
  !
  assign/display d,1 >Null
  clear/display
  display/lut off
  label/display "LAMP" {nice_dlab}
  assign/display d,2 >Null
  clear/display
  display/lut off
  label/display "EXTRACTED" {nice_dlab}
  if nice_dids(3) .eq. 1 then
    assign/display d,3 >Null
    clear/display
    display/lut off
  endif
  load/lut heat
  load/itt ramp
  if nice_gids(1) .eq. 1 then
    assign/graphic g,1 >Null
    clear/graphic
  endif
  if nice_gids(2) .eq. 1 then
    assign/graphic g,2 >Null
    clear/graphic
  endif
  !
endif
!
!
if nice_crea .eq. "y" then
  !
  !
  !        DISPLAYING OF LAMP FRAME AND ORDERS :
  !        ***********************************
  !
        mess/out 4 {PROID} "  displaying lamp frame"
  !
  !***********************************************
  @p fors_nicedisp {lampima} ? n 0,0 ? ? 0.1,0.1 n
  !***********************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of lamp frame failed !"
        mess/out 1 {PROID} "({lampima})"
    goto error
  endif
  !
        mess/out 4 {PROID} "    loading lamp image into display 1"
  assign/display d,1 >Null
  load/image {lampima} ? {outputi(1)},avg c,c
  label/display "LAMP" {nice_dlab}  ! removed by special load/image
  !
  load/table {ordtbl} :X :YFIT ? 5 3 5
  sort/table {ordtbl} :ORDER,:X
  foundords = {{ordtbl},:ORDER,@{ndata}}
  do ordloop = 1 foundords
    select/table {ordtbl} :ORDER.eq.{ordloop} >Null
    load/table {ordtbl} :X :YFIT :ORDER 5 0 8
  enddo
  select/table {ordtbl} all
  !
endif
!
!
!          CREATING FRAME WITH Y-AVERAGED EXTRACTED ORDERS :
!          ***********************************************
!
        mess/out 4 {PROID} "  averaging pixels within orders"
!
    char = m$indexb(lampima,".")-1
    extima = lampima(1:{char})//"_ext.bdf"
!
in_a   = lampima
in_b   = ordtbl
write/keyword inputi 0,0              ! "0,0": use all orders
inputc = "REGRESS"                    ! basename of descs. in ordtbl.
define/local in_c/c/1/7 average       ! average the pixel values
write/keyword inputr {ORDWID}.,0.,0.  ! ",0,0": angle,offset
out_a  = extima
!
!**********************
run STD_EXE:necexor.exe
!**********************
!
        mess/out 3 {PROID} -
          "    extracted  < {{extima},NPIX(2)} >  averaged orders"
!
!
if nice_crea .eq. "y" then
  !
  !
  !        DISPLAYING OF EXTRACTED FRAME :
  !        *****************************
  !
        mess/out 4 {PROID} "  displaying extracted frame"
  !
  !*********************************************
  @p fors_nicedisp {extima} ? n 0,0 ? ? 0.,0.2 n
  !*********************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of extracted frame failed !"
        mess/out 1 {PROID} "({extima})"
    goto error
  endif
  !
  ! MAGNIFY IMAGE IN ORDER DIRECTION:
  ! --------------------------------
  inputr(2) = {{extima},NPIX(2)}
  inputi(2) = nice_dscr(2)/2  ! use only half the height of the display
  if inputr(2) .ge. inputi(2) then
    outputi(2) = -m$nint((inputr(2)-1.)/(inputi(2)*2.)+0.5)
  else
    outputi(2) = m$nint((inputi(2)*2.)/inputr(2)-0.5)
  endif
  if outputi(2) .le. -1 .or. outputi(2) .ge. 2 then
    {extima},DISPLAY_DATA(2) = -1*{outputi(2)}
  else  ! 0 not possible, -1 equivalent to 1
    {extima},DISPLAY_DATA(2) = 1
  endif
  !
        mess/out 4 {PROID} "    loading extracted image into display 2"
  assign/display d,2 >Null
  load/image {extima} ? {outputi(1)},{outputi(2)} c,c  !### 2)},max not possible
                                                       !    bec. of Midas error
  label/display "EXTRACTED" {nice_dlab}  ! removed by special load/image
  label/display "(y-averaged)" 10,10 ? green 1
  !
  ! LABELS WITH RELATIVE ORDER NUMBERS:
  ! ----------------------------------------------------------------
  ! scry - scry0 = nint[ (frmy - frmy0) /scaly ] ,
  !   with  scry0 = nice_dscr(2)     (screen pixel in window centre)
  !   and   frmy0 = DISPLAY_DATA(4)  (frame  pixel in window centre
  !                                   after `load/ima ... c,c')
  !   and   scaly =     DISPLAY_DATA(2), if positive
  !                 -1./DISPLAY_DATA(2), if negative
  ! ----------------------------------------------------------------
  scry0 = nice_dscr(2)
  frmy0 = {{extima},DISPLAY_DATA(4)}
  dscly = {{extima},DISPLAY_DATA(2)}
  if dscly .ge. 1 then  ! 1,2,3,...
    scaly = dscly
  else                  ! ...,-3,-2 ; 0 not possible, -1 doesn't exist
    scaly = -1./dscly
  endif
  labobjxl = 6
  do ordloop = 1 foundords
    frmy  = ordloop+0.5
    scry = m$nint((frmy-frmy0)/scaly-0.5)+scry0-7
    if ordloop .lt. 10 then
      label/display "  {ordloop}" {scry},{labobjxl} ? cyan 1
    else
      label/display "{ordloop}" {scry},{labobjxl} ? cyan 1
    endif
  enddo
  !
endif
!
!
if warn .eq. 0 then
  return 0 {extima}
else
  return -1 {extima}
endif
!
error:
  set/format
  return 1 -99
!
!
!------------------------------------------------------------------------------
