! $Id: fors_extract.prg,v 1.7 1999/11/25 11:23:53 sbogun Exp $   
!+++ FORS_EXTRACT +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_extract.prg,v $
!
!.KEYWORDS      Extract sub image from input image
!
!.USAGE         @p fors_extract in out coord
!
!.INPUT         in      (P1): name of input image
!               out     (P2): name of output image (sub image)
!                               . set to + in order to overwrite input frame.
!               coord   (P3): pixel coordinates
!                               . set to + in order to set coordinates to
!                                 image section of the input frame
!                                 (frame without overscan regions).
!                               . standard coordinate string may be passed
!                                 by a character keyword or directly in the
!                                 usual way: (e.g.: [@17,@1:@2064,@2048]).
!                               . integer array:
!                                 (1): lower left x-coord,
!                                 (2): lower left y-coord,
!                                 (3): upper right x-coord,
!                                 (4): upper right y-coord.
!
!.OUTPUT        out     (P2): extracted image
!
!.RETURN        Q1:  0: successful return
!                    else: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Extract sub-image from input image.
!
!.ALGORITHM     call fors_extract.exe
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/12/22  Creation
!                   1998/12/23  Refpix bug fixed
!
!.COMMENT       Does not change datatype in contrast to EXTRACT/IMAGE !
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name input image:"
define/param p2 ? c "Enter name output image:"
define/param p3 + c "Enter name of pixel coordinate array (integer):"
!
! local variable declaration
!
define/local procid/c/1/10 "extract"

define/local coord/c/1/100 " " ALL
define/local IMSEC/i/1/4 0 ALL
define/local over/i/1/1 0 
define/local npix/i/1/1 0 


IN_A = P1
OUT_A = P2
if OUT_A(1:1) .eq. "+" then                     !overwrite
  over = 1
  OUT_A = "middumma.bdf"
else                                            !do not overwrite
  over = 0
endif

coord = P3
info/key {coord}

if mid$info(1) .eq. 1 then                      !check if P3 is an integer
  copy/keyw {coord}/i/1/4 IMSEC/i/1/4           !--> copy to IMSEC.
else if coord(1:1) .eq. "+" then                !get image section
  image/section {P1}
  if {Q1} .ne. 0 return -1
  copy/keyw OUTPUTI/i/1/4 IMSEC/i/1/4
else if coord(1:1) .eq. "[" then                !P3 is coordinate string ?
  @p fors_extract,EXTRCOOR {coord}
  if {Q1} .lt. 0 then
    mess/out 1 {procid} "Bad coordinate string !"
    return -1
  endif
  copy/keyw outputi/i/1/4 IMSEC/i/1/4
else if m$index({coord},"[") .gt. 0 then        !P3 is a character keyword
  @p fors_extract,EXTRCOOR {{coord}}            !.. with coordinates
  if {Q1} .lt. 0 then
    mess/out 1 {procid} "Bad coordinate string !"
    return -1
  endif
  copy/keyw outputi/i/1/4 IMSEC/i/1/4
endif

npix = m$value({IN_A},NPIX(1))
if IMSEC(1) .lt. 1 .OR. IMSEC(1) .gt. npix then
  mess/out 1 {procid} "Bad subimage selected !"
  return -1
endif
if IMSEC(3) .lt. 1 .OR. IMSEC(3) .gt. npix then
  mess/out 1 {procid} "Bad subimage selected !"
  return -1
endif

npix = m$value({IN_A},NPIX(2))
if IMSEC(2) .lt. 1 .OR. IMSEC(2) .gt. npix then
  mess/out 1 {procid} "Bad subimage selected !"
  return -1
endif
if IMSEC(4) .lt. 1 .OR. IMSEC(4) .gt. npix then
  mess/out 1 {procid} "Bad subimage selected !"
  return -1
endif



set/format I1
append/out "Extract image [@{IMSEC(1)},@{IMSEC(2)}:@{IMSEC(3)},@{IMSEC(4)}]\n"
append/out "from: {p1}"
mess/out 4 {procid}
set/format


run PIPE_EXE:fors_extract.exe                   !extract image




if m$index("{{IN_A},CUNIT}","PIXEL") .gt. 0 then
                                                !PIXEL coordinates:
  @a getcoord {IN_A} @{IMSEC(1)},@{IMSEC(2)} >Null
  {OUT_A},START(1) = outputd(1)
  {OUT_A},START(2) = outputd(2)
  {OUT_A},REFPIX(1) = {{OUT_A},REFPIX(1)} - ( IMSEC(1) - 1 )
  {OUT_A},REFPIX(2) = {{OUT_A},REFPIX(2)} - ( IMSEC(2) - 1 )

else                                            !WORLD coordinates (RA,DEC)
                                                !START holds world coordinates
                                                !of first frame pixel (@1,@1)
  outputd(1) = m$value({IN_A},START(1))         !as a linear extrapolation
  outputd(2) = m$value({IN_A},START(2))         !from REFPIX world coordinates.
                                                !-> shift 'old' START by linear
                                                !transformation:
                                                !
  {OUT_A},START(1) = outputd(1) + m$value({IN_A},STEP(1))*(IMSEC(1)-1)
  {OUT_A},START(2) = outputd(2) + m$value({IN_A},STEP(2))*(IMSEC(2)-1)
                                                !
                                                !--> shift REFPIX
  {OUT_A},REFPIX(1) = {{OUT_A},REFPIX(1)} - ( IMSEC(1) - 1 )
  {OUT_A},REFPIX(2) = {{OUT_A},REFPIX(2)} - ( IMSEC(2) - 1 )

endif

if over .eq. 1 -rename middumma.bdf {IN_A}      !rename if desired.

return {OUTPUTI(1)}                             !return num of pix. in sub.ima.




!=== SUBROUTINE ===============================================================

ENTRY EXTRCOOR

define/param p1 ? c "Enter coordinate string:"

define/local i1/i/1/1 0
define/local len/i/1/1 0
define/local pos1/i/1/1 0
define/local pos2/i/1/1 0
define/local coord/c/1/100 " " ALL

if "{p1(1:1)}" .eq. "[" then
  coord = "{p1}"
endif

if "{p1(1:1)}" .eq. "[" then

  pos1 = m$index(coord,"@")
  if pos1 .le. 0 return -1              !no pixel coordinate string

  pos1 = pos1 + 1
  len = m$strlen(coord)

  pos1 = m$index(coord,"@") + 1
  pos2 = m$index(coord,",") - 1
  outputi(1) = {coord({pos1}:{pos2})}
  coord = coord({pos2}:)

  pos1 = m$index(coord,"@") + 1
  pos2 = m$index(coord,":") - 1
  outputi(2) = {coord({pos1}:{pos2})}
  coord = coord({pos2}:)

  pos1 = m$index(coord,"@") + 1
  pos2 = m$index(coord,",") - 1
  outputi(3) = {coord({pos1}:{pos2})}
  coord = coord({pos2}:)

  pos1 = m$index(coord,"@") + 1
  pos2 = m$index(coord,"]") - 1
  outputi(4) = {coord({pos1}:{pos2})}

  return 1
else
  return -1
endif
