! $Id: fors_filter_check.prg,v 1.10 1999/11/25 11:23:53 sbogun Exp $
!+++ FORS_FILTER_CHECK ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_filter_check.prg,v $
!
!.KEYWORDS      Check filter FITS-keywords
!
!.USAGE         @p fors_filter_check in_frm
!
!.INPUT         in_frm  (P1): name of the input frame to be examined
!
!.OUTPUT        OUTPUTI(1): number of filters found
!               OUTPUTI(2): identification number (0;,1,10,100,11,101,110,111)
!               OUTPUTC:    name of FIRST valid filter (== Q?)
!
!.RETURN        Q1, Q2, Q3:     Filter-keyword entries (at maximum 3 filters)
!
!.DEFAULT
!
!.PURPOSE       Check the existance of the filter FITS-keywords and their
!               contents.
!
!.ALGORITHM     Uses standard MIDAS commands.
!               Reads ESO.INS.FILT* keywords.
!               An appropriate message will be send to stdout telling how
!               many and which filters had been used for the exposure.
!
!.ENVIRON       FORS context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1998/06/18  Creation
!
!.COMMENT
!------------------------------------------------------------------------------


define/param p1 ? c "Enter name of frame to be examined: "

define/local fchk/i/1/1 0
define/local filt1/c/1/30 "none" 
define/local filt2/c/1/30 "none" 
define/local filt3/c/1/30 "none" 

if m$existd(P1,"{h_filti(4)}") .gt. 0 then  ! for TC
  filt1 = "{{P1},{h_filti(4)}}"
  fchk = 1
  goto outlab
else
  filt1 = "none"
  fchk = 0
endif
if m$existd(P1,"{h_filti(1)}") .gt. 0 then
  filt1 = "{{P1},{h_filti(1)}}"
  fchk = 1
else
  filt1 = "none"
  fchk = 0
endif
if m$existd(P1,"{h_filti(2)}") .gt. 0 then
  filt2 = "{{P1},{h_filti(2)}}"
  fchk = fchk + 10
else
  filt2 = "none"
endif
if m$existd(P1,"{h_filti(3)}") .gt. 0 then
  filt3 = "{{P1},{h_filti(3)}}"
  fchk = fchk + 100
else
  filt3 = "none"
endif


OUTLAB:
BRANCH fchk 0,1,10,100,11,101,110,111 NON,O1,O2,O3,T12,T13,T23,ALL
mess/out 2 filter_check "{fchk} could not be identified!"

NON:
  mess/out 2 filter_check "NO filters used or FILTER-keywords missing!"
  outputi(1) = 0
  write/key outputc " " ALL
  goto endbra
O1:
  mess/out 4 filter_check "One filter used in `{P1}` 1:`{filt1}`."
  outputi(1) = 1
  outputc = "{filt1}"
  goto endbra
O2:
  mess/out 4 filter_check "One filter used in `{P1}` 1:`{filt1}`."
  outputi(1) = 1
  outputc = "{filt2}"
  goto endbra
O3:
  mess/out 4 filter_check "One filter used in `{P1}` 1: `{filt1}`."
  outputi(1) = 1
  outputc = "{filt3}"
  goto endbra
T12:
  mess/out 4 filter_check "Two filters used in `{P1}`: 1:`{filt1}`, 2:`{filt2}`."
  outputi(1) = 2
  outputc = "{filt1}"
  goto endbra
T13:
  mess/out 4 filter_check "Two filters used in `{P1}`: 1:`{filt1}`, 3:`{filt3}`."
  outputi(1) = 2
  outputc = "{filt1}"
  goto endbra
T23:
  mess/out 4 filter_check "Two filters used in `{P1}`: 2:`{filt2}`, 3:`{filt3}`."
  outputi(1) = 2
  outputc = "{filt2}"
  goto endbra
ALL:
 mess/out 4 filter_check "All filters used in `{P1}` \n1:`{filt1}`\n2:`{filt2}`\n3:`{filt3}`"
  outputi(1) = 3
  outputc = "{filt1}"
endbra:

outputi(2) = fchk
return {filt1} {filt2} {filt3}

