! $Id: fors_findorder_bac.prg,v 1.2 1999/11/25 11:24:12 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_findorder_bac.prg,v $
!
! .KEYWORDS   Order detection
!
! .PURPOSE    Use the order table from "fors_findorder_fit.prg" for
!             computing background areas.
! __________
!
! .USAGE      @p fors_findorder_bac [ordtbl] [sizes] [step]
!
! .INPUT      ordtbl (P1): order table from "fors_findorder_fit.prg";
!                            default: <FITDEF> (see below)
!
!             sizes  (P2): no.of x-pixels of order reference frame AND
!                          no.of y-pixels of order reference frame,
!                          separated by a comma;
!                            default: 2048,2048
!
!             step   (P3): distance of chosen image columns for centres
!                          of background areas;
!                            default: 50
!
! .OUTPUT     Q2: table with chosen background areas
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for order detection - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/07/28  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref ordtbl sizes step
!
!
define/local FITDEF/c/1/20 "order.tbl"
define/local OUTTBL/c/1/20 "back.tbl"
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "findorder_bac"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      ordtbl (P1): order table from "fors_findorder_fit.prg";"
  w/o "                     default: {FITDEF}"
  w/o " "
  w/o "      sizes  (P2): no.of x-pixels of order reference frame AND"
  w/o "                   no.of y-pixels of order reference frame,"
  w/o "                   separated by a comma;"
  w/o "                     default: 2048,2048"
  w/o " "
  w/o "      step   (P3): distance of chosen image columns for centres"
  w/o "                   of background areas;"
  w/o "                     default: 50"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(ordtbl):
!----------
define/parameter p1 {FITDEF} ? -
  "        Enter order table [def.:{FITDEF}]: "
define/local ordtbl/c/1/60 "{p1}"
if m$index(ordtbl,".tbl") .eq. 0 ordtbl = ordtbl //".tbl"
!
if m$exist(ordtbl) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({ordtbl})"
  goto error
endif
show/table {ordtbl} >Null
if outputi(1) .lt. 2 then
        write/out " "
        mess/out 1 {PROID} "Entered table has less than 2 columns !"
        mess/out 1 {PROID} "({ordtbl})"
  goto error
endif
define/local ndata/i/1/1 {outputi(2)}
if ndata .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({ordtbl})"
  goto error
endif
!
!
!P2(sizes):
!---------
define/parameter p2 2048,2048 ? -
  "        Enter no.of x-pix.,y-pix. of ord.ref.frame [def.:2048,2048]: "
outputi = m$parse(p2,"pix")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local xsize/i/1/1 {pix01}
define/local ysize/i/1/1 {pix02}
!
define/local pixbad/i/1/1 0
if xsize .lt. 1 then
        write/out " "
        mess/out 1 {PROID} "Impossible number of x-pixels !"
  pixbad = 1
endif
if ysize .lt. 1 then
        write/out " "
        mess/out 1 {PROID} "Impossible number of y-pixels !"
  pixbad = 1
endif
if pixbad .eq. 1 goto error
!
!
!P3(step):
!--------
define/parameter p3 50 n -
  "        Enter chosen x-dist. of background centres [def.:50]: "
define/local COLDIST/i/1/1 {p3}
inputi(1) = m$nint(xsize/2.-1.-0.5)
if COLDIST .lt. 1 .or. COLDIST .gt. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "Column x-distance out of range !"
        mess/out 1 {PROID} "({COLDIST} ; 1<=col.dist.<={inputi(1)})"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
define/local ncol/i/1/1 0
define/local maxord/i/1/1 0
define/local alldata/i/1/1 0
define/local xwidth/i/1/1 0
!
!
!
!          DETERMINATION OF BACKGROUND AREAS :
!          *********************************
!
        mess/out 4 {PROID} "  re-fitting background areas"
!
! X-COORDINATES FOR EACH AREA:
! ---------------------------
ncol = xsize/COLDIST
sort/table {ordtbl} :ORDER,:X
maxord = {{ordtbl},:ORDER,@{ndata}} +1  ! "+1": back.areas = orders+1
alldata = ncol*maxord
create/table tmp.tbl 7 {alldata} null
create/column tmp.tbl :ORDER " " F5.1 R*4
create/column tmp.tbl :X "pixel" F7.1 R*4
create/column tmp.tbl :XSTA "pixel" F7.1 R*4
create/column tmp.tbl :XEND "pixel" F7.1 R*4
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! :ORDER is # of 2nd independent column in previous fit (stored in {ordtbl});
! the following "-0.5" shifts the zero points of the fit between the orders:
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
compute/table tmp.tbl :ORDER = int((SEQ-1)/{ncol}+0.5) -0.5
compute/table tmp.tbl :X = (SEQ -{ncol}*(:ORDER-0.5)) *{COLDIST} +1  ! "+1":
xwidth = m$nint(COLDIST/5+0.5)                                       ! equiv.to
compute/table tmp.tbl :XSTA = :X -{xwidth}                           ! ordertbl.
compute/table tmp.tbl :XEND = :X +{xwidth}
!
! FITTED Y-COORDINATES FOR EACH AREA:
! ----------------------------------
copy/dd {ordtbl} REGRESS* tmp.tbl
tmp.tbl,REGRESSI(4) = 2  ! # of 1st independent column (:X)
create/column tmp.tbl :YBKG "pixel" F7.1 R*4
compute/regression tmp.tbl :YBKG = REGRESS
tmp.tbl,REGRESSI(4) = 3  ! # of 1st independent column (:XSTA)
create/column tmp.tbl :YSTA "pixel" F7.1 R*4
compute/regression tmp.tbl :YSTA = REGRESS
tmp.tbl,REGRESSI(4) = 4  ! # of 1st independent column (:XEND)
create/column tmp.tbl :YEND "pixel" F7.1 R*4
compute/regression tmp.tbl :YEND = REGRESS
select/table tmp.tbl min(:XSTA,:XEND).ge.1.and.max(:XSTA,:XEND).le.{xsize} >Null
select/table tmp.tbl sel.and. -
                     min(:YSTA,:YEND).ge.1.and.max(:YSTA,:YEND).le.{ysize} >Null
copy/table tmp.tbl {OUTTBL}
-delete tmp.tbl
!
!
if warn .eq. 0 then
  return 0 {OUTTBL}
else
  return -1 {OUTTBL}
endif
!
error:
  set/format
  return 1 -99
!
!
!------------------------------------------------------------------------------
