! $Id: fors_findorder_fit.prg,v 1.3 2000/12/14 17:01:39 nrainer Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_findorder_fit.prg,v $
!
! .KEYWORDS   Order detection
!
! .PURPOSE    Use the output table from "fors_findorder_tra.prg" for
!             fitting each order separately.
! __________
!
! .USAGE      @p fors_findorder_fit [tratbl] [extra] [sizes]
!
! .INPUT      tratbl (P1): output table from "fors_findorder_tra.prg";
!                            default: <TRADEF> (see below)
!
!             extra  (P2): flag for extrapolation of traced orders: <>:do not
!                          extrapolate, <y>es,extrapolate;
!                            default: <>
!
!             sizes  (P2): no.of x-pixels of order reference frame AND
!                          no.of y-pixels of order reference frame AND
!                          distance of chosen columns, separated by commas;
!                            default: 2048,2048,50
!
! .OUTPUT     Q2: order detection table with relative order numbers
!                 and fitted positions
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for order detection - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/07/14  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref tratbl extra sizes
!
!
define/local TRADEF/c/1/20 "find_tra.tbl"
define/local OUTTBL/c/1/20 "order.tbl"
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "findorder_fit"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      tratbl (P1): output table from "fors_findorder_tra.prg";"
  w/o "                     default: {TRADEF}"
  w/o " "
  w/o "      extra  (P2): flag for extrapolation of traced orders: <>:do not"
  w/o "                   extrapolate, <y>es,extrapolate;"
  w/o "                     default: <>"
  w/o " "
  w/o "      sizes  (P2): no.of x-pixels of order reference frame AND"
  w/o "                   no.of y-pixels of order reference frame AND"
  w/o "                   distance of chosen columns, separated by commas;"
  w/o "                     default: 2048,2048,50"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(tratbl):
!----------
define/parameter p1 {TRADEF} ? -
  "        Enter positions table w.assigned orders [def.:{TRADEF}]: "
define/local tratbl/c/1/60 "{p1}"
if m$index(tratbl,".tbl") .eq. 0 tratbl = tratbl //".tbl"
!
if m$exist(tratbl) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({tratbl})"
  goto error
endif
show/table {tratbl} >Null
if outputi(1) .lt. 3 then
        write/out " "
        mess/out 1 {PROID} "Entered table has less than 3 columns !"
        mess/out 1 {PROID} "({tratbl})"
  goto error
endif
if outputi(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({tratbl})"
  goto error
endif
!
!
!P2(extra):
!---------
define/parameter p2 + ? -
  "        Enter flag for extrapolation of orders [def.:n|<y>es]: "
define/local extra/c/1/1 "{p2}"
!
!
!P3(sizes):
!---------
define/parameter p3 2048,2048,50 ? -
  "        Enter no.of x-pix.,y-pix.,col.dist. [def.:2048,2048,50]: "
outputi = m$parse(p3,"pix")
if outputi(1) .ne. 3 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P3 !"
  goto error
endif
define/local xsize/i/1/1 {pix01}
define/local ysize/i/1/1 {pix02}
define/local COLDIST/i/1/1 {pix03}
!
define/local pixbad/i/1/1 0
if xsize .lt. 1 then
        write/out " "
        mess/out 1 {PROID} "Impossible number of x-pixels !"
  pixbad = 1
endif
if ysize .lt. 1 then
        write/out " "
        mess/out 1 {PROID} "Impossible number of y-pixels !"
  pixbad = 1
endif
inputi(1) = m$nint(xsize/2.-1.-0.5)
if COLDIST .lt. 1 .or. COLDIST .gt. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "Column x-distance out of range !"
        mess/out 1 {PROID} "({COLDIST} ; 1<=col.dist.<={inputi(1)})"
  pixbad = 1
endif
if pixbad .eq. 1 goto error
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
define/local ndata/i/1/1 0
define/local ncol/i/1/1 0
define/local maxord/i/1/1 0
define/local alldata/i/1/1 0
define/local dataloop/i/1/1 0
define/local xpix/r/1/1 0.
define/local extraloop/i/1/1 0
define/local ordloop/i/1/1 0
!
!
!
!          FITTING OF POSITIONS PER ORDER :
!          ******************************
!
        mess/out 4 {PROID} "  fitting positions per order"
!
-copy {tratbl} tmp1.tbl
create/column tmp1.tbl :YFIT "pixel" F7.1 R*4
create/column tmp1.tbl :RESIDUAL "pixel" F8.1 R*4
!
regression/poly tmp1.tbl :Y :X,:ORDER 2,3 >Null
save/regression tmp1.tbl REGRESS
compute/regression tmp1.tbl :YFIT = REGRESS
compute/table tmp1.tbl :RESIDUAL = :Y-:YFIT
!
!
!          EXTRAPOLATION OF ORDERS :
!          ***********************
!
if extra .ne. "y" then
  !
  sort/table tmp1.tbl :ORDER,:X
  projection/table tmp1.tbl {OUTTBL} :ORDER,X,Y,YFIT,RESIDUAL
  copy/dd tmp1.tbl REGRESS* {OUTTBL}
  ! necessary for "fors_findorder_bac.prg":
  {OUTTBL},REGRESSI(3) = 3  ! # of dependent column (:Y)
  {OUTTBL},REGRESSI(4) = 2  ! # of 1st independent column (:X)
  {OUTTBL},REGRESSI(5) = 1  ! # of 2nd independent column (:ORDER)
  -delete tmp1.tbl
  !
else
        mess/out 4 {PROID} "  extrapolating orders"
  !
  ! X-COORDINATES FOR EACH ORDER:
  ! ----------------------------
  ncol = xsize/COLDIST+1
  show/table tmp1.tbl >Null
  ndata = outputi(2)
  sort/table tmp1.tbl :ORDER,:X
  maxord = {tmp1.tbl,:ORDER,@{ndata}}
  alldata = ncol*maxord
  create/table tmp2.tbl 3 {alldata} null
  create/column tmp2.tbl :ORDER " " I5 I*4
  create/column tmp2.tbl :X "pixel" F7.1 R*4
  sort/table tmp1.tbl :X,:ORDER
  extraloop = 0
  do dataloop = 1 ndata
    if dataloop .eq. 1 xpix = -1.
    if {tmp1.tbl,:X,@{dataloop}} .eq. xpix goto next
    xpix = {tmp1.tbl,:X,@{dataloop}}
    do ordloop = 1 maxord
      extraloop = extraloop+1
      tmp2.tbl,:X,@{extraloop} = {xpix}
      tmp2.tbl,:ORDER,@{extraloop} = {ordloop}
    enddo
    next:
  enddo
  !
  ! Y-COLUMN WITHOUT EXTRAPOLATION:
  ! ------------------------------
  create/column tmp2.tbl :Y "pixel" F7.1 R*4
  create/column tmp2.tbl :IDX " " F7.1 R*4
  compute/table tmp2.tbl :IDX = SEQ
  join/table tmp1.tbl :X,:ORDER tmp2.tbl :X,:ORDER align.tbl 0,0.
  do dataloop = 1 ndata
    tmp2.tbl,:Y,@{align.tbl,:IDX_2,{dataloop}} = {align.tbl,:Y_1,{dataloop}}
  enddo
  delete/column tmp2.tbl :IDX
  -delete align.tbl
  !
  ! EXTRAPOLATED YFIT-COLUMN:
  ! ------------------------
  copy/dd tmp1.tbl REGRESS* tmp2.tbl
  ! necessary for "fors_findorder_bac.prg":
  tmp2.tbl,REGRESSI(3) = 3  ! # of dependent column (:Y)
  tmp2.tbl,REGRESSI(4) = 2  ! # of 1st independent column (:X)
  tmp2.tbl,REGRESSI(5) = 1  ! # of 2nd independent column (:ORDER)
  create/column tmp2.tbl :YFIT "pixel" F7.1 R*4
  compute/regression tmp2.tbl :YFIT = REGRESS(:X,:ORDER)
  sort/table tmp2.tbl :ORDER,:X
  select/table tmp2.tbl :YFIT.ge.1.and.:YFIT.le.{xsize} >Null
  copy/table tmp2.tbl {OUTTBL}
  -delete tmp1.tbl tmp2.tbl
  !
  ! RESIDUAL WITHOUT EXTRAPOLATION:
  ! ------------------------------
  create/column {OUTTBL} :RESIDUAL "pixel" F8.1 R*4
  compute/table {OUTTBL} :RESIDUAL = :Y-:YFIT
  !
endif
!
!
if warn .eq. 0 then
  return 0 {OUTTBL}
else
  return -1 {OUTTBL}
endif
!
error:
  set/format
  return 1 -99
!
!
!------------------------------------------------------------------------------
