! $Id: fors_findorder_lin.prg,v 1.3 1999/11/25 11:24:12 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_findorder_lin.prg,v $
!
! .KEYWORDS   Order detection
!
! .PURPOSE    Use the output table from "fors_findorder_pos.prg" as input
!             for "fors_findorder_lin.c" (Hough transformation) and invoke
!             it. Afterwards use the output image of the Hough transformation
!             as input for "necfindmax.c" (cluster detection) and invoke it.
! __________
!
! .USAGE      @p fors_findorder_lin [postbl] [hough] [clust]
!
! .INPUT      postbl (P1): output table from "fors_findorder_pos.prg";
!                            default: <INTBL> (see below)
!
!             hough  (P2): slope_start AND slope_end AND slope_step AND
!                          zero_point_step for the Hough transformation,
!                          separated by commas;
!                            default: 0.35,0.75,0.008,1.
!
!             clust  (P3): y-width of one order in original image AND
!                          number of x-pixels of original image, separated
!                          by a comma;
!                            default: 220,2048
!
! .OUTPUT     Q2: table with zero points and slopes of the straights of all
!                 found relative orders AND Hough image AND cluster image,
!                 separated by commas
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for order detection - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/07/09  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref postbl hough clust
!
!
define/local INTBL/c/1/20    "find_pos.tbl"    ! default
define/local HOUGHIMA/c/1/20 "find_hough.bdf"  ! fixed
define/local CLUSTIMA/c/1/20 "find_clust.bdf"  ! fixed
define/local OUTTBL/c/1/20   "find_lin.tbl"    ! fixed
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "findorder_lin"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      postbl (P1): output table from "fors_findorder_pos.prg";"
  w/o "                     default: {INTBL}"
  w/o " "
  w/o "      hough  (P2): slope_start AND slope_end AND slope_step AND"
  w/o "                   zero_point_step for the Hough transformation,"
  w/o "                   separated by commas;"
  w/o "                     default: 0.35,0.75,0.008,1."
  w/o " "
  w/o "      clust  (P3): y-width of one order in original image AND"
  w/o "                   number of x-pixels of original image, separated"
  w/o "                   by a comma;"
  w/o "                     default: 220,2048"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(postbl):
!----------
define/parameter p1 {INTBL} ? -
  "        Enter order positions table [def.:{INTBL}]: "
write/keyword in_a/c/1/60 "{p1}"
if m$index(in_a,".tbl") .eq. 0 in_a = in_a //".tbl"
!
if m$exist(in_a) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({in_a})"
  goto error
endif
show/table {in_a} >Null
if outputi(1) .lt. 2 then
        write/out " "
        mess/out 1 {PROID} "Entered table has less than 2 columns !"
        mess/out 1 {PROID} "({in_a})"
  goto error
endif
if outputi(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({in_a})"
  goto error
endif
!
!
!P2(hough):
!---------
define/parameter p2 0.35,0.75,0.008,1. ? -
  "        Enter Hough params. (see proc.header) [def:0.35,0.75,0.008,1.]: "
outputi = m$parse(p2,"hough")
if outputi(1) .ne. 4 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local SLOPMIN/r/1/1 {hough01}
define/local SLOPMAX/r/1/1 {hough02}
define/local SLOPSTEP/r/1/1 {hough03}
define/local ZEPOSTEP/r/1/1 {hough04}
!
define/local houghbad/i/1/1 0
set/format F5.3,
if SLOPMIN .ge. SLOPMAX then
        write/out " "
        mess/out 1 {PROID} "P2[1:2]: slope minimum >= slope maximum !"
        mess/out 1 {PROID} "({SLOPMIN} >= {SLOPMAX})"
  houghbad = 1
endif
inputr(1) = (SLOPMAX-SLOPMIN)/2.
if SLOPSTEP .ge. inputr(1) .and. houghbad .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "P2[3]: slope step too high !"
        mess/out 1 {PROID} "({SLOPSTEP} ; slope_step<{inputr(1)})"
  houghbad = 1
endif
statistics/table {in_a} #1 >Null
define/local xmin/r/1/1 {outputr(1)}
define/local xmax/r/1/1 {outputr(2)}
statistics/table {in_a} #2 >Null
define/local ymin/r/1/1 {outputr(1)}
define/local ymax/r/1/1 {outputr(2)}
define/local zepomin/r/1/1 0
define/local zepomax/r/1/1 0
zepomin = ymin - SLOPMAX*(xmax-xmin)
zepomax = ymax - SLOPMIN*(xmax-xmin)
inputr(1) = (zepomax-zepomin)/2.
if ZEPOSTEP .ge. inputr(1) then
        write/out " "
        mess/out 1 {PROID} "P2[4]: zero point step too high !"
        mess/out 1 {PROID} "({ZEPOSTEP} ; z.p._step<{inputr(1)})"
  houghbad = 1
endif
set/format {FMTDEF}
if houghbad .eq. 1 goto error
!
!
!P3(clust):
!---------
define/parameter p3 220,2048 ? -
  "        Enter cluster params. (see proc.header) [def:220,2048]: "
outputi = m$parse(p3,"clust")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P3 !"
  goto error
endif
define/local ORDWID/i/1/1 {clust01}
define/local NPIX1/i/1/1 {clust02}
!
if ORDWID .lt. 1 then
        write/out " "
        mess/out 1 {PROID} "P3[1]: Y-width of one order too low !"
        mess/out 1 {PROID} "({ORDWID} ; 1<=width)"
  goto error
endif
if NPIX1 .lt. 1 then
        write/out " "
        mess/out 1 {PROID} "P3[2]: No. of x-pixels too low !"
        mess/out 1 {PROID} "({NPIX1} ; 1<=no.x-pixels)"
  goto error
endif
!
!
!
!          DEFINITION OF FURTHER VARIABLES :
!          *******************************
!
! none
!
!
!
!          CREATION OF HOUGH TRANSFORMATION IMAGE :
!          **************************************
!
        mess/out 4 {PROID} "  performing Hough transformation"
!
in_b = HOUGHIMA
write/keyword inputr {SLOPMIN},{SLOPMAX},{SLOPSTEP},{ZEPOSTEP}
!
!**********************************
run PIPE_EXE:fors_findorder_lin.exe
!**********************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Hough transformation failed !"
  goto error
endif
!
!
!          DETERMINATION OF STRAIGHT COEFFICIENTS OF THE ORDERS :
!          ****************************************************
!
        mess/out 4 {PROID} "  performing cluster detection"
!
-copy {HOUGHIMA} {CLUSTIMA}
in_a = CLUSTIMA
in_b = OUTTBL
write/keyword inputc "l"        ! low contrast mode with manual order width
write/keyword inputi/i/1/1 {ORDWID}  ! order width
write/keyword inputi/i/2/1 {NPIX1}   ! x-pixels of original image
write/keyword inputi/i/3/1 1         ! step between pixels
write/keyword inputi/i/4/1 {NPIX1}   ! x-pixels of original image
write/keyword inputi/i/5/1 1         ! minimal detectable no. of orders
write/keyword inputi/i/6/1 1000      ! maximal detectable no. of orders
write/keyword inputr/r/1/1 0.68  ! intens.perc. around max.int./ord. (MAX.0.99!)
write/keyword inputr/r/2/1 0.    ! "0.": all clust. (.."1.": only 1 w.max.int.)
!
!*******************************
run STD_EXE:necfindmax.exe >Null
!*******************************
show/table {OUTTBL} >Null
if outputi(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No order found !"
  goto error
endif
!
!
if warn .eq. 0 then
  return 0 {OUTTBL},{HOUGHIMA},{CLUSTIMA}
else
  return -1 {OUTTBL},{HOUGHIMA},{CLUSTIMA}
endif
!
error:
  set/format
  return 1 -99,-99,-99
!
!
!------------------------------------------------------------------------------
