! $Id: fors_findorder_pos.prg,v 1.3 1999/11/25 11:24:12 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_findorder_pos.prg,v $
!
! .KEYWORDS   Order detection
!
! .PURPOSE    Create a table with first derivatives of chosen image columns as
!             input for "fors_findorder_pos.c" and invoke it.
! __________
!
! .USAGE      @p fors_findorder_pos ima [geom]
!
! .INPUT      ima  (P1): input flatfield frame for order detection;
!                          default: none
!
!             geom (P2): distance of chosen columns AND
!                        no. of neighbour pixels for median filtering AND
!                        minimal value for first derivative AND
!                        y-width of one order AND
!                        tolerance for this width, separated by commas;
!                          default: 50,43,2,220,10
!
! .OUTPUT     Q2: table with all found x and y-centre positions of the orders
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for order detection - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/07/07  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref ima geom
!
!
define/local OUTTBL/c/1/20 "find_pos.tbl"
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "findorder_pos"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      ima  (P1): input flatfield frame for order detection;"
  w/o "                   default: none"
  w/o " "
  w/o "      geom (P2): distance of chosen columns AND"
  w/o "                 no. of neighbour pixels for median filtering AND"
  w/o "                 minimal value for first derivative AND"
  w/o "                 y-width of one order AND"
  w/o "                 tolerance for this width, separated by commas;"
  w/o "                   default: 50,43,2,220,10"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(ima):
!-------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of flatfield frame: "
define/local orderflat/c/1/64 "{p1}"
if m$index(orderflat,".bdf") .eq. 0 orderflat = orderflat//".bdf"
!
if m$exist(orderflat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({orderflat})"
  goto error
endif
!
define/local npix/i/1/2 {{orderflat},NPIX(1)},{{orderflat},NPIX(2)}
!
!
!P2(geom):
!--------
define/parameter p2 50,43,2,220,10 ? -
  "        Enter geom.params. (see proc.header) [def:50,43,2,220,10]: "
outputi = m$parse(p2,"geom")
if outputi(1) .ne. 5 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local COLDIST/i/1/1 {geom01}
define/local YMED/i/1/1    {geom02}
define/local YDERTHR/i/1/1 {geom03}
define/local ORDWID/i/1/1  {geom04}
define/local WIDTOL/i/1/1  {geom05}
!
define/local geombad/i/1/1 0
inputi(1) = m$nint(npix(1)/2.-1.-0.5)
if COLDIST .lt. 1 .or. COLDIST .gt. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "P2[1]: Column x-distance out of range !"
        mess/out 1 {PROID} "({COLDIST} ; 1<=col.dist.<={inputi(1)})"
  geombad = 1
endif
if YDERTHR .lt. 1 then
        write/out " "
        mess/out 1 {PROID} "P2[3]: Minimal first y-derivative out of range !"
        mess/out 1 {PROID} "({YDERTHR} ; 1<=first_der.)"
  geombad = 1
endif
inputi(1) = npix(2)-3
if ORDWID .lt. 1 .or. ORDWID .gt. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "P2[4]: Y-width of one order out of range !"
        mess/out 1 {PROID} "({ORDWID} ; 1<=width<={inputi(1)})"
  geombad = 1
endif
inputi(1) = ORDWID/2
if WIDTOL .lt. 1 .or. WIDTOL .gt. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "P2[5]: Toler.f.y-width of one order out of range !"
        mess/out 1 {PROID} "({WIDTOL} ; 1<=width_tol.<={inputi(1)})"
  geombad = 1
endif
inputi(1) = ORDWID/2
if YMED .lt. 1 .or. YMED .gt. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "P2[2]: No.of neighb.pix. f.med.filt. out of range !"
        mess/out 1 {PROID} "({YMED} ; 1<=neighb.pix.<={inputi(1)})"
  geombad = 1
endif
if geombad .eq. 1 goto error
!
!
!
!          DEFINITION OF FURTHER VARIABLES :
!          *******************************
!
! none
!
!
!
!          CREATION OF Y-DERIVATION TABLE :
!          ******************************
!
        mess/out 4 {PROID} "  creating table with column derivatives"
!
transpose/image {orderflat} trans.bdf minor  ! columns <-> rows
@a scaler trans.bdf col.bdf 1,-{COLDIST} n
col.bdf,STEP(2) = {COLDIST}.
filter/median col.bdf filt.bdf {YMED},0,0. ya ? nne >Null
shift/image filt.bdf shift.bdf 1,0
compute/pixel derive.bdf = filt.bdf - shift.bdf
copy/it derive.bdf derive.tbl
-delete trans.bdf col.bdf filt.bdf shift.bdf derive.bdf
!
!
!          DETERMINATION OF Y-CENTRE POSITIONS OF THE ORDERS :
!          *************************************************
!
        mess/out 4 {PROID} "  determining order y-centres"
!
in_a = "derive.tbl"
in_b = OUTTBL
write/keyword inputi {COLDIST},{YDERTHR},{ORDWID},{WIDTOL}
define/local string/c/1/256 " " all
define/local mid$msel/i/1/1 {{in_a},TBLCONTR(4)}
define/local mid$selidx/i/1/{{in_a},TBLCONTR(4)} 0 all
!
!**********************************
run PIPE_EXE:fors_findorder_pos.exe
!**********************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Order detection failed !"
  goto error
endif
show/table {OUTTBL} >Null
if outputi(2) .lt. 2 then
        write/out " "
        mess/out 1 {PROID} "Found less than 2 positions !"
  goto error
endif
-delete {in_a}
!
!
if warn .eq. 0 then
  return 0 {OUTTBL}
else
  return -1 {OUTTBL}
endif
!
error:
  set/format
  return 1 -99
!
!
!------------------------------------------------------------------------------
