! $Id: fors_findorder_tra.prg,v 1.3 2000/02/23 14:49:33 nrainer Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_findorder_tra.prg,v $
!
! .KEYWORDS   Order detection
!
! .PURPOSE    Use the output tables from "fors_findorder_pos.prg" and
!             "fors_findorder_lin.prg" for tracing the order positions.
! __________
!
! .USAGE      @p fors_findorder_tra [postbl] [lintbl] [ordwid]
!
! .INPUT      postbl (P1): output table from "fors_findorder_pos.prg";
!                            default: <POSDEF> (see below)
!
!             lintbl (P2): output table from "fors_findorder_lin.prg" AND
!                          name of column with zero points AND
!                          name of column with slopes, separated by commas;
!                            default: <LINDEF>(see below),ORIG,SLOPE
!
!             ordwid (P3): y-width of one order in original image;
!                            default: 220
!
! .OUTPUT     Q2: table with those found x and y-centre positions
!                 to which relative order numbers could be assigned
!
!             Q3: total number of identified orders
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for order detection - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/07/14  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref postbl lintbl ordwid
!
!
define/local POSDEF/c/1/20 "find_pos.tbl"
define/local LINDEF/c/1/20 "find_lin.tbl"
define/local OUTTBL/c/1/20 "find_tra.tbl"
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "findorder_tra"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
write/keyword q3 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      postbl (P1): output table from "fors_findorder_pos.prg";"
  w/o "                     default: {POSDEF}"
  w/o " "
  w/o "      lintbl (P2): output table from "fors_findorder_lin.prg" AND"
  w/o "                   name of column with zero points AND"
  w/o "                   name of column with slopes, separated by commas;"
  w/o "                     default: {LINDEF},ORIG,SLOPE"
  w/o " "
  w/o "      ordwid (P3): y-width of one order in original image;"
  w/o "                     default: 220"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(postbl):
!----------
define/parameter p1 {POSDEF} ? -
  "        Enter order positions table [def.:{POSDEF}]: "
define/local postbl/c/1/60 "{p1}"
if m$index(postbl,".tbl") .eq. 0 postbl = postbl //".tbl"
!
if m$exist(postbl) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({postbl})"
  goto error
endif
show/table {postbl} >Null
if outputi(1) .lt. 2 then
        write/out " "
        mess/out 1 {PROID} "Entered table has less than 2 columns !"
        mess/out 1 {PROID} "({postbl})"
  goto error
endif
if outputi(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({postbl})"
  goto error
endif
!
!
!P2(lintbl):
!----------
define/parameter p2 {LINDEF},ORIG,SLOPE t -
  "        Tbl.w.straight coeffs.,zp.,slop.col. [def.:{LINDEF},ORIG,SLOPE]: "
define/local nparse/i/1/1 0
nparse = m$parse(p2,"lin")
if nparse .ne. 3 .and. nparse .ne. 1 then  ! 1 for defaults
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local lintbl/c/1/60 "{lin01}"
if m$index(lintbl,".tbl") .eq. 0 lintbl = lintbl //".tbl"
!
if m$exist(lintbl) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lintbl})"
  goto error
endif
show/table {lintbl} >Null
define/local nords/i/1/1 {outputi(2)}  ! needed for below
if nords .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({lintbl})"
  goto error
endif
!
if nparse .eq. 3 then
  define/local czepo/c/1/16 "{lin02}"
  define/local cslop/c/1/16 "{lin03}"
  if czepo(1:1) .eq. ":" czepo = czepo(2:)
  if cslop(1:1) .eq. ":" cslop = cslop(2:)
elseif nparse .eq. 1 then  ! defaults
  define/local czepo/c/1/16 "ORIG"
  define/local cslop/c/1/16 "SLOPE"
endif
!
if m$existc(lintbl,":{czepo}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({lintbl},:{czepo})"
  goto error
elseif m$existc(lintbl,":{cslop}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({lintbl},:{cslop})"
  goto error
endif
!
!
!P3(ordwid):
!----------
define/parameter p3 220 ? -
  "        Enter y-width of one order [def:220]: "
define/local ORDWID/i/1/1 {p3}
!
if ORDWID .lt. 1 then
        write/out " "
        mess/out 1 {PROID} "P3: Y-width of one order too low !"
        mess/out 1 {PROID} "({ORDWID} ; 1<=width)"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
define/local ordloop/i/1/1 0
define/local zepo/r/1/1 0.
define/local slop/r/1/1 0.
define/local ythresh/i/1/1 0
define/local foundords/i/1/1 0
!
!
!
!          ASSIGNMENT OF ORDER NUMBERS :
!          ***************************
!
        mess/out 4 {PROID} "  tracing positions per order"
!
sort/table {lintbl} :{czepo}
create/column {lintbl} :ORDER " " I5 I*4 >Null
compute/table {lintbl} :ORDER = SEQ
if m$existc(postbl,"ORDER") .eq. -1 create/column {postbl} :ORDER " " I5 I*4
compute/table {postbl} :ORDER = 0
!
ythresh = m$nint(ORDWID/2.)
do ordloop = 1 nords
  !
  zepo = {{lintbl},:{czepo},{ordloop}}
  slop = {{lintbl},:{cslop},{ordloop}}
  create/column {postbl} :YPIX_{ordloop} "pixel" F7.1 R*4 >Null
  compute/table {postbl} :YPIX_{ordloop} = {zepo} + {slop}*#1
  create/column {postbl} :YDIF_{ordloop} "pixel" F7.1 R*4 >Null
  compute/table {postbl} :YDIF_{ordloop} = abs(#2-:YPIX_{ordloop})
  !
  select/table {postbl} :ORDER.eq.0.AND.:YDIF_{ordloop}.lt.{ythresh} >Null
  if outputi(1) .gt. 0 then
    compute/table {postbl} :ORDER = :ORDER+SEL*{ordloop}
    foundords = foundords+1
  endif
  !
enddo
!
select/table {postbl} :ORDER.ne.0 >Null
copy/table {postbl} tmp.tbl
select/table {postbl} all
projection/table tmp.tbl {OUTTBL} #1..3
-delete tmp.tbl
!
if foundords .gt. 0 then
  !
        mess/out 3 {PROID} "  found  < {foundords} >  orders"
else
        write/out " "
        mess/out 1 {PROID} "No order could be identified !"
  goto error
endif
!
!
if warn .eq. 0 then
  return 0 {OUTTBL} {foundords}
else
  return -1 {OUTTBL} {foundords}
endif
!
error:
  set/format
  return 1 -99 -99
!
!
!------------------------------------------------------------------------------
