! $Id: fors_finish.prg,v 1.27 2000/07/24 16:00:50 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_finish.prg,v $
!
!.KEYWORDS    Pipeline shutdown
!
!.USAGE       finish/pipeline  catalog frm_set ctx [cat_plot]
!
!.INPUT       catalog     (P1)    Catalog of result frames
!   	      frm_set     (P2)    Set of output frames    
!   	      ctx         (P3)    Context identifier
!             main        (P4)    Main pipeline product identifier
!             cat_plot    (P5)    Catalog of plot files (optional)
!
!.OUTPUT      None
!
!.RETURNS     The procedure returns 0 on success, otherwise 1. The keyword
!             Q1 is updated with the exit status.
!
!.PURPOSE     The procedure disables the loaded context and converts the
!             output frames from MIDAS formats to FITS format. The names of
!             the result frames in the current working directory are taken
!             from 'catalog'. The names of the output FITS frames are
!             automatically generated. If the output set of frames is a comma
!             seperated list of MIDAS catalogs (i.e. the suffix '.cat' must be
!             present) and if each name of this list starts with the capital
!             letters 'XSF', the procedure assumes to run within a recipe
!             queue and copies (appends) the entries from the input catalog to
!             each catalog given in the list. In this case the environmet is
!             kept (i.e. no contexts can be deleted).        
!
!             If the name of the input catalog is a '+' any output is
!             suppressed, i.e. neither FITS files nor catalogs will be
!             produced or updated respectively. This is useful to just delete
!             some contexts. All contexts may be removed from the environment
!             by the context identifier '-all'.
!
!             If a plot catalog is given by parameter P5, all plots
!             contained in this catalog are copied to the product directory.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!
!------------------------------------------------------------------------------

crossref catname frames context cat_plot

define/param p1  ?      c    "Catalog of result frames:"
define/param p2  ?      c    "Basename of output frame(s):"
define/param p3  +      c    "Context identifier:"
define/param p4  +      c    "Main pipeline product:"
define/param p5  +      c    "Catalog of plot files:"

define/maxpar 5


! Procedure identification
define/local procID/c/1/80	"FinishPipe"


! Local variables
define/local pos/i/1/1     0
define/local i/i/1/1       0
define/local nctx/i/1/1    0
define/local nout/i/1/1    0
define/local errsts/i/1/1  0
define/local ctx/c/1/100   {p3}



ctx = m$lower(p3)

set/format i2
pos = m$index(p2,".cat")

if pos .ge. 1 then
  nout = m$parse(p2,"cout")

  do i = 1 nout 1
    pos = m$indexb(cout{i}, ".cat")

    if cout{i}(1:3) .ne. "XSF" .or. pos .le. 1 then
      mess/out 1 {procID} "Invalid file name {cout{i}}!"
      return 1
    else
      if p1(1:1) .ne. "+" then
        update/catalog {p1} {cout{i}}
        mess/out 4 {procID} "SUPER MODE: Adding output frames to {cout{i}}"
      endif
    endif
  enddo


! Check if context shall be disabled
  if p_ctxdel .ne. 0 then

    nctx = m$parse(ctx,"ctx")

    do i = 1 nctx 1
      clear/context {ctx{i}}
    enddo

    set/ctxdel reset
    mess/out 3 {procID} "Context(s) {ctx} removed from environment."
  endif  

! Close internal logging facility
!  shortlog/out -1 {p4}

else  

! Close internal logging facility and write shortlog to HISTORY buffer
  shortlog/out 0 {p1}

  if p1(1:1) .ne. "+" then

! Clean pipeline product headers
    if m$strlen(p_hdrpurge) .gt. 0 .and. m$exist(p_hdrpurge) .eq. 1 then
      purge/header {p1} {p_hdrpurge} {FORS_IDSTRNG}
      if {q1} .gt. 0 then
        mess/out 1 {procID} "Cleaning pipeline product headers failed!"
        return 1
      endif
    endif

! Copy the result files to the product directory
    put/frames {p1} {p2} {p4} {p5}
    errsts = {q1}
  endif

! Show short-log
  shortlog/out -2

! Create QC1 Log
  qc1log/out 0 

  if ctx .eq. "-all" then

    mess/out 2 {procID} "All contexts will be disabled!"
    clear/context -all >Null

  elseif ctx(1:1) .ne. "+" then

    nctx = m$parse(ctx,"ctx")

    do i = 1 nctx 1
      clear/context {ctx{i}}
    enddo

  endif
endif

set/format

! Account for errors not yet handled
if errsts .ne. 0 then
  return 1
endif

return 0
