! $Id: fors_fitextinc.prg,v 1.1 1999/03/15 10:12:54 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_fitextinc.prg,v $
!
!.KEYWORDS    Atmospheric extinction coefficients, robust linear regression
!
!.USAGE       fit/extinction extcat exttbl
!
!.INPUT       extcat      (P1)    Catalog with preprocessed extinction
!                                 correction factors.
!
!.OUTPUT      exttbl      (P2)    Table with fitted extinction coefficients.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status.
!
!.PURPOSE     The procedure takes all 3d tables with extinction corrections 
!             listed in the input catalog and computes the wavelength 
!             dependent extinction coefficients by fitting a straight line
!             to the data for each wavelength bin. A robust method (least
!             median of squares) is used as fit algorithm.
!             It is assumed that all spectra stored in a single table have
!             been observed at the same airmass. The required minimum number
!             of spectra at different airmass is 3. Furthermore all input
!             spectra must have the same wavelength limits and stepsize.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/12/02:  Creation
!
!------------------------------------------------------------------------------

crossref  extcat exttbl

define/param p1 ? c "Catalog of extinction correction tables:"
define/param p2 ? c "Table of fitted extinction coefficients:"
define/maxpar 2


! Procedure identification
define/local procID/c/1/20      "FitExtinction"


! Local variables
define/local pos/i/1/1         0
define/local extcat/c/1/100    {p1}
define/local exttbl/c/1/100    {p2}



pos = m$indexb(extcat,".cat")
if pos .le. 0 then
  extcat = extcat // ".cat"
elseif pos .eq. 1 then
  mess/out 1 {procID} "Invalid input catalog name!"
  return 1
endif

if m$exist(extcat) .eq. 0 then
  mess/out 1 {procID} "Input catalog >{extcat}< does not exist!"
  return 1
endif

run PIPE_EXE:fors_extcoeff.exe
if {q1} .ne. 0 then
  return 1
endif

return 0
