! $Id: fors_fits2ascii.prg,v 1.14 1999/11/25 11:23:54 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_fits2ascii.prg,v $
!
! .KEYWORDS   Format conversion
!
! .PURPOSE    Transform CalibDB FITS files to ASCII files for SExtractor.
! __________
!
! .USAGE      @p fors_fits2ascii fitsfile
!
! .INPUT      fitsfile (P1): FITS table from CalibDB;
!                              default: none
!
! .OUTPUT     <P1>.dat: ASCII table
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    -
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/06  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref fitsfile
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "fits2ascii"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      fitsfile (P1): FITS table from CalibDB (def.ext. ".tfits");"
  w/o "                       default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          ******
!
        if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? "        Enter FITS file (def.ext. ".tfits"): "
define/local fitsfile/c/1/64 "{p1}"
if m$index(fitsfile,".") .eq. 0 fitsfile = fitsfile//".tfits"
!
if m$exist(fitsfile) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "ENTERED FILE IS NOT PRESENT !"
        mess/out 1 {PROID} "({fitsfile})"
  goto error
endif
!
if m$filtyp(fitsfile," ") .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "ENTERED FILE IS NO FITS TABLE !"
        mess/out 1 {PROID} "({fitsfile})"
  goto error
endif
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local char/i/1/1 0
  define/local asciifile/c/1/64 " " all
  define/local filecode/i/1/2 0 all
  define/local totrows/i/1/1 0
  define/local row/i/1/1 0
!
!
!          FITS -> ASCII :
!          *************
!
  char = m$index(fitsfile,".")-1
  asciifile = fitsfile(1:{char})//".dat"
  !
        mess/out 4 {PROID} "converting: {fitsfile} -> {asciifile}"
!
! FITS -> TBL :
indisk/fits {fitsfile} tmp.tbl >Null
!
! TBL -> ASCII :
open/file {asciifile} write filecode
show/table tmp.tbl >Null
totrows = outputi(2)
do row = 1 totrows
  write/keyword inputc "{tmp.tbl,#1,@{row}}" >Null  ! Rem.trail.blanks - needed!
  write/file {filecode(1)} "{inputc}"
enddo
close/file {filecode(1)}
-delete tmp.tbl
!
!
set/format
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------
