! $Id: fors_focus.prg,v 1.53 2001/04/10 09:49:24 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_focus.prg,v $
!
! .KEYWORDS   Focal alignment
!
! .PURPOSE    Determine best focus position of a set of focus frames
!             by calculating smallest FWHM of the mapped pinhole.
! __________
!
! .USAGE      @p fors_focus icat [scan] [sext] [pix] [disp]
!
! .INPUT      icat (P1): image catalogue with at least {MINOBJ} (see below)
!                        input frames, each (!) with different focal position;
!                          default: none
!
!             sext (P2): 4 ascii files for SExtractor (input parameters, con-
!                        volution mask, output parameters), separated by
!                        commas;
!                          default: sex__inpar.dat,sex_convol.dat,sex_out-
!                                   par.dat,sex_neupar.dat
!
!             pix  (P3): names of descriptors with pixel scale and focal
!                        position, separated by a comma;
!                          default: PIXSCALE,POS
!
!             disp (P4): display selection: <p>:pipeline mode:read global key
!                        NICE_CREA [y|n], <n>:none, <l>:local, or
!                        host:display[.screen] - with the latter option
!                        you can specify the name or IP-address of a display;
!                          default: <l>
!
! .OUTPUT     OUTPUTD(1): Best focal position
!             OUTPUTR(1): Smallest full width at half maximum
!
!             Per catal. for brightest obj. (if obj. were found in any frame):
!               "<P1>_foc.tbl":  1 FRAME   : frame name
!                                2 XPIX,
!                                3 YPIX    : positions
!                                4 FWHM    : full width at half max.
!                                5 FOCPOS  : focal position from descriptor
!
!               In the last row the smallest value of all FWHMs and the thereby
!               received best FOCPOS are given.
!
!             Per frame for each obj. (if obj. were found in current frame):
!               "<P1-entries>_obj.tbl":  1 XPIX,
!                (`N':do NOT change)     2 YPIX    : positions
!                                        3 FLUX    : brightnesses (sky-subtr.)
!                                       ----------
!                                        4 FWHM    : full width at half max.
!                                        5 NPIX    : # pixels per object
!                                        6 ELL_A   : maj.ell.half-axis (1sigma)
!                                        7 ELL_B   : min.ell.half-axis (1sigma)
!                                     N  8 ROTANG  : angle(maj.axis,x-axis)
!                                        9 FLAGS   : obj.quality
!                                       ==========
!                                     N 10 COLOR   : colours f. ellipses
!                                     N 11 XSTARTPIX,
!                                     N 12 YSTARTPIX,
!                                     N 13 XENDPIX,
!                                     N 14 YENDPIX : rectangle surround. obj.
!
!             "<P1-entries>_skymap.bdf":  sky background frames
!             "<P1-entries>_skysub.bdf":  sky-subtracted frames
!
!             Q2: tot. number of skipped frames, because no objects were found
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  SExtractor v2.0.15 - see corresponding documents.
!
! .COMMENT    The ascii files with input parameters and output parameters (P3)
!             must NOT be changed; otherwise this programme must be altered
!             partially (see comments in section "Photometry" and numbers in
!             section "Creation of single output table") !!!
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/16  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref icat sext pix disp
!
!
!  MINOBJ is "+lower_levels" in fors_gen_tec_focus.prg
if m$existk("MINOBJ") .eq. 0 define/local MINOBJ/i/1/1 5 ? +lower_levels ! >=3 !

define/local ELLSIZE/r/1/1  5. ! # of sigmas of half-axes for drawing ellipses
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "focus"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For `longlog/out':
write/keyword out_a "given command/qualif not defined ..."
show/command longlog/out | write/keyword in_a
-delete Mid{mid$sess(11:12)}1Pipe
-delete midlis{mid$sess(11:12)}.prg
if in_a .eq. out_a create/command longlog/out "write/out"
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  set/format I1
  w/o "      ******************************************************************"
  w/o "      icat (P1): image catalogue with at least {MINOBJ} input frames;"
  w/o "                   default: none"
  w/o " "
  w/o "      sext (P2): 4 ascii files for SExtractor (input parameters, con-"
  w/o "                 volution mask, output parameters), separated by a"
  w/o "                 comma;"
  w/o "                   default: sex__inpar.dat,sex_convol.dat,sex_out-"
  w/o "                            par.dat,sex_neupar.dat"
  w/o " "
  w/o "      pix  (P3): names of descriptors with pixel scale and focal"
  w/o "                 position, separated by a comma;"
  w/o "                   default: PIXSCALE,POS"
  w/o " "
  w/o "      disp (P4): display selection: <p>:pipeline mode:read global key"
  w/o "                 NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                 host:display[.screen] - with the latter option"
  w/o "                 you can specify the name or IP-address of a display;"
  w/o "                   default: <l>"
  w/o "      ******************************************************************"
  set/format
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(icat):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter image cat.(BDF) w. input frames: "
define/local icat/c/1/64 "{p1}"
if m$index(icat,".cat") .eq. 0 icat = icat//".cat"
!
if m$exist(icat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
show/icat {icat} n
if outputi(1) .lt. MINOBJ then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue has less than {MINOBJ} entries !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
define/local catal/i/1/1 0
inloop:
  store/frame inputc {icat} 1 end_inloop
  if m$exist(inputc) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "At least one catalogue entry is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
    goto error
  endif
  goto inloop
end_inloop:
!
!
!P2(sext):
!--------
define/parameter p2 sex__inpar.dat,sex_convol.dat,sex_outpar.dat,sex_neupar.dat -
  ? "        4 ASCII files for SExtractor [if different from def.]: "
outputi = m$parse(p2,"foc")
if outputi(1) .ne. 4 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local inparfile/c/1/64 "{foc01}"
if m$index(inparfile,".") .eq. 0 inparfile = inparfile//".dat"
define/local convolfile/c/1/64 "{foc02}"
if m$index(convolfile,".") .eq. 0 convolfile = convolfile//".dat"
define/local outparfile/c/1/64 "{foc03}"
if m$index(outparfile,".") .eq. 0 outparfile = outparfile//".dat"
define/local neuparfile/c/1/64 "{foc04}"
if m$index(neuparfile,".") .eq. 0 neuparfile = neuparfile//".dat"
!
if m$exist(inparfile) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor input parameters"
        mess/out 1 {PROID} "is not present or can not be opened !"
        mess/out 1 {PROID} "({inparfile})"
  goto error
endif
!
if m$filtyp(inparfile," ") .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor input parameters"
        mess/out 1 {PROID} "is no ascii file ({inparfile}) !"
  goto error
endif
!
if m$exist(convolfile) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor convolution mask"
        mess/out 1 {PROID} "is not present or can not be opened !"
        mess/out 1 {PROID} "({convolfile})"
  goto error
endif
!
if m$filtyp(convolfile," ") .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor convolution mask"
        mess/out 1 {PROID} "is no ascii file ({convolfile}) !"
  goto error
endif
!
if m$exist(outparfile) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor output parameters"
        mess/out 1 {PROID} "is not present or can not be opened !"
        mess/out 1 {PROID} "({outparfile})"
  goto error
endif
!
if m$filtyp(outparfile," ") .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor output parameters"
        mess/out 1 {PROID} "is no ascii file ({outparfile}) !"
  goto error
endif
!
if m$exist(neuparfile) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with neuronal network weights"
        mess/out 1 {PROID} "is not present or can not be opened !"
        mess/out 1 {PROID} "({neuparfile})"
  goto error
endif
!
if m$filtyp(neuparfile," ") .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with neuronal network weights"
        mess/out 1 {PROID} "is no ascii file ({neuparfile}) !"
  goto error
endif
!
!
!P3(pix):
!-------
define/parameter p3 PIXSCALE,POS ? -
  "        Enter dscs. f.pix.scl./foc.pos. [def.:PIXSCALE,POS]: "
outputi = m$parse(p3,"pipo")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P3 !"
  goto error
endif
define/local pixscale/c/1/72 "{pipo01}"
define/local focalpos/c/1/72 "{pipo02}"
!
!
!P4(disp):
!--------
define/parameter p4 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p4}"
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Creation ... table" :
  define/local char/i/1/2 0 all
  define/local foctable/c/1/64 " " all
!
!     "L o o p" :
  define/local topicima/c/1/64 " " all
  define/local lopcount/i/1/1 0
  !
  !   "Removing" :
  define/local subxstart/i/1/1 0
  define/local subxend/i/1/1 0
  define/local xtopicima/c/1/64 " " all
  !
  !   "Photometry" :
  define/local topicfits/c/1/64 " " all
  define/local photascii/c/1/64 " " all
  define/local backima/c/1/64 " " all
  define/local foundima/c/1/64 " " all
  !
  !   "Checking ... objects" :
  define/local listcode/i/1/2 0 all
  define/local skip/i/1/1 0
  !
  !   "Creation ... table" :
  define/local formcode/i/1/2 0 all
  !
  !   "Removal ... from table" :
  define/local phottable/c/1/64 " " all
  define/local selrows/i/1/1 0
  define/local iloop/i/1/1 0
  !
  !   "FWHM" :
  define/local fwhm/r/1/1 0.
!
!     "FOCUS" :
  define/local totrowno/i/1/1 0
  define/local lowframe/c/1/64 " " all
  define/local tloop/i/1/1 0
  define/local lowentry/i/1/1 0
  define/local neighbours/i/1/1 0
  define/local lowlimit/i/1/2 0 all
  define/local lowrange/i/1/2 0 all
  define/local bestfocpos/d/1/1 0.
  define/local bestfwhm/r/1/1 0.
!
!
!
!          MANAGING DISPLAYS / ITT / LUT :
!          *****************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_dids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 1 is missing !"
    goto error
  endif
  if nice_dids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 2 is missing !"
    goto error
  endif
  if nice_dids(3) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 3 is missing !"
    goto error
  endif
  if nice_gids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 1 is missing !"
    goto error
  endif
  !
  assign/display d,1 >Null
  clear/display
  display/lut off
  label/display "ORIGINAL" {nice_dlab}
  assign/display d,2 >Null
  clear/display
  display/lut off
  label/display "BACKGROUND" {nice_dlab}
  assign/display d,3 >Null
  clear/display
  display/lut off
  label/display "FOUND OBJECTS" {nice_dlab}
  load/lut heat
  load/itt ramp
  assign/graphic g,1 >Null
  clear/graphic
  if nice_gids(2) .eq. 1 then
    assign/graphic g,2 >Null
    clear/graphic
  endif
  set/graphic default
  set/graphic pmode=0 bcolour=1
  !
endif
!
!
!          CREATION OF OVERALL OUTPUT TABLE :
!          ********************************
!
!        mess/out 4 {PROID} "preparing output"
!
  char = m$index(icat,".cat")-1
  foctable = icat(1:{char})//"_foc.tbl"
show/icat {icat} n
outputi(1) = outputi(1)+2  ! empty line + result line
create/table {foctable} 5 {outputi(1)} null
!
create/column {foctable} :FRAME    " "      A30   C*64
create/column {foctable} :XPIX     "pixel"  F10.3 R*4
create/column {foctable} :YPIX     "pixel"  F10.3 R*4
create/column {foctable} :FOCPOS   "cm"     F12.6 R*8
create/column {foctable} :FWHM     "arcsec" F8.2  R*4
create/column {foctable} :FWHM_FIT "arcsec" F8.2  R*4
!
!
!
!          L O O P   W I T H   F R A M E S :
!          * * * * * * * * * * * * * * * *
!
!
catal = 0
catloop:
  store/frame topicima {icat} 1 end_catloop
  lopcount = lopcount+1
  show/icat {icat} n
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "image {lopcount} / {outputi(1)} : {topicima}"
  !
  !
  !        REMOVING 1ST COLUMN WITH VERY HIGH LEVEL
  !        AND FOR CERTAINTY ALSO 2ND ONE
  !        (TECHNICAL PROBLEM DUE TO WINDOW READOUT) :
  !        *****************************************
  !
        mess/out 4 {PROID} "  removing first two columns (tech. problem)"
    xtopicima = "x_{topicima}"
    subxstart = 3
    subxend = {{topicima},NPIX(1)}
  extract/image {xtopicima} = {topicima} [@{subxstart},<:@{subxend},>]
  !
  if nice_crea .eq. "y" then
    !
    !
    !      DISPLAYING OF EXTRACTED FRAME :
    !      *****************************
    !
        mess/out 4 {PROID} "  Displaying image :"
    !
    !***********************************************
    @p fors_nicedisp {xtopicima} ? n 0,0 ? ? 1.,2. n
    !***********************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of extracted frame failed !"
        mess/out 1 {PROID} "({xtopicima})"
      goto error
    endif
    !
        mess/out 4 {PROID} "    loading extracted image into display 1"
    assign/display d,1 >Null
    if outputi(1) .le. -2 then
      load/image {xtopicima} ? {outputi(1)},avg c,c
    else
      load/image {xtopicima} ? {outputi(1)} c,c
    endif
    label/display "ORIGINAL" {nice_dlab}  ! removed by special load/image
    !
  endif
  !
  !
  !        PHOTOMETRY :
  !        **********
  !
        mess/out 3 {PROID} "  Positions, Sky and Brightnesses :"
  !
        mess/out 4 {PROID} "    converting input frame: BDF -> FITS"
        mess/out 3 {PROID} " "
  !
    char = m$indexb(xtopicima,".")-1
    topicfits = xtopicima(1:{char})//".fits"


  ! Call Sextractor
  $ {p_bindir}/sex {topicfits} -c               {inparfile}  -
                               -FILTER_NAME     {convolfile} -
                               -STARNNW_NAME    {neuparfile} -
                               -PARAMETERS_NAME {outparfile}


  if m$exist("sex_output.dat") .eq. 0 then
        mess/out 1 {PROID} "Object detection failed !"
    goto error
  endif
  !
    photascii = xtopicima(1:{char})//"_phot.dat"
  -rename sex_output.dat {photascii}
  -delete {topicfits}
  !
        mess/out 4 {PROID} "    converting output frames: FITS -> BDF"
  !
    char = m$indexb(xtopicima,".")-1
    backima = xtopicima(1:{char})//"_skymap.bdf"
  indisk/fits sex_skymap.fits {backima} >Null
  -delete sex_skymap.fits
    foundima = xtopicima(1:{char})//"_skysub.bdf"
  indisk/fits sex_skysub.fits {foundima} >Null
  -delete sex_skysub.fits
  !
  !
  !        DISPLAYING OF BACKGROUND :
  !        ************************
  !
  if nice_crea .eq. "y" then
    !
    !*********************************************
    @p fors_nicedisp {backima} ? n 0,0 ? ? 1.,2. n
    !*********************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of background frame failed !"
        mess/out 1 {PROID} "({backima})"
      goto error
    endif
    !
        mess/out 4 {PROID} "    loading background image into display 2"
    assign/display d,2 >Null
    if outputi(1) .le. -2 then
      load/image {backima} ? {outputi(1)},avg c,c
    else
      load/image {backima} ? {outputi(1)} c,c
    endif
    label/display "BACKGROUND" {nice_dlab}  ! removed by special load/image
    !
  endif
  !
  !
        if mess_level .eq. 3 then
          mess/out 3 {PROID} "  Object quality determination ..."
        elseif mess_level .eq. 4 then
          mess/out 3 {PROID} "  Object quality determination :"
        endif
  !
  !
  !        CHECKING OF NUMBER OF FOUND OBJECTS :
  !        ***********************************
  !
        mess/out 4 {PROID} "    checking number of found objects"
  !
  open/file {photascii} read listcode
  read/file {listcode(1)} outputc
  if listcode(2) .eq. -1 then
        mess/out 2 {PROID} "    Not a single object found -"
        mess/out 2 {PROID} "    skipping {topicima} !"
    close/file {listcode(1)}
    skip = skip+1
    warn = 1
    !
    goto catloop
  endif
  close/file {listcode(1)}
  !
  !
  !        CREATION OF SINGLE OUTPUT TABLE :
  !        *******************************
  !
        mess/out 4 {PROID} "    creating object table"
    !
    open/file sex.fmt write formcode
    if {formcode(1)} .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open new file sex.fmt !"
      goto error
    endif
    write/file {formcode(1)} define/field  56  66 R*4 F10.3 :XPIX   "pixel"  ! 1
    write/file {formcode(1)} define/field  67  77 R*4 F10.3 :YPIX   "pixel"  ! 2
    write/file {formcode(1)} define/field  11  23 R*4 G13.6 :FLUX   "ADU"    ! 3
    write/file {formcode(1)} define/field 135 143 R*4 F8.2  :FWHM   "arcsec" ! 4
    write/file {formcode(1)} define/field  46  55 I*4 I8    :NPIX   "#"      ! 5
    write/file {formcode(1)} define/field 102 110 R*4 F8.2  :ELL_A "pix1sig" ! 6
    write/file {formcode(1)} define/field 111 119 R*4 F8.2  :ELL_B "pix1sig" ! 7
    write/file {formcode(1)} define/field 120 125 R*4 F6.1  :ROTANG "deg"    ! 8
    write/file {formcode(1)} define/field 144 147 C*9 A9    :FLAGS "CMSTADO" ! 9
    close/file {formcode(1)}
    !
  create/table tmp.tbl 14 1 {photascii} sex.fmt  ! 5 cols. added later,
  -delete {photascii}                            ! rows are expanded autom.
  -delete sex.fmt
  !
  copy/dd {xtopicima} *,5 tmp.tbl
  !
  !
  !        REMOVAL OF BAD OBJECTS FROM OUTPUT TABLE :
  !        ****************************************
  !
    char = m$indexb(xtopicima,".")-1
    phottable = xtopicima(1:{char})//"_obj.tbl"
  !
        mess/out 4 {PROID} "    cleaning object table"
  !
  select/table tmp.tbl :FLUX.gt.0.and.:FWHM.gt.0 >Null  ! FLUX because of prior
  selrows = outputi(1)                                  ! sky subtraction and
  if selrows .eq. 0 then                                ! FWHM because of bug
        mess/out 2 {PROID} "    No objects with positive flux found -"
        mess/out 2 {PROID} "    skipping {topicima} !"  ! in SExtractor !!!
    skip = skip+1
    warn = 1
    !
    goto catloop
  endif
  copy/table tmp.tbl gt0.tbl
  -rename gt0.tbl tmp.tbl
  !
  do iloop = 1 selrows
    if "{tmp.tbl,:FLAGS,{iloop}}" .eq. " 032" then  ! unused error code,
      tmp.tbl,:FLAGS,{iloop} = " 000"               ! objects are good
    endif
  enddo
  !
  -rename tmp.tbl {phottable}
  !
  !
  !        DISPLAYING OF FOUND OBJECTS :
  !        ***************************
  !
  ! For drawing ellipses around found objects:
  !
  create/column {phottable} :COLOR "1-8" F5.0 R*4  ! col.10 - for below
  !
  create/column {phottable} :XSTARTPIX "pixel" F12.7 R*8  ! col.11
  create/column {phottable} :YSTARTPIX "pixel" F12.7 R*8  ! col.12
  create/column {phottable} :XENDPIX   "pixel" F12.7 R*8  ! col.13
  create/column {phottable} :YENDPIX   "pixel" F12.7 R*8  ! col.14
  !
  compute/table {phottable} :XSTARTPIX = :XPIX-{ELLSIZE}*:ELL_A
  compute/table {phottable} :YSTARTPIX = :YPIX-{ELLSIZE}*:ELL_B
  compute/table {phottable} :XENDPIX   = :XPIX+{ELLSIZE}*:ELL_A
  compute/table {phottable} :YENDPIX   = :YPIX+{ELLSIZE}*:ELL_B
  !
  if nice_crea .eq. "y" then
    !
    !**********************************************
    @p fors_nicedisp {foundima} ? n 0,0 ? ? 1.,2. n
    !**********************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of found objects failed !"
        mess/out 1 {PROID} "({foundima})"
      goto error
    endif
    !
        mess/out 4 {PROID} "    loading image with found objects into disp. 3"
    assign/display d,3 >Null
    clear/channel overlay
    if outputi(1) .le. -2 then
      load/image {foundima} ? {outputi(1)},avg c,c
    else
      load/image {foundima} ? {outputi(1)} c,c
    endif
    !
  endif
  !
  !
  !        CALCULATION OF FWHM OF BRIGHTEST OBJECT :
  !        ***************************************
  !
  if m$existd(topicima,pixscale) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered desc. for pixel scale does not exist -"
        mess/out 1 {PROID} "({topicima},{pixscale})"
        mess/out 1 {PROID} "no calculation of FWHM possible !"
    goto error
  endif
  if {{topicima},{pixscale}} .le. 0 then
        set/format F15.14
        write/out " "
        mess/out 1 {PROID} "Impossible value for descriptor"
        mess/out 1 {PROID} "`{pixscale}': {{topicima},{pixscale}}"
        set/format {FMTDEF}
    goto error
  endif
  compute/table {phottable} :FWHM = :FWHM*{{topicima},{pixscale}} !pix.->arcsec
  !
        mess/out 3 {PROID} "  Calculating FWHM of brightest object :"
  !
  sort/table {phottable} :FLUX(-)  ! SExtractor may detect more objects
  fwhm = {{phottable},:FWHM,1}
  !
  show/table {phottable} >Null
  do iloop = 1 outputi(2)
    if iloop .eq. 1 then  ! used object
      {phottable},:COLOR,{iloop} = 4.
    else
      {phottable},:COLOR,{iloop} = 5.
    endif
  enddo
  !
  if nice_crea .eq. "y" then
    !
    label/display "FOUND OBJECTS" {nice_dlab}  ! after load/image - otherwise
    !                                          ! removed by special load/image
    label/display "Used" 10,10 ? green 1
    label/display "Others" 10,80 ? blue 1
    draw/ellipse {phottable} p n >Null
    !
  endif
  !
        set/format T9.3,
        mess/out 3 {PROID} "    FWHM (arcsec): {fwhm}"
        set/format {FMTDEF}
  !
  {foctable},:FRAME,{lopcount} = "{topicima}"
  {foctable},:XPIX,{lopcount} = {{phottable},:XPIX,1}
  {foctable},:YPIX,{lopcount} = {{phottable},:YPIX,1}
  {foctable},:FWHM,{lopcount} = fwhm
  if m$existd(topicima,focalpos) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered desc. for focal position does not exist -"
        mess/out 1 {PROID} "({topicima},{focalpos})"
        mess/out 1 {PROID} "no calculation of best focus possible !"
    goto error
  endif
  {foctable},:FOCPOS,{lopcount} = {{topicima},{focalpos}}
  !
  !
  goto catloop
end_catloop:
!
!
!          E N D   O F   L O O P   W I T H   F R A M E S
!          * * * * * * * * * * * * * * * * * * * * * * *
!
!
!
show/table {foctable} >Null
totrowno = outputi(2)
if totrowno .ge. MINOBJ then
  !
  !
  !        FITTING FOR BEST FOCAL POSITION :
  !        *******************************
  !
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "fitting FWHM versus focal position"
  !
  !***************************
  @p fors_focus_fit {foctable}
  !***************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Fitting of FWHM versus focal pos. failed !"
        mess/out 1 {PROID} "({foctable})"
    goto error
  endif
  !
endif
!
!
!          OUTPUT :
!          ******
!
if totrowno .ge. MINOBJ then
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} ">>> FWHMs / focal positions"
        mess/out 4 {PROID} "    are in <{foctable}>. <<<"
        mess/out 4 {PROID} ">>> OUTPUTD(1): best focal pos. [cm]  <<<"
        mess/out 4 {PROID} ">>> OUTPUTR(1): fwhm thereof [arcsec] <<<"
        !
        set/format F8.2,F12.6
        longlog/out 1 "     best focal pos. [cm] : {outputd(1)}" >Null
        longlog/out 1 "     fwhm thereof [arcsec]: {outputr(1)}" >Null
        set/format {FMTDEF}
else
        write/out " "
        mess/out 1 {PROID} "Found objects on less than {MINOBJ} frames !"
        !
        longlog/out 1 "     Found objects on less than {MINOBJ} frames !"
  goto error
endif
!
!
set/format
if nice_crea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0 {skip}
else
  return -1 {skip}
endif
!
error:
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" set/graphic default
  endif
  return 1 -99
!
!
!-------------------------------------------------------------------------------
