! $Id: fors_focus_fit.prg,v 1.30 2000/02/23 14:49:33 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_focus_fit.prg,v $
!
! .KEYWORDS   Special table fitting
!
! .PURPOSE    Fitting of smallest FWHMs versus focal position in the focus
!             output table.
! __________
!
! .USAGE      @p fors_focus_fit tabcol
!
! .INPUT      tabcol (P1): name of focus table AND names of columns with frame
!                          names/focal positions/FWHM/fitted FWHM, separated by
!                          commas;
!                            default: none,FRAME,FOCPOS,FWHM,FWHM_FIT
!
! .OUTPUT     OUTPUTD(1): Best focal position
!             OUTPUTR(1): Smallest full width at half maximum
!
!             input table with (fitted -if displaying enabled- and)
!             "best" values
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Only the smallest FWHM and a few neigbours (adjustable constant)
!             around its focal position are used for a quadratical fit.
!
! .COMMENT    Special procedure for focal alignment - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/23  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!------------------------------------------------------------------------------
!
!
crossref tabcol
!
!
!  MINOBJ is "+lower_levels" in fors_focus.prg
if m$existk("MINOBJ") .eq. 0 define/local MINOBJ/i/1/1 5  ! >=3 !
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "focus_fit"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      tabcol (P1): name of focus table AND names of columns with frame"
  w/o "                   names/focal positions/FWHM/fitted FWHM, separated by"
  w/o "                   commas;"
  w/o "                     default: none,FRAME,FOCPOS,FWHM,FWHM_FIT"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(tabcol):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter focus table, cols.[def.:FRAME,FOCPOS,FWHM,FWHM_FIT]: "
outputi = m$parse(p1,"tbl")
if outputi(1) .ne. 5 .and. outputi(1) .ne. 1 then  ! 1 allowed for col.defaults
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P1 !"
  goto error
endif
define/local foctable/c/1/64 "{tbl01}"
if m$index(foctable,".tbl") .eq. 0 foctable = foctable//".tbl"
if outputi(1) .eq. 5 then
  define/local cframe/c/1/16 "{tbl02}"
  define/local cfocpos/c/1/16 "{tbl03}"
  define/local cfwhm/c/1/16 "{tbl04}"
  define/local cfwhmfit/c/1/16 "{tbl05}"
  if cframe(1:1) .eq. ":" cframe = cframe(2:)
  if cfocpos(1:1) .eq. ":" cfocpos = cfocpos(2:)
  if cfwhm(1:1) .eq. ":" cfwhm = cfwhm(2:)
  if cfwhmfit(1:1) .eq. ":" cfwhmfit = cfwhmfit(2:)
elseif outputi(1) .eq. 1 then  ! defaults
  define/local cframe/c/1/16 "FRAME"
  define/local cfocpos/c/1/16 "FOCPOS"
  define/local cfwhm/c/1/16 "FWHM"
  define/local cfwhmfit/c/1/16 "FWHM_FIT"
endif
!
if m$existc(foctable,":{cframe}") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({foctable})"
  goto error
elseif m$existc(foctable,":{cframe}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({foctable},:{cframe})"
  goto error
elseif m$existc(foctable,":{cfocpos}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({foctable},:{cfocpos})"
  goto error
elseif m$existc(foctable,":{cfwhm}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({foctable},:{cfwhm})"
  goto error
elseif m$existc(foctable,":{cfwhmfit}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({foctable},:{cfwhmfit})"
  goto error
endif
!
show/table {foctable} >Null
define/local totrowno/i/1/1 {outputi(2)}
if totrowno .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({foctable})"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local lowframe/c/1/64 " " all
  define/local currframe/c/1/64 " " all
  define/local tloop/i/1/1 0
  define/local lowentry/i/1/1 0
  define/local neighbours/i/1/1 0
  define/local lowlimit/i/1/2 0 all
  define/local lowrange/i/1/2 0 all
  define/local bestfocpos/d/1/1 0.
  define/local bestfwhm/r/1/1 0.
!
!
!
!          PLOTTING FWHMS VERSUS FOCAL POSITIONS :
!          *************************************
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "  plotting FWHMs versus focal positions"
  !
  sort/table {foctable} :{cfocpos}
  assign/graphic g,1 >Null
  set/graphic stype=1 ltype=0 colour=2
  plot/table {foctable} :{cfocpos} :{cfwhm} >Null
endif
!
!
!          SEARCHING SMALLEST FWHM AND NEIGHBOURS AROUND ITS FOC.POS. :
!          **********************************************************
!
        mess/out 4 {PROID} "  extracting smallest FWHMs"
!
sort/table {foctable} :{cfwhm}
lowframe = "{{foctable},:{cframe},1}"
sort/table {foctable} :{cfocpos}
do tloop = 1 totrowno
  write/keyword currframe "{{foctable},:{cframe},{tloop}}" >Null
  if currframe .eq. lowframe lowentry = tloop
enddo
if MINOBJ .ge. 5 then                          ! use 2 values at each side of
  neighbours = 2                               ! smallest FWHM
elseif MINOBJ .lt. 5 .and. MINOBJ .ge. 3 then  ! use 1 value at each side of
  neighbours = 1                               ! smallest FWHM
elseif MINOBJ .lt. 3 then
        write/out " "
        mess/out 1 {PROID} "No computation of best focus position possible -"
        mess/out 1 {PROID} "constant `MINOBJ' in "{PROID}" must be greater"
        mess/out 1 {PROID} "than or equal 3 !"
  goto error
endif
lowlimit(1) = neighbours+1
lowlimit(2) = totrowno-neighbours
if lowentry .lt. lowlimit(1) .or. lowentry .gt. lowlimit(2) then
        write/out " "
        mess/out 1 {PROID} "No computation of best focus position possible -"
        mess/out 1 {PROID} -
          "in each direction at least {neighbours} position values are"
        mess/out 1 {PROID} "needed beneath the fwhm minimum !"
  goto error
endif
lowrange(1) = lowentry-neighbours
lowrange(2) = lowentry+neighbours
!
if nice_crea .eq. "y" then
  create/column {foctable} :Index " " I5 I*2  ! for copy/tt below
  do outputi = 1 totrowno
    {foctable},:Index,{outputi} = outputi
  enddo
  set/refcol {foctable} :Index
endif
select/table {foctable} SEQ.ge.{lowrange(1)}.and.SEQ.le.{lowrange(2)} >Null
copy/table {foctable} tmpfoc.tbl
if nice_crea .eq. "y" then
  set/refcol tmpfoc.tbl :Index
endif
!
!
!          COMPUTATION OF BEST FOCUS POSITION :
!          **********************************
!
        mess/out 4 {PROID} "  computing best focus position"
!
regression/poly tmpfoc.tbl :{cfwhm} :{cfocpos} 2 >Null
if outputd(3) .le. 0. then
        write/out " "
        mess/out 1 {PROID} "No computation of best focus position possible -"
        mess/out 1 {PROID} "second derivation of quadratic fit is lower than"
        mess/out 1 {PROID} "or equal zero !"
        mess/out 1 {PROID} "  Values used for fit ( {cfwhm}({cfocpos}) ) :"
  read/table tmpfoc.tbl
        write/out " "
  goto error
endif
bestfocpos = (-1.*outputd(2))/(2.*outputd(3))
bestfwhm = outputd(3)*(bestfocpos**2)+outputd(2)*bestfocpos+outputd(1)
!
!
!          PLOTTING FITTED EXTRACTED FWHMS VERSUS FOC. POS. :
!          ************************************************
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "  plotting fit.extract. FWHMs versus foc.pos."
  !
  save/regression tmpfoc.tbl REGRESS
  !
  compute/regression tmpfoc.tbl :{cfwhmfit} = REGRESS
  copy/dd tmpfoc.tbl REGRESS* {foctable}
  copy/tt tmpfoc.tbl :{cfwhmfit} {foctable} :{cfwhmfit}
  !
  set/graphic stype=6 ltype=0 colour=3
  overplot/table {foctable} :{cfocpos} :{cfwhmfit} >Null
  set/graphic stype=0 ltype=1 colour=3
  overplot/table {foctable} :{cfocpos} :{cfwhmfit} >Null
  set/format ,E14.7  ! for overplot/line double length as char. too long
  overplot/line 3 {plrgrap(1)},{bestfwhm} {bestfocpos},{bestfwhm}
  overplot/line 3 {bestfocpos},{plrgrap(5)} {bestfocpos},{bestfwhm}
  set/format {FMTDEF}
endif
!
-delete tmpfoc.tbl
select/table {foctable} all
if nice_crea .eq. "y" then
  delete/column {foctable} :Index
endif
!
!
!          RESULTS :
!          *******
!
        set/format F8.2,F12.6
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "  Fit with smallest FWHMs =>"
        set/format ,T12.6
        mess/out 3 {PROID} "    Focal position [cm]: {bestfocpos}"
        set/format T8.2,
        mess/out 3 {PROID} "    FWHM thereof [arcs]: {bestfwhm}"
        set/format {FMTDEF}
!
!
!          OUTPUT :
!          ******
!
outputi(2) = totrowno+2
write/table {foctable} :{cframe} @{outputi(2)} -
  "                          BEST"  ! 26 blanks
write/table {foctable} :{cfocpos} @{outputi(2)} {bestfocpos}
write/table {foctable} :{cfwhm} @{outputi(2)} {bestfwhm}
outputd(1) = bestfocpos
outputr(1) = bestfwhm
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  if m$existk("foctable") .eq. 1 then
    if m$exist(foctable) .eq. 1 select/table {foctable} all
  endif
  return 1
!
!
!------------------------------------------------------------------------------
