! $Id: fors_frmclass.prg,v 1.11 1998/10/28 13:53:29 rpalsa Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_frmclass.prg,v $
!
!.KEYWORDS    Update data product keywords in frame header.
!
!.USAGE       classify/frame  frame [idstrng] [dpr_catg] [dpr_type] [dpr_tech]
!
!.INPUT       frame       (P1)    Frame to be updated.
!             idstrng     (P2)    Frame category name according to basic and
!                                 product data definitions.
!             dpr_catg    (P3)    New data product category.
!             dpr_type    (P4)    New data product type.
!             dpr_tech    (P5)    New data product observing technique.
!
!.OUTPUT      None
!
!.RETURNS     The recipe returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The procedure updates the frame header keywords specifying the
!             frame identification and data product classification. 
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/23:  Creation
!
!------------------------------------------------------------------------------

crossref frame idstrng category type technique

define/param p1 ?  c    "Enter frame:"
define/param p2 +  c    "Enter BDD/PDD identification string:"
define/param p3 +  c    "Enter data product category:"
define/param p4 +  c    "Enter data product type:"
define/param p5 +  c    "Enter data product technique:"

define/maxpar 5


! Procedure identification
define/local procID/c/1/80      "FrmClass"


! Local variables
define/local frame/c/1/100  {p1}
define/local id_strng/c/1/100  " "  all
define/local dpr_catg/c/1/100  " "  all
define/local dpr_type/c/1/100  " "  all
define/local dpr_tech/c/1/100  " "  all



id_strng = m$upper(p2)
dpr_catg = m$upper(p3)
dpr_type = m$upper(p4)
dpr_tech = m$upper(p5)


! Optionally a classification string according to the data definitions
! is written to the descriptor specified by FORS_IDSTRNG
if id_strng .ne. "+" then
  if m$existd(frame,FORS_IDSTRNG) .eq. 1 then
    delete/descr {frame} {FORS_IDSTRNG}
  else
    mess/out 3 {procID} "Creating descriptor {FORS_IDSTRNG}!"
  endif
  write/descr {frame} {FORS_IDSTRNG}/c/1/25 "{id_strng}"
endif
    

! Optionally the data product identifiers may be altered.
if dpr_catg .ne. "+" then
  if m$existd(frame,H_DPRCATG) .eq. 1 then
    delete/descr {frame} {H_DPRCATG}
  else
    mess/out 3 {procID} "Creating descriptor {H_DPRCATG}!"
  endif
  write/descr {frame} {H_DPRCATG}/c/1/15 "{dpr_catg}"
endif

if dpr_type .ne. "+" then
  if m$existd(frame,H_DPRTYPE) .eq. 1 then
    delete/descr {frame} {H_DPRTYPE}
  else
    mess/out 3 {procID} "Creating descriptor {H_DPRTYPE}!"
  endif
  write/descr {frame} {H_DPRTYPE}/c/1/15 "{dpr_type}"
endif

if dpr_tech .ne. "+" then
  if m$existd(frame,H_DPRTECH) .eq. 1 then
    delete/descr {frame} {H_DPRTECH}
  else
    mess/out 3 {procID} "Creating descriptor {H_DPRTECH}!"
  endif
  write/descr {frame} {H_DPRTECH}/c/1/15 "{dpr_tech}"
endif

return 0
