! $Id: fors_gen_key_purge.prg,v 1.4 1999/11/25 11:23:54 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_gen_key_purge.prg,v $
!
! .KEYWORDS   Descriptors
!
! .PURPOSE    Frame classification for fors_purge.prg:
!             Clearage of no longer needed descriptors in images or tables.
! __________
!
! .USAGE      @p fors_gen_key_purge frmcat inicat
!
! .INPUT      frmcat (P1): image catalogue with product images or tables;
!                            default: none
!
!             inicat (P2): image catalogue with all frames accepted by
!                          "initialize/pipeline" - needed table with purging
!                          informations is taken from it;
!                            default: none
!
! .OUTPUT     -
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  All descriptors which are marked in the table with the purging in-
!             formations (per frame type) are removed from the headers.
!
! .COMMENT    -
!
! .ENVIRON    FORS context
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.3  99/05/18  Creation
!                  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref frmcat inicat
!
!
define/local PROID/c/1/40 "gen_key_purge"
!
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      frmcat (P1): image catalogue with product images or tables;"
  w/o "                     default: none"
  w/o " "
  w/o "      inicat (P2): image catalogue with all frames accepted by"
  w/o "                   "initialize/pipeline" - needed table with purging"
  w/o "                   informations is taken from it;"
  w/o "                     default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          INPUT :
!          *****
!
!P1(frmcat):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter icat. with product images or tables : "
define/local frmcat/c/1/64 "{p1}"
if m$index(frmcat,".cat") .eq. 0 frmcat = frmcat//".cat"
!
! Tests are done in main procedure below.
!
!
!P2(inicat):
!----------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? i -
  "        Enter icat. with all accepted frames : "
define/local inicat/c/1/64 "{p2}"
if m$index(inicat,".cat") .eq. 0 inicat = inicat//".cat"
!
if m$exist(inicat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({inicat})"
 goto error
endif
!
show/icat {inicat} n
if outputi(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is empty !"
        mess/out 1 {PROID} "({inicat})"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local catal/i/1/1 0
  define/local topicfrm/c/1/76 " " all
  define/local id/c/1/25 " " all
  !
  define/local tblID/c/1/25 " " all
  define/local purgetbl/c/1/76 " " all
  define/local ref/i/1/1 0
!
!
!
!          IDENTIFICATION VIA DESCRIPTORS,
!          MAPPING OF TABLE TO PARAMETER  :
!          *******************************
!
        mess/out 4 {PROID} "Identifying table"
!
tblID = "{PURGET_ID}"
ref = 0
!
ident_loop:
  store/frame topicfrm {inicat} 1 end_ident_loop
  id = "{{topicfrm},{fors_idstrng}}"
  !
  if id .eq. "{tblID}" then
    purgetbl = topicfrm
        mess/out 4 {PROID} "{topicfrm} : {id} found"
    ref = 1
  endif
  !
  goto ident_loop
end_ident_loop:
!
if ref .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No table with purging info present !"
  goto error
endif
!
!
!          *********************************
!          *                               *
!          *  CALLING OF MAIN PROCEDURE :  *
!          *                               *
!          *********************************
!
        mess/out 4 {PROID} "Applying procedure "fors_purge" :"
!
!***********************************************
@p fors_purge {frmcat} {purgetbl} {fors_idstrng}
!***********************************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Purging of header(s) failed !"
  goto error
endif
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  write/out " "
  return 1
!
!
!-------------------------------------------------------------------------------
