! $Id: fors_get_imsec.prg,v 1.31 2000/01/26 20:19:34 nrainer Exp $   
!+++ FORS_GET_IMSEC +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_get_imsec.prg,v $
!
!.KEYWORDS      Determine the image section (frame without overscan areas)
!
!.USAGE         @p fors_get_imsec in_frm rdir
!
!.INPUT         in_frm  (P1): name of input frame to be examined
!               rdir    (P2): readout direction
!
!.OUTPUT        - OUTPUTC will be set to the used image section interval
!                 in world coordinates (MIDAS-syntax for intervals used).
!               - OUTPUTI will contain the limits as integer values
!                  and the binning-corrected pre-/postscan pixels:
!                   1,2   lower left corner of used image section  [index]
!                   3,4   upper right corner of used image section [index]
!                   5,6   pre- and postscan pixels (x-direction)
!                   7,8   pre- and postscan pixels (y-direction)
!                   9,10  binning in x and y
!               - OUTPUTD will contain the limits as double values
!           1,2 lower left corner of used image section  [world]
!                   3,4 upper right corner of used image section [world]
!
!                     (2,3)
!                   ##+------+##
!                   ##|      |##   y^
!                   ##| CCD  |##    |
!                   ##+------+##    +->x
!                    ^(1,2)    ^
!                    pre-      postscan
!
!               - INPUTI will contain the CCD-coordinates of the input frame,
!                 especially of interest for window readouts
!
!
!.RETURN        Q1:  -1: wrong readout direction specified
!                     0: no ERRORS occured
!                     1: unknown port number detected
!                     2: more than ONE window used
!                        (procedure works only with ONE window)
!                     3: FITS-keyword missing to determine number
!                        of windows used
!                     4: Instrument identification failure
!                     5: incorrect pre-/postscan value
!
!.DEFAULT
!
!.PURPOSE       Determine the image section (frame without overscan areas)
!
!.ALGORITHM     Read scan descriptors of the header of the image file.
!               If the overscan descriptors are missing the image section is
!               assumed to be the complete frame ([<,<:>>]). Consequently,
!               the pre- and postscan sizes returned in outputi are set to 0.
!
!.ENVIRON       FORS, CCDRED context
!
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1998/02/11  Creation
!               1.0 1998/09/03  Check old FIERA-controler software (OFFSET=3)
!               1.0 1998/09/21  OUTPUTC now represents world-coordinate area
!
!.COMMENT       Uses global keywords FORS_NOUT, FORS_OUTI which are assumed
!               to be properly set by a previous call of fors_getports !
!
!------------------------------------------------------------------------------
!       _                !x1,y1
!       ^:::||||||||||||||:::
!        :::|| C |||||||||:::           x0 = prx*xstp + START(1)
!NPXI(2) :::|||| C |||||||:::           y0 = pry*ystp + START(2)
!        :::|||||| D |||||:::           x1 = xmax - ovx*xstp
!        :::||||||||||||||:::           y1 = ymax - ovy*ystp
!       _:::||||||||||||||:::
!        !  !x0,y0          !           xmax = (NPIX(1)-1)*xstp + x0
!        !START(1),START(2) !           ymax = (NPIX(2)-1)*ystp + y0
!        !<---- NPIX(1) --->!
!
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name IMAGE frame:"
define/param p2 X c "Enter readout direction (X, Y):"
!
! local variable declaration
!
define/local x0/d/1/1 1
define/local x1/d/1/1 1
define/local y0/d/1/1 1
define/local y1/d/1/1 1
define/local xmax/d/1/1 1
define/local ymax/d/1/1 1
define/local xstp/d/1/1 0
define/local ystp/d/1/1 0
define/local xdim/i/1/1 0
define/local ydim/i/1/1 0
define/local i1x/i/1/1 0
define/local i0x/i/1/1 0
define/local i1y/i/1/1 0
define/local i0y/i/1/1 0
define/local prx/i/1/1 0
define/local ovx/i/1/1 0
define/local pry/i/1/1 0
define/local ovy/i/1/1 0
define/local binx/i/1/1 1
define/local biny/i/1/1 1
define/local nwin/i/1/1 1
define/local port/i/1/1 1
define/local nport/i/1/1 1
define/local dumy/i/1/1 0
define/local FORS/i/1/1 0
define/local MAXPORT/i/1/1 4            !maximum number of ports (FIERA)
define/local chkx/i/1/1 1
define/local chky/i/1/1 1
define/local winread/i/1/1 0            !window readout (0: NO, 1: YES)
define/local instrname/c/1/10 " " ALL
define/local procid/c/1/20 get_imsec
define/local warn1/c/1/80 " " ALL
warn1 = "Wrong scan region definition!\n--> Take complete image!"

set/format I1
write/keyw OUTPUTI/i/1/10 0 ALL
xdim = m$value({P1},NPIX(1))
ydim = m$value({P1},NPIX(2))
xstp = m$value({P1},STEP(1))
ystp = m$value({P1},STEP(2))


@p fors_getwin {P1}                             !check for window readout
winread = {Q1}                                  !1: window readout
inputi(1) = outputi(1)
inputi(2) = outputi(2)
inputi(3) = outputi(3)
inputi(4) = outputi(4)
inputi(5) = winread

i0x = 1
i0y = 1
i1x  = xdim !+ i0x-1
i1y  = ydim !+ i0y-1

@a getcoord {P1} @{i0x},@{i0y} >Null
x0   = outputd(1)                                       !lower left w-coord.
y0   = outputd(2)                                       !.. of complete frame
@a getcoord {P1} @{i1x},@{i1y} >Null
x1   = outputd(1)                                       !upper right w-coord.
y1   = outputd(2)                                       !.. of complete frame
xmax = x1
ymax = y1


OUTPUTC = "[<,<:>,>]"                                   !complete CCD range
OUT_B = "[@{i0x},@{i0y}:@{i1x},@{i1y}]"
write/keyw OUTPUTI/i/1/14 -
     {i0x},{i0y},{i1x},{i1y},{prx},{ovx},{pry},{ovy},{binx},{biny}
  

port = fors_outi(1)
if port .lt. 1 .OR. port .gt. {MAXPORT} then
  mess/out 1 {procid} "Unknown port number: {port}."
  mess/out 1 {procid} "--> {warn1}"
  return 1
else
  mess/out 4 {procid} "Check for CCD-PORT number {port}"
endif

if m$existd(P1,"{h_nwin}") .eq. 1 then
  nwin = m$value({P1},{h_nwin})                 !number of windows used
  if nwin .gt. 1 then
    mess/out 2 {procid} "More than ONE window used ({nwin})"
    mess/out 2 {procid} "--> {warn1}"
    return 2
  endif
else
  mess/out 2 {procid} -
    "Number of used WINDOWS not detectable!\nAppropriate Keyword is missing!"
!  mess/out 2 {procid} "--> {warn1}"
!  return 3
endif

if m$existd(P1,"{h_xwinbin}") .eq. 1 then
  binx = m$value({P1},{h_xwinbin})              !reference output
  if binx .gt. 1 mess/out 4 {procid} "Binning-x: {binx}"
endif
if m$existd(P1,"{h_ywinbin}") .eq. 1 then
  biny = m$value({P1},{h_ywinbin})              !reference output
  if biny .gt. 1 mess/out 4 {procid} "Binning-y: {biny}"
endif



if winread .eq. 1 then                          !window readout
  goto fini                                     !already finish as there is
endif                                           !no overscan region.




copy/dk {P1} {h_instrume} instrname

if instrname(1:4) .eq. "{FORS_NAME(1:4)}" then
  append/out "OLD OVERSCAN-definition in FIERA controler!\n"
  append/out "be corrected if necessary !\n"
  append/out "==> {instrname}: Subtract OFFSET=3 (= dumped columns)"
  mess/out 4 {procid}
  FORS = 1
else if instrname(1:4) .ne. "{TC_NAME(1:4)}" then
  mess/out 1 {procid} "Instrument could no be identified!\nNecessary due to NON-standards in controler software!"
  mess/out 2 {procid} "--> {warn1}"
  return 4
endif



if P2(1:1) .eq. "X" then
  if m$existd(P1,"{h_oscan({port})}") .eq. 1 then       !POSTSCAN
    ovx = m$value({P1},{h_oscan({port})})
    if ovx .eq. 19 .AND. FORS .eq. 1 ovx = 16           !old FIERA-controler
!    ovx = ovx / binx
    i1x = xdim - ovx
    @a getcoord {P1} @{i1x},@{i1y} >Null
    x1  = outputd(1)
    if i1x .gt. xdim .OR. i1x .lt. 1 then
      mess/out 2 {procid} "1: BAD X-postscan value {ovx} !"
      mess/out 2 {procid} "--> {warn1}"
      return 5
    endif
  else
    ovx = 0
    chkx = 0
  endif
  !
  if FORS_NOUT .eq. 4 then
    !
    if m$existd(P1,"{h_pscan(2)}") .eq. 1 then       !PRESCAN
      ovx = m$value({P1},{h_pscan(2)})
      if ovx .eq. 19 .AND. FORS .eq. 1 ovx = 16           !old FIERA-controler
!      ovx = ovx / binx
      i1x = xdim - ovx
      @a getcoord {P1} @{i1x},@{i1y} >Null
      x1  = outputd(1)
      if i1x .gt. xdim .OR. i1x .lt. 1 then
        mess/out 2 {procid} "1: BAD X-postscan value {ovx} !"
        mess/out 2 {procid} "--> {warn1}"
        return 5
      endif
    else
      ovx = 0
      chkx = 0
    endif
    !
  endif
  !
  if m$existd(P1,"{h_pscan({port})}") .eq. 1 then       !PRESCAN
    prx = m$value({P1},{h_pscan({port})})
    if prx .eq. 19 .AND. FORS .eq. 1 prx = 16           !old FIERA-controler
!    prx = prx / binx
    i0x = prx + 1
    @a getcoord {P1} @{i0x},@{i0y} >Null
    x0  = outputd(1)
    if i0x .gt. xdim .OR. i0x .lt. 1 then
      mess/out 2 {procid} "BAD X-prescan value {prx} !"
      mess/out 2 {procid} "--> {warn1}"
      return 5
    endif
    if i1x .le. i0x then
      mess/out 2 {procid} "2: BAD X-postscan value {ovx} !"
      mess/out 2 {procid} "--> {warn1}'
      return 5
    endif
  else
    prx = 0
    chkx = 0
  endif
else if P2(1:1) .eq. "Y" then
  if m$existd(P1,"{h_oscany({port})}") .eq. 1 then
    ovy = m$value({P1},{h_oscany({port})})
    i1y = ydim - ovy
    @a getcoord {P1} @{i1x},@{i1y} >Null
    y1  = outputd(2)
    if i1y .gt. ydim .OR. i1y .lt. 1 then
      mess/out 2 {procid} "1: BAD Y-postscan value {ovy} !"
      mess/out 2 {procid} "--> {warn1}"
      return 5
    endif
  else
    ovy = 0
    chky = 0
  endif
  if m$existd(P1,"{h_pscany({port})}") .eq. 1 then
    pry = m$value({P1},{h_pscany({port})})
    i0y = pry + 1
    @a getcoord {P1} @{i0x},@{i0y} >Null
    y0  = outputd(2)
    if i0y .gt. ydim .OR. i0y .lt. 1 then
      mess/out 2 {procid} "BAD Y-prescan value {pry} !"
      mess/out 2 {procid} "--> {warn1}"
      return 5
    endif
    if i1y .le. i0y then
      mess/out 2 {procid} "1: BAD Y-postscan value {ovy} !"
      mess/out 2 {procid} "--> {warn1}"
      return 5
    endif
  else
    pry = 0
    chky = 0
  endif
else
  mess/out 1 {procid} "NO valid readout direction specified (use P2)!"
  return -1
endif

if chkx .eq. 0 .AND. chky .eq. 0 then
  mess/out 2 {procid} "NO pre-/postscan keywords found"
  mess/out 2 {procid} "{warn1}"
endif

fini:
write/keyw OUTPUTI/i/1/10 -
        {i0x},{i0y},{i1x},{i1y},{prx},{ovx},{pry},{ovy},{binx},{biny}
write/keyw OUTPUTD/d/1/4 {x0},{y0},{x1},{y1}

set/format I1 ,F0.10
OUTPUTC = "[{x0},{y0}:{x1},{y1}]"
OUT_B   = "[@{outputi(1)},@{outputi(2)}:@{outputi(3)},@{outputi(4)}]"
append/out "Image section ({P1}):\n"
append/out "(world): {outputc}\n"
append/out "(pixel): {out_b}"
mess/out 4 {procid}
set/format
return 0

