! $Id: fors_getairmass.prg,v 1.5 1999/08/25 11:04:46 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_getairmass.prg,v $
!
!.KEYWORDS    Airmass determination, spectroscopy, photometry
!
!.USAGE       get/airmass frame
!
!.INPUT       frame    (P1)   Input frame for which the airmass is determined.
!
!.OUTPUT      Keyword OUTPUTR(1) is updated with the airmass value.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Calculate airmass for a given frame within an error handler.
!             COMPUTE/AIRMASS does the actual computation.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/11/17:  Creation
!
!------------------------------------------------------------------------------

crossref  frame

define/param p1 ?  c  "Enter input frame:"
define/maxpar 1


! Procedure identification
define/local procID/c/1/10      "GetAirmass"


! Local constants
define/local check_time/r/1/1  1997.0
define/local reffrm/c/1/20     "tmp_airmass.bdf"


! Local variables
define/local tflag/i/1/1    0
define/local geolon/r/1/1   0.
define/local geolat/r/1/1   0.
define/local airmass/r/1/1  0.
define/local frm/c/1/100    " "  all
define/local ofrm/c/1/100   " "  all



! To handle tables in the same way as images we have to copy the table
! descriptors necessary for the airmass calculation to a temporary image
tflag = m$filtyp(p1,".bdf")
if tflag .eq. 2 then
  ofrm = "{p1}"
  create/image {reffrm} ? 0 nodata
  copy/dd {p1} *,3 {reffrm}
  frm = reffrm
else
  ofrm = "{p1}"
  frm = ofrm
endif

 
! Get geographic longitude and latitude from header
if m$existd(frm,h_geolon) .eq. 1 then
  geolon = m$value({frm},{h_geolon})
else
  mess/out 1 {procID} "Missing Descriptor {h_geolon} in {ofrm}!"
  return 1
endif

if m$existd(frm,h_geolat) .eq. 1 then
  geolat = m$value({frm},{h_geolat})
else
  mess/out 1 {procID} "Missing Descriptor {h_geolat} in {ofrm}!"
  return 1
endif


! Check descriptor O_TIME(1)
if {{frm},O_TIME(1)} .le. check_time then
  mess/out 2 {procID} "Invalid observation date in O_TIME(1). Recalculating!"
  @p fors_jd2date {frm}
  {frm},O_TIME(1) = outputi(1)
  {frm},O_TIME(2) = outputi(2)
  {frm},O_TIME(3) = outputi(3)
  mess/out 2 {procID} -
           "Descriptors O_TIME(1) to O_TIME(3) updated with new values."
endif


! Calculate airmass
compute/airmass {frm} {geolon} {geolat} >Null
airmass = outputr(1)                         ! save result


! Check airmass
set/format F8.4
if airmass .lt. 1. .or. airmass .gt. 10. then
  mess/out 1 {procID} "Invalid airmass value {airmass} encountered!"
  mess/out 1 {procID} -
             "Check descriptors O_TIME, O_POS, {h_geolon} and {h_geolat}!"
  return 1  
else
  mess/out 3 {procID} "Airmass: {airmass} obtained from"
  mess/out 3 {procID} "         {ofrm}."
  longlog/out 1 "     Airmass: {airmass} obtained from"
  longlog/out 1 "              {ofrm}."
endif
set/format


! Cleanup
if tflag .eq. 2 then
  copy/dd {frm} *,3 {ofrm}
  -delete {reffrm}
endif


! Update output keyword
outputr(1) = airmass

return 0
