! $Id: fors_getfilter.prg,v 1.8 1999/08/31 10:14:43 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_getfilter.prg,v $
!
!.KEYWORDS    Determine instrument filter setup
!
!.USAGE       config/filter  frame
!
!.INPUT       frame    (P1)   Input frame from which the instrument filter
!                             setup is extracted.
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Get the number and the names of all active filters.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/07/03:  Creation
!             1.3  1999/08/31:  set/format added (SB).
!
!------------------------------------------------------------------------------

crossref  frame


define/param p1 ? c "Enter name of input frame:"
define/maxpar 1


! Procedure identification
define/local procID/c/1/80      "ForsGetFilter"


! Local variables
define/local i/i/1/1         0
define/local nf/i/1/1        1
define/local lockno/i/1/2    0     all
define/local frame/c/1/100   {p1}
define/local dscname/c/1/80  " "   all


set/format I1


! Unlock keywords if necessary
info/keyw fors_nfilt
lockno(1) = mid$info(4)
if lockno(1) .ne. 0 then
  unlock/key fors_nfilt {lockno(1)}
endif

info/keyw fors_filti
lockno(2) = mid$info(4)
if lockno(2) .ne. 0 then
  unlock/key fors_filti {lockno(2)}
endif


! Reset filter names
do i = 1 FORS_MAXFILT 1
  fors_filti({i}) = " "
enddo


! Check for filter keyword without index
dscname = kbase_filti // ksuff_filti
if m$existd(frame,dscname) .eq. 1 then
  fors_filti({nf}) = "{{frame},{dscname}}"
  nf = nf + 1
else
! Check for filter keywords with index
  set/format i1
  do i = 1 FORS_MAXFILT 1
    dscname = kbase_filti // "{i}" // ksuff_filti
    if m$existd(frame,dscname) .eq. 1 then
      fors_filti({nf}) = "{{frame},{dscname}}"
      nf = nf + 1
    endif
  enddo
  set/format
endif

fors_nfilt = nf - 1


! Protect keywords
if lockno(1) .ne. 0 then
  lock/key fors_nfilt {lockno(1)}
endif

if lockno(2) .ne. 0 then
  lock/key fors_filti {lockno(2)}
endif

return 0
