! $Id: fors_getflux.prg,v 1.2 1999/03/15 09:58:34 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_getflux.prg,v $
!
!.KEYWORDS    Extract reference flux spectrum from flux database,
!             3d tables, spectroscopy
!
!.USAGE       get/refflux database entry spectrum
!
!.INPUT       database    (P1)    Reference spectrum database (3d table).
!             standards   (P2)    Spectrum entry number in database table
!                                 (table row number)
!
!.OUTPUT      spectrum    (P3)    Extracted spectrum table.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status.
!
!.PURPOSE     Extract wavelength, flux and flux error columns for a given 
!             entry number (table row number) in the standard star flux
!             database from the data base and store it in an ordinary
!             MIDAS table. 
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/12/02:  Creation
!
!------------------------------------------------------------------------------

crossref  database entry spectrum

define/param p1 ? c "Enter standard star reference flux database:"
define/param p2 ? c "Enter database entry number:"
define/param p3 ? c "Enter standard reference spectrum:"
define/maxpar 3


! Procedure identification
define/local procID/c/1/20      "GetRefflux"


! Local variables
define/local ndata/i/1/1      0
define/local dbentry/i/1/1    0
define/local dbfile/c/1/100   {p1}
define/local spcfile/c/1/100  {p3}



! Check entry number
if m$tstno(p2) .eq. 0 then
  mess/out 1 {procID} "Flux database entry is not a number!"
  return 1
else
  dbentry = {p2}
  if dbentry .lt. 1 .or. dbentry .gt. m$value({dbfile},tblcontr(4)) then
    mess/out 1 {procID} "Invalid flux database entry number!"
    return 1
  endif
endif


! Get number of values to copy
ndata = m$value({dbfile},:{c_fxndata},{dbentry})


! Copy 3d table elements to ordinary table columns
extract/t3d {dbfile} :{c_fxwave} {dbentry} {spcfile} :{c_fxwave} {ndata}
extract/t3d {dbfile} :{c_fxflux} {dbentry} {spcfile} :{c_fxflux} {ndata}
extract/t3d {dbfile} :{c_fxbwidth} {dbentry} {spcfile} :{c_fxbwidth} {ndata}

return 0
