! $Id: fors_getframes.prg,v 1.27 2000/02/23 16:52:47 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_getframes.prg,v $
!
!.KEYWORDS    Pipeline input frame conversion
!
!.USAGE       get/frames  frm_set ref_set cat_accept [cat_reject]
!
!.INPUT       frm_set  	  (P1)   Name of ASCII file containing the raw frames
!             ref_set  	  (P2)   Name of ASCII file containing the reference 
!                                frames
!
!.OUTPUT      cat_accept  (P3)   Catalog name containing accepted frames
!             cat_reject  (P4)   Catalog name containing rejected frames
!
!.RETURNS     The recipe returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status.
!
!.PURPOSE     The procedure reads the ASCII file with the names of the input
!   	      data frames and copies the files to the current working 
!             directory. The files are converted from FITS format to MIDAS
!             internal formats. Finally, the file type of each frame is
!             determined and the name is written to the appropriate keywords.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!
!------------------------------------------------------------------------------

crossref input reference accept reject

define/param p1 ?  c    "Enter raw frame set file:"
define/param p2 ?  c    "Enter reference frame set file:"
define/param p3 ?  c    "Enter accepted frames catalog name:"
define/param p4 +  c    "Enter rejected frames catalog name:"

define/maxpar 4


! Procedure identification
define/local procID/c/1/80      "GetFrames"


! Local constant
define/local cat_tmp/c/1/10     "tmpfrm.cat"



! Local variables
define/local pos/i/1/1          -1
define/local i/i/1/1             0
define/local status/i/1/1        0
define/local catflag/i/1/1       0
define/local catal/i/1/1         0
define/local n_fmax/i/1/1        0
define/local n_frames/i/1/1      0
define/local n_accept/i/1/1      0
define/local n_reject/i/1/1      0
define/local frm_set/c/1/80      {p1}
define/local ref_set/c/1/80      {p2}
define/local cat_accept/c/1/80   {p3}
define/local cat_reject/c/1/80   {p4}
define/local cat_entry/c/1/80    " "           all
define/local obsmode/c/1/15      " "           all
define/local orgname/c/1/256     " "           all



! Read input set of frames 
mess/out 3 {procID} "Processing SetOfFrames"
mess/out 3 {procID} "  {frm_set} ...!"
@d set_of_frames IN {frm_set} {cat_tmp} CONV >Null
if outputi(1) .lt. 0 then
  mess/out 1 {procID} "Cannot read SetOfFrames {frm_set}!"
  status = 1
  goto clean
else
  n_frames = outputi(1)
endif


! Check if number of input frames exceed pipeline limit
if n_frames .gt. p_maxfrm(1) then
  set/format i1
  mess/out 1 {procID} -
           "Number of input frames exceeds pipeline limit ({p_maxfrm(1)})!"
  set/format
  status = 1
  goto clean
endif


! Extract instrument mode from the input frame header
if m$existd("#1_{cat_tmp}",H_DPRTECH) .eq. 1 then
  copy/dk #1_{cat_tmp} {H_DPRTECH} obsmode
  obsmode = m$upper(obsmode)
endif


! Associate observing technique with internal observing mode names
i = 1

mode_loop:

if obsmode .ne. p_supported({i}) then
  i = i + 1
  if i .le. p_nmodes then
    goto mode_loop
  else
    mess/out 1 {procID} "Unsupported observing mode encountered!"
    mess/out 1 {procID} "Check header keyword {H_DPRTECH} of input frames!"
    status = 1
    goto clean
  endif
endif

FORS_INSMODE = m$upper(p_modes({i}))

if n_frames .gt. 1 then
  do i = 2 n_frames 1
    if m$existd("#{i}_{cat_tmp}",H_DPRTECH) .eq. 1 then
      if "{#{i}_{cat_tmp},{H_DPRTECH}}" .ne. obsmode then
        FORS_INSMODE = "UNDEFINED"
        mess/out 1 {procID} -
                 "Ambiguous observing mode encountered in input frame set!"
        mess/out 1 {procID} "Check header keyword {H_DPRTECH} of input frames!"
        status = 1
        goto clean
      endif
    else
      mess/out 1 {procID} "Header keyword {h_dprtech} not present in header!"
      status = 1
      goto clean
    endif
  enddo
endif


! Initialize quality control status of raw frames if not already set
do i = 1 n_frames 1
  if m$existd("#{i}_{cat_tmp}",H_PROQCSTS) .eq. 0 then
    write/descr #{i}_{cat_tmp} {H_PROQCSTS}/c/1/10 "{d_uchk}"
    write/dhelp #{i}_{cat_tmp} {H_PROQCSTS} "{I_PROQCSTS}"
  endif
enddo


! Read reference set of frames if present
if p2 .ne. "+" then
  mess/out 3 {procID} "Processing SetOfFrames"
  mess/out 3 {procID} "  {ref_set} ...!"
  @d set_of_frames IN {ref_set} {cat_tmp},app CONV >Null
  if outputi(1) .lt. 0 then
    mess/out 1 {procID} "Cannot read SetOfFrames {ref_set}!"
    status = 1
    goto clean
  else
    n_frames = n_frames + outputi(1)
  endif


! Check if number of reference frames exceed pipeline limit
  if outputi(1) .gt. p_maxfrm(2) then
    set/format i1
    mess/out 1 {procID} -
        "Number of reference frames exceeds pipeline limit ({p_maxfrm(2)})!"
    set/format
    status = 1
    goto clean
  endif
endif


! Merge input and reference catalogs into one. Frame identification is 
! done on the fly.
pos = m$indexb(cat_accept,".cat") - 1
if pos .lt. 0 then
  cat_accept = cat_accept // ".cat"
endif
create/icat {cat_accept} NULL {FORS_IDSTRNG} >Null

if cat_reject .ne. "+" then
  pos = m$indexb(cat_reject,".cat") - 1
  if pos .lt. 0 then
    cat_reject = cat_reject // ".cat"
  endif
  create/icat {cat_reject} NULL {FORS_IDSTRNG} >Null
  catflag = 1
endif

set/format i1
n_fmax = n_frames - 1
do i = 0 n_fmax 1
  store/frame cat_entry {cat_tmp} 1
  if "{{cat_entry},{FORS_IDSTRNG}}" .eq. FORS_NOCLASS then
    if catflag .eq. 1 then
      add/icat {cat_reject} {cat_entry} >Null
      n_reject = n_reject + 1
    endif
    mess/out 2 {procID} "Frame type of {cat_entry} is unknown."
  elseif "{{cat_entry},{FORS_IDSTRNG}}" .eq. TEST_ID then
    if catflag .eq. 1 then
      add/icat {cat_reject} {cat_entry} >Null
      n_reject = n_reject + 1
    endif
    mess/out 2 {procID} "Ignoring frame {cat_entry} of type {TEST_ID}."
  else
    add/icat {cat_accept} {cat_entry} >Null
    n_accept = n_accept + 1
    if m$existd(cat_entry,H_FILENAME) .eq. 1 then
      orgname = "{{cat_entry},{H_FILENAME}}"
    else if m$existd(cat_entry,H_ORIGNAME) .eq. 1 then
      orgname = "{{cat_entry},{H_ORIGNAME}}"
    endif

    if m$strlen(orgname) .gt. 0 then
      pos = m$indexb(orgname,"/") + 1
      if pos .le. 0 then
        pos = 1
      endif
      mess/out 3 {procID} "Frame {orgname({pos}:)}"
    else
      mess/out 3 {procID} "Frame #{i}"
    endif
    mess/out 3 {procID} -
             "  converted to {cat_entry} ({{cat_entry},{FORS_IDSTRNG}})"
  endif
enddo
set/format

set/format j4 f7.2
mess/out 3 {procID} " "
mess/out 3 {procID} "*** Processing Log ***"
mess/out 3 {procID} " "
mess/out 3 {procID} "Name of input data set:"
mess/out 3 {procID} "  {frm_set}"
mess/out 3 {procID} "Name of reference data set:"
mess/out 3 {procID} "  {ref_set}"
mess/out 3 {procID} "No. frames processed:       {n_frames}"
mess/out 3 {procID} "No. accepted frames:        {n_accept}"
mess/out 3 {procID} "No. rejected frames:        {n_reject}"
mess/out 3 {procID} " "
set/format

if n_accept .eq. 0 then
  mess/out 1 {procID} "No frames have been accepted!"
  status = 1
elseif n_accept .ne. n_frames then
  mess/out 2 {procID} "Some frames have been rejected!"
endif


! Get readout port setup
config/ports #1_{cat_accept}
if {q1} .ne. 0 then
  status = 1
  goto clean
endif


! Get filter setup
config/filter #1_{cat_accept}


! Cleanup temporary files
clean:
-delete {cat_tmp}

return {status}
