! $Id: fors_getlines.prg,v 1.5 2000/05/24 15:16:17 nrainer Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_getlines.prg,v $
!
!.KEYWORDS    Spectroscopy, Wavelength calibration, line lists
!
!.USAGE       get/linecat  catalog linecat grism swidth
!
!.INPUT       catalog  (P1)   Master list of emission line catalogs
!                             for all combinations of grisms and slits.
!             grism    (P3)   Grism name.
!             swidth   (P4)   Slit width in arcsec.
!
!.OUTPUT      linecat  (P2)   Master line list of emission lines for all
!                             calibration lamps used by the instrument.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Selects an emission line catalog for a specific grism and
!             slit combination.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/03/10:  Creation
!                  1999/10/07:  modified for MOS context (JKR)
!------------------------------------------------------------------------------

crossref  catalog linecat grism swidth

define/param p1 ? c "Enter name of line list catalog:"
define/param p2 ? c "Enter name of emission line catalog:"
define/param p3 ? c "Enter name grism:"
define/param p4 ? c "Enter slit width:"
define/maxpar 4


! Procedure identification
define/local procID/c/1/80      "GetLineCat"


! Local variables
define/local nlin/i/1/1     0
define/local swid/r/1/1     0.
define/local mcat/c/1/100   {p1}
define/local lcat/c/1/100   {p2}
define/local gname/c/1/100  {p3}
define/local tmptab/c/1/100 "tmp.tbl"



! Process parameters
if m$tstno(p4) .ne. 1 then
  mess/out 1 {procID} "Invalid slit width encountered!"
  return 1
else
  swid = {p4}
endif


select/tab {mcat} :{c_lcgrism}.EQ."{gname}*"  >Null
! Search for instrument setup in line list catalog
if outputi(1) .le. 0 then
  mess/out 1 {procID} "Grism {gname} not found in line list catalog!"
  return 1
else

 copy/tab    {mcat} {tmptab}
 crea/col    {tmptab} :wid_diff "arcsec" F10.2 >Null
 compute/tab {tmptab} :wid_diff = abs(:{c_lcswid} - {swid})
 sort/tab    {tmptab} :wid_diff 

! use first table element (= min(abs(:{c_lcswid} - {swid})))
 nlin = m$value({tmptab},{c_lcnline},1)

 extract/t3d {tmptab} :{c_lcwave}   1 {lcat} :{c_lcwave}   {nlin}
 extract/t3d {tmptab} :{c_lcintens} 1 {lcat} :{c_lcintens} {nlin}
 extract/t3d {tmptab} :{c_lclamp}   1 {lcat} :{c_lclamp}   {nlin}

 copy/dd {tmptab} *,3 {lcat}

endif

return 0
