! $Id: fors_getports.prg,v 1.7 1998/10/16 14:08:03 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_getports.prg,v $
!
!.KEYWORDS    Determine readout port configuration
!
!.USAGE       config/ports  frame
!
!.INPUT       frame       (P1)   Input frame from which the detector port
!                                setup is extracted.
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is 
!             updated with the exit status.
!
!.PURPOSE     Get the number and the indices of all active detector ports.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/07/03:  Creation
!
!------------------------------------------------------------------------------

crossref  frame


define/param p1 ? c "Enter name of input frame:"
define/maxpar 1


! Procedure identification
define/local procID/c/1/80      "ForsGetPorts"


! Local variables
define/local i/i/1/1         0
define/local np/i/1/1        1
define/local lockno/i/1/2    0     all
define/local frame/c/1/100   {p1}
define/local dscname/c/1/80  " "  all



! Unlock keywords if necessary
info/keyw fors_nout
lockno(1) = mid$info(4)
if lockno(1) .ne. 0 then
  unlock/key fors_nout {lockno(1)}
endif

info/keyw fors_outi
lockno(2) = mid$info(4)
if lockno(2) .ne. 0 then
  unlock/key fors_outi {lockno(2)}
endif


! Reset port status
do i = 1 FORS_MAXOUT 1
  fors_outi({i}) = 0
enddo


! Get number of outputs
if m$existd(frame,h_nout) .eq. 1 then
  fors_nout = {{frame},{h_nout}}
else
  mess/out 1 {procID} "Header keyword {h_nout} not present!"
  return 1
endif

set/format i1
do i = 1 FORS_MAXOUT 1
  dscname = kbase_outi // "{i}" // ksuff_outi
  if m$existd(frame,dscname) .eq. 1 then
    fors_outi({np}) = i
    np = np + 1
  endif
enddo
set/format

np = np - 1

if np .eq. 0 then
  mess/out 1 {procID} "No readout port information found in frame header!"
  return 1
else if fors_nout .ne. np then
  mess/out 2 {procID} -
     "Number of active ports not equal to header keyword {h_nout}"
endif


! Protect keywords
if lockno(1) .ne. 0 then
  lock/key fors_nout {lockno(1)}
endif

if lockno(2) .ne. 0 then
  lock/key fors_outi {lockno(2)}
endif

return 0
