! $Id: fors_getpurge.prg,v 1.1 1999/06/07 11:35:41 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_getpurge.prg,v $
!
!.KEYWORDS    Pipeline configuration, pipeline products
!
!.USAGE       get/purge_table  cat_accept keyname
!
!.INPUT       cat_accept  (P1)   Name input frame catalog.
!             keyname     (P2)   Keyword name to be used for storing
!                                cleanup table name
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The procedure detects a header cleanup table in a list of
!             files and write its file name to the specified keyword.
!             Note that the target keyword must already exist.
!   
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/05/21:  Creation
!
!------------------------------------------------------------------------------

crossref  catalog keyword

define/param p1  ?      c    "Input frame catalog:"
define/param p2  ?      c    "Output keyword name:"


! Procedure identification
define/local procID/c/1/80	"GetPurgeTbl"



! Local variables
define/local lockno/i/1/1      0
define/local catal/i/1/1       0
define/local cat/c/1/20        {p1}
define/local cat_entry/c/1/80  " "   all
define/local keyname/c/1/16    " "  all



keyname = m$upper(p2)
info/keyw {keyname}
if mid$info(1) .ne. 0 then
  lockno = mid$info(4)
else
  return 1
endif

if lockno .ne. 0 then
  unlock/key {keyname} {lockno}
endif


! Reset keyword
write/keyw {keyname} " " all


! Search for cleanup table in input catalog
read_cat_entry:

store/frame cat_entry {cat} 1 exit_loop

if "{{cat_entry},{FORS_IDSTRNG}}" .eq. PURGET_ID then
  {keyname} = cat_entry
else
  goto read_cat_entry
endif

exit_loop:

if m$strlen({keyname}) .le. 0 then
  return 1
endif


! Recover initial state of target keyword
if lockno .ne. 0 then
  lock/key {keyname} {lockno}
endif

return 0
