! $Id: fors_getrefpix.prg,v 1.1 1999/07/20 08:24:35 rpalsa Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_getrefpix.prg,v $
!
!.KEYWORDS    Extraxt reference pixel coordinates from comment keyword
!
!.USAGE       @p getrefpix  frame
!
!.INPUT       frame    (P1)   Input frame from which the reference pixel
!                             coordinates are extracted.
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. If no error occured the keywords
!             OUTPUTR(1) and OUTPUTR(2) contains the x and y coordinates of
!             the reference pixel.
!
!.PURPOSE     Get the reference pixel coordinates from the input frame's
!             comment keyword. This implements a workaround for FORS1 data
!             and provides the wavelength calibration recipe with the
!             appropriate center coordinates.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/06/09:  Creation
!
!------------------------------------------------------------------------------

crossref  frame

define/param p1 ? c "Enter name of input frame:"
define/maxpar 1


! Procedure identification
define/local procID/c/1/80      "GetRefpix"


! Local constants
define/local maxaxis/i/1/1   2
define/local dscfile/c/1/10  "comdsc.dat"


! Local variables
define/local i/i/1/1       0
define/local pos/i/1/1     0
define/local fctrl/i/1/2   0
define/local line/c/1/80   " "  all
define/local frm/c/1/100   {p1}



! Check for COMMENT keyword
if m$existd(frm,"COMMENT") .ne. 1 then
  return 1
else
  read/descr {frm} COMMENT h >{dscfile}
endif


! Extract reference pixel coordinates
open/file {dscfile} READ fctrl
if fctrl(1) .eq. -1 then
  return 1
endif

read_loop:

read/file {fctrl(1)} line 80
if fctrl(2) .ne. -1 then
  if m$index(line,"CRPIX") .gt. 0 then
    i = i + 1
    pos = m$index(line," ") + 1
    outputr({i}) = {line({pos}:)}
 
    if i .lt. maxaxis then
      goto read_loop
    endif
  else
    goto read_loop
  endif
endif  

close/file {fctrl(1)}


! Check if all necessary data has been found
if i .ne. maxaxis then
  return 1
endif


! Cleanup
-delete {dscfile}

return 0