! $Id: fors_identstd.prg,v 1.5 1999/09/16 13:43:24 nrainer Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_identstd.prg,v $
!
!.KEYWORDS    Source identification, spectroscopy, flux calibration
!
!.USAGE       identify/standard objects standards ident climit
!
!.INPUT       objects     (P1)    3d table with detected object positions
!                                 and extracted spectra
!             standards   (P2)    3d table with positions and model spectra
!                                 of standard stars
!             climit      (P4)    Minimum confidence level (gaussian sigma).
!                                 Not yet implemented!
!
!.OUTPUT      ident       (P3)    Table of identified object spectra with
!                                 reference to the corresponding entry in
!                                 the standard star flux database.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. The keyword OUTPUTI(1) is updated
!             with the total number of identifications.
!
!.PURPOSE     The position of the standard stars are taken from the flux 
!             database and are compared with the detected object positions
!             taken from the table of observed spectra. If the positions match
!             within the uncertainty of the positions of the detected objects,
!             which are taken from the spectrum table the object is identified
!             as the standard star. If multiple identifications occur for the
!             same object the standard star closest to the detected position
!             is associated with the objects. All identified objects are 
!             written to the output table in the same format as the input
!             table of observed objects. An additional column holds the
!             objects entry number in the flux database. 
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/01/04:  Creation
!
!------------------------------------------------------------------------------

crossref  objects standards ident climit

define/param p1 ? c "Enter detected object table:"
define/param p2 ? c "Enter standard flux database:"
define/param p3 ? c "Enter identified object table:"
define/param p4 + c "Enter confidence limit:"
define/maxpar 4


! Procedure identification
define/local procID/c/1/80	"IdentStd"


! Local variables
define/local i/i/1/1         0
define/local j/i/1/1         0
define/local first/i/1/1     1
define/local nstd/i/1/1      0
define/local nobj/i/1/1      0
define/local nids/i/1/1      0
define/local objno/i/1/1     0
define/local dra/d/1/1       0.
define/local ddec/d/1/1      0.
define/local dra_min/d/1/1   0.
define/local ddec_min/d/1/1  0.
define/local s_ra/d/1/1      0.
define/local s_dec/d/1/1     0.
define/local o_ra/d/1/1      0.
define/local o_dec/d/1/1     0.
define/local o_raerr/d/1/1   0.
define/local o_decerr/d/1/1  0.
define/local climit/d/1/1    0.
define/local errscl/d/1/1    1.
define/local stdnam/c/1/100  " "
define/local spclst/c/1/100  {p1}
define/local flxlst/c/1/100  {p2}
define/local idslst/c/1/100  {p3}



! Process parameters
if m$tstno(p4) .eq. 1 then
  climit = {p4}
else
  climit = 1.
endif

  
! Get number of detected objects from the number of defined rows in the
! object spectrum list.
nobj = m$value({spclst},tblcontr(4))
if nobj .lt. 1 then
  mess/out 1 {procID} "No objects found in input table {spclst}
  return 1
endif


! Get number of available standard stars in the data base from the number
! of defined table rows.
nstd = m$value({flxlst},tblcontr(4))
if nstd .lt. 1 then
  mess/out 1 {procID} "Empty flux standard data base {flxlst} encountered!"
  return 1
endif


! Create index column for flux database entries
if m$existc(spclst,":{c_sorefid}") .lt. 0 then
  create/column {spclst} :{c_sorefid} "None" I6 I*4
endif


! Output to longlog
set/format i1
longlog/out 1 "     Extracted object spectra: {spclst}"
longlog/out 1 "     (contains {nobj} 1-d spectra)"
longlog/out 1 "     searched for matches in standard catalog: {flxlst}"
longlog/out 1 " "
set/format


! Loop over standard star data base entries and compare their expected
! positions on the chip with the location of the detected spectra.
do i = 1 nstd
  s_ra = m$value({flxlst},:{c_radeg},{i})
  s_dec = m$value({flxlst},:{c_decdeg},{i})

  objno = 0

  do j = 1 nobj
    o_ra = m$value({spclst},:{c_radeg},{j})
    o_dec = m$value({spclst},:{c_decdeg},{j})
    o_raerr = errscl * m$value({spclst},:{c_dradeg},{j})
    o_decerr = errscl * m$value({spclst},:{c_ddecdeg},{j})

! On first run set initial minimum source distances to search box size.
    if j .eq. 1 then
      dra_min = o_raerr
      ddec_min = o_decerr
    endif

    dra = m$abs(s_ra - o_ra)
    ddec = m$abs(s_dec - o_dec)

    if dra .lt. dra_min .and. ddec .lt. ddec_min then
      objno = j
      dra_min = dra
      ddec_min = ddec
    endif
  enddo

  if objno .gt. 0 then
    nids = nids + 1

    if first .eq. 1 then
      select/table {spclst} seq .eq. {objno} >Null
      first = 0
    else
      select/table {spclst} sel .or. seq .eq. {objno} >Null
    endif

! Update reference column
    {spclst},:{c_sorefid},{objno} = {i}

    stdnam = m$value({flxlst},:{c_fxoname},{i})

    set/format i1
    mess/out 3 {procID} -
     "Object #{objno} identified as >{stdnam}<"
    mess/out 4 {procID} "with offsets dRA = {dra_min}, dDEC = {ddec_min}"
    longlog/out 1 "     Object #{objno} identified as >{stdnam}<"
    longlog/out 1 "     offsets: dRA = {dra_min} deg, dDEC = {ddec_min} deg"
    longlog/out 1 " "
    set/format
  endif
enddo

longlog/out 1 "     Identified 1-d object spectra table is: {idslst}"

! Create final standard star table and return total number of
! identifications
if nids .gt. 0 then
  copy/table {spclst} {idslst}
  select/table {idslst} all
endif
outputi(1) = nids

return 0
