! $Id: fors_init.prg,v 1.48 2001/06/27 15:48:54 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_init.prg,v $
!
!.KEYWORDS    Pipeline initialization
!
!.USAGE       init/pipeline  raw_set ref_set cat_accept ctx [session]
!                            [obsmode] [cat_reject] [cat_result]
!
!.INPUT       raw_set     (P1)    Set of raw input frames to be processed or
!                                 a catalog of input frames.
!             ref_set     (P2)    Set of reference frames used for processing
!                                 (bad pixel mask, bias, dark ...)
!             ctx         (P4)    Identifier of required context
!             session     (P5)    Keyword defaults for pipline
!             mode        (P6)    Indicator of current observing mode
!
!.OUTPUT      cat_accept  (P3)    Name of accepted frames output catalog
!             cat_reject  (P7)    Name of rejected frames output catalog
!             cat_result  (P8)    Name of result frames output catalog
!
!.RETURNS     The recipe returns 0 on success. If no frames are found in
!             the input set the procedure returns 2, on any other error 1.
!             Keyword Q1 is updated with the exit code.
!
!.PURPOSE     The procedure starts the context MOS and initializes the
!             keywords with the pipeline defaults. The input frame set is
!             processed, i.e. the raw frames are copied to the local 
!             directory and converted from fits format to MIDAS formats.
!             If the input frame set is a MIDAS catalog (i.e. the suffix
!             '.cat') must be present, then the procedure assumes to run
!             within a recipe queue and skips the initialization and frames
!             conversion. Only the temporary catalog for the result frames is
!             created. 
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!             1.3  1999/08/30:  reset/slog call removed (SB).
!                               Deleting temporary files (SB).
!                               p_startdate initialized (SB).
!
!------------------------------------------------------------------------------

define/param p1  ?      c    "Input frame set:"
define/param p2  ?      c    "Reference frame set:"
define/param p3  +      c    "Accepted frames catalog name:"
define/param p4  +      c    "Context identifier: "
define/param p5  +      c    "Startup file:"
define/param p6  +      c    "Observing mode:"
define/param p7  +      c    "Rejected frames catalog name:"
define/param p8  +      c    "Result frames catalog name:"


! Procedure identification
define/local procID/c/1/80	"InitPipeline"



! Local variables
define/local ctx/c/1/10      " "  all
define/local session/c/1/10  " "  all
define/local obsmode/c/1/3   " "  all
define/local ctxflg/i/1/1    0
define/local pos/i/1/1       0
define/local lockno/i/1/2    0    all
define/local lockno1/i/1/2   0    all
define/local lockno2/i/1/2   0    all



if p4(1:1) .eq. "+" then
  ctx = "NONE"
else
  ctx = m$lower(p4)
endif

if p5(1:1) .eq. "+" then
  session = "NONE"
else
  session = p5
endif

if p6(1:1) .ne. "+" then
  obsmode = p6
endif


! Reset delete flag for auxiliary contexts
set/ctxdel reset

pos = m$indexb(p1,".cat")

if pos .gt. 0 then
  mess/out 4 {procID} "SUPER MODE: Restricted initialization!"
  -copy {p1} {p3}


! Create empty catalog for result frames
  if p8 .ne. "+" then
    create/icat {p8} NULL {FORS_IDSTRNG} >Null
  endif

! If a context is requested check if it is already enabled. If this is not
! the case it is loaded and initialized.
  if ctx .ne. "NONE" then
    show/context {ctx}
    ctxflg = outputi(5)

    if ctxflg .ne. 0 then
      mess/out 4 {procID} "Context {ctx} already enabled."
    else
      set/context {ctx} {p_ctxdir} >Null
      show/context {ctx}
      ctxflg = outputi(5)

      if ctxflg .ne. 0 then  
        mess/out 3 {procID} "Context {ctx} enabled."
        set/ctxdel
      else
        mess/out 1 {procID} "Context {ctx} cannot be enabled."
		return 1
      endif

      if ctxflg .ne. 0 .and. session .ne. "NONE" then
        config/drs {p3}
        if m$len(outputc) .ne. 0 then
          copy/tab {outputc} {session}KEYS.tbl

          if ctx .eq. "xmos" then
            init/mos {session}
            mess/out 4 {procID} "Initialization file {session}KEYS.tbl used."
          endif
        else
          mess/out 2 {procID} "Session {session} cannot be initialized."
        endif 
      else
        if session .ne. "NONE" then
          mess/out 1 {procID} "Cannot load session without context."
          return 1
        endif
      endif
    endif
  endif

else

! Get Pipeline version ID and QC dictionary version ID
! Unlock keywords if necessary
  info/keyw p_version
  lockno1 = mid$info(4)
  if lockno1 .ne. 0 then
    unlock/key p_version {lockno1}
  endif

  info/keyw p_qcdic_version
  lockno2 = mid$info(4)
  if lockno2 .ne. 0 then
    unlock/key p_qcdic_version {lockno2}
  endif

  if m$exist(p_version_file) .eq. 1 then
    write/_out {p_version_file} | write/key out_a

! Parsing of out_a
    outputi = m$parse(out_a,"parsing")
    if outputi(1) .ne. 2 then
      mess/out 1 {procID} "{p_version_file} has an incorrect syntax !"
      return 1
    endif

    inputi = m$index(parsing01,"=")+1
    p_version = parsing01({inputi}:)
    inputi = m$index(parsing02,"=")+1
    p_qcdic_version = m$upper(rbinstrument) // "_QC-" // parsing02({inputi}:)
  else
    p_version = m$lower(rbinstrument) // "/<development>"
    p_qcdic_version = m$upper(rbinstrument) // "_QC-<unknown>"
  endif
    
  if lockno1 .ne. 0 then
    lock/key p_version {lockno1}
  endif

  if lockno2 .ne. 0 then
    lock/key p_qcdic_version {lockno2}
  endif

! Greeting
  mess/out 3 {procID} "Welcome, {user}"
  mess/out 3 {procID} "to FORS Pipeline version: {p_version}"
  write/out

! Copy version ID to global key
info/key pipeline_vers
if mid$info(1) .ne. 0 .and. mid$info(2) .eq. 4 then
  pipeline_vers(4) = p_version
endif

! Set start date
! Unlock keyword if necessary
  info/keyw p_startdate
  lockno = mid$info(4)
  if lockno .ne. 0 then
    unlock/key p_startdate {lockno}
  endif

  p_startdate = m$isodate()//"            "  ! get date in ISO 8601 format

! Protect keyword
  if lockno .ne. 0 then
    lock/key p_startdate {lockno}
  endif


! Load context and create auxiliary keywords
  if ctx .ne. "NONE" then

! Enable context and check its availability
    set/context {ctx} {p_ctxdir} >Null
    show/context {ctx}
    ctxflg = outputi(5)

    if ctxflg .ne. 0 then  
      mess/out 3 {procID} "Context {ctx} enabled."
    else
      mess/out 1 {procID} "Context {ctx} cannot be enabled."
	  return 1
    endif
  endif


! Configure pipeline displays
  config/displays
  if {q1} .gt. 0 then
    mess/out 1 {procID} "Display configuration failed!"
    return 1
  endif


! Create empty catalog for result frames
  if p8 .ne. "+" then
    create/icat {p8} NULL {FORS_IDSTRNG} >Null
  endif


! Process set of input frames
  get/frames {p1} {p2} {p3} {p7}
  if {q1} .gt. 0 then
    return 2
  endif

  if m$len(obsmode) .ne. 0 then
    FORS_INSMODE = m$upper(obsmode)
  endif


! Quit if instrument mode is LSS/MOS/ECH and readout mode is four-port
  if FORS_NOUT .gt. 1 .and. FORS_INSMODE .ne. "IMG" then
    mess/out 1 {procID} "Four-port readout not supported in spectroscopy!"
    return 1
  endif


! Correct WCS keywords for windowed frames
  @p fors_corr_wcs {p3}

! Initialize global environment
  config/env {FORS_INSMODE}

! Setup header cleanup table 
  config/purge_table {p3} p_hdrpurge
  if {q1} .ne. 0 then
    mess/out 2 {procID} "Cannot setup header keyword cleanup table."
  endif


! Update descriptor names according to recipe index RECi
! Unlock keyword if necessary
  info/keyw h_prostat
  lockno = mid$info(4)
  if lockno .ne. 0 then
    unlock/key h_prostat {lockno}
  endif

  write/key h_prostat/c/1/60   "ESO.PRO.{reci}.STATUS"  ! Pipeline status of product frame

! Protect keyword
  if lockno .ne. 0 then
    lock/key h_prostat {lockno}
  endif


! Initialize working environment
  if ctxflg .ne. 0 .and. session .ne. "NONE" then

! Look for DRS configuration file in current frame set. If there is one,
! load it.
    config/drs {p3}
    if m$len(outputc) .ne. 0 then
      copy/tab {outputc} {session}KEYS.tbl

      if ctx .eq. "xmos" then
        init/mos {session}
        mess/out 4 {procID} "Initialization file {session}KEYS.tbl used."
      endif
    else
      mess/out 2 {procID} "Session {session} cannot be initialized."
    endif 
  else
    if session .ne. "NONE" then
      mess/out 1 {procID} "Cannot load session without context."
      return 1
    endif
  endif

! Delete temporary files possibly left over from previous session.
  -delete {slog_file}
  -delete {llog_file}
  -delete {llog_table}
  -delete {qc1log_file}
  -delete {qc1log_table}

endif

return 0
