! $Id: fors_locatespe.prg,v 1.8 2001/04/18 16:40:56 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_locatespe.prg,v $
!
!.KEYWORDS    Source detection, spectroscopy, flux calibration
!
!.USAGE       locate/spectrum frame slits mode windows poserr cooref
!                             [fwhm] [thres,width]
!
!.INPUT       frame       (P1)    Input frame with object spectrum.
!             slits       (P2)    Slit position table.
!             mode        (P3)    Observing mode (LSS,MOS).
!             poserr      (P5)    Inaccuracy of object positions.
!             cooref      (P6)    World coordinate reference frame for
!                                 input spectrum frame.
!             fwhm        (P7)    Maximum expected spatial FWHM of point
!                                 source spectrum (arcsec).
!             thres,width (P8)    Detection threshold and Scan row width
!
!.OUTPUT      windows     (P4)    MOS window table with columns of object
!                                 world coordinates added
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The object and sky regions are determined for the input
!             spectroscopic frame. The cooordinate system of the input 
!             frame is expected to be a pixel coordinate system. If a
!             coordinate reference is given, the columns for the source
!             positions causing the detected spectra are added to allow
!             source identifications. The positions are given in usual
!             world coordinates. The pixel position of the sources are
!             assumed to be given by the slit center (X position) and
!             the center of the detected object regions. The uncertainty of
!             the source positions is assumed to be the half width of
!             the slit along the X axis. The uncertainty of the Y position
!             has to be given by poserr (in arcsec). The pixel coordinates
!             and uncertainties are then converted to world coordinates.
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/01/04:  Creation
!
!------------------------------------------------------------------------------

crossref  frame slits mode windows poserr fwhm detpar cooref

define/param p1 ?    c  "Enter name of object frame:"
define/param p2 ?    c  "Enter slit position table:"
define/param p3 LSS  c  "Enter observing mode:"
define/param p4 ?    c  "Enter window table:"
define/param p5 ?    c  "Enter position inaccuracy:"
define/param p6 ?    c  "WCS reference frame:"
define/param p7 +    c  "Enter max. point source FWHM:"
define/param p8 +    c  "Detection threshold and scan row width:"
define/maxpar 8


! Procedure identification
define/local procID/c/1/80	"LocateSpec"


! Local constant
define/local tmpwin/c/1/15   "tmp_windows.tbl"


! Local variables
define/local i/i/1/1         0
define/local pos/i/1/1       0
define/local tabs/i/1/1      0
define/local xwidth/i/1/1    0
define/local nobj/i/1/1      0
define/local nrow/i/1/1      0
define/local slno/i/1/1      0
define/local slwid/i/1/1     0
define/local objpos/r/1/2    0.    all
define/local objerr/r/1/2    0.    all
define/local objwcs/r/1/2    0.    all
define/local poserr/r/1/1    0.
define/local fwhm/r/1/1      0.
define/local thres/r/1/1     0.
define/local tval/r/1/1      0.
define/local mode/c/1/3      " "  all
define/local dname/c/1/100   " "  all
define/local frm/c/1/100     {p1}
define/local spos/c/1/100    {p2}
define/local wtbl/c/1/100    {p4}
define/local cref/c/1/100    {p6}
define/local dpar/c/1/100    {p8}
define/local mid$selidx/i/1/5000 0 ALL +lower_levels



! Process parameters
mode = m$upper(p3(1:3))

if m$tstno(p5) .eq. 1 then
  poserr = {p5} / m$value({frm},{h_pixscale})
else
  mess/out 1 {procID} "Missing object position inaccuracy!"
  return 1
endif

if m$tstno(p7) .eq. 1 then
  fwhm = {p7} / m$value({frm},{h_pixscale})
else
  fwhm = {wind}
endif


if m$tstno(dpar) .eq. 1 then
  pos = m$index(dpar,",") - 1
  if pos .gt. 0 then
    thres = {dpar(1:{pos})}
    pos = pos + 2
    xwidth = {dpar({pos}:)}
  else if pos .eq. 0 then
    xwidth = {dpar(2:)}
    thres = {thresh}
  else
    thres = {dpar}
    xwidth = {xbin}
  endif
else
  thres = {thresh}
  xwidth = {xbin}
endif


! Detect object spectra
if thres .lt. 0. then
  tval = 100. * m$abs(thres)
  tabs = 0
else
  tval = thres
  tabs = 1
endif

set/format i1 f1.2
mess/out 3 {procID} "Searching point source spectra in >{frm}<."
mess/out 3 {procID} "Search window size:\t\t {fwhm} pxl"
if tabs .eq. 0 then
  mess/out 3 {procID} "Detection threshold:\t {tval} % above median"
else
  mess/out 3 {procID} "Detection threshold:\t {tval} adu"
endif
mess/out 3 {procID} "Minimum distance object/sky: {min_dist} pxl"
set/format

if mess_level .le. 3 then 
  define/mos {frm} {spos} {wtbl} {thres} {fwhm} {xwidth} 0 0 >Null
else
  define/mos {frm} {spos} {wtbl} {thres} {fwhm} {xwidth} 0 0
endif
mess/out 3 {procID} " "

-copy {wtbl} {tmpwin}
select/table {tmpwin} :{c_woslit} .ne. NULL >Null
nobj = outputi(1)

if nobj .eq. 0 then
  mess/out 1 {procID} "No object spectra found in {frm}!"
  return 1
else
  nrow = m$value({wtbl},tblcontr(4))
endif

set/format i1
mess/out 3 {procID} "{nobj} objects found."
mess/out 3 {procID} " "
set/format


! Output to longlog
set/format i1 f1.2
longlog/out 1 "     Searched point source spectra in {frm}"
longlog/out 1 "     Search window size:              {fwhm} pxl"
if tabs .eq. 0 then
  longlog/out 1 "     Detection threshold:             {tval} % above median"
else
  longlog/out 1 "     Detection threshold:             {tval} adu"
endif
longlog/out 1 "     Minimum distance object/sky:     {min_dist} pxl"
longlog/out 1 "     {nobj} objects found"
set/format

! Create columns for world coordinates and uncertainties
create/column {wtbl} :{c_radeg} "degrees" E18.8 R*8
create/column {wtbl} :{c_decdeg} "degrees" E18.8 R*8
create/column {wtbl} :{c_dradeg} "degrees" E18.8 R*8
create/column {wtbl} :{c_ddecdeg} "degrees" E18.8 R*8


! Calculate object positions
if mode .eq. "LSS" then
  slwid = 0.5 * m$value({frm},{h_slwidth}) / m$value({frm},{h_pixscale})

  do i = 1 nrow
    if m$value({tmpwin},sel,{i}) .eq. 1 then
      slno = m$value({wtbl},:{c_woslit},{i})

      select/tab {spos} :{c_slsno}.eq.{slno} >Null
      if outputi(1) .eq. 0 then
        mess/out 1 {procID} "Something is wrong with the slit position table {spos}."
        return 1
      endif

      objpos(1) = m$value({cref},refpix(1)) + -
                  m$value({spos},:{c_sloffset},{mid$selidx(1)})
      objpos(2) = 0.5 * (m$value({wtbl},:{c_woend},{i}) + -
                  m$value({wtbl},:{c_wostart},{i}))

      @a getcoord {cref} @{objpos(1)},@{objpos(2)} >Null
      objwcs(1) = outputd(1)
      objwcs(2) = outputd(2)

      objerr(1) = objpos(1) + poserr
      objerr(2) = objpos(2) + poserr
      @a getcoord {cref} @{objerr(1)},@{objerr(2)} >Null

      {wtbl},:{c_radeg},{i} = objwcs(1)
      {wtbl},:{c_decdeg},{i} = objwcs(2)
      {wtbl},:{c_dradeg},{i} = m$abs(outputd(1) - objwcs(1))
      {wtbl},:{c_ddecdeg},{i} = m$abs(outputd(2) - objwcs(2))
    endif
  enddo
else
  do i = 1 nrow
    if m$value({tmpwin},sel,{i}) .eq. 1 then
      slno = m$value({wtbl},:{c_woslit},{i})

      set/format i1
      dname = kbase_mosi // "{slno}" // ksuff_moswid
      set/format

      slwid = 0.5 * m$value({frm},{dname}) / m$value({frm},{h_pixscale})

      select/tab {spos} :{c_slsno}.eq.{slno} >Null
      if outputi(1) .eq. 0 then
        mess/out 1 {procID} "Something is wrong with the slit position table {spos}."
        return 1
      endif

      objpos(1) = m$value({cref},refpix(1)) + -
                  m$value({spos},:{c_sloffset},{mid$selidx(1)})
      objpos(2) = 0.5 * (m$value({wtbl},:{c_woend},{i}) + -
                  m$value({wtbl},:{c_wostart},{i}))

      @a getcoord {cref} @{objpos(1)},@{objpos(2)} >Null
      objwcs(1) = outputd(1)
      objwcs(2) = outputd(2)

      objerr(1) = objpos(1) + poserr
      objerr(2) = objpos(2) + poserr
      @a getcoord {cref} @{objerr(1)},@{objerr(2)} >Null

      {wtbl},:{c_radeg},{i} = objwcs(1)
      {wtbl},:{c_decdeg},{i} = objwcs(2)
      {wtbl},:{c_dradeg},{i} = m$abs(outputd(1) - objwcs(1))
      {wtbl},:{c_ddecdeg},{i} = m$abs(outputd(2) - objwcs(2))
    endif
  enddo
endif


! Cleanup
-delete {tmpwin}
select/tab {spos} all

shortlog/out 1 "Point source spectra searched."
return 0
