! $Id: fors_longlog.prg,v 1.24 2000/02/15 14:14:25 nrainer Exp $   
!+++ FORS_LONGLOG ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         $RCSfile: fors_longlog.prg,v $
!
!.KEYWORDS      Create a log file of reduction recipes
!
!.USAGE         longlog/out mode logentry
!
!.INPUT         mode    (P1): 3: open and add entry directly to the longlog 
!                                table without using the cache.
!                             2: open and add entry to log cache.
!                                Entry is not added, if procedure is called
!                                within a super recipe. The entries in the cache
!                                are first dumped to the real log as soon as 
!                                longlog/out 0 is called.
!                             1: open and add entry to log cache. The entries 
!                                in the cache are first dumped to the real log 
!                                as soon as longlog/out 0 is called.
!                             0: dump contents of the cache to the real longlog
!                                table.
!                            -1: replace all MIDAS file names by FITS names, 
!                                copy longlog table to real file and close.
!                                
!               entry   (P2): entry to add. If mode=-1 P2 must contain the
!                             product catalog.
!
!.OUTPUT        none.
!
!.RETURN        Q1:  0: successful return
!                    1: failed
!
!.DEFAULT
!
!.PURPOSE       Create a log file of reduction steps
!
!.ALGORITHM     longlog/out is used to create a detailed log file of a
!               reduction block execution. It is used to add text lines
!               to a buffer table, which is converted to a text file at the
!               end of the processing. MIDAS file names in the text are 
!               automatically converted to their FITS analogue upon the
!               conversion. A second buffer table is available to temporarily
!               store text before it is dumped to the main log table.
!               The different ways of using it are controlled by the parameter
!               P1. 
!               longlog/out is used in combination with shortlog/out, which
!               automatically creates an entry in the longlog table.
!
!.ENVIRON       FORS context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Stefan Bogun, ESO-DMO
!
!.VERSION       1.0 1999/04/15  Creation
!               1.1 1999/06/10  Minor bugfixes (SB)
!               1.3 1999/08/30  llog_file, llog_table moved to forsinit.prg (SB)
!
!.COMMENT
!------------------------------------------------------------------------------

define/param P1 ? i "Enter mode:"
define/param P2 + c "Enter log entry (catalog name when P1=-1):"

define/local procid/c/1/20 "longlog"
define/local cache_table/c/1/20 "lcache.tbl"

define/local mode/i/1/1 -1

define/local i/i/1/1 0
define/local nselect/i/1/1 0
define/local nrows/i/1/1 0
define/local rowloop/i/1/1 0
define/local catal/i/1/1 0
define/local SKIP/i/1/1 0
define/local midas_file/c/1/85 " " all
define/local pipe_file/c/1/85 " " all
define/local text/c/1/240 " " all
define/local textout/c/1/95 " " all
define/local len/i/1/3 0 all
define/local old/i/1/5 0 all
define/local new/i/1/5 0 all
define/local filecode/i/1/2 0 all
define/local date/c/1/30 " " ALL

define/local line/c/1/100 {P2}
mode = {P1}

if line(1:1) .eq. "+" line(1:1) = " "   !assume P2 holds filename of main prod.

if m$exist(llog_table) .eq. 0 .or. m$existk("llog_id") .eq. 0 then
  write/key LLOG_ID/i/1/2 0 all
  mess/out 4 {procid} " "
  mess/out 4 {procid} "Creating log table {llog_table}"
  create/table {llog_table} 1 1000 null
  create/column {llog_table} :logtext " " A95 C*95
  LLOG_ID(1) = 1
  date = m$time()
  {llog_table},:logtext,@{LLOG_ID(1)} = "Recipe log ({date}):"
  LLOG_ID(1) = LLOG_ID(1) + 1
endif

                                        !pipeline(6)=1 -> SUPER-recipe
if mode .eq. 2 .AND. pipeline(6) .eq. 1 SKIP = 1

branch mode -1,0,1,2,3 CLO,DMP,ADC,ADC,ADD

mess/out 1 {procid} "Wrong mode selected (-1,0,1,2,3 are allowed) !"
goto FIN
     
CLO:
     define/local mid$selidx/i/1/1000 0 ALL +lower_levels

     if LLOG_ID(1) .le. 0 then
       mess/out 1 {procid} -
         "No log file had been opened or\nan error occured while opening !"
       return 1
     endif

     if LLOG_ID(2) .ne. 0 longlog/out 0

     if m$index(line,".cat") .ne. 0 then
       outputi = m$len(line)
       out_a = line(1:{outputi(1)})
       if m$exist(out_a) .ne. 1 then
         mess/out 1 {procid} "{out_a} is not an existing catalog!"
         return 1
       endif
  
       mess/out 4 {procid} "Converting MIDAS names into FITS names"

CAT_LOOP:
       store/frame midas_file {line} 1 FINI_LOOP
       if m$existd(midas_file,H_PIPEFILE) .eq. 1 then
         pipe_file = "{{midas_file},{H_PIPEFILE}}"
       else
         pipe_file = "<NOT CONVERTED>"
       endif
       select/table {llog_table} :logtext.eq."* {midas_file}*" >Null
       nselect = outputi(1)
       if nselect .ge. 1 then
         do rowloop = 1 nselect
           text = "{{llog_table},:logtext,@{mid$selidx({rowloop})}}"
           !
           ! info/keyword text
           len(2) = m$strlen(midas_file)
           len(3) = m$strlen(pipe_file)
           ! outputi = mid$info(2)-len(2)+len(3)
           ! define/local textout/c/1/{outputi} " " all
           !
           old(1) = 1
           old(2) = m$index(text,midas_file)-1
           old(3) = old(2)+1
           old(4) = old(2)+len(2)
           old(5) = old(4)+1
           !
           new(1) = 1
           new(2) = m$index(text,midas_file)-1
           new(3) = new(2)+1
           new(4) = new(2)+len(3)
           new(5) = new(4)+1
           !
           if old(2) .gt. 0 then
             textout({new(1)}:{new(2)}) = text({old(1)}:{old(2)})
             textout({new(3)}:{new(4)}) = pipe_file
           else
             textout({new(1)}:{len(3)}) = pipe_file
           endif
             textout({new(5)}:)         = text({old(5)}:)
           !
           textout = textout(:95)
           !
           write/table {llog_table} :logtext @{mid$selidx({rowloop})} -
             "{textout}"
         enddo
       endif
       goto CAT_LOOP

FINI_LOOP:
     endif

     open/file {llog_file} write filecode
     if filecode(1) .eq. -1 then
        write/out " "
        mess/out 1 {procid} "Long log could not be written!"
        return 1
     endif
     show/table {llog_table} >Null
     nrows = outputi(2)
     do i = 1 nrows
       write/keyword text "{{llog_table},:logtext,@{i}} " >Null
       write/file {filecode(1)} "{text} "
     enddo
     close/file {filecode(1)}
     -delete {llog_table}
     write/key LLOG_ID/i/1/2 0

     info/key LONGLOG
     if mid$info(1) .ne. 0 LONGLOG = "{llog_file}"

     goto FIN
     
DMP:
     if LLOG_ID(2) .gt. 1 then
       nrows = LLOG_ID(2) - 1
       do i = 1 nrows
         outputi(1) = LLOG_ID(1) + i - 1
         write/table {llog_table} :logtext @{outputi(1)} -
           "{{cache_table},:logtext,@{i}} " >Null
       enddo
       LLOG_ID(1) = LLOG_ID(1) + nrows
       LLOG_ID(2) = 0
     else
       mess/out 4  {procid} "There's nothing to dump..."
     endif
     goto FIN

ADC: 
     if SKIP .eq. 1 goto FIN
     if LLOG_ID(2) .eq. 0 then
       mess/out 4 {procid} "Creating cache table {cache_table}"
       create/table {cache_table} 1 500 null
       create/column {cache_table} :logtext " " A95 C*95
       LLOG_ID(2) = 1
     endif
     {cache_table},:logtext,@{LLOG_ID(2)} = "{line} "
     LLOG_ID(2) = LLOG_ID(2) + 1
     goto FIN

ADD: 
     if LLOG_ID(1) .eq. 0 then
       mess/out 4 {procid} " "
       mess/out 4 {procid} "Creating log table {llog_table}"
       create/table {llog_table} 1 1000 null
       create/column {llog_table} :logtext " " A95 C*95
       LLOG_ID(1) = 1
       date = m$time()
       {llog_table},:logtext,@{LLOG_ID(1)} = "Recipe log ({date}):"
       LLOG_ID(1) = LLOG_ID(1) + 1
     endif

     {llog_table},:logtext,@{LLOG_ID(1)} = "{line} "
     LLOG_ID(1) = LLOG_ID(1) + 1

FIN:
return 0



