
! $Id: fors_longlogfinish.prg,v 1.9 2000/04/05 12:22:40 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_longlogfinish.prg,v $
!
!.KEYWORDS    Longlog finalization and closing
!
!.USAGE       finish/longlog input_cat result_cat main_prod lmode path
!
!.INPUT       input_cat   (P1)   Name of input catalog
!             result_cat  (P2)   Name of result catalog
!             main_prod   (P3)   Name of main product
!             lmode       (P4)   2: Longlog is not finalized, if procedure is 
!                                   called within a super recipe.
!                                1: Longlog is always finalized.
!             path	  (P5)   path + basename of output files
!
!.OUTPUT      none.
!
!.RETURNS     The recipe returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status.
!
!.PURPOSE     The recipe converts all midas file names present in the 
!             catalogs {input_cat} and {result_cat} into their FITS
!             analogs. If {result_cat} is really a catalog it writes the 
!             names of the output products to the longlog table. If  
!             {result_cat} = -1 it assumes that converting the products to
!             FITS failed and does not convert the midas file names.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      S. Bogun, ESO-DMO
!
!.VERSION     0.5  1999/04/19:  Creation
!             0.6  1999/06/10:  Minor bugfixes
!
!------------------------------------------------------------------------------

define/param p1 +  c    "Enter name of input catalog:"
define/param p2 ?  c    "Enter name of product catalog:"
define/param p3 +  c    "Enter main pipeline product"
define/param p4 ?  n    "Enter lmode:"
define/param p5 +  c    "Enter name of output-basename (+path):"


! Procedure identification
define/local procid/c/1/80      "LonglogFinish"


! Local variables
define/local input_cat/c/1/64   {p1}
define/local result_cat/c/1/64  {p2}
define/local main_prod/c/1/64   {p3}
define/local lmode/i/1/1        {p4}
define/local rusedi/c/1/3        "yes"
define/local rusedr/c/1/3        "yes"
define/local path/c/1/100       " "     all
define/local separ3/c/1/80      "-"     all
separ3(2:13) = " 3 PRODUCTS "

define/local i/i/1/1            0
define/local catal/i/1/1        0
define/local n_frames/i/1/1     0
define/local pos/i/1/1          0
define/local cat_entry/c/1/64   " "     all
define/local pipe_name/c/1/100  " "     all
define/local pro_catg/c/1/64    " "     all


! return if lmode=2 and procedure invoked within a superrecipe
if lmode .eq. 2 .and. pipeline(6) .eq. 1 then
  return 0
endif


if m$exist(input_cat) .eq. 0 .or. m$index(input_cat,".cat") .eq. 0 then
  rusedi = "no"
endif
 
if m$index(result_cat,".cat") .eq. 0 then
  rusedr = "no"
endif


! Write PIPEFILE into DO_FILENAME for result frames
if rusedr .eq. "yes" then  

! number of reference frames
  show/icat {result_cat} N
  n_frames = outputi(1)

  catal = 0
  do i = 1 n_frames 1
    store/frame cat_entry {result_cat} 1
    if m$existd(cat_entry,H_PIPEFILE) .eq. 1 -
      write/desc {cat_entry} {H_FILENAME}/c/1/128 {{cat_entry},{H_PIPEFILE}}
  enddo  

endif


! Write DO_FILENAME into PIPEFILE for input frames
if rusedi .eq. "yes" then  

! number of input frames
  show/icat {input_cat} N
  n_frames = outputi(1)

  catal = 0
  do i = 1 n_frames 1
    store/frame cat_entry {input_cat} 1
    if m$existd(cat_entry,H_FILENAME) .eq. 1 then
      out_a = "{{cat_entry},{H_FILENAME}}"
      outputi = m$indexb(out_a,"/") + 1
      write/desc {cat_entry} {H_PIPEFILE}/c/1/64 {out_a({outputi(1)}:)}
    endif
  enddo  

endif

! Recipe was successful:
if rusedr .eq. "yes" then  

! Write header lines
  longlog/out 1 " "
  longlog/out 1 " "
  longlog/out 1 "RB execution status: OK!"
  longlog/out 1 " "
  longlog/out 1 " "
  longlog/out 1 "{separ3}"
  longlog/out 1 " "  
  longlog/out 1 " "  


! print product path
  pos = m$indexb(p5,"/") - 1
  if pos .gt. 0 then
    path = p5(1:{pos})
  else
    path = p5
  endif
  
  longlog/out 1 "Data product path:"
  longlog/out 1 " "
  longlog/out 1 "{path}"


! number of products
  show/icat {result_cat} N
  n_frames = outputi(1)


  if main_prod(1:1) .ne. "+" then

! print main product
    if m$existd(main_prod,h_pipefile) .eq. 1 then
      pipe_name = "{{main_prod},{h_pipefile}}"
    else
      pipe_name = "<NOT CONVERTED>"
    endif
    if m$existd(main_prod,h_procatg) .eq. 1 then
      pro_catg = "{{main_prod},{h_procatg}}"
    else
      pro_catg = "<NO CATEGORY>"
    endif
    longlog/out 1 " "
    longlog/out 1 " "
    longlog/out 1 "Main product:"
    longlog/out 1 " "
    longlog/out 1 "{pipe_name} {pro_catg}"


! print other products:
    if n_frames .gt. 1 then
      longlog/out 1 " "
      longlog/out 1 " "
      longlog/out 1 "Other products:"
      longlog/out 1 " "

      catal = 0
      do i = 1 n_frames 1
        store/frame cat_entry {result_cat} 1
        if cat_entry .eq. main_prod goto ENDLOOP
        if m$existd(cat_entry,h_pipefile) .eq. 1 then
          pipe_name = "{{cat_entry},{h_pipefile}}"
        else
          pipe_name = "<NOT CONVERTED>"
        endif
        if m$existd(cat_entry,h_procatg) .eq. 1 then
          pro_catg = "{{cat_entry},{h_procatg}}"
        else
          pro_catg = "<NO CATEGORY>"
        endif
        longlog/out 1 "{pipe_name} {pro_catg}"
ENDLOOP:
      enddo
    endif

  else

!print all products
    if n_frames .ge. 1 then
      longlog/out 1 " "
      longlog/out 1 " "
      longlog/out 1 "Data products:"
      longlog/out 1 " "


      catal = 0
      do i = 1 n_frames 1
        store/frame cat_entry {result_cat} 1
        if m$existd(cat_entry,h_pipefile) .eq. 1 then
          pipe_name = "{{cat_entry},{h_pipefile}}"
        else
          pipe_name = "<NOT CONVERTED>"
        endif
        if m$existd(cat_entry,h_procatg) .eq. 1 then
          pro_catg = "{{cat_entry},{h_procatg}}"
        else
          pro_catg = "<NO CATEGORY>"
        endif
        longlog/out 1 "{pipe_name} {pro_catg}"
      enddo
    endif

  endif


  longlog/out 1 " "


! Dump longlog
  longlog/out 0


! Write longlog to file
  create/icat all NULL >Null
  add/icat all {result_cat} >Null
  if rusedi .eq. "yes" add/icat all {input_cat} >Null
  longlog/out -1 all.cat


! Recipe failed:
else

  longlog/out 1 " "
  longlog/out 1 " "
  longlog/out 1 "RB execution status: FAILED!"
  longlog/out 1 " "

! Dump longlog
  longlog/out 0

  if rusedi .eq. "yes" then
    longlog/out -1 {input_cat}
  else
    longlog/out -1 " "
  endif

endif
  
return 0
