! $Id: fors_longloginit.prg,v 1.6 1999/09/08 13:51:44 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_longloginit.prg,v $
!
!.KEYWORDS    Longlog initialization
!
!.USAGE       init/longlog frm_set ref_set lmode param1 param2 param3 param4 param5
!
!.INPUT       frm_set  	  (P1)   Name of ASCII file containing the raw frames
!             ref_set  	  (P2)   Name of ASCII file containing the reference 
!                                frames
!             lmode       (P3)   2: Longlog is not initialized, if procedure is 
!                                   called within a super recipe.
!                                1: Longlog is always initialized.
!             param1      (P4)   First parameter of recipe
!             param2      (P5)   Second parameter of recipe
!             param3      (P6)   Third parameter of recipe
!             param4      (P7)   Fourth parameter of recipe
!             param5      (P8)   Fifth parameter of recipe
!
!.OUTPUT      none.
!
!.RETURNS     The procedure returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status.
!
!.PURPOSE     The procedure writes name of recipe, reduction block,
!             file names of input and reference frames into the longlog table.
!             Parameters passed to the recipe are also written to the longlog,
!             if there are any.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      S. Bogun, ESO-DMO
!
!.VERSION     0.5  1999/04/16:  Creation
!
!------------------------------------------------------------------------------

define/param p1 ?  c    "Enter raw frame set file:"
define/param p2 ?  c    "Enter reference frame set file:"
define/param p3 ?  n    "Enter lmode:"
define/param p4 +  c    "Enter first parameter of recipe:"
define/param p5 +  c    "Enter second parameter of recipe:"
define/param p6 +  c    "Enter third parameter of recipe:"
define/param p7 +  c    "Enter fourth parameter of recipe:"
define/param p8 +  c    "Enter fifth parameter of recipe:"


! Procedure identification
define/local procid/c/1/80      "LonglogInit"


! Local variables
define/local fname/c/1/128      " "           all
define/local fc/i/1/2           0,0
define/local separ1/c/1/80      "-"           all
separ1(2:10) = " 1 INPUT "
define/local separ2/c/1/80      "-"           all
separ2(2:18) = " 2 REDUCTION LOG "

define/local lmode/i/1/1 {p3}

! return if lmode=2 and procedure invoked within a superrecipe
if lmode .eq. 2 .and. pipeline(6) .eq. 1 then
  return 0
endif


! Write header lines
longlog/out 1 " "
longlog/out 1 "{separ1}"
longlog/out 1 " "  
outputi = m$indexb(RBNAME,"/")+1
out_a = RBNAME({outputi(1)}:)
longlog/out 1 " "
longlog/out 1 "Name of RB:  {out_a}"
if m$existk("recipe_name") .eq. 1 then
  longlog/out 1 " "
  longlog/out 1 " "
  longlog/out 1 "Recipe name: {recipe_name}"
endif


! Write list of input frames to longlog
longlog/out 1 " "
longlog/out 1 " "
longlog/out 1 "Input frames:"
longlog/out 1 " "

open/file {p1} READ fc
if fc(1) .lt. 0 then
  mess/out 1 {procid} "Could not open input SetOfFrames {p1}!"
  return 1
endif
READ_LOOPI:
  read/file {fc(1)} fname
  if fc(2) .lt. 0 goto FINI_LOOPI
  outputi(1) = m$strlen(fname)
  if outputi(1) .gt. 96 fname = fname(1:96)
  longlog/out 1 "{fname}"
  goto READ_LOOPI
FINI_LOOPI:
close/file {fc(1)}



! Write list of reference frames to longlog
if p2 .ne. "+" then
  longlog/out 1 " "
  longlog/out 1 " "
  longlog/out 1 "Reference frames:"
  longlog/out 1 " "

  open/file {p2} READ fc
  if fc(1) .lt. 0 then
    mess/out 1 {procid} "Could not open reference SetOfFrames {p2}!"
    return 1
  endif
READ_LOOPR:
    read/file {fc(1)} fname
    if fc(2) .lt. 0 goto FINI_LOOPR
    outputi(1) = m$strlen(fname)
    if outputi(1) .gt. 96 fname = fname(1:96)
    longlog/out 1 "{fname}"
    goto READ_LOOPR
FINI_LOOPR:
  close/file {fc(1)}
endif


! Write list of parameters to longlog
longlog/out 1 " "
longlog/out 1 " "
longlog/out 1 "Input parameters:"
longlog/out 1 " "
if p4(1:1) .ne. "+" then
  longlog/out 1 {p4}
else
  longlog/out 1 "None."
endif

if p5(1:1) .ne. "+" then
  longlog/out 1 {p5}
endif

if p6(1:1) .ne. "+" then
  longlog/out 1 {p6}
endif

if p7(1:1) .ne. "+" then
  longlog/out 1 {p7}
endif

if p8(1:1) .ne. "+" then
  longlog/out 1 {p8}
endif

longlog/out 1 " "
longlog/out 1 " "
longlog/out 1 "{separ2}"
longlog/out 1 " "  


! Dump longlog
longlog/out 0

return 0
