! $Id: fors_mastercalib.prg,v 1.34 2001/06/15 14:16:52 sbogun Exp $   
!+++ FORS_MASTERCALIB +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         $RCSfile: fors_mastercalib.prg,v $
!
!.KEYWORDS      SUPER RECIPE: Master Creation and comparison with 'old' master
!
!.USAGE         @p (IDENT) sof_in sof_ref out avmode minnum thresh infac maxov
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!               sof_ref (P2): set of reference frames (ascii list)
!               avmode  (P4): averaging mode (0: constant, 1: mean, 2: median).
!                             If the mode is prefixed by 1 or 2, then the scaled
!                             bias correction method is used. For details see
!                             fors_gen_cal_mkmaster.prg
!               minnum  (P5): minimum number of frames to be averaged
!               thresh  (P6): thresholds
!               intfac  (P7): interval factors for median-mode
!               maxov   (P8): maximum number of overexposed pixels in %
!
!.OUTPUT        out     (P3): path + basename of output files:
!
!.RETURN        Q1:  0: successful return
!                    otherwise: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Produce a master calibration frame. The following steps will
!               be done:
!               a) Check for overexposed pixels (not for bias, dark).
!               b) Reduce single frames and create an average (median-stack)
!               c) Compare master frame with certified master from CalibDb.
!               In case of LSS and MOS a normalization of the flatfield
!               will be performed before c).
!
!.ALGORITHM     Calls single recipes.
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/12/04  Creation.
!                               main product defined: MASTER_<calib>
!                   1998/12/08  paragraphs implemented
!                   1998/12/10  error handler implemented (write/error)
!               1.2 1999/02/03  NO defaults for thresholds P6 and int.fact. P7
!                   1999/02/05  create high frequency flatfield (IMAGING)
!                   1999/02/08  LSS and MOS screenflat processing implemented
!                   1999/02/10  HF-flatfield shifted to superflat recipe!
!                   1999/02/11  handling of night flats,
!                               check minimum number of input frames
!               1.3 1999/04/29  Longlog implemented (SB)
!
!.COMMENT       COMMON BLOCK:
!               . exposure type is passed through global keyword INPUTI(1)
!                 (1: bias, 2: dark, 3: skyflat, 4: screenflat, 
!                  5: normflat[LSS]  6: normflat[MOS])
!               . INPUTI(2) == 1 indicates a night-skyflat
!
!------------------------------------------------------------------------------
!
define/param p1 ? c "Enter name of ASCII filelist with calibration frames:"
define/param p2 ? c "Enter name of ASCII filelist with reference frames:"
define/param p3 ? c "Enter name of output-basename (+path):"
define/param p4 22 n "Enter averaging mode and scaling method:"
define/param p5 4 n "Enter minimum number of input frames to be averaged:"
define/param p6 ? ? "Enter thresholds as defined in the rrd-files:"
define/param p7 ? ? "Enter mmedian-mode interval factors as defined in rrd: "
define/param p8 3 n "Enter maximum allowed number of overexposed pixels in %:"

define/local part/i/1/1 0
define/local ol/r/1/1 7.0
define/local minnum/i/1/1 0
define/local i2/i/1/1 0
define/local n1/i/1/1 0:
define/local etyp/i/1/1 0
define/local procid/c*25/1/6 " " ALL
define/local pid/c/1/25 " " ALL
define/local otyp/c/1/30 " " ALL
define/local ityp/c/1/30 " " ALL
define/local line/c/1/79 "-" ALL
define/local masterfrm/c/1/80 " " ALL
define/local errmsg/c*80/1/8 " " ALL

define/local setcertif/i/1/1 0                  !should be 0 in released mode

!------------------------------------------------------------------------------
! keyword initialisation
!
errmsg(1) = "Error in INIT/PIPELINE occured !"
errmsg(2) = "Error in fors_gen_qc0_overexpo occured !"
errmsg(3) = "Error in OVERSCAN/BIAS occured !"
errmsg(4) = "Error in COMPUTE/MASTER occured !"
errmsg(5) = "Error in NORMFLAT/LSS ocured !"
errmsg(7) = "Error while identifying main product !"
errmsg(8) = "Error in FINISH/PIPELINE !"

i2   = inputi(2)
etyp = inputi(1)                        !exposure type:
procid(1) = "s_gen_master_bias"         ! 1: bias
procid(2) = "s_gen_master_dark"         ! 2: dark
procid(3) = "s_img_master_skyflat"      ! 3: skyflat
procid(4) = "s_img_master_scrflat"      ! 4: screen flat
procid(5) = "s_lss_normflat"            ! 5: norm flat (LSS)
procid(6) = "s_mos_normflat"            ! 6: norm flat (MOS)
pid = procid({etyp})                    !process name
if inputi(2) .eq. 1 pid = "s_img_master_nightflat"

minnum = m$abs({P5})

!------------------------------------------------------------------------------
! Pipeline initialisation
!
LOG/REC {pid} IN SUPER
@p fors_paragraph 1 0

part = 1
INIT/PIPELINE {P1} {P2} XSFinput.cat CCDRED + + XSFreject.cat XSFresult.cat
if {Q1} .gt. 0 goto ABORT
!
init/prohead {p1} {p2} 1,"Master calibration frame generation" -
  "avmode,{p4}" "minnum,{p5}" "thresholds,{p6}" "intfac,{p7}" "maxov,{p8}"
if {q1} .ne. 0 goto ABORT
!
init/longlog {p1} {p2} 1 "avmode: {p4}" "minnum: {p5}" "thresholds: {p6}" -
             "intfac: {p7}" "maxov: {p8}"
if {Q1} .gt. 0 goto ABORT
!
!now all input frames and reference frames are stored
!in the image catalog:   XSFinput.cat
!thus P2 may be ignored (+) in the following recipe-calls.
!
procid(1) = BIAS_ID                     ! 1: bias
procid(2) = DARK_ID                     ! 2: dark
procid(3) = SKY_ID                      ! 3: skyflat
procid(4) = FLAT_ID                     ! 4: screen flat
procid(5) = FLAT_ID                     ! 5: norm flat (LSS)
procid(6) = FLAT_ID                     ! 5: norm flat (MOS)
ityp = procid({etyp})                   !input type
if i2 .eq. 1 ityp = "_IMG"              !SCIENCE_IMG or STANDARD_IMG

procid(1) = MBIAS_ID                    ! 1: bias
procid(2) = MDARK_ID                    ! 2: dark
procid(3) = MSKY_ID                     ! 3: skyflat
procid(4) = MFLAT_ID                    ! 4: screen flat
procid(5) = MNFLAT_ID                   ! 5: norm flat (LSS)
procid(6) = MNFLAT_ID                   ! 5: norm flat (MOS)
otyp = procid({etyp})                   !output type
if i2 .eq. 1 otyp = MNIGHT_ID           !MASTER NIGHT FLAT
!
!
@p fors_paragraph 2 0
!
!------------------------------------------------------------------------------
! Check required minimum number of INPUT frames!
!
@p fors_mastercalib,CHK_MINUM XSFinput.cat {ityp}
if {Q1} .ge. minnum then
  n1 = {Q1}
else
  set/format I1
  mess/out 1 {pid} "Too few input frames: {Q1} < {minnum} (required minimum)"
  errmsg(1) = "Abort due to min.number of input frames not reached."
  goto ABORT
endif
!
if etyp .lt. 3 goto MKMA                !no overexpose-check for bias and dark
!
!------------------------------------------------------------------------------
! Check if flatfields are overexposed
!
part = 2
write/out
mess/out 3 {pid} "Check overexposed pixels ..."
write/out {line}
@p fors_gen_qc0_overexpo XSFinput.cat + XSFinput.cat,XSFresult.cat {P8}
if {Q1} .gt. 0 goto ABORT
!
!------------------------------------------------------------------------------
!do the master creation
!
MKMA:
part = 4
write/out
mess/out 3 {pid} "Create master frame ..."
comput/mast XSFinput.cat + XSFinput.cat,XSFresult.cat {P4} {P5} {P6} {P7}
if {Q1} .ne. 0 goto ABORT
!
!------------------------------------------------------------------------------
!NORM-flat creation
!
Q1 = "0 "                                               !do the following
if etyp .lt. 4 goto CONT                                !only for screen flats,
part = 5                                                !otherwise skip.
BRANCH fors_insmode(1:3) IMG,LSS,MOS IMG,LSS,MOS
goto CONT
IMG:
  goto CONT

LSS:
  mess/out 3 {pid} "Normalize master frame ..."
  write/out {line}
  normflat/lss XSFinput.cat + XSFinput.cat,XSFresult.cat LSS
  otyp = MNFLAT_ID                      !DO-classification of master frame
  goto CONT

MOS:
  mess/out 3 {pid} "Normalize master frame ..."
  write/out {line}
  errmsg(5) = "Error in NORMFLAT/MOS ocured !"
  normflat/mos XSFinput.cat + XSFinput.cat,XSFresult.cat MOS
  otyp = MNFLAT_ID                      !DO-classification of master frame

CONT:
  if {Q1} .ne. 0 goto ABORT

FINI:
!------------------------------------------------------------------------------
!write FITS files to specified output directory
!
part = 7
@p fors_paragraph 3 0
search/icat XSFresult {otyp} >Null
if m$len(out_a) .gt. 0 then
  masterfrm = out_a
else
  mess/out 1 {pid} "No result frame ({otyp}) found !"
  goto ABORT
endif


!------------------------------------------------------------------------------
!close the pipeline
!
part = 8
write/out
mess/out 3 {pid} "Finish Pipeline"
finish/prohead XSFresult.cat 1
FINISH/PIPELINE XSFresult.cat {P3} CCDRED {masterfrm}
if {Q1} .gt. 0 goto ABORT
finish/longlog XSFinput.cat XSFresult.cat {masterfrm} 1 {p3}
@p fors_mastercalib,CHK_STATUS                          !recipe quality check

LOG/REC {pid} OUT SUPER
@p fors_paragraph 4 0
return 0


ABORT:
finish/longlog XSFinput.cat -1 ? 1 {p3}
send/mess "No master frame created ({otyp})!"
mess/out 1 {pid} "{errmsg({part})}"
write/error {part},APP,NO "{errmsg({part})}"




!
!=== SUBROUTINES ==============================================================

ENTRY CHK_STATUS
if RECQUALI .eq. 1 then                                 !check recipe status
  RECQUALM = "Master frame of less quality (see log) !"
else
  write/key RECQUALM " " all
endif



ENTRY CHK_MINUM
define/param p1 ? c "Enter name of INPUT catalog:"
define/param p2 ? c "Enter DO-classification to look for:"

define/local cnt/i/1/1 0
define/local fctrl/i/1/2 0 ALL

show/icat {P1} N

open/file {P1} READ fctrl
read/file {fctrl(1)} outputc
loop:
  read/file {fctrl(1)} outputc
  if fctrl(2) .gt. 0 then
    if m$index(outputc,p2) .gt. 0 then
      if m$index(outputc,"MASTER") .eq. 0 cnt = cnt + 1
    endif
    goto loop
  endif

close/file {fctrl(1)}
set/format I1
return {cnt}
