! $Id: fors_medsigmed.prg,v 1.61 1999/11/25 11:23:56 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_medsigmed.prg,v $
!
! .KEYWORDS   Statistics
!
! .PURPOSE    Handle in- and output for "fors_medsigmed.c" and invoke it.
! __________
!
! .USAGE      @p fors_medsigmed ima [scan] [calc] [out]
!
! .INPUT      ima  (P1): input frame;
!                          default: none
!
!             scan (P2): number of pre- AND overscan columns, separated by a
!                        comma;
!                          default: 16,16
!
!             calc (P3): calculation flag: <>:calculate median, or use entered
!                        median value;
!                          default: <>
!
!             mad  (P4): additional calc. flag: <>:calculate MAD_MED, <n>o;
!                          default: <>
!
!             out  (P5): flag for result output on terminal: <>:show, <n>o;
!                          default: <>
!
! .OUTPUT     OUTPUTR(8) : median
!             OUTPUTR(15): median deviation ("1 sigma") from the median
!             if P4(mad)!=n => OUTPUTR(14): mean abs.dev. from the median
!
!             Descriptor STATISTIC(8) : the same
!             Descriptor STATISTIC(15): the same
!             if P4(mad)!=n => Descriptor STATISTIC(14): the same
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    -
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/08/21  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref ima scan calc mad out
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "medsigmed"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!                                ! also needed for "fors_medsigmed.c"
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      ima  (P1): input frame;"
  w/o "                   default: none"
  w/o " "
  w/o "      scan (P2): number of pre- AND overscan columns, separated by a"
  w/o "                 comma;"
  w/o "                   default: 16,16"
  w/o " "
  w/o "      calc (P3): calculation flag: <>:calculate median, or use entered"
  w/o "                 median value;"
  w/o "                   default: <>"
  w/o " "
  w/o "      mad  (P4): additional calc. flag: <>:calculate MAD_MED, <n>o;"
  w/o "                   default: <>"
  w/o " "
  w/o "      out  (P5): flag for result output on terminal: <>:show, <n>o;"
  w/o "                   default: <>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of frame to be tested: "
write/keyword in_a "{p1}"
if m$index(in_a,".bdf") .eq. 0 in_a = in_a//".bdf"
!
if m$exist(in_a) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({in_a})"
  goto error
endif
!
!
define/parameter p2 16,16 ? -
  "        Enter pre-/overscan values [def.:16,16]: "
outputi = m$parse(p2,"scan")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
write/keyword inputi/i/1/1 {scan01}
write/keyword inputi/i/2/1 {scan02}
!
if inputi(1) .lt. 0 .or. inputi(2) .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible pre- or overscan value !"
  goto error
endif
!
!
define/parameter p3 + ? -
  "        Enter calc. flag [def.:calc. median] or median value: "
if m$tstno(p3) .eq. 1 then
  write/keyword inputc/c/1/1 "u"  ! use entered median
  write/keyword inputr/r/1/1 {p3}
else
  write/keyword inputc/c/1/1 "+"  ! calculate median
endif
!
!
define/parameter p4 y ? -
  "        Enter flag for additional MAD_MED calc. [def.:y|<n>o]: "
write/keyword inputc/c/2/1 "{p4}"
define/local addmad/c/1/1 "{p4}"
!
!
define/parameter p5 y ? -
  "        Enter flag for result output [def.:show|<n>o]: "
define/local show/c/1/1 "{p5}"
!
!
!          INVOCATION :
!          **********
!
!******************************
run PIPE_EXE:fors_medsigmed.exe
!******************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Calculation of deviation failed !"
  goto error
endif
!
!
!          OUTPUT :
!          ******
!
if show .ne. "n" .and. mess_level .ge. 3 then
        write/out " "
        set/format T12.4,
        mess/out 3 {PROID} "Results:"
        mess/out 3 {PROID} "-------"
        mess/out 3 {PROID} "Median               : {outputr(8)}"
        mess/out 3 {PROID} "Med.dev. of Med.     : {outputr(15)}"
  if addmad .ne. "n" then
        mess/out 3 {PROID} "Mean abs.dev. of Med.: {outputr(14)}"
  endif
endif
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------
