! $Id: fors_merorder.prg,v 1.7 2001/04/06 16:19:23 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_merorder.prg,v $
!
! .KEYWORDS   Order merging
!
! .PURPOSE    Merge all rebinned orders. Merged frame and all identified wave-
!             lengths can be displayed.
! __________
!
! .USAGE      @p fors_merorder rebimg [disptbl] [disp]
!
! .INPUT      rebimg  (P1): rebinned order frame;
!                             default: none
!
!             disptbl (P2): output table from "fors_echwave_dis.prg";
!                             default: <DISPDEF> (see below)
!
!             disp    (P3): display selection:<p>:pipeline mode:read global key
!                           NICE_CREA [y|n], <n>:none, <l>:local, or
!                           host:display[.screen] - with the latter option
!                           you can specify the name or IP-address of a display;
!                             default: <l>
!
! .OUTPUT     Q2: frame with merged orders "<P1>_mer.bdf"
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  In the overlapping regions <SKIPANGS> (see below) angstroms are
!             skipped at both edges. Then a weighted average is computed. The
!             normalized weight is a linear ramp between 0 and 1.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.5  99/11/10  Creation
!                  99/       Release
!
!------------------------------------------------------------------------------
!
!
crossref rebimg disptbl disp
!
!
define/local SKIPANGS/r/1/1 5.  ! skipped [angstrom] at order edges
define/local DISPDEF/c/1/20 "dispcoef.tbl"
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "merorder"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      rebimg  (P1): rebinned order frame;"
  w/o "                      default: none"
  w/o " "
  w/o "      disptbl (P2): output table from "fors_echwave_dis.prg";"
  w/o "                      default: {DISPDEF}"
  w/o " "
  w/o "      disp    (P3): display selection:<p>:pipeline mode:read global key"
  w/o "                    NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                    host:display[.screen] - with the latter option"
  w/o "                    you can specify the name or IP-address of a display;"
  w/o "                      default: <l>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(rebimg):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of rebinned order frame: "
define/local rebimg/c/1/64 "{p1}"
if m$index(rebimg,".bdf") .eq. 0 rebimg = rebimg//".bdf"
!
if m$exist(rebimg) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({rebimg})"
  goto error
endif
inputi = {{rebimg},NPIX(2)}
if m$existd(rebimg,"WSTART") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({rebimg},WSTART)"
  goto error
endif
show/descriptor {rebimg} WSTART h
if outputi(3) .lt. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "Descriptor has too few elements"
        mess/out 1 {PROID} "({rebimg},WSTART) - {inputi(1)} needed !"
  goto error
endif
if m$existd(rebimg,"NPTOT") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({rebimg},NPTOT)"
  goto error
endif
show/descriptor {rebimg} NPTOT h
if outputi(3) .lt. inputi(1) then
        write/out " "
        mess/out 1 {PROID} "Descriptor has too few elements"
        mess/out 1 {PROID} "({rebimg},NPTOT) - {inputi(1)} needed !"
  goto error
endif
!
!
!P2(disptbl):
!-----------
define/local disptbl/c/1/60 " " all
disptbl = DISPDEF
define/parameter p2 {disptbl} ? -
  "        Enter table w. finally identified lines: "
disptbl = p2
if m$index(disptbl,".tbl") .eq. 0 disptbl = disptbl //".tbl"
!
if m$existc(disptbl,":X") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({disptbl})"
  goto error
elseif m$existc(disptbl,":X") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({disptbl},:X)"
  goto error
elseif m$existc(disptbl,":Y") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({disptbl},:Y)"
  goto error
elseif m$existc(disptbl,":WAVEC") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({disptbl},:WAVEC)"
  goto error
elseif m$existc(disptbl,":IDENT") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Column is not present !"
        mess/out 1 {PROID} "({disptbl},:IDENT)"
  goto error
endif
!
if {{disptbl},TBLCONTR(4)} .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered line table is empty !"
        mess/out 1 {PROID} "({disptbl})"
  goto error
endif
!
!
!P3(disp):
!--------
define/parameter p3 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p3}"
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
define/local char/i/1/1 0
define/local merimg/c/1/64 " " all
define/local scrdif/i/1/1 0
define/local mertbl/c/1/64 " " all
!
!
!
!          MANAGING DISPLAYS / ITT / LUT :
!          *****************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_dids(3) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 3 is missing !"
    goto error
  endif
  if nice_gids(3) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 3 is missing !"
    goto error
  endif
  !
  !if nice_dids(1) .eq. 1 then
  !  assign/display d,1 >Null
  !  clear/display
  !  display/lut off
  !endif
  !if nice_dids(2) .eq. 1 then
  !  assign/display d,2 >Null
  !  clear/display
  !  display/lut off
  !endif
  assign/display d,3 >Null
  clear/display
  display/lut off
  label/display "MERGED" {nice_dlab}
  load/lut heat
  load/itt ramp
  !if nice_gids(1) .eq. 1 then
  !  assign/graphic g,1 >Null
  !  clear/graphic
  !endif
  !if nice_gids(2) .eq. 1 then
  !  assign/graphic g,2 >Null
  !  clear/graphic
  !endif
  assign/graphic g,3 >Null
  clear/graphic
  modify/graphic window >Null
  set/graphic default
  set/graphic pmode=0
  !
endif
!
!
!          MERGING REBINNED ORDER FRAME :
!          ****************************
!
        set/format T4.1,
        mess/out 3 {PROID} "  skipping {SKIPANGS} A  in overlaps (both edges),"
        mess/out 3 {PROID} "  comput. weight.avg. of overlaps (norm.lin.ramps),"
        mess/out 3 {PROID} "  connecting all orders"
        set/format {FMTDEF}
!
p1 = rebimg
  char = m$indexb(rebimg,"_reb")-1
  merimg = rebimg(1:{char})//"_mer.bdf"
p2 = merimg
p4 = "average"
inputr(1) = SKIPANGS
inputr(2) = -99.
!
!***********************
run STD_EXE:necmerge.exe
!***********************
!
!
if nice_crea .eq. "y" then
  !
  !
  !        DISPLAYING OF MERGED FRAME :
  !        **************************
  !
        mess/out 4 {PROID} "  displaying merged frame"
  !
  !**********************************************
  @p fors_nicedisp {merimg} ? 7. 0,0 ? ? 0.5,4. n
  !**********************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of merged frame failed !"
        mess/out 1 {PROID} "({merimg})"
    goto error
  endif
  !
  ! MAGNIFY IMAGE PERPEND. TO DISP. DIRECTION:
  ! -----------------------------------------
  outputi(2) = nice_dscr(2)/{{rebimg},NPIX(2)}  ! use only part of disp. height
  {merimg},DISPLAY_DATA(2) = -1*{outputi(2)}
  !
        mess/out 4 {PROID} "    loading merged image into display 3"
  assign/display d,3 >Null
  load/image {merimg} ? {outputi(1)},{outputi(2)} c,c  !### 2)},max ###
  label/display "MERGED" {nice_dlab}  ! removed by special load/image
  label/display "(y-averaged)" 10,10 ? green 1
  !
  ! RECTANGLE AROUND FRAME:
  ! ----------------------
  @a getcoord {merimg} @1,@0 >Null
  inputi(1) = outputi(1)
  inputi(2) = outputi(2)
  @a getcoord {merimg} @{{merimg},NPIX},@1 >Null
  inputi(2) = (inputi(2)+outputi(2))/2
  outputi(2) = outputi(2)*2-inputi(2)
  draw/rectangle {inputi(1)},{inputi(2)},{outputi(1)},{outputi(2)} -
    s ? green >Null
  !
  !
  !        VISUALIZATION OF IDENTIFIED WAVELENGTHS :
  !        ***************************************
  !
        mess/out 4 {PROID} "  plotting identified wavelengths"
  !
  copy/it {merimg} tmp.tbl :WAVEC
  name/column tmp.tbl :LAB002 :PEAK
  select/table tmp.tbl :PEAK.gt.0. >Null
    char = m$indexb(rebimg,"_reb")-1
    mertbl = rebimg(1:{char})//"_mer.tbl"
  copy/table tmp.tbl {mertbl}
  -delete tmp.tbl
  assign/graphic g,3 >Null
  set/graphic stype=0 ltype=1 colour=1
  plot/table {mertbl} :WAVEC log(:PEAK)
  set/graphic colour=4
  !***************
  @p fors_nicegrap
  !***************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of plot failed !"
    goto error
  endif
  label/graphic "IDENT (0.:not.ident.)" {nice_glabw}
  !
  sort/table {disptbl} :WAVEC
  p1 = disptbl
  p2 = ":WAVEC"  ! x-position
  p3 = ":IDENT"  ! label with identification
  p4 = "top"
  pcount(1) = 4
  !***********************
  run STD_EXE:overiden.exe
  !***********************
  plcdata(1:60) = disptbl
  plcdata(61:80) = "table"
  copy/graphic
  set/graphic colour=1
  sort/table {disptbl} :Y,:X
  !
  !
endif
!
!
set/format
if nice_crea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0 {merimg}
else
  return -1 {merimg}
endif
!
error:
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" set/graphic default
  endif
  return 1 -99
!
!
!------------------------------------------------------------------------------
