! $Id: fors_multiplt.prg,v 1.30 2000/09/28 12:28:29 nrainer Exp $
!+++ FORS_MULTIPLT +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_multiplt.prg,v $
!
!.KEYWORDS      Create a multi column multi row image plot
!
!.USAGE         @p fors_multiplt img_lst numc numr ixsz iysz cut text hardcp
!
!.INPUT         img_lst (P1): list of images, 3 different possibilities:
!                               'comma'-list,
!                               MIDAS iamge catalog
!                               ASCII list of images
!               numc    (P2): number of columns
!               numr    (P3): number of rows
!               ixsz    (P4): image x-size
!               iysz    (P5): image y-size
!               cut     (P6): cut specification (used in LOAD/IMA)
!               text    (P7): text will be put on bottom of the display
!                               (only valid for second column)
!               hardcp  (P8): H: create a hardcopy (POSTSCRIPT),
!                                output: multiplt.ps
!                             H,filename: same as H but output: filename
!                             N: NO hardcopy (= default)
!
!.OUTPUT        If parameter P2 AND/OR P3 are set to 0 no plot will be made!
!
!.RETURN        Q1:  0: successful return
!                    otherwise: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Create a multi column multi row image plot
!
!.ALGORITHM     
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1998/07/14  Creation
!               1.0 1998/12/09  Labeling improved
!               1.2 1999/02/09  scale bug fixed
!
!.COMMENT       In case of a negative P4 an already existing image display
!               is assumed. Then only the scaling factors will be calculated
!               and the series of images displayed.
!------------------------------------------------------------------------------
!
!
! Check parameters
!
define/parameter p1 ? character
define/parameter p2 1 number
define/parameter p3 1 number
define/parameter p4 100 number
define/parameter p5 100 number
define/parameter p6 3sigma character
define/parameter p7 "Processed by multiplt.prg" character
define/parameter p8 N character


if {p2} .eq. 0 .OR. {P3} .eq. 0 return
if nice_crea .eq. "n" return            !GRAPHIC disabled: do nothing

define/local MXSZ/i/1/1 1600            !has to be adjusted ..
define/local MYSZ/i/1/1 1200            !to terminal in use.

define/local XSIZE/i/1/1 0              !display size in x-direction

define/local catal/i/1/1 0
define/local flag/c/1/1 "N"
define/local date/c/1/24 " " ALL
define/local inst/c/1/20 " " ALL
define/local dim/i/1/2 0,0
define/local inum/i/1/1 0
define/local ic/i/1/1 0
define/local ir/i/1/1 0
define/local it/i/1/1 0
define/local xdm/r/1/1 0
define/local ydm/r/1/1 0
define/local xsc/r/1/1 0
define/local ysc/r/1/1 0
define/local xsz/i/1/1 0
define/local ysz/i/1/1 0
define/local xcn/i/1/1 0
define/local ycn/i/1/1 0
define/local xof/r/1/1 0
define/local yof/r/1/1 0
define/local x0/i/1/1 0
define/local x1/i/1/1 0
define/local y1/i/1/1 0
define/local x2/i/1/1 0
define/local y2/i/1/1 0
define/local ntot/i/1/1 0
define/local dispnr/i/1/1 1
define/local CREADISP/i/1/1 1

ntot = {P2} * {P3}
define/local frms/c*80/1/{ntot}
define/local labs/c*20/1/{ntot}
define/local SWITCH/i/1/6 0 ALL +lower_levels


date = m$time()
inum = m$index(P1,",")
if inum .gt. 0 then                                     !"comma" input-list
  inum = m$parse(P1,"frm")
  set/format I2
  do ir = 1 inum 1
    frms({ir}) = frm{ir}
    parse/path {frm{ir}}
    labs({ir}) = Q2(1:20)
  enddo
  set/format
else
  @p fors_parsepath {P1}
  if Q3(1:4) .eq. ".cat" then                           !image catalog
    inum = 1
    i_loop:
      store/frame outputc {P1} 1 i_fini
      frms({inum}) = outputc
      parse/path {outputc}
      labs({inum}) = Q2(1:20)
      inum = inum + 1
      goto i_loop
    i_fini:
  else if Q3(1:4) .eq. ".lis" then                      !ASCII list file
    if m$exist("{p1}") .eq. 1 then
      inum = 0
      open/file {p1} READ outputi
    nxt:
      read/file {outputi(1)} outputc 80
      if outputi(2) .ne. -1 .AND. inum .lt. ntot then
        inum = inum + 1
        frms({inum}) = outputc
        parse/path {outputc}
        labs({inum}) = Q2(1:20)
        goto nxt
      endif
      close/file {outputi(1)}
    endif
  else if Q3(1:4) .eq. ".bdf" then                      !single frame
     frms(1) = P1
     parse/path {P1}
     labs(1) = Q2(1:20)
  endif
endif



copy/dk {frms(1)} NPIX dim

xdm = {P4}
ydm = {P5}
if {P4} .lt. 0 then     !P4 now specifys display number!
  CREADISP = 0
  dispnr = -{P4}
  if dispnr .gt. 9 then
    mess/out 2 multiplt "Wrong display number ({dispnr}), set to 1!"
    dispnr = 1
  endif
  assign/disp D,{dispnr} >Null
  clear/channel
  clear/channel OVERL
  show/disp >Null
  XSIZE = outputi(2)                    !x-size of actual image display
  xdm = XSIZE/({P2}+0.5)                !maximum allowed display size of each
  ydm = outputi(3)/({P3}+1)             !... image in each column and row.
  xdm = m$nint(xdm)                     !only INTEGER are usefull
  ydm = m$nint(ydm)                     !... dimensions for the image!
  if xdm .lt. 120 .OR. ydm .lt. 120 then
    set/format I1                       !If window too small then
    do inum = 1 {ntot}                  !write the filenumber
      labs({inum}) = "{inum}"           !instead of its name
    enddo                               !as a label in the upper left corner
    set/format                          !of each window.        
  endif         

  @p fors_multiplt,SCALE {dim(1)} {xdm}
  xsc = outputi(1)
  xdm = outputi(2)
  @p fors_multiplt,SCALE {dim(2)} {ydm}
  ysc = outputi(1)
  ydm = outputi(2)
  if xsc .gt. ysc then                  !take the smaller scale factor for both
    xsc = ysc
    xdm = ydm
  else
    ysc = xsc
    ydm = xdm
  endif
endif
xof = xdm / 4
yof = ydm / 2
xsz = {P2} * xdm + 2 * xof              !xsize of window
ysz = {P3} * ydm + 2 * yof              !ysize of window
y1  = 500-ysz  + 650

xcn = XSIZE / 2.0                       !center coordinates ...
ycn = ysz - yof / 2 - ydm               !of image display
if xsz .gt. MXSZ then
  mess/out 1 multiplt "Specified X-size ({xsz}>{MXSZ}) is too big!"
  return -1
endif
if ysz .gt. MYSZ then
  mess/out 1 multiplt "Specified Y-size ({ysz}>{MYSZ}) is too big!"
  return -2
endif

if CREADISP .eq. 1 then
  delete/disp 9 >Null
  create/disp 9 {xsz},{ysz},0,{y1}
  x1 = xof - xdm
else
  x1 = xcn - {P2} * xdm / 2 - xdm
endif
!
it = 0
do ic = 1 {P2} 1
  x1 = x1 + xdm                 !(x1,y1) lower left corner
  x2 = x1 + xdm                 !(x2,y2) upper right corner
  y1 = ycn + ydm
  do ir = {P3} 1 -1
    it = it + 1
    x0 = x1
    x1 = x1 + 4
    y2 = y1 - 14
    y1 = y1 - ydm
    if m$len(frms({it})) .gt. 0 then
      if m$exist(frms({it})) .eq. 1 then
        @% loadima {frms({it})} fix=1,1,{x0},{y1} dirs=Up,{flag} -
             cuts=F,{P6} scale={xsc},{ysc} !>Null
        label/display {labs({it})} {y2},{x1} O blue
      endif
    endif
    x1 = x1 - 4                 !reset
    y2 = y2 + 14                !reset
    draw/rectangle {x1},{y1},{x2},{y2} S
    flag = "O"
  enddo

enddo
y1  = y1  - 28
@p fors_multiplt,LABELDISP {P7} {xcn} {y1}
y1  = y1  - 14
@p fors_multiplt,LABELDISP  "PLOT MADE ON: {date(1:23)}" {xcn} {y1}

if P8(1:1) .eq. "H" then
  mess/out 3 multiplt "Create hardcopy ..."
  define/local psname/c/1/100 multiplt.ps
  if P8(2:2) .eq. "," then
    outputi(1) = m$len(P8)
    if outputi(1) .gt. 2 then
      psname =  "{P8(3:{outputi(1)})}"
    endif
  endif
  copy/display LASER NOSTOP N ? NOPRINT
  -rename screen{mid$sess(11:12)}.ps {psname}
  mess/out 3 multiplt "POSTSCRIPT file `{psname}` created."
endif



ENTRY SCALE
define/param p1 ? n "Enter image dimension:"
define/param p2 ? n "Enter image window size:"

define/local dim/r/1/1 0.0
define/local wsz/r/1/1 0.0
define/local q/r/1/1 0.0
define/local sca/i/1/1 0

dim = {p1}
wsz = {p2}
if dim .gt. wsz then
  q = dim / wsz + 0.5
  sca = m$nint( q )
  if sca .eq. 0 sca = 1

  wsz = dim / sca + 0.5
  wsz = m$nint( wsz )
  q = dim / sca
  sca = -{sca}
else if dim .le. wsz then
  q = wsz / dim - 0.5
  sca = m$nint( q )
  if sca .eq. 0 sca = 1

  wsz = dim * sca + 0.5
  wsz = m$nint( wsz )
  q = dim * sca
endif

outputi(1) = sca
outputi(2) = wsz


ENTRY LABELDISP
define/param P1 ? c "label text:"
define/param P2 ? n "x-position"
define/param P3 ? n "y-position"

define/local xof/i/1/1 0
define/local yof/i/1/1 {P3}
define/local FONTWIDTH/i/1/1 6          !font width of normal font: 6 pixel

xof = m$strlen(P1) * FONTWIDTH
xof = {P2} - xof / 2

@p fors_max 0,{xof}
xof = {Q1}
@p fors_max 0,{yof}
yof = {Q1}

label/display {P1} {yof},{xof}
