! $Id: fors_nice.prg,v 1.40 2001/04/06 12:30:59 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_nice.prg,v $
!
! .KEYWORDS   Displays
!
! .PURPOSE    Inquire, whether displays are requested
!             and invoke fors_nicecrea.prg.
! __________
!
! .USAGE      @p fors_nice
!
! .INPUT      PIPELINE(5)    : flag for displays: <0>:no displays, <1>:displays
!
!             PIPELINE_DISP  : display selection: <local>:use local display, or
!                              host:display[.screen] - with the latter option
!                              you can specify the name or IP-address of a
!                              display;
!                                hint: ignored, if PIPELINE(5)=0
!
!             PIPELINE_AUX(3): percentage of root window width to be used as
!                              display area (windows will be squeezed) -
!                              possible range: [20.-100.], values lower than 20.
!                              are set to 20., higher than 100. to 100.;
!                                hint: ignored, if PIPELINE(5)=0
!
!             PIPELINE(10)   : number of Midas session to be used (necessary for
!                              background session);
!                                hint: ignored, if PIPELINE(5)=0
!
! .OUTPUT     if PIPELINE(5)=0 => NICE_CREA=n
!
!             if PIPELINE(5)=1 => NICE_CREA=y
!
!                                 NICE_SCR : needed for fors_nicedisp.prg
!
!                                 NICE_DLAB: useful for
!                                            `label/display "text" {NICE_DLAB}'
!                                            use
!                                            `clear/channel overlay' before,
!                                            because of MIDAS colour problems !
!
!                                 NICE_GLAB: useful for
!                                            `label/graphic "text" {NICE_GLAB}'
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    -
!
! .ENVIRON    FORS pipeline
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/08/28  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
define/local PROID/c/1/40 "Nice"
!
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      PIPELINE(5)    : flag for displays: <0>:no displays, <1>:displays"
  w/o " "
  w/o "      PIPELINE_DISP  : display selection: <local>:use local display, or"
  w/o "                       host:display[.screen] - with the latter option"
  w/o "                       you can specify the name or IP-address of a"
  w/o "                       display;"
  w/o "                         hint: ignored, if PIPELINE(5)=0"
  w/o " "
  w/o "      PIPELINE_AUX(3): percentage of root window width to be used as"
  w/o "                       display area (windows will be squeezed) -"
  w/o "                       possible range: [20.-100.], values lower than 20."
  w/o "                       are set to 20., higher than 100. to 100.;"
  w/o "                         hint: ignored, if PIPELINE(5)=0"
  w/o " "
  w/o "      PIPELINE(10)   : number of Midas session to be used (necessary for"
  w/o "                       background session);"
  w/o "                         hint: ignored, if PIPELINE(5)=0"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          INVOCATION OF DISPLAY CREATION :
!          ******************************
!
if RBINSTRUMENT .eq. RBINS_OLD then 
  if pipeline(5) .eq. p_dspold .and. pipeline_aux(3) .eq. p_sizold then
    if mid$sess(6:6) .ne. "-" .and. mid$sess(7:7) .ne. "-" goto do_nothing
  endif
endif
!
define/local dispflag/c/1/1 " "
if pipeline(5) .eq. 1 then
  dispflag = "y"         ! "p_dspmode":
  p_dspmode = ididev(7)  ! 0 =  8bit: PseudoColour
else                     ! 1 = 24bit: RGB
  dispflag = "n"         ! 2 = 24bit: Pseudo on DirectColour
endif                    ! 3 = 24bit: Pseudo on TrueColour
!
RBINS_OLD = RBINSTRUMENT
p_dspold = pipeline(5)
p_sizold = pipeline_aux(3)
!
        mess/out 4 {PROID} "Checking display options"
!
!***************************************************************
if dispflag .eq. "y" then
  !
  define/local scale/r/1/1 100.
  scale = pipeline_aux(3)
  if scale .lt. 20. scale = 20.
  if scale .gt. 100. scale = 100.
  !
  define/local session/c/1/2 " " all
  if pipeline(10) .eq. -1 then
    session(1:2) = mid$sess(11:12)
  else
    set/format I2
    session(1:2) = "{pipeline(10)}"
    set/format I1
  endif
  !
  if pipeline_disp .eq. "local" then
     mess/out 3 {PROID} "Local screen will be used"
     @p fors_nicecrea l {scale} ? {session(1:2)}
  else
     mess/out 3 {PROID} "<{pipeline_disp}> will be used"
     @p fors_nicecrea {pipeline_disp} {scale} ? {session(1:2)}
  endif
  !
else
        mess/out 3 {PROID} "No displays will be created"
  @p fors_nicecrea n
endif
!***************************************************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
  goto error
endif
        write/out " "
!
!
do_nothing:
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  return 1
!
!
!-------------------------------------------------------------------------------
