! $Id: fors_niceclear.prg,v 1.7 1999/11/25 11:23:58 sbogun Exp $   
!+++ FORS_NICECLEAR +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_niceclear.prg,v $
!
!.KEYWORDS      Clear display and/or graphics windows
!
!.USAGE         @p fors_niceclear sof_in out sof_ref [optpar]
!
!.INPUT         mode    (P1): selects 'devices' to be cleared
!                               D: all image displays
!                               G: all graphic displays
!
!.OUTPUT        
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed (keywords missing)
!                    1: graphics output disabled by pipeline
!
!.DEFAULT
!
!.PURPOSE       Clears all active devices. Image or graphic displays may be
!               selected, defaulted to both. At the end the default ITT will
!               be set to RAMP, LUT to HEAT.
!
!.ALGORITHM     Checks existence of NICE-keywords: NICE_CREA, NICE_DIDS and
!               NICE_GIDS. Then loop through the ID-arrays which indicate the
!               existence of the graphics-windows and perform a CLEAR/DISP or
!               CLEAR/GRAPH. Graphic window 3 is iconified. It can be de-iconified
!       with
!         assign/graphic g,3 >Null
!         modify/graphic window >Null
!
!.ENVIRON       FORS context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/01/07  Creation.
!           1.2 1999/02/05      load/itt ramp included.
!           1.5 1999/11/11  Iconification of graphic window 3 included.
!
!.COMMENT
!------------------------------------------------------------------------------

define/param p1 A c "Enter D: image-display, G: graphic-display, A: all "

define/local i1/i/1/1 0
define/local mode/c/1/1 {P1}

if m$existk("nice_crea") .eq. 0 return -1
if nice_crea .ne. "y" return 1                  !graphics output disabled


set/format I1

branch mode A,D,G ALL,DIS,GRA

ALL:
DIS:
  if m$existk("nice_dids") .eq. 0 return -1

  do i1 = 1 9
    if nice_dids({i1}) .eq. 1 then
      assign/display d,{i1} >Null
      clear/display
      display/lut off
    endif
  enddo
  if nice_dids(10) .eq. 1 then
    assign/display d,0 >Null
    clear/display
    display/lut off
  endif
  if mode .eq. "A" goto GRA
  goto fini

GRA:
  if m$existk("nice_gids") .eq. 0 return -1

  do i1 = 1 9
    if nice_gids({i1}) .eq. 1 then
      assign/graphic g,{i1} >Null
      clear/graphic
    endif
  enddo
  if nice_gids(10) .eq. 1 then
    assign/graphic g,0 >Null
    clear/graphic
  endif
  set/graphic default

  assign/graphic g,3 >Null  ! special large window on top of g,1 and g,2
  modify/graphic icon >Null

fini:
  assign/display d,1 >Null
  assign/graphic g,1 >Null
  load/itt ramp
  load/lut heat
  set/format
  return 0
