! $Id: fors_nicecrea.prg,v 1.60 2001/04/06 12:31:00 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_nicecrea.prg,v $
!
! .KEYWORDS   Displays
!
! .PURPOSE    If P1!=n : Determine window sizes and create displays.
!             If P1=n  : Use key to inhibit further display related commands.
!
! .USAGE      @p fors_nicecrea [disp] [scale] [dots] [sess]
!
! .INPUT      disp  (P1): display selection: <n>:none, <l>:local, or
!                         host:display[.screen] - with the latter option
!                         you can specify the name or IP-address of a display;
!                           default: <l>
!             scale (P2): percentage of root window width to be used as
!                         display area (windows will be squeezed) - [20.-100.];
!                           default: <100.>
!             dots  (P3): 4 integer values (numbers of pixels), separated by
!                         commas: width of root window, height of root window,
!                         height of window manager title bar, size of window
!                         manager border - e.g. 1600,1200,21,4 (omits temporary
!                         creation of small help window);
!                           default: calculated within procedure
!             sess  (P4): number of Midas session to be used (necessary for
!                         background session);
!                           default: session in which this procedure is invoked
!
! .OUTPUT     if P1(disp)=n  => NICE_CREA/c = n
!             if P1(disp)!=n => NICE_CREA/c = y
!
!                               NICE_DIDS/i : (1:10) 1:disp.exists, 0:no
!                               NICE_GIDS/i : (1:10) 1:grap.exists, 0:no
!                                 (10) corresponds to window 0 !
!                               NICE_DDIM/i : (1:2) display x-/y-size
!                               NICE_GDIM/i : (1:2) graphic x-/y-size
!                               NICE_DSCR   : needed for fors_nicedisp.prg
!                               NICE_DPOS/c : useful for
!                                            `label/display "text" {NICE_DPOS}
!                                             {mode} {colour} {size}';  use
!                                            `clear/channel overlay' before,
!                                             because of MIDAS colour problems !
!                               NICE_DLAB/c : useful for
!                                            `label/display "text" {NICE_DLAB}';
!                                             use
!                                            `clear/channel overlay' before,
!                                             because of MIDAS colour problems !
!                               NICE_GLAB/c : useful for
!                                            `label/graphic "text" {NICE_GLAB}'
!                               NICE_DISP/c : P1(disp) is saved
!
! .RETURN     Q1:  0: successful return
!                  1: an error occured
!
! .COMMENT    If P1!=n : Display windows 1,2,3 and graphics windows 1,2,3 are
!             created. If P1=l(ocal), the graphics windows are smaller because
!             of the needed xterms (also for P2=100.). Graphic window 3 is a
!             special large window and therefore iconified. It can be used with
!             the following commands:
!               assign/graphic g,3 >Null
!               modify/graphic window >Null
!             After use it should be re-iconified with:
!               assign/graphic g,3 >Null
!               modify/graphic icon >Null
!
!             Used UNIX commands: $ xwininfo (windows' sizes)
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/08/27  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!             1.4  99/10/21  Release
!             1.5  99/       Release
!	      1.6  01/04/06  KB
!
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
crossref disp scale dots sess
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "nicecrea"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!
if p1 .eq. "??" then
wr/out "      *****************************************************************"
wr/out "      disp  (P1): display selection: <n>:none, <>:local, or"
wr/out "                  host:display[.screen] - with the latter option"
wr/out "                  you can specify the name or IP-address of a display;"
wr/out "                    default: <>"
wr/out " "
wr/out "      scale (P2): percentage of root window width to be used as"
wr/out "                  display area (windows will be squeezed) - [20.-100.];"
wr/out "                    default: <100.>"
wr/out " "
wr/out "      dots  (P3): 4 integer values (numbers of pixels), separated by"
wr/out "                  commas: width of root window, height of root window,"
wr/out "                  height of window manager title bar, size of window"
wr/out "                  manager border - e.g. 1600,1200,21,4 (omits temporary"
wr/out "                  creation of small help window);"
wr/out "                    default: calculated within procedure"
wr/out " "
wr/out "      sess  (P4): number of Midas session to be used (necessary for"
wr/out "                  background session);"
wr/out "                    default: session in which this procedure is invoked"
wr/out "      *****************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
define/local rgbflag/i/1/1 0  ! for TrueColour displays
rgbflag = ididev(7)
reset/display >Null
ididev(7) = rgbflag
!
!          INPUT :
!          *****
!
!Global variables:
!----------------
write/keyword nice_crea/c/1/1 n
write/keyword nice_dids/i/1/10 0 all
write/keyword nice_gids/i/1/10 0 all
!
!
!P1(disp):
!--------
define/parameter p1 l ? -
  "        Enter display [def.:<>local|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p1}"
define/local dispname/c/1/30 " " all
if dispflag .eq. "n" then
  goto done
elseif dispflag .eq. "l" then
  dispname = m$symbol("DISPLAY")
else
  dispname = dispflag
endif
!
nice_crea = "y"
write/keyword nice_disp/c/1/30 "{dispflag} "
!
!
!P2(scale):
!---------
define/parameter p2 100. n -
  "        Enter scale factor for display area [def.:100.|>20.]: "
define/local scale/r/1/1 {p2}
if scale .lt. 20. .or. scale .gt. 100. then
  write/out " "
  mess/out 1 {PROID} "Impossible scale factor for display area !"
  mess/out 1 {PROID} "Allowed range: [20.-100.]"
  return 1
endif
!
!
!P3(dots):
!--------
define/parameter p3 -1,-1,-1,-1 ? -
  "        Enter window values [def.:calculate]: "
outputi = m$parse(p3,"win")
if outputi(1) .ne. 4 then
  write/out " "
  mess/out 1 {PROID} "Wrong number of elements in P3 !"
  return 1
endif
define/local rootw/i/1/1 {win01}
define/local rooth/i/1/1 {win02}
define/local wmtitle/i/1/1 {win03}
define/local wmborder/i/1/1 {win04}
!
!
!P4(sess):
!--------
define/parameter p4 {mid$sess(11:12)} ? -
  "        Enter Midas session number to be used for displays [def.:topical]: "
define/local sessint/i/1/1 {p4}
set/format I2
define/local session/c/1/2 "{sessint}"
set/format {FMTDEF}
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Creation of displays ..." :
!
!   "Automatic window positioning" :
  define/local AREAW/r/1/1 0.
  define/local AREAH/r/1/1 0.
  define/local ddimx/i/1/10 0 all
  define/local ddimy/i/1/10 0 all
  define/local doffx/i/1/10 0 all
  define/local doffy/i/1/10 0 all
  define/local gdimx/i/1/10 0 all
  define/local gdimy/i/1/10 0 all
  define/local goffx/i/1/10 0 all
  define/local goffy/i/1/10 0 all
!   "For automatic scaling" :
  define/local screenx/r/1/1 0.
  define/local screeny/r/1/1 0.
!   "For automatic display label positioning" :
  define/local dlabelx/i/1/1 0
  define/local dlabely/i/1/1 0
!
!          CREATION OF DISPLAYS AND LOADING OF ITT AND LUT :
!          ***********************************************
!
initialize/display ? ? ? ? ? {dispname}
!
        mess/out 4 {PROID} "creating displays"
  !
  !      AUTOMATIC WINDOW POSITIONING :
  !
  if rootw .lt. 0 then
    rootw = dazdevr(12)
    rooth = dazdevr(13)
    rgbflag = ididev(7)			! for TrueColour displays
    create/display 0 1,1,0,{rooth} 0 n ? {dispname} >Null 
                   ! "Null" against "LUTsize has been changed to ..."
    ididev(7) = rgbflag			! for TrueColour displays

    $ xwininfo -display {dispname} -name "MIDAS_{session(1:2)} display_0" -
      > getXwin.env
   !
   wmtitle = 0				!default values
   wmborder = 0
   ! 
   define/local inpi/i/1/4 0 all
   open/file getXwin.env read inpi
   if inpi .lt. 1 then
      write/out could not open getXwin.env ...
      goto final_act
   endif
   !
   read_loop:
   read/file {inpi(1)} out_b 60
   if inpi(2) .lt. 0 then
      write/out EOF reached - no info...
      close/file {inpi(1)}
      goto final_act
   else if inpi(2) .lt. 10 then
      goto read_loop
   endif

   inpi(3) = m$index(out_b,"Absolute")
   if inpi(3) .gt. 0 then
      inpi(3) = m$index(out_b,"X:")
      if inpi(3) .gt. 0 then
         inpi(3) = inpi(3) + 2
         wmborder = {out_b({inpi(3)}:)}
         inpi(4) = inpi(4) + 1
      else 
         inpi(3) = m$index(out_b,"Y:")
         if inpi(3) .gt. 0 then
            inpi(3) = inpi(3) + 2
            wmtitle = {out_b({inpi(3)}:)}
            inpi(4) = inpi(4) + 1
         endif
      endif
   endif
   if inpi(4) .lt. 2 goto read_loop
   !
   close/file {inpi(1)}			!we have all the info we want...


  final_act:
    wmtitle = wmtitle-wmborder
    delete/display all
    !
  endif
  !
  ! Dimensions without wmtitle/wmborder, without alpha channel (always y=52);
  ! offsets related to lower-left edge, with wmtitle/wmborder.
  !
  outputr(11) = scale * 0.01		! `scale' is parameter
  AREAW = rootw*outputr(11)
  AREAH = rooth*outputr(11)
  !
  inpi(1) = 2*wmborder
  inpi(2) = wmtitle + inpi(1)
  ! 
  ddimx(1) = m$nint((AREAW-6*wmborder)/3.)  ! e.g. for 3 windows
  ddimy(1) = ddimx(1)
  doffx(1) = 0
  doffy(1) = rooth-ddimy(1)-52 - inpi(2)
  !
  ddimx(2) = ddimx(1)
  ddimy(2) = ddimy(1)
  doffx(2) = ddimx(1)+inpi(1)
  !
  ddimx(3) = ddimx(1)
  ddimy(3) = ddimy(1)
  doffx(3) = doffx(2)+ddimx(2)+inpi(1)
  !
  if dispflag .eq. "l" then
    gdimx(1) = ddimx(1)
    goffx(1) = doffx(2)
    gdimy(1) = m$nint(gdimx(1)*0.713)
    goffy(1) = doffy(1)-gdimy(1)-inpi(2)
    if goffy(1) .lt. 0 then
      gdimy(1) = m$nint((doffy(1)-inpi(2)-(rooth-AREAH))*2./3.)
      goffy(1) = doffy(1)-gdimy(1)-inpi(2)
    endif
  else
    gdimx(1) = m$nint((AREAW-4*wmborder)/2.)
    goffx(1) = 0
    gdimy(1) = m$nint(gdimx(1)*0.713)
    goffy(1) = doffy(1)-gdimy(1)-inpi(2)
    if goffy(1) .lt. 0 then
      gdimy(1) = doffy(1)-inpi(2)
      goffy(1) = doffy(1)-gdimy(1)-inpi(2)
    endif
  endif
  !
  gdimx(2) = gdimx(1)
  if dispflag .eq. "l" then
    goffx(2) = doffx(3)
  else
    goffx(2) = m$nint(AREAW/2.)
  endif
  gdimy(2) = gdimy(1)
  !
  gdimx(3) = gdimx(1)+gdimx(2)+inpi(1)
  goffx(3) = goffx(1)
  gdimy(3) = gdimy(1)
  !
  doffy(1) = doffy(1) + inpi(2) - 2
  doffy(2) = doffy(1)
  doffy(3) = doffy(1)
  goffy(1) = goffy(1) + inpi(2) - 2
  goffy(2) = goffy(1)
  goffy(3) = goffy(1)
  !
rgbflag = ididev(7)    ! for TrueColour displays
create/display 1 {ddimx(1)},{ddimy(1)},{doffx(1)},{doffy(1)} -
  1 y 450000 {dispname} >Null
  ididev(7) = rgbflag  ! for TrueColour displays
  nice_dids(1) = 1
  write/keyword nice_ddim/i/1/2 {ddimx(1)},{ddimy(1)}
  !
  !      FOR AUTOMATIC SCALING :
  !
  screenx = inputi(1)/2.
  screeny = inputi(2)/2.
  write/keyword nice_dscr/i/1/2 {screenx},{screeny}
  !
  !      FOR AUTOMATIC DISPLAY LABEL POSITIONING :
  !
  dlabely = ddimy(1)-24 		! for label size 1
  dlabelx = 10
  write/keyword nice_dlab/c/1/30 "{dlabely},{dlabelx} ? green 1 "
  write/keyword nice_dpos/c/1/30 "{dlabely},{dlabelx} "
  !
rgbflag = ididev(7)    ! for TrueColour displays
create/display 2 {ddimx(2)},{ddimy(2)},{doffx(2)},{doffy(2)} -
  1 y 450000 {dispname} >Null
  ididev(7) = rgbflag  ! for TrueColour displays
  nice_dids(2) = 1
  !
rgbflag = ididev(7)    ! for TrueColour displays
create/display 3 {ddimx(3)},{ddimy(3)},{doffx(3)},{doffy(3)} -
  1 y 450000 {dispname} >Null
  ididev(7) = rgbflag  ! for TrueColour displays
  nice_dids(3) = 1
  !
rgbflag = ididev(7)    ! for TrueColour displays
create/graphic 1 {gdimx(1)},{gdimy(1)},{goffx(1)},{goffy(1)} -
  450000 {dispname} >Null
  ididev(7) = rgbflag  ! for TrueColour displays
  nice_gids(1) = 1
  write/keyword nice_gdim/i/1/2 {gdimx(1)},{gdimy(1)}
  !
  !      FOR AUTOMATIC GRAPHIC LABEL POSITIONING :
  !
  set/format F3.1,
  if dispflag .eq. "l" then
    outputr(1) = -22.2*outputr(11)
    outputr(2) = -8.9*outputr(11)
  else
    outputr(1) = -31.5*outputr(11)
    outputr(2) = -10.2*outputr(11)
  endif
  write/keyword nice_glab/c/1/30 "{outputr(1)},{outputr(2)},mm ? 0.8 1 "
  set/format {FMTDEF}
  !
rgbflag = ididev(7)			! for TrueColour displays
create/graphic 2 {gdimx(2)},{gdimy(2)},{goffx(2)},{goffy(2)} -
  450000 {dispname} >Null
  ididev(7) = rgbflag 			! for TrueColour displays
  nice_gids(2) = 1
  !
rgbflag = ididev(7)			! for TrueColour displays
create/graphic 3 {gdimx(3)},{gdimy(3)},{goffx(3)},{goffy(3)} -
  450000 {dispname} >Null
  ididev(7) = rgbflag			! for TrueColour displays
  nice_gids(3) = 1
wait/secs 1          ! without wait/sec mod/gra can fail on fast machines
modify/graphic icon  ! graphic window 3 only for special purposes
  !
set/graphic default
!
load/itt ramp
load/lut heat
!
assign/display d,1 >Null
assign/graphic g,1 >Null
!
!
done:
return 0
