! $Id: fors_nicedisp.prg,v 1.56 2000/03/02 10:09:41 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_nicedisp.prg,v $
!
! .KEYWORDS   Displays
!
! .PURPOSE    Scale a frame for a display created by fors_nicecrea.prg
!             and optionally cut the counts (around the "mode" value).
! __________
!
! .USAGE      @p fors_nicedisp ima [cut] [rem] [scan] [calc] [msig] [vals] [out]
!
! .INPUT      NICE_DSCR: Global keyword written by fors_nicecrea.prg
!
!             ima (P1): input frame;
!                         default: none
!
!             cut (P2): action flag: <>:calculate cuts, <n>o (only scale);
!                         default: <>
!
!          !T rem (P3): outliers: remove counts exceeding (median -/+ ENTERED
!          !E           FACTOR * med.dev.f.med.) and use newly created 1dim
!          !S            (extracted/)filtered frame "<ima>_f.bdf" for cutting
!          !T            (but store result values in ORIGINAL frame),
!          !V            or <n>o;
!          !E              default: 7.
!          !R.             hint: ignored, if P2=n
!
!             rem (P3): outliers: remove counts exceeding (median -/+ ENTERED
!                       FACTOR * med.dev.f.med.) or <n>o;
!                         default: 7.
!                         hint: ignored, if P2=n
!
!             scan(P4): number of pre- AND overscan columns, separated by a
!                       comma;
!                         default: 16,16
!                         hint: ignored, if P2=n
!
!             calc(P5): calculation flag: <>:calculate statistics of input
!                       frame, or <u>se "STATISTIC" descriptor and if P6=m
!                       (E_X_A_C_T MEDIAN IN DESCRIPTOR SUPPOSED - ELSE WRONG
!                       RESULTS !) calculate only median deviation from the
!                       median;
!                         default: <>
!                         hint: ignored, if P2=n or if P3=n
!
!             msig(P6): for cutting (around the "mode" value): <>:use standard
!                       deviation (from the mean), or use exact <m>edian
!                       deviation (from the median) (takes few seconds more);
!                         default: <>
!                         hint: ignored, if P2=n
!
!             vals(P7): factors (real) for deviations (numbers of "sigmas")
!                       to lower AND higher counts, separated by a comma -
!                       if one factor is negative, concerned factor is applied
!                       at opposite side of the mode value;
!                         default: 1.0,3.0
!                         hint: ignored, if P2=n
!
!             out (P8): flag for result output on terminal: <>:show, <n>o;
!                         default: <>
!
! .OUTPUT     OUTPUTI(1:2) & negated values in desc. DISPLAY_DATA(1:2) :
!               x,y scale factors for "load/image"
!               (for 2dim. images outputi(2)=outputi(1))
!
!             Following values only if P2(cut)!=n :
!
!               OUTPUTR(16) & desc. STATISTIC(16) & desc. LHCUTS(1) :
!                 low cut for "load/image"
!               OUTPUTR(17) & desc. STATISTIC(17) & desc. LHCUTS(2) :
!                 high cut for "load/image"
!
!           !T. If P3(rem)!=n :
!           !V.   OUT_A : name of extr./filt. 1D frame
!
!               OUTPUTD(1) : Percentage of removed outliers
!
!               Byproducts:
!                 OUTPUTR(1) : minimum
!                 OUTPUTR(2) : maximum
!                 ----------------------------------
!                 OUTPUTR(3) : mean
!                 OUTPUTR(4) : std. dev. ("1 sigma")
!                 ----------------------------------
!                 OUTPUTR(5) : 3rd moment
!                 OUTPUTR(6) : 4th moment
!                 OUTPUTR(7) : total intensity
!                 ----------------------------------
!                 OUTPUTR(8) : exact median
!                 OUTPUTR(9) : 1st mode
!                 OUTPUTR(10): # bins
!                 OUTPUTR(11): binsize
!                 OUTPUTR(12): mode
!                 ----------------------------------
!                 OUTPUTR(13): -
!                 OUTPUTR(14): -
!
!                 If P6(msig)=m :
!                   OUTPUTR(15): median deviation ("1 sigma") of the median
!
!                 If P5(calc)!=u :
!                   Desc. STATISTIC(1-12,15) is filled with the same values
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Computed scale [and cuts] are stored in the appropriate
!             descriptors (DISPLAY_DATA [and LHCUTS]) of the input frame;
!             thus it's enough to use
!
!               'load/image {image} [cuts=...]'
!
!             without further parameters afterwards. If the output should
!             get combined with other options however, it can be used for
!             parameters (e.g.):
!
!               'load/image {image} -
!                ? {outputi(1)},avg c,c {outputr(16)},{outputr(17)}'
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/09/14  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref ima cut rem scan calc msig vals out
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "nicedisp"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      NICE_DSCR: Global keyword written by fors_nicecrea.prg"
  w/o " "
  w/o "      ima (P1): input frame;"
  w/o "                  default: none"
  w/o " "
  w/o "      cut (P2): action flag: <>:calculate cuts, <n>o (only scale);"
  w/o "                  default: <>"
  w/o " "
  w/o "      rem (P3): outliers: remove counts exceeding (median -/+ ENTERED"
  w/o "                FACTOR * med.dev.f.med.) or <n>o;"
  w/o "                  default: 7."
  w/o "                  hint: ignored, if P2=n"
  w/o " "
  w/o "      scan(P4): number of pre- AND overscan columns, separated by a"
  w/o "                comma;"
  w/o "                  default: 16,16"
  w/o "                  hint: ignored, if P2=n"
  w/o " "
  w/o "      calc(P5): calculation flag: <>:calculate statistics of input"
  w/o "                frame, or <u>se "STATISTIC" descriptor and if P6=m"
  w/o "                (E_X_A_C_T MEDIAN IN DESCRIPTOR SUPPOSED - ELSE WRONG"
  w/o "                RESULTS !) calculate only median deviation from the"
  w/o "                median;"
  w/o "                  default: <>"
  w/o "                  hint: ignored, if P2=n or P3=n"
  w/o " "
  w/o "      msig(P6): for cutting (around the "mode" value): <>:use standard"
  w/o "                deviation (from the mean), or use exact <m>edian"
  w/o "                deviation (from the median) (takes few seconds more);"
  w/o "                  default: <>"
  w/o "                  hint: ignored, if P2=n"
  w/o " "
  w/o "      vals(P7): factors (real) for deviations (numbers of "sigmas")"
  w/o "                to lower AND higher counts, separated by a comma -"
  w/o "                if one factor is negative, concerned factor is applied"
  w/o "                at opposite side of the mode value;"
  w/o "                  default: 1.0,3.0"
  w/o "                  hint: ignored, if P2=n"
  w/o " "
  w/o "      out (P8): flag for result output on terminal: <>:show, <n>o;"
  w/o "                  default: <>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!Global variable:
!---------------
if m$existk("nice_dscr") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Global keyword "nice_dscr" is missing -"
        mess/out 1 {PROID} "fors_nicecrea.prg must be used before !"
  goto error
else
  if nice_dscr .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Global keyword "nice_dscr" is zero -"
        mess/out 1 {PROID} "fors_nicecrea.prg must be used before !"
    goto error
  endif
endif
!
!
!P1(ima):
!-------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of frame to be scaled/cut: "
define/local ima/c/1/64 "{p1}"
if m$index(ima,".bdf") .eq. 0 ima = ima//".bdf"
!
if m$exist(ima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({ima})"
  goto error
endif
!
!
!P2(cut):
!-------
define/parameter p2 + ? -
  "        Enter action flag [def.:calculate cuts|<n>o (only scale)]: "
define/local cut/c/1/1 "{p2}"
!
!
!P3(rem):
!-------
define/parameter p3 7. ? -
  "        Enter outlier factor [def.:7.] or <n>o: "
if m$tstno(p3) .eq. 1 then
  define/local rem/c/1/1 "+"  ! remove outliers
  define/local outfact/r/1/1 {p3}
  if outfact .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible outlier threshold !"
    goto error
  endif
else
  define/local rem/c/1/1 "n"  ! no removal
endif
!
!
!P4(scan):
!--------
define/parameter p4 16,16 ? -
  "        Enter pre-/overscan values [def.:16,16]: "
outputi = m$parse(p4,"scan")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P4 !"
  goto error
endif
define/local prescan/i/1/1 {scan01}
define/local ovescan/i/1/1 {scan02}
!
if prescan .lt. 0 .or. ovescan .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible pre- or overscan value !"
  goto error
endif
!
!
!P5(calc):
!--------
define/parameter p5 + ? -
  "        Enter calc. flag [def.:calc. stat|<u>se "STATISTIC" desc.]: "
define/local calc/c/1/1 "{p5}"
!
!
!P6(msig):
!--------
define/parameter p6 + ? -
  "        Enter cutting method [def.:std.dev.(mean)|<m>edian dev.(median)]: "
define/local msig/c/1/1 "{p6}"
!
!
!P7(vals):
!--------
define/parameter p7 1.0,3.0 ? -
  "        Enter sigma factors [def.:1.0,3.0]: "
outputi = m$parse(p7,"vals")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P7 !"
  goto error
endif
define/local lowfact/r/1/1 {vals01}
define/local highfact/r/1/1 {vals02}
define/local badfact/i/1/1 0
!
if lowfact .le. 0. .and. highfact .le. 0. badfact = 1
if lowfact .le. 0. .and. m$abs(lowfact) .ge. highfact badfact = 1
if highfact .le. 0. .and. m$abs(highfact) .ge. lowfact badfact = 1
if badfact .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Impossible sigma factor combination !"
  goto error
endif
!
!
!P8(out):
!-------
define/parameter p8 y ? -
  "        Enter flag for result output [def.:show|<n>o]: "
define/local show/c/1/1 "{p8}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  !   "... frame dimensions" :
  define/local naxis/i/1/1 0
  !   "... scale factor" :
  define/local xpix/i/1/1 0
  define/local ypix/i/1/1 0
  define/local scalxin/r/1/1 0.
  define/local scalyin/r/1/1 0.
  define/local scalin/r/1/1 0.
  !   "... outliers" :
  define/local origima/c/1/64 " " all
  define/local omedian/r/1/1 0.
  define/local osigmedmed/r/1/1 0.
  define/local outlow/r/1/1 0.
  define/local outhigh/r/1/1 0.
  !define/local binsize/r/1/1 0.
  !define/local totbin/i/1/1 0
  !   "... cuts" :
  define/local subxstart/i/1/1 0
  define/local subxend/i/1/1 0
  define/local area/c/1/80 " " all
  define/local outpcent/r/1/1 0.
  define/local mode/r/1/1 0.
  define/local stddev/r/1/1 0.
  define/local median/r/1/1 0.
  define/local sigmedmed/r/1/1 0.
  define/local lowcut/r/1/1 0.
  define/local highcut/r/1/1 0.
!
!
!
!          TESTING OF FRAME DIMENSIONS :
!          ***************************
!
        mess/out 4 {PROID} "    testing frame dimensions"
!
naxis = {{ima},NAXIS}
if naxis .lt. 1 .or. naxis .gt. 2 then
        write/out " "
        mess/out 1 {PROID} "Dimension not supported !"
        mess/out 1 {PROID} "(NAXIS = {naxis})"
  goto error
endif
!
!
!          CALCULATION OF SCALE FACTOR :
!          ***************************
!
        mess/out 4 {PROID} "    calculating scale factor"
!
if naxis .eq. 2 then
  !
  xpix = {{ima},NPIX(1)}
  ypix = {{ima},NPIX(2)}
  outputr(1) = xpix
  if xpix .ge. nice_dscr(1) then
    scalxin = -m$nint((outputr(1)-1.)/(nice_dscr(1)*2.)+0.5)
  else
    scalxin = m$nint((nice_dscr(1)*2.)/outputr(1)-0.5)
  endif
  outputr(2) = ypix
  if ypix .ge. nice_dscr(2) then
    scalyin = -m$nint((outputr(2)-1.)/(nice_dscr(2)*2.)+0.5)
  else
    scalyin = m$nint((nice_dscr(2)*2.)/outputr(2)-0.5)
  endif
  if scalxin .le. scalyin scalin = scalxin
  if scalxin .gt. scalyin scalin = scalyin
  !
  if m$existd(ima,"DISPLAY_DATA") .eq. 0 then
    write/descriptor {ima} DISPLAY_DATA/i/1/9 0 all
  endif
  if scalin .le. -1 .or. scalin .ge. 2 then
    {ima},DISPLAY_DATA(1) = -1*{scalin}
    {ima},DISPLAY_DATA(2) = -1*{scalin}
  else  ! 0 not possible, -1 equivalent to 1
    {ima},DISPLAY_DATA(1) = 1
    {ima},DISPLAY_DATA(2) = 1
  endif
  !
elseif naxis .eq. 1 then
  !
  xpix = {{ima},NPIX(1)}
  outputr(1) = xpix
  if xpix .ge. nice_dscr(1) then
    scalxin = -m$nint((outputr(1)-1.)/(nice_dscr(1)*2.)+0.5)
  else
    scalxin = m$nint((nice_dscr(1)*2.)/outputr(1)-0.5)
  endif
  scalyin = nice_dscr(2)/2
  !
  if m$existd(ima,"DISPLAY_DATA") .eq. 0 then
    write/descriptor {ima} DISPLAY_DATA/i/1/9 0 all
  endif
  if scalxin .le. -1 .or. scalxin .ge. 2 then
    {ima},DISPLAY_DATA(1) = -1*{scalxin}
  else  ! 0 not possible, -1 equivalent to 1
    {ima},DISPLAY_DATA(1) = 1
  endif
  {ima},DISPLAY_DATA(2) = -1*{scalyin}
  !
endif
!
!
!          OPTIONAL FILTERING OF OUTLIERS BEFORE CUTTING :
!          *********************************************
!
if cut .ne. "n" origima = ima  ! f. desc. update at the end -
!                              !   "orig" for TEST VERSION
!
if cut .ne. "n" .and. rem .ne. "n" then
  !
        mess/out 4 {PROID} "    calculating outlier thresholds"
  !
  if calc .eq. "u" .and. m$existd(ima,"STATISTIC") .eq. 0 then
        mess/out 2 {PROID} "    Descriptor "STATISTIC" is missing -"
        mess/out 2 {PROID} "    median will be calculated !"
        mess/out 2 {PROID} "    ({ima})"
  endif
  if calc .eq. "u" .and. m$existd(ima,"STATISTIC") .eq. 1 then
    !*******************************************************************
    @p fors_medsigmed {ima} {prescan},{ovescan} {{ima},STATISTIC(8)} n n
    !*******************************************************************
  else
    !************************************************
    @p fors_medsigmed {ima} {prescan},{ovescan} + n n
    !************************************************
  endif
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Calculation of outlier thresholds failed !"
        mess/out 1 {PROID} "({ima})"
      goto error
    endif
  !
  omedian = outputr(8)
  osigmedmed = outputr(15)
  if osigmedmed .eq. 0. then
        mess/out 4 {PROID} "    median sigma is zero -"
        mess/out 4 {PROID} "    using std.dev. instead (out.fact.=3.) !"
    osigmedmed = outputr(4)
    outfact = 3.
  endif
  outlow = omedian-outfact*osigmedmed
  outhigh = omedian+outfact*osigmedmed
  !
  !binsize = 1.
  !
  ! Following lines only because Midas can not handle more than 1024 bins:
  !
  !totbin = m$nint((outhigh-outlow) /binsize +1)
  !if totbin .lt. 10 totbin = 10
  !if totbin .gt. 1022 totbin = 1022  ! 2 excess bins
  !if totbin .eq. 10 .or. totbin .eq. 1022 then
  !  binsize = (outhigh-outlow) / (totbin-1.)
  !endif
  !
  !T ************************************************************
  !  @p fors_remout {ima} {prescan},{ovescan} {outlow},{outhigh} n
  !E ************************************************************
  !  if {q1} .eq. -1 then
  !S   warn = 1
  !  elseif {q1} .eq. 1 then
  !T       write/out " "
  !        mess/out 1 {PROID} "Removal of outliers failed !"
  !V       mess/out 1 {PROID} "({ima})"
  !    goto error
  !E endif
  !
  !R       set/format F8.4
  !        mess/out 4 {PROID} -
  !S         "        removed {outputi(2)} outliers = {outputr(1)}%"
  !        set/format {FMTDEF}
  !I
  !  ima = out_a  ! continue with filtered output frame
  !O outpcent = outputr(1)
  !  prescan = 0  ! already removed in filtered frame
  !N ovescan = 0
  !
endif
!
!
!          COMPUTATION OF CUTS :
!          *******************
!
if cut .ne. "n" then
  !
        mess/out 4 {PROID} "    computing cut values"
  !
  subxstart = prescan+1
  subxend = xpix-ovescan
  !
  if naxis .eq. 2 then
    area = "[@{subxstart},<:@{subxend},>]"
  elseif naxis .eq. 1 then
    area = "[@{subxstart}:@{subxend}]"
  endif
  !
  if rem .eq. "n" then
    statistics/image {ima} {area} ? ? gn >Null
  else
    !statistics/image {ima} {area} {binsize} {outlow},{outhigh} gn >Null
    statistics/image {ima} {area} #1022 {outlow},{outhigh} gn >Null
    inputr(1) = outputi(1)  ! `inputr' because `outputr' needed for output
    if naxis .eq. 2 then
      inputr(2) = (subxend-prescan)*{{ima},NPIX(2)}
    elseif naxis .eq. 1 then
      inputr(2) = (subxend-prescan)
    endif
    outpcent = ( 1. - inputr(1) / inputr(2) )*100.
    outputi(1) = inputr(2)-inputr(1)
        set/format F8.4
        mess/out 4 {PROID} -
          "      removed {outputi(1)} outliers = {outpcent}%"
        set/format {FMTDEF}
  endif
  !
  mode = {{ima},STATISTIC(12)}  ! NOT `outputr' because of above `if'
  stddev = {{ima},STATISTIC(4)}
  !
  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  !
  if msig .eq. "m" then
    !
    median = {{ima},STATISTIC(8)}  ! NOT `outputr' because of above `if'
    !
    !*******************************************************
    @p fors_medsigmed {ima} {prescan},{ovescan} {median} n n
    !*******************************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Cutting of {ima} failed !"
      goto error
    endif
    !
    sigmedmed = outputr(15)  ! median "std.dev." of the median
    if sigmedmed .eq. 0. then
        mess/out 2 {PROID} "    median sigma is zero -"
        mess/out 2 {PROID} "    using std.dev. instead (cuts=-0.5/+3.) !"
      sigmedmed = outputr(4)
      lowfact = 0.5
      highfact = 3.
    endif
    !
  endif
  !
  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  !
  if msig .eq. "m" then
    lowcut = mode - lowfact*sigmedmed
    highcut = mode + highfact*sigmedmed
  else
    lowcut = mode - lowfact*stddev
    highcut = mode + highfact*stddev
  endif
  if lowcut .eq. highcut highcut = highcut*1.00001+0.00001 ! enables loading of
  !                                                        ! total flat frames
  {origima},LHCUTS(1) = lowcut       ! "orig" for TEST VERSION
  {origima},LHCUTS(2) = highcut
  {origima},STATISTIC(16) = lowcut
  {origima},STATISTIC(17) = highcut
  outputr(16) = lowcut
  outputr(17) = highcut
  !
endif
!
!
!          OUTPUT :
!          ******
!
! if rem .ne. "n" out_a = ima  ! TEST VERSION
!
if naxis .eq. 2 then
  outputi(1) = scalin
elseif naxis .eq. 1 then
  outputi(1) = scalxin
  outputi(2) = scalyin
endif
outputd(1) = outpcent
!
if show .ne. "n" .and. mess_level .ge. 3 then
        mess/out 3 {PROID} " "
        set/format T11.4,T11.4
        mess/out 3 {PROID} "Results:"
        mess/out 3 {PROID} "-------"
  if naxis .eq. 2 then
        mess/out 3 {PROID} "Scale factor    : {outputi(1)}"
  elseif naxis .eq. 1 then
        mess/out 3 {PROID} "Scale factor x  : {outputi(1)}"
        mess/out 3 {PROID} "Scl.fct. y (1/2): {outputi(2)}"
  endif
  if cut .ne. "n" then
        mess/out 3 {PROID} "Low cut value   : {outputr(16)}"
        mess/out 3 {PROID} "High cut value  : {outputr(17)}"
    if rem .ne. "n" then
        mess/out 3 {PROID} "Removed outliers: {outputd(1)}%"
    endif
  endif
endif
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------
