! $Id: fors_nicegrap.prg,v 1.16 2000/01/27 22:00:56 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_nicegrap.prg,v $
!
! .KEYWORDS   Displays
!
! .PURPOSE    Determine "world coordinates" for label/graphic AFTER a plot
!             command.
! __________
!
! .USAGE      @p fors_nicegrap
!
! .INPUT      PLRGRAP: world coordinates of plot area and clip factors
!
!             out(P1): flag for result output on terminal: <>:no, <y>es;
!                        default: <>
!
! .OUTPUT     NICE_GLABW/c : for `label/graphic "text" {NICE_GLABW}'
!
!               World coordinates of entire graphics window:
!               OUTPUTR(1): x-start
!               OUTPUTR(2): x-end
!               OUTPUTR(3): y-start
!               OUTPUTR(4): y-end
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    This procedure can only be used for AUTOMATIC scale and offset
!             parameters in `set/graphic' and `plot/*' !
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/27  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "nicegrap"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      PLRGRAP: world coordinates of plot area and clip factors"
  w/o " "
  w/o "      out(P1): flag for result output on terminal: <>:no, <y>es;"
  w/o "                 default: <>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          DEVICE TEST :
!          ***********
!
if mid$sess(7:7) .eq. "-" then
  write/out " "
  mess/out 1 {PROID} "No graphic window exists !"
  goto error
endif
inputi = {mid$sess(7:7)}
if m$exist(m$lower("{mid$plot(1:9)}{inputi}{mid$sess(11:12)}.plt")) .eq. 0 then
  write/out " "
  mess/out 1 {PROID} -
    "No plot file for assigned graph. window {inputi} present !"
  goto error
endif
!
!
!          INPUT :
!          *****
!
outputr(1) = plrgrap(1) + 0.05*(plrgrap(2)-plrgrap(1))
outputr(2) = plrgrap(5) + 0.95*(plrgrap(6)-plrgrap(5))
if m$existk("nice_glabw") .eq. 0 then
  write/keyword nice_glabw/c/1/40 "{outputr(1)},{outputr(2)} ? 0.8 1"  ! prelim.
else
  nice_glabw = "{outputr(1)},{outputr(2)} ? 0.8 1"                     ! prelim.
endif
!
!
define/parameter p1 n ? -
  "        Enter flag for result output [def.:no|<y>es]: "
define/local show/c/1/1 "{p1}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local xws/r/1/1 0.
  define/local xwe/r/1/1 0.
  define/local xcs/r/1/1 0.
  define/local xce/r/1/1 0.
  define/local yws/r/1/1 0.
  define/local ywe/r/1/1 0.
  define/local ycs/r/1/1 0.
  define/local yce/r/1/1 0.
  define/local xw0/r/1/1 0.
  define/local xw1/r/1/1 0.
  define/local yw0/r/1/1 0.
  define/local yw1/r/1/1 0.
  define/local xwl/r/1/1 0.
  define/local ywl/r/1/1 0.
!
!
!
!          CALCULATION OF WORLD COORDINATES
!             OF ENTIRE GRAPHICS WINDOW :
!          ********************************
!
        mess/out 4 {PROID} "    calculating graphic coord."
!
xws = plrgrap(1)
xwe = plrgrap(2)
xcs = plrgrap(21)
xce = plrgrap(22)
yws = plrgrap(5)
ywe = plrgrap(6)
ycs = plrgrap(23)
yce = plrgrap(24)
!
xw0 = xws - xcs/(xce-xcs)*(xwe-xws)
xw1 = xwe + (1-xce)/(xce-xcs)*(xwe-xws)
yw0 = yws - ycs/(yce-ycs)*(ywe-yws)
yw1 = ywe + (1-yce)/(yce-ycs)*(ywe-yws)
!
xwl = xw0 + 0.125*(xws-xw0)
ywl = yw0 + 0.4*(yws-yw0)
!
!
!          OUTPUT :
!          ******
!
nice_glabw = "{xwl},{ywl} ? 0.8 1"
outputr(1) = xw0
outputr(2) = xw1
outputr(3) = yw0
outputr(4) = yw1
!
if show .eq. "y" .and. mess_level .ge. 3 then
        mess/out 3 {PROID} " "
        set/format g12.6,
        mess/out 3 {PROID} "RESULTS ("world" coordinates):"
        mess/out 3 {PROID} "            | x             | y"
        mess/out 3 {PROID} "------------+---------------+--------------"
        mess/out 3 {PROID} "left  lower | {xw0} | {yw0}"
        mess/out 3 {PROID} "right upper | {xw1} | {yw1}"
        mess/out 3 {PROID} " for labels | {xwl} | {ywl}"
endif
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------
