! $Id: fors_nicehist.prg,v 1.57 1999/11/25 12:11:07 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_nicehist.prg,v $
!
! .KEYWORDS   Displays
!
! .PURPOSE    Create histogrammes and set plot axes.
! __________
!
! .USAGE      @p fors_nicehist icat [rem] [scan] [calc] [msig] [vals] -
!                              [shift] [out]
!
! .INPUT      icat (P1): image catalogue with input frames;
!                          default: none
!
!             rem  (P2): outliers: remove counts exceeding (median -/+ ENTERED
!                        FACTOR * med.dev.f.med.) (only for calculation, not
!                        physically), or <n>o (assume filtered _2_dim_ input
!                        frames);
!                          default: 7.
!
!             scan (P3): number of pre- AND overscan columns, separated by a
!                        comma;
!                          default: 16,16
!
!             calc (P4): calculation flag: <>:calculate statistics of input
!                        frame, or <u>se "STATISTIC" descriptor and if P5=m
!                        (E_X_A_C_T MEDIAN IN DESCRIPTOR SUPPOSED - ELSE WRONG
!                        RESULTS !) calculate only median deviation from the
!                        median;
!                          default: <>
!
!             msig (P5): for x-axis ("mean" value in "middle"): <>:use standard
!                        deviation (from the mean), or use exact <m>edian
!                        deviation (from the median) (takes few seconds more);
!                          default: <>
!
!             vals (P6): factors (real) for deviations (numbers of "sigmas")
!                        to lower AND higher counts, separated by a comma -
!                        if one factor is negative, concerned factor is applied
!                        at opposite side of the mean value;
!                          default: 5.,5.
!
!             shift(P7): special mode flag for aligning histogrammes AND scale
!                        factor, separated by a comma - only two frames are
!                        allowed in the input catalogue and the according values
!                        of the second frame are devided by this factor (so that
!                        the 2nd histogramme overlays the 1st one - xaxis re-
!                        flects values of 1st image):
!                        <n>:no special mode, <s>,scalefactor:special mode;
!                          default: <n>
!
!             out  (P8): flag for result output on terminal: <>:show, <n>o;
!                          default: <>
!
! .OUTPUT     Desc.s HISTOGRAM : histogramme (if rem!=n, without outliers)
!
!             Desc.s HIST_BINS(1-5) : #bins, binsize,
!                                     low.exc.bin, high.exc.bin, #exc.bins
!
!             OUT_A : name of output table "<P1>_his.tbl" :
!                     If in column "total_flat" a frame is marked with an "x",
!                     then it is total flat and no histogramme could be created
!                     for this frame.
!                     If at least one frame is total flat, Q1=-1 .
!
!             Q2:  0: all frames are O.K.
!                 >0: total number of skipped frames because of total flatness
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Computed histogrammes are stored in the appropriate frame
!             descriptors (HISTOGRAM and HIST_BINS) and the axes for (over-)
!             plots of the histogrammes (of all input catalogue entries in one
!             single graphics window) are set; thus it's enough to use
!
!               'plot/histogramme #1_{icat}'  (or simply the frame name)
!               'overplot/histogramme #2_{icat}'
!               ...
!
!             without any further parameters afterwards. If the input frames
!             are already outlier-cleaned, the parameter "rem=n" should be used
!             in order to save time. If different colours or line types are
!             preferred, they can be set between the plot commands.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/09/28  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref icat rem scan calc msig vals shift out
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "nicehist"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      icat (P1): image catalogue with input frames;"
  w/o "                   default: none"
  w/o " "
  w/o "      rem  (P2): outliers: remove counts exceeding (median -/+ ENTERED"
  w/o "                 FACTOR * med.dev.f.med.) (only for calculation, not"
  w/o "                 physically), or <n>o (assume filtered _2_dim_ input"
  w/o "                 frames);"
  w/o "                   default: 7."
  w/o " "
  w/o "      scan (P3): number of pre- AND overscan columns, separated by a"
  w/o "                 comma;"
  w/o "                   default: 16,16"
  w/o " "
  w/o "      calc (P5): calculation flag: <>:calculate statistics of input"
  w/o "                 frame, or <u>se "STATISTIC" descriptor and if P5=m"
  w/o "                 (E_X_A_C_T MEDIAN IN DESCRIPTOR SUPPOSED - ELSE WRONG"
  w/o "                 RESULTS !) calculate only median deviation from the"
  w/o "                 median;"
  w/o "                   default: <>"
  w/o " "
  w/o "      msig (P5): for x-axis ("mean" value in "middle"): <>:use standard"
  w/o "                 deviation (from the mean), or use exact <m>edian"
  w/o "                 deviation (from the median) (takes few seconds more);"
  w/o "                   default: <>"
  w/o " "
  w/o "      vals (P6): factors (real) for deviations (numbers of "sigmas")"
  w/o "                 to lower AND higher counts, separated by a comma -"
  w/o "                 if one factor is negative, concerned factor is applied"
  w/o "                 at opposite side of the mean value;"
  w/o "                   default: 5.,5."
  w/o " "
  w/o "      shift(P7): special mode flag for aligning histogrammes AND scale"
  w/o "                 factor, separated by a comma - only two frames are"
  w/o "                 allowed in the input catalogue and the according values"
  w/o "                 of the second frame are devided by this factor (so that"
  w/o "                 the 2nd histogramme overlays the 1st one - xaxis re-"
  w/o "                 flects values of 1st image):"
  w/o "                 <n>:no special mode, <s>,scalefactor:special mode;"
  w/o "                   default: <n>"
  w/o " "
  w/o "      out  (P8): flag for result output on terminal: <>:show, <n>o;"
  w/o "                   default: <>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(icat):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter catalogue with input frames: "
define/local icat/c/1/64 "{p1}"
if m$index(icat,".cat") .eq. 0 icat = icat//".cat"
!
if m$exist(icat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
show/icat {icat} n
define/local totentries/i/1/1 {outputi(1)}
if totentries .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is empty !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
define/local catal/i/1/1 0
inloop:
  store/frame inputc {icat} 1 end_inloop
  if m$exist(inputc) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "At least one catalogue entry is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
    goto error
  endif
  goto inloop
end_inloop:
!
!
!P2(rem):
!-------
define/parameter p2 7. ? -
  "        Enter outlier factor [def.:7.] or <n>o: "
if m$tstno(p2) .eq. 1 then
  define/local rem/c/1/1 "+"  ! remove outliers
  define/local outfact/r/1/1 {p2}
  if outfact .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible outlier threshold !"
    goto error
  endif
else
  define/local rem/c/1/1 "n"  ! no removal
endif
!
!
!P3(scan):
!--------
define/parameter p3 16,16 ? -
  "        Enter pre-/overscan values [def.:16,16]: "
outputi = m$parse(p3,"scan")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P3 !"
  goto error
endif
define/local prescan/i/1/1 {scan01}
define/local ovescan/i/1/1 {scan02}
!
if prescan .lt. 0 .or. ovescan .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible pre- or overscan value !"
  goto error
endif
!
!
!P4(calc):
!--------
define/parameter p4 + ? -
  "        Enter calc. flag [def.:calc. stat|<u>se "STATISTIC" desc.]: "
define/local calc/c/1/1 "{p4}"
!
!
!P5(msig):
!--------
define/parameter p5 + ? -
  "        Enter dev.meth.(mean) [def.:std.dev.(mean)|<m>ed.dev.(med.)]: "
define/local msig/c/1/1 "{p5}"
!
!
!P6(vals):
!--------
define/parameter p6 5.0,5.0 ? -
  "        Enter sigma factors [def.:5.0,5.0]: "
outputi = m$parse(p6,"vals")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P6 !"
  goto error
endif
define/local lowfact/r/1/1 {vals01}
define/local highfact/r/1/1 {vals02}
define/local badfact/i/1/1 0
!
if lowfact .le. 0. .and. highfact .le. 0. badfact = 1
if lowfact .le. 0. .and. m$abs(lowfact) .ge. highfact badfact = 1
if highfact .le. 0. .and. m$abs(highfact) .ge. lowfact badfact = 1
if badfact .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Impossible sigma factor combination !"
  goto error
endif
!
!
!P7(shift):
!---------
define/parameter p7 + ? -
  "        Enter special mode flag [def.:<n>o|<s>,scalefactor]: "
outputi = m$parse(p7,"shift")
if outputi(1) .ne. 1 .and. outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P7 !"
  goto error
endif
define/local align/c/1/1 "{shift01}"
if outputi(1) .eq. 2 then
  define/local scalefact/r/1/1 {shift02}
endif
!
if align .eq. "s" .and. totentries .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "For special mode (aligning) exactly two"
        mess/out 1 {PROID} "entries must be in input catalogue !"
  goto error
endif
!
if align .eq. "s" then
  if scalefact .le. 0 then
        write/out " "
        mess/out 1 {PROID} "Scalefactor out of range !"
    goto error
  endif
endif
!
!
!P8(out):
!-------
define/parameter p8 y ? -
  "        Enter flag for result output [def.:show|<n>o]: "
define/local show/c/1/1 "{p8}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "help-table" :
  define/local char/i/1/1 0
  define/local histtable/c/1/64 " " all
!
!     "L o o p   1   w i t h   f r a m e s" :
  define/local topicima/c/1/64 " " all
  define/local lopcount/i/1/1 0
  define/local special/i/1/1 0
  !
  !   "outlier limits" :
  define/local omedian/r/1/{totentries} 0. all
  define/local osigmedmed/r/1/1 0.
  define/local outlow/r/1/1 0.
  define/local outhigh/r/1/1 0.
  !
  !   "min/max" :
  define/local min/r/1/{totentries} 0. all
  define/local max/r/1/{totentries} 0. all
  !
  !   "total number of bins" :
  define/local subxstart/i/1/1 0
  define/local subxend/i/1/1 0
  define/local tmin/r/1/{totentries} 0. all
  define/local tmax/r/1/{totentries} 0. all
  define/local binsize/r/1/{totentries} 0. all
  define/local totbin/i/1/{totentries} 0 all
!
!     "Calculations for alignment" :
  define/local tdiff/r/1/1 0.
!
!     "L o o p   2   w i t h   f r a m e s" :
  !   "histogramme" :
  define/local skip/i/1/1 0
  define/local histoloop/i/1/1 0
  define/local allbins/i/1/1 0
  define/local histogram/i/1/1024 -1 all
  !
  !   "x-axis ... single" :
  define/local median/r/1/1 0.
  define/local stddev/r/1/1 0.
  define/local sigmedmed/r/1/1 0.
  define/local xlow/r/1/1 0.
  define/local xhigh/r/1/1 0.
  define/local range/r/1/1 0.
  !
  !   "y-axis ... single" :
  define/local yhigh/r/1/1 0.
!
!     "common x-axes" :
  define/local xlowc/r/1/1 0.
  define/local xhighc/r/1/1 0.
  define/local xtickc/r/1/1 0.
!
!     "common y-axes" :
  define/local ylowc/r/1/1 0.
  define/local yhighc/r/1/1 0.
!
!
!
!          CREATING HELP-TABLE :
!          *******************
!
        mess/out 4 {PROID} "  preparing calculations"
!
  char = m$index(icat,".cat")-1
  histtable = icat(1:{char})//"_his.tbl"
create/table {histtable} 5 totentries null
!
create/column {histtable} :FRAME " " A55 C*64
create/column {histtable} :total_flat " " A11 C*10
create/column {histtable} :xlow "ADU" E13.6 R*4
create/column {histtable} :xhigh "ADU" E13.6 R*4
create/column {histtable} :yhigh " " E13.6 R*4
!
!
!
!          L O O P   1   W I T H   F R A M E S   ( B I N S ) :
!          * * * * * * * * * * * * * * * * * * * * * * * * *
!
!
        mess/out 3 {PROID} "  computing histo"
        !if mess_level .eq. 3 then
        !  mess/out 3 {PROID} "  calculating number of bins"
        !endif
!
catal = 0
catloop1:
  store/frame topicima {icat} 1 end_catloop1
  lopcount = lopcount+1
        mess/out 4 {PROID} -
          "  histo {lopcount} / {totentries} : {topicima}"
  {histtable},:FRAME,{lopcount} = "{topicima}"
  special = 0
  if align .eq. "s" .and. lopcount .eq. 2 special = 1
  !
  !
  !        DETERMINATION OF OUTLIER LIMITS :
  !        *******************************
  !
  if rem .ne. "n" then
    !
        mess/out 4 {PROID} "    calculating outlier limits"
    !
    if calc .eq. "u" .and. m$existd(topicima,"STATISTIC") .eq. 0 then
        mess/out 2 {PROID} "    Descriptor "STATISTIC" is missing -"
        mess/out 2 {PROID} "    median will be calculated !"
        mess/out 2 {PROID} "    ({topicima})"
    endif
    if calc .eq. "u" .and. m$existd(topicima,"STATISTIC") .eq. 1 then
      !*************************************************
      @p fors_medsigmed {topicima} {prescan},{ovescan} -
        {{topicima},STATISTIC(8)} n n
      !*************************************************
    else
      !*****************************************************
      @p fors_medsigmed {topicima} {prescan},{ovescan} + n n
      !*****************************************************
    endif
      if {q1} .eq. -1 then
        warn = 1
      elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Calculation of outlier limits failed !"
        mess/out 1 {PROID} "({topicima})"
        goto error
      endif
    !
    omedian({lopcount}) = outputr(8)
    if special .eq. 0 then
      osigmedmed = outputr(15)
      if osigmedmed .eq. 0. then
        mess/out 4 {PROID} "    median sigma is zero -"
        mess/out 4 {PROID} "    using std.dev. instead (out.fact.=3.) !"
        osigmedmed = outputr(4)
        outfact = 3.
      endif
      outlow = omedian({lopcount})-outfact*osigmedmed
      outhigh = omedian({lopcount})+outfact*osigmedmed
    endif
    !
  endif
  !
  !
  !        CHECK OF MIN/MAX :
  !        ****************
  !
  if calc .eq. "u" .and. m$existd(topicima,"STATISTIC") .eq. 0 then
        mess/out 2 {PROID} "    Descriptor "STATISTIC" is missing -"
        mess/out 2 {PROID} "    min./max. will be calculated !"
        mess/out 2 {PROID} "    ({topicima})"
  endif
  subxstart = prescan+1
  subxend = {{topicima},NPIX(1)}-ovescan
  if calc .ne. "u" .or. m$existd(topicima,"STATISTIC") .eq. 0 then
    statistics/image {topicima} [@{subxstart},<:@{subxend},>] ? ? mn >Null
  endif
  min({lopcount}) = {{topicima},STATISTIC(1)}  ! NOT `outputr' ->`if' above
  max({lopcount}) = {{topicima},STATISTIC(2)}
  !
  !###if outlow .lt. min({lopcount}) outlow = min({lopcount})
  !###if outhigh .gt. max({lopcount}) outhigh = max({lopcount})
  !
  !
  !        CALCULATION OF TOTAL NUMBER OF BINS :
  !        ***********************************
  !
  if special .eq. 0 then
    !
        mess/out 4 {PROID} "    calculating total number of bins"
    !
    if rem .ne. "n" then
      tmin({lopcount}) = outlow
      tmax({lopcount}) = outhigh
    else
      tmin({lopcount}) = min({lopcount})
      tmax({lopcount}) = max({lopcount})
    endif
    !
    binsize({lopcount}) = 1.
    !
    ! Following lines only because Midas can not handle more than 1024 bins;
    ! tmin,tmax are identical to outlow,outhigh if P2.ne."n" ;
    ! totbin is without the two excess bins from loop 2 below :
    !
    totbin({lopcount}) -
      = m$nint((tmax({lopcount})-tmin({lopcount})) /binsize({lopcount}) +1)
    if totbin({lopcount}) .lt. 10 totbin({lopcount}) = 10
                               ! >=2 needed, 10 better for resolution
    if totbin({lopcount}) .gt. 1022 totbin({lopcount}) = 1022  ! 2 excess bins
    if totbin({lopcount}) .eq. 10 .or. totbin({lopcount}) .eq. 1022 then
      binsize({lopcount}) -
        = (tmax({lopcount})-tmin({lopcount})) / (totbin({lopcount})-1.)
    endif
    !
  endif
  !
  !
  goto catloop1
end_catloop1:
!
!
!          E N D   O F   L O O P   1   W I T H   F R A M E S
!          * * * * * * * * * * * * * * * * * * * * * * * * *
!
!
!
!          CALCULATIONS FOR ALIGNMENT (SPECIAL MODE) :
!          *****************************************
!
if align .eq. "s" then
  !
        mess/out 4 {PROID} "  calculating alignment"
  !
  binsize(2) = binsize(1)
  totbin(2) = totbin(1)
  tdiff = binsize(1) * scalefact * (totbin(2)-1)
  tmin(2) = omedian(2) - tdiff/2.
  tmax(2) = omedian(2) + tdiff/2.
  !
  !###if tmin(2) .lt. min(2) tmin(2) = min(2)
  !###if tmax(2) .gt. max(2) tmax(2) = max(2)
  !
endif
!
!
!
!          L O O P   2   W I T H   F R A M E S   ( H I S T O S ) :
!          * * * * * * * * * * * * * * * * * * * * * * * * * * *
!
!
        !if mess_level .eq. 3 then
        !  mess/out 3 {PROID} "  creating histo"
        !endif
!
catal = 0
lopcount = 0
catloop2:
  store/frame topicima {icat} 1 end_catloop2
  lopcount = lopcount+1
        mess/out 4 {PROID} -
          "  histo {lopcount} / {totentries} : {topicima}"
  !
  !
  !        CREATION OF HISTOGRAMME :
  !        ***********************
  !
        mess/out 4 {PROID} "    creating histogramme"
  !
  !*********************************************************************
  statistics/image {topicima} [@{subxstart},<:@{subxend},>] -
    {binsize({lopcount})} {tmin({lopcount})},{tmax({lopcount})} gn >Null
  !*********************************************************************
  !
  if m$existd(topicima,"HIST_BINS") .eq. 0 then  ! MIDAS error (statis/imag) !
        mess/out 2 {PROID} "    Frame absolutely flat - no histo possible !"
        mess/out 2 {PROID} "    ({topicima})"
    warn = 1
    !
    skip = skip+1
    {histtable},:total_flat,{lopcount} = "x"
    goto catloop2
  endif
  !
  ! SCALING FOR ALIGNMENT OF TWO HISTOGRAMMES:
  ! (ONLY TWO ENTRIES ARE IN INPUT CATALOGUE)
  if align .eq. "s" .and. lopcount .eq. 2 then
    !
    ! Do NOT scale hist_bins(1) (#bins) !
    {topicima},hist_bins(2) = {{topicima},hist_bins(2)} /scalefact  ! binsize
    {topicima},hist_bins(3) = {{topicima},hist_bins(3)} /scalefact  ! low excess
    {topicima},hist_bins(4) = {{topicima},hist_bins(4)} /scalefact  ! high exc.
    !    
    copy/dk {topicima} HISTOGRAM histogram
    show/desc {topicima} HISTOGRAM h
    allbins = outputi(3)
    do histoloop = 1 allbins
      if histogram({histoloop}) .ne. -1 then
        histogram({histoloop}) = histogram({histoloop}) * scalefact
      endif
    enddo
    copy/kd histogram {topicima} HISTOGRAM
    !
  endif
  !
  !
  !        CALCULATION OF X-AXIS RANGE (SINGLE FRAME) :
  !        ******************************************
  !
        mess/out 4 {PROID} "    calculating range of single x-axis"
  !
  median = outputr(8)
  stddev = outputr(4)
  !
  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  if msig .eq. "m" then
    !************************************************************
    @p fors_medsigmed {topicima} {prescan},{ovescan} {median} n n
    !************************************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Creating of histogramme failed !"
        mess/out 1 {PROID} "({topicima})"
      goto error
    endif
    sigmedmed = outputr(15)  ! median "std.dev." of the median
    if sigmedmed .eq. 0. then
        mess/out 2 {PROID} "    median sigma is zero -"
        mess/out 2 {PROID} "    using std.dev. instead (cuts=-0.5/+3.) !"
      sigmedmed = outputr(4)
      lowfact = 0.5
      highfact = 3.
    endif
  endif
  !- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  !
  if msig .eq. "m" then
    xlow = median - lowfact*sigmedmed
    xhigh = median + highfact*sigmedmed
  else
    xlow = median - lowfact*stddev
    xhigh = median + highfact*stddev
  endif
  !
  ! SCALING FOR ALIGNMENT OF TWO HISTOGRAMMES:
  ! (ONLY TWO ENTRIES ARE IN INPUT CATALOGUE)
  if align .eq. "s" .and. lopcount .eq. 2 then
    xlow = xlow/scalefact
    xhigh = xhigh/scalefact
  endif
  !
  !###if xlow .lt. min({lopcount}) xlow = min({lopcount})
  !###if xhigh .gt. max({lopcount}) xhigh = max({lopcount})
  !###range = xhigh-xlow
  !###xlow = xlow-0.05*range
  !###xhigh = xhigh+0.05*range
  !
  {histtable},:xlow,{lopcount} = xlow
  {histtable},:xhigh,{lopcount} = xhigh
  !
  !
  !        CALCULATION OF Y-AXIS END (SINGLE FRAME) :
  !        ****************************************
  !
        mess/out 4 {PROID} "    calculating range of single y-axis"
  !
  copy/dima {topicima} HISTOGRAM tmp.bdf
  find/minmax tmp.bdf >Null
  yhigh = outputr(2)
  {histtable},:yhigh,{lopcount} = yhigh
  !
  !
  goto catloop2
end_catloop2:
!
!
-delete tmp.bdf
!
!
!          E N D   O F   L O O P   2   W I T H   F R A M E S
!          * * * * * * * * * * * * * * * * * * * * * * * * *
!
!
!
        !if mess_level .eq. 3 then
        !  mess/out 3 {PROID} "  setting axes"
        !endif
!
!          COMPUTATION AND SETTING OF COMMON X-AXES RANGE (ALL FRAMES) :
!          ***********************************************************
!
        mess/out 4 {PROID} "  calculating common range of all x-axes"
!
statistics/table {histtable} :xlow >Null
xlowc = outputr(1)
statistics/table {histtable} :xhigh >Null
xhighc = outputr(2)
xtickc = (xhighc-xlowc)/5.
set/graphic xaxis={xlowc},{xhighc},{xtickc} >Null  ! ">Null" if all are flat
!
!
!          COMPUTATION AND SETTING OF COMMON Y-AXES RANGE (ALL FRAMES) :
!          ***********************************************************
!
        mess/out 4 {PROID} "  calculating common range of all y-axes"
!
statistics/table {histtable} :yhigh >Null
yhighc = 1.05*outputr(2)
ylowc = -0.05*yhighc
set/graphic yaxis={ylowc},{yhighc} >Null  ! ">Null" if all are flat
!
set/graphic xformat=F2.1
!
!
!          OUTPUT :
!          ******
!
out_a = histtable
!
if show .ne. "n" .and. mess_level .ge. 3 then
        mess/out 4 {PROID} " "
        set/format T11.2,
        mess/out 3 {PROID} "Results:"
        mess/out 3 {PROID} "-------"
        mess/out 3 {PROID} "Low  x-axis value: {xlowc}"
        mess/out 3 {PROID} "High x-axis value: {xhighc}"
        mess/out 3 {PROID} "High y-axis value: {yhighc}"
  if skip .gt. 0 then
        mess/out 3 {PROID} "Skipped frames   : {skip}"
  endif
endif
!
if skip .gt. 0 then
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} "> Table w. skipped frames is <{histtable}>. <"
endif
!
!
set/format
!
!
if warn .eq. 0 then
  return 0 {skip}
else
  return -1 {skip}
endif
!
error:
  set/format
  return 1 -99
!
!
!-------------------------------------------------------------------------------
