! $Id: fors_normbias.prg,v 1.3 2000/03/06 14:31:36 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_normbias.prg,v $
!
!.KEYWORDS    Basic CCD reduction, master bias normalization
!
!.USAGE       normalize/mbias frame nbias method
!
!.INPUT       frame       (P1)   Master bias frame to be normalized.
!             method      (P3)   Method used for normalization: MEAN, MEDIAN,
!                                OVERSCAN
!
!.OUTPUT      nbias       (P2)   Normalized master bias frame.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is 
!             updated with the exit status.
!
!.PURPOSE     Normalize the input master bias frame for each individual
!             readout port.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/08/16:  Creation
!
!------------------------------------------------------------------------------

crossref  frame method

define/param p1 ?   c "Enter name of input frame:"
define/param p2 ?   c "Enter normalized bias template:"
define/param p3 MEA c "Enter name of normalization method:"
define/maxpar 3


! Procedure identification
define/local procID/c/1/80      "NormMbias"


! Local variables
define/local i/i/1/1       0
define/local j/i/1/1       0
define/local n/i/1/1       0
define/local index/i/1/1   0
define/local nport/i/1/1   0
define/local x1/r/1/1      0.
define/local y1/r/1/1      0.
define/local x2/r/1/1      0.
define/local y2/r/1/1      0.
define/local xsize/r/1/1   0.
define/local ysize/r/1/1   0.
define/local imsec/r/1/4   0.    all
define/local scale/r/1/1   0.
define/local mtd/c/1/3     " "
define/local frm/c/1/100   {p1}
define/local nbias/c/1/100 {p2}



! Process parameters
mtd = m$upper(p3(1:3))
if mtd .ne. "MEA" .and. mtd .ne. "MED" then
  if mtd .ne. "OVE" then
    mess/out 1 {procID} "Unsupported normalization method >{mtd}<!"
    return 1
  endif
endif

if m$index("{{frm},{FORS_IDSTRNG}}",MBIAS_ID) .le. 0 then
  mess/out 1 {procID} "Input frame is not a >{MBIAS_ID}<!"
  return 1
endif

if mtd .eq. "OVE" .and. m$existd(frm,h_prqcobi(1)) .ne. 1 then
  mess/out 1 {procID} "Overscan bias values missing in >{frm}<!"
  return 1
endif

! Convert input frame to REAL*4 if necessary
info/image {frm} >Null
if mid$info(1) .ne. 10 .and. mid$info(1) .ne. 18 then
  copy/ii {frm} {nbias} r4
else
  -copy {frm} {nbias}
endif


! Get number of readout ports
nport = m$value({nbias},{h_nout})
if nport .ne. 1 .and. nport .ne. 4 then
  mess/out 1 {procID} "Invalid readout port setup!"
  return 1
endif


! Get valid image area
image/section {nbias} {FORS_CCD_RDDIR}
if {q1} .eq. 0 then
  imsec(1) = outputd(1)
  imsec(2) = outputd(2)
  imsec(3) = outputd(3)
  imsec(4) = outputd(4)
else
  mess/out 1 {procID} "Cannot determine image layout from >{frm}<!"
  return 1
endif


! Calculate number of valid pixels along the x and y axis of all ports
xsize = imsec(3) - imsec(1) + 1.
ysize = imsec(4) - imsec(2) + 1.

if nport .eq. 4 then
  xsize = xsize / 2.
  ysize = ysize / 2.
endif


! Normalize master bias
if nport .eq. 4 then

! Note that the following method assumes that the ports are located at the
! corners of a square, with the port numbering starting at the lower left
! corner.
  n = nport / 2

  do i = 1 n 1
    y1 = imsec(2) + (i - 1) * ysize
    y2 = y1 + ysize - 1.

    do j = 1 n 1  
      x1 = imsec(1) + (j - 1) * xsize
      x2 = x1 + xsize - 1.
      
      if mtd .eq. "MEA" then
        statistics/image {nbias} [{x1},{y1}:{x2},{y2}] option=SN
        scale = outputr(3)
      else if mtd .eq. "MED" then
        statistics/image {nbias} [{x1},{y1}:{x2},{y2}] option=XN
        scale = outputr(8)
      else
        index = j + 2*(i-1)
        set/format i1
        scale = m$value({nbias},{h_prqcobi({index})})
        set/format
      endif

      compute/image {nbias}[{x1},{y1}:{x2},{y2}] = -
                    {nbias}[{x1},{y1}:{x2},{y2}] / {scale}
    enddo
  enddo
else
  x1 = imsec(1)
  y1 = imsec(2)
  x2 = imsec(3)
  y2 = imsec(4)

  if mtd .eq. "MEA" then
    statistics/image {nbias} [{x1},{y1}:{x2},{y2}] option=SN
    scale = outputr(3)
  else if mtd .eq. "MED" then
    statistics/image {nbias} [{x1},{y1}:{x2},{y2}] option=XN
    scale = outputr(8)
  else
    scale = m$value({nbias},{h_proqcobia})
  endif

  compute/image {nbias}[{x1},{y1}:{x2},{y2}] = -
                {nbias}[{x1},{y1}:{x2},{y2}] / {scale}
endif

return 0
