
! $Id: fors_oscbias.prg,v 1.5 2000/01/03 09:15:55 jreetz Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_oscbias.prg,v $
!
!.KEYWORDS    Basic CCD reduction, bias computation using frame overscan
!
!.USAGE       compute/ovbias  frame method
!
!.INPUT       frame       (P1)   Input frame for which the bias is determined
!                                from the overscan area(s).
!             method      (P2)   Method used for bias computation: MEDIAN or
!                                FIT. For method fit the fit accuracy, i.e.
!                                the difference of successive chi^2 values,
!                                has to be given in the form FIT,<value>.
!
!.OUTPUT      none.
!
!.RETURNS     The recipe returns 0 on success, -1, if the frame does not contain
!             pre-/overscan areas, otherwise 1. Keyword Q1 is 
!             updated with the exit status. The computed bias values are
!             written to the frame descriptor BIAS_MEAN.
!
!.PURPOSE     Compute the bias value from the overscan area of the input frame.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/08/12:  Creation
!
!------------------------------------------------------------------------------

crossref  frame method

define/param p1 ?   c "Enter name of input frame:"
define/param p2 MED c "Enter name of computational method:"
define/maxpar 2


! Procedure identification
define/local procID/c/1/80      "CompOvBias"


! Local variables
define/local cflag/i/1/1             0
define/local i/i/1/1                 0
define/local j/i/1/1                 0
define/local k/i/1/1                 0
define/local xskip/i/1/1             0
define/local yskip/i/1/1             0
define/local ovnpix/i/1/2            0     all
define/local ovsec/r/1/4             0     all
define/local imsec/r/1/4             0     all
define/local bs/r/1/2                0.    all
define/local ns/r/1/2                0.    all
define/local bias/r/1/{FORS_MAXOUT}  -1.   all
define/local bias_mean/r/1/1         0.
define/local noise/r/1/{FORS_MAXOUT} -1.   all
define/local noise_max/r/1/1         0.
define/local ifrm/c/1/100            {p1}
define/local mtd/c/1/100             {p2}


! Change image coordinate system to pixels if not yet done
swap/units {ifrm} pixel
if {q1} .eq. 1 then
  cflag = 1
endif


! Get image specifications
image/section {ifrm} {FORS_CCD_RDDIR}
if {q1} .eq. 0 then
  if FORS_CCD_RDDIR .eq. "X" then
    ovnpix(1) = outputi(5)
    ovnpix(2) = outputi(6)
  else
    ovnpix(1) = outputi(7)
    ovnpix(2) = outputi(8)
  endif

  imsec(1) = outputd(1)
  imsec(2) = outputd(2)
  imsec(3) = outputd(3)
  imsec(4) = outputd(4)

  if ovnpix(1) .eq. 0 .or. ovnpix(2) .eq. 0 then
    mess/out 2 {procID} "No overscan areas found in >{ifrm}<!"
    return -1
  endif
else
  mess/out 1 {procID} "Cannot determine image layout from >{ifrm}<!"
  return 1
endif


! Compute bias for all active readout ports
if FORS_NOUT .eq. 1 then
  if FORS_CCD_RDDIR .eq. "Y" then
    ovsec(1) = imsec(1)
    ovsec(2) = imsec(2) - ovnpix(1)
    ovsec(3) = imsec(3)
    yskip = (imsec(4) - imsec(2) + 1) + ovnpix(1)

    do i = 1 2 1
      ovsec(2) = ovsec(2) + (i - 1) * yskip
      ovsec(4) = ovsec(2) + ovnpix({i}) - 1
      model/overscan {ifrm} {ovsec(1)},{ovsec(2)},{ovsec(3)},{ovsec(4)} -
                     {FORS_CCD_RDDIR} {mtd}
      bs({i}) = outputr(1)
      ns({i}) = outputr(2)
    enddo
  else
    ovsec(1) = imsec(1) - ovnpix(1)
    ovsec(2) = imsec(2)
    ovsec(4) = imsec(4)
    xskip = (imsec(3) - imsec(1) + 1) + ovnpix(1)

    do i = 1 2 1
      ovsec(1) = ovsec(1) + (i - 1) * xskip
      ovsec(3) = ovsec(1) + ovnpix({i}) - 1
      model/overscan {ifrm} {ovsec(1)},{ovsec(2)},{ovsec(3)},{ovsec(4)} -
                     {FORS_CCD_RDDIR} {mtd}
      bs({i}) = outputr(1)
      ns({i}) = outputr(2)
    enddo
  endif

! For 1-port readout the bias is the arithmetic mean of both
! overscan regions, for the noise the maximum of both values is computed
  bias(1) = 0.5 * (bs(1) + bs(2))
  bias_mean = bias(1)
  @p fors_max {ns(1)},{ns(2)}
  noise(1) = {Q1}
  noise_max = noise(1)

elseif FORS_NOUT .eq. 4 then

! Note that the handling of the different ports below assumes that
! the readout ports are located at the corners of a square. Additionally
! port 1 is assumed to be at the lower left corner and port 4 at the
! upper right corner of the CCD.

  if FORS_CCD_RDDIR .eq. "Y" then
    xskip = (imsec(3) - imsec(1) + 1) / 2
    yskip = (imsec(4) - imsec(2) + 1) + ovnpix(1)

    do i = 1 2 1
      ovsec(1) = imsec(1) + (i - 1) * xskip
      ovsec(2) = imsec(2) - ovnpix(1)
      ovsec(3) = ovsec(1) + xskip - 1

      do j = 1 2 1
        ovsec(2) = ovsec(2) + (j - 1) * yskip
        ovsec(4) = ovsec(2) + ovnpix({j}) - 1
        model/overscan {ifrm} {ovsec(1)},{ovsec(2)},{ovsec(3)},{ovsec(4)} -
                       {FORS_CCD_RDDIR} {mtd}
        k = 2 * (i - 1) + j
        bias({k}) = outputr(1)
        noise({k}) = outputr(2)
      enddo
    enddo
  else
    xskip = (imsec(3) - imsec(1) + 1) + ovnpix(1)
    yskip = (imsec(4) - imsec(2) + 1) / 2

    do i = 1 2 1
      ovsec(1) = imsec(1) - ovnpix(1)
      ovsec(2) = imsec(2) + (i - 1) * yskip
      ovsec(4) = ovsec(2) + yskip - 1

      do j = 1 2 1
        ovsec(1) = ovsec(1) + (j - 1) * xskip
        ovsec(3) = ovsec(1) + ovnpix({j}) - 1
        model/overscan {ifrm} {ovsec(1)},{ovsec(2)},{ovsec(3)},{ovsec(4)} -
                       {FORS_CCD_RDDIR} {mtd}
        k = 2 * (i - 1) + j
        bias({k}) = outputr(1)
        noise({k}) = outputr(2)
      enddo
    enddo
  endif

  bias_mean = ( bias(1) + bias(2) + bias(3) + bias(4) ) / 4.
  @p fors_max {noise(1)},{noise(2)},{noise(3)},{noise(4)}
  noise_max = {q1}
else
  mess/out 1 {procID} "Invalid readout port setup encountered in >{ifrm}<!"
  return 1
endif


! Write overscan bias to frame header
copy/kd bias_mean {ifrm} {h_proqcobia}/r/1/1
copy/kd noise_max {ifrm} {h_proqconoi}/r/1/1

write/dhelp {ifrm} {h_proqcobia} "{i_proqcobia}"
write/dhelp {ifrm} {h_proqconoi} "{i_proqconoi}"

set/format I1
do i = 1 {FORS_NOUT} 1
  copy/kd bias/r/{i}/1 {ifrm} {h_prqcobi({i})}/r/1/1
  copy/kd noise/r/{i}/1 {ifrm} {h_prqcono({i})}/r/1/1

  write/dhelp {ifrm} {h_prqcobi({i})} "{i_prqcobi}"
  write/dhelp {ifrm} {h_prqcono({i})} "{i_prqcono}"
enddo

! Restore world coordinates if necessary
if cflag .eq. 1 then
  swap/units {ifrm} world
endif

return 0
