! $Id: fors_oscframes.prg,v 1.2 1999/09/16 10:46:07 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_oscframes.prg,v $
!
!.KEYWORDS    Basic CCD reduction, compute overscan bias for a list of frames
!
!.USAGE       overscan/frames  catalog class mtd
!
!.INPUT       catalog     (P1)   Input catalog of frames to be processed.
!             class       (P2)   Classification of frames to be selected
!                                from catalog for bias computation.
!             mtd         (P3)   Method of bias computation: median or fit.
!                                In the latter case the fit accuracy has to
!                                be given like this: fit,<value>. The accuracy
!                                is the difference of successive chi^2 values.
!
!.OUTPUT      none.
!
!.RETURNS     The recipe returns 0 on success, -1, if one of the frame does not 
!             contain pre-/overscan areas, otherwise 1. Keyword Q1 is 
!             updated with the exit status. The computed bias values for
!             each processed frame are written to the frame descriptor 
!             BIAS_MEAN.
!
!.PURPOSE     Compute the bias values from the overscan area for all frames
!             in the input catalog, matching the given frame classification.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/08/12:  Creation
!
!------------------------------------------------------------------------------

crossref  catalog class method

define/param p1 ?   c "Enter name of input catalog:"
define/param p2 ?   c "Enter frame classification:"
define/param p3 MED c "Enter computational method:"
define/maxpar 3


! Procedure identification
define/local procID/c/1/80      "OscanFrames"


! Local variables
define/local pos/i/1/1          0
define/local catal/i/1/1        0
define/local cat_entry/c/1/100  " "  all
define/local cat_name/c/1/100   {p1}
define/local frm_class/c/1/100  {p2}
define/local mtd/c/1/100        {p3}



! Check catalog name
if m$indexb(cat_name,".cat") .le. 0 then
  cat_name = cat_name // ".cat"
endif


! Loop over catalog entries

cat_loop:

store/frame cat_entry {cat_name} 1 exit_loop
if m$index("{{cat_entry},{FORS_IDSTRNG}}",frm_class) .eq. 1 then
  compute/ovbias {cat_entry} {mtd}
  if {q1} .eq. -1 then 
    return -1
  else if {q1} .ne. 0 then
    mess/out 1 {procID} -
      "Bias computation using overscan failed for >{cat_entry}<!"
    return 1
  endif
endif

goto cat_loop
 

exit_loop:

return 0
