! $Id: fors_over_mask.prg,v 1.21 2000/02/16 16:22:23 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_over_mask.prg,v $
!
! .KEYWORDS   Pixel values
!
! .PURPOSE    Handle in- and output for "fors_ovexpomap.c" and invoke it;
!             it creates an image mask with overexposed pixels.
! __________
!
! .USAGE      @p fors_over_mask ima [thresh] [maxper] [out]
!
! .INPUT      ima    (P1): input frame;
!                            default: none
!
!             thresh (P2): integer threshold (ADU) for finding overexposed
!                          pixels and integer flag for creation of mask,
!                          separated by a comma;
!                            default: 65534,65535
!
!             maxpix (P3): minimal forbidden number of overexposed pixels -
!                          "1" means, that a frame with 1 overexp.pix. will
!                          be declared bad;
!                            default: 1
!
!             out    (P4): flag for result output on terminal: <>:show, <n>o;
!                            default: <>
!
! .OUTPUT     "<P1>_oem.bdf":  mask with overexposed pixels
!
!             OUTPUTI(1): total number of overexposed pixels
!
!             OUT_A     : name of mask image
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    -
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/12/07  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!------------------------------------------------------------------------------
!
!
crossref ima thresh maxpix out
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "over_mask"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      ima    (P1): input frame;"
  w/o "                     default: none"
  w/o " "
  w/o "      thresh (P2): integer threshold (ADU) for finding overexposed"
  w/o "                   pixels and integer flag for creation of mask,"
  w/o "                   separated by a comma;"
  w/o "                     default: 65534,65535"
  w/o " "
  w/o "      maxpix (P3): minimal forbidden number of overexposed pixels -"
  w/o "                   "1" means, that a frame with 1 overexp.pix. will"
  w/o "                   be declared bad;"
  w/o "                     default: 1"
  w/o " "
  w/o "      out    (P4): flag for result output on terminal: <>:show, <n>o;"
  w/o "                     default: <>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of frame: "
define/local ifrm/c/1/80 "{p1}"
if m$index(ifrm,".bdf") .eq. 0 ifrm = ifrm//".bdf"
!
if m$exist(ifrm) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({ifrm})"
  goto error
endif
!
!
define/parameter p2 65534,65535 ? -
  "        Enter thresh./flag for overexposed ADUs [def.:65534,65535]: "
outputi = m$parse(p2,"ovex")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local hgh/i/1/1 {ovex01}     ! threshold for search
define/local hghflg/i/1/1 {ovex02}  ! high value for output mask
!
!
define/parameter p3 1 n -
  "        Enter min. forbidden #oexPIX. [def.:1]: "
define/local npixmax/i/1/1 {p3}
!
outputi(1) = {{ifrm},NPIX(1)}*{{ifrm},NPIX(2)}
if npixmax .lt. 1 .or. npixmax .gt. outputi(1) then
        write/out " "
        mess/out 1 {PROID} "Could not compute the maximum number of"
        mess/out 1 {PROID} "overexposed pixels. Please check the input frame !"
    goto error
endif
!
!
define/parameter p4 y ? -
  "        Enter flag for result output [def.:show|<n>o]: "
define/local show/c/1/1 "{p4}"
!
!
!          PREPARATIONS FOR OUTPUT :
!          ***********************
!
define/local imsec/i/1/4 0 all
imsec(1) = 1                 ! (if wanted without prescan: prescan+1)
imsec(2) = 1
imsec(3) = {{ifrm},NPIX(1)}  ! (if wanted without overscan: npix(1)-overscan)
imsec(4) = {{ifrm},NPIX(2)}
!
define/local char/i/1/1 0
define/local mask/c/1/80 " " all
  char = m$index(ifrm,".bdf")-1
  mask = ifrm(1:{char})//"_oem.bdf"
-copy {ifrm} {mask}
!
!
!          INVOCATION :
!          **********
!
!******************************
!debug/module
run PIPE_EXE:fors_ovexpomap.exe
!debug/module off
!******************************
!
if outputi(1) .eq. -1 then  ! "outputi" instead of "q1" and
        write/out " "       ! negative error are exceptions
        mess/out 1 {PROID} "Mapping of overexposed pixels failed !"
  goto error
endif
!
!
!          OUTPUT :
!          ******
!
write/keyword out_a {mask}
!
if show .ne. "n" .and. mess_level .ge. 3 then
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "Result:"
        mess/out 3 {PROID} "------"
        mess/out 3 {PROID} "Number of overexposed pixels: {outputi(1)}"
        mess/out 3 {PROID} "Name of image mask          : {out_a}"
endif
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!------------------------------------------------------------------------------
