! $Id: fors_overexpo.prg,v 1.143 2001/04/06 16:19:24 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_overexpo.prg,v $
!
! .KEYWORDS   Pixel values, simple statistics
!
! .PURPOSE    Check if the raw images contain overexposed pixels and write
!             their number into a table per input frame. For each input image
!             with at least one overexposed pixel an output image is created
!             as mask with the overexposed pixels. This mask is intended to be
!             used after the basic reductions (devision by flatfield etc. and
!             devision by exposure time before photometry). It's name is writ-
!             ten into a descriptor of the input frame; the descriptor is
!             named by parameter 4 and created by the procedure.
!             Frames which have at least <P3> percent overexposed pixels are
!             removed from the input catalogue.
!             The images can be displayed with the overexposed pixels marked
!             blue.
! __________
!
! .USAGE      @p fors_overexpo icat [thresh] [maxper] [desc] [disp] 
!
! .INPUT      icat   (P1): image catalogue with input frames;
!                            default: none
!
!             thresh (P2): integer threshold (ADU) for finding overexposed
!                          pixels and integer flag for creation of mask,
!                          separated by a comma;
!                            default: 65534,65535
!
!             maxper (P3): percentage (of number of all pixels) for minimal
!                          forbidden number of overexposed pixels - frames with
!                          at least `maxper' overexposed pixels are removed
!                          from the input catalogue;
!                            default: 3.00
!
!             desc   (P4): name for descriptor with name of output mask image -
!                          will be created by the procedure;
!                            default: OVEREXPO_MASK
!
!             disp   (P5): display selection:<p>:pipeline mode:read global key
!                          NICE_CREA [y|n], <n>:none, <l>:local, or
!                          host:display[.screen] - with the latter option
!                          you can specify the name or IP-address of a display;
!                            default: <l>
!
! .OUTPUT     A catalogue "ovex_bad.cat" will be created which contains all
!             frames that are removed from the input catalogue; the name of the
!             new catalogue is written into Q3, directly followed by a comma.
!
!             A catalogue "ovex_out.cat" will be created, where all output
!             files are stored; this name is written into Q3, after the comma.
!
!             Q2 :  total number of frames with at most <P3>%-1Pix. overexp.pix.
!
!             "*_oex.tbl":  numbers of overexposed pixels
!
!             "*_oem.bdf":  masks with overexposed pixels
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Viewing: creation of a special colour lookup table and per image
!             creation of a special intensity transfer table - for belittled
!             representation the maximum values including omitted columns/rows
!             are indicated.
!             Search / Mask creation: C programme.
!
! .COMMENT    Last calculated special ITT and LUT remain in the display after
!             termination.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    0.9  98/05/04  Creation
!                  98/07/15  Release
!             1.0  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref icat thresh maxper desc disp
!
!
define/local BADCAT/c/1/20 "ovex_bad.cat"
define/local OUTCAT/c/1/20 "ovex_out.cat"
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "overexpo"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For `longlog/out':
write/keyword out_a "given command/qualif not defined ..."
show/command longlog/out | write/keyword in_a
-delete Mid{mid$sess(11:12)}1Pipe
-delete midlis{mid$sess(11:12)}.prg
if in_a .eq. out_a create/command longlog/out "write/out"
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
write/keyword q3 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      icat   (P1): image catalogue with input frames;"
  w/o "                     default: none"
  w/o " "
  w/o "      thresh (P2): integer threshold (ADU) for finding overexposed"
  w/o "                   pixels and integer flag for creation of mask,"
  w/o "                   separated by a comma;"
  w/o "                     default: 65534,65535"
  w/o " "
  w/o "      maxper (P3): percentage (of number of all pixels) for minimal"
  w/o "                   forbidden number of overexposed pixels - frames with"
  w/o "                   at least `maxper' overexposed pixels are removed"
  w/o "                   from the input catalogue;"
  w/o "                     default: 3.00"
  w/o " "
  w/o "      desc   (P4): name for descriptor with name of output mask image -"
  w/o "                   will be created by the procedure;"
  w/o "                     default: OVEREXPO_MASK"
  w/o " "
  w/o "      disp   (P5): display selection: <p>:pipeline mode:read global key"
  w/o "                   NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                   host:display[.screen] - with the latter option"
  w/o "                   you can specify the name or IP-address of a display;"
  w/o "                     default: <l>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(icat):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of catalogue with images to be examined: "
define/local icat/c/1/64 "{p1}"
if m$index(icat,".cat") .eq. 0 icat = icat//".cat"
!
if m$exist(icat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
 goto error
endif
!
show/icat {icat} n
define/local totentries/i/1/1 {outputi(1)}
if totentries .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is empty !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
define/local catal/i/1/1 0
inloop:
  store/frame inputc {icat} 1 end_inloop
  if m$exist(inputc) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "At least one catalogue entry is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
    goto error
  endif
  goto inloop
end_inloop:
!
!
!P2(thresh):
!----------
define/parameter p2 65534,65535 ? -
  "        Enter thresh./flag for overexposed ADUs [def.:65534,65535]: "
outputi = m$parse(p2,"ovex")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local findval/i/1/1 {ovex01}  ! threshold for search
define/local thresh/r/1/1 {ovex01}   ! for cuts
define/local maskflg/i/1/1 {ovex02}  ! high value for output mask
!
!
!P3(maxper):
!----------
define/parameter p3 3. n -
  "        Enter percent. (of #allPIX.) for min. forbidden #oexPIX. [def.:3.]: "
define/local maxper/r/1/1 {p3}
!
if maxper .lt. 0. .or. maxper .gt. 100. then
        write/out " "
        mess/out 1 {PROID} "Impossible percentage !"
    goto error
endif
!
!
!P4(desc):
!--------
define/parameter p4 OVEREXPO_MASK ? -
  "        Enter desc. f.names of overexp.masks [def.:OVEREXPO_MASK]: "
define/local ovexmask/c/1/64 {p4}
!
!
!P5(disp):
!--------
define/parameter p5 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p5}"
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "W o r k i n g" :
  define/local topicima/c/1/64 " " all
  define/local lopcount/i/1/1 0
  !
  !   "Calculating scale" :
  define/local scale/r/1/1 0.
  define/local lowcut/r/1/1 0.
  define/local highcut/r/1/1 0.
  define/local min/r/1/1 0.
  !
  !   "Creation ... ITT" :
  define/local stretch/r/1/1 0.
  define/local row/i/1/2 0 all
  !
  !   "Displaying" :
  define/local eps/r/1/1 0.
  !
  !   "Preparing" :
  define/local char/i/1/1 0
  define/local ovextable/c/1/64 " " all
  !
  !   "Search for overexposed pixels" :
  define/local maskima/c/1/64 " " all
  define/local xpix/i/1/1 0
  define/local ypix/i/1/1 0
  define/local npixmax/i/1/1 0
  define/local oexpix/i/1/1 0
  define/local percent/r/1/1 0.
  define/local good/i/1/1 0
  define/local bad/i/1/1 0
  define/local verybad/i/1/1 0
!
!     "Output" :
  define/local notremoved/i/1/1 0
!
!
!
!          MANAGING DISPLAY / SPECIAL LUT :
!          ******************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_dids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 1 is missing !"
    goto error
  endif
  !
  assign/display d,1 >Null
  clear/display
  display/lut off
  if nice_dids(2) .eq. 1 then
    assign/display d,2 >Null
    clear/display
    display/lut off
  endif
  if nice_dids(3) .eq. 1 then
    assign/display d,3 >Null
    clear/display
    display/lut off
  endif
  if nice_gids(1) .eq. 1 then
    assign/graphic g,1 >Null
    clear/graphic
  endif
  if nice_gids(2) .eq. 1 then
    assign/graphic g,2 >Null
    clear/graphic
  endif
  !
  !        CREATION OF OWN LUT :
  !        -------------------
  !
        mess/out 4 {PROID} "creating special LUT"
  !
    -copy MID_SYSTAB:heat.lut ovex.lut
    ovex.lut,:RED,256 = 0.
    ovex.lut,:GREEN,256 = 0.
    ovex.lut,:BLUE,256 = 1.
  assign/display d,1 >Null
  load/lut ovex.lut
  !
endif
!
!
!          PREPARING OUTPUT :
!          ****************
!
create/icat {BADCAT} null >Null
create/icat {OUTCAT} null >Null
!
!
!
!          W O R K I N G   W I T H   I M A G E S  :
!          * * * * * * * * * * * * * * * * * * *
!
        mess/out 3 {PROID} " "
        set/format T6.2,
        mess/out 3 {PROID} "max. allowed frac. of overexp. pix.: {maxper}%"
        set/format {FMTDEF}
        mess/out 3 {PROID} " "
!
catal = 0
good = 0
bad = 0
verybad = 0
test_expo:
  store/frame topicima {icat} 1 end_test_expo
  lopcount = lopcount+1
      if totentries .gt. 1 then
        mess/out 3 {PROID} "image {lopcount} / {totentries} : {topicima}"
        longlog/out 1 "     image {lopcount} / {totentries} : {topicima}" >Null
      else
        mess/out 3 {PROID} "image: {topicima}"
        longlog/out 1 "     image: {topicima}" >Null
      endif
  !
  if nice_crea .eq. "y" then
    !
    !      CALCULATING SCALE FACTOR AND CUTS :
    !      ---------------------------------
    !
    !***********************************************
    @p fors_nicedisp {topicima} ? n 0,0 ? ? 0.5,3. n  ! p3=n !!!
    !***********************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of frame failed !"
        mess/out 1 {PROID} "({topicima})"
      goto error
    endif
    !
    scale = outputi(1)
    lowcut = outputr(16)
    highcut = outputr(17)
    min = outputr(1)
    !
    !      CREATION OF SPECIAL ITT PER FRAME :
    !      ---------------------------------
    !
        mess/out 4 {PROID} "  creating special ITT"
    !
    -copy MID_SYSTAB:expo.itt ovex.itt
    !
    if thresh .gt. highcut then
      stretch = (thresh-lowcut)/(highcut-lowcut)  ! stretching of ITT
    else
      stretch = 1.
    endif
    !
    do row = 1 255                                         ! because of
      ovex.itt,:ITT,{row} = {ovex.itt,:ITT,{row}}*{stretch}*256./(256-16*3)
      if {ovex.itt,:ITT,{row}} .gt. 1. then                ! particular ITT
        do row(2) = row 255
          ovex.itt,:ITT,{row(2)} = 255./256.  ! = white
        enddo
        goto end_itt
      endif
    enddo
    end_itt:
    ovex.itt,:ITT,1 = 0.
    !
    clear/display
    load/lut ovex.lut  ! without this repetition colours can disappear
    load/itt ovex.itt
    display/lut
    !
    !
    !      DISPLAYING OF OVEREXPOSED PIXELS :
    !      ********************************
    !
        mess/out 4 {PROID} "  loading image"
    !
    if thresh .gt. lowcut then                        ! THRESH, NOT HIGHCUT !
      load/image {topicima} ? {scale},max c,c {lowcut},{thresh}
    elseif thresh .gt. min then
      load/image {topicima} ? {scale},max c,c {min},{thresh}
    else
      eps = thresh-1.
      load/image {topicima} ? {scale},max c,c {eps},{thresh}
    endif
    draw/rectangle <,<,>,> f ? green >Null
    !
  endif
  !
  !
  !        PREPARING STATISTICS :
  !        ********************
  !
        mess/out 4 {PROID} "  preparing statistics"
  !
    char = m$index(topicima,".bdf")-1
    ovextable = topicima(1:{char})//"_oex.tbl"
  create/table {ovextable} 6 1 null
  create/column {ovextable} :FRAME " " A30 C*64
  create/column {ovextable} :THRESH " " F9.1 R*4
  create/column {ovextable} :OEX_PIX " " I8 I*4
  create/column {ovextable} :OEX_PCEN "%" F8.3 R*4
  create/column {ovextable} :MAX_PIX " " I8 I*4
  create/column {ovextable} :MAX_PCEN "%" F8.2 R*4
  !
  copy/dd {topicima} *,5 {ovextable}
  copy/dd {topicima} IDENT {ovextable}
  !
  !
  !        CHECKING NUMBER OF OVEREXPOSED PIXELS
  !        AND CREATING MASK IMAGE :
  !        *************************************
  !
        mess/out 4 {PROID} "  searching for overexposed pixels"
  !
    xpix = {{topicima},NPIX(1)}
    ypix = {{topicima},NPIX(2)}
  npixmax = m$nint(xpix*ypix*maxper/100.-0.5)
  if npixmax .lt. 1 npixmax = 1
  !
  !***********************************************************
  @p fors_over_mask {topicima} {findval},{maskflg} {npixmax} n
  !***********************************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Creation of overexposure mask failed !"
        mess/out 1 {PROID} "({topicima})"
    goto error
  endif
  !
  oexpix = outputi(1)
  maskima = out_a
  !
  copy/dd {topicima} *,5 {maskima}
  copy/dd {topicima} IDENT {maskima}
  !
  outputr(1) = oexpix
  percent = outputr(1)/(xpix*ypix)*100.
  set/format F7.3,
  if oexpix .eq. 0 then
    !
        mess/out 3 {PROID} "  found none"
        longlog/out 1 "       found none" >Null
    !
    good = good+1
  elseif oexpix .lt. npixmax then
    !
        mess/out 3 {PROID} "  found {oexpix} = {percent}%"
        longlog/out 1 "       found {oexpix} = {percent}%" >Null
    bad = bad+1
  else
        mess/out 3 {PROID} "  found at least {oexpix} = {percent}% -"
        mess/out 3 {PROID} "  frame will be removed from catalogue !"
        longlog/out 1 "       found at least {oexpix} = {percent}% -" >Null
        longlog/out 1 "       frame will be removed from catalogue !" >Null
    !
    verybad = verybad+1
    add/icat {BADCAT} {topicima} >Null
  endif
  !
  {ovextable},:FRAME,1 = "{topicima}"
  {ovextable},:THRESH,1 = {thresh}
  if oexpix .lt. npixmax then
    {ovextable},:OEX_PIX,1 = {oexpix}
    {ovextable},:OEX_PCEN,1 = {percent}
  else
    {ovextable},:OEX_PIX,1 = 99999999
    {ovextable},:OEX_PCEN,1 = 9999.999
  endif
  {ovextable},:MAX_PIX,1 = {npixmax}
  {ovextable},:MAX_PCEN,1 = {maxper}
  set/format {FMTDEF}
  longlog/out 1 "       overexposure table: {ovextable}"
  !
  add/icat {OUTCAT} {ovextable} >Null
  if oexpix .gt. 0 .and. oexpix .lt. npixmax then
    !
        mess/out 4 {PROID} "  keeping mask image {maskima}"
    !
    !!!add/icat {OUTCAT} {maskima} >Null  ! no official product
    write/descriptor {topicima} {ovexmask}/c/1/76 {maskima}  ! for later
    write/dhelp {topicima} {ovexmask} "Assigned overexposure mask"
  else                                                       ! assignment !!!
    !
        mess/out 4 {PROID} "  deleting mask image {maskima}"
    !
    -delete {maskima}
  endif
  !
  goto test_expo
end_test_expo:
!
!          E N D   O F   L O O P
!          * * * * * * * * * * *
!
!
!
!          REMOVING TOO BAD FRAMES FROM INPUT CATALOGUE :
!          ********************************************
!
if bad .eq. 0 .and. verybad .eq. 0 then
        mess/out 3 {PROID} "Result: All frames are O.K."
        mess/out 3 {PROID} "------"
  good = totentries
else
  good = totentries-bad-verybad
  !
        mess/out 3 {PROID} " "
        set/format J3 T6.2,
        mess/out 3 {PROID} "Result:"
        mess/out 3 {PROID} "------"
        mess/out 3 {PROID} "Number of frames       : {totentries}"
        mess/out 3 {PROID} "Not overexposed        : {good}"
        mess/out 3 {PROID} "Overexposed (< {maxper}%): {bad}"
  if verybad .eq. 0 then
        mess/out 3 {PROID} "Overexposed (>={maxper}%): {verybad}"
  else
        mess/out 3 {PROID} "Overexposed (>={maxper}%): {verybad}-"
        mess/out 3 {PROID} "  removing from input catalogue ..."
    !
    subtract/icat {icat} {BADCAT} >Null
    -delete dirfile.ascii
  endif
        set/format {FMTDEF}
  !
endif
!
!
!          OUTPUT :
!          ******
!
notremoved = totentries-verybad
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} ">>> The output tables are in <{OUTCAT}>: <<<"
        mess/out 4 {PROID} ">>>                             <*_oex.tbl>. <<<"
        mess/out 4 {PROID} ">>> Mask    frames    are       <*_oem.bdf>. <<<"
if mess_level .eq. 4 then
  read/icat {OUTCAT}
endif
!
!
set/format
!if nice_crea .eq. "y" then
!  -delete ovex.itt
!  -delete ovex.lut
!  load/lut heat
!  load/itt ramp
!endif
!
!
if warn .eq. 0 then
  return 0 {notremoved} {BADCAT},{OUTCAT}
else
  return -1 {notremoved} {BADCAT},{OUTCAT}
endif
!
error:
  set/format
  !if m$existk("nice_crea") .eq. 1 then
  !  if nice_crea .eq. "y" then
  !    load/lut heat
  !    load/itt ramp
  !  endif
  !endif
  return 1 -99 -99,-99
!
!
!-------------------------------------------------------------------------------
