! $Id: fors_photo_fitlim.prg,v 1.50 2000/02/23 14:49:32 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_photo_fitlim.prg,v $
!
! .KEYWORDS   Special table fitting
!
! .PURPOSE    Fitting of signal-to-noise versus flux in the photometry output
!             table.
! __________
!
! .USAGE      @p fors_photo_fitlim tabcol [snvals]
!
! .INPUT      tabcol (P1): name of photometry table AND names of columns with
!                          error_flags/flux/signal-to-noise, separated by
!                          commas;
!                            default: none,FLAGS,FLUX,S2N
!
!             snvals (P2): signal-to-noise value for limiting magnitude AND
!                          upper signal-to-noise limit for creation of fit,
!                          separated by a comma;
!                            default: 5.,10.
!
! .OUTPUT     OUTPUTR(1): limiting flux
!             OUTPUTR(2): limiting magnitude
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Only proper objects (no error flags in concerning table column)
!             with S/N lower than the second number in P2 are used for a robust
!             linear fit (approximation of S/N formula near background).
!
! .COMMENT    Special procedure for photometry - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/11/05  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!------------------------------------------------------------------------------
!
!
crossref tabcol snvals
!
!
!  FAILNO is "+lower_levels" in fors_photometry.prg
if m$existk("FAILNO") .eq. 0 define/local FAILNO/r/1/1 -99999.9
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "photo_fitlim"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      tabcol (P1): name of photometry table AND names of columns with"
  w/o "                   error_flags/flux/signal-to-noise, separated by"
  w/o "                   commas;"
  w/o "                     default: none,FLAGS,FLUX,S2N"
  w/o " "
  w/o "      snvals (P2): signal-to-noise value for limiting magnitude AND"
  w/o "                   upper signal-to-noise limit for creation of fit,"
  w/o "                   separated by a comma;"
  w/o "                     default: 5.,10."
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(tabcol):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter photometry table, cols.[def.:FLAGS,FLUX,S2N]: "
outputi = m$parse(p1,"tbl")
if outputi(1) .ne. 4 .and. outputi(1) .ne. 1 then  ! 1 allowed for col.defaults
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P1 !"
  goto error
endif
define/local phottable/c/1/64 "{tbl01}"
if m$index(phottable,".tbl") .eq. 0 phottable = phottable//".tbl"
if outputi(1) .eq. 4 then
  define/local cflags/c/1/16 "{tbl02}"
  define/local cflux/c/1/16 "{tbl03}"
  define/local cs2n/c/1/16 "{tbl04}"
  if cflags(1:1) .eq. ":" cflags = cflags(2:)
  if cflux(1:1) .eq. ":" cflux = cflux(2:)
  if cs2n(1:1) .eq. ":" cs2n = cs2n(2:)
elseif outputi(1) .eq. 1 then  ! defaults
  define/local cflags/c/1/16 "FLAGS"
  define/local cflux/c/1/16 "FLUX"
  define/local cs2n/c/1/16 "S2N"
endif
!
if m$existc(phottable,":{cflags}") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({phottable})"
  goto error
elseif m$existc(phottable,":{cflags}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({phottable},:{cflags})"
  goto error
elseif m$existc(phottable,":{cflux}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({phottable},:{cflux})"
  goto error
elseif m$existc(phottable,":{cs2n}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({phottable},:{cs2n})"
  goto error
endif
!
show/table {phottable} >Null
define/local allrows/i/1/1 {outputi(2)}  ! needed below, too
if allrows .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({phottable})"
  goto error
endif
!
!
!P2(snvals):
!----------
define/parameter p2 5.,10. n -
  "        Enter S/N val.f.lim.mag., upper S/N lim.f.fit [def.:5.,10.]: "
outputi = m$parse(p2,"minmax")
!
define/local lims2n/r/1/1 {minmax01}
if lims2n .le. 0. then
        write/out " "
        mess/out 1 {PROID} "Impossible s/n value for limiting mag. !"
  goto error
endif
!
define/local maxs2n/r/1/1 {minmax02}
if maxs2n .le. lims2n then
        write/out " "
        mess/out 1 {PROID} "Upper S/N limit f.fit is <= S/N value f.lim.mag. !"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local median/r/1/1 0.
  define/local zerop/r/1/1 0.
  define/local slope/r/1/1 0.
  define/local badcoef/i/1/1 0
  define/local limflux/r/1/1 0.
  define/local limmag/r/1/1 0.
  define/local fluxlow/r/1/1 0.
  define/local s2nlow/r/1/1 0.
!
!
!
!          PLOTTING SIGNAL-TO-NOISE VALUES VERSUS FLUXES :
!          *********************************************
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "    plotting S/Ns versus fluxes"
  !
  create/column {phottable} :Index " " I5 I*2  ! after sort/table below original
  do outputi = 1 allrows                       ! order must be restored (in last
    {phottable},:Index,{outputi} = outputi     ! rows are special results) !
  enddo
  sort/table {phottable} :{cflux}
  assign/graphic g,1 >Null
  set/graphic default
  set/graphic pmode=0 bcolour=1 stype=1 ltype=0
endif
select/table {phottable} :{cflags}.eq."  -------".and.:{cs2n}.lt.{MAXS2N} >Null
  !                                 ! "  -------": 2 blanks, 7 dashes !
  !
  ! flux<0 (prior sky subtraction) and fwhm<0 (bug in
  ! -------------------------------------------------
  ! SExtractor) already removed in main procedure !
  ! ---------------------------------------------
  !
if nice_crea .eq. "y" then
  set/graphic colour=1
  plot/table {phottable} :{cflux} :{cs2n} >Null
  set/graphic xaxis=0.,{plrgrap(2)} yaxis=0.,{plrgrap(6)}
  set/graphic colour=2
  plot/table {phottable} :{cflux} :{cs2n} >Null
endif
!
!
!          COMPUTATION OF LIMITING MAGNITUDE :
!          *********************************
!
        mess/out 4 {PROID} "    computing limiting magnitude"
!
create/column {phottable} :SLOPE " " F10.3 R*4
!
compute/table {phottable} :SLOPE = :{cs2n}/:{cflux}  ! for flux=0 s/n=0
select/table {phottable} :{cflags}.eq."  -------".and.:{cs2n}.lt.{MAXS2N} >Null
!                                   ! "  -------": 2 blanks, 7 dashes !
copy/ti {phottable} stat.bdf
set/refc {phottable} :SLOPE
show/table {phottable} >Null
set/refc {phottable} seq
statistics/image stat.bdf [<,@{outputi(4)}:>,@{outputi(4)}] ? ? xn >Null
median = outputr(8)
-delete stat.bdf
!
zerop = 0.
slope = median
!
! calculate x for: slope*x = lims2n-zerop
!   ^
! S |   x
! / |  x       slope > 0                         => direct proportional
! N | x        zerop < lims2n [=S/N f.lim.mag.]  => lim.mag. > 0
!   |x
!   +------->
!       Flux
!
if slope .gt. 0. then
  limflux = (lims2n-zerop)/slope
  limmag = -2.5*m$log10(limflux)
  badcoef = 0
else
        mess/out 2 {PROID} "    No computation of limiting flux possible !"
        mess/out 2 {PROID} "    (slope of fit is equal zero)"
  limflux = FAILNO
  limmag = FAILNO
  badcoef = 1
  warn = 1
endif
!
!
!          CALCULATING FIT AND PLOTTING FITTED
!          SIGNAL-TO-NOISE VALUES VERSUS FLUXES :
!          ************************************
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "    plotting fitted S/Ns versus fluxes"
  !
  create/column {phottable} :S2N_FIT " " F10.3 R*4
  compute/table {phottable} :S2N_FIT = :{cflux}*{slope}
  !
  set/graphic stype=0 ltype=1 colour=3
  overplot/table {phottable} :{cflux} :S2N_FIT >Null
  !
  set/format ,E14.7  ! for overplot/line double length as char. too long
  fluxlow = {{phottable},:{cflux},@1}  ! table has been sorted above
  s2nlow = {{phottable},:S2N_FIT,@1}
  overplot/line 3 0.,{zerop} {fluxlow},{s2nlow}
  if badcoef .eq. 0 then
    overplot/line 3 {plrgrap(1)},{lims2n} {limflux},{lims2n}
    overplot/line 3 {limflux},{plrgrap(5)} {limflux},{lims2n}
  else
    overplot/line 3 {plrgrap(1)},{lims2n} {plrgrap(2)},{lims2n}
  endif
  set/format {FMTDEF}
  !
  !***************
  @p fors_nicegrap
  !***************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of plot failed !"
    goto error
  endif
  set/format F3.0,
  label/graphic "Lim.Flux ( S2N = {lims2n} )" {nice_glabw}
  set/format {FMTDEF}
  !
  sort/table {phottable} :Index
  delete/column {phottable} :Index
  delete/column {phottable} :S2N_FIT
  !
endif
!  
delete/column {phottable} :SLOPE
!
!
!          OUTPUT :
!          ******
!
outputr(1) = limflux
outputr(2) = limmag
!
!
select/table {phottable} all >Null
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  return 1
!
!
!------------------------------------------------------------------------------
