! $Id: fors_photo_flags.prg,v 1.18 2000/02/23 14:54:26 nrainer Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_photo_flags.prg,v $
!
! .KEYWORDS   Special character conversion
!
! .PURPOSE    Handle in- and output for "fors_photo_flags.c" and invoke it.
! __________
!
! .USAGE      @p fors_photo_flags tabcol maxrow
!
! .INPUT      tabcol (P1): name of photometry table AND name of flag column,
!                          separated by a comma;
!                            default: none,FLAGS
!
!             maxrow (P2): number of last row to be converted (last table rows
!                          can be empty - NO "NULL" VALUES ARE ALLOWED);
!                            default: <all rows>
!
! .OUTPUT     input table with converted FLAG column
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for photometry - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/11  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!------------------------------------------------------------------------------
!
!
crossref tabcol maxrow
!
!
!  For `mess/out':
define/local PROID/c/1/40 "photo_flags"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      tabcol (P1): name of photometry output table AND name of flag;"
  w/o "                   column, separated by a comma;"
  w/o "                     default: none,FLAGS"
  w/o " "
  w/o "      maxrow (P2): number of last row to be converted (last table rows"
  w/o "                   can be empty)"
  w/o "                     default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format I1
set/format E14.7,E23.15
!
!
!          INPUT :
!          *****
!
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter photometry table, flag column [def.:FLAGS]: "
outputi = m$parse(p1,"tbl")
if outputi(1) .gt. 2 then  ! 1 allowed for column default
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P1 !"
  goto error
endif
write/keyword in_a/c/1/60 "{tbl01}"
if m$index(in_a,".tbl") .eq. 0 in_a = in_a //".tbl"
if outputi(1) .eq. 2 then
  write/keyword in_b/c/1/16 "{tbl02}"
  if in_b(1:1) .eq. ":" in_b = in_b(2:)
elseif outputi(1) .eq. 1 then  ! default
  write/keyword in_b/c/1/16 "FLAGS"
endif
!
if m$existc(in_a,":{in_b}") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({in_a})"
  goto error
elseif m$existc(in_a,":{in_b}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({in_a},:{in_b})"
  goto error
endif
!
show/table {in_a} >Null
define/local totrows/i/1/1 {outputi(2)}
if totrows .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({in_a})"
  goto error
endif
!
!
define/parameter p2 {totrows} n -
  "        Enter last row to be converted : "
write/keyword inputi/i/1/1 {p2}
!
if inputi(1) .gt. totrows then
        write/out " "
        mess/out 1 {PROID} "Entered row number does not exist !"
  goto error
endif
!
!
select/table {in_a} seq.le.{inputi(1)}.and.:{in_b}.eq."" >Null ! `""': NO blank!
if outputi(1) .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "flag col. in phot.tbl. contains `NULL'"
        mess/out 1 {PROID} "values below entered max. row number !"
        mess/out 1 {PROID} "({in_a},:{in_b},"<{inputi(1)}")"
  goto error
endif
select/table {in_a} all
!
!
!          INVOCATION :
!          **********
!
!********************************
run PIPE_EXE:fors_photo_flags.exe
!********************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Flag conversion failed !"
  goto error
endif
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!------------------------------------------------------------------------------
