! $Id: fors_photo_plotsee.prg,v 1.36 2000/02/23 14:49:31 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_photo_plotsee.prg,v $
!
! .KEYWORDS   Special table plotting
!
! .PURPOSE    Plotting of FWHMs versus flux in the photometry output table.
! __________
!
! .USAGE      @p fors_photo_plotsee tabcol [mins2n] [maxell] [minpix] fwhm
!
! .INPUT      tabcol (P1): name of photometry table AND names of columns with
!                          s2n/ellipticity/#pix_per_obj./error_flags/flux/FWHM,
!                          separated by commas;
!                            default: none,S2N,ELLIP,NPIX,FLAGS,FLUX,FWHM
!
!             mins2n (P2): minimal allowed s/n value for inclusion of object
!                          in plot;
!                            default: 60.
!
!             maxell (P3): maximal allowed ellipticity for inclusion of object
!                          in plot;
!                            default: 0.15
!
!             minpix (P4): minimal allowed number of pixels per object for
!                          inclusion of object in plot;
!                            default: 25
!
!             fwhm   (P5): median value of FWHM table column, sigma;
!                            default: none,none
!
! .OUTPUT     -
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Only proper objects (no error flags in concerning table column)
!             with ellipticities <= {P2} and number of pixels >= {P3} are used
!             for a plot.
!
! .COMMENT    Special procedure for photometry - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/11/05  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!------------------------------------------------------------------------------
!
!
crossref tabcol mins2n maxell minpix fwhm
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "photo_plotsee"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      tabcol (P1): name of photometry table AND names of columns with"
  w/o "                   s2n/ellipticity/#pix_per_obj./error_flags/flux/FWHM,"
  w/o "                   separated by commas;"
  w/o "                     default: none,S2N,ELLIP,NPIX,FLAGS,FLUX,FWHM"
  w/o " "
  w/o "      maxell (P2): minimal allowed s/n value for inclusion of object"
  w/o "                   in plot;"
  w/o "                     default: 60."
  w/o " "
  w/o "      maxell (P3): maximal allowed ellipticity for inclusion of object"
  w/o "                   in plot;"
  w/o "                     default: 0.15"
  w/o " "
  w/o "      minpix (P4): minimal allowed number of pixels per object for"
  w/o "                   inclusion of object in plot;"
  w/o "                     default: 25"
  w/o " "
  w/o "      fwhm   (P5): median value of FWHM table column, sigma;"
  w/o "                     default: none,none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(tabcol):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter photometry table, cols.[def.:S2N,ELLIP,NPIX,FLAGS,FLUX,FWHM]: "
outputi = m$parse(p1,"tbl")
if outputi(1) .ne. 7 .and. outputi(1) .ne. 1 then  ! 1 allowed for col.defaults
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P1 !"
  goto error
endif
define/local phottable/c/1/64 "{tbl01}"
if m$index(phottable,".tbl") .eq. 0 phottable = phottable//".tbl"
if outputi(1) .eq. 7 then
  define/local cs2n/c/1/16 "{tbl02}"
  define/local cell/c/1/16 "{tbl03}"
  define/local cpix/c/1/16 "{tbl04}"
  define/local cflags/c/1/16 "{tbl05}"
  define/local cflux/c/1/16 "{tbl06}"
  define/local cfwhm/c/1/16 "{tbl07}"
  if cs2n(1:1) .eq. ":" cs2n = cs2n(2:)
  if cell(1:1) .eq. ":" cell = cell(2:)
  if cpix(1:1) .eq. ":" cpix = cpix(2:)
  if cflags(1:1) .eq. ":" cflags = cflags(2:)
  if cflux(1:1) .eq. ":" cflux = cflux(2:)
  if cfwhm(1:1) .eq. ":" cfwhm = cfwhm(2:)
elseif outputi(1) .eq. 1 then  ! defaults
  define/local cs2n/c/1/16 "S2N"
  define/local cell/c/1/16 "ELLIP"
  define/local cpix/c/1/16 "NPIX"
  define/local cflags/c/1/16 "FLAGS"
  define/local cflux/c/1/16 "FLUX"
  define/local cfwhm/c/1/16 "FWHM"
endif
!
if m$existc(phottable,":{cs2n}") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({phottable})"
  goto error
elseif m$existc(phottable,":{cs2n}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({phottable},:{cs2n})"
  goto error
elseif m$existc(phottable,":{cell}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({phottable},:{cell})"
  goto error
elseif m$existc(phottable,":{cpix}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({phottable},:{cpix})"
  goto error
elseif m$existc(phottable,":{cflags}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({phottable},:{cflags})"
  goto error
elseif m$existc(phottable,":{cflux}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({phottable},:{cflux})"
  goto error
elseif m$existc(phottable,":{cfwhm}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({phottable},:{cfwhm})"
  goto error
endif
!
show/table {phottable} >Null
define/local allrows/i/1/1 {outputi(2)}  ! needed below, too
if allrows .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({phottable})"
  goto error
endif
!
!
!P2(mins2n):
!----------
define/parameter p2 60. n -
  "        Enter lower threshold for s/n [def.:60.]: "
define/local mins2n/r/1/1 {p2}
!
if mins2n .le. 0. then
        write/out " "
        mess/out 1 {PROID} "Impossible s/n threshold !"
  goto error
endif
!
!
!P3(maxell):
!----------
define/parameter p3 0.15 n -
  "        Enter upper threshold for ellipticity [def.:0.15]: "
define/local maxell/r/1/1 {p3}
!
if maxell .lt. 0. .or. maxell .ge. 1. then
        write/out " "
        mess/out 1 {PROID} "Impossible ellipticity threshold !"
  goto error
endif
!
!
!P4(minpix):
!----------
define/parameter p4 25 n -
  "        Enter lower threshold for #pix/obj. [def.:25]: "
define/local minpix/i/1/1 {p4}
!
if minpix .lt. 1 then
        write/out " "
        mess/out 1 {PROID} "Impossible threshold for number of pix./obj. !"
  goto error
endif
!
!
!P5(fwhm):
!--------
define/parameter p5 ? ? -
  "        Enter FWHM median value,sigma: "
outputi = m$parse(p5,"fwhm")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P5 !"
  goto error
endif
define/local fmedian/r/1/1 {fwhm01}
define/local fsigma/r/1/1 {fwhm02}
!
if fmedian .le. 0 .or. fsigma .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible FWHM median value or sigma !"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
! none
!
!
!          PLOTTING FWHMS VERSUS FLUXES :
!          ****************************
!
        mess/out 4 {PROID} "    plotting FWHMs versus fluxes"
!
assign/graphic g,2 >Null
set/graphic default
set/graphic pmode=0 bcolour=1 stype=1 ltype=0
set/graphic colour=1
select/table {phottable} :{cs2n}.ge.{mins2n} -
                    .and.:{cell}.le.{maxell} -
                    .and.:{cpix}.ge.{minpix} -
                    .and.:{cflags}.eq."  -------" >Null
  !                                   "  -------": 2 blanks, 7 dashes !
  ! flux<0 (prior sky subtraction) and fwhm<0 (bug in
  ! -------------------------------------------------
  ! SExtractor) already removed in main procedure !
  ! ---------------------------------------------
  !
plot/table {phottable} log(:{cflux}) :{cfwhm} >Null
set/graphic yaxis=0.,{plrgrap(6)}
set/graphic colour=2
plot/table {phottable} log(:{cflux}) :{cfwhm} >Null
!
!
!          PLOTTING MEAN AND SIGMA RANGES :
!          ******************************
!
        mess/out 4 {PROID} "    plotting FWHM median and 1-sigma ranges"
!
set/graphic colour=3
overplot/line 1 {plrgrap(1)},{fmedian} {plrgrap(2)},{fmedian}
outputr(1) = fmedian-fsigma
if outputr(1) .gt. plrgrap(5) then
  overplot/line 3 {plrgrap(1)},{outputr(1)} {plrgrap(2)},{outputr(1)}
endif
outputr(2) = fmedian+fsigma
if outputr(2) .lt. plrgrap(6) then
  overplot/line 3 {plrgrap(1)},{outputr(2)} {plrgrap(2)},{outputr(2)}
endif
!
!***************
@p fors_nicegrap
!***************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of plot failed !"
  goto error
endif
label/graphic "FWHM: median +- 1sigma" {nice_glabw}
!
select/table {phottable} all >Null
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  return 1
!
!
!------------------------------------------------------------------------------
