! $Id: fors_plotlid.prg,v 1.5 2000/07/04 19:04:27 nrainer Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_plotlid.prg,v $
!
!.KEYWORDS    Show line identifications
!
!.USAGE       plot/ident wlc linpos mos slit ybin gwnd range color
!
!.INPUT       wlc      (P1)   Calibration frame
!             linpos   (P2)   Input table with line positions and
!                             identifications (linpos).
!             mos      (P3)   Slit boundary table
!             slit     (P4)   Slit number
!             ybin     (P5)   Step and binning along y-axis
!             gwnd     (P6)   Graphic window ID to be used.
!             range    (P7)   Y-axis range in ADU
!             color    (P8)   Color for wavelength labels
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Plot a cut through a calibration frame along the dispersion
!             axis and label identified emission lines.
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/10/07:  Creation
!
!------------------------------------------------------------------------------

crossref wlc linpos mos slit ybin gwnd range color


define/param p1 ?       c  "Enter name of calibration frame:"
define/param p2 ?       c  "Enter name of line positions table:"
define/param p3 {mos}   c  "Enter slit table:"  
define/param p4 1       c  "Enter slit selection:"
define/param p5 {ybin}  c  "Enter step and bin width along y-axis:"
define/param p6 +       c  "Enter graphic window ID:"
define/param p7 +       c  "Enter yaxis range:"
define/param p8 4       c  "Enter label color:"
define/maxpar 8


! Procedure identification
define/local procID/c/1/80      "PlotIdent"


! Local constants
define/local tmp/c/1/12     "middumma.tbl"


! Local variables
define/local gidx/i/1/1     0
define/local width/i/1/1    0
define/local slim/i/1/2     0     all
define/local edge/i/1/2     0     all
define/local center/r/1/1   0.
define/local xrlim/r/1/2    0.    all
define/local yrlim/r/1/2    0.    all
define/local lpos/r/1/2     0.    all
define/local snum/i/1/1     {p4}
define/local bin/i/1/2      {p5}
define/local lcol/i/1/1     {p8}
define/local frm/c/1/100    {p1}
define/local ltab/c/1/100   {p2}
define/local stab/c/1/100   {p3}
define/local range/c/1/100  {p7}
define/local mid$selidx/i/1/1 0 ? +low
define/local srow/i/1/1     0



! Process parameters
if m$tstno(p6) .eq. 1 then
  gidx = {p6}
endif

if range(1:1) .ne. "+" then
  write/key yrlim {range}
else
  yrlim(1) = m$value({frm},lhcuts(3))
  yrlim(2) = m$value({frm},lhcuts(4))
  yrlim(1) = m$nint(yrlim(1))
  yrlim(2) = m$nint(yrlim(2))
endif
xrlim(1) = m$value({frm},start(1))
xrlim(2) = m$value({frm},start(1)) + (m$value({frm},npix(1)) - 1) * -
           m$value({frm},step(1))

! Check availability of graphic window
if nice_gids({gidx}) .ne. 1 then
  mess/out 2 {procID} "Requested graphic window is not active!"
  return 0
endif


! Get lower and upper limit of central scan row
width = 2 * bin(2) + 1

select/table {stab} :{c_ltslit}.eq.{snum} >Null
if outputi .eq. 0 then
  mess/out 2 {procID} "Requested slit number is not used !"
  return 0
endif
srow = mid$selidx
select/table {stab} all

slim(1) = (m$value({stab},:{c_slystart},{srow}) - m$value({frm},start(2))) / -
           m$value({frm},step(2))
slim(2) = (m$value({stab},:{c_slyend},{srow}) - m$value({frm},start(2))) / -
           m$value({frm},step(2)) + 1

center = m$nint(0.5*(slim(1) + slim(2))) * m$value({frm},step(2)) + -
         m$value({frm},start(2))

edge(1) = center - bin(2)
edge(2) = edge(1) + width - 1


! Average central scan row
average/row {tmp} = {frm} {edge(1)},{edge(2)}


! Select identified emission lines
select/tab {ltab} abs(:{c_ltypos} - {center}) .lt. 0.5*{bin(2)} >Null


! Setup plotting device
set/format i1
assign/graph g,{gidx} >Null
set/graph xaxis={xrlim(1)},{xrlim(2)} yaxis={yrlim(1)},{yrlim(2)} -
          font=0 pmode=1
clear/graph
set/format


! Plot averaged scan row
set/grap color=1
plot/axes ? ? ?  "X-Position [pxl]" "Mean Intensity [adu]"
overplot/row {tmp}
set/grap color=4
over/ident {ltab} :{c_ltxpos} :{c_ltwave}

lpos(1) = 1.5 * xrlim(1)
lpos(2) = 1.03 * yrlim(2)

set/format i1
set/grap tsize=0.75
label/graphic "Slit #{snum}: Final line identifications of central scan row" -
              {lpos(1)},{lpos(2)} 0. 1 1
set/format


! Cleanup
select/tab {ltab} all
set/graph
-delete {tmp}

return 0

