! $Id: fors_plotrsp.prg,v 1.2 1999/03/22 09:30:55 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_plotrsp.prg,v $
!
!.KEYWORDS    Plot computed instrument response
!
!.USAGE       view/response response stdlst objno gwnd color
!
!.INPUT       response (P1)   Instrument response file
!             stdlst   (P2)   Standard star spectrum from which response
!                             is derived
!             objno    (P3)   Object entry in standard start list
!             gwnd     (P4)   Graphic window ID to be used.
!             range    (P5)   X axis range to be plotted
!             color    (P6)   Color for annotations
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Plot the derived instrument reponse function.
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/10/07:  Creation
!
!------------------------------------------------------------------------------

crossref response stdlst objno window range color


define/param p1 ?       c  "Enter response file:"
define/param p2 ?       c  "Enter standard star spectrum:"
define/param p3 ?       c  "Standard star object number:"  
define/param p4 1       c  "Enter graphic window ID:"
define/param p5 +       c  "Enter x-axis range:"
define/param p6 4       c  "Enter color for annotations:"
define/maxpar 6


! Procedure identification
define/local procID/c/1/80      "ViewResponse"


! Local variables
define/local n/i/1/1        0
define/local gidx/i/1/1     0
define/local snum/i/1/1     0
define/local ymin/r/1/1     0.
define/local ymax/r/1/1     0.
define/local swid/r/1/1     0.
define/local xrlim/r/1/2    0.    all
define/local yrlim/r/1/2    0.    all
define/local lpos/d/1/2     0.    all
define/local gname/c/1/100  " "   all
define/local fname/c/1/100  " "   all
define/local objno/i/1/1    {p3}
define/local lcol/i/1/1     {p6}
define/local frm/c/1/100    {p1}
define/local stab/c/1/100   {p2}
define/local range/c/1/100  {p5}



! Process parameters
if m$tstno(p4) .eq. 1 then
  gidx = {p4}
endif

if range(1:1) .ne. "+" then
  write/key xrlim {range}
else
  if m$filtyp(frm,".bdf") .eq. 2 then
    statistics/table {frm} :{c_rswave} >Null
    xrlim(1) = outputr(1)
    xrlim(2) = outputr(2)
  else
    xrlim(1) = m$value({frm},start(1))
    xrlim(2) = xrlim(1) + (m$value({frm},npix(1)) - 1) * m$value({frm},step(1))
  endif

  xrlim(1) = m$nint(xrlim(1))
  xrlim(2) = m$nint(xrlim(2))
endif

if m$filtyp(frm,".bdf") .eq. 2 then
  statistics/table {frm} :{c_rsratio} >Null
else
  statistics/image {frm} option=XN
endif
yrlim(1) = outputr(1)
yrlim(2) = outputr(2)


! Check availability of graphic window
if nice_gids({gidx}) .ne. 1 then
  mess/out 2 {procID} "Requested graphic window is not active!"
  return 1
endif


! Get number of slitlet where standard star was found
snum = m$value({stab},:{c_woslit},{objno})


! Get instrument setup information
gname = m$value({frm},{h_grism1})
swid = m$value({frm},{h_slwidth})

if fors_nfilt .gt. 0 then
  n = fors_nfilt - 1
  if n .gt. 0 then
    do i = 1 n
      fname = fname // fors_filti({i}) // ","
    enddo
  endif
  fname = fname // fors_filti({fors_nfilt})
else
  fname = "None"
endif


! Setup plotting device
set/format i1
assign/graph g,{gidx} >Null
set/graph xaxis={xrlim(1)},{xrlim(2)} yaxis={yrlim(1)},{yrlim(2)} -
          font=0 pmode=0
clear/graph
set/format


! Plot instrument response
set/grap color=1 ltype=1 stype=0
plot/response {frm} ratio
if yrlim(1) .lt. 0. then
  set/grap color=2
  overplot/line 4 {xrlim(1)},0. {xrlim(2)},0.
endif

if yrlim(2) .gt. 1. then
  set/grap color=2
  overplot/line 4 {xrlim(1)},1. {xrlim(2)},1.
endif

set/graph color=4

set/format i1 f1.2,e18.8
lpos(1) = xrlim(1) + 0.025 * (xrlim(2) - xrlim(1))

lpos(2) = yrlim(1) + 0.950 * (yrlim(2) - yrlim(1))
label/graph "Grism: {gname}" {lpos(1)},{lpos(2)} 0. 1 1

lpos(2) = lpos(2) - 0.055 * (yrlim(2) - yrlim(1))
label/graph "Slit width: {swid} arcsec" {lpos(1)},{lpos(2)} 0. 1 1

lpos(2) = lpos(2) - 0.055 * (yrlim(2) - yrlim(1))
label/graph "Filter: {fname}" {lpos(1)},{lpos(2)} 0. 1 1


! Title string
lpos(1) = xrlim(1) + 0.025 * (xrlim(2) - xrlim(1))
lpos(2) = yrlim(1) + 1.025 * (yrlim(2) - yrlim(1))

set/grap tsize=0.75
label/graph "FORS instrument response from slit #{snum}" -
            {lpos(1)},{lpos(2)} 0. 1 1
set/format


! Cleanup
set/graph

return 0

