! $Id: fors_plt_align.prg,v 1.17 1999/11/25 11:24:04 sbogun Exp $
!+++ FORS_PLT_ALIGN +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_plt_align.prg,v $
!
!.KEYWORDS      Produces a plot of aligned standard star measurments
!
!.USAGE         @p fors_plt_align in_tbl filter part hardcp dispno limmag
!
!.INPUT         in_tbl  (P1): input table to be plotted
!           filter  (P2): reference filter used
!           part    (P3): plot part to be plotted (out of 2)
!           hardcp      (P4): hardcopy switch
!           dispno      (P5): display number to use
!           limmag      (P6): limiting magnitude for plot (upper limit)
!           offset  (P7): plot offset for right ascension
!
!.OUTPUT        terminal output [optional: POSTSCRIPT hardcopy]
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Plots the location (rec,dec) of observed standard stars in a
!           (rec,dec)-diagram. Coincidences between observation and ref.
!           star catalog (checked by recipe fors_img_cal_align) are
!           surrounded by a red rectangle.
!
!.ALGORITHM     Calls standard MIDAS-commands (plot/tab)
!
!.ENVIRON       FORS, CCDRED context
!           Uses global keywords: c_rect, c_decl, NICE_GDIM
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.9 1998/07/15  Creation
!           1.2 1999/02/24  limiting magnitude added (P6)
!           1.3 1999/09/09  same plot offset (P3=1->offsetP7->P3=2) (NR).
!
!.COMMENT       Part 1: Plots the observed standard star locations
!           Part 2: Marks the coincidences and creates a hardcopy,
!                   if desired.
!
!           WARNING:
!           A graphic display G,{P5} has to be created before calling this
!           procedure, otherwise a standard display will be created.
!
!------------------------------------------------------------------------------
!
! Check if graphic enabled
!
if nice_crea .eq. "n" return    !GRAPHIC disabled: do nothing
!
! Check parameters
!
define/param p1 ? c "Enter name table to be plotted:"
define/param p2 ? c "Enter name of reference colour used:"
define/param p3 0 n "Enter number of part to be plotted (1,2):"
define/param p4 N c "Hardcopy Yes or No (Y,N):"
define/param p5 1 n "Graphic display number to use:"
define/param p6 + n "Enter limiting magnitude (upper limit): "
define/param p7 + ? "Enter offset for plotting RA (degrees): "
!
! local keywords
!
define/local xsc/r/1/1 624.0    !do not change these
define/local ysc/r/1/1 445.0    !...  two values
define/local mxrow/i/1/1 14     !maximum number of object names displayed.

define/local iramin/r/1/1 0
define/local ramin/r/1/1 0
define/local ramax/r/1/1 0
define/local mode/i/1/1 0
define/local i1/i/1/1 0
define/local nrow/i/1/1 0
define/local wid/r/1/1 1.0
define/local hig/r/1/1 1.0
define/local xof/r/1/1 0.0
define/local yof/r/1/1 0.0
define/local x1/r/1/1 0.0
define/local x2/r/1/1 0.0
define/local dr/r/1/1 0.0
define/local c_magx/c/1/10 " " ALL
define/local name/c/1/80 " " ALL

mode = {P3}


xsc = NICE_GDIM(1) / xsc
ysc = NICE_GDIM(2) / ysc
!xsc = 1.
!ysc = 1.
wid = 100.0 * xsc
hig = 100.0 * ysc
xof = 35.  * xsc
yof = 17.  * ysc
x1  = 50.  * xsc
x2  = 107. * ysc

set/format I1 F9.4
show/table {P1} >Null
nrow = outputi(2)

statistic/table {P1} :RA >Null
!
!--- added (NR): ------------------
if P7 .eq. "+" then
  iramin = m$nint({outputr(1)})*1.0
else
  iramin = {P7}
endif
!----------------------------------
!
ramin = outputr(1) - iramin
ramax = outputr(2) - iramin
dr = m$abs(ramax - ramin)                               !check if RA-range is
if dr .lt. 0.008 then                                   !too small
  dr = (ramax + ramin) / 2.0                            !otherwise labels on
  ramax = dr + 0.004                                    !x-axis are not
  ramin = dr - 0.004                                    !readable.
endif
compute/table {P1} :RA = :RA - {iramin}

if P6(1:1) .ne. "+" then
  c_magx = ":{c_mag}_{P2}"
  select/table {P1} {c_magx}.lt.{P6} >NULL
  mess/out 4 plt_align "Plot only {outputi(1)} objects with magnitude < {P6}"
endif
set/graph DEFAULT


BRANCH mode 0,1,2 INP,INP,RES

INP:                                                    !plot input data
  assign/graph G,{P5}
  set/graph default                                     !reset graphics
  set/graph pmode=0
  set/graph colour=1 ltype=0 lwidth=2 stype=8 cleargra=on !set graphics specif.
                                                        !plot table
  set/format F15.8
  set/graph xaxis={ramax},{ramin}
  set/format F4.1
  plot/table {P1} :{c_rect} :{c_decl} -{wid},-{hig},{xof},{yof}
  label/graph "STANDARD STARS [{P2} filter]" {x1},{x2},mm ? 1 0
  x1 =  wid + 2.0 * xsc
  x2 = 3 * ysc
  label/graph "RA-offset=+{iramin}" {x1},{x2},mm 0 0.95 1
  if mode .eq. 0 goto RES
  goto fini

RES:
  compute/table {P1} :{c_rect}_mea = :{c_rect}_mea - {iramin}
  set/graph colour=2 ltype=0 lwidth=3 stype=3 cleargra=on
  overplot/table {P1} :{c_rect}_mea :{c_decl}_mea               !plot result
  compute/table {P1} :{c_rect}_mea = :{c_rect}_mea + {iramin}
  mess/out 4 plt_align "PLOT: Coincidences are marked by squares"

  inputc = "Processed by IMG_CAL_ALIGN"
  @p fors_plt_align,PROC_LABEL {xof} {yof} {ysc}

  x1 = wid + 2.0 * xsc
  x2 = 12.0 * ysc
  if nrow .gt. mxrow then
    nrow = mxrow
    label/graph ":" {x1},{x2},mm 0 0.95 1
    x2 = x2 + 4 * ysc
  endif
  do i1 = 1 {nrow}
    name = "{{P1},:OBJECT,{i1}}"
    label/graph "{name}" {x1},{x2},mm 0 0.95 1
    x2 = x2 + 4 * ysc
  enddo

  outputc = m$upper(P4(1:1))
  if outputc(1:1) .eq. "Y" then
    mess/out 3 plt_align "Create hardcopy: align{P2}.ps ..."
    copy/graph postscript
    -rename postscript.ps align{P2}.ps
  endif
fini:

compute/table {P1} :RA = :RA + {iramin}
set/graph DEFAULT
set/format


return 0 {iramin}



ENTRY PROC_LABEL
define/param p1 ? n "Enter x-offset of diagram (see plot/table):"
define/param p2 ? n "Enter x-offset of diagram (see plot/table):"
define/param p3 ? n "Enter character scaling factor:"

define/local x1/r/1/1 0.0
define/local x2/r/1/1 0.0

x1 = {P1} - 1
x2 = {P2} - 1

label/graph "{inputc}" -{x1},-{x2},mm 0 {P3} 1

