! $Id: fors_plt_extc.prg,v 1.9 1999/11/25 11:24:04 sbogun Exp $   
!+++ FORS_PLT_EXTC ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_plt_extc.prg,v $
!
!.KEYWORDS      Plot fit to extract extinction coefficient 
!
!.USAGE         @p fors_plt_extc photom extco mode winnum
!
!.INPUT         photom  (P1): name of aligned photometry table
!               extco   (P2): name of extinction coefficient table
!               mode    (P3): mode used to determine extinction coefficients
!               winnum  (P4): graphics window number
!               psname  (P5): create a POSTSCRIPT hard copy
!                               *: take default file name
!                               +: do not make a hard copy
!                               <name> take <name> as PS-filename.
!
!.OUTPUT        graphic
!
!.RETURN        
!
!.DEFAULT       P1: photom.tbl
!               P2: extcoeff.tbl
!               P3: ROB
!               P4: 0
!               P5: +
!
!.PURPOSE       Create a graphics plot of the linear fit to the photometry
!
!.ALGORITHM     
!
!.ENVIRON       FORS context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/10/29  Creation
!               1.2 1999/02/26  Hardcopy handling improved!
!
!.COMMENT
!------------------------------------------------------------------------------

define/param p1 photom.tbl c "Enter name of aligned photometry table: "
define/param p2 extcoeff.tbl c "Name of extinction table: "
define/param p3 ROB c "Linear fit mode: LSQ, ROB, MVR, HOU:"
define/param p4 0 n "Graphics window number: "
define/param p5 + c "Enter name of POSTSCRIPT file (+ --> no hardcopy):"

define/local refcol/c/1/1 U
define/local c_magx/c/1/10 " " ALL
define/local extco/r/1/1 0.0
define/local zerop/r/1/1 0.0

if nice_crea .eq. "n" return                            !GRAPHIC disabled

config/filter {P1}                                      !get filter
refcol = fors_filti(1:1)                                !remember filter
c_magx = "{c_mag}_{refcol}"
zerop = m$value({P2},ZEROPOINT,1)
extco = m$value({P2},EXTCOEF,1)
if P5(1:1) .eq. "+" then                                !check if hardcopy
  define/local psname/c/1/80 none                       !to POSTSCRIPT file
else if P5(1:1) .eq. "*" then                           !is required
  define/local psname/c/1/80 extcoeff{refcol}.ps        !.. use default name
else                                                    !
  define/local psname/c/1/80 P5                         !.. use user-defined
endif                                                   !PS-filename.


name/column {p1} :{c_magx} :{c_magx}_LIT                !renamed temporarily

plot/robl {p1} :{c_airm} :{c_magx}_LIT {zerop} {extco} -
  "Extinction Coefficient [{refcol}] - {P3}-fit" {psname} {p4}

name/column {p1} :{c_magx}_LIT :{c_magx}
