! $Id: fors_plt_oversc.prg,v 1.17 1999/11/25 11:24:04 sbogun Exp $   
!+++ FORS_PLT_OVERSC ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_plt_oversc.prg,v $
!
!.KEYWORDS      Creates overscan plots
!
!.USAGE         @p fors_plt_oversc in_tbl xax title winid psnam quadr
!
!.INPUT         in_frm  (P1): name of input table to be plotted
!               xax     (P2): name of x-axis table column
!               title   (P3): titel of the plot
!               winid   (P4): graphics window id
!               hcopy   (P5): YES: do a hard copy, NO: do not.
!               quadr   (P6): current CCD-quadrant in work
!                               +--+--+
!                               |3 |4 |
!                               +--+--+
!                               |1 |2 |
!                               +--+--+
!
!.OUTPUT        
!
!.RETURN        
!
!.DEFAULT
!               P5: N
!
!.PURPOSE       Plot the results of the pre-/postscan calculations made
!               by the recipe fors_gen_qc0_overscan.prg
!
!.ALGORITHM     
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1998/02/04  Creation
!               1.0 1998/11/25  charactersize of labels scale automatically
!                               pmode set to 0 (no additional info on graph.)
!               1.2 1999/02/26  P5 changed: hard copy (=YES or NO)
!
!.COMMENT
!------------------------------------------------------------------------------
!
! Check if graphic enabled
!
if nice_crea .eq. "n" return    !GRAPHIC disabled: do nothing
!
! Check parameters
!
define/param p1 ? c "Enter name INPUT table:"
define/param p2 none c "Enter name of x-axis table-column:"
define/param p3 OVERSCAN c "Enter title of the plot:"
define/param p4 1 n "Enter window number (1 or 2):"
define/param p5 N c "Enter YES or NO for doing hard copies:"

define/local xsc/r/1/1 624.0
define/local ysc/r/1/1 445.0

define/local wid/r/1/1 1.0
define/local hig/r/1/1 1.0
define/local hig2/r/1/1 1.0
define/local hi2/r/1/1 1.0
define/local xof/r/1/1 0.0
define/local yof/r/1/1 0.0
define/local yof2/r/1/1 0.0
define/local x1/r/1/1 0.0
define/local x2/r/1/1 0.0
define/local sz/r/1/1 1.0
define/local quad/c*3/1/4 " " ALL
define/local text1/c/1/30 " " ALL
define/local text2/c/1/30 " " ALL
define/local text3/c/1/30 " " ALL

quad(1) = "A"
quad(2) = "B"
quad(3) = "C"
quad(4) = "D"

xsc = NICE_GDIM(1) / xsc
ysc = NICE_GDIM(2) / ysc


BRANCH {P4} 1,2 PLT1,PLT2

PLT1:
assign/graphics G,1
set/graph default
set/graph pmode=0
set/graph bcolour=0 colour=1 ltype=1 lwidth=2 stype=- cleargra=on
name/column {P1} :FLUX :COUNTRATE
text1 = "FIT TO AVERAGE COLUMN"
text2 = "OF OVERSCAN REGION"
text3 = "OUTPUT:"
goto PLTC

PLT2:
assign/graph G,2
set/graph default
set/graph pmode=0
set/graph bcolour=0 colour=1 ltype=1 lwidth=2 stype=5 cleargra=on
name/col {P1} :Y :COUNTRATE
text1 = "FIT TO AVERAGE ROW"
text2 = "OF OVERSCAN REGION"
text3 = "OUTPUT:"

PLTC:
wid = -120.0 * xsc
hig = -80.0 * ysc
xof = 25. * xsc
yof = 40. * ysc
name/column {P1} :X {P2}



plot/table {P1} {P2} :COUNTRATE {wid},{hig},{xof},{yof}         !plot data



x1 = 60.0 * xsc
x2 = 87.0 * ysc
label/graph {P3} {x1},{x2},mm ? 1 0                             !write title
set/graph bcolour=0 colour=2 ltype=1 lwidth=3 stype=0
overplot/table {P1} {P2} :FIT                                   !plot fit


hig2 = -15.0 * ysc
yof2 = 10. * ysc
set/graph bcolour=0 colour=1 cleargra=off pmode=0
plot/table {P1} {P2} :RESIDUAL {wid},{hig2},{xof},{yof2}        !plot residuals


x1 = -wid + 2 * xsc
x2 = -hig + yof - 12 * ysc
sz = 2.0 * ysc
label/graph "{text1}" {x1},{x2},mm ? {sz} 1                     !write info
x2 = x2 - sz * 1.5
label/graph "{text2}" {x1},{x2},mm ? {sz} 1                     !write info


set/graph bcolour=0 colour=1 cleargra=on lwidth=2 pmode=2
wid = 15.0 * xsc                                                !prepare
hig = 15.0 * ysc                                                !positions for
hi2 = hig / 2.0                                                 !CCD-port-label
x1 = 122.0 * xsc
x2 = x1 + wid
!
! CCD-port-label:
!
if {P6} .gt. 0 then
  overplot/line 1 {x1},0.,mm  {x2},0.,mm
  overplot/line 1 {x1},{hi2},mm {x2},{hi2},mm
  overplot/line 1 {x1},{hig},mm {x2},{hig},mm
  overplot/line 1 {x1},0.,mm  {x1},{hig},mm
  x1 = x1 + wid / 2.0
  overplot/line 1 {x1},0.,mm  {x1},{hig},mm
  overplot/line 1 {x2},0.,mm  {x2},{hig},mm

  x1 = x1 - wid / 2.0
  x2 = 17.0 * ysc
  sz = 2*{ysc}
  label/graph "4-ports:" {x1},{x2},mm ? {sz} 1

  x1 = x1 + wid / 4.0
  x2 = x1 + wid / 2.0
  define/local xlab/r/1/4 {x1},{x2},{x1},{x2}
  x1 = hig / 4.0
  x2 = x1 + hi2
  define/local ylab/r/1/4 {x1},{x1},{x2},{x2}
  label/graph {quad(1)} {xlab(1)},{ylab(1)},mm ? 3 0
  label/graph {quad(2)} {xlab(2)},{ylab(2)},mm ? 3 0
  label/graph {quad(3)} {xlab(3)},{ylab(3)},mm ? 3 0
  label/graph {quad(4)} {xlab(4)},{ylab(4)},mm ? 3 0
  set/graph twidth=2
  label/graph {quad({P6})} {xlab({P6})},{ylab({P6})},mm ? 3 0
  set/graph twidth=1
else
  overplot/line 1 {x1},0.,mm  {x2},0.,mm
  overplot/line 1 {x1},{hig},mm {x2},{hig},mm
  overplot/line 1 {x1},0.,mm  {x1},{hig},mm
  overplot/line 1 {x2},0.,mm  {x2},{hig},mm

  x2 = 17.0 * ysc
  sz = 2*{ysc}
  label/gra "1-port:" {x1},{x2},mm ? {sz} 1

  set/graph twidth=2
  x1 = x1 + wid / 2.0
  label/graph A {x1},{hi2},mm ? 3 0
  set/graph twidth=1
endif

if P5(1:1) .ne. "N" then
  mess/out 3 overscplt "Create hardcopy: {P1}.ps"
  copy/gra postscript
  -rename postscript.ps {P1}.ps
endif

set/graph tsize=2 colour=2
yof = 5
label/graph "Processed by GEN_QC0_OVERSCAN" -{xof},-{yof},mm 0 {ysc} 1
set/graph default
