! $Id: fors_preparesea.prg,v 1.13 2001/01/19 14:37:35 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_preparesea.prg,v $
!
!.KEYWORDS    Preparation of emission line search, spectroscopy
!
!.USAGE       prepare/search wfrm sfrm method fsmooth threshold [fgauss]
!                            [slits]
!
!.INPUT       wfrm        (P1)    Wavelength calibration frame
!             method      (P3)    Method of background subtraction
!             metpar      (P4)    Method parameter: Threshold for cutting of
!                                 emission lines for method smooth and order
!                                 of spline fit and smoothing value for method
!                                 fit. 
!             fsmooth     (P5)    Filter size for smooth filter in units of
!                                 the slit width.
!             fgauss      (P6)    Filter size for optional gaussian filter
!                                 and sigma of the gaussian in terms of the
!                                 slit width.
!             slits       (P7)    Input slit position table
!
!.OUTPUT      sfrm        (P2)    Preprocessed calibration frame used
!                                 for emission line search.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. The approximate offset between
!             the original image and the preprocessed version is written
!             to OUTPUTR(1).
!
!.PURPOSE     The procedure provides two methods for removing the background
!             emission from a calibration frame. For method SMOOTH the median
!             of the calibration frame is calculated and all pixels with an
!             intensity exceeding the limit given by threshold times median
!             are replaced by this limit. The resulting frame is filtered
!             along the x direction with a smoothing filter to get rid of the
!             residual emission lines and to obtain an estimation of the
!             background. Optionally, i.e. if the parameters for the gaussian
!             filter are present, the already smoothed image is filtered once
!             more with a gaussian filter.
!             For method FIT local minima are searched in the averaged and
!             median filtered frame. The minima are fitted by a spline to
!             obtain an approximation of the background emission.
!             Finally the smoothed background frame is subtracted from the
!             calibration frame and all pixels with negative Intensities 
!             are replaced by 0. The output frame is normalized to the
!             exposure time.
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/02/28:  Creation
!
!------------------------------------------------------------------------------

crossref  wfrm sfrm method metpar fsmooth fgauss slits

define/param p1 ? c "Enter calibration frame:"
define/param p2 ? c "Enter result frame name:"
define/param p3 + c "Enter method:"
define/param p4 + c "Enter method parameters:"
define/param p5 + c "Enter size of smoothing filter:"
define/param p6 + c "Enter parameters for gaussian filter:"
define/param p7 + c "Enter input slit table:"
define/maxpar 7

! Procedure identification
define/local procID/c/1/80      "PrepareSearch"

! Local constants
define/local smmin/r/1/1     3
define/local gmin/r/1/2      3,1
define/local avrfrm/c/1/20   "tmp_avrwlc.bdf"
define/local conttbl/c/1/20  "tmp_cont.tbl"
define/local tmp0/c/1/12     "middumma.bdf"
define/local tmp1/c/1/12     "middummb.bdf"
define/local tmp2/c/1/12     "middummc.bdf"
define/local tmptab2/c/1/12  "middumma.tbl"
define/local tmp_h_slwidth/c/1/60   " "

! Local variables
define/local binx/i/1/1      1
define/local biny/i/1/1      1
define/local pixscale/r/1/1  0.
define/local pos/i/1/1       0
define/local act_slit/i/1/1  0
define/local gflag/i/1/1     0
define/local fynpix/i/1/1    0
define/local spord/i/1/1     1
define/local smwidth1/r/1/1  0.
define/local smwidth/r/1/1   0.
define/local slitwidth/r/1/1 0.
define/local gwidth/r/1/1    0.
define/local gsigma/r/1/1    0.
define/local stab/c/1/80     " "  all
define/local factor/r/1/1    1.
define/local spsmooth/r/1/1  1.
define/local exptime/r/1/1   0.
define/local limit/r/1/1     0.
define/local fystart/d/1/1   0.
define/local fystep/d/1/1    0.
define/local method/c/1/3    " "  all
define/local obsmode/c/1/10  " "  all
define/local wfrm/c/1/100   {p1}
define/local sfrm/c/1/100   {p2}


define/local xs/i/1/1       0
define/local xe/i/1/1       0
define/local ys/i/1/1       0
define/local ye/i/1/1       0

! Process parameters
if p3(1:1) .ne. "+" then
  method = m$upper(p3)
else
  method = m$upper(wlcsmmeth)
endif

if method(1:3) .eq. "SMO" then
  if p4(1:1) .eq. "+" then
    factor = wlcsmpar(1)
  else
    pos = m$index(p4,",") - 1
    if m$tstno(p4(1:{pos}) .ne. 1 then
      mess/out 1 {procID} "Invalid threshold value given!"
      return 1
    else
      factor = {p4(1:{pos})}
   endif
  endif
else
  pos = m$index(p4,",") - 1 
  if pos .lt. 0 then
    spord = m$nint({p4})
    spsmooth = wlcsmpar(2)
  else if pos .eq. 0 then
    spord = m$nint(wlcsmpar(1))
    spsmooth = {p4(2:)}
  else
    spord = m$nint({p4(1:{pos})})
    pos = pos + 2
    spsmooth = {p4({pos}:)}
  endif
endif

if p5(1:1) .eq. "+" then
  smwidth1 = wlcsmooth(1)
else
  if m$tstno(p5) .ne. 1 then
    mess/out 1 {procID} "Invalid smoothing factor given!"
    return 1
  else
    smwidth1 = {p5}
  endif
endif

if method(1:3) .eq. "SMO" then
  if p6(1:1) .ne. "+" then
    pos = m$index(p6,",") - 1
    if pos .lt. 0 then
      gwidth = {p6}
      gsigma = wlcsmooth(4)
    else if pos .eq. 0 then
      gwidth = wlcsmooth(3)
      gsigma = {p6(2:)}
    else
      gwidth = {p6(1:{pos})}
      pos = pos + 2
      gsigma = {p6({pos}:)}
    endif

    if gwidth .lt. 0 .or. gsigma .lt. 0 then
      gflag = 0
    else
      gflag = 1
    endif
  endif
endif

if p7 .ne. "+" then
  stab = p7
else
  stab = mos
endif


if m$existd(wfrm,h_xwinbin) .ne. 1 then
  mess/out 1 {procID} -
       "Assuming binx  = 1 because descriptor {h_xwinbin} is missing in {wfrm}!"
  binx = 1
else
  binx = m$value({wfrm},{h_xwinbin})
endif

if m$existd(wfrm,h_ywinbin) .ne. 1 then
  mess/out 1 {procID} -
       "Assuming biny  = 1 because descriptor {h_ywinbin} is missing in {wfrm}!"
  biny = 1
else
  biny     = m$value({wfrm},{h_ywinbin})
endif

pixscale = m$value({wfrm},{h_pixscale})
obsmode  = m$value({wfrm},{h_insmode})
exptime  = m$value({wfrm},O_TIME(7))
compute/image {tmp2} = {wfrm} / {exptime}

xs =  1
xe =  {{tmp2},NPIX(1)}  
if obsmode .EQ. p_modes(3) create/image {sfrm} = {wfrm} poly 0.

do islit = 1 {nslit} 1

if obsmode .EQ. p_modes(2) then               
! if "LSS" ....
 -copy {tmp2} {tmp0}
  tmp_h_slwidth = h_slwidth
  slitwidth = m$value({wfrm},{tmp_h_slwidth})
  nslit = 1
else if obsmode .EQ. p_modes(3) then          
! if "MOS" ....
  set/format i1 
  act_slit = {{stab},:{c_slsno},@{islit}} ! slitlet index may differ from loop
                                          ! counter
  tmp_h_slwidth = h_moswid({act_slit})    ! slitwidth in arcsec
  slitwidth = m$value({wfrm},{tmp_h_slwidth})
  ys = {{stab},:ystart,@{islit}}          ! CCD world coordinates
  ye = {{stab},:yend,@{islit}}            !
  extract/image {tmp0} = {tmp2} [<,{ys}:>,{ye}]
else
  mess/out 1 {procID} "Unexpected instrument mode in {wlc}: {obsmode} !" 
  return 1
endif

! Calculate actual smoothing parameter
smwidth = 0.5 * {smwidth1} * slitwidth / pixscale

set/format i1 f1.2
mess/out 4 {procID} "  Half width of smoothing filter: {smwidth} pxl."
if smwidth .lt. smmin then
  mess/out 2 {procID} "Actual smoothing paramerter too low ({smwidth} pxl)."
  mess/out 2 {procID} "Replaced by minimum value {smmin} pxl!"

  smwidth = smmin
endif
set/format


if method(1:3) .eq. "SMO" then

! Calculate actual cut parameter
  statistics/image {tmp0} option=XN
  limit = factor * outputr(8)

  set/format i1 f1.2
  mess/out 4 {procID} "  Exposure time: {exptime} s."
  mess/out 4 {procID} "  Cut level used: {limit} adu/s."
  set/format

! setup gaussian filter if necessary
  if gflag .eq. 1 then
    gwidth = 0.5 * gwidth * slitwidth / pixscale
    gsigma = gsigma * slitwidth / pixscale

    set/format i1 f1.2
    mess/out 4 {procID} "  Half width of gauss filter: {gwidth} pxl."
    mess/out 4 {procID} "  Sigma of gaussian: {gsigma} pxl."
    set/format

    if gwidth .lt. gmin(1) then
      set/format i1 f1.2
      mess/out 2 {procID} "Actual gaussian filter size too low ({gwidth} pxl)."
      mess/out 2 {procID} "Replaced by minimum value {gmin(1)} pxl!"
      set/format

      gwidth = gmin(1)
    endif

    if gsigma .lt. gmin(2) then
      set/format i1 f1.2
      mess/out 2 {procID} "Actual gaussian sigma is too small ({gsigma} pxl)."
      mess/out 2 {procID} "replaced by minimum value {gmin(2)} pxl!"
      set/format

      gsigma = gmin(2)
    endif
  endif


! Cut off emission lines
  replace/image {tmp0} {tmp0} {limit},>={limit} >Null


! Smooth frame
  filter/smooth {tmp0} {tmp1} {smwidth},0,0. ya >Null

  if gflag .eq. 1 then
    filter/gauss {tmp1} {tmp0} {gwidth},0 {gwidth},{gsigma} >Null
  else 
    -copy {tmp1} {tmp0}
  endif
else
  if spord .gt. 5 then
    mess/out 1 {procID} "Order of spline is out of range. Maximum order is 5!"
    return 1
  endif


! Get input frame dimensions along y axis
  fystart = m$value({tmp0},start(2))
  fystep  = m$value({tmp0},step(2))
  fynpix  = m$value({tmp0},npix(2))

! Average frame
  average/row {avrfrm} = {tmp0} <,>

! Selection of continuum points
  if nslit .eq. 1 then
    mess/out 3 {procID} "Selecting continuum points."
  else
    set/format I2
    mess/out 3 {procID} "Slit #{act_slit}: Selecting continuum points."
    set/format
  endif
  filter/median {avrfrm} {tmp0} {smwidth} >Null
  filter/smooth {tmp0} {tmp1} {smwidth} >Null

  in_a  = tmp1
  in_b  = "MIN"
  out_a = tmptab2

  run STD_EXE:localmax


! Interpolate by a smoothing spline
  copy/it {avrfrm} {conttbl} :x
  name/column {conttbl} #2 :flux
  compute/table {tmptab2} :v = 0.

  interpolate/tt {conttbl} :x,:flux {tmptab2} :x,:value {spsmooth} {spord} >Null

  delete/column {conttbl} :x
  copy/ti {conttbl} {tmp1}
  copy/dd {avrfrm} * {tmp1}

! Expand approximated continuum to full size
  grow/image {tmp0} = {tmp1} {fystart},{fystep},{fynpix}
!  copy/dd {wfrm} * {tmp0}
endif


! Remove background

compute/image {tmp1} = {tmp2} - {tmp0}
statistics/image {tmp0} option=XN
outputr(1) = outputr(8)

replace/image {tmp1} {tmp0} <,0.=0. >Null
{tmp0},LHCUTS(3) = 0.

if obsmode .eq. p_modes(3) then
! mos
  insert/image {tmp0} {sfrm}
else
  -copy {tmp0} {sfrm}
endif

enddo

copy/dd {wfrm} *,3 {sfrm}


! Cleanup
if method(1:3) .eq. "FIT" then
  -delete {tmptab2}
  -delete {conttbl}
  -delete {avrfrm}
endif

-delete {tmp0}
-delete {tmp1}


return 0
