! $Id: fors_prohead.prg,v 1.2 2001/05/08 16:14:59 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_prohead.prg,v $
!
!.KEYWORDS    Write a descriptor into a temporary file 
!
!.USAGE       prohead/out mode keyname keyvalue keycomment
!
!.INPUT       mode       (P1): 1: open and add descriptor to temporary file
!                              0: copy descriptors in temporary file to 
!                                 all products. P2 must contain the product catalog
!                                 in this case.
!                                
!             keyname    (P2): name of descriptor to add. If mode=-1 P2 must
!                              contain the product catalog.
!
!             keyvalue   (P3): value of P2 to add. Can be omitted if P1=0.
!
!             keycomment (P4): comment for P2 to add. Can be omitted if P1=0.
!
!             keytype    (P5): type of P2. Can be omitted if P1=0.
!                              1: if type = integer
!                              2: if type = float
!                              3: if type = string
!
!.OUTPUT      none.
!
!.RETURNS     The procedure returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status.
!
!.PURPOSE     prohead/out is used to prepare a FITS keyword (MIDAS descriptor)
!             block that has to be added after the processing to all product frames.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      S. Bogun, ESO-DMO
!
!.VERSION     Changes:
!             $Log: fors_prohead.prg,v $
!             Revision 1.2  2001/05/08 16:14:59  sbogun
!             Modifications according to new product header
!             creation scheme.
!
!
!------------------------------------------------------------------------------

define/param P1 ? i "Enter mode:"
define/param P2 + c "Enter descriptor name (catalog name when P1=-1):"
define/param P3 + c "Enter descriptor value:"
define/param P4 + c "Enter descriptor comment:"
define/param P5 0 i "Enter descriptor type:"


! Procedure identification
define/local procid/c/1/80  "prohead"

define/local mode/i/1/1 {P1}
define/local keyname/c/1/80 {P2}
define/local keyvalue/c/1/80 {P3}
define/local keycomment/c/1/80 {P4}
define/local keytype/i/1/1 {P5}
define/local keylen/i/1/1 0
define/local fid/i/1/2 0 ALL
define/local midas_file/c/1/80 " " all
define/local catal/i/1/1 0


set/format I1


branch mode 0,1 CLO,ADD

mess/out 1 {procid} "Wrong mode selected (0,1 are allowed) !"
return 1

CLO:
  if m$exist(prohead_file) .eq. 0 then
    mess/out 2 {procid} "There's no descriptors to write..."
    return 1
  endif

  if m$index(keyname,".cat") .ne. 0 then
    outputi = m$len(keyname)
    out_a = keyname(1:{outputi(1)})
    if m$exist(out_a) .ne. 1 then
      mess/out 1 {procid} "{out_a} is not an existing catalog!"
      return 1
    endif
CAT_LOOP:
    store/frame midas_file {keyname} 1 FINI_LOOP
    write/descr {midas_file} <{prohead_file}
    @p fors_dummy ! Needed due to a bug in write/descr
    goto CAT_LOOP
FINI_LOOP:
  else
    mess/out 1 {procid} "{keyname} is not an existing catalog!"
    return 1
  endif

!  -delete {prohead_file}
  goto FIN   
 
ADD: 
  if m$exist(prohead_file) .eq. 1 then
    open/file {prohead_file} APPEND fid
  else
    mess/out 4 {procid} "Creating temporary header file {prohead_file}"
    open/file {prohead_file} WRITE fid    
  endif
  if fid(1) .le. 0 then
    mess/out 1 {procid} "Opening temporary header file {prohead_file} failed!"
    return 1
  endif

  keylen = m$strlen(keyvalue)

  if keyname .eq. "COMMENT" then
    if keylen .gt. 0 then
      write/file {fid(1)} {keyname}/c/-1/{keylen} "{keyvalue}"
    else
      write/file {fid(1)} {keyname}/c/-1/1 " "
    endif
  else
    if keylen .gt. 0 then
      if keytype .eq. 1 then
        write/file {fid(1)} {keyname}/i/1/1 {keyvalue}
      else if keytype .eq. 2 then
        write/file {fid(1)} {keyname}/d/1/1 {keyvalue}
      else if keytype .eq. 3 then
        write/file {fid(1)} {keyname}/c/1/{keylen} "{keyvalue}"
      else
        mess/out 1 {procid} "Unsupported Keyword type!"
      endif
    else
      write/file {fid(1)} {keyname}/c/1/1 " "
    endif
  endif

  if keycomment(1:1) .ne. "+" then
    write/file {fid(1)} {keyname}/h/1/40 "{keycomment}"
  endif

  close/file {fid(1)}

FIN:
return 0


 




