! $Id: fors_proheadinit.prg,v 1.10 2001/06/15 14:18:37 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_proheadinit.prg,v $
!
!.KEYWORDS    Product header initialization
!
!.USAGE       init/prohead frm_set ref_set lmodedesc param1 param2 param3 param4 -
!                          param5
!
!.INPUT       frm_set     (P1)   Name of ASCII file containing the raw frames
!             ref_set     (P2)   Name of ASCII file containing the reference 
!                                frames
!             lmodedesc   (P3)   Comma-separated list of two parameters:
!                                  lmode:
!                                    2: Longlog is not initialized, if procedure
!                                       is called within a super recipe.
!                                    1: Longlog is always initialized.
!                                  desc:
!                                    Text describing the function of a recipe 
!             param1      (P4)   First parameter of recipe
!             param2      (P5)   Second parameter of recipe
!             param3      (P6)   Third parameter of recipe
!             param4      (P7)   Fourth parameter of recipe
!             param5      (P8)   Fifth parameter of recipe
!
!.OUTPUT      none.
!
!.RETURNS     The procedure returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status.
!
!.PURPOSE     The procedure creates a standard data product header and writes
!             it to a temporary file (by calling prohead/out).
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      S. Bogun, ESO-DMO
!
!.VERSION     Changes:
!             $Log: fors_proheadinit.prg,v $
!             Revision 1.10  2001/06/15 14:18:37  sbogun
!             Copying of MD5 signature.
!             Standard PRO keywords moved to set_of_frames.prg
!
!             Revision 1.9  2001/05/17 09:01:12  sbogun
!             Added DATAMD5 keywords.
!
!             Revision 1.8  2001/05/09 15:58:09  sbogun
!             Correct creation of FITS keyword also if parameter is
!             a comma separated list.
!
!             Revision 1.7  2001/05/08 16:14:59  sbogun
!             Modifications according to new product header
!             creation scheme.
!
!
!------------------------------------------------------------------------------

define/param p1 ?  c    "Enter raw frame set file:"
define/param p2 ?  c    "Enter reference frame set file:"
define/param p3 ?  c    "Enter lmode/description"
define/param p4 +  c    "Enter first parameter of recipe:"
define/param p5 +  c    "Enter second parameter of recipe:"
define/param p6 +  c    "Enter third parameter of recipe:"
define/param p7 +  c    "Enter fourth parameter of recipe:"
define/param p8 +  c    "Enter fifth parameter of recipe:"


! Procedure identification
define/local procid/c/1/80  "ProHeadInit"


! Local constants
define/local frm_set/c/1/80 {P1}
define/local ref_set/c/1/80 {P2}
define/local param1/c/1/80  {P4}
define/local param2/c/1/80  {P5}
define/local param3/c/1/80  {P6}
define/local param4/c/1/80  {P7}
define/local param5/c/1/80  {P8}


! Parsing of P3
outputi = m$parse(P3,"parsing")
if outputi(1) .ne. 2 then
  mess/out 1 {procid} "Wrong no. of elements in P3 !"
  return 1
endif

define/local lmode/i/1/1 {parsing01}
define/local desc/c/1/72 "{parsing02}"


define/local filename/c*80/1/99 " "           all  ! To hold all filenames
define/local fullname/c*80/1/99 " "           all  ! To hold all full filenames
define/local doclass/c*25/1/99  " "           all  ! To hold all DO classes
define/local fname/c/1/128      " "           all
define/local paramkey/c*6/1/3   " "           all
define/local fc/i/1/2           0,0
define/local separ/c/1/72       "-"           all
define/local i/i/1/1            1
define/local ninput/i/1/1       0                  ! Number of input frames
define/local nrefer/i/1/1       0                  ! Number of reference frames
define/local ntotal/i/1/1       0                  ! Total number of frames
define/local kname/c/1/72       " "           all
define/local kvalue/c/1/72      " "           all
define/local kdesc/c/1/72       " "           all

paramkey(1) = "ivalue"
paramkey(2) = "dvalue"
paramkey(3) = "svalue"

! return if lmode=2 and procedure invoked within a superrecipe
if lmode .eq. 2 .and. pipeline(6) .eq. 1 then
  return 0
endif


! First use, delete file
-delete {prohead_file}


! Standard keys
define/local h_start_date/c/1/80   "eso.pro.{reci}.start"
define/local v_start_date/c/1/80   "{p_startdate}"
define/local i_start_date/c/1/80   "Date when recipe execution started"


prohead/out 1 {h_start_date} {v_start_date} "{i_start_date}" 3


set/format I1


! Retrieve file names and do classifications of input frames
open/file {frm_set} READ fc
if fc(1) .lt. 0 then
  mess/out 1 {procid} "Could not open input SetOfFrames {frm_set}!"
  return 1
endif
READ_LOOPI:
  read/file {fc(1)} fname
  if fc(2) .lt. 0 goto READ_FINII
  if m$len(fname) .eq. 0 goto READ_LOOPI
  ntotal = ntotal+1
  outputi(1) = m$index(fname," ")-1
  outputi(2) = m$index(fname," ")+1

  if outputi(1) .eq. -1 then
    mess/out 1 {procid} "SetOfFrames {frm_set} has an incorrect format!"
    return 1
  endif
  
  doclass({ntotal})  = fname({outputi(2)}:)
  out_a = fname(1:{outputi(1)})
  outputi = m$indexb(out_a,"/")+1
  filename({ntotal}) = out_a({outputi(1)}:)
  fullname({ntotal}) = out_a
  goto READ_LOOPI
READ_FINII:
close/file {fc(1)}
ninput = ntotal


! Retrieve file names and do classifications of reference frames
if ref_set .ne. "+" then
  open/file {ref_set} READ fc
  if fc(1) .lt. 0 then
    mess/out 1 {procid} "Could not open reference SetOfFrames {ref_set}!"
    return 1
  endif
READ_LOOPR:
    read/file {fc(1)} fname
    if fc(2) .lt. 0 goto READ_FINIR
    if m$len(fname) .eq. 0 goto READ_LOOPR
    ntotal = ntotal+1
    outputi(1) = m$index(fname," ")-1
    outputi(2) = m$index(fname," ")+1

    if outputi(1) .eq. -1 then
      mess/out 1 {procid} "SetOfFrames {ref_set} has an incorrect format!"
      return 1
    endif
  
  doclass({ntotal})  = fname({outputi(2)}:)
  out_a = fname(1:{outputi(1)})
  outputi = m$indexb(out_a,"/")+1
  filename({ntotal}) = out_a({outputi(1)}:)
  fullname({ntotal}) = out_a
  goto READ_LOOPR
READ_FINIR:
  close/file {fc(1)}
endif
nrefer = ntotal-ninput


! Write all input frame keywords
do i = 1 ninput
  kname  = "eso.pro.{reci}.raw{i}.name"
  kvalue = "{filename({i})}"
  kdesc  = "File name of raw frame #{i}"
  prohead/out 1 {kname} {kvalue} "{kdesc}" 3

  kname  = "eso.pro.{reci}.raw{i}.catg"
  kvalue = "{doclass({i})}"
  kdesc  = "Frame category of raw frame #{i}"
  prohead/out 1 {kname} {kvalue} "{kdesc}" 3
enddo


! Write all reference frame keywords
if nrefer .gt. 0 then 
  do i = 1 nrefer
    outputi = ninput+i
    kname  = "eso.pro.{reci}.cal{i}.name"
    kvalue = "{filename({outputi(1)})}"
    kdesc  = "File name of calibration frame #{i}"
    prohead/out 1 {kname} {kvalue} "{kdesc}" 3

    kname  = "eso.pro.{reci}.cal{i}.catg"
    kvalue = "{doclass({outputi(1)})}"
    kdesc  = "Frame category of calibration frame #{i}"
    prohead/out 1 {kname} {kvalue} "{kdesc}" 3

    if m$existd("{fullname({outputi(1)})}[0]",h_prodmd5) .eq. 1 then
      kname  = "eso.pro.{reci}.cal{i}.datamd5"
      copy/dk {fullname({outputi(1)})}[0] {h_prodmd5}/c/1/32 kvalue
      kdesc  = "MD5 data signature of calibration frame #{i}"
      prohead/out 1 {kname} {kvalue} "{kdesc}" 3
    endif
  enddo
endif



! Write all parameter keywords


if param1(1:1) .ne. "+" then
! Parsing of param1
  outputi = m$index(param1,",")
  if outputi(1) .lt. 2 .or. outputi(1) .ge. m$len(param1) then
    mess/out 1 {procid} "Wrong no. of elements in parameter 1!"
    return 1
  endif
  outputi(2) = outputi(1)-1
  outputi(3) = outputi(1)+1    
 
  kname  = "eso.pro.{reci}.param1.name"
  kvalue = "{param1(1:{outputi(2)})}"
  kdesc  = "Name of recipe parameter #1"
  prohead/out 1 {kname} {kvalue} "{kdesc}" 3

! Check type of parameter
  @p fors_proheadinit,get_type {param1({outputi(3)}:)}
  kname = "eso.pro.{reci}.param1.{paramkey({q1})}"
  kvalue = "{param1({outputi(3)}:)}"
  kdesc = "Value of recipe parameter #1"
  prohead/out 1 {kname} {kvalue} "{kdesc}" {q1}
endif


if param2(1:1) .ne. "+" then
! Parsing of param2
  outputi = m$index(param2,",")
  if outputi(1) .lt. 2 .or. outputi(1) .ge. m$len(param2) then
    mess/out 1 {procid} "Wrong no. of elements in parameter 1!"
    return 1
  endif
  outputi(2) = outputi(1)-1
  outputi(3) = outputi(1)+1    
 
  kname  = "eso.pro.{reci}.param2.name"
  kvalue = "{param2(1:{outputi(2)})}"
  kdesc  = "Name of recipe parameter #2"
  prohead/out 1 {kname} {kvalue} "{kdesc}" 3

! Check type of parameter
  @p fors_proheadinit,get_type {param2({outputi(3)}:)}
  kname = "eso.pro.{reci}.param2.{paramkey({q1})}"
  kvalue = "{param2({outputi(3)}:)}"
  kdesc = "Value of recipe parameter #2"
  prohead/out 1 {kname} {kvalue} "{kdesc}" {q1}
endif


if param3(1:1) .ne. "+" then
! Parsing of param3
  outputi = m$index(param3,",")
  if outputi(1) .lt. 2 .or. outputi(1) .ge. m$len(param3) then
    mess/out 1 {procid} "Wrong no. of elements in parameter 1!"
    return 1
  endif
  outputi(2) = outputi(1)-1
  outputi(3) = outputi(1)+1    
 
  kname  = "eso.pro.{reci}.param3.name"
  kvalue = "{param3(1:{outputi(2)})}"
  kdesc  = "Name of recipe parameter #3"
  prohead/out 1 {kname} {kvalue} "{kdesc}" 3

! Check type of parameter
  @p fors_proheadinit,get_type {param3({outputi(3)}:)}
  kname = "eso.pro.{reci}.param3.{paramkey({q1})}"
  kvalue = "{param3({outputi(3)}:)}"
  kdesc = "Value of recipe parameter #3"
  prohead/out 1 {kname} {kvalue} "{kdesc}" {q1}
endif


if param4(1:1) .ne. "+" then
! Parsing of param4
  outputi = m$index(param4,",")
  if outputi(1) .lt. 2 .or. outputi(1) .ge. m$len(param4) then
    mess/out 1 {procid} "Wrong no. of elements in parameter 1!"
    return 1
  endif
  outputi(2) = outputi(1)-1
  outputi(3) = outputi(1)+1    
 
  kname  = "eso.pro.{reci}.param4.name"
  kvalue = "{param4(1:{outputi(2)})}"
  kdesc  = "Name of recipe parameter #4"
  prohead/out 1 {kname} {kvalue} "{kdesc}" 3

! Check type of parameter
  @p fors_proheadinit,get_type {param4({outputi(3)}:)}
  kname = "eso.pro.{reci}.param4.{paramkey({q1})}"
  kvalue = "{param4({outputi(3)}:)}"
  kdesc = "Value of recipe parameter #4"
  prohead/out 1 {kname} {kvalue} "{kdesc}" {q1}
endif


if param5(1:1) .ne. "+" then
! Parsing of param5
  outputi = m$index(param5,",")
  if outputi(1) .lt. 2 .or. outputi(1) .ge. m$len(param5) then
    mess/out 1 {procid} "Wrong no. of elements in parameter 1!"
    return 1
  endif
  outputi(2) = outputi(1)-1
  outputi(3) = outputi(1)+1    
 
  kname  = "eso.pro.{reci}.param5.name"
  kvalue = "{param5(1:{outputi(2)})}"
  kdesc  = "Name of recipe parameter #5"
  prohead/out 1 {kname} {kvalue} "{kdesc}" 3

! Check type of parameter
  @p fors_proheadinit,get_type {param5({outputi(3)}:)}
  kname = "eso.pro.{reci}.param5.{paramkey({q1})}"
  kvalue = "{param5({outputi(3)}:)}"
  kdesc = "Value of recipe parameter #5"
  prohead/out 1 {kname} {kvalue} "{kdesc}" {q1}
endif


return 0



entry get_type
! Entry to check type of a keyword
! Returns Q1=1 if type = integer
!         Q1=2 if type = float
!         Q1=3 if type = string

define/parameter P1 ? c "Enter parameter to check:"

! Check for comma (m$tstno doesn't catch this)
outputi = m$index(p1,",")
if outputi(1) .ne. 0 then
  write/keyword q1 3
  return {q1}
endif

! Check if number
outputi = m$tstno(p1)

if outputi(1) .eq. 0 then
  write/keyword q1 3
else
  outputi = m$index(p1,".")
  if outputi(1) .eq. 0 then
    write/keyword q1 1
  else
    write/keyword q1 2
  endif
endif

return {q1}
