! $Id: fors_purge.prg,v 1.13 2000/05/04 19:25:42 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_purge.prg,v $
!
! .KEYWORDS   Descriptors
!
! .PURPOSE    Clearage of no longer needed descriptors in images or tables.
! __________
!
! .USAGE      @p fors_purge icat dtab [docl]
!
! .INPUT      icat (P1): image catalogue with any kind of images or tables;
!                          default: none
!
!             dtab (P2): table - one column with all descriptors, columns for
!                        all frame types with descriptors to be deleted marked
!                        AND name of column with descriptors, separated by a
!                        comma;
!                          default: none,DESCRIPTORS
!
!             docl (P3): name of descriptor with frame classification string;
!                          default: DO_CLASSIFICATION
!
! .OUTPUT     Q2: tot. number of frames which were skipped because assignment
!                 classification <-> table went wrong
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  All descriptors which are marked in the table (per frame type)
!             are removed from the headers.
!
! .COMMENT    -
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.3  99/04/28  Creation
!                  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref icat dtab docl
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "purge"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      icat (P1): image catalogue with any kind of images or tables;"
  w/o "                   default: none"
  w/o " "
  w/o "      dtab (P2): table - one column with all descriptors, columns for"
  w/o "                 all frame types with descriptors to be deleted marked"
  w/o "                 AND name of column with descriptors, separated by a"
  w/o "                 comma;"
  w/o "                   default: none,DESCRIPTORS"
  w/o " "
  w/o "      docl (P3): name of descriptor with frame classification string;"
  w/o "                   default: DO_CLASSIFICATION"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(icat):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter icat. with images or tables : "
define/local icat/c/1/64 "{p1}"
if m$index(icat,".cat") .eq. 0 icat = icat//".cat"
!
if m$exist(icat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
 goto error
endif
!
show/icat {icat} n
define/local totfrmno/i/1/1 {outputi(1)}  ! needed below, too
if totfrmno .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is empty !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
define/local catal/i/1/1 0
inloop:
  store/frame inputc {icat} 1 end_inloop
  if m$exist(inputc) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "At least one catalogue entry is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
    goto error
  endif
  goto inloop
end_inloop:
!
!
!P2(dtab):
!--------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter tbl.w.descs.p.frm.type,desc.col.[def.:none,DESCRIPTORS]: "
outputi = m$parse(p2,"tbl")
if outputi(1) .ne. 2 .and. outputi(1) .ne. 1 then  ! 1 allowed for col.default
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local dtab/c/1/64 "{tbl01}"
if m$index(dtab,".tbl") .eq. 0 dtab = dtab//".tbl"
if outputi(1) .eq. 2 then
  define/local cdesc/c/1/16 "{tbl02}"
  if cdesc(1:1) .eq. ":" cdesc = cdesc(2:)
elseif outputi(1) .eq. 1 then  ! default
  define/local cdesc/c/1/16 "DESCRIPTORS"
endif
!
if m$existc(dtab,":{cdesc}") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({dtab})"
  goto error
elseif m$existc(dtab,":{cdesc}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({dtab},:{cdesc})"
  goto error
endif
!
show/table {dtab} >Null
define/local totdescno/i/1/1 {outputi(2)}  ! needed below, too
if totdescno .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No descriptor specified in input table !"
        mess/out 1 {PROID} "({dtab})"
  goto error
endif
!
!
!P3(docl):
!--------
define/parameter p3 DO_CLASSIFICATION ? -
  "        Enter desc. with frame classification [def.:DO_CLASSIFICATION]: "
define/local docl/c/1/72 "{p3}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "L o o p" :
  define/local topicfrm/c/1/64 " " all
  define/local loopcount/i/1/1 0
  !
  !   "Classification" :
  define/local skip/i/1/1 0
  define/local class/c/1/48 " " all +lower_levels
  define/local pcol/c/1/48 " " all +lower_levels
  !
  !   "Deletion" :
  define/local mid$selidx/i/1/300 0 all +lower_levels
  define/local seldescs/i/1/1 0
  define/local descloop/i/1/1 0
!
!
!
!          L O O P   W I T H   F R A M E S :
!          * * * * * * * * * * * * * * * *
!
!
catal = 0
catloop:
  store/frame topicfrm {icat} 1 end_catloop
  loopcount = loopcount+1
        mess/out 4 {PROID} "purging header {loopcount} / {totfrmno} : {topicfrm}"
  !
  !
  !        CLASSIFICATION - DESCRIPTOR AND TABLE COLUMN TEST :
  !        *************************************************
  !
        mess/out 4 {PROID} "  checking frame classification"
  !
  if m$existd(topicfrm,docl) .eq. 0 then
        write/out " "
        mess/out 2 {PROID} "Descriptor missing ({topicfrm},{docl}) -"
        mess/out 2 {PROID} "can not clean {topicfrm} !"
    skip = skip+1
    warn = 1
    goto catloop
  endif
  write/key class " " all
  class = "{{topicfrm},{docl}}"
  !
  ! MIDAS TABLE COLUMN LABELS CAN NOT EXCEED 16 CHARACTERS:
  ! ------------------------------------------------------
  !
  @p fors_{PROID},label
  !
  if pcol(1:1) .eq. ":" pcol = pcol(2:)
  if m$existc(dtab,":{pcol}") .eq. -1 then
        write/out " "
        mess/out 2 {PROID} "Table column missing ({dtab},:{pcol}) -"
        mess/out 2 {PROID} "can not clean {topicfrm} !"
    skip = skip+1
    warn = 1
    goto catloop
  endif
  !
  !
  !        DELETION OF DESCRIPTORS BY MEANS OF TABLE :
  !        *****************************************
  !
        mess/out 4 {PROID} "  deleting unnecessary descriptors"
  !
  select/table {dtab} :{pcol}.eq.1 >Null
  seldescs = outputi(1)
  !
  if seldescs .gt. 0 then
    do descloop = 1 seldescs
      delete/descriptor {topicfrm} -
      {{dtab},:{cdesc},@{mid$selidx({descloop})}} no
    enddo
  endif
  !
  !
  goto catloop
end_catloop:
!
select/table {dtab} all >Null
!
!
!          E N D   O F   L O O P
!          * * * * * * * * * * *
!
!
!
set/format
!
!
if warn .eq. 0 then
  return 0 {skip}
else
  return -1 {skip}
endif
!
error:
  if m$existk("dtab") .eq. 1 then
    if m$exist(dtab) .eq. 1 select/table {dtab} all >Null
  endif
  set/format
  return 1 -99
!
!
!-------------------------------------------------------------------------------
!
entry label
!
  !_BASIC_
  if class .eq. "BAD_PIXEL_MAP"              pcol = "BAD_PIXEL_MAP"
  if class .eq. "BAD_PIXEL_TABLE"            pcol = "BAD_PIXEL_TABLE"
  if class .eq. "OVEREXPO_MASK"              pcol = "OVEREXPO_MASK"
  if class .eq. "OVEREXPO_TABLE"             pcol = "OVEREXPO_TABLE"
  if class .eq. "STATISTICS_TABLE"           pcol = "STATISTICS_TABLE"
  if class .eq. "OVERSCAN_TABLE"             pcol = "OVERSCAN_TABLE"
  if class .eq. "REDUCED_BIAS"               pcol = "REDUCED_BIAS"
  if class .eq. "MASTER_BIAS"                pcol = "MASTER_BIAS"
  if class .eq. "REDUCED_DARK"               pcol = "REDUCED_DARK"
  if class .eq. "MASTER_DARK"                pcol = "MASTER_DARK"
  if class .eq. "REDUCED_SCREEN_FLAT_IMG"    pcol = "RED_SCR_FLAT_IMG"
  if class .eq. "REDUCED_SCREEN_FLAT_LSS"    pcol = "RED_SCR_FLAT_LSS"
  if class .eq. "REDUCED_SCREEN_FLAT_MOS"    pcol = "RED_SCR_FLAT_MOS"
  if class .eq. "NORM_FLAT_LSS"              pcol = "NORM_FLAT_LSS"
  if class .eq. "NORM_FLAT_MOS"              pcol = "NORM_FLAT_MOS"
  if class .eq. "REDUCED_SKY_FLAT_IMG"       pcol = "RED_SKY_FLAT_IMG"
  if class .eq. "MASTER_SCREEN_FLAT_IMG"     pcol = "MST_SCR_FLAT_IMG"
  if class .eq. "MASTER_SCREEN_FLAT_LSS"     pcol = "MST_SCR_FLAT_LSS"
  if class .eq. "MASTER_SCREEN_FLAT_MOS"     pcol = "MST_SCR_FLAT_MOS"
  if class .eq. "MASTER_NORM_FLAT_LSS"       pcol = "MST_NRM_FLAT_LSS"
  if class .eq. "MASTER_NORM_FLAT_MOS"       pcol = "MST_NRM_FLAT_MOS"
  if class .eq. "MASTER_SKY_FLAT_IMG"        pcol = "MST_SKY_FLAT_IMG"
  if class .eq. "SUPER_SKY_FLAT_IMG"         pcol = "SUP_SKY_FLAT_IMG"
  if class .eq. "MASTER_NIGHT_FLAT_IMG"      pcol = "MST_NHT_FLAT_IMG"
  if class .eq. "SUPER_NIGHT_FLAT_IMG"       pcol = "SUP_NHT_FLAT_IMG"
  if class .eq. "CHKCAL_TABLE"               pcol = "CHKCAL_TABLE"
  if class .eq. "TSTMASTER_TABLE"            pcol = "TSTMASTER_TABLE"
  !
  !_PHOTOMETRY_
  if class .eq. "STANDARD_REDUCED_IMG"       pcol = "STD_REDUCED_IMG"
  if class .eq. "SCIENCE_REDUCED_IMG"        pcol = "SCI_REDUCED_IMG"
  if class .eq. "PHOTOMETRY_TABLE"           pcol = "PHOTOMETRY_TABLE"
  if class .eq. "PHOT_BACKGROUND_IMG"        pcol = "PHOT_BACKGR_IMG"
  if class .eq. "PHOT_BACKSUBTR_IMG"         pcol = "PHOT_BACKSUB_IMG"
  if class .eq. "ALIGNED_PHOTOMETRY_TABLE"   pcol = "ALIGNED_PHOT_TBL"
  if class .eq. "ZEROPOINT_TABLE_IMG"        pcol = "ZEROPNT_TBL_IMG"
  if class .eq. "EXTCOEFF_TABLE_IMG"         pcol = "EXTCOEFF_TBL_IMG"
  if class .eq. "COLCOEFF_TABLE_IMG"         pcol = "COLCOEFF_TBL_IMG"
  if class .eq. "COL_TRANS_FORM"             pcol = "COL_TRANS_FORM"
  if class .eq. "SCIENCE_FLXCAL_IMG"         pcol = "SCI_FLXCAL_IMG"
  if class .eq. "PHOTOMETRY_FLXCAL_TABLE"    pcol = "PHOT_FLXCAL_TBL"
  !
  !_SPECTROSCOPY_
  if class .eq. "DISP_COEFF_LSS"             pcol = "DISP_COEFF_LSS"
  if class .eq. "DISP_COEFF_MOS"             pcol = "DISP_COEFF_MOS"
  if class .eq. "REDUCED_LAMP_LSS"           pcol = "REDUCED_LAMP_LSS"
  if class .eq. "LINEPOS_LSS"                pcol = "LINEPOS_LSS"
  if class .eq. "LINEPOS_MOS"                pcol = "LINEPOS_MOS"
  if class .eq. "RESOL_TABLE"                pcol = "RESOL_TABLE"
  if class .eq. "STANDARD_REDUCED_LSS"       pcol = "STD_REDUCED_LSS"
  if class .eq. "STANDARD_REDUCED_MOS"       pcol = "STD_REDUCED_MOS"
  if class .eq. "STANDARD_SKY_LSS"           pcol = "STD_SKY_LSS"
  if class .eq. "STANDARD_SKY_MOS"           pcol = "STD_SKY_MOS"
  if class .eq. "STANDARD_EXTRACTED_LSS"     pcol = "STD_EXTRACT_LSS"
  if class .eq. "STANDARD_EXTRACTED_MOS"     pcol = "STD_EXTRACT_MOS"
  if class .eq. "STANDARD_SOURCE_LSS"        pcol = "STD_SOURCE_LSS"
  if class .eq. "STANDARD_SOURCE_MOS"        pcol = "STD_SOURCE_MOS"
  if class .eq. "CHKOBS_TABLE"               pcol = "CHKOBS_TABLE"
  if class .eq. "STANDARD_ALIGNED_LSS"       pcol = "STD_ALIGNED_LSS"
  if class .eq. "STANDARD_ALIGNED_MOS"       pcol = "STD_ALIGNED_MOS"
  if class .eq. "EXTCOEFF_TABLE_LSS"         pcol = "EXTCOEFF_TBL_LSS"
  if class .eq. "EXTCOEFF_TABLE_MOS"         pcol = "EXTCOEFF_TBL_MOS"
  if class .eq. "INSTR_RESPONSE_LSS"         pcol = "INSTR_RESPON_LSS"
  if class .eq. "INSTR_RESPONSE_MOS"         pcol = "INSTR_RESPON_MOS"
  if class .eq. "SCIENCE_REDUCED_LSS"        pcol = "SCI_REDUCED_LSS"
  if class .eq. "SCIENCE_REDUCED_MOS"        pcol = "SCI_REDUCED_MOS"
  if class .eq. "SCIENCE_FLXCAL_LSS"         pcol = "SCI_FLXCAL_LSS"
  if class .eq. "SCIENCE_FLXCAL_MOS"         pcol = "SCI_FLXCAL_MOS"
  !
  !_TECHNICAL_
  if class .eq. "FOCUS_TABLE"                pcol = "FOCUS_TABLE"
  if class .eq. "CCDCTE_TABLE"               pcol = "CCDCTE_TABLE"
  if class .eq. "LIN_GAIN_TABLE"             pcol = "LIN_GAIN_TABLE"
!
!
!-------------------------------------------------------------------------------
