! $Id: fors_putframes.prg,v 1.21 2001/06/27 15:46:29 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_putframes.prg,v $
!
!.KEYWORDS    Write result frames to master catalog/disk file.
!
!.USAGE       put/frames  catname froot [main] [cat_plot]
!
!.INPUT       catname     (P1)    Catalog containing result frames.
!             cat_plot    (P4)    Catalog of plot files (optional)
!
!.OUTPUT      froot       (P2)    Basename of all output files.
!             main        (P3)    Main pipeline product identifier
!
!.RETURNS     The recipe returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The procedure converts the frames stored in the catalog
!             'catname' from MIDAS internal data formats to FITS. The
!             descriptor given by the keyword FORS_IDSTRNG is removed from the
!             result frames before they are converted to FITS format. The
!             MIDAS file name of the main pipeline data product may be given,
!             so that it is the first product frame that is converted to FITS
!             format.
!             Additionally, all plot files in the catalog defined by P4
!             are copied to the product directory.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!             1.5  2000/04/14:  Plot catalog entries are copied, too (SB).
!
!------------------------------------------------------------------------------

crossref  results froot main cat_plot
define/param p1 ?  c    "Enter result frame catalog:"
define/param p2 ?  c    "Enter output file root:"
define/param p3 +  c    "Enter main data product name:"
define/param p4 +  c    "Catalog of plot files:"

define/maxpar 4


! Procedure identification
define/local procID/c/1/80      "PutFrames"


! Local constants
define/local maxchar/i/1/1   256
define/local failmsg/c/1/20  "NOT converted..."


! Local variables
define/local i/i/1/1                 0
define/local pos/i/1/1               0
define/local pos1/i/1/1              0
define/local pos2/i/1/1              0
define/local catal/i/1/1             0
define/local n_frames/i/1/1          0
define/local n_ok/i/1/1              0
define/local n_failed/i/1/1          0
define/local n_deliver/i/1/1         0
define/local fctrl/i/1/2             0     all
define/local cat_result/c/1/100      {p1}
define/local froot/c/1/100           {p2}
define/local mdpr/c/1/100            {p3}
define/local cat_plot/c/1/100        {p4}
define/local prolist/c/1/100         "products.lst"
define/local plotlist/c/1/100        "plotfiles.lst"
define/local fdir/c/1/100            " "   all
define/local cat_entry/c/1/80        " "   all
define/local class/c/1/30            " "   all
define/local errmsg/c/1/80           " "   all
define/local proinfo/c/1/{maxchar}   " "   all
define/local pipefile/c/1/{maxchar}  " "   all


! Get catalog information
show/icat {cat_result} N
n_frames = outputi(1)
if n_frames .eq. 0 return 1

! Update file header of output frames
do i = 1 n_frames 1
  store/frame cat_entry {cat_result} 1

  if m$existd(cat_entry,FORS_IDSTRNG) .eq. 1 then
    delete/descr {cat_entry} {FORS_IDSTRNG}
  endif

  if m$existd(cat_entry,H_FILENAME) .eq. 1 then
    delete/descr {cat_entry} {H_FILENAME}
  endif

! To satisfy DICB
  if m$existd(cat_entry,h_proqcsts) .eq. 1 then
    delete/descr {cat_entry} {h_proqcsts}
  endif

  if m$existd(cat_entry,h_proqccert) .eq. 1 then
    delete/descr {cat_entry} {h_proqccert}
  endif

! Write QC.DID
  read/desc {cat_entry} eso.qc.* >Null
  if outputi(1) .gt. 0 then
    write/desc {cat_entry} {h_proqcdid}/c/1/30 "{p_qcdic_version}"
    write/dhelp {cat_entry} {h_proqcdid} "{i_proqcdid}"
  endif
enddo

! Write output to disk
pos1 = m$indexb(froot,"/") - 1
fdir = froot(1:{pos1})

if mdpr(1:1) .eq. "+" then
  @d set_of_frames OUT {cat_result} {froot} + {prolist} >Null
else
  @d set_of_frames OUT {cat_result} {froot} {mdpr} {prolist} >Null
endif


errmsg = mid$errmess
n_ok = outputi(1)

if n_ok .eq. 0 then
  mess/out 1 {procID} "Cannot write product frames!"
  return 1
else if n_ok .lt. 0 then
  n_ok = m$abs(n_ok) - 1
  n_failed = n_frames - n_ok
endif



mess/out 3 {procID} " "
mess/out 3 {procID} "*** Processing Log ***"
mess/out 3 {procID} " "
mess/out 3 {procID} "Product directory:"
mess/out 3 {procID} "  {fdir}"
mess/out 3 {procID} " "


! Write out list of converted files
if m$exist(prolist) .eq. 1 then
  open/file {prolist} READ fctrl
  i = 0

  pro_loop:

  read/file {fctrl(1)} proinfo {maxchar}
  if fctrl(2) .ge. 0 then
    i = i + 1
    pos1 = m$index(proinfo," ") - 1
    pipefile = proinfo(1:{pos1})
    pos1 = pos1 + 2
    proinfo = proinfo({pos1}:)

    if m$indexb(proinfo,failmsg) .le. 0 then

      pos2 = m$indexb(proinfo,"/")
      if pos2 .gt. 0 then  
        pos2 = pos2 + 1
      endif

      if m$existd(pipefile,h_procatg) .eq. 1 then
        class = m$value({pipefile},{h_procatg})
        mess/out 3 {procID} "Frame {pipefile} ({class})"
        mess/out 3 {procID} "  saved as {proinfo({pos2}:)}"
      else
        mess/out 3 {procID} "Frame {pipefile}"
        mess/out 3 {procID} "  saved as {proinfo({pos2}:)}"
      endif

      if p_mainonly .gt. 0 .and. i .eq. 1 then
        mess/out 3 {procID} "  and delivered to DHS as main product"
        n_deliver = 1
      else if p_mainonly .eq. 0 then
        mess/out 3 {procID} "  and delivered to DHS"
        n_deliver = n_deliver + 1
      endif
    else
      if m$existd(pipefile,h_procatg) .eq. 1 then
        class = m$value({pipefile},{h_procatg})
        mess/out 3 {procID} -
                 "Frame {pipefile} ({class}) {proinfo}"
      else
        mess/out 3 {procID} -
                 "Frame {pipefile} ({class}) {proinfo}"
      endif
    endif
       
    goto pro_loop
  else
    close/file {fctrl(1)}
    if n_failed .gt. 0 then
      errmsg(1:1) = m$upper(errmsg(1:1))
      mess/out 1 {procID} "{errmsg}"
    endif
    mess/out 3 {procID} " "
  endif
else
  if n_ok .gt. 0 .and. p_mainonly .eq. 1 then
    n_deliver = 1
  else
    n_deliver = n_ok
  endif

  if n_failed .gt. 0 then
    errmsg(1:1) = m$upper(errmsg(1:1))
    mess/out 1 {procID} "{errmsg}"
    mess/out 3 {procID} " "
  endif
endif

set/format i1
mess/out 3 {procID} "{n_ok} out of {n_frames} frames successfully written!"


! If pipeline delivery information is available show number
! of frames delivered to DHS
if p_mainonly .ge. 0 then
  mess/out 3 {procID} "{n_deliver} frame(s) delivered to DHS!"
endif
set/format

if n_failed .gt. 0 then
  set/format i1
  mess/out 1 {procID} "{n_failed} frame(s) could not be saved!"
  set/format
  return 1
endif



! Write plot files to disk
if cat_plot(1:1) .ne. "+" then
  pos = m$indexb(cat_plot,".cat") - 1
  if pos .lt. 0 then
    cat_plot = cat_plot // ".cat"
  endif

  if m$exist(cat_plot) .ne. 0 then

! Get catalog information
    show/acat {cat_plot} N
    n_frames = outputi(1)
    if n_frames .eq. 0 return 0

    mess/out 3 {procID} " "
    mess/out 3 {procID} " "

    if mdpr(1:1) .eq. "+" then
      @d set_of_frames OUT {cat_plot} {froot} + {plotlist} >Null
    else
      @d set_of_frames OUT {cat_plot} {froot} {mdpr} {plotlist} >Null
    endif

    errmsg = mid$errmess
    n_ok = outputi(1)

    if n_ok .eq. 0 then
      mess/out 1 {procID} "Cannot write plot files!"
      return 1
    else if n_ok .lt. 0 then
      n_ok = m$abs(n_ok) - 1
      n_failed = n_frames - n_ok
    endif



! Write out list of copied plot files
    if m$exist(plotlist) .eq. 1 then
      open/file {plotlist} READ fctrl
      i = 0

      plot_loop:

      read/file {fctrl(1)} proinfo {maxchar}
      if fctrl(2) .ge. 0 then
        i = i + 1
        pos1 = m$index(proinfo," ") - 1
        pipefile = proinfo(1:{pos1})
        pos1 = pos1 + 2
        proinfo = proinfo({pos1}:)

        if m$indexb(proinfo,failmsg) .le. 0 then
          pos2 = m$indexb(proinfo,"/")
          if pos2 .gt. 0 then  
            pos2 = pos2 + 1
          endif

          mess/out 3 {procID} "Plot file {pipefile}"
          mess/out 3 {procID} "  saved as {proinfo({pos2}:)}"
        else
          mess/out 3 {procID} -
                     "Plot file {pipefile} {proinfo}"
        endif
       
        goto plot_loop
      else
        close/file {fctrl(1)}
        if n_failed .gt. 0 then
          errmsg(1:1) = m$upper(errmsg(1:1))
          mess/out 1 {procID} "{errmsg}"
        endif
        mess/out 3 {procID} " "
      endif
    else
      n_deliver = n_ok

      if n_failed .gt. 0 then
        errmsg(1:1) = m$upper(errmsg(1:1))
        mess/out 1 {procID} "{errmsg}"
        mess/out 3 {procID} " "
      endif
    endif

    set/format i1
    mess/out 3 {procID} "{n_ok} out of {n_frames} plot files successfully written!"
  endif
endif

return 0
