! $Id: fors_qc1_log.prg,v 1.9 2001/06/27 15:46:28 sbogun Exp $   
!+++ FORS_QC1_LOG ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 2000  European Southern Observatory
!
!.IDENT         $RCSfile: fors_qc1_log.prg,v $
!
!.KEYWORDS      Write keyword or comment into QC1 Log file
!
!.USAGE         qc1log/out mode frame keyword value comment
!
!.INPUT         mode    (P1): 0: dump contents of the cache to the real qc1log.
!                             1: open and add entry to log cache. The entries 
!                                in the cache are first dumped to the real log 
!                                as soon as qc1log/out 0 is called.
!                             2: start a new group.
!                             3: end a group.
!               frame   (P2): Name of frame referred to.
!               keyword (P3): Name of keyword to be written.
!               value   (P4): Value of keyword 'keyword' to be written.
!               comment (P5): Optional comment string.
!               dict    (P6): Dictionary name (required, if not QC dictionary).
!
!.OUTPUT        none.
!
!.RETURN        Q1:   0: successful return
!                     1: failed
!                    -1: warning, could not create log
!
!.DEFAULT
!
!.PURPOSE       Writes into QC1 Log file
!
!.ALGORITHM     qc1log/out creates an entry in a QC1 Log File using the
!               UNIX command 'QC1LogWriter'. For details of QC1LogWriter
!               please refer to the documentation of the command. It also
!               creates the same keyword in the header of concerned frame.
!
!.ENVIRON       FORS context
!   
!.LANGUAGE      MIDAS 99NOV
!
!.AUTHOR        Stefan Bogun, ESO-DMO
!
!.VERSION       $Log: fors_qc1_log.prg,v $
!.VERSION       Revision 1.9  2001/06/27 15:46:28  sbogun
!.VERSION       QC.DID is now written, too.
!.VERSION
!.VERSION       Revision 1.8  2001/06/20 12:42:52  sbogun
!.VERSION       QC1LogWriter called for every single keyword to dump.
!.VERSION       QC1LogWriter call implemented in shell script.
!.VERSION
!.VERSION       Revision 1.7  2001/06/15 15:53:22  sbogun
!.VERSION       Complete revision:
!.VERSION       Can handle groups now
!.VERSION       supports extra dictionaries
!.VERSION       ...
!.VERSION
!
!.COMMENT
!------------------------------------------------------------------------------


crossref mode frame keyword value comment

define/param P1 ? c "Enter mode:"
define/param P2 + c "Enter name of frame:"
define/param P3 + c "Enter name of keyword:"
define/param P4 + c "Enter value of keyword:"
define/param P5 + c "Enter comment string:"
define/param P6 + c "Enter dictionary name:"

define/local procid/c/1/20       "qc1_log"
define/local dict_prefix/c/1/20  "ESO-DFS-DIC."
define/local instrument/c/1/20   " " all

define/local mode/i/1/1          {p1}
define/local frame/c/1/100       {p2}
define/local keyword/c/1/100     {p3}
define/local value/c/1/100       " " all
define/local comment/c/1/100     " " all
define/local dict/c/1/100        " " all
define/local dict_path/c/1/100   " " all
define/local opt/c/1/255         " " all
define/local command/c/1/255     " " all +lower_levels
define/local i/i/1/1             0
define/local n/i/1/1             0
define/local frame_fits/c/1/100  " " all
define/local quote/c/1/1         "
define/local singlequote/c/1/1   '

write/keyword value              {p4}
write/keyword comment            {p5}
write/keyword dict               {p6}
instrument = m$upper(rbinstrument)

! Check whether cache table exists already, otherwise create it
if m$exist(qc1log_table) .eq. 0 .or. m$existk("qc1log_id") .eq. 0 then
  write/key QC1LOG_ID/i/1/1 0
  mess/out 4 {procid} " "
  mess/out 4 {procid} "Creating qc1 log table {qc1log_table}"
  create/table {qc1log_table} 5 1000 null
  create/column {qc1log_table} :frame   " " A100 C*100
  create/column {qc1log_table} :keyword " " A100 C*100
  create/column {qc1log_table} :value   " " A100 C*100
  create/column {qc1log_table} :comment " " A100 C*100
  create/column {qc1log_table} :dict    " " A100 C*100
  QC1LOG_ID = 1
endif

! Check whether group key exists
if m$existk("qc1log_group") .eq. 0 then
  write/key QC1LOG_GROUP/i/1/1 0
endif


branch mode 0,1,2,3 DMP,ADD,START,STOP

mess/out 1 {procid} "Wrong mode selected (0,1,2,3 are allowed) !"
goto FIN


DMP:
  if QC1LOG_ID .le. 1 then
       mess/out 4  {procid} "There's nothing to log..."
    return -1
  endif

! Check whether system variables DFS_QC1_LOG_DIR and DFS_QC1_DIC_DIR exist
  out_a = m$symbol("DFS_QC1_LOG_DIR")
  if out_a .eq. "DFS_QC1_LOG_DIR" then
    append/out "Shell variable DFS_QC1_LOG_DIR not defined. Cannot\n"
    append/out "create QC1 Log!"
    mess/out 2 {procid}
    return -1
  endif

  out_a = m$symbol("DFS_QC1_DIC_DIR")
  if out_a .eq. "DFS_QC1_DIC_DIR" then
    append/out "Shell variable DFS_QC1_DIC_DIR not defined. Cannot\n"
    append/out "create QC1 Log!"
    mess/out 2 {procid}
    return -1
  endif


! Check whether dictionary exists
  out_a = m$symbol("DFS_QC1_DIC_DIR") // "/" // dict_prefix // instrument -
          // "_QC"
  if m$exist(out_a) .eq. 0 then
    append/out "Dictionary {out_a} does not exist.\n"
    append/out "Cannot create QC1 Log!"
    mess/out 2 {procid}
    return -1
  endif


! Check whether last group ended
  if qc1log_group .eq. 1 then
    qc1log/out 1 "stop_group" 0 0
  endif
  qc1log_group = 0


! Loop over all entries in QC1Log Cache
  mess/out 4 {procid} " "
  n = qc1log_id - 1
  do i = 1 n

    opt = " "

    frame   = "{{qc1log_table},:frame,@{i}}"
    keyword = "{{qc1log_table},:keyword,@{i}}"
    value   = "{{qc1log_table},:value,@{i}}"
    comment = "{{qc1log_table},:comment,@{i}}"
    dict    = "{{qc1log_table},:dict,@{i}}"

! Start group keyword
    if frame .eq. "start_group" then
      $QC1LogWriter -s {instrument} -g start
      goto ENDLOOP
    endif

! Stop group keyword
    if frame .eq. "stop_group" then
      $QC1LogWriter -s {instrument} -g stop
      goto ENDLOOP
    endif

! Replace MIDAS file name by FITS file name
    if m$exist(frame) .eq. 1 then
      if m$existd(frame,H_PIPEFILE) .eq. 1 then
        frame_fits = "{{frame},{H_PIPEFILE}}"
        if keyword .eq. "{H_PIPEFILE}" then
          value = frame_fits
        endif
      else if m$existd(frame,H_FILENAME) .eq. 1 then
        frame_fits = "{{frame},{H_FILENAME}}"
        outputi = m$indexb(frame_fits,"/")+1
        frame_fits = frame_fits({outputi(1)}:)
      endif
      frame = frame_fits      
    endif

! Create QC keyword name
    out_a = m$upper(keyword)
    outputi = m$index(out_a,"ESO.")
    if outputi .eq. 1 then
      keyword = keyword(5:)
    endif

! Check for VLT dictionaries
    if dict .ne. "+" then

! Check whether system variable INS_ROOT exists
      out_a = m$symbol("INS_ROOT")
      if out_a .eq. "INS_ROOT" then
        append/out "Shell variable INS_ROOT not defined. Cannot\n"
        append/out "qc1log keywords defined in VLT dictionaries!"
        mess/out 2 {procid}
        goto ENDLOOP
      endif

      dict_path = m$symbol("INS_ROOT") // "/SYSTEM/Dictionary"
      opt = opt // "~-I~" // quote // dict_path // quote // "~-D~" // quote // -
            dict // quote
    endif

 
! Create log entry
    if keyword .ne. "+" then
      @p fors_qc1_log,get_type {value}
      if {q1} .eq. 3 then
        opt = opt // "~-k~" // quote // keyword // "=" // singlequote // value // -
              singlequote // quote
      else
        opt = opt // "~-k~" // keyword // "=" // quote // value // quote
      endif
    endif

    if comment .ne. "+" then
      opt = opt // "~-c~" // quote // comment // quote
    endif

! Build the required system call and execute it.
! This will be done in a clean way with a later MIDAS release.
    command = "QC1LogWriter -s {instrument} -f {frame} " // opt
    command = m$repla(command,"~"," ")

    @p fors_qc1_log,system
    if {q1} .ne. 0 then
      mess/out 1 {procid} "UNIX command could not be executed."
      return 1
    else if {q2} .ne. 0 then
      mess/out 1 {procid} "QC1LogWriter call failed."
      return -1
	endif

    mess/out 4 {procid} "Entry added to QC1 Log file."

ENDLOOP:
  enddo


  QC1LOG_ID = 0
  goto FIN


ADD:
! Check whether cache table exists already, otherwise create it
  if qc1log_id .eq. 0 then
    mess/out 4 {procid} " "
    mess/out 4 {procid} "Creating qc1 log table {qc1log_table}"
    create/table {qc1log_table} 5 1000 null
    create/column {qc1log_table} :frame   " " A100 C*100
    create/column {qc1log_table} :keyword " " A100 C*100
    create/column {qc1log_table} :value   " " A100 C*100
    create/column {qc1log_table} :comment " " A100 C*100
    create/column {qc1log_table} :dict    " " A100 C*100
    QC1LOG_ID = 1
  endif

  if frame .eq. "+" then
    mess/out 1 {procid} "Please specify a frame name (P2)!"
    return 1
  endif

  if keyword .ne. "+" .and. value .eq. "+" then
    mess/out 1 {procid} "Please specify a value (P4) for keyword {keyword}!"
    return 1
  endif

  if comment .eq. "+" .and. keyword .eq. "+" then
    append/out "Please specify either keyword (P3) and value (P4)\n"
    append/out "or comment (P5)!"
    mess/out 1 {procid}
    return 1
  endif


! Create entry into table
  {qc1log_table},:frame,@{QC1LOG_ID}   = "{frame}"
  {qc1log_table},:keyword,@{QC1LOG_ID} = "{keyword}"
  {qc1log_table},:value,@{QC1LOG_ID}   = "{value}"
  {qc1log_table},:comment,@{QC1LOG_ID} = "{comment}"
  {qc1log_table},:dict,@{QC1LOG_ID}    = "{dict}"
  

! Write keyword into concerned frame
  if m$exist(frame) .eq. 1 .and. dict .eq. "+" then
    @p fors_qc1_log,get_type {value}
    if {q1} .eq. 1 then
      write/desc {frame} {keyword}/i/1/1 {value}
    else if {q1} .eq. 2 then
      write/desc {frame} {keyword}/d/1/1 {value}
    else
      outputi(1) = m$strlen(value)
      write/desc {frame} {keyword}/c/1/{outputi(1)} "{value}"
    endif
    if comment .ne. "+" then
      write/dhelp {frame} {keyword} "{comment}"
    endif
  endif

  mess/out 4 {procid} "Entry added to QC1 log table {qc1log_table}."
  QC1LOG_ID = QC1LOG_ID + 1
  goto FIN


START:
  if qc1log_group .eq. 1 then
    qc1log/out 1 "stop_group" 0 0
  endif

  qc1log/out 1 "start_group" 0 0
  qc1log/out 1 0 {h_proqcdid} {p_qcdic_version}
  qc1log_group = 1

  goto FIN


STOP:
  if qc1log_group .eq. 0 then
    mess/out 2 {procid} "There's no group to stop! Ignoring command."
    goto FIN
  endif

  qc1log/out 1 "stop_group" 0 0
  qc1log_group = 0

  goto FIN


FIN:
return 0




entry get_type
! Entry to check type of a keyword
! Returns Q1=1 if type = integer
!         Q1=2 if type = float
!         Q1=3 if type = string

define/parameter P1 ? c "Enter parameter to check:"

! Check for comma (m$tstno doesn't catch this)
outputi = m$index(p1,",")
if outputi(1) .ne. 0 then
  write/keyword q1 3
  return {q1}
endif

! Check if number
outputi = m$tstno(p1)

if outputi(1) .eq. 0 then
  write/keyword q1 3
else
  outputi = m$index(p1,".")
  if outputi(1) .eq. 0 then
    write/keyword q1 1
  else
    write/keyword q1 2
  endif
endif

return {q1}




entry system
! Entry to execute UNIX command
!
! UNIX command to be executed has to be written to the keyword COMMAND
! prior to call of fors_qc1_log,system
!
! Returns Q1=1 if command could not be executed
!         Q2=error code of UNIX command

define/local filecode/i/1/2   0    all
define/local returncode/i/1/1 0
define/local sfile/c/1/32     "qc1script"
define/local cfile/c/1/32     "qc1code"

! Command should exist and not have length 0.
if m$existk("command") .eq. 0 then
  return 1 0
endif
if m$strlen(command) .eq. 0 then
  return 1 0
endif

! To verify that {cfile} is writable
open/file {cfile} write filecode
if filecode(1) .eq. -1 then
  return 1 0
endif
close/file {filecode(1)}

! Write script text to {sfile}
open/file {sfile} write filecode
if filecode(1) .eq. -1 then
  return 1 0
endif

write/file {filecode(1)} "#!/bin/sh"
write/file {filecode(1)},key command
write/file {filecode(1)} "echo $? > {cfile}"

close/file {filecode(1)}

! Execute script
$chmod u+x {sfile}
$./{sfile}

! Read return code
write/_out {cfile} | write/key returncode

-delete {sfile}
-delete {cfile}

return 0 {returncode}
