! $Id: fors_qc1report_master.prg,v 1.8 2001/07/13 09:43:40 sbogun Exp $
!+++ FORS_QC1REPORT_MASTER ++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 2000  European Southern Observatory
!
!.IDENT         $RCSfile: fors_qc1report_master.prg,v $
!
!.KEYWORDS      Create QC1 report for Master Calibration files
!
!.USAGE         @p fors_qc1report_master 
!
!.INPUT         master   (P1): file name of master frame
!               catalog  (P2): catalog containing raw frames that were
!                              averaged to build master
!               category (P3): frame category of master frame
!
!.OUTPUT        none
!
!.RETURN        Q1:   0: successful return
!                     1: failed 
!                    -1: warning, QC1 report could not be created
!
!.DEFAULT      
!
!.PURPOSE       Create QC1 report for newly generated Master Calibration files
!
!.ALGORITHM     Keyword entries in the QC1 Log file are created depending on
!               the type of master frame as given by P3 (MASTER_BIAS,
!               MASTER_SCREEN_FLAT_IMG, MASTER_SKY_FLAT_IMG).
!
!.ENVIRON       FORS context
!
!.AUTHOR        Stefan Bogun, ESO-DMO
!
!.VERSION       1.0 2000/07/11  Creation
!
!.COMMENT
!---------------------------------------------------------------------------

crossref master catalog category

define/param P1 ? c "Enter file name of master frame:"
define/param P2 ? c "Enter file name of catalog containing raw frames:"
define/param P3 ? c "Enter frame category of master frame:"

define/local procid/c/1/20       "qc1report_master"

define/local master/c/1/100      {p1}
define/local catalog/c/1/100     {p2}
define/local category/c/1/100    {p3}

define/local n_frames/i/1/1       0
define/local raw1/c/1/100         " "  all
define/local raw2/c/1/100         " "  all
define/local catal/i/1/1          0
define/local imsec/d/1/4          0.0  all
define/local xsize/d/1/1          0.0
define/local ysize/d/1/1          0.0
define/local nport/i/1/1          0
define/local Ax1/d/1/1            0.0
define/local Ay1/d/1/1            0.0
define/local Ax2/d/1/1            0.0
define/local Ay2/d/1/1            0.0
define/local Bx1/d/1/1            0.0
define/local By1/d/1/1            0.0
define/local Bx2/d/1/1            0.0
define/local By2/d/1/1            0.0
define/local Cx1/d/1/1            0.0
define/local Cy1/d/1/1            0.0
define/local Cx2/d/1/1            0.0
define/local Cy2/d/1/1            0.0
define/local Dx1/d/1/1            0.0
define/local Dy1/d/1/1            0.0
define/local Dx2/d/1/1            0.0
define/local Dy2/d/1/1            0.0
define/local x1w/d/1/1            0.0
define/local y1w/d/1/1            0.0
define/local x2w/d/1/1            0.0
define/local y2w/d/1/1            0.0
define/local qcval/d/1/40         0.0  all
define/local instrname/c/1/10 " " ALL
define/local dictinst/c/1/10  " " ALL
define/local dpr_type/c/1/64  " " ALL
define/local tpl_id/c/1/64    " " ALL
define/local arcfile1/c/1/64  " " ALL
define/local arcfile2/c/1/64  " " ALL
define/local pipefile/c/1/64  " " ALL
define/local det_clock/c/1/64 " " ALL
define/local insmode/c/1/10   " " ALL
define/local exptime/r/1/1        0.0
define/local filter/c/1/64    " " ALL
define/local coll/c/1/64      " " ALL
define/local h_qcbiadf/c/1/60 " " ALL

! Create proper format
set/format f1.7,f1.15 i1


! Quit if catalog contains less than two frames
show/icat {catalog} N
n_frames = outputi(1)
if n_frames .lt. 2 then
  mess/out 2 {procid} "Too few frames for creating a QC1 Report!"
  return -1
endif


! Get name of first two raw frames in catalog
store/frame raw1 {catalog} 1
store/frame raw2 {catalog} 1


! Get instrument name
copy/dk {raw1} {h_instrume} instrname
dictinst = instrname
if instrname(1:4) .eq. "FORS" then
  dictinst = "FORS"
endif


! Get valid image area of master frame
image/section {master} X
if {q1} .eq. 0 then
  imsec(1) = outputd(1)
  imsec(2) = outputd(2)
  imsec(3) = outputd(3)
  imsec(4) = outputd(4)
else
  mess/out 1 {procid} "Cannot determine image layout from >{P1}<!"
  return 1
endif


! Calculate number of valid pixels along the x and y axis in one quadrant
xsize = (imsec(3) - imsec(1) + 1) / 2
ysize = (imsec(4) - imsec(2) + 1) / 2

if xsize .lt. 512 .or. ysize .lt. 512 then
  mess/out 2 {procid} "Image too small. Cannot compute QC1 parameters!"
  return -1
endif


! Start group
qc1log/out 2


! Standard keywords
if m$existd(raw1,h_dprtype) .eq. 1 then
  copy/dk {raw1} {h_dprtype} dpr_type
  qc1log/out 1 {raw1} {h_dprtype} "{dpr_type}" + DPR
endif
 
if m$existd(raw1,h_tpid) .eq. 1 then
  copy/dk {raw1} {h_tpid} tpl_id
  qc1log/out 1 {raw1} {h_tpid} "{tpl_id}" + TPL
endif

if m$existd(raw1,h_outclock) .eq. 1 then
  copy/dk {raw1} {h_outclock} det_clock
  qc1log/out 1 {raw1} {h_outclock} "{det_clock}" + CCDDCS
endif

if m$existd(raw1,h_nout) .eq. 1 then
  copy/dk {raw1} {h_nout} nport
  if nport .ne. 1 .and. nport .ne. 4 then
    mess/out 1 {procid} "Invalid readout port setup!"
    return 1
  endif
  qc1log/out 1 {raw1} {h_nout} {nport} + CCDDCS
endif

if m$existd(raw1,h_arcfile) .eq. 1 then
  copy/dk {raw1} {h_arcfile} arcfile1
else
  mess/out 1 {procid} "Frame {raw1} does not contain {h_arcfile} descriptor!"
endif

if m$existd(raw2,h_arcfile) .eq. 1 then
  copy/dk {raw2} {h_arcfile} arcfile2
else
  mess/out 1 {procid} "Frame {raw2} does not contain {h_arcfile} descriptor!"
endif

!Keyword PIPEFILE does not yet exist -> use MIDAS file name
pipefile = master


! Position of first quadrant
Ax1 = imsec(1)
Ax2 = Ax1 + xsize - 1 
Ay1 = imsec(2)
Ay2 = Ay1 + ysize - 1

! Position of second quadrant
Bx1 = imsec(1) + xsize
Bx2 = Bx1 + xsize - 1 
By1 = imsec(2)
By2 = By1 + ysize - 1

! Position of third quadrant
Cx1 = imsec(1)
Cx2 = Cx1 + xsize - 1 
Cy1 = imsec(2) + ysize
Cy2 = Cy1 + ysize - 1

! Position of fourth quadrant
Dx1 = imsec(1) + xsize
Dx2 = Dx1 + xsize - 1 
Dy1 = imsec(2) + ysize
Dy2 = Dy1 + ysize - 1


BRANCH category {MBIAS_ID},{MDARK_ID},{MFLAT_ID},{MSKY_ID},{MNIGHT_ID} -
            mbias,mdark,mscrflat,mskyflat,mngtflat
goto end_loop

mbias:
! Master Bias QC Log

! Position of sub-window of quadrant A
  x1w = Ax1 + (xsize-fors_qcstatwin)/2
  x2w = x1w + fors_qcstatwin - 1
  y1w = Ay1 + (ysize-fors_qcstatwin)/2
  y2w = y1w + fors_qcstatwin - 1

! qcval(1) is the standard deviation over the whole quadrant A of the
!          first raw frame
! qcval(2) is the median over the whole quadrant A of the first raw frame
  statistics/image {raw1} [{Ax1},{Ay1}:{Ax2},{Ay2}] p5=xn
  qcval(1) = outputr(4)
  qcval(2) = outputr(8)

! qcval(3) is the standard deviation over a subframe of quadrant A
!          of the first raw frame
! qcval(4) is the structure of the first raw frame
  statistics/image {raw1} [{x1w},{y1w}:{x2w},{y2w}] p5=sn
  qcval(3) = outputr(4)
  if qcval(1) .ge. qcval(3) then
    qcval(4) = m$sqrt(qcval(1)**2 - qcval(3)**2)
  else
    qcval(4) = -1.
  endif
  
! qcval(5) is the standard deviation over the whole quadrant A of the
!          master
! qcval(6) is the median over the whole quadrant A of the master
  statistics/image {master} [{Ax1},{Ay1}:{Ax2},{Ay2}] p5=xn
  qcval(5) = outputr(4)
  qcval(6) = outputr(8)

! qcval(7) is the standard deviation over a subframe of quadrant A
!          of the master
! qcval(8) is the structure of the master
  statistics/image {master} [{x1w},{y1w}:{x2w},{y2w}] p5=sn
  qcval(7) = outputr(4)
  if qcval(5) .ge. qcval(7) then
    qcval(8) = m$sqrt(qcval(5)**2 - qcval(7)**2)
  else
    qcval(8) = -1.
  endif

! qcval(9) is the expected standard deviation over a subframe of quadrant A
!          of the master (value of raw frame divided by sqrt(n_comb))
  if m$existd(master,h_proncom) .ne. 0 then
    outputi = {{master},{h_proncom}}
    qcval(9) = qcval(3) / m$sqrt({outputi(1)})
  else
    qcval(9) = -1.
  endif
  

  if nport .eq. 4 then


    if instrname(1:5) .eq. "FORS2" then
! For FORS2 compute fractional port difference between A and D

! qcval(10) is the mean of quadrant A of the first raw frame
      statistics/image {raw1} [{Ax1},{Ay1}:{Ax2},{Ay2}] p5=sn
      qcval(10) = outputr(3)

! qcval(11) is the mean of quadrant A of the master frame
      statistics/image {master} [{Cx1},{Cy1}:{Cx2},{Cy2}] p5=sn
      qcval(11) = outputr(3)

! qcval(12) is the mean of quadrant D of the first raw frame
      statistics/image {raw1} [{Dx1},{Dy1}:{Dx2},{Dy2}] p5=sn
      qcval(12) = outputr(3)

! qcval(13) is the mean of quadrant D of the master frame
      statistics/image {master} [{Dx1},{Dy1}:{Dx2},{Dy2}] p5=sn
      qcval(13) = outputr(3)

! qcval(14) is the median of the whole raw frame
      statistics/image {raw1} [{imsec(1)},{imsec(2)}:{imsec(3)},{imsec(4)}] -
                       p5=xn
      qcval(14) = outputr(8)

! qcval(15) is the median of the whole master frame
      if m$existd(master,h_promedi) .ne. 0 then
        qcval(15) = {{master},{h_promedi}}
      else
        statistics/image {master} [{imsec(1)},{imsec(2)}:{imsec(3)},{imsec(4)}] -
                         p5=xn
        qcval(15) = outputr(8)
      endif

! qcval(16) is the fractional port difference between port A and D for
!          the first raw frame
      qcval(16) = m$abs(qcval(10)-qcval(12)) / qcval(14) * 100.

! qcval(17) is the fractional port difference between port A and D for
!          the master frame
      qcval(17) = m$abs(qcval(11)-qcval(13)) / qcval(15) * 100.

! Use correct keyword name
      h_qcbiadf = h_qcbiadfad

    else
! For FORS1 compute fractional port difference between C and D

! qcval(10) is the mean of quadrant C of the first raw frame
      statistics/image {raw1} [{Cx1},{Cy1}:{Cx2},{Cy2}] p5=sn
      qcval(10) = outputr(3)

! qcval(11) is the mean of quadrant C of the master frame
      statistics/image {master} [{Cx1},{Cy1}:{Cx2},{Cy2}] p5=sn
      qcval(11) = outputr(3)

! qcval(12) is the mean of quadrant D of the first raw frame
      statistics/image {raw1} [{Dx1},{Dy1}:{Dx2},{Dy2}] p5=sn
      qcval(12) = outputr(3)

! qcval(13) is the mean of quadrant D of the master frame
      statistics/image {master} [{Dx1},{Dy1}:{Dx2},{Dy2}] p5=sn
      qcval(13) = outputr(3)

! qcval(14) is the median of the whole raw frame
      statistics/image {raw1} [{imsec(1)},{imsec(2)}:{imsec(3)},{imsec(4)}] -
                       p5=xn
      qcval(14) = outputr(8)

! qcval(15) is the median of the whole master frame
      if m$existd(master,h_promedi) .ne. 0 then
        qcval(15) = {{master},{h_promedi}}
      else
        statistics/image {master} [{imsec(1)},{imsec(2)}:{imsec(3)},{imsec(4)}] -
                         p5=xn
        qcval(15) = outputr(8)
      endif

! qcval(16) is the fractional port difference between port C and D for
!          the first raw frame
      qcval(16) = m$abs(qcval(10)-qcval(12)) / qcval(14) * 100.

! qcval(17) is the fractional port difference between port C and D for
!          the master frame
      qcval(17) = m$abs(qcval(11)-qcval(13)) / qcval(15) * 100.

! Use correct keyword name
      h_qcbiadf = h_qcbiadfcd

    endif

  endif


  qc1log/out 1 {raw1} {h_arcfile} {arcfile1} + PRIMARY-FITS

  qc1log/out 1 {raw1} {h_qcbiameda} {qcval(2)}
  if {q1} .gt. 0 then
    mess/out 1 {procid} "Creating the QC1 log failed!"
    return 1
  else if {q1} .lt. 0 then
    return -1
  endif

  qc1log/out 1 {raw1} {h_qcbianoma} {qcval(3)}
  if {q1} .gt. 0 then
    mess/out 1 {procid} "Creating the QC1 log failed!"
    return 1
  else if {q1} .lt. 0 then
    return -1
  endif

  if nport .eq. 4 then
    qc1log/out 1 {raw1} {h_qcbiadf} {qcval(16)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif
  endif

  qc1log/out 1 {master} {h_pipefile} {pipefile} + PRO

  qc1log/out 1 {master} {h_qcbiameda} {qcval(6)}
  if {q1} .gt. 0 then
    mess/out 1 {procid} "Creating the QC1 log failed!"
    return 1
  else if {q1} .lt. 0 then
    return -1
  endif

  qc1log/out 1 {master} {h_qcbiastra} {qcval(8)}
  if {q1} .gt. 0 then
    mess/out 1 {procid} "Creating the QC1 log failed!"
    return 1
  else if {q1} .lt. 0 then
    return -1
  endif

  qc1log/out 1 {master} {h_qcbianoma} {qcval(7)}
  if {q1} .gt. 0 then
    mess/out 1 {procid} "Creating the QC1 log failed!"
    return 1
  else if {q1} .lt. 0 then
    return -1
  endif

  qc1log/out 1 {master} {h_qcbianoea} {qcval(9)}
  if {q1} .gt. 0 then
    mess/out 1 {procid} "Creating the QC1 log failed!"
    return 1
  else if {q1} .lt. 0 then
    return -1
  endif

  if nport .eq. 4 then
    qc1log/out 1 {master} {h_qcbiadf} {qcval(17)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif
  endif

  goto end_loop


mdark:
  goto end_loop


mscrflat:
mskyflat:
! Master Screen Flat (Imaging)
  if m$existd(master,h_insmode) .eq. 0 then
    mess/out 1 {procid} "Instrument mode could not be determined!"
    return 1
  endif
  insmode = m$value({master},{h_insmode})
  
  if m$existd(raw1,h_filti(1)) .eq. 1 then
    copy/dk {raw1} {h_filti(1)} filter
    qc1log/out 1 {raw1} {h_filti(1)} {filter} + {dictinst}
  endif

  if m$existd(raw1,h_coll) .eq. 1 then
    copy/dk {raw1} {h_coll} coll
    qc1log/out 1 {raw1} {h_coll} {coll} + {dictinst}
  endif


  if insmode .eq. "IMG" then

! qcval(1) is the mean of quadrant A of the master
! qcval(2) is the standard deviation of quadrant A of the master
! qcval(3) is the median divided by the exposure time of the master
! ("efficiency")
    if m$existd(master,"O_TIME") .eq. 0 then
      mess/out 1 {procid} "Exposure time could not be determined!"
      return 1
    endif
    exptime = m$value({master},O_TIME(7))
    statistics/image {master} [{Ax1},{Ay1}:{Ax2},{Ay2}] p5=xn
    qcval(1) = outputr(3)
    qcval(2) = outputr(4)
    qcval(3) = outputr(8) / exptime

! qcval(4) is the photon noise in quadrant A
    extract/image win1 = {raw1} [{Ax1},{Ay1}:{Ax2},{Ay2}]
    extract/image win2 = {raw2} [{Ax1},{Ay1}:{Ax2},{Ay2}]
    compute/image diff = win1 - win2
    statistics/image diff p5=sn
    qcval(4) = outputr(4) / m$sqrt(2.)

! qcval(5) is the photon noise in quadrant B
    extract/image win1 = {raw1} [{Bx1},{By1}:{Bx2},{By2}]
    extract/image win2 = {raw2} [{Bx1},{By1}:{Bx2},{By2}]
    compute/image diff = win1 - win2
    statistics/image diff p5=sn
    qcval(5) = outputr(4) / m$sqrt(2.)

! qcval(6) is the photon noise in quadrant C
    extract/image win1 = {raw1} [{Cx1},{Cy1}:{Cx2},{Cy2}]
    extract/image win2 = {raw2} [{Cx1},{Cy1}:{Cx2},{Cy2}]
    compute/image diff = win1 - win2
    statistics/image diff p5=sn
    qcval(6) = outputr(4) / m$sqrt(2.)

! qcval(7) is the photon noise in quadrant D
    extract/image win1 = {raw1} [{Dx1},{Dy1}:{Dx2},{Dy2}]
    extract/image win2 = {raw2} [{Dx1},{Dy1}:{Dx2},{Dy2}]
    compute/image diff = win1 - win2
    statistics/image diff p5=sn
    qcval(7) = outputr(4) / m$sqrt(2.)

! qcval(8) is the mean over the whole quadrant A of the first raw frame
! qcval(9) is the median over the whole quadrant A of the first raw frame
! qcval(10) is the conversion factor (CONAD) in quadrant A
    statistics/image {raw1} [{Ax1},{Ay1}:{Ax2},{Ay2}] p5=xn
    qcval(8) = outputr(3)
    qcval(9) = outputr(8)
    qcval(10) = qcval(9) / qcval(4)**2

! qcval(11) is the mean over the whole quadrant B of the first raw frame
! qcval(12) is the median over the whole quadrant B of the first raw frame
! qcval(13) is the conversion factor (CONAD) in quadrant B
    statistics/image {raw1} [{Bx1},{By1}:{Bx2},{By2}] p5=xn
    qcval(11) = outputr(3)
    qcval(12) = outputr(8)
    qcval(13) = qcval(12) / qcval(5)**2

! qcval(14) is the mean over the whole quadrant C of the first raw frame
! qcval(15) is the median over the whole quadrant C of the first raw frame
! qcval(16) is the conversion factor (CONAD) in quadrant C
    statistics/image {raw1} [{Cx1},{Cy1}:{Cx2},{Cy2}] p5=xn
    qcval(14) = outputr(3)
    qcval(15) = outputr(8)
    qcval(16) = qcval(15) / qcval(6)**2

! qcval(17) is the mean over the whole quadrant D of the first raw frame
! qcval(18) is the median over the whole quadrant D of the first raw frame
! qcval(19) is the conversion factor (CONAD) in quadrant D
    statistics/image {raw1} [{Dx1},{Dy1}:{Dx2},{Dy2}] p5=xn
    qcval(17) = outputr(3)
    qcval(18) = outputr(8)
    qcval(19) = qcval(18) / qcval(7)**2

! Position of 1. sub-window of quadrant A
    x1w = Ax1 + xsize - 2 * fors_qcstatwin - 8
    x2w = x1w + fors_qcstatwin - 1
    y1w = Ay1 + ysize - 2 * fors_qcstatwin - 8
    y2w = y1w + fors_qcstatwin - 1
! qcval(20) is the mean of a subframe of quadrant A of the master
! qcval(21) is the standard deviation of a subframe of quadrant A of the master
! ("total noise")
    extract/image win1 = {master} [{x1w},{y1w}:{x2w},{y2w}]
    statistics/image win1 p5=sn
    qcval(20) = outputr(3)
    qcval(21) = outputr(4)

! Position of 2. sub-window of quadrant A
    x1w = Ax1 + xsize - 2 * fors_qcstatwin - 18
    x2w = x1w + fors_qcstatwin - 1
    y1w = Ay1 + ysize - 2 * fors_qcstatwin - 18
    y2w = y1w + fors_qcstatwin - 1
! qcval(22) is the fixed-pattern noise
    extract/image win2 = {master} [{x1w},{y1w}:{x2w},{y2w}]
    compute/pixel diff = win1 - win2
    statistics/image diff p5=sn
    outputr(4) = outputr(4) / m$sqrt(2.)
    if outputr(4) .ge. qcval(4) then
      qcval(22) = m$sqrt(outputr(4)**2 - qcval(4)**2)
    else
      qcval(22) = -1.
    endif

! Position of large subwindow (for large scale structure determination)
    x1w = Ax1 + 5
    x2w = x1w + 2 * fors_qcstatwin - 1
    y1w = Ay1 + 611
    y2w = y1w + 2 * fors_qcstatwin - 1
! qcval(23) is the large scale structure of the first raw frame,
! measured as standard deviation of a smoothed 200x200 window in quadrant A
    extract/image win1 = {raw1} [{x1w},{y1w}:{x2w},{y2w}]
    filter/median win1 smooth1 8,8,0.0 Y,Q >Null
    statistics/image smooth1 p5=sn
    qcval(23) = outputr(4) / qcval(8) * 100.
! qcval(24) is the large scale structure of the master,
! measured as standard deviation of a smoothed 200x200 window in quadrant A
    extract/image win1 = {master} [{x1w},{y1w}:{x2w},{y2w}]
    filter/median win1 smooth1 8,8,0.0 Y,Q >Null
    statistics/image smooth1 p5=sn
    qcval(24) = outputr(4) / qcval(1) * 100.


    if nport .eq. 4 then

! qcval(25) is the mean of quadrant B of the master
      statistics/image {master} [{Bx1},{By1}:{Bx2},{By2}] p5=xn
      qcval(25) = outputr(3)

! qcval(26) is the median of the whole raw frame
      statistics/image {raw1} [{imsec(1)},{imsec(2)}:{imsec(3)},{imsec(4)}] -
                       p5=xn
      qcval(26) = outputr(8)

! qcval(27) is the median of the master frame
      if m$existd(master,h_promedi) .ne. 0 then
        qcval(27) = {{master},{h_promedi}}
      else
        statistics/image {master} [{imsec(1)},{imsec(2)}:{imsec(3)},{imsec(4)}] -
                         p5=xn
        qcval(27) = outputr(8)
      endif

! qcval(28) is the fractional port difference between port A and B for
!          the first raw frame
      qcval(28) = m$abs(qcval(8)-qcval(11)) / qcval(26) * 100.

! qcval(29) is the fractional port difference between port A and B for
!          the master frame
      qcval(29) = m$abs(qcval(1)-qcval(25)) / qcval(27) * 100.

    endif


    qc1log/out 1 {raw1} {h_arcfile} {arcfile1} + PRIMARY-FITS

    if nport .eq. 4 then
      qc1log/out 1 {raw1} {h_qcfltdfab} {qcval(28)}
      if {q1} .gt. 0 then
        mess/out 1 {procid} "Creating the QC1 log failed!"
        return 1
      else if {q1} .lt. 0 then
        return -1
      endif
    endif

    qc1log/out 1 {raw1} {h_arcfile} {arcfile1} + PRIMARY-FITS
    qc1log/out 1 {raw2} {h_arcfile} {arcfile2} + PRIMARY-FITS

    qc1log/out 1 {raw1} {h_qcfltnoph} {qcval(4)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif

    qc1log/out 1 {raw1} {h_qcfltcont} {qcval(23)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif

    qc1log/out 1 {raw1} {h_qcfltcnva} {qcval(10)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif

    qc1log/out 1 {raw1} {h_qcfltcnvb} {qcval(13)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif

    qc1log/out 1 {raw1} {h_qcfltcnvc} {qcval(16)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif

    qc1log/out 1 {raw1} {h_qcfltcnvd} {qcval(19)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif

    qc1log/out 1 {master} {h_pipefile} {pipefile} + PRO

    qc1log/out 1 {master} {h_qcflteffa} {qcval(3)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif

    qc1log/out 1 {master} {h_qcfltnofp} {qcval(22)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif

    qc1log/out 1 {master} {h_qcfltcont} {qcval(24)}
    if {q1} .gt. 0 then
      mess/out 1 {procid} "Creating the QC1 log failed!"
      return 1
    else if {q1} .lt. 0 then
      return -1
    endif

    if nport .eq. 4 then
      qc1log/out 1 {master} {h_qcfltdfab} {qcval(29)}
      if {q1} .gt. 0 then
        mess/out 1 {procid} "Creating the QC1 log failed!"
        return 1
      else if {q1} .lt. 0 then
        return -1
      endif
    endif


  endif


  goto end_loop


mngtflat:
  goto end_loop


end_loop:
! End group
qc1log/out 3

return 0
