! $Id: fors_r_cosmic.prg,v 1.7 1999/11/24 09:47:50 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.IDENT       $RCSfile: fors_r_cosmic.prg,v $
!.PURPOSE:         MIDAS procedure to remove and delete cosmic ray events 
!                  on single CCD exposure.
!.USE:             FILTER/COSMIC p1 p2 p3 p4  where:
!                     P1: input image containing the cosmic rays,
!                     P2: median filter of the input image 
!                     P3: SKY,RON,G,NS,RC || P3: SGIMA-factor
!                     P4: mask name
!                     P5: high/low quality mode (low --> higher speed)
!.AUTHOR:          P.Magain, M.Remy, Institut d'Astrophysique de Liege  
!.VERSION:         911121 MP implemented in Portable MIDAS
!.VERSION:         971120 SW extended copy of rcosmic.prg
!------------------------------------------------------------------------------
! 
define/param p1 ? IMA "Enter input image:"
define/param p4 + IMA "Enter mask name for cosmics:"
define/param p5 "L" C "Enter H=high quality, L=low quality:"
define/param p6 1,1,0.0 N "Enter parameter P3 for FILTER/MEDIAN:"
define/param p7 NA C "Enter parameter P4 for FILTER/MEDIAN:"
define/maxpar 7
define/local tim/i/1/1 0
!
!
!
if {p5} .eq. "L" goto low

define/param p2 ? IMA "Enter output image:"
define/param p3 ? NUM "Enter SKY,G,RON,NS,RC:"

write/keyw in_a {p1}
define/local outima/c/1/60 {p2} 
define/local params/r/1/5 -1,-1,-1,4,2
write/keyw params {p3}
define/local cosmic/c/1/60 {p4}
! 
write/out "Beginning of the median filter"
filter/median {in_a} middumma {p6} {p7}
! 
write/out "Beginning of the cosmic rays removal"
! 
!run MID_EXE:RCOSMIC
!r_cosmic.exe may be a bit faster if SKY is negative. In that case the
!neighbourhood-search is disabled. Moreover the number of cosmics are count
!down. So if that count reaches 0 the cosmic removal has finished.
run PIPE_EXE:fors_r_cosmic.exe

goto fini


low:
!LOW-QUALITY MODE:
!(1) FIND PIXEL where their value is greater than MEAN + {p3}*STDDEV
!(2) REPLACE found PIXEL by the MEAN of left and right neighbour
!--> this mode is about 7 times faster than the HIGH-QUALITY MODE especially
!    if INPUT and OUTPUT frame are the same (no copy!)

define/param p2 {p1} IMA "Enter output image:"
define/param p3 ? NUM "Enter SIGMA-factor for cosmic search:"

!write/out "low quality but high speed mode"
define/local hi/r/1/1 0.0
define/local i/i/1/1 0
define/local k/i/1/1 0
define/local l/i/1/1 0
define/local k1/i/1/1 0
define/local k2/i/1/1 0
define/local lo/r/1/1 0.0

if m$existd("{p1}.bdf","COSMCOR") .gt. 0 goto fin
if "{p2}" .ne. "{p1}" copy/II {p1} {p2}

hi = {{p1},statistic(3)} + {{p1},statistic(4)}*{p3}
lo = {{p1},statistic(3)} - {{p1},statistic(4)}*{p3}
FIND/PIXEL {p1} {lo},{hi} OUT A middumma !>Null
if {outputi(1)} .gt. 0 then
  do i = 1 {outputi(1)} 1
     k = {middumma,:X_PIX,{i}}
     l = {middumma,:Y_PIX,{i}}
     k1 = {k} - 1
     k2 = {k} + 1
     {p2}[@{k},@{l}] = ({{p2}[@{k1},@{l}]} + {{p2}[@{k2},@{l}]})/2.0
  enddo
endif
stat/ima {p2} ? ? ? SN
!update LHCUTS
{p2},lhcuts(1) = {{p2},statistic(3)} - 3.0*{{p2},statistic(4)}
{p2},lhcuts(2) = {{p2},statistic(3)} + 3.0*{{p2},statistic(4)}

fini:
write/keyw history "r_cosmic.prg"
write/desc {p2} COSMCOR/C/1/5/ "done"
fin:
