! $Id: fors_rebima.prg,v 1.1 1999/03/03 17:14:39 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999 European Southern Observatory
!
!.IDENT       $RCSfile: fors_rebima.prg,v $
!
!.KEYWORDS    Linear rebinning, flux conservation, images
!
!.USAGE       rebin/image  infrm outfrm refgrd method
!
!.INPUT       infrm     (P1)   Input frame to be rebinned. The input
!                              grid specification is read from the
!                              descriptors START, STEP and NPIX.
!             refgrd    (P3)   Reference for the new data grid. This may
!                              be a list of values interpreted as start, step
!                              and number of bins or a reference frame with
!                              the descriptors START, STEP and NPIX.
!             method    (P4)   Rebinning method (PIXEL, LINEAR).
!
!.OUTPUT      outfrm    (P2)   Rebinned output frame.
!
!.RETURNS     The procedure returns 0 on success or 1 on error. The exit
!             status is stored in keyword Q1.
!
!.PURPOSE     The procedure rebins the input frame according to the new
!             grid given by REFGRD. The input grid is constructed using
!             the frame descriptors START, STEP and NPIX. The new grid 
!             specification may be a list of values which are interpreted as
!             start, step and number of bins or a reference frame from which
!             the new grid is created using the descriptors START, STEP and
!             NPIX. The rebinning method is provided by METHOD. Supported
!             methods are PIXEL and LINEAR. 
!             The data grids must be sorted in ascending order. Furthermore
!             the dependent variable of the input grid is assumed to be the
!             integral over the bin, normalized to the binwidth. Similarly,
!             the output data is normalized to the binwidth of the new grid.
!             The independent variable is assumed to refer to the center of
!             the bins.
!             
!.ENVIRON     MIDAS 98NOV, FORS pipeline
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.1 1998/02/28: Creation, RP
!
!------------------------------------------------------------------------------

crossref  infrm outfrm refgrd method

define/param p1 ? c "Input frame:"
define/param p2 ? c "Output frame:"
define/param p3 ? c "New grid specification:"
define/param p4 + c "Rebinning method:"
define/maxpar 4

! Procedure identification
define/local procID/c/1/20      "RebinIma"

! Local variables
define/local infrm/c/1/100   {p1}
define/local outfrm/c/1/100  {p2}
define/local refgrd/c/1/100  {p3}
define/local method/c/1/100  {p4}



! Process parameters
method = m$upper(method)

run PIPE_EXE:fors_rebima.exe
if {q1} .ne. 0 then
  mess/out 1 {procId} "Rebinning of >{intbl}< failed!"
  return 1
endif

return 0
