! $Id: fors_reborder.prg,v 1.6 2001/04/06 16:19:25 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_reborder.prg,v $
!
! .KEYWORDS   Rebinning
!
! .PURPOSE    Rebin each order to the same step size and write the different
!             wavelength start values into the header. The deviations of the
!             overlap areas are calculated. Rebinned frame, overlap areas /
!             deviations and all orders combined can be plotted.
! __________
!
! .USAGE      @p fors_reborder extimg [disptbl] davg [disp]
!
! .INPUT      extimg  (P1): order frame;
!                             default: none
!
!             disptbl (P2): output table from "fors_echwave_dis.prg";
!                             default: <DISPDEF> (see below)
!
!             davg    (P3): average (or minimal) dispersion coefficient,
!                           calculated by "fors_echwave_dis.prg";
!                             default: none
!
!             disp    (P4): display selection:<p>:pipeline mode:read global key
!                           NICE_CREA [y|n], <n>:none, <l>:local, or
!                           host:display[.screen] - with the latter option
!                           you can specify the name or IP-address of a display;
!                             default: <l>
!
! .OUTPUT     Q2: frame with rebinned orders "<P1>_reb.bdf"
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Non-linear rebinning according to the dispersion coefficients.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.5  99/11/08  Creation
!                  99/       Release
!
!------------------------------------------------------------------------------
!
!
crossref extimg disptbl davg disp
!
!
define/local DISPDEF/c/1/20 "dispcoef.tbl"
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "reborder"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      extimg  (P1): order frame;"
  w/o "                      default: none"
  w/o " "
  w/o "      disptbl (P2): output table from "fors_echwave_dis.prg";"
  w/o "                      default: {DISPDEF}"
  w/o " "
  w/o "      davg    (P3): average (or minimal) dispersion coefficient,"
  w/o "                    calculated by "fors_echwave_dis.prg";"
  w/o "                      default: none"
  w/o " "
  w/o "      disp    (P4): display selection:<p>:pipeline mode:read global key"
  w/o "                    NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                    host:display[.screen] - with the latter option"
  w/o "                    you can specify the name or IP-address of a display;"
  w/o "                      default: <l>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(extimg):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of order frame: "
define/local extimg/c/1/64 "{p1}"
if m$index(extimg,".bdf") .eq. 0 extimg = extimg//".bdf"
!
if m$exist(extimg) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({extimg})"
  goto error
endif
!
!
!P2(disptbl):
!-----------
define/local disptbl/c/1/60 " " all
disptbl = DISPDEF
define/parameter p2 {disptbl} ? -
  "        Enter table w. dispersion coefficients: "
disptbl = p2
if m$index(disptbl,".tbl") .eq. 0 disptbl = disptbl //".tbl"
!
if m$exist(disptbl) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({disptbl})"
  goto error
endif
outputi = 0
if m$existd(disptbl,"COEFS") .eq. 0 outputi = 1
if m$existd(disptbl,"COEFI") .eq. 0 outputi = 1
if m$existd(disptbl,"COEFD") .eq. 0 outputi = 1
if outputi(1) .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({disptbl},COEF[ S/I/D ])"
  goto error
endif
if m$existd(disptbl,"ORDER") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({disptbl},ORDER)"
  goto error
endif
show/descriptor {disptbl} ORDER h
if outputi(3) .lt. 2 then
        write/out " "
        mess/out 1 {PROID} "Descriptor has too few elements"
        mess/out 1 {PROID} "({disptbl},ORDER) - 2 needed !"
  goto error
endif
if {{disptbl},ORDER(1)} .eq. {{disptbl},ORDER(2)} then
        write/out " "
        mess/out 1 {PROID} "Highest abs.ord.no. ({{disptbl},ORDER(1)}) =="
        mess/out 1 {PROID} " lowest abs.ord.no. ({{disptbl},ORDER(2)}) !"
        mess/out 1 {PROID} " ({disptbl},ORDER)"
  goto error
endif
!
!
!P3(davg):
!--------
if p3 .eq. "?" mess/out 2 {PROID} " "
define/parameter p3 ? ? -
  "        Enter avg. (or min.) pixel size: "
define/local davg/d/1/1 {p3}
!
if davg .le. 0. then
        write/out " "
        mess/out 1 {PROID} "Pixel size <= 0. A !"
  goto error
endif
!
!
!P4(disp):
!--------
define/parameter p4 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p4}"
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
define/local char/i/1/1 0
define/local rebimg/c/1/64 " " all
define/local totordno/i/1/1 0
define/local xstep/d/1/1 0.
define/local ordloop/i/1/1 0
define/local wstart/d/1/1 0.
define/local wend/d/1/1 0.
define/local scrdif/i/1/1 0
define/local scry0/i/1/1 0
define/local frmy0/r/1/1 0.
define/local dscly/i/1/1 0
define/local scaly/r/1/1 0.
define/local labobjxl/i/1/1 0
define/local frmy/r/1/1 0.
define/local scry/i/1/1 0
define/local ordcol/i/1/1 0
define/local intlow/r/1/1 0.
define/local inthigh/r/1/1 0.
define/local ymin/r/1/1 0.
define/local ymax/r/1/1 0.
define/local lamblow/d/1/1 0.
define/local lambhigh/d/1/1 0.
define/local colour/i/1/1 0
!
!
!
!          MANAGING DISPLAYS / ITT / LUT :
!          *****************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_dids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 2 is missing !"
    goto error
  endif
  if nice_gids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 1 is missing !"
    goto error
  endif
  if nice_gids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 2 is missing !"
    goto error
  endif
  !
  !if nice_dids(1) .eq. 1 then
  !  assign/display d,1 >Null
  !  clear/display
  !  display/lut off
  !endif
  !assign/display d,2 >Null
  !clear/display
  !display/lut off
  !label/display "REBINNED" {nice_dlab}
  if nice_dids(3) .eq. 1 then
    assign/display d,3 >Null
    clear/display
    display/lut off
  endif
  load/lut heat
  load/itt ramp
  !assign/graphic g,1 >Null
  !clear/graphic
  !assign/graphic g,2 >Null
  !clear/graphic
  set/graphic default
  set/graphic pmode=0
  !
endif
!
!
!          REBINNING ORDER FRAME :
!          *********************
!
        mess/out 3 {PROID} "  rebinning each order to same stepsize,"
        mess/out 3 {PROID} "  storing starting wavelength per order"
!
in_a = extimg
in_b = disptbl
set/format ,F6.4
p3 = "{davg}"
set/format {FMTDEF}
p4 = "nonlinear"
  char = m$indexb(extimg,"_ext")-1
  rebimg = extimg(1:{char})//"_reb.bdf"
out_a = rebimg
!
!****************************
run STD_EXE:necrebi.exe >Null  ! creates descriptors: NORDER, NPTOT, WSTART
!****************************
!
info/descriptor {rebimg} NORDER
totordno = mid$info(2)
!
        mess/out 3 {PROID} "    resulting wavelengths:"
!
xstep = {{rebimg},STEP(1)}
do ordloop = 1 totordno
  inputi = {{rebimg},NORDER({ordloop})}
  wstart = {{rebimg},WSTART({ordloop})}
  wend = wstart+xstep*{{rebimg},NPTOT({ordloop})}
        set/format J2 ,T8.2
        mess/out 3 {PROID} "    abs.ord. {inputi}: {wstart}  - {wend}  A"
        set/format {FMTDEF}
enddo
!
!
if nice_crea .eq. "y" then
  !
  !
  !        DISPLAYING OF REBINNED FRAME :
  !        ****************************
  !
        mess/out 4 {PROID} "  displaying rebinned frame"
  !
  !**********************************************
  @p fors_nicedisp {rebimg} ? 7. 0,0 ? ? 0.5,4. n
  !**********************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of rebinned frame failed !"
        mess/out 1 {PROID} "({rebimg})"
    goto error
  endif
  !
  ! MAGNIFY IMAGE IN ORDER DIRECTION:
  ! --------------------------------
  inputr(2) = {{rebimg},NPIX(2)}
  inputi(2) = nice_dscr(2)/2  ! use only half the height of the display
  if inputr(2) .ge. inputi(2) then
    outputi(2) = -m$nint((inputr(2)-1.)/(inputi(2)*2.)+0.5)
  else
    outputi(2) = m$nint((inputi(2)*2.)/inputr(2)-0.5)
  endif
  if outputi(2) .le. -1 .or. outputi(2) .ge. 2 then
    {rebimg},DISPLAY_DATA(2) = -1*{outputi(2)}
  else  ! 0 not possible, -1 equivalent to 1
    {rebimg},DISPLAY_DATA(2) = 1
  endif
  !
        mess/out 4 {PROID} "    loading rebinned image into display 2"
  assign/display d,2 >Null
  clear/channel overlay
  load/image {rebimg} ? {outputi(1)},{outputi(2)} c,c  !### 2)},max ###
  label/display "REBINNED" {nice_dlab}  ! removed by special load/image
  label/display "(y-averaged)" 10,10 ? green 1
  !
  ! RECTANGLES AROUND ORDERS:
  ! ------------------------
  @a getcoord {rebimg} @1,@1 >Null
  inputi(2) = outputi(2)
  @a getcoord {rebimg} @1,@2 >Null
  scrdif = (outputi(2)-inputi(2))/2.
  do ordloop = 1 totordno
    @a getcoord {rebimg} @1,@{ordloop} >Null
    inputi(1) = outputi(1)
    inputi(2) = outputi(2)-scrdif
    @a getcoord {rebimg} @{{rebimg},NPTOT({ordloop})},@{ordloop} >Null
    outputi(2) = outputi(2)+scrdif
    draw/rectangle {inputi(1)},{inputi(2)},{outputi(1)},{outputi(2)} -
      s ? green >Null
  enddo
  !
  ! LABELS WITH RELATIVE ORDER NUMBERS:
  ! ----------------------------------------------------------------
  ! scry - scry0 = nint[ (frmy - frmy0) /scaly ] ,
  !   with  scry0 = nice_dscr(2)     (screen pixel in window centre)
  !   and   frmy0 = DISPLAY_DATA(4)  (frame  pixel in window centre
  !                                   after `load/ima ... c,c')
  !   and   scaly =     DISPLAY_DATA(2), if positive
  !                 -1./DISPLAY_DATA(2), if negative
  ! ----------------------------------------------------------------
  scry0 = nice_dscr(2)
  frmy0 = {{rebimg},DISPLAY_DATA(4)}
  dscly = {{rebimg},DISPLAY_DATA(2)}
  if dscly .ge. 1 then  ! 1,2,3,...
    scaly = dscly
  else                  ! ...,-3,-2 ; 0 not possible, -1 doesn't exist
    scaly = -1./dscly
  endif
  labobjxl = 6
  do ordloop = 1 totordno
    frmy  = ordloop+0.5
    scry = m$nint((frmy-frmy0)/scaly-0.5)+scry0-7
    if ordloop .lt. 10 then
      label/display "  {ordloop}" {scry},{labobjxl} ? cyan 1
    else
      label/display "{ordloop}" {scry},{labobjxl} ? cyan 1
    endif
  enddo
  !
  !
endif
!
!
!          VERIFICATION OF OVERLAP AREAS :
!          *****************************
!
        mess/out 3 {PROID} "  calculating deviations of overlap areas"
!
!****************************
@p fors_reborder_olp {rebimg}
!****************************
!
!
if nice_crea .eq. "y" then
  !
  !
  !        COMBINED PLOTTING OF ALL ORDERS :
  !        *******************************
  !
        mess/out 4 {PROID} "  plotting all orders"
  !
  copy/it {rebimg} tmp.tbl :LAM_REL
  set/format I1
  do ordloop = 1 totordno
    ordcol = ordloop+1
    select/table tmp.tbl :LAB00{ordcol}.gt.0. >Null
    statistics/table tmp.tbl :LAB00{ordcol} >Null
    intlow = outputr(3)-0.25*outputr(4)
    if ordloop .eq. 1 ymin = intlow
    if intlow .lt. ymin ymin = intlow
    inthigh = outputr(3)+0.25*outputr(4)
    if inthigh .gt. ymax ymax = inthigh
    statistics/table tmp.tbl :LAM_REL >Null
    if ordloop .eq. 1 then
      lamblow = outputr(1)+{{rebimg},WSTART({ordloop})}
    elseif ordloop .eq. totordno then
      lambhigh = outputr(2)+{{rebimg},WSTART({ordloop})}
    endif
  enddo
  -delete tmp.tbl
  assign/graphic g,2
  set/graphic xaxis={lamblow},{lambhigh} yaxis={ymin},{ymax} pmode=0
  !
  -copy {rebimg} tmp.bdf
  tmp.bdf,LHCUTS(1) = 0.
  tmp.bdf,LHCUTS(2) = 0.
  do ordloop = 1 totordno
    tmp.bdf,START(1) = {tmp.bdf,WSTART({ordloop})}
    if ordloop .eq. 1 then
      set/graphic colour=0
      plot/row tmp.bdf @1
      set/graphic colour=1
      plot/axes {plrgrap(1)},{plrgrap(2)},{plrgrap(3)},{plrgrap(4)} -
                {plrgrap(5)},{plrgrap(6)},{plrgrap(7)},{plrgrap(8)} -
                ? "Wavelength (Angstrom)" "Flux"
      overplot/row tmp.bdf @1
      colour = 1
    else
      if colour .eq. 1 then
        colour = 2
      else
        colour = 1
      endif
      set/graphic colour={colour}
      overplot/row tmp.bdf @{ordloop}
    endif
  enddo
  -delete tmp.bdf
  !
  !
endif
!
!
set/format
if nice_crea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0 {rebimg}
else
  return -1 {rebimg}
endif
!
error:
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" set/graphic default
  endif
  return 1 -99
!
!
!------------------------------------------------------------------------------
