! $Id: fors_reborder_olp.prg,v 1.3 1999/11/25 11:24:15 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_reborder_olp.prg,v $
!
! .KEYWORDS   Rebinning test
!
! .PURPOSE    Calculate deviations of overlap areas. Areas and deviations
!             can be plotted.
! __________
!
! .USAGE      @p fors_reborder_olp rebimg
!
! .INPUT      rebimg (P1): rebinned order frame;
!                            default: none
!
! .OUTPUT     -
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  deviation = common_overlap(ord.i+1)/common_overlap(ord.i)-1.
!
! .COMMENT    Special procedure for rebinning - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.5  99/11/08  Creation
!                  99/       Release
!
!------------------------------------------------------------------------------
!
!
crossref rebimg
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "reborder_olp"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      rebimg (P1): rebinned order frame;"
  w/o "                     default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(rebimg):
!----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of rebinned order frame: "
define/local rebimg/c/1/64 "{p1}"
if m$index(rebimg,".bdf") .eq. 0 rebimg = rebimg//".bdf"
!
if m$exist(rebimg) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({rebimg})"
  goto error
endif
if m$existd(rebimg,"WSTART") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Descriptor is not present !"
        mess/out 1 {PROID} "({rebimg},WSTART)"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
define/local totordno/i/1/1 0
define/local lastbutone/i/1/1 0
define/local ordloop/i/1/1 0
define/local nextord/i/1/1 0
define/local intlow/r/1/2 0. all
define/local inthigh/r/1/2 0. all
define/local lamblow/r/1/2 0. all
define/local lambhigh/r/1/2 0. all
define/local xmin/r/1/1 0.
define/local xmax/r/1/1 0.
define/local ymin/r/1/1 0.
define/local ymax/r/1/1 0.
define/local xrat/r/1/1 0.
define/local yrat/r/1/1 0.
define/local XSZ1/r/1/1 0.
define/local YSZ1/r/1/1 0.
define/local XOF1/r/1/1 0.
define/local YOF1/r/1/1 0.
define/local XSZ2/r/1/1 0.
define/local YSZ2/r/1/1 0.
define/local XOF2/r/1/1 0.
define/local YOF2/r/1/1 0.
!
!
!
!          L O O P   W I T H   O R D E R S :
!          * * * * * * * * * * * * * * * *
!
!
set/graphic default
set/graphic pmode=0
!
totordno = {{rebimg},NPIX(2)}
lastbutone = totordno-1
do ordloop = 1 lastbutone
  nextord = ordloop+1
  !
  !
  if nice_crea .eq. "y" then
    !
    !      OVERLAP AREAS :
    !      *************
    !
        mess/out 4 {PROID} "    plotting overlap"
    !
    ! RANGES OF AXES:
    ! --------------
    copy/it {rebimg} tmp.tbl :LAM_REL
    set/format I1
    inputi = ordloop+1
    select/table tmp.tbl :LAB00{inputi}.gt.0. >Null
    statistics/table tmp.tbl :LAB00{inputi} >Null
    intlow(1) = outputr(3)-0.25*outputr(4)
    inthigh(1) = outputr(3)+0.25*outputr(4)
    statistics/table tmp.tbl :LAM_REL >Null
    lamblow(1) = outputr(1)+{{rebimg},WSTART({ordloop})}
    lambhigh(1) = outputr(2)+{{rebimg},WSTART({ordloop})}
    inputi = nextord+1
    select/table tmp.tbl :LAB00{inputi}.gt.0. >Null
    statistics/table tmp.tbl :LAB00{inputi} >Null
    intlow(2) = outputr(3)-0.25*outputr(4)
    inthigh(2) = outputr(3)+0.25*outputr(4)
    statistics/table tmp.tbl :LAM_REL >Null
    lamblow(2) = outputr(1)+{{rebimg},WSTART({nextord})}
    lambhigh(2) = outputr(2)+{{rebimg},WSTART({nextord})}
    if lamblow(1) .le. lamblow(2) then
      xmin = lamblow(1)
    else
      xmin = lamblow(2)
    endif
    if lambhigh(1) .ge. lambhigh(2) then
      xmax = lambhigh(1)
    else
      xmax = lambhigh(2)
    endif
    if intlow(1) .le. intlow(2) then
      ymin = intlow(1)
    else
      ymin = intlow(2)
    endif
    if inthigh(1) .ge. inthigh(2) then
      ymax = inthigh(1)
    else
      ymax = inthigh(2)
    endif
    set/graphic xaxis={xmin},{xmax} yaxis={ymin},{ymax}
    !
    ! COORDINATES OF PLOT AREA:
    ! ------------------------
    assign/graphic g,1 >Null
    show/display >Null
    xrat = outputi(13)/625. ! ### ### ###
    yrat = outputi(14)/425. ! ### ### ###
    XSZ1 = -150.*xrat
    YSZ1 = -70.*yrat
    XOF1 = 25.*xrat
    YOF1 = 50.*yrat
    set/graphic cleargra=on colour=1
    plot/axes ? ? {XSZ1},{YSZ1},{XOF1},{YOF1} -
      "Wavelength (Angstrom)" "Flux"
    !
    ! X-SHIFTS / PLOTS:
    ! ----------------
    -copy {rebimg} tmp.bdf
    tmp.bdf,LHCUTS(1) = 0.
    tmp.bdf,LHCUTS(2) = 0.
    tmp.bdf,START(1) = {tmp.bdf,WSTART({ordloop})}
    overplot/row tmp.bdf @{ordloop}
    inputr(1) = plrgrap(1)+0.5*plrgrap(4)
    inputr(2) = ymax-2.*plrgrap(8)
    label/graphic "Rel.order {ordloop}" {inputr(1)},{inputr(2)} 0 1 1
    tmp.bdf,START(1) = {tmp.bdf,WSTART({nextord})}
    set/graphic colour=2
    overplot/row tmp.bdf @{nextord}
    inputr(1) = plrgrap(1)+0.5*plrgrap(4)
    inputr(2) = ymax-plrgrap(8)
    label/graphic "Rel.order {nextord}" {inputr(1)},{inputr(2)} 0 1 1
    !
    !
    !      DEVIATIONS OF OVERLAP AREAS :
    !      ***************************
    !
          mess/out 4 {PROID} "    plotting deviation of overlap"
    !
    ! RANGES OF AXES:
    ! --------------
    set/graphic yaxis=-1.,1.
    !
    ! COORDINATES OF PLOT AREA:
    ! ------------------------
    XSZ2 = XSZ1
    YSZ2 = -25.*yrat
    XOF2 = XOF1
    YOF2 = 10.*yrat
    set/graphic cleargra=off colour=1
    plot/axes ? ? {XSZ2},{YSZ2},{XOF2},{YOF2} -
      "Wavelength (Angstrom)" "Rel.deviation"
    !
  endif
    !
    ! DEVIATION IMAGE:
    ! ---------------
    extract/image tmpa.bdf = {rebimg} [<,@{ordloop}:>,@{ordloop}]
    tmpa.bdf,LHCUTS(1) = 0.
    tmpa.bdf,LHCUTS(2) = 0.
    tmpa.bdf,START(1) = {{rebimg},WSTART({ordloop})}
    extract/image tmpb.bdf = {rebimg} [<,@{nextord}:>,@{nextord}]
    tmpb.bdf,LHCUTS(1) = 0.
    tmpb.bdf,LHCUTS(2) = 0.
    tmpb.bdf,START(1) = {{rebimg},WSTART({nextord})}
    compute/image tmpc.bdf = tmpb.bdf/tmpa.bdf-1. >Null  ! shifted x-start vals.
    !                                                    ! are considerd
  if nice_crea .eq. "y" then
    !
    ! PLOTS:
    ! -----
    overplot/row tmpc.bdf @1
    inputr(1) = plrgrap(1)+0.5*plrgrap(4)
    inputr(2) = plrgrap(6)-3.*plrgrap(8)
    set/graphic colour=2
    label/graphic "o.{nextord}/o.{ordloop}-1" {inputr(1)},{inputr(2)} 0 2 1
    !
  endif
    !
          mess/out 4 {PROID} "    calculating median +- med.abs.dev."
    !
    copy/it tmpc.bdf tmpc.tbl
    select/table tmpc.tbl :LAB001.gt.-1. >Null
    copy/ti tmpc.tbl tmpd.bdf
    statistics/image tmpd.bdf ? ? ? wn
    !
          set/format J2 T6.3,
          inputi = {{rebimg},NORDER({ordloop})}
          mess/out 3 {PROID} -
            "    abs.ord. {inputi}:  {outputr(8)}  +- {outputr(14)}  med.dev."
          set/format {FMTDEF}
  !
  !
enddo
!
-delete tmp*.bdf tmp*.tbl
set/graphic
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!------------------------------------------------------------------------------
