! $Id: fors_rebtbl.prg,v 1.1 1999/03/03 17:14:23 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999 European Southern Observatory
!
!.IDENT       $RCSfile: fors_rebtbl.prg,v $
!
!.KEYWORDS    Linear rebinning, flux conservation, tables
!
!.USAGE       REBIN/TABLE  intbl ingrd outtbl refgrd method reftbl
!
!.INPUT       INTBL     (P1)   Input table to be rebinned.
!             INGRD     (P2)   Input grid specification, i.e column
!                              references for independent variable, 
!                              dependent variable and bin size.
!             REFGRD    (P4)   Reference for the new data grid. This may
!                              be a list of values interpreted as start, step
!                              and number of bins or a list of column
!                              references for the independent and dependent
!                              variable and the new bin size.
!             METHOD    (P5)   Rebinning method (PIXEL, LINEAR).
!             REFTBL    (P6)   Reference table name to look up the column 
!                              references given in REFGRD.
!
!.OUTPUT      OUTTBL    (P3)   Rebinned output table.
!
!.RETURNS     The procedure returns 0 on success or 1 on error. The exit
!             status is stored in keyword Q1.
!
!.PURPOSE     The procedure rebins the binned 1d data set stored in the
!             input table and specified by the list of column references
!             provided by INGRD according to the new grid given by REFGRD.
!             The new grid specification may be a list of values which are
!             interpreted as start, step and number of bins or a list of
!             column references which provide the values for independent
!             variable and bin width of the new grid. In the latter case
!             a reference table is expected to be given as REFTBL from which
!             the new grid is read. The rebinning method is provided by
!             METHOD. Supported methods are PIXEL and LINEAR.
!             The data grids must be sorted in ascending order and have to
!             be continuous. Furthermore the dependent variable of the input
!             grid is assumed to be the integral over the bin, normalized to
!             the binwidth. Similarly, the output data is normalized to the
!             binwidth of the new grid. The independent variable is assumed to
!             refer to the center of the bins.
!             
!.ENVIRON     MIDAS 98NOV, FORS pipeline
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.1 1998/02/28: Creation, RP
!
!------------------------------------------------------------------------------

crossref  intbl ingrd outtbl refgrd method reftbl

define/param p1 ? c "Input table:"
define/param p2 ? c "Input column references:"
define/param p3 ? c "Output table:"
define/param p4 ? c "New grid specification:"
define/param p5 + c "Rebinning method:"
define/param p6 + c "Reference table:"
define/maxpar 6

! Procedure identification
define/local procID/c/1/20      "RebinTbl"

! Local variables
define/local intbl/c/1/100   {p1}
define/local ingrd/c/1/100   {p2}
define/local outtbl/c/1/100  {p3}
define/local refgrd/c/1/100  {p4}
define/local method/c/1/100  {p5}
define/local reftbl/c/1/100  {p6}



! Process parameters
ingrd = m$upper(ingrd)
method = m$upper(method)

if refgrd(1:1) .eq. ":" then
  if reftbl(1:1) .eq. "+" .or. m$exist(reftbl) .eq. 0 then
    mess/out 1 {procID} "Invalid reference table!"
    return 1
  endif
endif

run PIPE_EXE:fors_rebtbl.exe
if {q1} .ne. 0 then
  mess/out 1 {procId} "Rebinning of >{intbl}< failed!"
  return 1
endif

return 0
