! $Id: fors_redinfo.prg,v 1.17 1999/11/30 10:40:02 sbogun Exp $   
!+++ FORS_REDINFO ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         fors_redinfo.prg
!
!.KEYWORDS      Put reduction information of result frame on the terminal
!
!.USAGE         @p (IDENT) inframe messlev procid    
!
!.INPUT         inframe (P1): name of frame of which info should be displayed
!               datatyp (P2): data type of inframe
!               scale   (P3): scaled master bias has been used for reduction
!
!.OUTPUT        
!
!.RETURN        
!
!.DEFAULTS      P2:     3
!               P3:     NO
!
!.PURPOSE       Display all reduction steps made to produce P1
!
!.ALGORITHM     Check status descriptor and put appropriate information on
!               the terminal using message/out.
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/10/01  Creation
!               1.2 1999/02/03  set recipe status (RECQUALI, RECQUALM)
!               1.3 1999/02/22  do not check for bad pix. corr. in case of sci.
!
!.COMMENT
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name of reduced frame: "
define/param p2 3 c "Enter data type of reduced frame: "
define/param p3 NO c "Scaling method for master bias (MED/OVE/NO):"


define/local i1/i/1/1 0                         !do-counter
define/local j1/i/1/1 0                         !do-counter
define/local datt/i/1/1 0                       !data type
define/local status/i/1/10 0 ALL                !status
define/local fftype/c/1/32 " " ALL              !flatfield type
define/local mes/c*80/1/8 " " ALL
define/local SC_BSTPL/c/1/100 " " ALL
define/local nport/i/1/1   0                    !no of ports

! Get number of readout ports
nport = m$value({p1},{h_nout})
if nport .ne. 1 .and. nport .ne. 4 then
  mess/out 1 redinfo "Invalid readout port setup!"
  return 1
endif

if m$existd(P1,"CCDSTAT") .le. 0 return -1      !descr. CCDSTAT does not exist
copy/dk {P1} CCDSTAT status

if status(1) .eq. 1 shortlog/out 1 "Pre/overscan subtraction done."

if status(2) .eq. 1 shortlog/out 1 "Pre- and overscan regions chopped off."

!if status(3) .eq. 1 shortlog/out 1 "Bad pixel correction done."

if status(4) .eq. 1 then 
  if m$tstno(SC_BSFRM) .ne. 1 then
    if p3 .eq. "MED" then
      SC_BSTPL = SC_BSFRM(4:)
      longlog/out 1 "     Master Bias frame subtracted: {SC_BSTPL}"
      if nport .eq. 4 then
        longlog/out 1 "       (each quadrant divided by its median and multiplied by the"
        longlog/out 1 "       corresponding pre- or overscan value of the input frame)"
      else
        longlog/out 1 "       (divided by its median and multiplied by the average of pre-"
        longlog/out 1 "       and overscan values of the input frame)"
      endif
    else if p3 .eq. "OVE" then
      SC_BSTPL = SC_BSFRM(4:)
      longlog/out 1 "     Master Bias frame subtracted: {SC_BSTPL}"
      if nport .eq. 4 then
        longlog/out 1 "       (each quadrant divided by its corresponding pre- or overscan"
        longlog/out 1 "       value and multiplied by the corresponding pre- or overscan value"
        longlog/out 1 "       of the input frame)"
      else
        longlog/out 1 "       (divided by the average of its pre- and overscan values and multiplied"
        longlog/out 1 "       by the average of pre- and overscan values of the input frame)"
      endif
    else
      longlog/out 1 "     Master Bias frame subtracted: {SC_BSFRM}"
    endif
  else
    longlog/out 1 "     Constant BIAS value subtracted: {SC_BSFRM}"
  endif
  shortlog/out 1 "Bias correction done."
endif

if status(5) .eq. 1 then 
  if m$tstno(SC_DKFRM) .ne. 1 then
    longlog/out 1 "     Master Dark frame subtracted: {SC_DKFRM}"
    longlog/out 1 "     (scaled to exposure time of raw frame)"
  else
    longlog/out 1 "     Constant DARK value subtracted: {SC_DKFRM}"
  endif
  shortlog/out 1 "Dark correction done."
endif

if status(6) .eq. 1 then 
  fftype = "{{SC_FFFRM},{FORS_IDSTRNG}}"
  branch fftype MASTER_SKY_FLAT_IMG,MASTER_NIGHT_FLAT_IMG,SUPER_SKY_FLAT_IMG,SUPER_NIGHT_FLAT_IMG -
                MSKY,MNGT,SSKY,SNGT
  goto CONT

MSKY:
    longlog/out 1 -
      "     Used flatfield type: MASTER_SKY_FLAT_IMG"
    goto CONT

MNGT:
    longlog/out 1 -
      "     Used flatfield type: MASTER_NIGHT_FLAT_IMG"
    goto CONT

SSKY:
    longlog/out 1 -
      "     Used flatfield type: SUPER_SKY_FLAT_IMG"
    longlog/out 1 -
      "       (containing low spatial frequencies from a MASTER_SKY_FLAT_IMG"
    longlog/out 1 -
      "       + high spatial frequencies from a MASTER_SCREEN_FLAT_IMG)"
    goto CONT

SNGT:
    longlog/out 1 -
      "     Used flatfield type: SUPER_NIGHT_FLAT_IMG"
    longlog/out 1 -
      "       (containing low spatial frequencies from a MASTER_NIGHT_FLAT_IMG"
    longlog/out 1 -
      "       + high spatial frequencies from a MASTER_SCREEN_FLAT_IMG)"
    goto CONT

CONT:
  longlog/out 1 "     Master Flatfield frame used: {SC_FFFRM}"
  longlog/out 1 "       (raw frame was divided by flat normalized to one.)"
  shortlog/out 1 "Flat field correction done."
endif

if status(7) .eq. 1 then
  longlog/out 1 "     Illumination correction frame used: {SC_ILFRM}"
  shortlog/out 1 "Illumination correction done."
endif

if status(8) .eq. 1 then
  longlog/out 1 "     Fringe correction frame used: {SC_FRFRM}"
  shortlog/out 1 "Fringe correction done."
endif

j1 = status(1) + status(2) + status(3) + status(4) 
j1 = j1 + status(5) + status(6) + status(7) + status(8) 

if j1 .eq. 0 then
  shortlog/out 1 "NO reduction steps had been done !"
  RECQUALI = 1                                          !recipe quality status
  goto fin
endif

datt = {P2}
if datt .lt. 0 datt = 0
branch datt 0,1,2,3,4,5 SCI,BIA,DRK,SKY,SCR,STD

mess/out 2 redinfo "unknown data type for recipe status check."
goto fin

SCI:
STD:
!  i1 = status(3) * status(4) * status(6) DO not check for bad-pix. correction
  i1 = status(4) * status(6)
  if i1 .eq. 0 RECQUALI = 1
  goto fin
BIA:
  if status(3) .eq. 0 RECQUALI = 1
  goto fin

DRK:
SCR:
SKY:
  i1 = status(3) * status(4)
  if i1 .eq. 0 RECQUALI = 1
  goto fin


fin:
