! $Id: fors_remout.prg,v 1.19 1999/11/25 11:24:06 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_remout.prg,v $
!
! .KEYWORDS   Statistics
!
! .PURPOSE    Handle in- and output for "fors_remout.c" and invoke it.
! __________
!
! .USAGE      @p fors_remout ima [scan] [thresh] [out]
!
! .INPUT      ima    (P1): input frame;
!                            default: none
!
!             scan   (P2): number of pre- AND overscan columns, separated by a
!                          comma;
!                            default: 16,16
!
!             thresh (P3): lower AND upper float threshold (ADU) for outliers,
!                          separated by a comma - pixels outside of the interval
!                          are ignored;
!                            default: -65534.,65534.
!
!             out    (P4): flag for result output on terminal: <>:show, <n>o;
!                            default: <>
!
! .OUTPUT     OUT_A : name of extr./filt. 1D frame
!
!             OUTPUTI(1): Number of all pixels without pre-/overscan
!             OUTPUTI(2): Number of outliers
!             OUTPUTI(3): Number of left pixels
!
!             OUTPUTR(1): Percentage of outliers
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    -
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/09/21  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref ima scan thresh out
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "remout"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!                                ! also needed for "fors_remout.c"
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      ima    (P1): input frame;"
  w/o "                     default: none"
  w/o " "
  w/o "      scan   (P2): number of pre- AND overscan columns, separated by a"
  w/o "                   comma;"
  w/o "                     default: 16,16"
  w/o " "
  w/o "      thresh (P3): lower AND upper float threshold (ADU) for outliers,"
  w/o "                   separated by a comma - pixels outside of the interval"
  w/o "                   are ignored;"
  w/o "                     default: -65534.,65534."
  w/o " "
  w/o "      out    (P4): flag for result output on terminal: <>:show, <n>o;"
  w/o "                     default: <>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of frame: "
write/keyword in_a "{p1}"
if m$index(in_a,".bdf") .eq. 0 in_a = in_a//".bdf"
!
if m$exist(in_a) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({in_a})"
  goto error
endif
!
!
define/parameter p2 16,16 ? -
  "        Enter pre-/overscan values [def.:16,16]: "
outputi = m$parse(p2,"scan")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
write/keyword inputi/i/1/1 {scan01}
write/keyword inputi/i/2/1 {scan02}
!
if inputi(1) .lt. 0 .or. inputi(2) .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible pre- or overscan value !"
  goto error
endif
!
!
define/parameter p3 -65534.,65534. n -
  "        Enter low,high thresh. f. filt. outl. [def.:-65534.,+~]: "
outputi = m$parse(p3,"thresh")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P3 !"
  goto error
endif
write/keyword inputr/r/1/1 {thresh01}
write/keyword inputr/r/2/1 {thresh02}
!
if inputr(1) .ge. inputr(2) then
        write/out " "
        mess/out 1 {PROID} "Impossible thresh. f. filt. outl. !"
  goto error
endif
!
!
define/parameter p4 y ? -
  "        Enter flag for result output [def.:show|<n>o]: "
define/local show/c/1/1 "{p4}"
!
!
!          INVOCATION :
!          **********
!
!***************************
run PIPE_EXE:fors_remout.exe
!***************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Outlier removal failed !"
  goto error
endif
!
!
!          OUTPUT :
!          ******
!
if show .ne. "n" .and. mess_level .ge. 3 then
        mess/out 3 {PROID} " "
        set/format F8.4
        mess/out 3 {PROID} "Results:"
        mess/out 3 {PROID} "-------"
        mess/out 3 {PROID} -
          "ALL PIXELS: {outputi(1)} / OUT: {outputi(2)} / LEFT: {outputi(3)}"
        mess/out 3 {PROID} -
          "OUTLIERS: {outputr(1)}%"
endif
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------
