! $Id: fors_remstar.prg,v 1.14 2001/04/10 09:49:24 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_remstar.prg,v $
!
! .KEYWORDS   Sky flatfields
!
! .PURPOSE    Subtract objects from input frames.
! __________
!
! .USAGE      @p fors_remstar icat [sext] [over] [disp] [scan]
!
! .INPUT      icat (P1): image catalogue with input frames;
!                          default: none
!
!             sext (P2): 4 ascii files for SExtractor (input parameters, con-
!                        volution mask, output parameters), separated by
!                        commas;
!                          default: sex__inpar.dat,sex_convol.dat,sex_out-
!                                   par.dat,sex_neupar.dat
!
!             over (P3): output flag: <>:object subtraction is stored in a new
!                        frame, <overwrite>(no abbreviation !):input image is
!                        overwritten;
!                          default: <>
!
!             disp (P4): display selection: <p>:pipeline mode:read global key
!                        NICE_CREA [y|n], <n>:none, <l>:local, or
!                        host:display[.screen] - with the latter option
!                        you can specify the name or IP-address of a display;
!                          default: <l>
!
!             scan (P5): numbers of pre- AND overscan columns, separated by
!                        a comma (only for calc. of cut vals.);
!                          default: 16,16
!                          hint: ignored, if P4=n
!
! .OUTPUT     "<P1-entries>_objmap.bdf":  frames with found objects
!
!             If P3 != "overwrite":
!             "<P1-entries>_objsub.bdf":  object-subtracted frames
!
!             If objects were found:
!
!               "<P1-entries>_obj.tbl":  1 XPIX,
!                (`N':do NOT change)     2 YPIX    : POSITIONS
!                                       ----------
!                                        3 FLUX,     (sky-subtracted)
!                                        4 SKY     : BRIGHTNESSES
!                                       ----------
!                                        5 ELL_A   : maj.ell.half-axis (1sigma)
!                                        6 ELL_B   : min.ell.half-axis (1sigma)
!                                     N  7 ROTANG  : angle(maj.axis,x-axis)
!                                       ==========
!                                     N  8 XSTARTPIX,
!                                     N  9 YSTARTPIX,
!                                     N 10 XENDPIX,
!                                     N 11 YENDPIX : rectangle surround. ellip.
!
!               Hint for columns 7-11 (object marks):
!                 With `draw/ellipse <P1-entries>_obj.tbl p ? green' the
!                 found objects can be marked onto the displayed current
!                 <P1-entry> frame.
!
!             Q2: tot. number of skipped frames, because no objects were found
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  SExtractor v2.0.15 - see corresponding documents.
!
! .COMMENT    The ascii files with input parameters and output parameters (P2)
!             must NOT be changed; otherwise this programme must be altered
!             partially (see comments in section "Photometry" and numbers in
!             section "Creation of output table") !!!
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/01/25  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref icat sext over disp scan
!
!
define/local ELLSIZE/r/1/1  7.  ! # of sigmas of half-axes for drawing ell.
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "remstar"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      icat (P1): image catalogue with input frames;"
  w/o "                   default: none"
  w/o " "
  w/o "      sext (P2): 4 ascii files for SExtractor (input parameters, con-"
  w/o "                 volution mask, output parameters), separated by a"
  w/o "                 comma;"
  w/o "                   default: sex__inpar.dat,sex_convol.dat,sex_out-"
  w/o "                            par.dat,sex_neupar.dat"
  w/o " "
  w/o "      over (P3): output flag: <>:object subtraction is stored in a new"
  w/o "                 frame, <overwrite>(no abbreviation !):input image is"
  w/o "                 overwritten;"
  w/o "                   default: <>"
  w/o " "
  w/o "      disp (P4): display selection: <p>:pipeline mode:read global key"
  w/o "                 NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                 host:display[.screen] - with the latter option"
  w/o "                 you can specify the name or IP-address of a display;"
  w/o "                   default: <l>"
  w/o " "
  w/o "      scan (P5): numbers of pre- AND overscan columns, separated by"
  w/o "                 a comma (only for calc. of cut vals.);"
  w/o "                   default: 16,16"
  w/o "                   hint: ignored, if P4=n"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(icat):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter name of catalogue (BDF): "
define/local icat/c/1/64 "{p1}"
if m$index(icat,".cat") .eq. 0 icat = icat//".cat"
!
if m$exist(icat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
show/icat {icat} n
if outputi(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is empty !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
define/local catal/i/1/1 0
inloop:
  store/frame inputc {icat} 1 end_inloop
  if m$exist(inputc) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "At least one catalogue entry is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
    goto error
  endif
  goto inloop
end_inloop:
!
!
!P2(sext):
!--------
define/parameter p2 sex__inpar.dat,sex_convol.dat,sex_outpar.dat,sex_neupar.dat -
  ? "        4 ASCII files for SExtractor [if different from def.]: "
outputi = m$parse(p2,"sex")
if outputi(1) .ne. 3 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local inparfile/c/1/64 "{sex01}"
if m$index(inparfile,".") .eq. 0 inparfile = inparfile//".dat"
define/local convolfile/c/1/64 "{sex02}"
if m$index(convolfile,".") .eq. 0 convolfile = convolfile//".dat"
define/local outparfile/c/1/64 "{sex03}"
if m$index(outparfile,".") .eq. 0 outparfile = outparfile//".dat"
define/local neuparfile/c/1/64 "{sex04}"
if m$index(neuparfile,".") .eq. 0 neuparfile = neuparfile//".dat"
!
if m$exist(inparfile) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor input parameters"
        mess/out 1 {PROID} "is not present or can not be opened !"
        mess/out 1 {PROID} "({inparfile})"
  goto error
endif
!
if m$filtyp(inparfile," ") .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor input parameters"
        mess/out 1 {PROID} "is no ascii file ({inparfile}) !"
  goto error
endif
!
if m$exist(convolfile) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor convolution mask"
        mess/out 1 {PROID} "is not present or can not be opened !"
        mess/out 1 {PROID} "({convolfile})"
  goto error
endif
!
if m$filtyp(convolfile," ") .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor convolution mask"
        mess/out 1 {PROID} "is no ascii file ({convolfile}) !"
  goto error
endif
!
if m$exist(outparfile) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor output parameters"
        mess/out 1 {PROID} "is not present or can not be opened !"
        mess/out 1 {PROID} "({outparfile})"
  goto error
endif
!
if m$filtyp(outparfile," ") .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with sextractor output parameters"
        mess/out 1 {PROID} "is no ascii file ({outparfile}) !"
  goto error
endif
!
if m$exist(neuparfile) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with neuronal network weights"
        mess/out 1 {PROID} "is not present or can not be opened !"
        mess/out 1 {PROID} "({neuparfile})"
  goto error
endif
!
if m$filtyp(neuparfile," ") .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file with neuronal network weights"
        mess/out 1 {PROID} "is no ascii file ({neuparfile}) !"
  goto error
endif
!
!
!P3(over):
!--------
define/parameter p3 n ? -
  "        Enter overwrite flag [def.:n|<overwrite>]: "
define/local overout/c/1/9 "{p3}"
!
!
!P4(disp):
!--------
define/parameter p4 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p4}"
!
!
!P5(scan):
!--------
define/parameter p5 16,16 ? -
  "        Enter pre-/overscan vals. [def.:16,16]: "
outputi = m$parse(p5,"over")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P4 !"
  goto error
endif
define/local prescan/i/1/1 {over01}
define/local ovescan/i/1/1 {over02}
!
if prescan .lt. 0 .or. ovescan .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible pre- or overscan value !"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "L o o p" :
  define/local topicima/c/1/64 " " all
  define/local lopcount/i/1/1 0
  !
  !   "Displaying ... original" :
  define/local orilowcut/r/1/1 0.
  define/local orihighcut/r/1/1 0.
  !
  !   "Photometry" :
  define/local char/i/1/1 0
  define/local topicfits/c/1/64 " " all
  define/local photascii/c/1/64 " " all
  define/local objima/c/1/64 " " all
  !
  !   "Checking ... objects" :
  define/local listcode/i/1/2 0 all
  define/local skip/i/1/1 0
  !
  !   "Creation ... table" :
  define/local formcode/i/1/2 0 all
  define/local phottable/c/1/64 " " all
  !
  !   "Creation ... frame" :
  define/local subima/c/1/64 " " all
!
!
!          MANAGING DISPLAYS / ITT / LUT :
!          *****************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_dids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 1 is missing !"
    goto error
  endif
  if nice_dids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 2 is missing !"
    goto error
  endif
  if nice_dids(3) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 3 is missing !"
    goto error
  endif
  !
  assign/display d,1 >Null
  clear/display
  display/lut off
  label/display "ORIGINAL" {nice_dlab}
  assign/display d,2 >Null
  clear/display
  display/lut off
  label/display "FOUND OBJECTS" {nice_dlab}
  assign/display d,3 >Null
  clear/display
  display/lut off
  label/display "OBJECTS REMOVED" {nice_dlab}
  load/lut heat
  load/itt ramp
  if nice_gids(1) .eq. 1 then
    assign/graphic g,1 >Null
    clear/graphic
  endif
  if nice_gids(2) .eq. 1 then
    assign/graphic g,2 >Null
    clear/graphic
  endif
  !
endif
!
!
!
!          L O O P   W I T H   F R A M E S :
!          * * * * * * * * * * * * * * * *
!
!
catal = 0
catloop:
  store/frame topicima {icat} 1 end_catloop
  lopcount = lopcount+1
  show/icat {icat} n
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "image {lopcount} / {outputi(1)} : {topicima}"
  !
  if nice_crea .eq. "y" then
    !
    !
    !      DISPLAYING OF ORIGINAL FRAME :
    !      ****************************
    !
        mess/out 4 {PROID} " "
    !
    !****************************************************************
    @p fors_nicedisp {topicima} ? 7. {prescan},{ovescan} ? ? 0.5,3. n
    !****************************************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of original frame failed !"
        mess/out 1 {PROID} "({topicima})"
      goto error
    endif
    !
        mess/out 4 {PROID} "  loading original image into display 1"
    assign/display d,1 >Null
    load/image {topicima} ? {outputi(1)},avg c,c
    label/display "ORIGINAL" {nice_dlab}  ! removed by special load/image
    !
    !        FOR OBJECT-SUBTRACTED FRAME :
    orilowcut = {{topicima},LHCUTS(1)}
    orihighcut = {{topicima},LHCUTS(2)}
    !
  endif
  !
  !
  !        PHOTOMETRY :
  !        **********
  !
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "  Object detection :"
  !
        mess/out 3 {PROID} "    converting input frame: BDF -> FITS"
        mess/out 3 {PROID} " "
  !
    char = m$indexb(topicima,".")-1
    topicfits = topicima(1:{char})//".fits"
  outdisk/fits {topicima} {topicfits} >Null


  ! Call Sextractor
  $ {p_bindir}/sex {topicfits} -c   {inparfile}  -
                   -FILTER_NAME     {convolfile} -
                   -PARAMETERS_NAME {outparfile} -
                   -CHECKIMAGE_TYPE OBJECTS      -
                   -STARNNW_NAME    {neuparfile} -
                   -CHECKIMAGE_NAME sex_objmap.fits


  if m$exist("sex_output.dat") .eq. 0 then
        mess/out 1 {PROID} "Object detection failed !"
    goto error
  endif
  !
    photascii = topicima(1:{char})//"_phot.dat"
  -rename sex_output.dat {photascii}
  -delete {topicfits}
  !
        mess/out 3 {PROID} "    converting output frame: FITS -> BDF"
  !
    objima = topicima(1:{char})//"_objmap.bdf"
  indisk/fits sex_objmap.fits {objima} >Null
  -delete sex_objmap.fits
  !
  !
        mess/out 4 {PROID} " "
        mess/out 3 {PROID} "  Object elimination :"
  !
  !
  !        CHECKING OF NUMBER OF FOUND OBJECTS :
  !        ***********************************
  !
        mess/out 4 {PROID} "    checking number of found objects"
  !
  open/file {photascii} read listcode
  read/file {listcode(1)} outputc
  if listcode(2) .eq. -1 then
        mess/out 2 {PROID} "    Not a single object found -"
        mess/out 2 {PROID} "    skipping {topicima} !"
    close/file {listcode(1)}
    skip = skip+1
    warn = 1
    !
    goto catloop
  endif
  close/file {listcode(1)}
  !
  !
  !        CREATION OF OUTPUT TABLE :
  !        ************************
  !
        mess/out 4 {PROID} "    creating object table"
    !
    open/file sex.fmt write formcode
    if {formcode(1)} .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open new file sex.fmt !"
      goto error
    endif
    write/file {formcode(1)} define/field  56  66 R*4 F10.3 :XPIX   "pixel"
    write/file {formcode(1)} define/field  67  77 R*4 F10.3 :YPIX   "pixel"
    write/file {formcode(1)} define/field  11  23 R*4 G13.6 :FLUX   "ADU"
    write/file {formcode(1)} define/field  33  45 R*4 G12.6 :SKY    "ADU"
    write/file {formcode(1)} define/field 102 110 R*4 F8.2  :ELL_A  "pix1sig"
    write/file {formcode(1)} define/field 111 119 R*4 F8.2  :ELL_B  "pix1sig"
    write/file {formcode(1)} define/field 120 125 R*4 F6.1  :ROTANG "deg"
    close/file {formcode(1)}
    !
    char = m$indexb(topicima,".")-1
    phottable = topicima(1:{char})//"_obj.tbl"
  create/table {phottable} 11 1 {photascii} sex.fmt  ! 4 cols. added below,
  -delete {photascii}                                ! rows are expanded autom.
  -delete sex.fmt
  !
  copy/dd {topicima} *,5 {phottable}
  !
  !
  !        DISPLAYING OF FOUND OBJECTS :
  !        ***************************
  !
  ! For drawing ellipses around found objects:
  !
  create/column {phottable} :XSTARTPIX "pixel" F12.7 R*8
  create/column {phottable} :YSTARTPIX "pixel" F12.7 R*8
  create/column {phottable} :XENDPIX   "pixel" F12.7 R*8
  create/column {phottable} :YENDPIX   "pixel" F12.7 R*8
  !
  compute/table {phottable} :XSTARTPIX = :XPIX-{ELLSIZE}*:ELL_A
  compute/table {phottable} :YSTARTPIX = :YPIX-{ELLSIZE}*:ELL_B
  compute/table {phottable} :XENDPIX   = :XPIX+{ELLSIZE}*:ELL_A
  compute/table {phottable} :YENDPIX   = :YPIX+{ELLSIZE}*:ELL_B
  !
  if nice_crea .eq. "y" then
    !
    !*************************************************************
    @p fors_nicedisp {objima} ? 7. {prescan},{ovescan} ? ? 1.,3. n
    !*************************************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of found objects failed !"
        mess/out 1 {PROID} "({objima})"
      goto error
    endif
    !
        mess/out 4 {PROID} "    loading image with found objects into disp. 2"
    assign/display d,2 >Null
    clear/channel overlay
    load/image {objima} ? {outputi(1)},avg c,c
    label/display "FOUND OBJECTS" {nice_dlab}  ! after load/image - otherwise
    !                                          ! removed by special load/image
    draw/ellipse {phottable} p n green >Null
    !
  endif
  !
  !
  !        CREATION OF OBJECT-SUBTRACTED FRAME :
  !        ***********************************
  !
        mess/out 3 {PROID} "    creating object-subtracted frame"
  !
  if overout .eq. "overwrite" then
      subima = topicima
  else
      subima = topicima(1:{char})//"_objsub.bdf"
  endif
  compute/pixel {subima} = {topicima} - {objima} >Null
  !
  !
  !        DISPLAYING OF OBJECT-SUBTRACTED FRAME :
  !        *************************************
  !
  if nice_crea .eq. "y" then
    !
    !********************************
    @p fors_nicedisp {subima} n out=n
    !********************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of object-subtracted frame failed !"
        mess/out 1 {PROID} "({subima})"
      goto error
    endif
    !
        mess/out 4 {PROID} "    loading object-subtracted image into display 3"
    assign/display d,3 >Null
    load/image {subima} ? {outputi(1)},avg c,c {orilowcut},{orihighcut}
    label/display "OBJECTS REMOVED" {nice_dlab}  ! removed by special load/image
    !
  endif
  !
  !
  goto catloop
end_catloop:
!
!
!          E N D   O F   L O O P   W I T H   F R A M E S
!          * * * * * * * * * * * * * * * * * * * * * * *
!
!
!
!          OUTPUT :
!          ******
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} ">>> frames  w. found objects are <*_objmap.bdf> <<<"
        mess/out 4 {PROID} ">>> object-subtracted frames are <*_objsub.bdf> <<<"
        mess/out 4 {PROID} ">>> pos. and brightness. are in  <*_obj.tbl>    <<<"
!
set/format
!
!
if warn .eq. 0 then
  return 0 {skip}
else
  return -1 {skip}
endif
!
error:
  set/format
  return 1 -99
!
!
!-------------------------------------------------------------------------------
