! $Id: fors_resol.prg,v 1.19 2001/04/06 16:19:23 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_resol.prg,v $
!
! .KEYWORDS   Spectroscopical quality, wavelength calibration
!
! .PURPOSE    Compute the spectral resolution of lamp emission lines in each
!             slit, and that per bin perpendicular to the dispersion direction.
! __________
!
! .USAGE      @p fors_resol rela slits lines dcoef habi [descs] [disp] [scan]
!
! .INPUT      rela  (P1): input lamp frame (reduced);
!                           default: none
!
!             slits (P2): input table with positions of used slits AND names of
!                         columns with slit_no./slit_y-low_coords./
!                         slit_y-high_coords., separated by commas;
!                           default: none,SLIT,YSTART,YEND
!
!             lines (P3): input table with identified lines AND names of columns
!                         with slit_no./detector_x-pix./det._y-pix./fitted_wave-
!                         lengths/fit_result_flags, separated by commas;
!                           default: none,Slit,X,Y,WAVEC,REJECT
!
!             dcoef (P4): input table with dispersion solution AND name of table
!                         descriptor(!) prefix with wavelength_values (the slit
!                         number is appended) AND name of descriptor with
!                         det._x-pix.vals., separated by commas;
!                           default: none,COEFSLIT,CENTERXY
!
!             habi  (P5): bin-to-bin stepsize AND half binsize perpendicular to
!                         the dispersion direction, separated by a comma,
!                         1 <  step <= half no. of y-pixels,
!                         1 <= habi <  half no. of y-pixels;
!                           example: 25,10
!                           default: none,none
!
!             descs (P6): either: mode:<lss> AND name of descriptor with
!                                 slit width (arcsec!!!)
!                         or:     mode:<mos> AND name of descriptor prefix with
!                                 slit width (arcsec!!!) AND name of descriptor
!                                 suffix (with leading "." if present)
!                                 (the slit number is inserted between)
!                         AND name of descriptor with pixel scale (arcsec/pix.),
!                         separated by commas;
!                           example: mos,ESO.INS.MOS,.WID,ESO.INS.PIXSCALE
!                           default: lss,WID,PIXSCALE
!
!             disp  (P7): display selection: <p>:pipeline mode:read global key
!                         NICE_CREA [y|n], <n>:none, <l>:local, or
!                         host:display[.screen] - with the latter option
!                         you can specify the name or IP-address of a display;
!                           default: <l>
!
!             scan  (P8): number of pre- AND overscan columns, separated by
!                         a comma;
!                           default: 16,16
!                           hint: ignored, if P7=n
!
!             P3 is copied to <P1(no ext.)>_lines.tbl .
!             P4 is copied to <P1(no ext.)>_dcoef.tbl .
!
! .OUTPUT     "<P1>_resol.tbl":    1 SLIT       ,
!              (`d':detect. pix.   2 SLT_STRT d ,
!               = world coords.)   3 SLT_END  d : slit positions
!              (`A':angstrom)     -------------
!              (`n':do NOT change) 4 X        d ,
!                                  5 Y        d ,
!                                  6 WAVEC    A : line positions
!                                  7 REJECT     : "1" = wave calib. O.K.
!                                 -------------
!                               n  8 XSTART   d ,
!                               n  9 XEND     d ,
!                               n 10 YSTART   d ,
!                               n 11 YEND     d : bin positions for fitting
!                                 12 XCEN     d : fit results: line centre ,
!                                 13 STATUS     :              "0" = fit O.K. ,
!                                 14 FWHM     d :              FWHM (per bin)
!                                 -------------
!                                 15 STEP   A/d : dispe.(1.ord.) f.FWHM convers.
!                                 16 D_WAVEC  A : convert.FWHM as "delta_lambda"
!                                 17 SPECRES    : SPECTRAL RESOLUTION (per bin)
!
!               Hint for columns 8-11 (bin marks):
!
!                 With
!                   `select/table <P1>_resol.tbl :REJECT.eq.1.and.:STATUS.eq.0'
!                   `draw/rectangle <P1>_resol.tbl f n green,0'
!                 the used bins can be marked onto the displayed <P1> frame.
!
!               Legend for column 13 (bin qual.flags):
!
!                 <0 : estimation failed (during Gauss fit)
!                  0 : O.K.
!                  1 : ambiguous sources
!                  2 : no source
!                  3 : iteration failed
!                 >3 : routine error
!
!               Formula for column 17 (spectral resolution):
!
!                 SPECRES = WAVEC / D_WAVEC , with D_WAVEC = FWHM * STEP
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/03/05  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref rela slits lines dcoef habi descs disp scan
!
!
if m$existk("XFITFACT") .eq. 0 -
  define/local XFITFACT/r/1/1 3. ? +lower_levels
!                             ! multiplier for fitting slit width
if m$existk("FORS_MAXSLITLET") .eq. 0 -
  define/local FORS_MAXSLITLET/i/1/1 19
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "resol"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      rela  (P1): input lamp frame (reduced);"
  w/o "                    default: none"
  w/o " "
  w/o "      slits (P2): input table with positions of used slits AND names of"
  w/o "                  columns with slit_no./slit_y-low_coords./"
  w/o "                  slit_y-high_coords., separated by commas;"
  w/o "                    default: none,SLIT,YSTART,YEND"
  w/o " "
  w/o "      lines (P3): input table with identified lines AND names of columns"
  w/o "                  with slit_no./detector_x-pix./det._y-pix./fitted_wave-"
  w/o "                  lengths/fit_result_flags, separated by commas;"
  w/o "                    default: none,X,Y,WAVEC,REJECT"
  w/o " "
  w/o "      dcoef (P4): input table with dispersion solution AND name of table"
  w/o "                  descriptor(!) prefix with wavelength_values (the slit"
  w/o "                  number is appended) AND name of descriptor with"
  w/o "                  det._x-pix.vals., separated by commas;"
  w/o "                    default: none,COEFSLIT,CENTERXY"
  w/o " "
  w/o "      habi  (P5): bin-to-bin stepsize AND half binsize perpendicular to"
  w/o "                  the dispersion direction, separated by a comma,"
  w/o "                  1 <  step <= half no. of y-pixels,"
  w/o "                  1 <= habi <  half no. of y-pixels;"
  w/o "                    example: 25,10"
  w/o "                    default: none,none"
  w/o " "
  w/o "      descs (P6): either: mode:<lss> AND name of descriptor with"
  w/o "                          slit width (arcsec!!!)"
  w/o "                  or:     mode:<mos> AND name of descriptor prefix with"
  w/o "                          slit width (arcsec!!!) AND name of descriptor"
  w/o "                          suffix (with leading "." if present)"
  w/o "                          (the slit number is inserted between)"
  w/o "                  AND name of descriptor with pixel scale (arcsec/pix.),"
  w/o "                  separated by commas;"
  w/o "                    example: mos,ESO.INS.MOS,.WID,ESO.INS.PIXSCALE"
  w/o "                    default: lss,WID,PIXSCALE"
  w/o " "
  w/o "      disp  (P7): display selection: <p>:pipeline mode:read global key"
  w/o "                  NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                  host:display[.screen] - with the latter option"
  w/o "                  you can specify the name or IP-address of a display;"
  w/o "                    default: <l>"
  w/o " "
  w/o "      scan  (P8): number of pre- AND overscan columns, separated by"
  w/o "                  a comma;"
  w/o "                    default: 16,16"
  w/o "                    hint: ignored, if P7=n"
  w/o " "
  w/o "      P3 is copied to <P1(no ext.)>_lines.tbl ."
  w/o "      P4 is copied to <P1(no ext.)>_dcoef.tbl ."
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(rela):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of reduced lamp frame: "
define/local lampima/c/1/64 "{p1}"
if m$index(lampima,".bdf") .eq. 0 lampima = lampima//".bdf"
!
if m$exist(lampima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered reduced lamp frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lampima})"
  goto error
endif
!
!
!P2(slits):
!---------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? t -
  "        Enter slit table,cols [def:none,SLIT,YSTART,YEND]: "
outputi = m$parse(p2,"slits")
if outputi(1) .ne. 4 .and. outputi(1) .ne. 1 then  ! 1 allowed for col.defaults
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P2 !"
  goto error
endif
define/local slittbl/c/1/64 "{slits01}"
if outputi(1) .eq. 4 then
  define/local csslit/c/1/16 "{slits02}" ? +lower_levels
  define/local cystart/c/1/16 "{slits03}" ? +lower_levels
  define/local cyend/c/1/16 "{slits04}" ? +lower_levels
  if csslit(1:1) .eq. ":" csslit = csslit(2:)
  if cystart(1:1) .eq. ":" cystart = cystart(2:)
  if cyend(1:1) .eq. ":" cyend = cyend(2:)
elseif outputi(1) .eq. 1 then  ! defaults
  define/local csslit/c/1/16 "SLIT" ? +lower_levels
  define/local cystart/c/1/16 "YSTART" ? +lower_levels
  define/local cyend/c/1/16 "YEND" ? +lower_levels
endif
!
if m$existc(slittbl,":{csslit}") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({slittbl})"
  goto error
elseif m$existc(slittbl,":{csslit}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({slittbl},:{csslit})"
  goto error
elseif m$existc(slittbl,":{cystart}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({slittbl},:{cystart})"
  goto error
elseif m$existc(slittbl,":{cyend}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({slittbl},:{cyend})"
  goto error
endif
!
show/table {slittbl} >Null
define/local totslitno/i/1/1 {outputi(2)} ? +lower_levels
if totslitno .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered slit table is empty !"
        mess/out 1 {PROID} "({slittbl})"
  goto error
endif
!
!
!P3(lines):
!---------
if p3 .eq. "?" mess/out 2 {PROID} " "
define/parameter p3 ? t -
  "        Enter line table,cols [def:none,Slit,X,Y,WAVEC,REJECT]: "
outputi = m$parse(p3,"lines")
if outputi(1) .ne. 6 .and. outputi(1) .ne. 1 then  ! 1 allowed for col.defaults
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P3 !"
  goto error
endif
if m$index(lines01,".tbl") .eq. 0 lines01 = lines01//".tbl"
!
if m$exist(lines01) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lines01})"
  goto error
endif
!
! COPY TABLE:
define/local linetbl/c/1/64 " " all
  define/local char/i/1/1 0
  char = m$indexb(lampima,".")-1
  linetbl = lampima(1:{char})//"_lines.tbl"
if lines01 .ne. linetbl -copy {lines01} {linetbl}
!
if outputi(1) .eq. 6 then
  define/local clslit/c/1/16 "{lines02}" ? +lower_levels
  define/local cx/c/1/16 "{lines03}" ? +lower_levels
  define/local cy/c/1/16 "{lines04}" ? +lower_levels
  define/local cwavec/c/1/16 "{lines05}" ? +lower_levels
  define/local creject/c/1/16 "{lines06}" ? +lower_levels
  if clslit(1:1) .eq. ":" clslit = clslit(2:)
  if cx(1:1) .eq. ":" cx = cx(2:)
  if cy(1:1) .eq. ":" cy = cy(2:)
  if cwavec(1:1) .eq. ":" cwavec = cwavec(2:)
  if creject(1:1) .eq. ":" creject = creject(2:)
elseif outputi(1) .eq. 1 then  ! defaults
  define/local clslit/c/1/16 "Slit" ? +lower_levels
  define/local cx/c/1/16 "X" ? +lower_levels
  define/local cy/c/1/16 "Y" ? +lower_levels
  define/local cwavec/c/1/16 "WAVEC" ? +lower_levels
  define/local creject/c/1/16 "REJECT" ? +lower_levels
endif
!
if m$existc(linetbl,":{clslit}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({linetbl},:{clslit})"
  goto error
elseif m$existc(linetbl,":{cx}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({linetbl},:{cx})"
  goto error
elseif m$existc(linetbl,":{cy}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({linetbl},:{cy})"
  goto error
elseif m$existc(linetbl,":{cwavec}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({linetbl},:{cwavec})"
  goto error
elseif m$existc(linetbl,":{creject}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered column is not present !"
        mess/out 1 {PROID} "({linetbl},:{creject})"
  goto error
endif
!
! DUE TO ERROR IN "mosslin.c" (NULL VALUES POSSIBLE IN :X-COLUMN):
select/table {linetbl} :X.eq.NULL >Null
if outputi(1) .gt. 0 then
        mess/out 4 {PROID} "Skipping {outputi(1)} lines with undefined x-positions !"
endif
select/table {linetbl} :X.ne.NULL >Null
copy/table {linetbl} tmp.tbl
-rename tmp.tbl {linetbl}
!
show/table {linetbl} >Null
define/local totlineno/i/1/1 {outputi(2)} ? +lower_levels
if totlineno .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered line table is empty !"
        mess/out 1 {PROID} "({linetbl})"
  goto error
endif
!
!
!P4(dcoef):
!---------
if p4 .eq. "?" mess/out 2 {PROID} " "
define/parameter p4 ? t -
  "        Enter dispers. tbl.,desc_pref,desc [def:none,COEFSLIT,CENTERXY]: "
outputi = m$parse(p4,"coefs")
if outputi(1) .ne. 3 .and. outputi(1) .ne. 1 then  ! 1 allowed for desc.defaults
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P4 !"
  goto error
endif
if m$index(coefs01,".tbl") .eq. 0 coefs01 = coefs01//".tbl"
!
if m$exist(coefs01) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({coefs01})"
  goto error
endif
!
! COPY TABLE:
define/local dcoeftbl/c/1/64 " " all
  char = m$indexb(lampima,".")-1
  dcoeftbl = lampima(1:{char})//"_dcoef.tbl"
if coefs01 .ne. dcoeftbl -copy {coefs01} {dcoeftbl}
!
if outputi(1) .eq. 3 then
  define/local dlambda/c/1/16 "{coefs02}" ? +lower_levels
  define/local ddetpix/c/1/16 "{coefs03}" ? +lower_levels
elseif outputi(1) .eq. 1 then  ! defaults
  define/local dlambda/c/1/16 "COEFSLIT" ? +lower_levels
  define/local ddetpix/c/1/16 "CENTERXY" ? +lower_levels
endif
!
if m$existd(dcoeftbl,ddetpix) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table descriptor is not present !"
        mess/out 1 {PROID} "({dcoeftbl},{ddetpix})"
  goto error
endif
define/local slitloop/i/1/1 0
define/local slit/i/1/1 0
do slitloop = 1 totslitno
  slit = {{slittbl},:{csslit},{slitloop}}
  if m$existd(dcoeftbl,"{dlambda}{slit}") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Table desc. for entered prefix is not present !"
        mess/out 1 {PROID} "({dcoeftbl},{dlambda}{slit})"
  goto error
  endif
enddo
!
!
!P5(habi):
!--------
if p5 .eq. "?" mess/out 2 {PROID} " "
define/parameter p5 ? ? -
  "        Enter bin_stepsize,half_binsize: "
outputi = m$parse(p5,"binval")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P5 !"
  goto error
endif
define/local binstep/i/1/1 {binval01}
define/local halfbin/i/1/1 {binval02}
!
outputi(1) = {{lampima},NPIX(2)}/binstep
if outputi(1) .ge. 2 then      ! at least 2 bins present
  do outputi(2) = 2 totlineno  ! start is row 2 (either 2nd line or 2nd bin)
    outputi(3) = outputi(2)-1
    outputi(4) = m$nint({{linetbl},:{cy},@{outputi(2)}}-{{linetbl},:{cy},@{outputi(3)}})
    if outputi(4) .ne. 0 outputi(2) = totlineno  ! "break"
  enddo
  if outputi(4) .ne. binstep then
        write/out " "
        mess/out 1 {PROID} "Binning stepsize and line table do not fit !"
    goto error
  endif
endif
!
outputi(1) = m$nint({{lampima},NPIX(2)}/2.)
if binstep .le. 1 .or. binstep .gt. outputi(1) then
        write/out " "
        mess/out 1 {PROID} "Binning stepsize out of range !"
        mess/out 1 {PROID} "({binstep} ; 1<binstep<={outputi(1)})"
  goto error
endif
!
if halfbin .lt. 1 .or. halfbin .ge. outputi(1) then
        write/out " "
        mess/out 1 {PROID} "Half binsize out of range !"
        mess/out 1 {PROID} "({halfbin} ; 1<=halfbin<{outputi(1)})"
  goto error
endif
!
outputi(1) = m$nint((binstep-1)/2.-0.5)
if halfbin .gt. outputi(1) then
        mess/out 3 {PROID} "bin stepsize in y direction: {binstep}"
        mess/out 3 {PROID} "half binsize in y direction: {halfbin}"
        mess/out 3 {PROID} "(bins overlap)"
endif
!
!
!P6(descs):
!---------
define/parameter p6 lss,WID,PIXSCALE ? -
  "        Enter mode,descs.f.slitwid.(arcs!)/pixscl. [def.:lss,WID,PIXSCALE]: "
define/local parsed/i/1/1 0
parsed = m$parse(p6,"slitx")
if parsed .lt. 3 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P6 !"
  goto error
endif
define/local mode/c/1/3 "{slitx01}"
if mode .eq. "lss" then
  !
        mess/out 4 {PROID} "using <{mode}> mode ..."
  define/local slitwid/c/1/72 "{slitx02}"
  define/local pixscale/c/1/72 "{slitx03}"
  if m$existd(lampima,slitwid) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered desc. for slit width does not exist -"
        mess/out 1 {PROID} "({lampima},{slitwid})"
        mess/out 1 {PROID} "no calculation of fit areas possible !"
    goto error
  endif
  if {{lampima},{slitwid}} .le. 0 then
        set/format F15.14
        write/out " "
        mess/out 1 {PROID} "Impossible value for descriptor"
        mess/out 1 {PROID} "`{slitwid}': {{lampima},{slitwid}}"
        set/format {FMTDEF}
    goto error
  endif
  !
elseif mode .eq. "mos" then
  !
        mess/out 4 {PROID} "using <{mode}> mode ..."
  if parsed .ne. 4 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P6 !"
    goto error
  endif
  define/local slitpre/c/1/72 "{slitx02}"
  define/local slitsuf/c/1/72 "{slitx03}"
  define/local slitwid/c/1/72 " " all  ! for below
  define/local pixscale/c/1/72 "{slitx04}"
  !
else
        write/out " "
        mess/out 1 {PROID} "Mode <{mode}> not supported !"
  goto error
endif
!
if m$existd(lampima,pixscale) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered desc. for pixel scale does not exist -"
        mess/out 1 {PROID} "({lampima},{pixscale})"
        mess/out 1 {PROID} "no calculation of fit areas possible !"
  goto error
endif
if {{lampima},{pixscale}} .le. 0 then
        set/format F15.14
        write/out " "
        mess/out 1 {PROID} "Impossible value for descriptor"
        mess/out 1 {PROID} "`{pixscale}': {{lampima},{pixscale}}"
        set/format {FMTDEF}
  goto error
endif
!
!
!P7(disp):
!--------
define/parameter p7 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p7}"
!
!
!P8(scan):
!--------
define/parameter p8 16,16 ? -
  "        Enter pre-/overscan values [def.:16,16]: "
outputi = m$parse(p8,"scan")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P8 !"
  goto error
endif
define/local prescan/i/1/1 {scan01}
define/local ovescan/i/1/1 {scan02}
!
if prescan .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible prescan value ({prescan}) !"
  goto error
endif
if ovescan .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible overscan value ({ovescan}) !"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "W o r k i n g  ...  s l i t s" :
  !
  !   "Extraction" :
  define/local resoltbl/c/1/64 " " all
  !
  !   "Table with coords." :
  define/local bintbl/c/1/64 " " all
  define/local lineloop/i/1/1 0
  define/local halfslit/r/1/{FORS_MAXSLITLET} 0. all
  define/local asec2pix/r/1/1 0.
  define/local x/r/1/1 0.
  define/local x1/r/1/1 0.
  define/local x2/r/1/1 0.
!
!
!
!          MANAGING DISPLAYS / ITT / LUT :
!          *****************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then
  !
  if nice_dids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 1 is missing !"
    goto error
  endif
  if nice_gids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 1 is missing !"
    goto error
  endif
  if nice_gids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 2 is missing !"
    goto error
  endif
  !
  assign/display d,1 >Null
  clear/display
  display/lut off
  label/display "REDUCED LAMP" {nice_dlab}
  if nice_dids(2) .eq. 1 then
    assign/display d,2 >Null
    clear/display
    display/lut off
  endif
  if nice_dids(3) .eq. 1 then
    assign/display d,3 >Null
    clear/display
    display/lut off
  endif
  load/lut heat
  load/itt ramp
  assign/graphic g,1 >Null
  clear/graphic
  assign/graphic g,2 >Null
  clear/graphic
  set/graphic default
  set/graphic pmode=0 bcolour=1
  !
endif
!
!
!          DISPLAYING OF REDUCED LAMP FRAME :
!          ********************************
!
if nice_crea .eq. "y" then
  !
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "displaying reduced lamp frame"
        mess/out 4 {PROID} "  ({lampima})"
  !
  !****************************************************************
  @p fors_nicedisp {lampima} ? 7. {prescan},{ovescan} ? ? 0.,10.5 n
  !****************************************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of reduced lamp frame failed !"
        mess/out 1 {PROID} "({lampima})"
    goto error
  endif
  !
        mess/out 4 {PROID} "  loading reduced lamp image into display 1"
  assign/display d,1 >Null
  load/image {lampima} ? {outputi(1)},avg c,c
  label/display "REDUCED LAMP" {nice_dlab}  ! removed by special load/image
  !
endif
!
!
!
!          W O R K I N G   W I T H   S L I T S :
!          * * * * * * * * * * * * * * * * * *
  !
  !
  !        EXTRACTION FROM INPUT TABLES :
  !        ****************************
  !
    char = m$indexb(lampima,".")-1
    resoltbl = lampima(1:{char})//"_resol.tbl"
  !
  sort/table {slittbl} :{csslit}(-)
  outputi = {{slittbl},:{csslit},1}
  sort/table {slittbl} :{csslit}
  define/local ylow/r/1/{outputi} 0. all +lower_levels
  define/local yhigh/r/1/{outputi} 0. all +lower_levels
  define/local ystartpix/r/1/{outputi} 0. all +lower_levels
  define/local yendpix/r/1/{outputi} 0. all +lower_levels
  !
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "extracting line positions from line list"
        mess/out 4 {PROID} "  ({slittbl} + {linetbl}"
        mess/out 4 {PROID} "   -> {resoltbl})"
  !
  !**********************************************************
  @p fors_resol_exin {slittbl} {linetbl} {lampima} {resoltbl}
  !**********************************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Extraction from input tables failed !"
    goto error
  endif
  !
  copy/dd {lampima} *,5 {resoltbl}
  copy/dd {lampima} IDENT {resoltbl}
  !
  !
  !        CREATION OF TABLE WITH COORDS. OF INPUT FRAME
  !        (X-RANGE OF LINES AND Y-COORDS. OF BINS) :
  !        *********************************************
  !
    char = m$indexb(lampima,".")-1
    bintbl = lampima(1:{char})//"_bins.tbl"
  !
        mess/out 4 {PROID} "Extracting bins around line positions"
        mess/out 4 {PROID} "  (-> {bintbl})"
  !
  create/table {bintbl} 18 {totlineno} null  ! 11 cols. added later
  create/column {bintbl} :SLIT   " "      I4   I*4
  create/column {bintbl} :XSTART "detpix" F8.2 R*4
  create/column {bintbl} :X      "detpix" F8.2 R*4
  create/column {bintbl} :XEND   "detpix" F8.2 R*4
  create/column {bintbl} :YSTART "detpix" F8.2 R*4
  create/column {bintbl} :Y      "detpix" F8.2 R*4
  create/column {bintbl} :YEND   "detpix" F8.2 R*4
  !
  copy/tt {linetbl} :{cx} {bintbl} :X
  asec2pix = {{lampima},{pixscale}}
  !
  if mode .eq. "lss" then
    !
    halfslit = {{lampima},{slitwid}}/2.
    compute/table {bintbl} :XSTART = :X-{XFITFACT}*{halfslit}/{asec2pix}
    compute/table {bintbl} :XEND   = :X+{XFITFACT}*{halfslit}/{asec2pix}
    !
  elseif mode .eq. "mos" then
    !
    set/format I1
    do slitloop = 1 totslitno
      slit = {{slittbl},:{csslit},{slitloop}}   ! slit number
      slitwid = slitpre // "{slit}" // slitsuf  ! descriptor with slit width
      halfslit({slit}) = {{lampima},{slitwid}}/2.
    enddo
    !
    ! Clearer than the substitution below, but only half as fast:
    ! do lineloop = 1 totlineno
    !   x = {{bintbl},:X,{lineloop}}
    !   slit = {{linetbl},:{clslit},{lineloop}}
    !   x1 = x-XFITFACT*halfslit({slit})/asec2pix
    !   x2 = x+XFITFACT*halfslit({slit})/asec2pix
    !   {bintbl},:XSTART,{lineloop} = {x1}
    !   {bintbl},:XEND,{lineloop}   = {x2}
    ! enddo
    !
    copy/tt {linetbl} :{clslit} {bintbl} :SLIT
    select/table {bintbl} seq.eq.0 >Null
    copy/table {bintbl} tmpall.tbl
    do slitloop = 1 totslitno
      slit = {{slittbl},:{csslit},{slitloop}}
      select/table {bintbl} :SLIT.eq.{slit} >Null
      copy/table {bintbl} tmp{slit}.tbl
      compute/table tmp{slit}.tbl :XSTART = :X -
        -{XFITFACT}*{halfslit({slit})}/{asec2pix}
      compute/table tmp{slit}.tbl :XEND   = :X -
        +{XFITFACT}*{halfslit({slit})}/{asec2pix}
      merge/table tmpall.tbl tmp{slit}.tbl tmptmp.tbl
      -rename tmptmp.tbl tmpall.tbl
    enddo
    -rename tmpall.tbl {bintbl}
    -delete tmp*.tbl
    !
  endif
  !
  copy/tt {linetbl} :{cy} {bintbl} :Y
  compute/table {bintbl} :YSTART = :Y-{halfbin}
  compute/table {bintbl} :YEND   = :Y+{halfbin}
  !
  !
  !        COMPUTING SPECTRAL RESOLUTION VALUES
  !        BY FITTING EACH BIN OF THE LINES
  !        (IN DISPERSION DIRECTION) :
  !        ************************************
  !
        mess/out 4 {PROID} "computing spectral resolution"
        mess/out 4 {PROID} "  ({bintbl} + {dcoeftbl}"
        mess/out 4 {PROID} "   -> {resoltbl})"
  !
  !********************************************************************
  @p fors_resol_fwhm {lampima} {bintbl} {slittbl} {dcoeftbl} {resoltbl}
  !********************************************************************
  !
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Computation of fitted values failed !"
    goto error
  endif
  !
  !
!
!
!          E N D   O F   W O R K I N G   W I T H   S L I T S
!          * * * * * * * * * * * * * * * * * * * * * * * * *
!
!
!
!          OUTPUT :
!          ******
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} ">>> Spectral resolutions : {resoltbl} <<<"
        mess/out 3 {PROID} " "
!
!
set/format
if nice_crea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  if m$existk("resoltbl") .eq. 1 then
    if m$exist(resoltbl) .eq. 1 select/table {resoltbl} all >Null
  endif
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" set/graphic default
  endif
  return 1
!
!
!-------------------------------------------------------------------------------
