! $Id: fors_resol_exin.prg,v 1.5 2000/08/16 22:28:04 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_resol_exin.prg,v $
!
! .KEYWORDS   Spectroscopical quality, wavelength calibration
!
! .PURPOSE    Extract positional values of all slits from slit table and 
!             spec. values of all lines from line table.
! __________
!
! .USAGE      @p fors_resol_exin slittbl linetbl lampima outtbl
!
! .INPUT      slittbl (P1): input table with positions of used slits;
!                             default: none
!
!             linetbl (P2): input table with identified lines;
!                             default: none
!
!             lampima (P3): input lamp frame;
!                             default: none
!
!             outtbl  (P4): name of output table (will be created);
!                             default: none
!
! .OUTPUT     "<P4>": table with positions and emission lines
!
!             y-coords.(start,end) of slit in lamp frame via "+lower":
!               YLOW,YHIGH       : [detector pixel ("world coord.")]
!               YSTARTPIX,YENDPIX: [frame pixel]
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for spectroscopy - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/03/09  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref slittbl linetbl lampima outtbl
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "resol_exin"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      slittbl (P1): input table with positions of used slits;"
  w/o "                      default: none"
  w/o " "
  w/o "      linetbl (P2): input table with identified lines;"
  w/o "                      default: none"
  w/o " "
  w/o "      lampima (P3): input lamp frame;"
  w/o "                      default: none"
  w/o " "
  w/o "      outtbl  (P4): name of output table (will be created);"
  w/o "                      default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(slittbl):
!-----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? t -
  "        Enter input slit table: "
define/local slittbl/c/1/64 "{p1}"
if m$index(slittbl,".tbl") .eq. 0 slittbl = slittbl//".tbl"
!
if m$existc(slittbl,":{csslit}") .eq. -2 then  ! col. names are "+lower_levels"
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({slittbl})"
  goto error
elseif m$existc(slittbl,":{csslit}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({slittbl},:{csslit})"
  goto error
elseif m$existc(slittbl,":{cystart}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({slittbl},:{cystart})"
  goto error
elseif m$existc(slittbl,":{cyend}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({slittbl},:{cyend})"
  goto error
endif
!
!
!P2(linetbl):
!-----------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? t -
  "        Enter input line table: "
define/local linetbl/c/1/64 "{p2}"
if m$index(linetbl,".tbl") .eq. 0 linetbl = linetbl//".tbl"
!
if m$existc(linetbl,":{clslit}") .eq. -2 then  ! col. names are "+lower_levels"
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({linetbl})"
  goto error
elseif m$existc(linetbl,":{clslit}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({linetbl},:{clslit})"
  goto error
elseif m$existc(linetbl,":{cx}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({linetbl},:{cx})"
  goto error
elseif m$existc(linetbl,":{cy}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({linetbl},:{cy})"
  goto error
elseif m$existc(linetbl,":{cwavec}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({linetbl},:{cwavec})"
  goto error
elseif m$existc(linetbl,":{creject}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({linetbl},:{creject})"
  goto error
endif
!
!
!P3(lampima):
!-----------
if p3 .eq. "?" mess/out 2 {PROID} " "
define/parameter p3 ? i -
  "        Enter name of lamp frame: "
define/local lampima/c/1/64 "{p3}"
if m$index(lampima,".bdf") .eq. 0 lampima = lampima//".bdf"
!
if m$exist(lampima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lampima})"
  goto error
endif
!
!
!P4(outtbl):
!----------
if p4 .eq. "?" mess/out 2 {PROID} " "
define/parameter p4 ? ? -
  "        Enter name of output table: "
define/local resoltbl/c/1/64 "{p4}"
if m$index(resoltbl,".tbl") .eq. 0 resoltbl = resoltbl//".tbl"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local slit/i/1/1 0
  define/local slitloop/i/1/1 0
  define/local lineloop/i/1/1 0
  define/local ystart/d/1/1 0.
  define/local ystep/d/1/1 0.
!
!
!
!          EXTRACTING VALUES
!          FROM SLIT AND LINE INPUT TABLES :
!          *******************************
!
create/table {resoltbl} 17 {totlineno} null  ! 10 cols. added later
!
        mess/out 4 {PROID} "  filling slit values into result table"
!
create/column {resoltbl} :SLIT     " "      I4   I*4
create/column {resoltbl} :SLT_STRT "detpix" F8.1 R*4
create/column {resoltbl} :SLT_END  "detpix" F8.1 R*4
!
do slitloop = 1 totslitno
  slit = {{slittbl},:{csslit},{slitloop}}
  ylow({slit}) = {{slittbl},:{cystart},{slitloop}}   ! returned via "+lower"
  yhigh({slit}) = {{slittbl},:{cyend},{slitloop}}    ! returned via "+lower"
enddo
!
do lineloop = 1 totlineno
  slit = {{linetbl},:{clslit},{lineloop}}
  {resoltbl},:SLIT,{lineloop} = {slit}
  {resoltbl},:SLT_STRT,{lineloop} = {ylow({slit})}
  {resoltbl},:SLT_END,{lineloop} = {yhigh({slit})}
enddo
!
ystart = {{lampima},START(2)}
ystep = {{lampima},STEP(2)}
do slitloop = 1 totslitno
  slit = {{slittbl},:{csslit},{slitloop}}
  ystartpix({slit}) = m$nint((ylow({slit})-ystart)/ystep)+1  ! ret. via "+lower"
  yendpix({slit}) = m$nint((yhigh({slit})-ystart)/ystep)+1   ! ret. via "+lower"
enddo
!
        mess/out 4 {PROID} "  filling line values into result table"
!
create/column {resoltbl} :X      "detpix"   F9.2  R*4
create/column {resoltbl} :Y      "detpix"   F9.2  R*4
create/column {resoltbl} :WAVEC  "angstrom" F10.2 R*4
create/column {resoltbl} :REJECT " "        I6    I*4
!
copy/tt {linetbl} :{cx}      {resoltbl} :X
copy/tt {linetbl} :{cy}      {resoltbl} :Y
copy/tt {linetbl} :{cwavec}  {resoltbl} :WAVEC
copy/tt {linetbl} :{creject} {resoltbl} :REJECT
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------
