! $Id: fors_resol_fwhm.prg,v 1.12 2001/06/15 14:20:53 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_resol_fwhm.prg,v $
!
! .KEYWORDS   Spectroscopical quality, wavelength calibration
!
! .PURPOSE    Compute FWHM values of all lines in all slits (per bin perpendi-
!             cular to dispersion direction, fit in dispersion direction).
! __________
!
! .USAGE      @p fors_resol_fwhm lampima bintbl slittbl dcoeftbl resoltbl
!
! .INPUT      lampima  (P1): input lamp frame (reduced);
!                              default: none
!
!             bintbl   (P2): table with bin coordinates;
!                              default: none
!
!             slittbl  (P3): input table with positions of used slits;
!                              default: none
!
!             dcoeftbl (P4): input table with dispersion solution;
!                              default: none
!
!             resoltbl (P5): table with positions and emission lines;
!                              default: none
!
! .OUTPUT     "<P4>": columns XCEN,STATUS,FWHM,STEP,D_WAVEC,SPECRES computed
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    Special procedure for spectroscopy - not to be used generally.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/03/11  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref lampima bintbl slittbl dcoeftbl resoltbl
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "resol_fwhm"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      lampima  (P1): input lamp frame (reduced);"
  w/o "                       default: none"
  w/o " "
  w/o "      bintbl   (P2): table with bin coordinates;"
  w/o "                       default: none"
  w/o " "
  w/o "      slittbl  (P3): input table with positions of used slits;"
  w/o "                       default: none"
  w/o " "
  w/o "      dcoeftbl (P4): input table with dispersion solution;"
  w/o "                       default: none"
  w/o " "
  w/o "      resoltbl (P5): table with positions and emission lines;"
  w/o "                       default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(lampima):
!-----------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of reduced lamp frame: "
define/local lampima/c/1/64 "{p1}"
if m$index(lampima,".bdf") .eq. 0 lampima = lampima//".bdf"
!
if m$exist(lampima) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({lampima})"
  goto error
endif
!
!
!P2(bintbl):
!----------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter name of table with bin positions: "
define/local bintbl/c/1/64 "{p2}"
if m$index(bintbl,".tbl") .eq. 0 bintbl = bintbl//".tbl"
!
if m$existc(bintbl,"XSTART") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({bintbl})"
  goto error
elseif m$existc(bintbl,"XSTART") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({bintbl},:XSTART)"
  goto error
elseif m$existc(bintbl,"XEND") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({bintbl},:XEND)"
  goto error
elseif m$existc(bintbl,"YSTART") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({bintbl},:YSTART)"
  goto error
elseif m$existc(bintbl,"YEND") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({bintbl},:YEND)"
  goto error
endif
!
show/table {bintbl} >Null
if outputi(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({bintbl})"
  goto error
endif
!
!
!P3(slittbl):
!-----------
if p3 .eq. "?" mess/out 2 {PROID} " "
define/parameter p3 ? t -
  "        Enter input slit table: "
define/local slittbl/c/1/64 "{p3}"
if m$index(slittbl,".tbl") .eq. 0 slittbl = slittbl//".tbl"
!
if m$existc(slittbl,":{csslit}") .eq. -2 then  ! col. names are "+lower_levels"
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({slittbl})"
  goto error
elseif m$existc(slittbl,":{csslit}") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({slittbl},:{csslit})"
  goto error
endif
!
!
!P4(dcoeftbl):
!------------
if p4 .eq. "?" mess/out 2 {PROID} " "
define/parameter p4 ? ? -
  "        Enter name of table with dispersion solution: "
define/local dcoeftbl/c/1/64 "{p4}"
if m$index(dcoeftbl,".tbl") .eq. 0 dcoeftbl = dcoeftbl//".tbl"
!
if m$exist(dcoeftbl) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({dcoeftbl})"
  goto error
elseif m$existd(dcoeftbl,ddetpix) .eq. 0 then  ! desc. names are "+lower_levels"
        write/out " "
        mess/out 1 {PROID} "Entered table descriptor is not present !"
        mess/out 1 {PROID} "({dcoeftbl},{ddetpix})"
  goto error
endif
define/local slitloop/i/1/1 0
define/local slit/i/1/1 0
do slitloop = 1 totslitno
  slit = {{slittbl},:{csslit},{slitloop}}
  if m$existd(dcoeftbl,"{dlambda}{slit}") .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Table desc. for entered prefix is not present !"
        mess/out 1 {PROID} "({dcoeftbl},{dlambda}{slit})"
  goto error
  endif
enddo
!
!
!P5(resoltbl):
!------------
if p5 .eq. "?" mess/out 2 {PROID} " "
define/parameter p5 ? ? -
  "        Enter name of table with emission lines: "
define/local resoltbl/c/1/64 "{p5}"
if m$index(resoltbl,".tbl") .eq. 0 resoltbl = resoltbl//".tbl"
!
if m$existc(resoltbl,":SLIT") .eq. -2 then
        write/out " "
        mess/out 1 {PROID} "Entered table is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({resoltbl})"
  goto error
elseif m$existc(resoltbl,":SLIT") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({resoltbl},:SLIT)"
  goto error
elseif m$existc(resoltbl,"WAVEC") .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({resoltbl},:WAVEC)"
  goto error
endif
!
show/table {resoltbl} >Null
if outputi(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered table is empty !"
        mess/out 1 {PROID} "({resoltbl})"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Drawing":
  define/local goodbins/i/1/1 0
  define/local labxm/i/1/3 0 all
  define/local labxr/i/1/1 0
  define/local scry0/i/1/1 0
  define/local frmy/i/1/1 0
  define/local frmy0/i/1/1 0
  define/local dscly/i/1/1 0
  define/local scaly/r/1/1 0.
  define/local scry/i/1/1 0
  define/local labobjxr/i/1/1 0
!
!     "Adding":
  define/local lineloop/i/1/1 0
  define/local step/r/1/1 0.
!
!
!
!          APPLYING GAUSS FIT
!          IN DISPERSION DIRECTION
!          TO EACH BIN OF EACH LINE :
!          ************************
!
create/column {bintbl} :XCEN   "detpix" F9.2  R*4
create/column {bintbl} :YCEN   "detpix" F7.2  R*4
create/column {bintbl} :STATUS " "      I6    I*4
create/column {bintbl} :XERR   "detpix" F9.2  R*4
create/column {bintbl} :YERR   "detpix" F7.2  R*4
create/column {bintbl} :XSIG   "detpix" F9.2  R*4
create/column {bintbl} :YSIG   "detpix" F7.2  R*4
create/column {bintbl} :XFWHM  "detpix" F9.3  R*4
create/column {bintbl} :YFWHM  "detpix" F7.3  R*4
create/column {bintbl} :ICENT  "ADU"    G13.6 R*4
create/column {bintbl} :IDENT  " "      A8    C*8
!
        mess/out 3 {PROID} "fitting line profile in each bin"
!
center/gauss {lampima},{bintbl} {bintbl} >Null
!
delete/column {bintbl} :YCEN :XERR :YERR :XSIG :YSIG :YFWHM :ICENT
delete/column {bintbl} :IDENT
name/column {bintbl} :XFWHM :FWHM
!
!
!          DRAWING BIN QUALITY (RESULTED FROM FIT) IN DISPLAY :
!          **************************************************
!
select/table {bintbl} :STATUS.eq.0 >Null
goodbins = outputi(1)
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "  displaying bin quality"
  !
  assign/display d,1 >Null
  if nice_ddim(1) .ge. 285 then
    labxm(1) = nice_dscr(1)-88
    labxm(2) = labxm(1)+54
    labxm(3) = labxm(2)+72
    !
    label/display "good" 10,{labxm(1)} ? cyan 1
    label/display "^BINS^" 10,{labxm(2)} ? green 1
    label/display "failed" 10,{labxm(3)} ? red 1
  else
    labxm(1) = nice_dscr(1)-28
    labxm(3) = labxm(1)+33
    !
    label/display "gd" 10,{labxm(1)} ? cyan 1
    label/display "bd" 10,{labxm(3)} ? red 1
  endif
  labxr  = nice_dscr(1)*2-46
  label/display "SLIT" 10,{labxr} ? green 1
  outputi = labxr+29
  label/display "^" 23,{outputi(1)} ? green 1
  !
  ! ----------------------------------------------------------------
  ! scry - scry0 = nint[ (frmy - frmy0) /scaly ] ,
  !   with  scry0 = nice_dscr(2)     (screen pixel in window centre)
  !   and   frmy0 = DISPLAY_DATA(4)  (frame  pixel in window centre
  !                                   after `load/ima ... c,c')
  !   and   scaly =     DISPLAY_DATA(2), if positive
  !                 -1./DISPLAY_DATA(2), if negative
  ! ----------------------------------------------------------------
  !
  do slitloop = 1 totslitno
    !
    slit = {{slittbl},:{csslit},{slitloop}}
    !
    scry0 = nice_dscr(2)
    frmy  = ystartpix({slit})
    frmy0 = {{lampima},DISPLAY_DATA(4)}
    dscly = {{lampima},DISPLAY_DATA(2)}
    if dscly .ge. 1 then  ! 1,2,3,...
      scaly = dscly
    else                  ! ...,-3,-2 ; 0 not possible, -1 doesn't exist
      scaly = -1./dscly
    endif
    scry = m$nint((frmy-frmy0)/scaly-0.5)+scry0
    !
    labobjxr = nice_dscr(1)*2-26
    if slit .lt. 10 then
      label/display "  {slit}" {scry},{labobjxr} ? green 1
    else
      label/display "{slit}" {scry},{labobjxr} ? green 1
    endif
    !
  enddo
  !
  ! Next four lines: Bad bins first only because of graphical reasons:
  select/table {bintbl} :STATUS.ne.0 >Null
  draw/rectangle {bintbl} f n red,0 >Null
  select/table {bintbl} :STATUS.eq.0 >Null
  draw/rectangle {bintbl} f n cyan,0 >Null
  !
endif
!
select/table {bintbl} all
if goodbins .eq. 0 then
        mess/out 2 {PROID} "  No good bin found !"
  warn = 1
  goto skipped
endif
!
!
!          ADDING FWHM VALUES TO RESULT TABLE,
!          DELETING BINS WITH COORDINATES BEYOND FRAME,
!          CONVERTING FWHM FROM DETECTOR PIXELS TO WAVELENGTHS,
!          CALCULATING SPECTRAL RESOLUTIONS :
!          ****************************************************
!
create/column {resoltbl} :XSTART  "detpix"   F9.2 R*4
create/column {resoltbl} :XEND    "detpix"   F9.2 R*4
create/column {resoltbl} :YSTART  "detpix"   F9.2 R*4
create/column {resoltbl} :YEND    "detpix"   F9.2 R*4
create/column {resoltbl} :XCEN    "detpix"   F9.2 R*4
create/column {resoltbl} :STATUS  " "        I6   I*4
create/column {resoltbl} :FWHM    "detpix"   F9.3 R*4
create/column {resoltbl} :STEP    "ang/dpix" F9.5 R*4
create/column {resoltbl} :D_WAVEC "angstrom" F9.3 R*4
create/column {resoltbl} :SPECRES " "        F9.2 R*4
!
copy/tt {bintbl} :XSTART {resoltbl} :XSTART
copy/tt {bintbl} :XEND   {resoltbl} :XEND
copy/tt {bintbl} :YSTART {resoltbl} :YSTART
copy/tt {bintbl} :YEND   {resoltbl} :YEND
copy/tt {bintbl} :XCEN   {resoltbl} :XCEN
copy/tt {bintbl} :STATUS {resoltbl} :STATUS
copy/tt {bintbl} :FWHM   {resoltbl} :FWHM
!
        mess/out 4 {PROID} "  deleting bins with coordinates beyond frame"
!
! If coordinates in {bintbl} are beyond frame, center/gauss above skips
! such rows - then {resoltbl},:STATUS is NULL and "lineloop" below would
! return errors. Therefore the next four lines are needed:
!
select/table {resoltbl} :STATUS.ne.NULL >Null
totlineno = outputi(1)
copy/table {resoltbl} midtmp.tbl
-rename midtmp.tbl {resoltbl}
!
        mess/out 3 {PROID} "applying dispersion solution"
!
! ------------------------------------------------------------------------------
! application of dispersion solution in 1st order:
! lambdai - lambda0 = (detpixi - detpix0) * step ,
!   with  lambda0 = {{dcoeftbl},{dlambda}<slitno>(1)}  (desc. of dispers. table)
!   and   detpix0 = {{dcoeftbl},{ddetpix}(1)}          (desc. of dispers. table)
!   and   step    = {{dcoeftbl},{dlambda}<slitno>(2)} /
!                   {{dcoeftbl},{ddetpix}(3)} ;
! => per slit:
! delta_lambda = lambda2-lambda1 = (detpix2-detpix1)*step = delta_detpix*step ,
!   with  delta_detpix in {resoltbl},:FWHM
!   and   delta_lambda to be calculated in {resoltbl},:D_WAVEC .
! ------------------------------------------------------------------------------
!
do lineloop = 1 totlineno
  slit = {{resoltbl},:SLIT,{lineloop}}
  step = {{dcoeftbl},{dlambda}{slit}(2)}/{{dcoeftbl},{ddetpix}(3)}
  {resoltbl},:STEP,@{lineloop} = {step}
  {resoltbl},:D_WAVEC,@{lineloop} = {{resoltbl},:FWHM,@{lineloop}}*{step}
enddo
!
compute/table {resoltbl} :SPECRES = :WAVEC/:D_WAVEC
!
!
!          RE-MARKING BINS WITH FAILED WAVE. CALIB. :
!          ****************************************
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "  excluding bins with failed wave. calib."
  !
  select/table {resoltbl} :REJECT.ne.1.and.:STATUS.eq.0 >Null
  draw/rectangle {resoltbl} f n red,0 >Null
  ! Next two lines: Repetition of good bins only because of graphical reasons:
  select/table {resoltbl} :REJECT.eq.1.and.:STATUS.eq.0 >Null
  draw/rectangle {resoltbl} f n cyan,0 >Null
  !
endif
!
!
!          PLOTTING FWHMS OF WELL-FITTED BINS :
!          **********************************
!
select/table {resoltbl} :REJECT.eq.1.and.:STATUS.eq.0 >Null
!
if outputi(1) .eq. 0 then
        mess/out 2 {PROID} "  No bin with gd. fit and gd. wave.calib. found !"
  warn = 1
  goto skipped
endif
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "  plotting FWHMs of well-fitted bins"
  !
  assign/graphic g,1 >Null
  set/graphic stype=1 ltype=0 colour=3
  !
  plot/table {resoltbl} :WAVEC :D_WAVEC >Null
  !
  !***************
  @p fors_nicegrap
  !***************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of FWHM plot failed !"
    goto error
  endif
  label/graphic "FWHMs of bins" {nice_glabw}
  !
endif
!
!
!          PLOTTING SPECTRAL RESOLUTIONS OF WELL-FITTED BINS :
!          *************************************************
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} "  plotting spec. resolutions of well-fitted bins"
  !
  assign/graphic g,2 >Null
  !
  plot/table {resoltbl} :WAVEC :SPECRES >Null
  !
  !***************
  @p fors_nicegrap
  !***************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of spec. resol. plot failed !"
    goto error
  endif
  label/graphic "Spec. resol. of bins" {nice_glabw}
  !
endif
!
select/table {resoltbl} all
!
!-delete {bintbl}
!
!
skipped:
!
!
set/format
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  return 1
!
!
!-------------------------------------------------------------------------------
