! $Id: fors_rmean.prg,v 1.6 1999/11/25 11:24:07 sbogun Exp $   
!+++ FORS_RMEAN +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_rmean.prg,v $
!
!.KEYWORDS      Calculates the mean of an image area, center excluded
!
!.USAGE         @p fors_rmean in_frm
!
!.INPUT         in_frm  (P1): input frame
!
!.OUTPUT        UPDATED input frame P1
!
!.RETURN        Q1:  sum of the data values (see ALGORITHM)
!
!.DEFAULT
!
!.PURPOSE       Calculates the mean of an area, center excluded, replace
!               center by mean (used by corr_bpxl.prg).
!
!.ALGORITHM     Determine the sum over all data values, subtract value of
!               center and finally devide by number of pixels minus one.
!               The area is defined through the coordinates of the lower left
!               and upper right corner stored in (INPUTR(1),INPUTR(2)) and
!               (INPUTR(3),INPUTR(4)), respectively.
!               Does no checks on area boundary!
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1998/04/07  Creation
!
!.COMMENT
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name of INPUT frame:"
!
! local variable declaration
!
define/local i1/i/1/1 0
define/local i2/i/1/1 0
define/local nx/i/1/1 0
define/local ny/i/1/1 0
define/local num/i/1/1 0
define/local xl/r/1/1 0.0
define/local xu/r/1/1 0.0
define/local yl/r/1/1 0.0
define/local yu/r/1/1 0.0
define/local xc/r/1/1 0.0
define/local yc/r/1/1 0.0
define/local xi/r/1/1 0.0
define/local yi/r/1/1 0.0
define/local sum/r/1/1 0.0

xl = inputr(1)
yl = inputr(2)
xu = inputr(3)
yu = inputr(4)

xc = (xu - xl + 1.0)
nx = m$nint(xc)
yc = (yu - yl + 1.0)
ny = m$nint(yc)
xc = (xl + xu)/2.0
yc = (yl + yu)/2.0

yi = yl - 1.0
do i2 = 1 ny
  yi  = yi + 1.0
  xi = xl - 1.0
  do i1 = 1 nx
    xi  = xi + 1.0
    sum = sum + {{P1}[{xi},{yi}]}
    num = num + 1
  enddo
enddo

yl = sum / num
sum = (sum - {{P1}[{xc},{yc}]}) / (num - 1.0)

{P1}[{xc},{yc}] = sum

return {sum}
