! $Id: fors_roblinplt.prg,v 1.11 1999/11/25 11:24:08 sbogun Exp $
!+++ ROBLINPLT ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_roblinplt.prg,v $
!
!.KEYWORDS      Plot the results of the robust linear fit
!
!.USAGE         @p fors_roblinplt in_tbl xax yax const fact title psname
!
!.INPUT         in_tbl  (P1): name of input table to be plotted
!               xax     (P2): name of x-axis table-column
!               yax     (P3): name of y-axis table-column
!               const   (P4): constant term of linear fit
!               fact    (P5): factor in linear fit
!               title   (P6): title of the plot
!               psname  (P7): name of POSTSCRIPT hardcopy file
!               dispno  (P8): graphic display number
!
!.OUTPUT        graphical output & POSTSCRIPT file
!
!.RETURN        
!
!.DEFAULT
!
!.PURPOSE       Plot the results of the robust linear fit
!
!.ALGORITHM     
!
!.ENVIRON       FORS, CCDRED context
!               uses global keyword: pipeline, NICE_GDIM
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.9 1998/06/30  Creation
!
!.COMMENT       
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name INPUT table:"
define/param p2 ? c "Enter name of x-axis table-column:"
define/param p3 ? c "Enter name of y-axis table-column:"
define/param p4 {outputr(1)} n "Enter constant c of linear fit (y=k*x+c):"
define/param p5 {outputr(2)} n "Enter coefficient k of linear fit (y=k*x+c):"
define/param p6 "Robust linear fit" c "Enter title of the plot:"
define/param p7 none c "Enter name of POSTSCRIPT-file:"
define/param p8 1 n "Graphic display number:"

define/local num/i/1/1 0
define/local x1/r/1/1 0.0
define/local y1/r/1/1 0.0
define/local x2/r/1/1 0.0
define/local y2/r/1/1 0.0
define/local xsc/r/1/1 624.0
define/local ysc/r/1/1 445.0
define/local wid/r/1/1 1.0
define/local hig/r/1/1 1.0
define/local xof/r/1/1 0.0
define/local yof/r/1/1 0.0
define/local coeff/r/1/1 {P5}
define/local zerop/r/1/1 {P4}

if nice_crea .eq. "n" return    !GRAPHIC disabled: do nothing

xsc = NICE_GDIM(1) / xsc
ysc = NICE_GDIM(2) / ysc
!
!
! Determine equation of linear fit
!
statistic/table {P1} {P2} >Null
x1 = outputr(1)                                                 !MINIMUM
x2 = outputr(2)                                                 !MAXIMUM
y1 = coeff * x1 + zerop
y2 = coeff * x2 + zerop
create/column {P1} :RESIDUAL "mag" >Null                        !calcul. resid.
compute/table {P1} :RESIDUAL = {P3} - ({P5} * {P2} + {P4})

assign/graph G,{P8}
set/graph default
set/graph colour=1 stype=9 ltype=0 lwidth=2 stype=8 cleargra=on pmode=2
wid = -120.0 * xsc
hig = -80.0 * ysc
xof = 20.0 * xsc
yof = 40.0 * ysc
plot/table {P1} {P2} {P3} {wid},{hig},{xof},{yof}

xof = 60.0 * xsc
yof = 87.0 * ysc
set/graph tsize=1.5
label/graph {P6} {xof},{yof},mm ? 1 0

set/graph bcolour=0 colour=2 ltype=1 lwidth=3 stype=0           !display fit
overplot/line 1 {x1},{y1} {x2},{y2}

set/graph default
set/graph colour=1 ltype=0 stype=6 cleargra=off pmode=0         !display resid.
wid = -120.0 * xsc
hig = -15.0 * ysc
xof = 20.0 * xsc
yof = 10.0 * ysc
plot/table {P1} {P2} :RESIDUAL {wid},{hig},{xof},{yof}
xof = xof - 1
yof = yof - 1
copy/dk {P1} HISTORY/c/1/60 outputc/c/1/60
set/graph tsize=2 colour=2
label/graph "{outputc}" -{xof},-{yof},mm ? 1 1
set/graph default
set/format F8.3
xof = 122.0 * xsc
yof = 14.0 * ysc
label/graph "Coefficients:" {xof},{yof},mm ? 2 1
yof = 10.0 * ysc
label/graph "k = {coeff}" {xof},{yof},mm ? 2 1
yof = 7.0 * ysc
label/graph "z = {zerop}" {xof},{yof},mm ? 2 1
set/format

if P7 .ne. "none" then
  mess/out 3 roblinplt "Create hardcopy: {P7}"
  copy/graph postscript
  -rename postscript.ps {P7}
endif

set/graph default
