! $Id: fors_scalebias.prg,v 1.2 1999/09/16 10:46:06 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_scalebias.prg,v $
!
!.KEYWORDS    Basic CCD reduction, scaling of master bias frames
!
!.USAGE       scale/mbias template frame sbias
!
!.INPUT       frame       (P1)   Frame from which the overscan bias values
!                                are taken as the scale factors.
!             template    (P2)   Master bias template frame to be scaled
!
!.OUTPUT      sbias       (P3)   Scaled master bias
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is 
!             updated with the exit status.
!
!.PURPOSE     Scale the input master bias frame for each individual
!             readout port. The scale factors are the overscan
!             bias values taken from the input frame.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/08/23:  Creation
!
!------------------------------------------------------------------------------

crossref  frame template sbias

define/param p1 ?   c "Enter name of input frame:"
define/param p2 ?   c "Enter name of master bias template:"
define/param p3 ?   c "Enter name of scaled master bias:"
define/maxpar 3


! Procedure identification
define/local procID/c/1/80      "ScaleMbias"


! Local constants
define/local tmpfrm/c/1/20  "tmp_mbias.bdf"


! Local variables
define/local i/i/1/1                  0
define/local j/i/1/1                  0
define/local k/i/1/1                  0
define/local n/i/1/1                  0
define/local nport/i/1/1              0
define/local x1/i/1/1                 0
define/local y1/i/1/1                 0
define/local x2/i/1/1                 0
define/local y2/i/1/1                 0
define/local xsize/i/1/1              0
define/local ysize/i/1/1              0
define/local imsec/i/1/4              0     all
define/local frm/c/1/100              {p1}
define/local tpl/c/1/100              {p2}
define/local sbias/c/1/100            {p3}
define/local scale/r/1/{FORS_MAXOUT}  0.  all



! Process parameters
if m$indexb(frm,".bdf") .le. 0 then
  frm = frm // ".bdf"
endif

if m$exist(frm) .ne. 1 then
  mess/out 1 {procID} "Input frame >{frm}< does not exist!"
  return 1
endif

if m$exist(tpl) .ne. 1 then
  mess/out 1 {procID} "Template frame >{tpl}< does not exist!"
  return 1
endif


! Convert input frame to REAL*4 if necessary
info/image {tpl} >Null
if mid$info(1) .ne. 10 .and. mid$info(1) .ne. 18 then
  copy/ii {tpl} {tmpfrm} r4
else
  -copy {tpl} {tmpfrm}
endif


! Get number of readout ports
nport = m$value({frm},{h_nout})
if nport .ne. 1 .and. nport .ne. 4 then
  mess/out 1 {procID} "Invalid readout port setup!"
  return 1
else
  if m$existd(frm,h_prqcobi(1)) .eq. 1 then
    do i = 1 nport 1
      scale({i}) = m$value({frm},{h_prqcobi({i})})
    enddo
  else

! For compatibility reasons

    if m$existd(frm,h_bsmean) .ne. 1 then
      mess/out 1 {procID} "Overscan bias values missing in >{frm}<!"
      return 1
    else
      copy/dk {frm} {h_bsmean} scale
    endif
  endif
endif


! Get valid image area
image/section {tmpfrm} {FORS_CCD_RDDIR}
if {q1} .eq. 0 then
  imsec(1) = outputd(1)
  imsec(2) = outputd(2)
  imsec(3) = outputd(3)
  imsec(4) = outputd(4)
else
  mess/out 1 {procID} "Cannot determine image layout from >{tpl}<!"
  return 1
endif


! Calculate number of valid pixels along the x and y axis of all ports
xsize = imsec(3) - imsec(1) + 1
ysize = imsec(4) - imsec(2) + 1

if nport .eq. 4 then
  xsize = xsize / 2
  ysize = ysize / 2
endif


! Scale master bias
if nport .eq. 4 then

! Note that the following method assumes that the ports are located at the
! corners of a square, with the port numbering starting at the lower left
! corner.
  n = nport / 2

  do i = 1 n 1
    y1 = imsec(2) + (i - 1) * ysize
    y2 = y1 + ysize - 1

    do j = 1 n 1  
      x1 = imsec(1) + (j - 1) * xsize
      x2 = x1 + xsize - 1
      k = (i - 1) * n + j

      compute/image {tmpfrm}[{x1},{y1}:{x2},{y2}] = -
                    {tmpfrm}[{x1},{y1}:{x2},{y2}] * {scale({k})}
    enddo
  enddo
else
  x1 = imsec(1)
  y1 = imsec(2)
  x2 = imsec(3)
  y2 = imsec(4)

  compute/image {tmpfrm}[{x1},{y1}:{x2},{y2}] = -
                {tmpfrm}[{x1},{y1}:{x2},{y2}] * {scale(1)}
endif

-rename {tmpfrm} {sbias}

return 0