! $Id: fors_setmod.prg,v 1.4 1998/09/14 17:05:06 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_setmod.prg,v $
!
!.KEYWORDS    Mode dependent frame identifier setup.
!
!.USAGE       set/obsmod  keyname obsmode lock
!
!.INPUT       keyname     (P1)   Frame identifier keyword
!             obsmode     (P2)	 Observation mode (LSS,MOS,IMG)
!             lock        (P3)   Lock number
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The procedure appends the current observation mode to a frame   
!   	      identifier keyword. A previously set observation mode is        
!   	      overwritten. If 'lock' is not 0 the keyword is locked           
!   	      after appending the observation mode using the given number     
!   	      as lock number. If 'lock' is set to 0 but the given keyword     
!   	      is already locked the changes are made and the keyword is locked
!   	      again using it old lock number.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/07/10:  Creation
!
!------------------------------------------------------------------------------

crossref keyname obsmode lock

define/param p1  ?   c     "Enter keyword name:"
define/param p2  ?   c     "Enter observing mode:"
define/param p3  0   num   "Enter keyword lock number:"
define/maxpar 3


! Procedure identification
define/local procID/c/1/80      "ForsSetMode"


! Local constants
define/local skey/c/1/1      "_"


! Local variables
define/local pos/i/1/1       0
define/local len/i/1/1       0
define/local lock/i/1/1      {p3}
define/local lockno/i/1/1    0
define/local keyname/c/1/16  " "  all
define/local keyval/c/1/60   " "  all
define/local obsmode/c/1/4   " "  all



keyname = m$upper(p1)
obsmode = skey // m$upper(p2)

keyval = {keyname}


! Get position of obsmode suffix
pos = m$indexb(keyval,skey)
len = m$len(obsmode)

! Check if keyword is locked
info/keyw {keyname}
lockno = mid$info(4)

if lockno .ne. 0 then
  unlock/key {keyname} {lockno}
endif

if m$existk(keyname) .eq. 1 then
  write/key {keyname}/c/{pos}/{len} {obsmode}
else
  mess/out 1 {procID} "Keyword {keyname} to be modified does not exist!"
  return 1
endif

if lockno .ne. 0 then
  lock/key {keyname} {lockno}
else if lock .ne. 0 then
  lock/key {keyname} {lock}
endif
  
return 0


