! $Id: fors_shortlog.prg,v 1.12 1999/11/24 09:47:57 sbogun Exp $   
!+++ FORS_SHORTLOG ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_shortlog.prg,v $
!
!.KEYWORDS      Create a short log file of reduction recipes
!
!.USAGE         shortlog/out mode logentry
!
!.INPUT         mode    (P1): 1: open and add entry to log
!                             0: close and put log on terminal
!                            -2: Puts the log of the previously closed log file
!                                on the terminal.
!                                
!               entry   (P2): entry to add
!
!.OUTPUT        
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Create a short log file of reduction steps
!
!.ALGORITHM     Short log file means, that for each reduction step only ONE
!               text line should be entered.
!
!.ENVIRON       FORS context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/12/17  Creation
!                   1999/03/10  Write shortlog to HISTORY buffer to frame
!                               specified in P2 when P1=-1.
!                   1999/04/13  mode=-1 not necessary anymore
!                               file suffix .log -> .slog (SB)
!                               Output also to LONGLOG
!               1.3 1999/08/30  Full revision (SB)
!
!.COMMENT
!------------------------------------------------------------------------------

define/param P1 ? i "Enter mode:"
define/param P2 + c "Enter log entry:"
!read/key p1,p2,pipeline
define/local procid/c/1/20 "shortlog"

define/local mode/i/1/1 -1
define/local date/c/1/30 " " ALL

define/local i1/i/1/1 0
define/local fid/i/1/2 0 ALL
define/local line/c/1/100 {P2}

mode = {P1}
if line(1:1) .eq. "+" line(1:1) = " "   !assume P2 holds filename of main prod.


branch mode -2,0,1 OUT,CLO,ADD

mess/out 1 {procid} "Wrong mode selected (-2,0,1 are allowed) !"
goto FIN
     
CLO:
     if m$exist(slog_file) .eq. 0 then
       mess/out 1 {procid} -
         "No log file had been opened or\nan error occured while opening !"
       return 1
     else
       mess/out 4 {procid} "Close log file: {slog_file}"
       write/key outputc "." ALL
       open/file {slog_file} APPEND fid
       write/file {fid(1)} {outputc(1:50)}
       close/file {fid(1)}
       @p fors_shortlog,TODESCR {procid} {line}
     endif
     goto FIN
OUT:
     open/file {slog_file} R fid
     write/out " "
!     append/out "Reduction recipe log:\n"
     set/format J2
     i1 = 0
loop:
     read/file {fid(1)} inputc
     if fid(2) .eq. -1 then
       close/file {fid(1)}
       mess/out 3 {procid}
       goto FIN
     endif
     if m$len(inputc) .eq. 0 goto loop          !skip empty lines
     if inputc(1:5) .eq. "....." goto loop      !.. and: ----- (last line)
     if i1 .eq. 0 then
       append/out "{inputc}\n    "
     else
       append/out "{i1}: {inputc}\n    "
     endif
     i1 = i1 + 1
     goto loop
     
ADD: 
     if m$exist(slog_file) .eq. 0 .or. m$existk("ilong") .eq. 0 then
       mess/out 4 {procid} "Open log file: {slog_file}"
       open/file {slog_file} WRITE fid
       date = m$time()
       write/file {fid(1)} "Recipe log ({date}):"
       write/key ilong/i/1/1 1
     else
      open/file {slog_file} APPEND fid
     endif

     write/file {fid(1)} "{line}"
     close/file {fid(1)}
     set/format J2
     longlog/out 3 " "
     longlog/out 3 "{ilong}: {line}"
     longlog/out 3 " "
     longlog/out 0
     ilong = ilong + 1
     
     goto FIN

FIN:
return




!==============================================================================
!=== SUBROUTINES ==============================================================
!==============================================================================


ENTRY TODESCR

define/local line/c/1/100 {P2}
define/local ext/c/1/100 " " ALL
define/local catal/i/1/1 0
@p fors_parsepath {line}
ext = m$upper(q3)
if ext(1:4) .ne. ".CAT" goto SINGLE

CATALOG:
  store/frame line {p2} 1 FINI

SINGLE:
if m$strlen(line) .ne. 0 then   !assume P2 holds filename of main prod.
  mess/out 4 {p1} "Write shortlog to HISTORY of `{line}`"
  delete/descr {line} HISTORY N
  copy/ad {slog_file} {line} SHORTLOG -1
endif
if ext(1:4) .ne. ".CAT" goto FINI
goto CATALOG
FINI:
