! $Id: fors_statistics.prg,v 1.52 2001/04/06 16:19:23 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_statistics.prg,v $
!
! .KEYWORDS   Simple statistics
!
! .PURPOSE    Determine statistics of a science or standard star frame.
!             The histogrammes can be displayed.
! __________
!
! .USAGE      @p fors_statistics icat mbias [disp] [calc] [scan] [ps]
!
! .INPUT      icat  (P1): image catalogue with input frames;
!                           default: none
!
!             mbias (P2): master bias frame;
!                           default: none
!
!             disp  (P3): display selection: <p>:pipeline mode:read global key
!                         NICE_CREA [y|n], <n>:none, <l>:local, or
!                         host:display[.screen] - with the latter option
!                         you can specify the name or IP-address of a display;
!                           default: <l>
!
!             calc  (P4): calculation flag: <>:calculate statistics of
!                         input frames, or <u>se "STATISTIC" descriptors and
!                         calculate only median deviation from the median if
!                         needed;
!                           default: <>
!
!             scan  (P5): numbers of pre- AND overscan columns, separated by
!                         a comma;
!                           default: 16,16
!
!             ps    (P6): flag for creation of histogramme postscript
!                         file: <>:do not create, <y>es,create;
!                           default: <>
!                           hint: ignored, if P3=n
!
! .OUTPUT     "<P1>_sta.tbl":  simple statistics
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Statistics on whole frame and quadrants.
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/09/23  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref icat mbias disp calc scan ps
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "statistics"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For `longlog/out':
write/keyword out_a "given command/qualif not defined ..."
show/command longlog/out | write/keyword in_a
-delete Mid{mid$sess(11:12)}1Pipe
-delete midlis{mid$sess(11:12)}.prg
if in_a .eq. out_a create/command longlog/out "write/out"
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      icat  (P1): image catalogue with input frames;"
  w/o "                    default: none"
  w/o " "
  w/o "      mbias (P2): master bias frame;"
  w/o "                    default: none"
  w/o " "
  w/o "      disp  (P3): display selection: <p>:pipeline mode:read global key"
  w/o "                  NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                  host:display[.screen] - with the latter option"
  w/o "                  you can specify the name or IP-address of a display;"
  w/o "                    default: <l>"
  w/o " "
  w/o "      calc  (P4): calculation flag: <>:calculate statistics of"
  w/o "                  input frames, or <u>se "STATISTIC" descriptors and"
  w/o "                  calculate only median deviation from the median if"
  w/o "                  needed;"
  w/o "                    default: <>"
  w/o " "
  w/o "      scan  (P5): number of pre- and overscan columns, separated"
  w/o "                  by a comma;"
  w/o "                    default: 16,16"
  w/o " "
  w/o "      ps    (P6): flag for creation of histogramme postscript"
  w/o "                  file: <>:do not create, <y>es,create;"
  w/o "                    default: <>"
  w/o "                    hint: ignored, if P3=n"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(icat):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? i -
  "        Enter name of catalogue with images to be examined: "
define/local icat/c/1/64 "{p1}"
if m$index(icat,".cat") .eq. 0 icat = icat//".cat"
!
if m$exist(icat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
 goto error
endif
!
show/icat {icat} n
if outputi(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is empty !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
define/local catal/i/1/1 0
inloop:
  store/frame inputc {icat} 1 end_inloop
  if m$exist(inputc) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "At least one catalogue entry is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
    goto error
  endif
  goto inloop
end_inloop:
!
!
!P2(mbias):
!---------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? i -
  "        Enter name of master bias frame: "
define/local mbias/c/1/64 "{p2}"
if m$index(mbias,".bdf") .eq. 0 mbias = mbias//".bdf"
!
if m$exist(mbias) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered master bias frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({mbias})"
  goto error
endif
!
!
!P3(disp):
!--------
define/parameter p3 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p3}"
!
!
!P4(calc):
!--------
define/parameter p4 + ? -
  "        Enter calculation flag [def.:calc_stat|<u>se_desc]:"
define/local calc/c/1/1 "{p4}"
!
!
!P5(scan):
!--------
define/parameter p5 16,16 ? -
  "        Enter pre-/overscan values [def.:16,16]: "
outputi = m$parse(p5,"scan")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P5 !"
  goto error
endif
define/local prescan/i/1/1 {scan01}
define/local ovescan/i/1/1 {scan02}
!
if prescan .lt. 0 .or. ovescan .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible pre- or overscan value !"
  goto error
endif
!
!
!P6(ps):
!------
define/parameter p6 + ? -
  "        Enter flag for PS files of histo.s [def.:n|<y>es,create]: "
define/local ps/c/1/1 "{p6}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Preparing tests ..." :
  define/local char/i/1/2 0 all
  define/local stattable/c/1/64 " " all
!
!     "... master bias ..." :
  define/local mpix/i/1/1 0
  define/local msubxstart/i/1/1 0
  define/local msubxend/i/1/1 0
  define/local mmean/d/1/1 0.
  define/local msdev/d/1/1 0.
!
!     "L o o p   w i t h   t e s t   ..." :
  define/local topicima/c/1/64 " " all
  define/local lopcount/i/1/1 0
!
!     "... size ..." :
  define/local tpix/i/1/2 0 all
  define/local xsize/i/1/1 0
!
!     "... level ..." :
  define/local tsubxstart/i/1/1 0
  define/local tsubxend/i/1/1 0
  define/local twmin0/d/1/1 0.
  define/local twmode0/d/1/1 0.
  define/local twmedian0/d/1/1 0.
  define/local twmean0/d/1/1 0.
  define/local twstddev0/d/1/1 0.
  define/local twmax0/d/1/1 0.
  !
  define/local txodd/i/1/1 0
  define/local tyodd/i/1/1 0
  define/local x2mid/i/1/1 0
  define/local y2mid/i/1/1 0
  define/local txs/i/1/4 0 all
  define/local tys/i/1/4 0 all
  define/local txe/i/1/4 0 all
  define/local tye/i/1/4 0 all
  !
  define/local twmin/d/1/4 0. all
  define/local twmode/d/1/4 0. all
  define/local twmedian/d/1/4 0. all
  define/local twmean/d/1/4 0. all
  define/local twstddev/d/1/4 0. all
  define/local twmax/d/1/4 0. all
!
!     "Displaying histogramme" :
  define/local pscr/c/1/64 " " all
!
!
!          MANAGING DISPLAYS :
!          *****************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then  ! written by fors_nicecrea.prg
  !
  if nice_gids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 1 is missing !"
    goto error
  endif
  !
  if nice_dids(1) .eq. 1 then
    assign/display d,1 >Null
    clear/display
    display/lut off
  endif
  if nice_dids(2) .eq. 1 then
    assign/display d,2 >Null
    clear/display
    display/lut off
  endif
  if nice_dids(3) .eq. 1 then
    assign/display d,3 >Null
    clear/display
    display/lut off
  endif
  assign/graphic g,1 >Null
  clear/graphic
  if nice_gids(2) .eq. 1 then
    assign/graphic g,2 >Null
    clear/graphic
  endif
  set/graphic default
  set/graphic pmode=0 stype=0 bcolour=1
  !
endif
!
!
!          PREPARING EXAMINATIONS IN LOOP :
!          ******************************
!
        mess/out 4 {PROID} "preparing examinations"
!
  char = m$index(icat,".cat")-1
  stattable = icat(1:{char})//"_sta.tbl"
show/icat {icat} n
create/table {stattable} 32 {outputi(1)} null
!
create/column {stattable} :FRAME " " A64 C*64
create/column {stattable} :MIN_0 "ADU" F10.2 R*4
create/column {stattable} :MODE_0 "ADU" F10.2 R*4
create/column {stattable} :MEDIAN_0 "ADU" F10.2 R*4
create/column {stattable} :MEAN_0 "ADU" F10.2 R*4
create/column {stattable} :STDDEV_0 "ADU" F10.2 R*4
create/column {stattable} :MAX_0 "ADU" F10.2 R*4
create/column {stattable} :Min_1 "ADU" F10.2 R*4
create/column {stattable} :Mode_1 "ADU" F10.2 R*4
create/column {stattable} :Median_1 "ADU" F10.2 R*4
create/column {stattable} :Mean_1 "ADU" F10.2 R*4
create/column {stattable} :Stddev_1 "ADU" F10.2 R*4
create/column {stattable} :Max_1 "ADU" F10.2 R*4
create/column {stattable} :Min_2 "ADU" F10.2 R*4
create/column {stattable} :Mode_2 "ADU" F10.2 R*4
create/column {stattable} :Median_2 "ADU" F10.2 R*4
create/column {stattable} :Mean_2 "ADU" F10.2 R*4
create/column {stattable} :Stddev_2 "ADU" F10.2 R*4
create/column {stattable} :Max_2 "ADU" F10.2 R*4
create/column {stattable} :Min_3 "ADU" F10.2 R*4
create/column {stattable} :Mode_3 "ADU" F10.2 R*4
create/column {stattable} :Median_3 "ADU" F10.2 R*4
create/column {stattable} :Mean_3 "ADU" F10.2 R*4
create/column {stattable} :Stddev_3 "ADU" F10.2 R*4
create/column {stattable} :Max_3 "ADU" F10.2 R*4
create/column {stattable} :Min_4 "ADU" F10.2 R*4
create/column {stattable} :Mode_4 "ADU" F10.2 R*4
create/column {stattable} :Median_4 "ADU" F10.2 R*4
create/column {stattable} :Mean_4 "ADU" F10.2 R*4
create/column {stattable} :Stddev_4 "ADU" F10.2 R*4
create/column {stattable} :Max_4 "ADU" F10.2 R*4
!
!
!          CALCULATING LEVEL OF MASTER BIAS FRAME :
!          **************************************
!
!          FOR "LOOP WITH TEST FRAMES" :
!
mpix = {{mbias},NPIX(1)}
!
        mess/out 3 {PROID} "checking count level of master bias frame"
!
msubxstart = prescan+1
msubxend = mpix(1)-ovescan
if calc .eq. "u" then
  if m$existd(mbias,"STATISTIC") .eq. 1 then
        mess/out 4 {PROID} "  reading statistics from descriptor STATISTIC"
  else
        mess/out 2 {PROID} "  Master bias: Descriptor "STATISTIC" is missing -"
        mess/out 2 {PROID} "  statistics will be calculated !"
        mess/out 2 {PROID} "  ({mbias})"
    statistics/image {mbias} [@{msubxstart},<:@{msubxend},>] ? ? sn >Null
  endif
else
        mess/out 4 {PROID} "  calculating statistics"
  statistics/image {mbias} [@{msubxstart},<:@{msubxend},>] ? ? sn >Null
endif
!
mmean = {{mbias},STATISTIC(3)}  ! NOT `outputr' because of above `if'
msdev = {{mbias},STATISTIC(4)}
!
        set/format ,F7.2
        mess/out 3 {PROID} "  avg. bias level: {mmean} +- {msdev}"
        longlog/out 1 -
          "     avg. level of master bias frame: {mmean} +- {msdev}" >Null
        set/format {FMTDEF}
!
!
!
!          L O O P   W I T H   T E S T   F R A M E S :
!          * * * * * * * * * * * * * * * * * * * * *
!
!
        if mess_level .eq. 3 mess/out 3 {PROID} " "
                             longlog/out 1 " " >Null
!
catal = 0
catloop:
  store/frame topicima {icat} 1 end_catloop
  lopcount = lopcount+1
  show/icat {icat} n
        mess/out 4 {PROID} " "
        mess/out 3 {PROID} -
          "test image {lopcount} / {outputi(1)} : {topicima}"
        longlog/out 1 -
          "     {topicima}:" >Null
  !
  {stattable},:FRAME,{lopcount} = "{topicima}"
  !
  !
  !        CHECKING LEVEL AND FURTHER SIMPLE STATISTICS :
  !        ********************************************
  !
  tpix(1) = {{topicima},NPIX(1)}
  tpix(2) = {{topicima},NPIX(2)}
  !
        mess/out 3 {PROID} "  calculating statistics of whole frame"
  !
  tsubxstart = prescan+1
  tsubxend = tpix(1)-ovescan
  if calc .eq. "u" .and. m$existd(topicima,"STATISTIC") .eq. 0 then
        mess/out 2 {PROID} "  Test frame: Descriptor "STATISTIC" is missing -"
        mess/out 2 {PROID} "  statistics will be calculated !"
        mess/out 2 {PROID} "  ({topicima})"
  endif
  if calc .ne. "u" .or. m$existd(topicima,"STATISTIC") .eq. 0 then
    statistics/image {topicima} [@{tsubxstart},<:@{tsubxend},>] ? ? gn >Null
  endif
  twmean0 = {{topicima},STATISTIC(3)}  ! NOT `outputr' because of above `if'
  !
  twmin0 = {{topicima},STATISTIC(1)}
  twmode0 = {{topicima},STATISTIC(12)}
  twmedian0 = {{topicima},STATISTIC(8)}
  twstddev0 = {{topicima},STATISTIC(4)}
  twmax0 = {{topicima},STATISTIC(2)}
  !
  {stattable},:MIN_0,{lopcount} = twmin0
  {stattable},:MODE_0,{lopcount} = twmode0
  {stattable},:MEDIAN_0,{lopcount} = twmedian0
  {stattable},:MEAN_0,{lopcount} = twmean0
  {stattable},:STDDEV_0,{lopcount} = twstddev0
  {stattable},:MAX_0,{lopcount} = twmax0
  !
        set/format ,T10.2
        mess/out 3 {PROID} -
               "  min:{twmin0} / mod:{twmode0} / med:{twmedian0}"
        longlog/out 1 -
          "     min:{twmin0} / mod:{twmode0} / med:{twmedian0}" >Null
        mess/out 3 {PROID} -
               "  avg:{twmean0} / sig:{twstddev0} / max:{twmax0}"
        longlog/out 1 -
          "     avg:{twmean0} / sig:{twstddev0} / max:{twmax0}" >Null
        set/format {FMTDEF}
  !
  if twmean0 .le. mmean then
    !
        write/out " "
        mess/out 1 {PROID} "Mean level of test frame is lower than"
        mess/out 1 {PROID} "mean level of bias frame !"
        mess/out 1 {PROID} "({topicima})"
    goto error
    !
  endif
  !
        mess/out 3 {PROID} "  calculating statistics of quadrants"
  !
  !     3(C) | 4(D)
  !     -----+-----
  !     1(A) | 2(B)
  !
  if m$nint(tpix(1)/2.)-tpix(1)/2. .eq. 0. then
    txodd = 0
  else
    txodd = 1  ! odd number of x-pixels
  endif
  if m$nint(tpix(2)/2.)-tpix(2)/2. .eq. 0. then
    tyodd = 0
  else
    tyodd = 1  ! odd number of y-pixels
  endif
  x2mid = prescan+tpix(1)-ovescan+txodd
  y2mid = tpix(2)+tyodd
  !
  txs(1) = prescan+1
  tys(1) = 1
  txe(1) = x2mid/2
  tye(1) = y2mid/2
  txs(2) = x2mid/2+1
  tys(2) = 1
  txe(2) = tpix(1)-ovescan
  tye(2) = y2mid/2
  txs(3) = prescan+1
  tys(3) = y2mid/2+1
  txe(3) = x2mid/2
  tye(3) = tpix(2)
  txs(4) = x2mid/2+1
  tys(4) = y2mid/2+1
  txe(4) = tpix(1)-ovescan
  tye(4) = tpix(2)
  !
  statistics/image {topicima} [@{txs(1)},@{tys(1)}:@{txe(1)},@{tye(1)}] -
    ? ? gn >Null
  twmin(1) = {outputr(1)}
  twmode(1) = {outputr(12)}
  twmedian(1) = {outputr(8)}
  twmean(1) = {outputr(3)}
  twstddev(1) = {outputr(4)}
  twmax(1) = {outputr(2)}
  !
  statistics/image {topicima} [@{txs(2)},@{tys(2)}:@{txe(2)},@{tye(2)}] -
    ? ? gn >Null
  twmin(2) = {outputr(1)}
  twmode(2) = {outputr(12)}
  twmedian(2) = {outputr(8)}
  twmean(2) = {outputr(3)}
  twstddev(2) = {outputr(4)}
  twmax(2) = {outputr(2)}
  !
  statistics/image {topicima} [@{txs(3)},@{tys(3)}:@{txe(3)},@{tye(3)}] -
    ? ? gn >Null
  twmin(3) = {outputr(1)}
  twmode(3) = {outputr(12)}
  twmedian(3) = {outputr(8)}
  twmean(3) = {outputr(3)}
  twstddev(3) = {outputr(4)}
  twmax(3) = {outputr(2)}
  !
  statistics/image {topicima} [@{txs(4)},@{tys(4)}:@{txe(4)},@{tye(4)}] -
    ? ? gn >Null
  twmin(4) = {outputr(1)}
  twmode(4) = {outputr(12)}
  twmedian(4) = {outputr(8)}
  twmean(4) = {outputr(3)}
  twstddev(4) = {outputr(4)}
  twmax(4) = {outputr(2)}
  !
  {stattable},:Min_1,{lopcount} = twmin(1)
  {stattable},:Mode_1,{lopcount} = twmode(1)
  {stattable},:Median_1,{lopcount} = twmedian(1)
  {stattable},:Mean_1,{lopcount} = twmean(1)
  {stattable},:Stddev_1,{lopcount} = twstddev(1)
  {stattable},:Max_1,{lopcount} = twmax(1)
  !
  {stattable},:Min_2,{lopcount} = twmin(2)
  {stattable},:Mode_2,{lopcount} = twmode(2)
  {stattable},:Median_2,{lopcount} = twmedian(2)
  {stattable},:Mean_2,{lopcount} = twmean(2)
  {stattable},:Stddev_2,{lopcount} = twstddev(2)
  {stattable},:Max_2,{lopcount} = twmax(2)
  !
  {stattable},:Min_3,{lopcount} = twmin(3)
  {stattable},:Mode_3,{lopcount} = twmode(3)
  {stattable},:Median_3,{lopcount} = twmedian(3)
  {stattable},:Mean_3,{lopcount} = twmean(3)
  {stattable},:Stddev_3,{lopcount} = twstddev(3)
  {stattable},:Max_3,{lopcount} = twmax(3)
  !
  {stattable},:Min_4,{lopcount} = twmin(4)
  {stattable},:Mode_4,{lopcount} = twmode(4)
  {stattable},:Median_4,{lopcount} = twmedian(4)
  {stattable},:Mean_4,{lopcount} = twmean(4)
  {stattable},:Stddev_4,{lopcount} = twstddev(4)
  {stattable},:Max_4,{lopcount} = twmax(4)
  !
  !
  !        DISPLAYING HISTOGRAMME :
  !        **********************
  !
  if nice_crea .eq. "y" then
    !
    create/icat histo.cat null >Null
    add/icat histo.cat {topicima} >Null
    !
    !**************************************************************
    @p fors_nicehist histo.cat 7. {prescan},{ovescan} ? ? 5.,5. ? n
    !**************************************************************
    if {q1} .eq. -1 then
      warn = 1
      if "{{out_a},:total_flat,1}" .eq. "x" goto hist_end
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Creation of histogramme failed !"
        mess/out 1 {PROID} "({topicima})"
      goto error
    endif
    !
    assign/graphic g,1 >Null
        mess/out 4 {PROID} "  plotting histogramme"
    set/graphic colour=7
    plot/histogram {topicima}
    !
    if ps .eq. "y" then
      !
        mess/out 4 {PROID} "  creating postscript file of histogramme"
      !
      assign/graphic g,1 >Null
      copy/graphic postscript
        char = m$index(topicima,".bdf")-1
        pscr = topicima(1:{char})//"_hist.ps"
      -rename postscript.ps {pscr}
    endif
    !
  endif
  hist_end:
  !
  !
  goto catloop
end_catloop:
!
!
-delete histo.cat
-delete histo_his.tbl
!
!
!          E N D   O F   L O O P   W I T H   T E S T   F R A M E S
!          * * * * * * * * * * * * * * * * * * * * * * * * * * * *
!
!
!
!          OUTPUT :
!          ******
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} ">>> Statistics are in <{stattable}>. <<<"
!
if nice_crea .eq. "y" .and. ps .eq. "y" then
        mess/out 4 {PROID} ">>> PS files of the plots are <*_hist.ps>. <<<"
endif
!
!
set/format
if nice_crea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" set/graphic default
  endif
  return 1
!
!
!-------------------------------------------------------------------------------
