! $Id: fors_swapunits.prg,v 1.9 1999/11/25 11:24:08 sbogun Exp $   
!+++ FORS_SWAPUNITS +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_swapunits.prg,v $
!
!.KEYWORDS      change coordinate systems
!
!.USAGE         @p fors_swapunits image system
!
!.INPUT         image   (P1): name of BDF image file
!               system  (P2): coordinates system: PIXEL or WORLD
!               imgsec  (P3): name of key holding image section in pixel coord.
!                               as real array:
!                                (1),(2) start in x and y
!                                (3),(4) end in x and y
!               coorsel (P4): coordinate restoration selection switch:
!                               0: restore X and Y
!                               1: restore only X
!                               2: restore only Y
!                             works only for P2=WORLD.
!               delew   (P5): delete world coordinates
!                               0: no, do not delete
!                               1: yes
!                             (do not save WORLD coordinates in descriptors)
!
!.OUTPUT        P1 will be changed.
!
!.RETURN        Q1:  1: successful swap return if PIXEL
!                    0: successful swap return if WORLD
!                   -1: a successful return failed
!
!.DEFAULT       P3 = +
!               P4 = 0
!               P5 = 0
!
!.PURPOSE       Save and change coordinate system descripitors
!
!.ALGORITHM     P2 = PIXEL: will set the coordinate system to PIXEL
!               . Check if coordinate system is already in PIXELs.
!                 - If so, then all done and quit.
!                 - Otherwise, store actual coordinate system descriptors
!                   (START,STEP,CUNIT) in new descriptors (S_...).
!                   Finally, set START,STEP,CUNIT to appropriate PIXEL values.
!                   START will be set to hierarchical keyword {h_winstrx} and
!                   {h_winstry} whereas STEP will be set to {h_xwinbin} and
!                   {h_ywinbin}.
!                   In case P3 has been passed the coordinates of this index
!                   array interpreted as pixel coordinates will be used. The
!                   two values represent the START values in x and y.
!
!               P2 = WORLD: will reset the WORLD coordinate system.
!               . Check if coordinate system is already a WORLD system.
!                 - If so, then no change had been done, quit.
!                 - Otherwise, reset START,STEP,CUNIT to the previously saved
!                   coordinate system descriptors and finally delete these
!                   save descriptors.
!                   Before the reset is done it will be checked if a world
!                   coordinate system has ever been saved before, otherwise
!                   the procedure will return with an error (-1).
!
!.ENVIRON       FORS context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/10/14  Creation
!
!.COMMENT       . Only 2-dimensional are processed correctly!
!               . Be aware that in case of P5=1 NO world coordinate
!                 RESTORATION may be done !
!
!------------------------------------------------------------------------------
define/param p1 ? c "Enter image name:"
define/param p2 ? c "Enter WORLD or PIXEL"
define/param p3 + c "Enter name of key holding image section in pixel coord."
define/param p4 0 n "Enter coordinate restoration index (0,1,2): "
define/param p5 0 n "Do not save world coordinates (1:YES, 0:NO): "


define/local i1/i/1/1 17
define/local i2/i/1/1 32
define/local j1/i/1/1 33
define/local j2/i/1/1 48

if P2(1:1) .eq. "P" then                        !set to PIXEL coordinates

  if m$existd(P1,"CUNIT") .eq. 0 return -1
  if m$index("{{P1},CUNIT({i1}:{i2})}","PIXEL") .gt. 0 .AND. -
     m$index("{{P1},CUNIT({j1}:{j2})}","PIXEL") .gt. 0 return 0
!  if m$index("{{P1},CUNIT}","PIXEL") .gt. 0 return 0
  if m$existd(P1,"S_CUNIT") .eq. 1 return -1


  if {P5} .eq. 0 then
    copy/dd {P1} CUNIT/c/1/48 {P1} S_CUNIT      !save CUNIT (= world coord.)
    copy/dd {P1} STEP/d/1/2   {P1} S_STEP
    copy/dd {P1} START/d/1/2  {P1} S_START
  endif

  if P3(1:1) .eq. "+" then
    {P1},START(1) = {{P1},{h_xwinstr}}
    {P1},START(2) = {{P1},{h_ywinstr}}
  else
    {P1},START(1) = {P3}(1)
    {P1},START(2) = {P3}(2)
  endif
  {P1},STEP(1)  = {{P1},{h_xwinbin}}
  {P1},STEP(2)  = {{P1},{h_ywinbin}}
  write/desc {P1} CUNIT/c/17/30 "PIXEL           PIXEL"

  return 1
else if P2(1:1) .eq. "W" then                   !reset to WORLD coordinates

!  if m$index("{{P1},CUNIT}","PIXEL") .eq. 0 return 0
  if m$index("{{P1},CUNIT({i1}:{i2})}","PIXEL") .eq. 0 .AND. -
     m$index("{{P1},CUNIT({j1}:{j2})}","PIXEL") .eq. 0 return 0

  if m$existd(P1,"S_CUNIT") .eq. 0 return -1


  if {P4} .eq. 0 .OR. {P4} .eq. 1 then          !P4 coordinate selection
    {P1},CUNIT({i1}:{i2}) = "{{P1},S_CUNIT({i1}:{i2})}" !restore world unit (X)
    {P1},START(1) = m$value({P1},S_START(1))    !0: restore both coordinates
    {P1},STEP(1)  = m$value({P1},S_STEP(1))     !1: restore only "X"-coordinate
  endif                                         !2: restore only "Y"-coordinate
  if {P4} .eq. 0 .OR. {P4} .eq. 2 then
    {P1},CUNIT({j1}:{j2}) = "{{P1},S_CUNIT({j1}:{j2})}" !restore world unit (Y)
    {P1},START(2) = m$value({P1},S_START(2))
    {P1},STEP(2)  = m$value({P1},S_STEP(2))
  endif

  delete/desc {P1} S_CUNIT
  delete/desc {P1} S_STEP
  delete/desc {P1} S_START

  return 0
endif

