! $Id: fors_t3dcpyima.prg,v 1.2 1999/02/22 11:26:17 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998 European Southern Observatory
!
!.IDENT       $RCSfile: fors_t3dcpyima.prg,v $
!
!.KEYWORDS    3d tables, copy an image row to a table element
!
!.USAGE       copy/t3d source scolumn crange target tcolumn trow
!
!.INPUT       SOURCE    (P1)  Source frame
!             SROW      (P2)  Source row specification
!             CRANGE    (P3)  First and last frame column to be copied
!             TCOLUMN   (P5)  Column label of 3d target table
!             TROW      (P6)  Row number of 3d target table
!
!.OUTPUT      TARGET    (P4)  Table name for 3d target table
!
!.RETURNS     The procedure returns 0 on success or 1 on error. The exit
!             status is stored in keyword Q1.
!
!.PURPOSE     Insert a pixel row of the source frame into a table element of
!             the target table, whose elements are one dimensional arrays.
!             The 3d table element must be sufficiently large to store the
!             frame row. This implies that the the column has to be
!             created with the appropriate sizes before the command can
!             be used. The frame row may be selected by either using world
!             coordinates or frame coordinates. Frame coordinates are
!             assumed if the row number string starts with the '@' 
!             character. Furthermore, a subframe of the selected row may be
!             specified by providing the first and the last pixel number.
!             Only this range of pixels is then copied to the target table
!             element.
!
!.COMMENT     The FORS version of copy/t3d differs from the general opy/t3d
!             in the way errors are handled and the format of the terminal
!             output. The general command is overwritten by the FORS version
!             when the FORS context is started. 
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.1 1998/12/18: Creation, RP
!
!------------------------------------------------------------------------------

crossref  source srow crange target tcolumn trow

define/param p1 ?  tab  "Enter source table name:"
define/param p2 ?  c    "Enter frame row:"
define/param p3 +  c    "Enter frame pixels (first,last):"
define/param p4 ?  tab  "Enter target table name:"
define/param p5 ?  c    "Enter target column name:"
define/param p6 ?  num  "Enter target row number:"
define/maxpar 6


! Local variables
define/local pos/i/1/1       0
define/local npxl/i/1/1      0
define/local crange/i/1/2    0     all
define/local source/c/1/60   {p1}
define/local srow/c/1/16     {p2}
define/local srange/c/1/60   {p3}
define/local target/c/1/60   {p4}
define/local tcolumn/c/1/16  {p5}
define/local trow/i/1/1      {p6}



npxl = m$value({p1},npix(1))

if p3(1:1) .eq. "+" then
  crange(1) = 1
  crange(2) = npxl
else
  pos = m$index(p3,",") - 1
  if pos .lt. 0 then
    crange(1) = {p3(1:{pos})}
    crange(2) = npxl
  elseif pos .eq. 0 then
    pos = pos + 2
    crange(1) = 1
    crange(2) = {p3({pos}:)}
  else
    crange(1) = {p3(1:{pos})}
    pos = pos + 2
    crange(2) = {p3({pos}:)}
  endif
endif


! Insert frame row into 3d table element

run PIPE_EXE:fors_t3dcpyima.exe
if {q1} .ne. 0 then
  return 1
endif

return 0
