! $Id: fors_t3dextract.prg,v 1.5 1999/04/16 17:31:53 forsp Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998 European Southern Observatory
!
!.IDENT       $RCSfile: fors_t3dextract.prg,v $
!
!.KEYWORDS    3d tables, extract table element array to table column
!
!.USAGE       extract/t3d source scolumn srow target tcolumn nval
!
!.INPUT       SOURCE    (P1)  Source 3d table file
!             SCOLUMN   (P2)  Source column label
!             SROW      (P3)  Row number of source table
!             TCOLUMN   (P5)  Column label of target table
!             NVAL      (P6)  Number of values to copy
!
!.OUTPUT      TARGET    (P4)  Table name for extracted element array
!
!.RETURNS     The procedure returns 0 on success or 1 on error. The exit
!             status is stored in keyword Q1.
!
!.PURPOSE     Extract an element array from the table column denoted
!             by SCOLUMN. The one dimensional array is copied to a column
!             of an ordinary table. The output column is created if it does
!             not already exist. If the column is already defined it is
!             replaced. The output table is created if it does not already
!             exist.
!
!.COMMENT     The FORS version of extract/t3d differs from the general
!             extract/t3d in the way errors are handled and the format
!             of the terminal output. The general command is overwritten
!             by the FORS version when the FORS context is started.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.1 1998/12/18: Creation, RP
!
!------------------------------------------------------------------------------

crossref  source scolumn target tcolumn trow nval

define/param p1 ?  tab  "Enter source table name:"
define/param p2 ?  c    "Enter source column name:"
define/param p3 ?  num  "Enter source row number:"
define/param p4 ?  tab  "Enter target table name:"
define/param p5 ?  c    "Enter target column name:"
define/param p6 +  num  "Enter number of array elements:"
define/maxpar 6


! Local variables
define/local pos/i/1/1       0
define/local ndata/i/1/1     0
define/local source/c/1/60   {p1}
define/local scolumn/c/1/16  {p2}
define/local srow/i/1/1      {p3}
define/local target/c/1/60   {p4}
define/local tcolumn/c/1/16  {p5}
define/local create/i/1/1    0



! Process parameters
if p6(1:1) .ne. "+" then
  ndata = {p6}
endif


! Check existance of output table
pos = m$indexb(target,".")
if pos .lt. 1 then
  target = target // ".tbl"
endif

if m$exist(target) .eq. 0 then
  create = 1
endif


! Extract element array to table column

run PIPE_EXE:fors_t3dextract.exe
if {q1} .ne. 0 then
  return 1
endif

return 0
