! $Id: fors_t3dinsert.prg,v 1.2 1999/02/22 11:26:37 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998 European Southern Observatory
!
!.IDENT       $RCSfile: fors_t3dinsert.prg,v $
!
!.KEYWORDS    3d tables, copy an ordinary table column to a table element
!
!.USAGE       insert/t3d source scolumn srange target tcolumn trow
!
!.INPUT       SOURCE    (P1)  Source table file
!             SCOLUMN   (P2)  Source column label
!             SRANGE    (P3)  First and last row to be copied
!             TCOLUMN   (P5)  Column label of 3d target table
!             TROW      (P6)  Row number of 3d target table
!
!.OUTPUT      TARGET    (P4)  Table name for 3d target table
!
!.RETURNS     The procedure returns 0 on success or 1 on error. The exit
!             status is stored in keyword Q1.
!
!.PURPOSE     Insert a column of the source table into a table element of
!             the target table, whose elements are one dimensional arrays.
!             The 3d table element must be sufficiently large to store the
!             table column. This implies that the the column has to be
!             created with the appropriate sizes before the command can
!             be used.
!
!.COMMENT     The FORS version of insert/t3d differs from the general
!             insert/t3d in the way errors are handled and the format
!             of the terminal output. The general command is overwritten
!             by the FORS version when the FORS context is started.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.1 1998/12/18: Creation, RP
!
!------------------------------------------------------------------------------

crossref  source scolumn srange target tcolumn trow

define/param p1 ?  tab  "Enter source table name:"
define/param p2 ?  c    "Enter source column name:"
define/param p3 +  c    "Enter range of rows (first,last):"
define/param p4 ?  tab  "Enter target table name:"
define/param p5 ?  c    "Enter target column name:"
define/param p6 ?  num  "Enter target row number:"
define/maxpar 6


! Local variables
define/local pos/i/1/1       0
define/local nrow/i/1/1      0
define/local rrange/i/1/2    0     all
define/local source/c/1/60   {p1}
define/local scolumn/c/1/16  {p2}
define/local srange/c/1/60   {p3}
define/local target/c/1/60   {p4}
define/local tcolumn/c/1/16  {p5}
define/local trow/i/1/1      {p6}



nrow = m$value({p1},tblcontr(4))

if p3(1:1) .eq. "+" then
  rrange(1) = 1
  rrange(2) = nrow
else
  pos = m$index(p3,",") - 1
  if pos .lt. 0 then
    rrange(1) = {p3(1:{pos})}
    rrange(2) = nrow
  elseif pos .eq. 0 then
    pos = pos + 2
    rrange(1) = 1
    rrange(2) = {p3({pos}:)}
  else
    rrange(1) = {p3(1:{pos})}
    pos = pos + 2
    rrange(2) = {p3({pos}:)}
  endif
endif


! Insert source column into 3d table element

run PIPE_EXE:fors_t3dinsert.exe
if {q1} .ne. 0 then
  return 1
endif

return 0
