! $Id: fors_tbl2ascii.prg,v 1.8 2001/04/09 16:03:05 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_tbl2ascii.prg,v $
!
! .KEYWORDS   Format conversion
!
! .PURPOSE    Transform table files to ASCII files (for SExtractor).
! __________
!
! .USAGE      @p fors_tbl2ascii midtbl
!
! .INPUT      midtbl (P1): MIDAS table;
!                            default: none
!
! .OUTPUT     <P1>.dat: ASCII table
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
!
!             Q2:  name of output ascii file
! __________
!
! .ALGORITHM  -
!
! .COMMENT    -
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/01/27  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref midtbl
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "tbl2ascii"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      midtbl (P1): MIDAS table (def.ext. ".tbl");"
  w/o "                     default: none"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
        if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? "        Enter MIDAS table (def.ext. ".tbl"): "
define/local intab/c/1/64 "{p1}"
if m$index(intab,".") .eq. 0 intab = intab//".tbl"
!
if m$exist(intab) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered file is not present !"
        mess/out 1 {PROID} "({intab})"
  goto error
endif
!
if m$filtyp(intab," ") .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Entered file is no MIDAS table !"
        mess/out 1 {PROID} "({intab})"
  goto error
endif
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
  define/local char/i/1/1 0
  define/local outascii/c/1/64 " " all
  define/local filecode/i/1/2 0 all
  define/local totrows/i/1/1 0
  define/local row/i/1/1 0
  define/local string/c/1/160 " " all
!
!
!          TBL -> ASCII :
!          ************
!
  char = m$index(intab,".")-1
  outascii = intab(1:{char})//".dat"
  !
        mess/out 4 {PROID} "converting: {intab} -> {outascii}"
!
open/file {outascii} write filecode
if filecode(1) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open new file {outascii} !"
  goto error
endif
show/table {intab} >Null
totrows = outputi(2)
do row = 1 totrows
  write/keyword string "{{intab},#1,@{row}}" >Null  ! Rem.trail.blanks - needed!
  write/file {filecode(1)} "{string}"
enddo
close/file {filecode(1)}
!
!
set/format
!
if warn .eq. 0 then
  return 0 {outascii}
else
  return -1 {outascii}
endif
!
error:
  set/format
  return 1 -99
!
!
!-------------------------------------------------------------------------------
