! $Id: fors_trim.prg,v 1.10 1999/04/26 12:26:26 nrainer Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_trim.prg,v $
!
!.KEYWORDS    Basic CCD reduction, pre- and overscan removal
!
!.USAGE       trim/frame  raw_frm red_frm trim
!
!.INPUT       raw_frm	  (P1)	  Input frame to be corrected.
!             trim	  (P3)	  Frame coordinates for extracting useful
!                                 image parts.
!.OUTPUT      red_frm	  (P2)	  Corrected frame.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. Keyword OUTPUTC holds the
!             coordinate string used for extraction of the image region.
!
!.PURPOSE     The recipe removes the pre- and overscan area from the input
!             frame. If a coordinate string is given, this image section is
!             regarded as the valid frame segment. The coordinate strings
!             maybe of any form usable with EXTRACT/IMAGE. If no coordinates
!             are given the useful frame segment is determined before it is
!             extracted. The extracted frame is written to the file specified
!             by 'red_frm', if it is present. Otherwise the input frame is
!             replaced by the extracted image.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/09/21:  Creation
!
!------------------------------------------------------------------------------

crossref input output window

define/param p1 ?  c  "Enter name of input frame:"
define/param p2 +  c  "Enter name of reduced frame:"
define/param p3 +  c  "Enter window coordinates:"
define/param p4 X  c  "Readout direction:"
define/maxpar 4


! Procedure identification
define/local procID/c/1/80	"TrimFrame"


! Local constants
define/local tmp/c/1/12      "middumma.bdf"


! Local variables
define/local i/i/1/1         0
define/local pidx/i/1/1      0
define/local raw_frm/c/1/80  {p1}
define/local red_frm/c/1/80  {p2}
define/local trim/c/1/80     {p3}
define/local rdir/c/1/1      {p4}


if red_frm(1:1) .eq. "+" then
  red_frm = raw_frm
  rename/ima {raw_frm} {tmp}
  raw_frm = tmp
endif

! Extract image segment
if trim(1:1) .eq. "+" then
  image/section {raw_frm} {rdir}
  if {q1} .ne. 0 then
    return 1
  else
!    trim = outputc			!world-coordinates of trim-area
    trim = out_b			!pixel-coordinates of trim-area
  endif
endif

!extract/image {red_frm} = {raw_frm} {trim}
extract/frame {raw_frm} {red_frm} {trim}
if {q1} .lt. 0 return 1

! Remove pre- and overscan descriptors from frame header
do i = 1 fors_nout
  pidx = fors_outi({i})
  if m$existd(red_frm,h_pscan({pidx})) .eq. 1 then
    delete/descr {red_frm} {h_pscan({pidx})}
  endif
  if m$existd(red_frm,h_oscan({pidx})) .eq. 1 then
    delete/descr {red_frm} {h_oscan({pidx})}
  endif
enddo


! Cleanup
-delete {tmp}

return 0
