! $Id: fors_tstmaster.prg,v 1.54 2001/04/06 16:19:24 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_tstmaster.prg,v $
!
! .KEYWORDS   Statistics
!
! .PURPOSE    Determine differences between (a) newly created master-frame(s)
!             and a certified one by the use of statistical tests.
!             The images and statistical plots can be displayed.
! __________
!
! .USAGE      @p fors_tstmaster imbi cert type [disp] [calc] [scan] -
!                               [ps] [thresh]
!
! .INPUT      imbi   (P1): _im_age catalogue with (a) new created master
!                          frame(s) AND _bi_nning factor, separated by a comma;
!                            default: none,none
!
!             cert   (P2): certified master frame;
!                            default: none
!
!             type   (P3): number for type of input frames: <1>:bias,<2>:dark,
!                          <3>:flatfield (<4>-<6> are treated as <3>);
!                            default: none
!
!             disp   (P4): display selection: <p>:pipeline mode:read global key
!                          NICE_CREA [y|n], <n>:none, <l>:local, or
!                          host:display[.screen] - with the latter option
!                          you can specify the name or IP-address of a display;
!                            default: <l>
!
!             calc   (P5): calculation flag: <>:calculate statistics of frames,
!                          or <u>se descriptor STATISTIC - a missing descriptor
!                          is calculated;
!                          if an input frame contains the special descriptor
!                          {MEDDESC} (see below), it is always used as median;
!                            default: <>
!
!             scan   (P6): number of pre- AND overscan columns, separated by
!                          a comma;
!                            default: 16,16
!
!             ps     (P7): flag for creation of histogramme postscript
!                          files: <>:do not create, <y>es,create;
!                            default: <>
!                            hint: ignored, if P4=n
!
!             thresh (P8): median+FACTOR*mediansigma -> outlier limit;
!                            default: 7.
!
! .OUTPUT     "<P1>_tst.tbl": statistics - see ALGORITHM
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Comparisons between
!             each binned difference frame "(certif.master-scalefactor*
!               new_master)", with the following scalefactors: for bias none,
!               for darks exp.time(certif.master)/exp.time(new_master), for
!               flats median(certif.master)/median(new_master),
!             and ideal zero values:
!               - mean (64x64 bins)
!               - standard deviation (64x64 bins)
!               - standard deviation (2x2 bins = 4 quadrants)
!             A FACTOR for calculating the outlier limit is needed: (median
!               value of the frame + this FACTOR * the median sigma of the
!               frame) yields the limit for detecting outliers;"
!
! .ENVIRON    -
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/03  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref imbi cert type disp calc scan ps thresh
!
!
if m$existk("MEDDESC") .eq. 0 -
  define/local MEDDESC/c/1/30 "ESO.PRO.DATAMEDI"
!
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "tstmaster"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For `longlog/out':
write/keyword out_a "given command/qualif not defined ..."
show/command longlog/out | write/keyword in_a
-delete Mid{mid$sess(11:12)}1Pipe
-delete midlis{mid$sess(11:12)}.prg
define/local longyes/i/1/1 1
if in_a .eq. out_a then
  create/command longlog/out "write/out"
  longyes = 0
endif
!
!  For return value:
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      imbi   (P1): _im_age catalogue with (a) new created master"
  w/o "                   frame(s) AND _bi_nning factor, separated by a comma;"
  w/o "                     default: none,none"
  w/o " "
  w/o "      cert   (P2): certified master frame;"
  w/o "                     default: none"
  w/o " "
  w/o "      type   (P3): number for type of input frames: <1>:bias,<2>:dark,"
  w/o "                   <3>:flatfield (<4>-<6> are treated as <3>);"
  w/o "                     default: none"
  w/o " "
  w/o "      disp   (P4): display selection: <p>:pipeline mode:read global key"
  w/o "                   NICE_CREA [y|n], <n>:none, <l>:local, or"
  w/o "                   host:display[.screen] - with the latter option"
  w/o "                   you can specify the name or IP-address of a display;"
  w/o "                     default: <l>"
  w/o " "
  w/o "      calc   (P5): calculation flag: <>:calculate statistics of frames,"
  w/o "                   or <u>se descriptor STATISTIC - a missing descriptor"
  w/o "                   is calculated;"
  w/o "                   if an input frame contains the special descriptor"
  w/o "                   {MEDDESC}, it is always used as median;"
  w/o "                     default: <>"
  w/o " "
  w/o "      scan   (P6): number of pre- and overscan columns, separated"
  w/o "                   by a comma;"
  w/o "                     default: 16,16"
  w/o " "
  w/o "      ps     (P7): flag for creation of histogramme postscript"
  w/o "                   files: <>:do not create, <y>es,create;"
  w/o "                     default: <>"
  w/o "                     hint: ignored, if P4=n"
  w/o " "
  w/o "      thresh (P8): median+FACTOR*mediansigma -> outlier limit;"
  w/o "                     default: 7."
  w/o "      ******************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(imbi):
!--------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Cat. with new master(s) AND binning factor: "
outputi = m$parse(p1,"catval")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P1 !"
  goto error
endif
define/local icat/c/1/64 "{catval01}"
if m$index(icat,".cat") .eq. 0 icat = icat//".cat"
!
if m$exist(icat) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
show/icat {icat} n
if {outputi(1)} .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered catalogue is empty !"
        mess/out 1 {PROID} "({icat})"
  goto error
endif
!
define/local catal/i/1/1 0
inloop:
  store/frame inputc {icat} 1 end_inloop
  if m$exist(inputc) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "At least one catalogue entry is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({icat})"
    goto error
  endif
  goto inloop
end_inloop:
!
! Following "+lower" for context independent
! test version of subprocedure "fors_bindif":
define/local binfact/i/1/1 {catval02} ? +lower_levels
!
if binfact .ne. 1 .and. binfact .ne. 2 then
  if binfact .ne. 4 then
        write/out " "
        mess/out 1 {PROID} "Binning factor not supported !"
        mess/out 1 {PROID} "({binfact})"
    goto error
  endif
endif
!
!
!P2(cert):
!--------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? i -
  "        Enter name of certified master frame for comparison: "
define/local cert/c/1/64 "{p2}"
if m$index(cert,".bdf") .eq. 0 cert = cert//".bdf"
!
if m$exist(cert) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered certified frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({cert})"
  goto error
endif
!
!
!P3(type):
!--------
define/parameter p3 ? ? -
  "        Enter image type [1:bias|2:dark|3:flat]:"
define/local type/i/1/1 {p3}
!
if type .ge. 4 .and. type .le. 6 then  ! for compatibility with other procedures
  type = 3
elseif type .lt. 1 .or. type .gt. 6 then
        write/out " "
        mess/out 1 {PROID} "Impossible frame type !"
  goto error
endif
!
!
!P4(disp):
!--------
define/parameter p4 l ? -
  "        Enter display [def.:<l>ocal|<p>ipeline|<n>one|host:disp[.screen]]: "
define/local dispflag/c/1/30 "{p4}"
!
!
!P5(calc):
!--------
define/parameter p5 + ? -
  "        Enter calculation flag [def.:calc_stat|<u>se_desc]:"
define/local calc/c/1/1 "{p5}"
!
!
!P6(scan):
!--------
define/parameter p6 16,16 ? -
  "        Enter pre-/overscan values [def.:16,16]: "
outputi = m$parse(p6,"scan")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in P6 !"
  goto error
endif
! Following "+lower" for context independent
! test version of subprocedure "fors_bindif":
define/local prescan/i/1/1 {scan01} ? +lower_levels 
define/local ovescan/i/1/1 {scan02} ? +lower_levels
!
if prescan .lt. 0 .or. ovescan .lt. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible pre- or overscan value !"
  goto error
endif
!
!
!P7(ps):
!------
define/parameter p7 + ? -
  "        Enter flag for PS files of histo.s [def.:n|<y>es,create]: "
define/local ps/c/1/1 "{p7}"
!
!
!P8(thresh):
!----------
define/parameter p8 7. n -
  "        Enter threshold factor for outliers : "
define/local thresh/r/1/1 {p8}
if thresh .le. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible thresh.fact. for outliers !"
  goto error
endif
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Preparing tests" :
  define/local char/i/1/2 0 all
  define/local stattable/c/1/64 " " all +lower_levels
!
!     "size of certified" :
  define/local cpix/i/1/2 0 all
!
!     "displaying of certified" :
  define/local clowcut/r/1/1 0.
  define/local chighcut/r/1/1 0.
!
!     "median of certified" :
  define/local cmedian/r/1/1 0.
  define/local csubxstart/i/1/1 0
  define/local csubxend/i/1/1 0
!
!     "L o o p   w i t h   n e w" :
  define/local topicima/c/1/64 " " all
  define/local lopcount/i/1/1 0
  !
  !   "size of new" :
  define/local npix/i/1/2 0 all
  !
  !   "level of new" :
  define/local nsubxstart/i/1/1 0
  define/local nsubxend/i/1/1 0
  define/local nmean/r/1/1 0.
  !
  !   "median of new" :
  define/local nmedian/r/1/1 0.
  !
  !   "creation ... diff" :
  define/local testscale/r/1/1 0.
  define/local bdmean2q/r/1/4 0.
  define/local bdmean64frm/c/1/64 " " all
  !
  !   "statistics ... diff" :
  define/local bdmean2/r/1/1 0.
  define/local bdstddev2/r/1/1 0.
  define/local bdmean64/r/1/1 0.
  define/local bdstddev64/r/1/1 0.
  !
  !   "Displaying histogrammes" :
  define/local pscr/c/1/64 " " all
!
!
!
!          MANAGING DISPLAYS / ITT / LUT :
!          *****************************
!
if dispflag .ne. "p" then
  !
  !**************************
  @p fors_nicecrea {dispflag}
  !**************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Managing of display options failed !"
    goto error
  endif
  !
endif
!
if nice_crea .eq. "y" then  ! written by fors_nicecrea.prg
  !
  if nice_dids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 1 is missing !"
    goto error
  endif
  if nice_dids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 2 is missing !"
    goto error
  endif
  if nice_dids(3) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Display window 3 is missing !"
    goto error
  endif
  if nice_gids(1) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 1 is missing !"
    goto error
  endif
  if nice_gids(2) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Graphic window 2 is missing !"
    goto error
  endif
  !
  assign/display d,1 >Null
  clear/display
  display/lut off
  label/display "CERTIFIED MASTER" {nice_dlab}
  assign/display d,2 >Null
  clear/display
  display/lut off
  label/display "NEW MASTER" {nice_dlab}
  assign/display d,3 >Null
  clear/display
  display/lut off
  label/display "DIFFERENCE" {nice_dlab}
  load/lut heat
  load/itt ramp
  assign/graphic g,1 >Null
  clear/graphic
  assign/graphic g,2 >Null
  clear/graphic
  set/graphic default
  set/graphic pmode=0 bcolour=1
  !
endif
!
!
!          PREPARING TESTS IN LOOP :
!          ***********************
!
        mess/out 4 {PROID} "preparing statistical tests"
!
  char = m$index(icat,".cat")-1
  stattable = icat(1:{char})//"_tst.tbl"
show/icat {icat} n
create/table {stattable} 8 {outputi(1)} null
!
create/column {stattable} :FRAME " " A49 C*64
create/column {stattable} :MEAN_DIFF "ADU" F9.2 R*4
create/column {stattable} :SIGM_DIFF "1_sigma_ADU" F9.2 R*4
create/column {stattable} :MEAN_A "ADU" F9.2 R*4
create/column {stattable} :MEAN_B "ADU" F9.2 R*4
create/column {stattable} :MEAN_C "ADU" F9.2 R*4
create/column {stattable} :MEAN_D "ADU" F9.2 R*4
create/column {stattable} :SIGM_ABCD "1_sigma_ADU" F9.2 R*4
!
!
!          DETERMINING SIZE OF CERTIFIED MASTER FRAME :
!          ******************************************
!
!          FOR "DETERMINING MEDIAN" AND "LOOP WITH NEW MASTER FRAMES" :
cpix(1) = {{cert},NPIX(1)}
cpix(2) = {{cert},NPIX(2)}
!
!
!          DISPLAYING OF CERTIFIED MASTER FRAME :
!          ************************************
!
if nice_crea .eq. "y" then
  !
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} "displaying certified frame"
        mess/out 4 {PROID} "({cert})"
  !
  !***************************************************************
  @p fors_nicedisp {cert} ? ? {prescan},{ovescan} {calc} ? 3.,3. n
  !***************************************************************
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of certified frame failed !"
        mess/out 1 {PROID} "({cert})"
    goto error
  endif
  !
        mess/out 4 {PROID} "  loading image into display 1"
  assign/display d,1 >Null
  load/image {cert} ? {outputi(1)},avg c,c
  label/display "CERTIFIED MASTER" {nice_dlab} ! removed by special load/image
  !
  !        FOR "LOOP WITH NEW MASTER FRAME(S)" :
  clowcut = {{cert},LHCUTS(1)}
  chighcut = {{cert},LHCUTS(2)}
  !
endif
!
!
!          FOR DIFFERENCE FLAT FRAME :
!          DETERMINING MEDIAN OF CERTIFIED MASTER FRAME :
!          ********************************************
!
if type .eq. 3 then  ! only needed for flats
  !
        mess/out 4 {PROID} "determining median of certified master frame"
  !
  if m$existd(cert,MEDDESC) .eq. 1 then
    !
        mess/out 4 {PROID} "  from descriptor {MEDDESC}"
    !
    cmedian = {{cert},{MEDDESC}}
  else
    !
        mess/out 4 {PROID} "  by new calculation"
    !
    csubxstart = prescan+1
    csubxend = cpix(1)-ovescan
    statistics/image {cert} [@{csubxstart},<:@{csubxend},>] ? ? gn >Null
    cmedian = outputr(8)
  endif
  !
endif
!
!
!
!          L O O P   W I T H   N E W   M A S T E R   F R A M E S :
!          * * * * * * * * * * * * * * * * * * * * * * * * * * *
!
!
        if mess_level .eq. 3 mess/out 3 {PROID} " "
!
catal = 0
catloop:
  store/frame topicima {icat} 1 end_catloop
  lopcount = lopcount+1
  show/icat {icat} n
        mess/out 4 {PROID} " "
        mess/out 3 {PROID} -
          "new master {lopcount} / {outputi(1)} : {topicima}"
  !
  {stattable},:FRAME,{lopcount} = "{topicima}"
  !
  !
  !        CHECKING SIZE OF NEW MASTER FRAME :
  !        *********************************
  !
  npix(1) = {{topicima},NPIX(1)}
  npix(2) = {{topicima},NPIX(2)}
  if npix(1) .ne. cpix(1) .or. npix(2) .ne. cpix(2) then
        set/format J5
        write/out " "
        mess/out 1 {PROID} "New and certified frame have different sizes:"
        mess/out 1 {PROID} "   X      Y"
        mess/out 1 {PROID} "  {npix(1)} {npix(2)}  <- {topicima}"
        mess/out 1 {PROID} "  {cpix(1)} {cpix(2)}  <- {cert}"
        set/format {FMTDEF}
    goto error
  endif
  !
  !
  !        CHECKING LEVEL OF NEW MASTER FRAME :
  !        **********************************
  !
        mess/out 3 {PROID} "  checking count level"
  !
  if calc .eq. "u" .and. m$existd(topicima,"STATISTIC") .eq. 0 then
        mess/out 2 {PROID} "  New frame: Descriptor "STATISTIC" is missing -"
        mess/out 2 {PROID} "  statistics will be calculated !"
        mess/out 2 {PROID} "  ({topicima})"
  endif
  if calc .ne. "u" .or. m$existd(topicima,"STATISTIC") .eq. 0 then
    nsubxstart = prescan+1
    nsubxend = npix(1)-ovescan
    statistics/image {topicima} [@{nsubxstart},<:@{nsubxend},>] ? ? sn >Null
  endif
  nmean = {{topicima},STATISTIC(3)}  ! NOT `outputr' because of above `if'
  !
  if nmean .le. 0. then  ! bias is already subtracted (if no bias frame) !
        write/out " "
        mess/out 1 {PROID} "New frame has negative or zero mean value !"
        mess/out 1 {PROID} "({topicima})"
    goto error
  endif
  !
  !
  !        DISPLAYING OF NEW MASTER FRAME :
  !        ******************************
  !
  if nice_crea .eq. "y" then
    !
        mess/out 4 {PROID} "  displaying new master frame"
    !
    !****************************************
    @p fors_nicedisp {topicima} n ? ? ? ? ? n
    !****************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of new frame failed !"
        mess/out 1 {PROID} "({topicima})"
      goto error
    endif
    !
        mess/out 4 {PROID} "    loading image into display 2"
    assign/display d,2 >Null
    ! cuts from certified frame:
    load/image {topicima} ? {outputi(1)},avg c,c {clowcut},{chighcut}
    label/display "NEW MASTER" {nice_dlab}  ! removed by special load/image
    !
  endif
  !
  !
  !        FOR DIFFERENCE FLAT FRAME :
  !        DETERMINING MEDIAN OF NEW MASTER FRAME :
  !        **************************************
  !
  if type .eq. 3 then  ! only needed for flats
    !
        mess/out 4 {PROID} "  determining median of new master frame"
    !
    if m$existd(topicima,MEDDESC) .eq. 1 then
      !
        mess/out 4 {PROID} "    from descriptor {MEDDESC}"
      !
      nmedian = {{topicima},{MEDDESC}}
    else
      !
        mess/out 4 {PROID} "    by new calculation"
      !
      nsubxstart = prescan+1
      nsubxend = npix(1)-ovescan
      statistics/image {topicima} [@{nsubxstart},<:@{nsubxend},>] ? ? gn >Null
      nmedian = outputr(8)
    endif
    !
  endif
  !
  !
  !        CREATION OF BINNED DIFFERENCE FRAME :
  !        ***********************************
  !
        mess/out 3 {PROID} "  creating binned difference frame (64x64 bins)"
  !
  branch type 1,2,3 bias,dark,flat
    bias:
      testscale = 1.
      goto end_scaletype
    dark:
      testscale = {{topicima},O_TIME(7)} / {{cert},O_TIME(7)}
      goto end_scaletype
    flat:
      testscale = nmedian / cmedian
  end_scaletype:
  !
  ! So scaling is done according to the formula :
  ! diff. = cert. - new/testscale
  !
  !***************************************************************
  @p fors_bindif {cert} {topicima} 64 64 {thresh} 1 1. {testscale}
  !***************************************************************
  if {q1} .eq. -1 then  ! negative error is exception
        write/out " "
        mess/out 1 {PROID} "Creation of binned frames failed !"
    goto error
  endif
  !
  bdmean2q(1) = outputd(1)  ! means of quadrants:
  bdmean2q(2) = outputd(2)  ! 3(C) | 4(D)
  bdmean2q(3) = outputd(3)  ! -----+-----
  bdmean2q(4) = outputd(4)  ! 1(A) | 2(B)
  bdmean64frm = out_a
  !
  !
  !        DISPLAYING OF BINNED DIFFERENCE FRAME :
  !        *************************************
  !
  if nice_crea .eq. "y" then
    !
        mess/out 3 {PROID} "  displaying difference frame"
    !
    !*****************************************************************
    @p fors_nicedisp {bdmean64frm} ? n {prescan},{ovescan} ? ? 2.,2. n
    !*****************************************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Displaying of difference frame failed !"
        mess/out 1 {PROID} "({bdmean64frm})"
      goto error
    endif
    !
        mess/out 4 {PROID} "    loading image into display 3"
    assign/display d,3 >Null
    if outputi(1) .le. -2 then
      load/image {bdmean64frm} ? {outputi(1)},avg c,c
    else
      load/image {bdmean64frm} ? {outputi(1)} c,c
    endif
    label/display "DIFFERENCE" {nice_dlab}  ! removed by special load/image
    !
  endif
  !
  !
  !        STATISTICS OF BINNED DIFFERENCE FRAME
  !           FOR DETERMINATION OF GOODNESS :
  !        *************************************
  !
        mess/out 3 {PROID} "  calculating statistics of difference frame"
        mess/out 4 {PROID} "    checking quadrants"
  !
  {stattable},:MEAN_A,{lopcount} = {bdmean2q(1)}
  {stattable},:MEAN_B,{lopcount} = {bdmean2q(2)}
  {stattable},:MEAN_C,{lopcount} = {bdmean2q(3)}
  {stattable},:MEAN_D,{lopcount} = {bdmean2q(4)}
  bdmean2 = (bdmean2q(1)+bdmean2q(2)+bdmean2q(3)+bdmean2q(4))/4.
  bdmean2q(1) = (bdmean2q(1)-bdmean2)**2
  bdmean2q(2) = (bdmean2q(2)-bdmean2)**2
  bdmean2q(3) = (bdmean2q(3)-bdmean2)**2
  bdmean2q(4) = (bdmean2q(4)-bdmean2)**2
  bdstddev2 = m$sqrt( (bdmean2q(1)+bdmean2q(2)+bdmean2q(3)+bdmean2q(4))/3. )
  {stattable},:SIGM_ABCD,{lopcount} = {bdstddev2}
  !
           mess/out 4 {PROID} "    checking all 64x64 bins"
  !
  statistics/image {bdmean64frm} + ? ? sn >Null  ! no pre-/overscan present
  bdmean64 = outputr(3)
  bdstddev64 = outputr(4)
  {stattable},:MEAN_DIFF,{lopcount} = {bdmean64}
  {stattable},:SIGM_DIFF,{lopcount} = {bdstddev64}
  !
  !
  !        DISPLAYING HISTOGRAMMES :
  !        ***********************
  !
  if nice_crea .eq. "y" then
    !
    create/icat histo.cat null >Null
    add/icat histo.cat {cert} >Null
    add/icat histo.cat {topicima} >Null
    !
    !***********************************************************
    @p fors_nicehist histo.cat 7. {prescan},{ovescan} {calc} ? -
      5.,5. s,{testscale} n
    !***********************************************************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Creation of histogrammes failed !"
        mess/out 1 {PROID} "({cert}/{topicima})"
      goto error
    endif
    !
    assign/graphic g,1 >Null
        mess/out 4 {PROID} "  plotting histogramme of cert. master (red)"
    set/graphic colour=2
    plot/histogram {cert}
    set/graphic colour=5
    overplot/histogram {topicima}
    !
    !***************
    @p fors_nicegrap
    !***************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of histogrammes failed !"
        mess/out 1 {PROID} "({cert}/{topicima})"
      goto error
    endif
    !
        mess/out 4 {PROID} "  plotting histogramme of new master (yellow)"
    set/graphic colour=3
    label/graphic "{cert} (red)  {topicima} (yellow)" {nice_glabw}
    !
    if ps .eq. "y" then
        mess/out 4 {PROID} "  creating postscript file of cert./new hist."
      assign/graphic g,1 >Null
      copy/graphic postscript
        char = m$index(cert,".bdf")-1
        char(2) = m$index(topicima,".bdf")-1
        pscr = cert(1:{char}) // "__" // topicima(1:{char(2)}) // ".ps"
      -rename postscript.ps {pscr}
    endif
    !
  endif
  !
  if nice_crea .eq. "y" then
    !
    create/icat histo.cat null >Null
    add/icat histo.cat {bdmean64frm} >Null
    !
    !**************************************************
    @p fors_nicehist histo.cat n 0,0 {calc} ? 5.,5. ? n  ! no pre-/overscan
    !**************************************************
    if {q1} .eq. -1 then
      warn = 1
      if "{{out_a},:total_flat,1}" .eq. "x" goto hist_end
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Creation of histogramme failed !"
        mess/out 1 {PROID} "({bdmean64frm})"
      goto error
    endif
    !
    assign/graphic g,2 >Null
        mess/out 4 {PROID} "  plotting difference histogramme"
    set/graphic colour=7
    plot/histogram {bdmean64frm}
    !
    !***************
    @p fors_nicegrap
    !***************
    if {q1} .eq. -1 then
      warn = 1
    elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Label of histogramme failed !"
        mess/out 1 {PROID} "({bdmean64frm})"
      goto error
    endif
    !
    set/graphic colour=3
    label/graphic "{bdmean64frm}" {nice_glabw}
    !
    if ps .eq. "y" then
        mess/out 4 {PROID} "  creating postscript file of diff. hist."
      assign/graphic g,2 >Null
      copy/graphic postscript
        pscr = cert(1:{char}) // "__" // topicima(1:{char(2)}) // "__diff.ps"
      -rename postscript.ps {pscr}
    endif
    !
  endif
  hist_end:
  !
  !
  goto catloop
end_catloop:
!
!
-delete histo.cat
-delete histo_his.tbl
!
!
!          E N D   O F   L O O P   W I T H   N E W   M A S T E R   F R A M E S
!          * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
!
!
!
!          OUTPUT :
!          ******
!
if mess_level .ge. 3 then
        mess/out 3 {PROID} " "
        mess/out 3 {PROID} "Statistical results:"
        write/out " "
  read/table {stattable}
        write/out " "
endif
!
if longyes .eq. 1 then
  @p fors_tstmaster,longlog
endif
!
        mess/out 4 {PROID} " "
        mess/out 4 {PROID} ">>> Statistics are in <{stattable}>. <<<"
!
if nice_crea .eq. "y" .and. ps .eq. "y" then
        mess/out 4 {PROID} ">>> PS files of the plots are <*__*.ps>. <<<"
endif
!
!
set/format
if nice_crea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" set/graphic default
  endif
  return 1
!
!
!-------------------------------------------------------------------------------
!
entry longlog
!
  define/local tmpcode/i/1/2 0 all
  define/local lastrow/i/1/1 0
  define/local txtline/i/1/1 0
  define/local oneline/c/1/130 " " all
  define/local head/c/1/69 " " all
  define/local minus/c/1/69 " " all
  !
  assign/print file tmp.txt
  print/table {stattable} 130
  assign/print terminal
  !
  open/file tmp.txt read tmpcode
  if tmpcode(1) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open tmp.txt !"
    goto error
  endif
  show/table {stattable} >Null
  lastrow = outputi(2)+3
  do txtline = 1 lastrow
    read/file {tmpcode(1)} oneline
    if txtline .eq. 2 head = oneline(61:)
    if txtline .eq. 3 minus = oneline(61:)
  enddo
  close/file {tmpcode(1)}
  longlog/out 1 "     {head}" >Null
  longlog/out 1 "     {minus}" >Null
  open/file tmp.txt read tmpcode
  do txtline = 1 lastrow
    read/file {tmpcode(1)} oneline
    if txtline .ge. 4 then
      !
      outputc = oneline(11:59)
      longlog/out 1 "     {outputc}:" >Null
      longlog/out 1 "     {oneline(61:)}" >Null
      !
    endif
  enddo
  close/file {tmpcode(1)}
  longlog/out 1 "     {minus}" >Null
  !
  -delete tmp.txt
!
!
!-------------------------------------------------------------------------------
