! $Id: fors_ucalib_test.prg,v 1.37 2000/02/08 10:49:38 sbogun Exp $   
!+++ FORS_UCALIB_TEST +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         fors_ucalib_test.prg
!
!.KEYWORDS      SUPER RECIPE: Perform a calibration data test
!
!.USAGE         @p (IDENT) sof_in sof_ref out type
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!               sof_ref (P2): set of reference frames (ascii list)
!               type    (P4): exposure type
!                             (1: bias, 2: dark, 3: skyflat, 4: screenflat,
!                              5: screenflat(LSS))
!               thresh  (P5): threshold
!               maxov   (P6): maximum allowed number of overexpos. pixels in %
!               scale   (P7): "MED": the bias correction will be done using the 
!                                   scaled master bias. The scaling only applies
!                                   in four-port mode and is done as follows: 
!                                   Each of the quadrants of the master bias is 
!                                   divided by its median and multiplied by the
!                                   median of the corresponding pre-/overscan 
!                                   region of the input frame.
!                            "OVE": like "MED", but each of the quadrants of the
!                                   master bias is divided by the median of its
!                                   corresponding pre-/overscan region and
!                                   multiplied by the median of the pre-/overscan
!                                   region of the input frame.
!                            "NO":  The master bias will not be scaled.
!
!.OUTPUT        out     (P3): path + basename of output files:
!
!.RETURN        Q1:  0: successful return
!                    otherwise: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Perform a complete calibration data test. That includes:
!                       a) Check for overexposed pixels (not for bias, dark).
!                       b) Determine bias from overscan regions.
!                       c) Compare reduced calibration frame with its master.
!
!.ALGORITHM     Calls single recipes.
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1997/12/18  Creation
!               1.0 1998/12/03  main product defined: CHKCAL-TABLE
!                               parameter handling implemented
!                   1998/12/08  paragraphs implemented
!                   1999/01/07  shortlog implemented
!               1.2 1999/02/03  NO defaults for thresholds P6.
!                               update write/error call
!                   1999/02/08  LSS and MOS screenflat processing implemented
!                   1999/02/10  exposure id extended: 5=screenflat(LSS)
!               1.3 1999/09/03  Product header generation implemented (SB).
!
!.COMMENT
!------------------------------------------------------------------------------
define/param p1 ? c "Enter name of ASCII filelist with science frames:"
define/param p2 ? c "Enter name of ASCII filelist with reference frames:"
define/param p3 ? c "Enter name of output-basename (+path):"
!define/param p3 ? c "Enter name of ASCII filelist with output frames:"
define/param p4 0 n "Exposure type (1: bias, 2: dark, 3: skyflat, 4: screenflat(IMG), 5: screenflat(LSS)):"
define/param p5 ? ? "Enter thresholds as defined in the rrd-files:"
define/param p6 3 n "Enter maximum allowed number of overexposed pixels in %:"
define/param p7 OVE c "Use scaled master bias (MED/OVE/NO):"


define/local part/i/1/1 0
define/local etyp/i/1/1 0
define/local typs/i/1/10 0 ALL
define/local procid/c*25/1/5 " " ALL
define/local typid/c*25/1/5 " " ALL
define/local pid/c/1/25 " " ALL
define/local ityp/c/1/30 " " ALL
define/local line/c/1/79 "-" ALL
define/local chkcaltbl/c/1/80 " " ALL
define/local errmsg/c*80/1/10 " " ALL
define/local scale/c/1/3   " "
scale = m$upper(p7(1:3))

errmsg(1) = "Error in INIT/PIPELINE occured !"
errmsg(2) = "Error in fors_gen_qc0_overexpo occured !"
errmsg(3) = "Error in CHECK/OVERSCAN occured !"
errmsg(4) = "Error in OVERSCAN/BIAS occured !"
errmsg(5) = "Error in REDUCE/IMAGE occured !"
errmsg(6) = "Error in HF-flatfield creation occured !"
errmsg(7) = "Error in fors_gen_qc1_chkcal occured !"
errmsg(8) = "Error in fors_gen_cal_illflat occured !"
errmsg(9) = "Error while identifying main product !"
errmsg(10) = "Error in FINISH/PIPELINE !"

etyp = {p4} + 5
if etyp .gt. 9 etyp = 9                 !necessary for NORMFLATs
procid(1) = "fors_s_gen_ubias_test"
procid(2) = "fors_s_gen_udark_test"
procid(3) = "fors_s_img_uskyflat_test"
procid(4) = "fors_s_img_uscrflat_test"
procid(5) = "fors_s_lss_uscrflat_test"

pid = procid({P4})                      !process name

typid(1) = BIAS_ID                     ! 1: bias
typid(2) = DARK_ID                     ! 2: dark
typid(3) = SKY_ID                      ! 3: skyflat
typid(4) = FLAT_ID                     ! 4: screen flat
typid(5) = FLAT_ID                     ! 5: norm flat (LSS)

ityp = typid({p4})                     ! frame type


LOG/REC {pid} IN SUPER
@p fors_paragraph 1 0


part = 1 
INIT/PIPELINE {P1} {P2} XSFinput.cat CCDRED + + XSFreject.cat XSFresult.cat
if {Q1} .gt. 0 goto ABORT
!
init/prohead {p1} {p2} 1,"Compare calibration frame with master" -
             "thresh,{p5}" "maxper,{p6}" "scale,{p7}"
if {Q1} .gt. 0 goto ABORT

initialize/longlog {p1} {p2} 1 -
  "thresholds:       {p5}" "max.overexp.pix.: {p6}%" "scale: {p7}"
!
!now all input frames and reference frames are stored in the image catalog:
!   XSFinput.cat
!thus P2 may be ignored (+) in the following recipe-calls.
!
!

if {P4} .lt. 3 goto OVSC               ! no overexposed-check for bias and dark
!------------------------------------------------------------------------------
!check if flatfield is overexposed
!
part = 2
@p fors_paragraph 2 0
mess/out 3 {pid} "Check for overexposed pixels ..."
write/out {line}
@p fors_gen_qc0_overexpo XSFinput.cat + XSFinput.cat,XSFresult.cat {P6}
if {Q1} .gt. 0 goto ABORT

!------------------------------------------------------------------------------
!determine overscan
!
OVSC:
!! part = 3
!! write/out
!! mess/out 3 {pid} "Determine bias from overscan ..."
!! check/overscan XSFinput.cat + XSFinput.cat,XSFresult.cat
!! if {Q1} .ne. 0 goto ABORT
!! 
!------------------------------------------------------------------------------
!do the reduction
!
REDU:
part = 5
write/out
mess/out 3 {pid} "Reduce calibration frame ..."
reduce/image XSFinput.cat + XSFinput.cat,XSFresult.cat P6={etyp} P7={scale}
if {Q1} .ne. 0 goto ABORT
!------------------------------------------------------------------------------
!NORM-flat creation for spectroscopic mode
!
Q1 = "0 "                                               !do the following
if {P4} .lt. 4 goto CONT                                !only for screen flats,
part = 6                                                !otherwise skip.
BRANCH fors_insmode(1:3) IMG,LSS,MOS IMG,LSS,MOS
goto CONT
IMG:
  goto CONT

LSS:
  errmsg(6) = "Error in NORMFLAT/LSS ocured !"
  normflat/lss XSFinput.cat + XSFinput.cat,XSFresult.cat LSS + + 0,0,N REDUCED
  goto CONT

MOS:
  errmsg(6) = "Error in NORMFLAT/MOS ocured !"
  normflat/mos XSFinput.cat + XSFinput.cat,XSFresult.cat MOS + + 0,0,N REDUCED

CONT:
  if {Q1} .ne. 0 goto ABORT


!------------------------------------------------------------------------------
!check data
!
part = 7
write/out
mess/out 3 {pid} "Check calibration frame ..."
write/out {line}
@p fors_gen_qc1_chkcal XSFinput.cat + XSFinput.cat,XSFresult.cat {p4} {p5}
if {Q1} .gt. 0 goto ABORT

!------------------------------------------------------------------------------
!write FITS files to specified output directory
!
part = 9
@p fors_paragraph 3 0
search/icat XSFresult {CHKCALT_ID} >Null
if m$len(out_a) .gt. 0 then
  chkcaltbl = out_a
else
  mess/out 1 {pid} "No result frame ({CHKCALT_ID}) found !"
  goto ABORT
endif


part = 10
write/out
mess/out 3 {pid} "Finish Pipeline"
finish/prohead XSFresult.cat 1
FINISH/PIPELINE XSFresult.cat {P3} CCDRED {chkcaltbl}
if {Q1} .gt. 0 goto ABORT
!
finish/longlog XSFinput.cat XSFresult.cat {chkcaltbl} 1 {p3}


LOG/REC {pid} OUT SUPER
@p fors_paragraph 4 0
return 0


ABORT:
finish/longlog XSFinput.cat -1 ? 1 {p3}
mess/out 1 {pid} "{errmsg({part})}"
write/error {part},APP,NO "{errmsg({part})}"

