! $Id: fors_verifyrange.prg,v 1.5 1999/03/16 08:27:37 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_verifyrange.prg,v $
!
!.KEYWORDS    Check if spectra are defined over the same wavelength range,
!             extinction coefficient computation.
!
!.USAGE       verfy/ranges stdcat wstart wend
!
!.INPUT       stdcat      (P1)    Catalog of standard star spectra to be
!                                 checked.
!             wstart      (P2)    Maximum start wavelength (Angstrom) of
!                                 spectral range.
!             wend        (P3)    Minimum end wavelength (Angstrom) of
!                                 spectral range.
!
!.OUTPUT      None
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. The keyword OUTPUTI(1) is 
!             updated with the number of standard star tables in the catalog.
!
!.PURPOSE     The procedure verifies that the wavelength range defined by
!             wstart and wend is common to all standard star spectra listed
!             in the input catalog. Spectra listed in the catalog with a
!             start wavelength larger than wstart or an end wavelength
!             smaller than wend are removed from the input catalog. The
!             remaining number of standard star tables in the catalog is
!             written to the keyword outputi(1) (cf. RETURNS). If both, 
!             wstart and wend are equal to zero, the limiting wavelengths
!             are determined from the first table in the input catalog.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/12/02:  Creation
!
!------------------------------------------------------------------------------

crossref  stdcat wstart wend

define/param p1 ? c  "Catalog of standard star tables:"
define/param p2 + c  "Start wavelength (Angstrom):"
define/param p3 + c  "End wavelength (Angstrom):"
define/maxpar 3


! Procedure identification
define/local procID/c/1/20      "VerifyRange"


! Local constants
define/local tmpcat/c/1/20      "tmp_stdspec.cat"


! Local variables
define/local ntbl/i/1/1         0
define/local state/i/1/1        0
define/local catal/i/1/1        0
define/local wstart/d/1/1       0.
define/local wend/d/1/1         0.
define/local rstart/d/1/1       0.
define/local rend/d/1/1         0.
define/local cat_entry/c/1/512  " "   all
define/local stdcat/c/1/100     {p1}



! Process parameters
if m$tstno(p2) .ne. 1 then
  mess/out 1 {procID} "Invalid start wavelength!"
  return 1
else
  wstart = {p2}
endif

if m$tstno(p3) .ne. 1 then
  mess/out 1 {procID} "Invalid start wavelength!"
  return 1
else
  wend = {p3}
endif


! Get number of catalog entries
show/icat {stdcat} N
ntbl = outputi(1)

-copy {stdcat} {tmpcat}

read_cat_entry:

store/frame cat_entry {tmpcat} 1 exit_loop

! Check existence and the presence of columns START, STEP and NDATA in
! standard star table.
if m$exist(cat_entry) .ne. 1 then
  mess/out 1 {proID} "Input table >{cat_entry}< does not exist!"
  state = 1
  goto exit_loop
else
  if m$existc(cat_entry,":{c_sostart}") .lt. 0 then
    mess/out 1 {procID} "Column :{c_sostart} not found in {cat_entry}!"
    state = 1
    goto exit_loop
  else if m$existc(cat_entry,":{c_sostep}") .lt. 0 then
    mess/out 1 {procID} "Column :{c_sostep} not found in {cat_entry}!"
    state = 1
    goto exit_loop
  else if m$existc(cat_entry,":{c_sondata}") .lt. 0 then
    mess/out 1 {procID} "Column :{c_sondata} not found in {cat_entry}!"
    state = 1
    goto exit_loop
  else

! If no range is set explicitly we use the range of the first table
! as reference.
    if wstart .eq. wend .and. wstart .eq. 0. then
      wstart = m$value({cat_entry},:{c_sostart},1)
      wend = wstart + m$value({cat_entry},:{c_sondata},1) * -
             m$value({cat_entry},:{c_sostep},1)
    endif
    rstart = m$value({cat_entry},:{c_sostart},1)
    rend = rstart + m$value({cat_entry},:{c_sondata},1) * -
           m$value({cat_entry},:{c_sostep},1)
  endif
endif

if rstart .gt. wstart .or. rend .lt. wend then
  subtract/icat {stdcat} {cat_entry}
  ntbl = ntbl - 1
endif

goto read_cat_entry


exit_loop:


! Update wavelength range
rebstrt = wstart
rebend = wend
outputi(1) = ntbl


! Cleanup
-delete {tmpcat}

return {state}
