! $Id: fors_view_delta.prg,v 1.32 2000/12/15 14:33:01 nrainer Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 2000  European Southern Observatory (ESO)
!
! .IDENT      $RCSfile: fors_view_delta.prg,v $
!
! .KEYWORDS   Spectroscopical quality, wavelength calibration
!
! .PURPOSE    Compute residuals between the fit used for the calculation
!             of the dispersion solution and the single identified lines
!             which contributed to the fit.
!             The high order differences (constant and linear term are
!             subtracted) can be plotted.
! __________
!
! .USAGE      @p fors_view_delta [wlc] slit [linpos] mos [linfit] [graph]
!
! .INPUT      wlc    (P1): reduced lamp frame;
!                            default: fors_linsrch.bdf
!
!             slit   (P2): slit number to be used;
!                            default: none
!
!             linpos (P3): table with identified line positions;
!                            default: fors_linpos.tbl
!
!             mos    (P4): table with slit offset;
!                            default: none
!
!             linfit (P5): table with dispersion solution;
!                            default: fors_coerbr.tbl
!
!             graph  (P6): <n> for no graphic OR number of graph. window (0-9)
!                          -if the window exists it is used, else it is created-
!                          AND flag for background colour: <w>hite or <b>lack,
!                          separated by a comma;
!                            default: 1,w
!
! .OUTPUT     Q2: name of table with residuals
!
!             Statistics of residuals:
!               OUTPUTR(1): minimum
!               OUTPUTR(2): maximum
!               OUTPUTR(3): mean value
!               OUTPUTR(4): standard deviation
!
!             Central wavelength:
!               in OUTPUTD(1) and in descriptor {Q2},DELTA_LAMCEN
!
!             Archive file name:
!               in descriptor {Q2},DELTA_ARCFIL
!
!             If P6(graph)!="n":
!             Q3: Name of saved plot file
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  0. Creation of "artificial" x-pixel values
!             1. Application of original dispersion coefficients to (0.)
!             2. Linear regression with the original identified lines
!             3. Subtraction of constant and linear term (2.) from original
!                fit (1.) (and optional plotting)
!             4. Subtraction of constant and linear term (2.) from original
!                identified lines (input) (and optional overplotting)
!             5. Calculation of residuals: (4.) minus (3.) (for (3.) techni-
!                cally new original fit because of different number of rows)
!
! .COMMENT    * The dispersion solution must have been computed with
!               'calibrate/grism' (="moscalifors.prg") AND THE TYPE
!               OF THE USED POLYNOMIAL MUST HAVE BEEN "POLYNOM" (NOT
!               "CHEBYSHEV" OR "LEGENDRE")!
!             * The slit position table must have been selected with
!               'define/position' (="fors_defpositions.prg").
!
! .ENVIRON    MOS context for programmes mentioned in .COMMENT
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.5  00/06/21  Creation
!                  00/       Release
!
!------------------------------------------------------------------------------
!
!
crossref wlc slit linpos mos linfit graph
!
!
define/local MAXSLITNO/i/1/1  19             ! high limit of slit number
define/local MINGRA/i/1/1     0              ! low limit of graph.win.no.
define/local MAXGRA/i/1/1     9              ! high limit of graph.win.no.
define/local GRACOO/c/1/19    710,506,0,692  ! geometry of graphic window
define/local PLOT_FILE/c/1/40 "delta.plt"    ! name for saved plot file
define/local DELTAB/c/1/64    "delta.tbl"    ! for fit and results
define/local SELTAB/c/1/64    "selec.tbl"    ! for identified lines
define/local EPS/r/1/1        0.01           ! TO FIND THE CORRECT Y-VALUES
!
!  For `set/format':
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
!  For `mess/out':
define/local PROID/c/1/40 "view_delta"
if m$existk("mess_level") .eq. 0 write/keyword mess_level/i/1/1 3
!
!  For `longlog/out':
write/keyword out_a "given command/qualif not defined ..."
show/command longlog/out | write/keyword in_a
-delete Mid{mid$sess(11:12)}1Pipe
-delete midlis{mid$sess(11:12)}.prg
if in_a .eq. out_a create/command longlog/out "write/out"
!
!  For return values:
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
write/keyword q3 0
!
!
if p1 .eq. "??" then
  w/o "      *****************************************************************"
  w/o "      wlc    (P1): reduced lamp frame;"
  w/o "                     default: fors_linsrch.bdf"
  w/o " "
  w/o "      slit   (P2): slit number to be used;"
  w/o "                     default: none"
  w/o " "
  w/o "      linpos (P3): table with identified line positions;"
  w/o "                     default: fors_linpos.tbl"
  w/o " "
  w/o "      mos    (P4): table with slit offset;"
  w/o "                     default: none"
  w/o " "
  w/o "      linfit (P5): table with dispersion solution;"
  w/o "                     default: fors_coerbr.tbl"
  w/o " "
  w/o "      graph  (P6): <n> for no graphic OR number of graph. window (0-9)"
  w/o "                   -if the window exists it is used, else it's created-"
  w/o "                   AND flag for background colour: <w>hite or <b>lack,"
  w/o "                   separated by a comma;"
  w/o "                     default: 1,w"
  w/o "      _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _"
  w/o "      Q2: name of table with residuals"
  w/o " "
  w/o "      Statistics of residuals:"
  w/o "        OUTPUTR(1): minimum"
  w/o "        OUTPUTR(2): maximum"
  w/o "        OUTPUTR(3): mean value"
  w/o "        OUTPUTR(4): standard deviation"
  w/o " "
  w/o "      Central wavelength:"
  w/o "        OUTPUTD(1)"
  w/o "      *****************************************************************"
  return/exit
endif
!
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(wlc):
!-------
define/parameter p1 fors_linsrch.bdf ? -
  "        Enter reduced lamp frame [def.:fors_linsrch.bdf]: "
define/local wlc/c/1/64 "{p1}"
if m$index(wlc,".bdf") .eq. 0 wlc = wlc//".bdf"
if m$exist(wlc) .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered reduced lamp frame is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({wlc})"
  goto error
endif
!
!
!P2(slit):
!--------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter slit number [def.:none]: "
define/local slitno/i/1/1 {p2}
if slitno .lt. 1 .or. slitno .gt. MAXSLITNO then
        write/out "      "
        mess/out 1 {PROID} "Entered slit number out of range !"
        mess/out 1 {PROID} "({slitno}; 1<=slit#<={MAXSLITNO}!)"
  goto error
endif
!
!
!P3(linpos):
!----------
define/parameter p3 fors_linpos.tbl ? -
  "        Enter table with identified lines [def.:fors_linpos.tbl]: "
define/local linposori/c/1/64 "{p3}"
if m$index(linposori,".tbl") .eq. 0 linposori = linposori//".tbl"
if m$exist(linposori) .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered tbl. w. ident. lines is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({linposori})"
  goto error
endif
!
if m$existc(linposori,":Slit") .eq. -1 then
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({linposori},:Slit)"
  goto error
elseif m$existc(linposori,":Y") .eq. -1 then
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({linposori},:Y)"
  goto error
elseif m$existc(linposori,":WAVE") .eq. -1 then
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({linposori},:WAVE)"
  goto error
endif
select/table {linposori} :Slit.eq.{slitno} >Null
if outputi .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Slit <{slitno}> not active !"
  goto error
endif
set/format I2
define/local linpos/c/1/64 "fors_linpos_{slitno}.tbl"
copy/table {linposori} {linpos}
set/format {FMTDEF}
!
!
!P4(mos):
!-------
if p4 .eq. "?" mess/out 2 {PROID} " "
define/parameter p4 ? ? -
  "        Enter table with slit offset [def.:none]: "
define/local mos/c/1/64 "{p4}"
if m$index(mos,".tbl") .eq. 0 mos = mos//".tbl"
if m$exist(mos) .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered tbl. w. slit offset is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({mos})"
  goto error
endif
if m$existc(mos,":SLIT") .eq. -1 then
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({mos},:SLIT)"
  goto error
elseif m$existc(mos,":XOFFSET") .eq. -1 then
        write/out "      "
        mess/out 1 {PROID} "Needed column is not present !"
        mess/out 1 {PROID} "({mos},:XOFFSET)"
  goto error
endif
!
!
!P5(linfit):
!----------
define/parameter p5 fors_coerbr.tbl ? -
  "        Enter table with dispersion solution [def.:fors_coerbr.tbl]: "
define/local linfit/c/1/64 "{p5}"
if m$index(linfit,".tbl") .eq. 0 linfit = linfit//".tbl"
if m$exist(linfit) .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered tbl. w. dispers. solution is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({linfit})"
  goto error
endif
if m$existd(linfit,"NORMCOORD") .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered desc. w. fit degrees is not present"
        mess/out 1 {PROID} "in tbl. w. dispers. solution !"
        mess/out 1 {PROID} "({linfit},NORMCOORD)"
  goto error
endif
if m$existd(linfit,"CENTERXY") .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered desc. w. norm. constants is not present"
        mess/out 1 {PROID} "in tbl. w. dispers. solution !"
        mess/out 1 {PROID} "({linfit},CENTERXY)"
  goto error
endif
if m$existd(linfit,"COEFSLIT{slitno}") .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered desc. w. dispers. solution is not present"
        mess/out 1 {PROID} "in tbl. w. dispers. solution !"
        mess/out 1 {PROID} "({linfit},COEFSLIT{slitno})"
  goto error
endif
!
!
!P6(graph):
!---------
define/parameter p6 1,w ? -
  "        Graph.wnd.[<n>|def.:1|{MINGRA}-{MAXGRA}],bck.col.[def.<w>|<b>]: "
outputi = m$parse(p6,"graph")
if m$tstno(graph01) .eq. 1 .and. outputi .ne. 2 then
        write/out "      "
        mess/out 1 {PROID} "Wrong number of elements in P6(graph) !"
  goto error
endif
define/local graphcrea/c/1/1 " "
define/local graphwin/i/1/1 0
if m$tstno(graph01) .eq. 1 then
  graphcrea = "y"
  graphwin = {graph01}
  if graphwin .lt. MINGRA .or. graphwin .gt. MAXGRA then
        write/out "      "
        mess/out 1 {PROID} "Number of graphic window out of range !"
        mess/out 1 {PROID} "({graphwin} ; {MINGRA}<=window#<={MAXGRA})"
    goto error
  endif
else
  graphcrea = "n"
endif
if m$tstno(graph01) .eq. 1 define/local backcol/c/1/1 "{graph02}"
!
!
!          READING OF DESCRIPTORS :
!          **********************
!
define/local xdeg/i/1/1 0
                  xdeg = {{linfit},NORMCOORD(2)}-1
if xdeg .lt. 1 then
        write/out "      "
        mess/out 1 {PROID} "x-order <1 ! ({linfit},NORMCOORD(2)-1)"
  goto error
endif
define/local ydeg/i/1/1  {{linfit},NORMCOORD(4)}
if ydeg .lt. 1 then
        write/out "      "
        mess/out 1 {PROID} "y-order <1 ! ({linfit},NORMCOORD(4))"
  goto error
endif
define/local xydeg/i/1/1 {{linfit},NORMCOORD(3)}
if xydeg .lt. 1 then
        write/out "      "
        mess/out 1 {PROID} "mixed order <1 ! ({linfit},NORMCOORD(3))"
  goto error
endif
!
define/local xref/r/1/1  {{linfit},CENTERXY(1)}
define/local yref/r/1/1  {{linfit},CENTERXY(2)}
define/local norm/r/1/1  {{linfit},CENTERXY(3)}
!
define/local ncoef/i/1/1 0
                  ncoef = 1+xdeg+ydeg+xydeg
define/local coef/r/1/{ncoef} 0. all
                  copy/dk {linfit} COEFSLIT{slitno} coef
!
!
!
!          DEFINITION OF FURTHER VARIABLES :
!          *******************************
!
define/local mid$selidx/i/1/1 0 ? +low
define/local centrow/r/1/1 0.
define/local usedrow/r/1/1 0.
define/local usedrown/r/1/1 0.
define/local degloop/i/1/1 0
define/local coefloop/i/1/1 0
define/local const/r/1/1 0.
define/local slope/r/1/1 0.
define/local lam_central/r/1/1 0.
  define/local xws/r/1/1 0.
  define/local xwe/r/1/1 0.
  define/local xcs/r/1/1 0.
  define/local xce/r/1/1 0.
  define/local yws/r/1/1 0.
  define/local ywe/r/1/1 0.
  define/local ycs/r/1/1 0.
  define/local yce/r/1/1 0.
  define/local xw0/r/1/1 0.
  define/local xw1/r/1/1 0.
  define/local yw0/r/1/1 0.
  define/local yw1/r/1/1 0.
  define/local xwl1/r/1/1 0.
  define/local xwl2/r/1/1 0.
  define/local ywl/r/1/1 0.
!
!
!
!          CREATION OR ASSIGNMENT OF GRAPHIC WINDOW :
!          ****************************************
!
if graphcrea .eq. "y" then
  !
        mess/out 4 {PROID} "  preparing plot"
  !
  inputi = graphwin+11  ! winopen(11+i) reflects gra.wnd. 0+i
  if winopen({inputi}) .eq. 0 then
    inputi(7) = ididev(7)  ! create/graphic deletes TrueColour info
    create/graphic {graphwin} {GRACOO} 450000 >Null
    ididev(7) = inputi(7)
  elseif winopen({inputi}) .eq. 1 then
    assign/graphic g,{graphwin} >Null
  elseif winopen({inputi}) .eq. 2 then
    assign/graphic g,{graphwin} >Null
    modify/graphic window
  endif
  set/graphic default
  set/graphic pmode=0
  !
endif
!
!
!          0a. X-PIXEL :
!          ***********
!
        mess/out 4 {PROID} "  creation of "artificial" x-pixel values"
!
create/table {DELTAB} 5 {{wlc},NPIX(1)} null
create/column {DELTAB} :pix       "pixel" F9.3  R*4
create/column {DELTAB} :pixn      "norm.pixel" F10.7 R*4 >Null
create/column {DELTAB} :lam_orig  "angs." E13.7 R*4
create/column {DELTAB} :lam_lin   "angs." E13.7 R*4
create/column {DELTAB} :lam_delta "angs.,orig-lin" E13.7 R*4
!
! Normalizing:
! -----------
compute/table {DELTAB} :pix = seq
compute/table {DELTAB} :pixn = (:pix-{xref})/{norm}
!
!
!          0b. Y-PIXEL :
!              CALCULATION OF USED ROW NEAR TO
!              MIDDLE SCAN ROW (OF SELECTED SLIT) :
!          **************************************
!
        mess/out 4 {PROID} "  calculation of used y-pixel value"
!
select/table {mos} :SLIT.eq.{slitno} >Null
inputr(1) = {{mos},:YSTART,@{mid$selidx}}
inputr(2) = {{mos},:YEND,@{mid$selidx}}
centrow = m$nint((inputr(1)+inputr(2))/2.-0.5)
!
create/column {linpos} :Yref "pixel" F10.2 R*4
compute/table {linpos} :Yref = abs(:Y-{centrow})
select/table {mos} all
statistics/table {linpos} :Yref >Null
select/table {linpos} abs(:Yref-{outputr(1)}).lt.{EPS} >Null
copy/table {linpos} mid_lintmp.tbl
select/table {linpos} all
usedrow = {mid_lintmp.tbl,:Y,@1}
!
        inputi = usedrow
        mess/out 3 {PROID} -
          "  using only those lines identified in row #{inputi}"
!
! Normalizing:
! -----------
usedrown = (usedrow-yref)/norm
!
!
!          1. ORIGINAL FIT :
!          ***************
!
        mess/out 4 {PROID} "  application of original dispers. coef."
!
  compute/table {DELTAB} -
    :lam_orig = {coef(1)}
do degloop = 1 xdeg
  coefloop = 1+degloop
  compute/table {DELTAB} -
    :lam_orig = :lam_orig +{coef({coefloop})}*(:pixn**{degloop})
enddo
do degloop = 1 ydeg
  coefloop = 1+xdeg+degloop
  compute/table {DELTAB} -
    :lam_orig = :lam_orig +{coef({coefloop})}*(({usedrown})**{degloop})
enddo
do degloop = 1 xydeg
  coefloop = 1+xdeg+ydeg+degloop
  compute/table {DELTAB} -
    :lam_orig = :lam_orig +{coef({coefloop})}*(:pixn*({usedrown})**{degloop})
enddo
!
  ! Selection of actually existing wavelength range on CCD:
  ! ------------------------------------------------------
  define/range {wlc} {mos} {linfit} n
  if {q1} .eq. -1 then
    warn = 1
  elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Selection of wavelength range failed !"
    goto error
  endif
  select/table {DELTAB} :lam_orig.ge.{outputr(1)} -
                   .and.:lam_orig.le.{outputr(2)} >Null
  copy/table {DELTAB} mid_deltatmp.tbl
  -rename mid_deltatmp.tbl {DELTAB}
!
!
!          2. LINEAR Y-FIT :
!          ***************
!
        mess/out 4 {PROID} "  linear regres. with the originally ident. lines"
!
select/table mid_lintmp.tbl :WAVE.ne.NULL >Null  ! no failed fit
copy/table mid_lintmp.tbl {SELTAB}
-delete mid_lintmp.tbl
!
create/column {SELTAB} :XN "norm.pixel" F10.7 R*4 >Null
compute/table {SELTAB} :XN = (:X-{xref})/{norm}
regress/linear {SELTAB} :WAVE :XN >Null
const = outputd(1)
slope = outputd(2)
compute/table {DELTAB} :lam_lin = {const} +{slope}*:pixn
create/column {SELTAB} :LIN "angs." E13.7 R*4
compute/table {SELTAB} :LIN = {const} +{slope}*:XN
!
!
!          3A. SUBTRACTION OF CONSTANT AND LINEAR TERM (2.)
!              FROM ORIGINAL FIT (1.) :
!          ************************************************
!
        mess/out 4 {PROID} "  subtr. of const. and lin. term from orig. fit"
!
compute/table {DELTAB} :lam_delta = :lam_orig -:lam_lin
!
!
!          4A. SUBTRACTION OF CONSTANT AND LINEAR TERM (2.)
!              FROM ORIGINALLY IDENTIFIED LINES (INPUT) :
!          ************************************************
!
        mess/out 4 {PROID} "  subtr. of const. and lin. term from orig. lines"
!
create/column {SELTAB} :DELTA "angs.,WAVE-LIN" E13.7 R*4
compute/table {SELTAB} :DELTA = :WAVE -:LIN
!
!
!          3B. OPTIONAL PLOTTING OF "3A"
!              AND CALCULATION OF CENTRAL WAVELENGTH :
!          *****************************************
!
if graphcrea .eq. "y" then
        mess/out 4 {PROID} "    plotting high order terms of fit ("delta")"
  set/graphic stype=0 ltype=1
  statistics/table {SELTAB} :DELTA >Null
  inputr(1) = outputr(1) -(outputr(2)-outputr(1))*0.2
  inputr(2) = outputr(2) +(outputr(2)-outputr(1))*0.2
  set/graphic yaxis={inputr(1)},{inputr(2)}
  if backcol .eq. "w" then
    set/graphic bcolour=8 colour=8
  else
    set/graphic bcolour=1 colour=1
  endif
  plot/table {DELTAB} :lam_orig :lam_delta
  if backcol .eq. "w" then
    set/graphic bcolour=8 colour=1
  else
    set/graphic bcolour=1 colour=3
  endif
  plot/axes {plrgrap(1)},{plrgrap(2)} {plrgrap(5)},{plrgrap(6)} ? -
    "Lambda [A]" "Non-linearity [A]"
  overplot/table {DELTAB} :lam_orig :lam_delta
    xws = plrgrap(1)
    xwe = plrgrap(2)
    xcs = plrgrap(21)
    xce = plrgrap(22)
    yws = plrgrap(5)
    ywe = plrgrap(6)
    ycs = plrgrap(23)
    yce = plrgrap(24)
    xw0 = xws - xcs/(xce-xcs)*(xwe-xws)
   !xw1 = xwe + (1-xce)/(xce-xcs)*(xwe-xws)
   !yw0 = yws - ycs/(yce-ycs)*(ywe-yws)
    yw1 = ywe + (1-yce)/(yce-ycs)*(ywe-yws)
    xwl1 = xws
    xwl2 = xwe
    ywl = ywe + 0.4*(yw1-ywe)
  if m$existd(wlc,"ARCFILE") .eq. 1 then
    label/graphic "{{wlc},ARCFILE}" {xwl1},{ywl} ? 0.8 1
  else
        mess/out 2 {PROID} "Descriptor "ARCFILE" is not present"
        mess/out 2 {PROID} "in "{wlc}" - can not label plot !"
    warn = 1
  endif
  set/format I2
  inputi = usedrow
  label/graphic "Slit #{slitno} / Row #{inputi}" {xwl2},{ywl} ? 0.9 2
  set/format {FMTDEF}
endif
!
select/table {mos} :SLIT.eq.{slitno} >Null
if outputi .eq. 0 then
        write/out "      "
        mess/out 1 {PROID} "Entered slit not active ({slitno}) !"
  goto error
elseif outputi .gt. 1 then
        write/out "      "
        mess/out 1 {PROID} "Tbl. w. dispers. solution not properly selected -"
        mess/out 1 {PROID} "must be done with <define/position> before !"
        mess/out 1 {PROID} "({mos})"
  goto error
endif
inputr = m$nint({xref} +{{mos},:XOFFSET,@{mid$selidx}})
select/table {DELTAB} :pix.eq.{inputr} >Null
lam_central = {{DELTAB},:lam_orig,@{mid$selidx}}
select/table {DELTAB} all
select/table {mos} all
!
if graphcrea .eq. "y" then
  overplot/linear 2 {lam_central},{plrgrap(5)} {lam_central},{plrgrap(6)}
  overplot/linear 2 {plrgrap(1)},0. {plrgrap(2)},0.
endif
!
!
!          4B. OPTIONAL OVERPLOTTING OF "4A" :
!          *********************************
!
if graphcrea .eq. "y" then
        mess/out 4 {PROID} "    overplotting high order terms of lines"
  set/graphic stype=5 ssize=0.72 ltype=0 lwidth=3 colour=2
  overplot/table {SELTAB} :WAVE :DELTA
endif
!
!
!          5. CALCULATION OF RESIDUALS: (4.) MINUS (3.)
!             (TECHNICALLY INSTEAD OF (3.) NEW APPLICATION OF ORIGINAL FIT
!             TO (4.) BECAUSE OF DIFFERENT NUMBER OF ROWS IN "{DELTAB}"
!             AND "{SELTAB}") :
!          ***************************************************************
!
        mess/out 4 {PROID} -
          "  calc. of residuals \\\\"lines(hi.ord) - fit(hi.ord)\\\\" "
!
create/column {SELTAB} :FIT "angs." E13.7 R*4
  compute/table {SELTAB} -
    :FIT = {coef(1)}
do degloop = 1 xdeg
  coefloop = 1+degloop
  compute/table {SELTAB} -
    :FIT = :FIT +{coef({coefloop})}*(:XN**{degloop})
enddo
do degloop = 1 ydeg
  coefloop = 1+xdeg+degloop
  compute/table {SELTAB} -
    :FIT = :FIT +{coef({coefloop})}*(({usedrown})**{degloop})
enddo
do degloop = 1 xydeg
  coefloop = 1+xdeg+ydeg+degloop
  compute/table {SELTAB} -
    :FIT = :FIT +{coef({coefloop})}*(:XN*({usedrown})**{degloop})
enddo
!
create/column {SELTAB} :RESID "angs.,FIT-DELTA" E13.7 R*4
compute/table {SELTAB} :RESID = :DELTA -(:FIT -({const}+{slope}*:XN))
!
if graphcrea .eq. "y" then
        mess/out 4 {PROID} "    overplotting residual bars"
  overplot/error {SELTAB} :WAVE :DELTA :RESID 6 n
endif
!
!
!          OUTPUT :
!          ******
!
copy/tt {SELTAB} :WAVE {DELTAB}
copy/tt {SELTAB} :DELTA {DELTAB}
copy/tt {SELTAB} :RESID {DELTAB}
!
write/keyword outputr 0. all
statistics/table {SELTAB} :RESID >Null
set/format T7.2,
mess/out 3 {PROID} " "
mess/out 3 {PROID} "      min     mean +-  sigma      max"
mess/out 3 {PROID} "  -----------------------------------"
mess/out 3 {PROID} "  {outputr(1)}  {outputr(3)} +-{outputr(4)}  {outputr(2)}"
mess/out 3 {PROID} "            ------     -----"
mess/out 3 {PROID} " "
!
longlog/out 1 " " >Null
longlog/out 1 "     Deviation of dispers. relation from linear solution"
longlog/out 1 "     (slit #{slitno}):"
longlog/out 1 "         min     mean +-  sigma      max" >Null
longlog/out 1 "     -----------------------------------" >Null
longlog/out 1 -
  "     {outputr(1)}  {outputr(3)} +-{outputr(4)}  {outputr(2)}" >Null
longlog/out 1 "               ------     -----" >Null
set/format {FMTDEF}
!
write/keyword outputd 0. all
outputd(1) = lam_central
!
if m$existd(wlc,"ARCFILE") .eq. 1 -
  copy/dd {wlc} ARCFILE {DELTAB} DELTA_ARCFIL
write/descriptor {DELTAB} DELTA_SLITNO/i/1/1 {slitno}
inputi = usedrow
write/descriptor {DELTAB} DELTA_USEROW/i/1/1 {inputi}
write/descriptor {DELTAB} DELTA_LAMCEN/r/1/1 {lam_central}
!
!
!          SAVING PLOT FILE :
!          ****************
!
if graphcrea .eq. "y" then
  !
  in_a = " "
  in_a(1:9) = mid$plot(1:9)
  in_a(10:10) = mid$sess(7:7)
  in_a(11:12) = mid$sess(11:12)
  in_a(13:16) = ".plt"
  if m$exist(m$lower(in_a)) .eq. 0 then
      write/out " "
      mess/out 1 {PROID} -
        "No plot file for assigned graph. window {in_a(10:10)} present !"
    goto error
  endif
  !
        mess/out 4 {PROID} "  saving colour plot to <{PLOT_FILE}>"
  !
  -copy {in_a} {PLOT_FILE}
else
  write/keyword PLOT_FILE " " all
endif
!
!
set/format
if graphcrea .eq. "y" set/graphic default
!
!
if warn .eq. 0 then
  return 0 {DELTAB} {PLOT_FILE}
else
  return -1 {DELTAB} {PLOT_FILE}
endif
!
error:
  set/format
  if m$existk("graphcrea") .eq. 1 then
    if graphcrea .eq. "y" set/graphic default
  endif
  return 1 -99 -99
!
!
!------------------------------------------------------------------------------
