! $Id: fors_viewextrac.prg,v 1.3 1999/03/22 09:33:45 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1999  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_viewextrac.prg,v $
!
!.KEYWORDS    Plot extracted spectrum
!
!.USAGE       view/extracted speclst objno gwnd color
!
!.INPUT       speclst  (P1)   List of extracted spectra
!             objno    (P2)   Object entry in spectrum list
!             gwnd     (P3)   Graphic window ID to be used.
!             range    (P4)   X axis range to be plotted
!             color    (P5)   Color for annotations
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Plot the extracted spectrum referenced by the given object
!             number.
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/10/07:  Creation
!
!------------------------------------------------------------------------------

crossref  speclst objno window range color


define/param p1 ?       c  "Enter standard star spectrum:"
define/param p2 ?       c  "Standard star object number:"  
define/param p3 1       c  "Enter graphic window ID:"
define/param p4 +       c  "Enter x-axis range:"
define/param p5 4       c  "Enter color for annotations:"
define/maxpar 5


! Procedure identification
define/local procID/c/1/80      "ViewExtracted"


! Local constants
define/local tmpspec/c/1/20  "tmp_extspec.tbl"


! Local variables
define/local n/i/1/1        0
define/local ndata/i/1/1    0
define/local gidx/i/1/1     0
define/local snum/i/1/1     0
define/local ymin/r/1/1     0.
define/local ymax/r/1/1     0.
define/local swid/r/1/1     0.
define/local xrlim/r/1/2    0.    all
define/local yrlim/r/1/2    0.    all
define/local lpos/d/1/2     0.    all
define/local gname/c/1/100  " "   all
define/local fname/c/1/100  " "   all
define/local objno/i/1/1    {p2}
define/local lcol/i/1/1     {p5}
define/local slst/c/1/100   {p1}
define/local range/c/1/100  {p4}



! Process parameters
if m$tstno(p3) .eq. 1 then
  gidx = {p3}
endif


! Extract desired object spectrum from list
ndata = m$value({slst},:{c_sondata},{objno})
extract/t3d {slst} :{c_sowave} {objno} {tmpspec} :{c_sowave} {ndata}
extract/t3d {slst} :{c_soflux} {objno} {tmpspec} :{c_soflux} {ndata}
copy/dd {slst} *,3 {tmpspec}


! Setup plot ranges
if range(1:1) .ne. "+" then
  write/key xrlim {range}
else
  statistics/table {tmpspec} :{c_sowave} >Null
  xrlim(1) = outputr(1)
  xrlim(2) = outputr(2)

  xrlim(1) = m$nint(xrlim(1))
  xrlim(2) = m$nint(xrlim(2))
endif

statistics/table {tmpspec} :{c_soflux} >Null
yrlim(1) = outputr(1)
yrlim(2) = outputr(2)


! Check availability of graphic window
if nice_gids({gidx}) .ne. 1 then
  mess/out 2 {procID} "Requested graphic window is not active!"
  return 1
endif


! Get number of slitlet where standard star was found
snum = m$value({slst},:{c_woslit},{objno})


! Get instrument setup information
gname = m$value({tmpspec},{h_grism1})
swid = m$value({tmpspec},{h_slwidth})

if fors_nfilt .gt. 0 then
  n = fors_nfilt - 1
  if n .gt. 0 then
    do i = 1 n
      fname = fname // fors_filti({i}) // ","
    enddo
  endif
  fname = fname // fors_filti({fors_nfilt})
else
  fname = "None"
endif


! Setup plotting device
set/format i1
assign/graph g,{gidx} >Null
set/graph xaxis={xrlim(1)},{xrlim(2)} yaxis={yrlim(1)},{yrlim(2)} -
          font=0 pmode=0
clear/graph
set/format


! Plot extracted spectrum
set/grap color=1 ltype=1 stype=0
plot/spectrum {tmpspec} :{c_sowave} :{c_soflux}

set/graph color=4

set/format i1 f1.2,e18.8
lpos(1) = xrlim(1) + 0.025 * (xrlim(2) - xrlim(1))

lpos(2) = yrlim(1) + 0.925 * (yrlim(2) - yrlim(1))
label/graph "Grism: {gname}" {lpos(1)},{lpos(2)} 0. 1 1

lpos(2) = lpos(2) - 0.055 * (yrlim(2) - yrlim(1))
label/graph "Slit width: {swid} arcsec" {lpos(1)},{lpos(2)} 0. 1 1

lpos(2) = lpos(2) - 0.055 * (yrlim(2) - yrlim(1))
label/graph "Filter: {fname}" {lpos(1)},{lpos(2)} 0. 1 1


! Title string
lpos(1) = xrlim(1) + 0.025 * (xrlim(2) - xrlim(1))
lpos(2) = yrlim(1) + 1.025 * (yrlim(2) - yrlim(1))

set/grap tsize=0.75
label/graph "Extracted object spectrum from slit #{snum}" -
            {lpos(1)},{lpos(2)} 0. 1 1
set/format


! Cleanup
set/graph
-delete {tmpspec}

return 0

