! $Id: fors_viewfrm.prg,v 1.8 1999/03/22 12:06:28 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_viewfrm.prg,v $
!
!.KEYWORDS    Display image
!
!.USAGE       view/frame  frame dwnd cuts lut method title tpos tcolor
!
!.INPUT       frame    (P1)   Input frame to be displayed
!             dwnd     (P2)   Display ID to be used
!             cuts     (P3)   cut levels
!             lut      (P4)   name of color lookup table,intensity transfer
!                             table
!             method   (P5)   scaling method (min,max,avr)
!             title    (P6)   frame title
!             tpos     (P7)   title position (default: 10,5)
!             tcolor   (P8)   title color
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Display an image and do automatic scaling. Cuts may be given
!             in any syntax understood by LOAD/IMAGE. Only display channel 0
!             is used.
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/10/01:  Creation
!
!------------------------------------------------------------------------------

crossref frame dwnd cuts lut method title tpos tcolor


define/param p1 ?     c  "Enter name of frame:"
define/param p2 +     c  "Enter display window ID:"
define/param p3 +     c  "Enter cut levels:"
define/param p4 +     c  "Enter lut,itt name:"
define/param p5 +     c  "Enter scaling method:"
define/param p6 +     c  "Enter title:"
define/param p7 10,5  c  "Enter title position:"
define/param p8 +     c  "Enter title color:"
define/maxpar 8


! Procedure identification
define/local procID/c/1/80      "ViewFrame"


! Local variables
define/local pos/i/1/1      0
define/local didx/i/1/1     1
define/local iscl/i/1/2     1     all
define/local tfont/i/1/1    0
define/local tpos/i/1/2     0     all
define/local rscl/r/1/2     1     all
define/local ssize/r/1/2    0.    all
define/local fsize/r/1/2    0.    all
define/local scl/c/1/100    " "   all
define/local cut/c/1/100    " "   all
define/local method/c/1/100 " "   all
define/local tcol/c/1/10    " "   all
define/local frm/c/1/100    {p1}
define/local lut/c/1/100    {p4}
define/local title/c/1/100  {p6}



if m$tstno(p2) .eq. 1 then
  didx = {p2}
endif

cut = m$upper(p3)
method = m$upper(p5)
tcol = m$upper(p8)

write/key tpos/i/1/2 {p7}


! Check frame dimensions
if {{frm},naxis} .gt. 2 then
  mess/out 2 {procID} "Maximum number of frame dimensions is 2!"
  return 0
endif

if nice_dids({didx}) .ne. 1 then
  mess/out 2 {procID} "Requested display ID is not active!"
  return 0
endif


! Get screen size along x- and y-axis
ssize(1) = ididev(2)
ssize(2) = ididev(3)


! Get frame size
fsize(1) = m$value({frm},npix(1))
fsize(2) = m$value({frm},npix(2))


! Compute scale factor
rscl(1) = ssize(1)/fsize(1)
rscl(2) = ssize(2)/fsize(2)

if rscl(1) .lt. 1.0 then
  rscl(1) = -1./rscl(1)
endif

if rscl(2) .lt. 1.0 then
  rscl(2) = -1./rscl(2)
endif

iscl(1) = m$nint(rscl(1))
iscl(2) = m$nint(rscl(2))


! Reset things
assign/display d,{didx} >Null

clear/channel 0
clear/channel o

display/channel 0

if lut(1:1) .ne. "+" then
  pos = m$index(lut,",") - 1
  if pos .lt. 0 then
    load/lut {lut}
  else if pos .eq. 0 then
    load/itt {lut}
  else
    load/lut {lut(1:{pos})}
    pos = pos + 2
    load/itt {lut({pos}:)}
  endif
endif

scl = "{iscl(1)},{iscl(2)}"
if method(1:1) .ne. "+" then
  scl = scl // ",{method}"
endif

if cut(1:1) .ne. "+" then
  load/image {frm} scale={scl} cuts={cut} center=c,c >Null
else
  load/image {frm} scale={scl} center=c,c >Null
endif

if title(1:1) .ne. "+" then
  config/label
  if {q1} .ne. 0 then
    mess/out 2 {procID} -
             "Cannot update label font and color. Using defaults!"
    tfont = outputi(1)
    tcol = outputc
  else
    tfont = outputi(1)
    if tcol(1:1) .eq. "+" then
      tcol = outputc
    endif
  endif  

  label/display "{title}" {tpos(1)},{tpos(2)} O {tcol} {tfont}
endif

return 0
