! $Id: fors_viewlid.prg,v 1.5 2000/08/18 14:47:18 nrainer Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_viewlid.prg,v $
!
!.KEYWORDS    Show identified line positions
!
!.USAGE       view/ident  wlc linpos type [dwnd] [cuts] [options]
!                         [scolor] [info]
!
!.INPUT       wlc      (P1)   Calibration frame
!             linpos   (P2)   Input table with line positions and
!                             identifications (linpos) AND slit pos.
!                             table, separated by a comma.
!             type     (P3)   Type of identifications (initial, final)
!             dwnd     (P4)   Display window ID to be used.
!             cuts     (P5)   Cut values
!             options  (P6)   Display options: Name of color lookup table,
!                             scaling method (min,max,avr).
!             scolor   (P7)   Symbol color.
!             info     (P8)   Flag to show instrument setup
!                             information.
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Display calibration frame and mark identified emission lines.
!             Cut values may be given in any form understood by LOAD/IMAGE.
!             Addionally cuts of the form 'seapar,factor' may be used. In the
!             latter case the lower cut value is set to the minimum pixel
!             value and the upper cut value is factor*seapar(1), where
!             seapar(1) is the threshold value for line detection during the
!             emission line search that is stored in the MOS keyword seapar.
!
!.ENVIRON     FORS context, MOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/10/07:  Creation
!             1.5  2000/08/18:  Marking of slit positions added (NR).
!
!------------------------------------------------------------------------------

crossref wlc linpos type dwnd cuts options scolor info


define/param p1 ?         c  "Enter name of calibration frame:"
define/param p2 ?         c  "Line positions table, slit pos. table:"
define/param p3 F         c  "Enter identification type:"
define/param p4 0         c  "Enter display window ID:"
define/param p5 +         c  "Enter cut values:"
define/param p6 real,avr  c  "Enter display options:"
define/param p7 4         c  "Enter symbol color:"
define/param p8 y         c  "Show instrument setup?"
define/maxpar 8


! Procedure identification
define/local procID/c/1/80      "ViewIdent"


! Local constants
define/local symb/i/1/1      0
define/local rdir/c/1/1      "X"
define/local tmpfile/c/1/20  "tmp_lines.tbl"


! Local variables
define/local didx/i/1/1     0
define/local pos/i/1/1      0
define/local ssize/i/1/1    0
define/local lypos/i/1/1    0
define/local xstart/d/1/1   0.
define/local xstep/d/1/1    0.
define/local cutval/d/1/2   0.    all
define/local type/c/1/1     " "
define/local cut/c/1/100    " "   all
define/local lut/c/1/100    " "   all
define/local method/c/1/100 " "   all
define/local frm/c/1/100    {p1}
outputi = m$parse(p2,"tbl")
if outputi(1) .ne. 2 then
  write/out " "
  mess/out 1 {procID} "Wrong number of elements in P2 !"
  return 1
endif
define/local ltab/c/1/100   "{tbl01}"
define/local stab/c/1/100   "{tbl02}"
define/local scolor/c/1/100 {p7}
define/local info/c/1/1     " "



! Process parameters
type = m$upper(p3(1:1))

if m$tstno(p4) .eq. 1 then
  didx = {p4}
endif

cut = m$upper(p5)
if m$index(cut,"SEAPAR") .eq. 1 then
  if m$existk("seapar") .eq. 0 then
    mess/out 2 {procID} "Keyword SEAPAR does not exist! Using defaults."
    cut = "+"
  else
    cutval(1) = m$value({frm},lhcuts(3))
    pos = m$index(seapar,",") - 1
    cutval(2) = {seapar(1:{pos})}
    pos = m$index(cut,",") + 1
    cutval(2) = cutval(2) * {cut({pos}:)}
    cut = "{cutval(1)},{cutval(2)}"
  endif
else if m$index(cut,"MEAN") .eq. 1 then
  pos = m$index(cut,",") + 1
  statistics/image {frm} option=RN
  cutval(1) = outputr(1)
  cutval(2) = outputr(3) * {cut({pos}:)}
  if cutval(1) .lt. 0 then
    cutval(1) = 0.
  endif
  cut = "{cutval(1)},{cutval(2)}"
endif

pos = m$index(p6,",") - 1
if pos .lt. 0 then
  lut = p6
  method = "avr"
elseif pos .eq. 0 then
  lut = "real"
  method = p6(1:)
else
  lut = p6(1:{pos})
  pos = pos + 2
  method = p6({pos}:)
endif

info = m$upper(p8(1:1))


! Check availability of graphic window
if nice_dids({didx}) .ne. 1 then
  mess/out 2 {procID} "Requested display window is not active!"
  return 0
endif


! Get display y-size
lypos = ididev(3)


! Set symbol size according to binning factor
ssize = m$value({frm},{h_xwinbin}) + 1


! Display calibration frame and mark detected emission lines in each
! scan row
display/lut off

view/frame {frm} {didx} {cut} {lut} {method}


! Mark slit positions - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
define/local labxr/i/1/1 0
define/local slitloop/i/1/1 0
  show/table {stab} >Null
define/local totslitno/i/1/1 {outputi(2)}
define/local ystart/d/1/1 0.
define/local ystep/d/1/1 0.
define/local slit/i/1/1 0
  sort/table {stab} :{c_slsno}(-)
  outputi = {{stab},:{c_slsno},1}
  sort/table {stab} :{c_slsno}
define/local ylow/r/1/{outputi} 0. all
define/local ystartpix/r/1/{outputi} 0. all
define/local scry0/i/1/1 0
define/local frmy/i/1/1 0
define/local frmy0/i/1/1 0
define/local dscly/i/1/1 0
define/local scaly/r/1/1 0.
define/local scry/i/1/1 0
define/local labobjxr/i/1/1 0
!
! LABEL SLIT NUMBERS:
assign/display d,{didx} >Null
labxr  = nice_dscr(1)*2-46
label/display "SLIT" 10,{labxr} ? green 1
outputi = labxr+29
label/display "^" 23,{outputi(1)} ? green 1
!
! ----------------------------------------------------------------
! scry - scry0 = nint[ (frmy - frmy0) /scaly ] ,
!   with  scry0 = nice_dscr(2)     (screen pixel in window centre)
!   and   frmy0 = DISPLAY_DATA(4)  (frame  pixel in window centre
!                                   after `load/ima ... c,c')
!   and   scaly =     DISPLAY_DATA(2), if positive
!                 -1./DISPLAY_DATA(2), if negative
! ----------------------------------------------------------------
!
ystart = {{frm},START(2)}
ystep = {{frm},STEP(2)}
set/format I1
do slitloop = 1 totslitno
  !
  slit = {{stab},:{c_slsno},{slitloop}}
  ylow({slit}) = {{stab},:{c_slystart},{slitloop}}
  ystartpix({slit}) = m$nint((ylow({slit})-ystart)/ystep)+1
  !
  scry0 = nice_dscr(2)
  frmy  = ystartpix({slit})
  frmy0 = {{frm},DISPLAY_DATA(4)}
  dscly = {{frm},DISPLAY_DATA(2)}
  if dscly .ge. 1 then  ! 1,2,3,...
    scaly = dscly
  else                  ! ...,-3,-2 ; 0 not possible, -1 doesn't exist
    scaly = -1./dscly
  endif
  scry = m$nint((frmy-frmy0)/scaly-0.5)+scry0
  !
  labobjxr = nice_dscr(1)*2-26
  if slit .lt. 10 then
    label/display "  {slit}" {scry},{labobjxr} ? green 1
  else
    label/display "{slit}" {scry},{labobjxr} ? green 1
  endif
  !
enddo
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


! Workaround for load/table interpretation of column units 'PIXEL'
-copy {ltab} {tmpfile}

image/section {frm} {rdir}
if {q1} .eq. 0 then
  if outputi(5) .eq. 0 then
    xstart = m$value({frm},start(1))
    xstep = m$value({frm},step(1))

    compute/table {tmpfile} :{c_ltxpos} = -
                            (:{c_ltxpos} - {xstart}) / {xstep} + 1
  endif
else
  -delete {tmpfile}
  return 1
endif


if type .eq. "I" then
  select/table {tmpfile} :ident .gt. 0 >Null
else
  select/table {tmpfile} :{c_ltwave} .gt. 0 >Null
endif

load/table {tmpfile} :{c_ltxpos} :{c_ltypos} ? {symb} {ssize} {scolor}

-delete {tmpfile}


! Display grism, slit, filter and lamp setup at top of screen
if info .eq. "Y" then
  if type .eq. "I" then
    annotate/spectrum {frm} {didx} "INITIAL LINE IDENTIFICATIONS"
  else
    annotate/spectrum {frm} {didx} "FINAL LINE IDENTIFICATIONS"
  endif
endif

return 0



