! $Id: fors_viewres.prg,v 1.3 1998/10/15 14:50:35 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_viewres.prg,v $
!
!.KEYWORDS    Show fit residuals of dispersion relation
!
!.USAGE       view/residuals linpos slit gwnd range
!
!.INPUT       linpos   (P1)   Input table with line positions and
!                             identifications (linpos).
!             slit     (P2)   Slit number
!             gwnd     (P3)   Graphic window ID to be used.
!             range    (P4)   Y-axis range
!             option   (P5)   Optionally plot mean and standard deviation
!                             of wavelength residuals
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Create a scatter plot of fitted line residuals.
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/10/07:  Creation
!
!------------------------------------------------------------------------------

crossref linpos slit gwnd range option


define/param p1 ?       c  "Enter name of line positions table:"
define/param p2 1       c  "Enter slit selection:"
define/param p3 +       c  "Enter graphic window ID:"
define/param p4 +       c  "Enter yaxis range:"
define/param p5 +       c  "Enter plot option:"
define/maxpar 5


! Procedure identification
define/local procID/c/1/80      "ViewResiduals"


! Local variables
define/local gidx/i/1/1     0
define/local nsel/i/1/1     0
define/local snum/i/1/1     {p2}
define/local mean/r/1/1     0.
define/local sdev/r/1/1     0.
define/local xrlim/r/1/2    0.    all
define/local yrlim/r/1/2    0.    all
define/local lpos/r/1/2     0.    all
define/local sflag/c/1/1    " "
define/local ltab/c/1/100   {p1}
define/local range/c/1/100  {p4}
define/local keysel/c/1/64  x     all



! Process parameters
if m$indexb(p1,".tbl") .le. 0 then
  ltab = "{ltab}.tbl"
endif

if m$tstno(p3) .eq. 1 then
  gidx = {p3}
endif

sflag =  m$upper(p5(1:1))


! Check availability of graphic window
if nice_gids({gidx}) .ne. 1 then
  mess/out 2 {procID} "Requested graphic window is not active!"
  return 0
endif


! Select identified lines for current slitlet and save previous selection
keysel = m$value({ltab},tseltabl)

select/table {ltab} :{c_ltslit} .eq. {snum} .and. :{c_ltreject} .gt. 0 >Null
nsel = outputi(1)


! Setup plot device
set/format i1
assign/graph g,{gidx} >Null

if range(1:1) .ne. "+" then
  write/key yrlim {range}
else
  stat/tab {ltab} :{c_ltwave} >Null
  xrlim(1) = 0.9 * outputr(1)
  xrlim(2) = 1.1 * outputr(2)

  stat/tab {ltab} :{c_ltresid} >Null
  if outputr(1) .lt. 0. then
    yrlim(1) = 1.1 * outputr(1)
  else
    yrlim(1) = 0.9 * outputr(1)
  endif

  if outputr(2) .lt. 0. then
    yrlim(2) = 1.1 * outputr(2)
  else
    yrlim(2) = 0.9 * outputr(2)
  endif
  mean = outputr(3)
  sdev = outputr(4)
endif

set/graph xaxis={xrlim(1)},{xrlim(2)} yaxis={yrlim(1)},{yrlim(2)} -
          font=0 pmode=0 ssize=1.0 stype=1 yformat=8.2f
clear/graph


! Plot fitted  and rejected lines
if nsel .gt. 0 then
  set/grap color=1
  plot/table {ltab} :{c_ltwave} :{c_ltresid}

  if yrlim(1) .lt. 0 .and. yrlim(2) .gt. 0 then
    set/graph color=2
    overplot/line 4 {xrlim(1)},0. {xrlim(2)},0.
  endif

  if sflag .eq. "Y" then
    set/graph color=3

    overplot/line 3 {xrlim(1)},{mean} {xrlim(2)},{mean}
    overplot/line 2 {xrlim(1)},{sdev} {xrlim(2)},{sdev}
    overplot/line 2 {xrlim(1)},-{sdev} {xrlim(2)},-{sdev}

    lpos(1) = xrlim(1) + 0.03 * (xrlim(2) - xrlim(1))
    lpos(2) = yrlim(1) + 0.12 * (yrlim(2) - yrlim(1))
    label/graph "\mu \ = {mean}" {lpos(1)},{lpos(2)} 0. 1 1
    lpos(2) = yrlim(1) + 0.07 * (yrlim(2) - yrlim(1))
    label/graph "\sigma \ = {sdev}" {lpos(1)},{lpos(2)} 0. 1 1
  endif

  set/graph color=4
  lpos(1) = xrlim(1) + 0.975 * (xrlim(2) - xrlim(1))
  lpos(2) = yrlim(1) + 0.925 * (yrlim(2) - yrlim(1))
  label/graph "Slit #{snum}" {lpos(1)},{lpos(2)} 0. 1 2
endif


! Cleanup
if keysel(1:1) .eq. "-" then
  select/table {ltab} all >Null
else
  select/table {ltab}  ({keysel}) >Null
endif
set/graph

return 0
