! $Id: fors_viewspe.prg,v 1.7 1999/03/22 11:54:28 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_viewspe.prg,v $
!
!.KEYWORDS    Show 2-d object spectrum
!
!.USAGE       view/spectrum  obj dwnd cuts lut method title info
!
!.INPUT       obj      (P1)   Object spectrum frame 
!             dwnd     (P2)   Display window ID to be used.
!             cuts     (P3)   Cut values
!             lut      (P4)   Name of color lookup table.
!             method   (P5)   Scaling method (min,max,avr)
!             title    (P6)   Frame title.
!             info     (P7)   Flag to show instrument setup
!                             information.
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Display a object spectrum frame and optionally some
!             instrument setup information. Cut values may be specified
!             in any format understood by LOAD/IMAGE. The title string may
!             contain spaces (must be enclosed by double quotes).
!
!.ENVIRON     FORS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/10/07:  Creation
!
!------------------------------------------------------------------------------

crossref obj dwnd cuts lut method title info


define/param p1 ?       c  "Enter name of calibration frame:"
define/param p2 0       c  "Enter display window ID:"
define/param p3 +       c  "Enter cut values:"
define/param p4 real    c  "Enter lut,itt name:"
define/param p5 avr     c  "Enter scaling method:"
define/param p6 +       c  "Enter frame title:"
define/param p7 y       c  "Show intrument setup?"
define/maxpar 7


! Procedure identification
define/local procID/c/1/80      "ViewSpectrum"


! Local constants
define/local loffset/i/1/1   20


! Local variables
define/local didx/i/1/1     0
define/local pos/i/1/1      0
define/local lypos/i/1/1    0
define/local cutval/d/1/2   0.    all
define/local cut/c/1/100    " "   all
define/local frm/c/1/100    {p1}
define/local lut/c/1/100    {p4}
define/local method/c/1/100 {p5}
define/local title/c/1/100  {p6}
define/local info/c/1/1     " "



! Process parameters
if m$tstno(p2) .eq. 1 then
  didx = {p2}
endif

cut = m$upper(p3)
if m$index(cut,"SEAPAR") .eq. 1 then
  if m$existk("seapar") .eq. 0 then
    mess/out 2 {procID} "Keyword SEAPAR does not exist! Using defaults."
    cut = "+"
  else
    cutval(1) = m$value({frm},lhcuts(3))
    pos = m$index(seapar,",") - 1
    cutval(2) = {seapar(1:{pos})}
    pos = m$index(cut,",") + 1
    cutval(2) = cutval(2) * {cut({pos}:)}
    cut = "{cutval(1)},{cutval(2)}"
  endif
else if m$index(cut,"MEAN") .eq. 1 then
  pos = m$index(cut,",") + 1
  statistics/image {frm} option=RN
  cutval(1) = outputr(1)
  cutval(2) = outputr(3) * {cut({pos}:)}
  if cutval(1) .lt. 0 then
    cutval(1) = 0.
  endif
  cut = "{cutval(1)},{cutval(2)}"
endif

info = m$upper(p7(1:1))


! Check availability of graphic window
if nice_dids({didx}) .ne. 1 then
  mess/out 2 {procID} "Requested display window is not active!"
  return 0
endif


! Get display y-size
lypos = ididev(3)


! Display spectrum frame
display/lut off

if info .ne. "Y" .and. title(1:1) .ne. "+" then
  lypos = ididev(3) - loffset
  view/frame {frm} {didx} {cut} {lut} {method} "{title}" {lypos},5
else
  view/frame {frm} {didx} {cut} {lut} {method}
endif


! Display grism, slit and filter setup at top of screen
if info .eq. "Y" then
  annotate/spectrum {frm} {didx} "{title}"
endif

return 0
