/* $Id: fors_extract.c,v 1.3 1999/02/17 15:29:12 swolf Exp $    */
/*+++ FORS_EXTRACT ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		fors_extract
 
 .KEYWORDS	Extract sub image from input image
 
 .USAGE		run PIPE_EXE:(IDENT)
 
 .INPUT		IN_A	   (char)  input frame (flats, science, std.star)
		OUT_A      (char)  name of output frame: sub image.
		IMSEC      (int)   image section (pixel coordinates)
		MESS_LEVEL (int)   message level

 .OUTPUT	OUTPUTI(1) (int)   number of pixels within sub image.
                                   -1 in case an error had occured.
 
 .RETURN	

 .PURPOSE	X
 
 .ALGORITHM	X
 
 .ENVIRON	MIDAS
    
 .LANGUAGE	C
 
 .AUTHOR	Sebastian Wolf, ESO-DMO
 
 .VERSION	1.0 1998/12/04	SW Creation.
 
 .COMMENT       Does not change datatype in contrast to EXTRACT/IMAGE !

-----------------------------------------------------------------------------*/

#include <string.h>
#include <stdio.h>
#include <tblsys.h>
#include <tblerr.h>
#include <tbldef.h>
#include <macrogen.h>
#include <atype.h>
#include <midas_def.h>
#include <math.h>
#include "matrix.h"

#define TRUE 1
#define FALSE 0
#define BELL  0x7
#define MLEN  255
#define NLEN  160


int   Mlev;                             /* message level MESS_LEVEL */
int   NMx, NMy;                         /* dimensions of input frame */
int   Ifrm;                             /* file id of image frame */
int   Ofrm;                             /* file id of mask frame */
int   Num;                              /* total number of pixels in img */
int   NOut;                             /* total number of pixels in sub.img */
int   DatformI;                         /* dataformat of input files */
int   SS, SI, SF, SD;                   /* size of short, int, float, ... */
int   Imsc[5];                          /* image section */


double         *Frmd, **MFrmd;          /* image frame, matrix represent. */
double         *Outd;                   /* output frame */
float          *Frmf, **MFrmf;          /* image frame, matrix represent. */
float          *Outf;                   /* output frame */
int            *Frmi, **MFrmi;          /* image frame, matrix represent. */
int            *Outi;                   /* output frame */
short          *Frms, **MFrms;          /* image frame, matrix represent. */
short          *Outs;                   /* output frame */
unsigned short *Frus, **MFrus;          /* image frame, matrix represent. */
unsigned short *Ouus;                   /* output frame */
char           *Frmc, **MFrmc;          /* image frame, matrix represent. */
char           *Outc;                   /* output frame */

char  Mes[MLEN];                        /* message buffer */
char  Imgname[NLEN] = "in.bdf";         /* filename of image frame */
char  Outname[NLEN] = "iut.bdf";        /* filename of output frame */

char  Prgnam[]  = "extract";
char  Version[] = "1998.12.22-SW";

char *malloc();

main()
{
  int    nbad, uni;
  char   *ptr;

  SCSPRO(Prgnam);
  
  SS = sizeof(short);
  SF = sizeof(float);
  SD = sizeof(double);

  NOut = -1;
  SCKWRI("OUTPUTI",&NOut,1,1,&uni);                  /* return status */

  extract();                                         /* extract subimage */

  SCKWRI("OUTPUTI",&NOut,1,1,&uni);                  /* return status */
  SCSEPI();
}





int extract()
{
  char   *ptr;
  int    i, j, k;
  int    len;
  int    uni, null;
  int    finfo[4];
  int    npix[2];
  double start[2], step[2];
  double ostart[2];

  SCKGETC("IN_A",1,NLEN,&len,Imgname);      /* input image */
  if (Imgname[0] == '?'){
    i=-1;
    sprintf(Mes,"PROGRAM: %s -- %s",Prgnam,Version);
    SCTPUT(Mes);
    SCKWRI("OUTPUTI",&i,1,1,&uni);          /* return status */
    SCSEPI();
    exit(0);
  }
  SCKGETC("OUT_A",1,NLEN,&len,Outname);     /* output image: sub image */
  SCKRDI("IMSEC",1,4,&len,Imsc,&uni,&null); /* image section */
  Imsc[0]--; Imsc[1]--; Imsc[2]--; Imsc[3]--; /* work with indices */
  SCKRDI("MESS_LEVEL",1,1,&len,&Mlev,&uni,&null); /* message level */


  SCFINF(Imgname,4,&finfo[0]);              /* get file information(IN_A) */
  DatformI = finfo[1];                      /* data format */

  SCFOPN(Imgname,DatformI,0,F_IMA_TYPE,&Ifrm); /* open input frame */
  SCDRDD(Ifrm,"START",1,2,&len,start,&uni,&null);
  SCDRDD(Ifrm,"STEP",1,2,&len,step,&uni,&null);
  SCDRDI(Ifrm,"NPIX",1,2,&len,npix,&uni,&null);
  NMx = npix[0];
  NMy = npix[1];
  Num = npix[0]*npix[1];

  NOut = (Imsc[2]-Imsc[0]+1)*(Imsc[3]-Imsc[1]+1);
  SCFCRE(Outname,DatformI,F_O_MODE,F_IMA_TYPE,NOut,&Ofrm);


  switch(DatformI)
    {
    case D_R8_FORMAT:
      Frmd = (double*) malloc((size_t) Num*SD);
      if (Frmd == NULL) no_memory("Frmd",Num*SD);
      ptr = (char*) Frmd;
      SCFGET(Ifrm,1,Num,&len,ptr);             /* read the whole frame */
      MFrmd = dmap2mat(Frmd,NMx,NMy,0,0);      /* map data for matrix access */

      Outd = (double*) malloc((size_t) NOut*SD);
      if (Outd == NULL) no_memory("Outd",NOut*SD);

      k = 0;
      for (j=Imsc[1];j<=Imsc[3];j++)
	for (i=Imsc[0];i<=Imsc[2];i++)
	  Outd[k++] = MFrmd[j][i];

      ptr = (char*) Outd;
      break;
    case D_R4_FORMAT:
      Frmf = (float*) malloc((size_t) Num*SF);
      if (Frmf == NULL) no_memory("Frmf",Num*SF);
      ptr = (char*) Frmf;
      SCFGET(Ifrm,1,Num,&len,ptr);             /* read the whole frame */
      MFrmf = map2mat(Frmf,NMx,NMy,0,0);       /* map data for matrix access */

      Outf = (float*) malloc((size_t) NOut*SF);
      if (Outf == NULL) no_memory("Outf",NOut*SF);

      k = 0;
      for (j=Imsc[1];j<=Imsc[3];j++)
	for (i=Imsc[0];i<=Imsc[2];i++)
	  Outf[k++] = MFrmf[j][i];

      ptr = (char*) Outf;
      break;
    case D_I4_FORMAT:
      Frmi = (int *) malloc((size_t) Num*SI);
      if (Frmi == NULL) no_memory("Frmi",Num*SI);
      ptr = (char*) Frmi;
      SCFGET(Ifrm,1,Num,&len,ptr);             /* read the whole frame */
      MFrmi = imap2mat(Frmi,NMx,NMy,0,0);      /* map data for matrix access */

      Outi = (int *) malloc((size_t) NOut*SI);
      if (Outi == NULL) no_memory("Outi",NOut*SI);

      k = 0;
      for (j=Imsc[1];j<=Imsc[3];j++)
	for (i=Imsc[0];i<=Imsc[2];i++)
	  Outi[k++] = MFrmi[j][i];

      ptr = (char*) Outi;
      break;
    case D_I2_FORMAT:
      Frms = (short *) malloc((size_t) Num*SS);
      if (Frms == NULL) no_memory("Frms",Num*SS);
      ptr = (char*) Frms;
      SCFGET(Ifrm,1,Num,&len,ptr);             /* read the whole frame */
      MFrms = smap2mat(Frms,NMx,NMy,0,0);      /* map data for matrix access */

      Outs = (short *) malloc((size_t) NOut*SS);
      if (Outs == NULL) no_memory("Outs",NOut*SS);

      k = 0;
      for (j=Imsc[1];j<=Imsc[3];j++)
	for (i=Imsc[0];i<=Imsc[2];i++)
	  Outs[k++] = MFrms[j][i];

      ptr = (char*) Outs;
      break;
    case D_UI2_FORMAT:
      Frus = (unsigned short*) malloc((size_t) Num*SS);
      if (Frus == NULL) no_memory("Frus",Num*SS);
      ptr = (char*) Frus;
      SCFGET(Ifrm,1,Num,&len,ptr);             /* read the whole frame */
      MFrus = usmap2mat(Frus,NMx,NMy,0,0);     /* map data for matrix access */

      Ouus = (unsigned short *) malloc((size_t) NOut*SS);
      if (Ouus == NULL) no_memory("Ouus",NOut*SS);

      k = 0;
      for (j=Imsc[1];j<=Imsc[3];j++)
	for (i=Imsc[0];i<=Imsc[2];i++)
	  Ouus[k++] = MFrus[j][i];

      ptr = (char*) Ouus;
      break;
    case D_I1_FORMAT:
      Frmc = (char *) malloc((size_t) Num);
      if (Frmc == NULL) no_memory("Frmc",Num);
      SCFGET(Ifrm,1,Num,&len,Frmc);            /* read the whole frame */
      MFrmc = cmap2mat(Frmc,NMx,NMy,0,0);      /* map data for matrix access */

      Outc = (char *) malloc((size_t) NOut);
      if (Outc == NULL) no_memory("Outc",NOut);

      k = 0;
      for (j=Imsc[1];j<=Imsc[3];j++)
	for (i=Imsc[0];i<=Imsc[2];i++)
	  Outc[k++] = MFrmc[j][i];

      ptr = (char*) Outc;
      break;
    default:
      warn(-2);                                /* un-supported format, exit */
    }

  SCFPUT(Ofrm,1,NOut,ptr);                     /* store in output frame */
  SCDCOP(Ifrm,Ofrm,1,ptr);                     /* copy descriptors to output */

  ostart[0] = start[0] + Imsc[0] * step[0];
  ostart[1] = start[1] + Imsc[1] * step[1];
  npix[0] = Imsc[2] - Imsc[0] + 1;
  npix[1] = Imsc[3] - Imsc[1] + 1;
  SCDWRD(Ofrm,"START",ostart,1,2,&uni);
  SCDWRI(Ofrm,"NPIX",npix,1,2,&uni);
  SCFCLO(Ofrm);
  SCFCLO(Ifrm);

}
/* === utilities =========================================================== */
int no_memory(svar,num)
int   num;
char  *svar;
{
  int nbad, uni;

  nbad = -1;
  SCKWRI("OUTPUTI",&nbad,1,1,&uni);                  /* return status */
  SCETER(-1,"*** memory-error");
  SCSEPI();
  exit(1);
}


int warn(i)
int i;
{
  if (i>0){
    strncpy(Mes,"text",MLEN);
    messout(Mlev,2,Prgnam,Mes);
  }else{
    switch(-i)
      {
      case 2: sprintf(Mes,"Dataformat %d not supported !",DatformI);
	messout(Mlev,1,Prgnam,Mes);
	SCETER(-2,"*** dataformat-error");
	SCSEPI();
	exit(1);
      }
  }
}
