/* $Id: fors_findorder_pos.c,v 1.2 2000/05/15 19:31:48 nrainer Exp $ */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

 .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)

 .IDENT      fors_findorder_pos.c

 .KEYWORDS   Order detection

 .PURPOSE    Detection of x and y-centre positions of the orders
 __________

 .USAGE      run PIPE_EXE:(IDENT) (invoked from fors_findorder_pos.prg)

 .INPUT      Global variables:
               IN_A     : input table with column derivations
               IN_B     : name for new output table
               INPUTI(1): distance of chosen columns
               INPUTI(2): minimal value for first derivative
               INPUTI(3): y-width of one order
               INPUTI(4): tolerance for this width

             Local variables:
               STRING/c/1/256: (empty, but needed)
               MID$MSEL/i/1/1: {{in_a},TBLCONTR(4)}
               MID$SELIDX/i/1/{{in_a},TBLCONTR(4)}: (empty, but needed)

 .OUTPUT     IN_B: table with all found x and y-centre positions of the orders

 .RETURN     Q1:  0: successful return
                 -1: a warning message occured
                  1: an error occured
 __________

 .ALGORITHM  Minimum distances between positive and negative derivatives in
             column direction are computed for all chosen columns.

 .COMMENT    Special programme for order detection - not to be used generally.

 .ENVIRON    MIDAS 99NOV

 .LANGUAGE   C (ANSI)

 .AUTHOR     N.Rainer  DPG/DMD/ESO

 .VERSION    1.4  99/07/07  Creation
                  99/10/21  Release

----------------------------------------------------------------------------*/


/*  PROTOTYPES  :
    **********/

#ifdef __STDC__  /* ANSI */
  static void  error(void);
#else  /* K&R */
  static void  error();
#endif


/*  INCLUDED FILES  :
    **************/

#include  <midas_def.h>
#include  <stdio.h>
#include  <stdlib.h>


/*  DEFINITIONS OF CONSTANTS AND MACROS  :
    ***********************************/

#define  PROID    "findorder_pos.c"


/*  GLOBAL VARIABLES  :
    ****************/

static int  messlevel;


/****************************************************************************/


#ifdef __STDC__  /* ANSI */
  void main(int argc, char *argv[])
#else  /* K&R */
  void main(argc, argv)
  int  argc;
  char *argv[];
#endif
{

/*  LOCAL VARIABLES  :
    ***************/

char          dertbl[65], tmps[160], dettbl[65], string[256];

float         xpix, ypix;

int           r, nval, unit, null, info, loop, warn = 0;
int           COLDIST, YDERTHR, ORDWID, WIDTOL;
int           iddertbl, nxpix, nypix;
int           *selidx, *ypos, *yneg;
int           iddettbl, xno, yno;
int           colloop, OUTNO=1, OUTYES=0, nsel[2], colno;
int           npos, pixloop, sel[3], selloop, nneg;
int           posloop, negloop, ydist[2], ydistmin, ynegmin, detrow;


SCSPRO("findorder_pos");


if (argc > 1 && *argv[1] == '-' && tolower(*(argv[1]+1)) == 'h') {
  SCTPUT("      ******************************************************************");
  SCTPUT("      Global variables:");
  SCTPUT("        IN_A     : input table with column derivations");
  SCTPUT("        IN_B     : name for new output table");
  SCTPUT("        INPUTI(1): distance of chosen columns");
  SCTPUT("        INPUTI(2): minimal value for first derivative");
  SCTPUT("        INPUTI(3): y-width of one order");
  SCTPUT("        INPUTI(4): tolerance for this width");
  SCTPUT(" ");
  SCTPUT("      Local variables:");
  SCTPUT("        STRING/c/1/256: (empty, but needed)");
  SCTPUT("        MID$MSEL/i/1/1: {{in_a},TBLCONTR(4)}");
  SCTPUT("        MID$SELIDX/i/1/{{in_a},TBLCONTR(4)}: (empty, but needed)");
  SCTPUT("      ******************************************************************");
  SCSEPI();
}


/*  INPUT  :
    *****/

r = SCKRDI("MESS_LEVEL",1,1,&nval,&messlevel,&unit,&null);

r = SCKGETC("IN_A",1,64,&nval,dertbl);
r = SCFINF(dertbl,99,&info);
if (r != 0) {
  SCTPUT(" ");
  messout(messlevel,1,PROID,"Table not found or can not be opened !");
  sprintf(&tmps[0],"(%s)",dertbl);
  messout(messlevel,1,PROID,tmps);
  error();
}

r = SCKGETC("IN_B",1,16,&nval,dettbl);
r = SCKRDI("INPUTI",1,1,&nval,&COLDIST,&unit,&null);
r = SCKRDI("INPUTI",2,1,&nval,&YDERTHR,&unit,&null);
r = SCKRDI("INPUTI",3,1,&nval,&ORDWID,&unit,&null);
r = SCKRDI("INPUTI",4,1,&nval,&WIDTOL,&unit,&null);


/*  ALLOCATION OF ARRAYS  :
    ********************/

r = TCTOPN(dertbl,F_IO_MODE,&iddertbl);
r = TCIGET(iddertbl,&nxpix,&nypix,&null,&null,&null);
r = TCTCLO(iddertbl);

selidx = (int *) malloc ((unsigned int)(2048*sizeof(int)));
if (selidx == 0) {
  SCTPUT(" ");
  messout(messlevel,1,PROID,"Allocation not possible !");
  sprintf(&tmps[0],"(selidx)");
  messout(messlevel,1,PROID,tmps);
  error();
}
ypos = (int *) malloc ((unsigned int)(2048*sizeof(int)));
if (ypos == 0) {
  SCTPUT(" ");
  messout(messlevel,1,PROID,"Allocation not possible !");
  sprintf(&tmps[0],"(ypos)");
  messout(messlevel,1,PROID,tmps);
  error();
}
yneg = (int *) malloc ((unsigned int)(2048*sizeof(int)));
if (yneg == 0) {
  SCTPUT(" ");
  messout(messlevel,1,PROID,"Allocation not possible !");
  sprintf(&tmps[0],"(yneg)");
  messout(messlevel,1,PROID,tmps);
  error();
}
for (loop=0; loop<2048; loop++) {
  selidx[loop] = 0;
  ypos[loop] = 0;
  yneg[loop] = 0;
}


/**/


/***************************
 *                         *
 *  ACTUAL COMPUTATIONS  : *
 *                         *
 ***************************/


r = TCTINI(dettbl,F_TRANS,F_O_MODE,2,1,&iddettbl);
r = TCCINI(iddettbl,D_R4_FORMAT,1,"F7.1","pixel","X",&xno);
r = TCCINI(iddettbl,D_R4_FORMAT,1,"F7.1","pixel","Y",&yno);



detrow = 0;
for (colloop=1; colloop<=nxpix; colloop++) {
  xpix = (float)(1+(colloop-1)*COLDIST);


  /* POSITIVE DERIVATIVES */
  /* -------------------- */
  sprintf(string,":LAB00%1d.gt.%1d",colloop,YDERTHR);
  r = SCKWRC("STRING",1,string,1,256,&unit);
  r = SCKWRI("LOG",&OUTNO,4,1,&unit);
  r = tbl_select();
  r = SCKWRI("LOG",&OUTYES,4,1,&unit);
  if (r != 0) {
    messout(messlevel,1,PROID,"Problems with table selection !");
    sprintf(&tmps[0],"(%s)",dertbl);
    messout(messlevel,1,PROID,tmps);
    error();
    }
  r = SCKRDI("OUTPUTI",1,1,&nval,&nsel[0],&unit,&null);
  if (nsel[0] == 0) {
    sprintf(&tmps[0],"    selected column %1.0f: no order found",xpix);
    messout(messlevel,4,PROID,tmps);
    continue;
    }
  r = SCKRDI("MID$SELIDX",1,nypix,&nval,selidx,&unit,&null);

  r = TCTOPN(dertbl,F_IO_MODE,&iddertbl);
  sprintf(string,":LAB00%1d",colloop);
  r = TCCSER(iddertbl,string,&colno);
  r = TCTCLO(iddertbl);
  npos = 0;
  for (pixloop=1; pixloop<=nsel[0]; pixloop++) {
    npos++;
    sprintf(string,":LAB00%1d.gt.%1d .and. ABS(SEQ-%1d).lt.%1d.",
      colloop,YDERTHR,selidx[pixloop-1],WIDTOL);
    r = SCKWRC("STRING",1,string,1,256,&unit);
    r = SCKWRI("LOG",&OUTNO,4,1,&unit);
    r = tbl_select();                                    /* SEPAR. OF ORDERS */
    r = SCKWRI("LOG",&OUTYES,4,1,&unit);
    if (r != 0) {
      messout(messlevel,1,PROID,"Problems with table selection !");
      sprintf(&tmps[0],"(%s)",dertbl);
      messout(messlevel,1,PROID,tmps);
      error();
    }
    r = SCKRDI("OUTPUTI",1,1,&nval,&nsel[1],&unit,&null);
    sel[2] = 0;
    for (selloop=1; selloop<=nsel[1]; selloop++) {         /* SELECT Y-PIXEL */
      r = SCKRDI("MID$SELIDX",selloop,1,&nval,&sel[0],&unit,&null);  /* WITH */
      r = TCTOPN(dertbl,F_IO_MODE,&iddertbl);              /* MAX.DERIVATIVE */
      r = TCERDI(iddertbl,sel[0],colno,&sel[1],&null);     /* PER SELECTED   */
      r = TCTCLO(iddertbl);                                /* ORDER .        */
      if (sel[1] > sel[2]) {
        ypos[npos-1] = sel[0];
        sel[2] = sel[1];
        }
    }
    pixloop = pixloop+nsel[1]-1;
  }


  /* NEGATIVE DERIVATIVES */
  /* -------------------- */
  sprintf(string,":LAB00%1d.lt.-%1d",colloop,YDERTHR);
  r = SCKWRC("STRING",1,string,1,256,&unit);
  r = SCKWRI("LOG",&OUTNO,4,1,&unit);
  r = tbl_select();
  r = SCKWRI("LOG",&OUTYES,4,1,&unit);
  if (r != 0) {
    messout(messlevel,1,PROID,"problems with table selection");
    sprintf(&tmps[0],"(%s)",dertbl);
    messout(messlevel,1,PROID,tmps);
    error();
    }
  r = SCKRDI("OUTPUTI",1,1,&nval,&nsel[0],&unit,&null);
  if (nsel[0] == 0) {
    sprintf(&tmps[0],"    selected column %1.0f: no order found",xpix);
    messout(messlevel,4,PROID,tmps);
    continue;
    }
  r = SCKRDI("MID$SELIDX",1,nypix,&nval,selidx,&unit,&null);

  r = TCTOPN(dertbl,F_IO_MODE,&iddertbl);
  sprintf(string,":LAB00%1d",colloop);
  r = TCCSER(iddertbl,string,&colno);
  r = TCTCLO(iddertbl);
  nneg = 0;
  for (pixloop=1; pixloop<=nsel[0]; pixloop++) {
    nneg++;
    sprintf(string,":LAB00%1d.lt.-%1d .and. ABS(SEQ-%1d).lt.%1d.",
      colloop,YDERTHR,selidx[pixloop-1],WIDTOL);
    r = SCKWRC("STRING",1,string,1,256,&unit);
    r = SCKWRI("LOG",&OUTNO,4,1,&unit);
    r = tbl_select();                                    /* SEPAR. OF ORDERS */
    r = SCKWRI("LOG",&OUTYES,4,1,&unit);
    if (r != 0) {
      messout(messlevel,1,PROID,"Problems with table selection !");
      sprintf(&tmps[0],"(%s)",dertbl);
      messout(messlevel,1,PROID,tmps);
      error();
    }
    r = SCKRDI("OUTPUTI",1,1,&nval,&nsel[1],&unit,&null);
    sel[2] = 0;
    for (selloop=1; selloop<=nsel[1]; selloop++) {         /* SELECT Y-PIXEL */
      r = SCKRDI("MID$SELIDX",selloop,1,&nval,&sel[0],&unit,&null);  /* WITH */
      r = TCTOPN(dertbl,F_IO_MODE,&iddertbl);              /* MIN.DERIVATIVE */
      r = TCERDI(iddertbl,sel[0],colno,&sel[1],&null);     /* PER SELECTED   */
      r = TCTCLO(iddertbl);                                /* ORDER .        */
      if (sel[1] < sel[2]) {
        yneg[nneg-1] = sel[0];
        sel[2] = sel[1];
        }
    }
    pixloop = pixloop+nsel[1]-1;
  }


  /* MIN.DIST.BETWEEN DERIVATIVES */
  /* ---------------------------- */
  for (posloop=1; posloop<=npos; posloop++) {
    for (negloop=1; negloop<=nneg; negloop++) {
      ydist[0] = abs(yneg[negloop-1]-ypos[posloop-1]-ORDWID);
      if (negloop == 1) ydist[1] = ydist[0]+1;
      if (ydist[0] < ydist[1]) {
        ydistmin = ydist[0];
        ynegmin = yneg[negloop-1];
        ydist[1] = ydist[0];
      }
    }
    if (ydistmin < WIDTOL) {                           /* MIN.DIST.<WIDTOL . */
      ypix = (float)((ynegmin+ypos[posloop-1])/2.);    /* MIDDLE OF ORDER .  */
      detrow = detrow+1;
      r = TCEWRR(iddettbl,detrow,xno,&xpix);
      r = TCEWRR(iddettbl,detrow,yno,&ypix);
    }
  }


} /* for (colloop=1; colloop<=nxpix; colloop++) */



free(selidx);
r = TCTCLO(iddettbl);


if (warn == 0)
  r = SCKWRC("Q1",1,"0",1,60,&unit);
else
  r = SCKWRC("Q1",1,"-1",1,60,&unit);
r = SCSEPI();


} /* void main(int argc, char *argv[]) */


/****************************************************************************/

#ifdef __STDC__  /* ANSI */
  static void error(void)
#else  /* K&R */
  static void error()
#endif
{
  int  r, unit;

  r = SCKWRC("Q1",1,"1",1,60,&unit);
  r = SCSEPI();
}

/******************************************************************************/
