/* $Id: fors_invert.c,v 1.3 1998/12/04 18:06:35 swolf Exp $    */
/*+++ FORS_INVERT +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		fors_invert.exe
 
 .KEYWORDS	Do matrix inversion
 
 .USAGE		run PIPE_EXE:(IDENT)
 
 .INPUT		IN_A	   (char)  1. Image
		NX         (int)   binning in x-direction
		DTYP       (int)   data type:
		                   4: double, 2: float (see INFO/KEY)
 .OUTPUT	OUTPUTI(1) (int)   number of outliers found
                OUTPUTD    (int)   (1..4): mean of each quadrant

 .RETURN	Q1:  0: successful return
 		    -1: a successful return failed
 
 .PURPOSE	Do a binning of the single input frame IN_A or of the scaled
                difference frame (IN_A - IN_B/SCALB)/SCALA
 
 .ALGORITHM	X
 
 .ENVIRON	MIDAS
    
 .LANGUAGE	C
 
 .AUTHOR	Sebastian Wolf, ESO-DMO
 
 .VERSION	1.0 1998/09/04	Creation
 
 .COMMENT       X

-----------------------------------------------------------------------------*/


#include <string.h>
#include <stdio.h>
#include <tblsys.h>
#include <tblerr.h>
#include <tbldef.h>
#include <macrogen.h>
#include <atype.h>
#include <midas_def.h>
#include <math.h>

#include "matrix.h"

#define TRUE 1
#define FALSE 0
#define BELL  0x7
#define NLEN  160
#define MLEN  255

int   SI, SF, SD;                       /* size of int, size of float */
int   Mlev;                             /* message level MESS_LEVEL */

int   Id1;                              /* file id of IMAGE frame */
int   NIx, NIy;                         /* dimensions of INPUT frame */
int   NI;                               /* total number of pixels in Ima */
int   Dtyp;                             /* data type */
int   Idx, Idy;                         /* bin-size in x and y */

double *Imd, **MImd;                     /* image frame, matrix represent. */
float *Ima, **MIma;                     /* image frame, matrix represent. */

float **map2mat();                      /* map a matrix on a float vector */


char  Mes[MLEN];                        /* message buffer */
char  Imagname[NLEN] = "image.bdf";     /* filename of image frame */

char  Prgnam[]  = "invert";
char  Version[] = "1998.12.01-SW";

char *malloc();

main()

{
  int uni;
  int i, j, k;
  float **f;
  double **d;

  SCSPRO("invert");
  
  SD = sizeof(double);
  SF = sizeof(float);
  SI = sizeof(int);

  get_params();                                      /* get MIDAS parameters */

  switch (Dtyp) {
  case 2:                                            /* double data matrix */
    d = d_matrix(1,NIx,1,1);                         /* passed by a keyword */
    i = dgaussj(MImd,NIx,d,1);
    SCKWRD("OUTPUTD",Imd,1,NI,&uni);                 /* return matrix in */
                                                     /* column order */
    break;
  case 1:                                            /* double data matrix */
    f = f_matrix(1,NIx,1,1);                         /* passed by a keyword */
    i = gaussj(MIma,NIx,f,1);
    SCKWRR("OUTPUTR",Ima,1,NI,&uni);                 /* return matrix in */
                                                     /* column order */
    break;
  case -1:                                           /* float data matrix */
    f = f_matrix(1,NIx,1,1);                         /* passed by a frame */
    i = gaussj(MIma,NIx,f,1);
    printf("OUTPUT still missing !\n");
    break;
  }

  SCSEPI();
}


/* === interface to MIDAS ================================================== */

int get_params()
{
  char   *ptr;
  int    i, j, len;
  int    uni, null;
  int    nx, ny;
  int    npix[2];
  double **dmap2mat();

  SCKGETC("IN_A",1,NLEN,&len,Imagname);           /* get name of image frame */
  if (Imagname[0] == '?'){
    i=0;
    sprintf(Mes,"PROGRAM: invert -- %s",Version);
    SCTPUT(Mes);
    SCSEPI();
    exit(0);
  }
  SCKRDI("NX",1,1,&len,&nx,&uni,&null);           /* dimension in x-dir */
  SCKRDI("DTYP",1,1,&len,&Dtyp,&uni,&null);       /* data type */
  ny = nx;                                        /* matrix has to be quadr. */

  if (nx > 0) {
    NIx = nx;
    NIy = ny;
    NI  = nx * ny;
    Dtyp = Dtyp/2;
    switch (Dtyp){
    case 2:
      Imd = (double*) malloc((size_t) NI*sizeof(double));
      SCKRDD(Imagname,1,NI,&len,Imd,&uni,&null);  /* double data array */
      MImd = dmap2mat(Imd,NIx,NIy,1,1);           /* map data for mat. acc. */
      break;
    case 1:
      Ima = (float*) malloc((size_t) NI*SF);
      SCKRDR(Imagname,1,NI,&len,Ima,&uni,&null); /* float data array */
      MIma = map2mat(Ima,NIx,NIy,1,1);            /* map data for mat. acc. */
      break;
    }
  }
  else{
    Dtyp = -1;
    SCFOPN(Imagname,D_R4_FORMAT,0,F_IMA_TYPE,&Id1);
    SCDRDI(Id1,"NPIX",1,2,&len,npix,&uni,&null);
    NIx = npix[0];                               /* x-size of complete frame */
    NIy = npix[1];                               /* y-size of complete frame */
    NI  = npix[0]*npix[1];                       /* total number of pixels */
    if (NIx != NIy){
      printf("Matrix has to be quadratic !\n");
      return(-4);
    }
    Ima = (float*) malloc((size_t) NI*SF);
    if (Ima == NULL) nomemory("Ima",NI*SF);
    ptr = (char*) Ima;
    SCFGET(Id1,1,NI,&len,ptr);                   /* read the whole frame */
    SCFCLO(Id1);
    nx = NIx;
    ny = NIy;
    MIma = map2mat(Ima,NIx,NIy,1,1);            /* map data for matrix acc. */
  }    

}


int showm(){
int i,j;
  for (j=1;j<=NIy;j++,printf("\n"))
    for (i=1;i<=NIx;i++) printf("%g ",MImd[j][i]);
  printf("\n");
}

