/* $Id: fors_message.c,v 1.9 1998/12/18 18:50:06 swolf Exp $    */
/*+++ FORS_MESSAGE ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		message()
 
 .KEYWORDS	Put a formatted string to stdout and MIDAS log
 
 .USAGE         run PIPE_EXE:(IDENT)
 
 .INPUT		ID             (int) message identifier
		               ( 1: ERROR, 2: WARNING, 3,4: INFO)
			       negative id has the same meaning except that
			       NO  \n  will be append at the end of the
			       message text!
	        PNAM           (char) name of the calling program
	        PLEN           (int) length of PNAM
		CBUF           (char) message text
	        CLEN           (int) length of CBUF

 .OUTPUT	terminal and MIDAS-log output
 
 .RETURN	-1: the message identifier is wrong
                0:  no error occured
 
 .PURPOSE	Put a formatted string to stdout. The message identifier
                controls the format of the messagetext:

		id  | format                   | BELL 
		----+--------------------------+-----
		1   | ERROR   [prgname]: text  | ON
		2   | WARNING [prgname]: text  | OFF
		3   | INFO    [prgname]: text  | OFF
		4   | INFO    [prgname]: text  | OFF
 
		In case the identifier is the negative compliment then
		NO  \n  will be appended to the text.
		Following printf() may continue at that point.

 .ALGORITHM	The string passed through messagetext will be formatted and
                send to stdout (printf()). Furthermore this messagetext will
		be logged in the standard MIDAS log.
		The format of the messagetext depends on the message
		identifier id.
 
 .ENVIRON	MIDAS
    
 .LANGUAGE	C
 
 .AUTHOR	Sebastian Wolf, ESO-DMO
 
 .VERSION	1.0 1998.01.05 created.
                    1998.04.21 uses subroutines stored now in fors_messages.c
 
 .COMMENT       The message may contain special characters to format your
                text:   \n, \r, \t, \b
-----------------------------------------------------------------------------*/

#include <stdio.h>
#include <errno.h>
#include "messages.h"

char  Version[]="1998.12.18-SW\n";

main()
{
  char   *cbuf, *pnam;
  int    len, clen, plen;
  int    num, id;
  int    null, unit;
  int    err;


  SCSPRO("message");

  SCKRDI("ID",1,1,&num,&id,&unit,&null);
  SCKRDI("PLEN",1,1,&num,&plen,&unit,&null);
  pnam = (char*) malloc( plen+1 );
  SCKGETC("PNAM",1,plen,&len,pnam);

  SCKRDI("CLEN",1,1,&num,&clen,&unit,&null);
  cbuf = (char*) malloc( clen+2 );
  SCKGETC("CBUF",1,clen,&len,cbuf);
  cbuf[strlen(cbuf)] = '\0';
  SCKRDI("INDENT",1,1,&num,&INDENT,&unit,&null);

  err=message(id,pnam,cbuf);    
  if (err < 0)
    SCKWRC("Q1",1,"-1",1,2,&unit);
  else
    SCKWRC("Q1",1,"0",1,1,&unit);

  SCSEPI();
}
