/* $Id: fors_messages.c,v 1.16 1998/12/18 18:50:07 swolf Exp $    */
/*+++ FORS_MESSAGES +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		fors_messages.c
 
 .KEYWORDS	Put a formated message on the terminal.
 
 .USAGE		supports two calls:
                (1) messout()
		(2) messages()
 
-----------------------------------------------------------------------------*/

#include <string.h>
#include <stdio.h>
#include <errno.h>
#include "messages.h"

#define MIN(A,B) ((A)<(B)?(A):(B))      /* minimum function */

int INDENT;
char *check_special();
char  Mtyp[MXID][MXLN] = {"","ERROR   ","WARNING ","INFO    ","INFO    "};





/*+++ messout +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		messout()
 
 .KEYWORDS	Put a formatted string to stdout and MIDAS-log
 
 .USAGE         messout(messlevel, id, prgname, messagetext)
 
 .INPUT		messlevel      (int) message level (all messages having an
                               abs(id) <= messlevel
			       will be displayed!
		id             (int) message identifier
		               ( 1: ERROR, 2: WARNING, 3,4: INFO)
			       negative id has the same meaning except that
			       NO  \n  will be append at the end of the
			       message text!
	        *prgname       (char) name of the calling program
		*messagetext   (char) message text

 .OUTPUT	terminal output and MIDAS-log
 
 .RETURN	-1: the message identifier is wrong
                0:  no error occured
 
 .PURPOSE	Put a formatted string to stdout
 
 .ALGORITHM	calls message()
 
 .ENVIRON	MIDAS
    
 .LANGUAGE	C
 
 .AUTHOR	Sebastian Wolf, ESO-DMO
 
 .VERSION	1.0 1998/03/03	Creation
 
 .COMMENT       The message may contain special characters to format your
                text:   \n, \r, \t, \b
-----------------------------------------------------------------------------*/

int messout(messlevel, id, pnam, mes)
int  id, messlevel;
char *pnam, *mes;
{
  if (abs(id) > messlevel) return(-1);
  message(id,pnam,mes);
}



/*+++ message +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		message()
 
 .KEYWORDS	Put a formatted string to stdout and MIDAS-log
 
 .USAGE         message(messlevel, id, prgname, messagetext)
 
 .INPUT		id             (int) message identifier
		               ( 1: ERROR, 2: WARNING, 3,4: INFO)
			       negative id has the same meaning except that
			       NO  \n  will be append at the end of the
			       message text!
	        *prgname       (char) name of the calling program
		*messagetext   (char) message text

 .OUTPUT	terminal and MIDAS-log output
 
 .RETURN	-1: the message identifier is wrong
                0:  no error occured
 
 .PURPOSE	Put a formatted string to stdout. The message identifier
                controls the format of the messagetext:

		id  | format                   | BELL 
		----+--------------------------+-----
		1   | ERROR   [prgname]: text  | ON
		2   | WARNING [prgname]: text  | OFF
		3   | INFO    [prgname]: text  | OFF
		4   | INFO    [prgname]: text  | OFF
 
		In case the identifier is the negative compliment then
		NO  \n  will be appended to the text.
		Following printf() may continue at that point.

 .ALGORITHM	The string passed through messagetext will be formatted and
                send to stdout (printf()). Furthermore this messagetext will
		be logged in the standard MIDAS log.
		The format of the messagetext depends on the message
		identifier id.
 
 .ENVIRON	MIDAS
    
 .LANGUAGE	C
 
 .AUTHOR	Sebastian Wolf, ESO-DMO
 
 .VERSION	1.0 1998/03/03	Creation
 
 .COMMENT       The message may contain special characters to format your
                text:   \n, \r, \t, \b
-----------------------------------------------------------------------------*/
int message(id, pnam, mess)
int  id;
char *pnam, *mess;
{
  char bell;
  char *mes, *txt;
  char CR='\n';
  size_t  len, plen;
  int     ibck;

  ibck = 0;
  mes = check_special(mess);
  if (id >= MXID) return(-1);
  if (id < 0){
    id = -id;
    CR = '\0';
    ibck = -1;
  }
  if (id > 1) bell = '\0';
  else        bell = BELL;
  if (id == 0) bell = '\0';

  plen = strlen(pnam);
  if (plen>MAXPLEN){
    pnam[MAXPLEN] = '\0';    /* cut program name if too long */
    plen = MAXPLEN;
  }
  len = strlen(mes) + OFFSET + 10;
  txt = (char *)malloc((size_t) len);

  sprintf(txt,"%-8s[%-19s%s%c",&Mtyp[id],pnam,mes,bell);
  txt[plen+9] = ']';
  txt[plen+10] = ':';
  printf("%s%c",txt,CR);
  MID_LOG('G',txt);

  free(txt);
  free(mes);
  return(0);
}


char *check_special(cbuf)
char *cbuf;
{
  int i, j, num, numc, ioff;
  char *ccbuf, *chb;

  num = strlen(cbuf);
  for (i=0,j=0;j<num;j++){
    if (cbuf[j] == '\n') i++;
  }

  if (i==0){
    ccbuf = (char*) malloc((unsigned) num + 1);
    ccbuf[num] = '\0';
    numc = num;
    for (i=0,j=0;j<num;j++)
      {
	if (cbuf[j]=='\\')
	  {
	    switch(cbuf[j+1])
	      {
	      case 'b':
		ccbuf[i++] = '\b';
		j++;
		break;
	      case 'n':
		ccbuf[i++] = '\n';
		numc = numc + OFFSET-4 + INDENT;
		ccbuf = (char*) realloc(ccbuf,(unsigned) numc);
		ioff = i + OFFSET-4+INDENT;
		for (;i<ioff;i++) ccbuf[i] = ' ';
		j++;
		break;
	      case 'r':
		ccbuf[i++] = '\r';
		j++;
		break;
	      case 't':
		ccbuf[i++] = '\t';
		j++;
		break;
	      default:
		ccbuf[i++] = cbuf[j];
	      }
	  }else{
	    ccbuf[i++] = cbuf[j];
	  }
      }
    if (i<=num) ccbuf[i] = '\0';
  }else{
    numc = num + i*(OFFSET-4+INDENT);
    ccbuf = (char*) malloc( numc+1 );
    for (i=0,j=0;j<num;j++){
      ccbuf[i++] = cbuf[j];
      if (cbuf[j] == '\n'){
	for (ioff=i+OFFSET-4+INDENT;i<ioff;i++) ccbuf[i] = ' ';
      }
    }
    ccbuf[numc] = '\0';
  }
  return(ccbuf);
}
