#include <string.h>
#include <stdio.h>
#include <tblsys.h>
#include <tblerr.h>
#include <tbldef.h>
#include <macrogen.h>
#include <atype.h>
#include <midas_def.h>
#include <math.h>
/* #include <sunmath.h> */
#include "nrutil.h"

#define TRUE 1
#define FALSE 0
#define BELL  0x7
#define MLEN  1024
#define NLEN  80
#define MXCOL 8
#define IMAX  300

int   Dtid;                             /* table id of data table */
int   Ptid;                             /* table id of parameter table */
int   Icol[MXCOL];                      /* columns indices */
int   Dcoff[6];                         /* columns offsets & number of data */
int   Pcoff[4];                         /* columns offsets & number of data */
int   Ndcol, Ndrow, Npcol, Nprow;       /* number of columns, rows in tables */
int   Mlev;
char  Mes[MLEN];
char  Datname[NLEN] = "fit.tbl";        /* filename of data table, */
char  Parname[NLEN] = "par.tbl";        /* filename of parameter table */
float  *Dat;
float  *Par;

char  Prgnam[]  = "nonlfit";
char  Version[] = "1998.01.29-SW";

/*
  Icol:   1       2       3       4       5       6       7      8
          :X      :Y      :SIG    :FIT    :RESID  :PARAM  :FIX   :RESULT
*/
void user01();

main()
{
  int    i, len, num, uni, null;
  int    mfit, *listp, *ivector();
  int    szf, err;
  int    *iptr;
  char   *cptr;
  float  ochisq, chisq, acc, alamda;
  float  *x, *y, *sig, *fit, *res;
  float  *param;
  float  **covar;
  float  **alpha;

  szf = sizeof(float);

  SCSPRO("nonlfit");

  sprintf(Mes,"PROGRAM: nonlfit -- %s",Version);
  SCTPUT(Mes);
  
  SCKGETC("IN_A",1,NLEN,&len,Datname);     /* data table */
  SCKGETC("IN_B",1,NLEN,&len,Parname);     /* parameter table */
  SCKRDR("INPUTR",1,1,&len,&acc,&uni,&null);
  SCKRDI("INPUTI",1,8,&len,Icol,&uni,&null);
  SCKRDI("MESS_LEVEL",1,1,&len,&Mlev,&uni,&null);

  sprintf(Mes,"INPUT:");
  sprintf(&Mes[strlen(Mes)],"\n  Data table:        %s",Datname);
  sprintf(&Mes[strlen(Mes)],"\n  Parameter table:   %s",Parname);
  sprintf(&Mes[strlen(Mes)],"\n  Table-column number used:");
  sprintf(&Mes[strlen(Mes)],"\n    Abszissa (X):    %d (data)",Icol[0]);
  sprintf(&Mes[strlen(Mes)],"\n    Measurement (Y): %d (data)",Icol[1]);
  sprintf(&Mes[strlen(Mes)],"\n    Sigmas (SIG):    %d (data)",Icol[2]);
  sprintf(&Mes[strlen(Mes)],"\n    Parameters:      %d (parameter)",Icol[5]);
  sprintf(&Mes[strlen(Mes)],"\n    FIX              %d (parameter)",Icol[6]);
  messout(Mlev,4,Prgnam,Mes);

  TCTOPN(Datname,F_IO_MODE,&Dtid);
  TCTOPN(Parname,F_IO_MODE,&Ptid);

  TCIGET(Dtid,&Ndcol,&Ndrow,&null,&null,&null);
  num = Ndcol*Ndrow - 1;
  Dat = vector(0,num);
  read_table(Dtid,Ndcol,Ndrow,Dat,Dcoff);
  /*  show_table(Dtid,Ndcol,Ndrow,Dat,Dcoff); */

  TCIGET(Ptid,&Npcol,&Nprow,&null,&null,&null);
  num = Npcol*Nprow - 1;
  Par = vector(0,num);
  read_table(Ptid,Npcol,Nprow,Par,Pcoff);
  /*  show_table(Ptid,Npcol,Nprow,Par,Pcoff); */

  covar = matrix(1,Nprow,1,Nprow);
  alpha = matrix(1,Nprow,1,Nprow);
  listp = ivector(1,Nprow);
  for (i=0;i<MXCOL;i++) Icol[i] = Icol[i] - 1;  /* as C counts from 0 */

  check_table(Ptid,Icol[6],Nprow,Par,Pcoff,&listp[1],&mfit);

  x     = &Dat[ Dcoff[Icol[0]] ] - 1;
  y     = &Dat[ Dcoff[Icol[1]] ] - 1;
  sig   = &Dat[ Dcoff[Icol[2]] ] - 1;
  fit   = &Dat[ Dcoff[Icol[3]] ] - 1;
  res   = &Dat[ Dcoff[Icol[4]] ] - 1;
  param = &Par[ Pcoff[Icol[5]] ] - 1; 
  alamda = -1.0;   /* initialization */

  show_param(param,listp,Nprow,mfit);

  err = mrqmin(x,y,sig,Ndrow,param,Nprow,listp,mfit,covar,alpha,&chisq,user01,&alamda);
  if (err < 0) { err_fini(err,-1.0); return(err); }
  i = 0;
  do
    {
      ochisq = chisq;
      sprintf(Mes,"Iteration: %d\tChi^2  = %f",i+1,chisq);
      sprintf(&Mes[strlen(Mes)],"\nParam  : %f %f %f",param[1],param[2],param[3]);
      messout(Mlev,4,Prgnam,Mes);

      err=mrqmin(x,y,sig,Ndrow,param,Nprow,listp,mfit,covar,alpha,&chisq,user01,&alamda);
      i++;
      /*      printf("%f %f %g %d\n",ochisq,chisq-ochisq,alamda,err); */
      if (err < 0) { err_fini(err,-1.0); return(err); }
    }while ((alamda > 1.E-6) && (i<IMAX)); /* && (chisq != ochisq)); */

  if (i>1){
   sprintf(Mes,"Iteration: %d\tChi^2  = %f",i,chisq);
   sprintf(&Mes[strlen(Mes)],"\nParam  : %f %f %f",param[1],param[2],param[3]);
   messout(Mlev,4,Prgnam,Mes);
  }

  /* Calculate covariance and curvature matrix. */
  sprintf(Mes,"Save fit.");
  messout(Mlev,4,Prgnam,Mes);
  alamda=0.0;
  err=mrqmin(x,y,sig,Ndrow,param,Nprow,listp,mfit,covar,alpha,&chisq,user01,&alamda);
  if (err < 0) { err_fini(err,-1.0); return(err); }
  calcul_fit(x, y, fit, res, user01, param, Ndrow, Nprow);

  copy_column(&param[1],&Par[ Pcoff[Icol[7]] ],Nprow);
  write_tablec(Ptid,Icol[7],Nprow,Par,Pcoff); /* save result parameters */
  write_tablec(Dtid,Icol[3],Ndrow,Dat,Dcoff); /* save fit */
  write_tablec(Dtid,Icol[4],Ndrow,Dat,Dcoff); /* save residuals */

  TCTCLO(Dtid);
  TCTCLO(Ptid);
  SCKWRR("OUTPUTR",&chisq,1,1,&uni);                  /* return chi^2 */
  SCKWRC("Q1",1,"0",1,1,&uni);                       /* return Q1=0, ok! */
  SCSEPI();
}


int calcul_fit(x, y, fit, res, funcs, param, num, npar)
float *x, *y, *fit, *res, *param;
int   num, npar;
void (*funcs)();
{
  int i;
  float *dy;

  dy = vector(1,npar);
  for (i=1;i<=num;i++){
    (*funcs)(x[i], param, &fit[i], dy);
    /*printf("%d %f %f %f\n",i,x[i],y[i],fit[i]); */
    res[i] = y[i] - fit[i];
  }
}


void user01(x, param, y, deriv)
float x, *y, *deriv, *param;
{
  float e;

  e = exp(-param[2]*x);
  *y = param[1] * e  +  param[3];
  /*  printf("p1=%f p2=%f p3=%f\n",param[1],param[2],param[3]);
  printf("x=%f e=%f y=%f\n",x,e,*y); */
  deriv[1] = e;
  deriv[2] = -param[1]*x * e;
  deriv[3] = 1.0;
}


int show_param(param,listp,num,mfit)
float *param;
int   *listp, num, mfit;
{
  int i;
  
  sprintf(Mes,"Initial parameter set:\n\t\t\tVALUE\t\tUSED-FLAG");
  for (i=1;i<=num;i++){
    sprintf(&Mes[strlen(Mes)],"\nparameter_%d:\t%f\t%d",i,param[i],listp[i]);
  }
    
  sprintf(&Mes[strlen(Mes)],"\n(USED-FLAG: 0 indicates a fixed parameter)");
  messout(Mlev,4,Prgnam,Mes);
}


int err_fini(err,chisq)
int   err;
float chisq;
{
  int uni;

  TCTCLO(Dtid);
  TCTCLO(Ptid);
  SCKWRR("OUTPUTR",&chisq,1,1,&uni);                  /* return chi^2 */
  SCKWRC("Q1",1,"1",1,1,&uni);                       /* return Q1=0, ok! */
  SCSEPI();
}

int nomemory(svar,num)
int   num;
char  *svar;
{
  int nbad, uni;

  nbad = 0;
  sprintf(Mes,"Cannot allocate memory for %s [%d Byte]!\n",svar,num);
  messout(Mlev,1,Prgnam,Mes);
  if (Dtid != -1) TCTCLO(Dtid);
  if (Ptid != -1) TCTCLO(Ptid);
  SCSEPI();
  exit(1);
}
