/* $Id: fors_photo_flags.c,v 1.6 2000/05/15 19:31:50 nrainer Exp $    */
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

 .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)

 .IDENT      fors_photo_flags.c

 .KEYWORDS   Special character conversion

 .PURPOSE    Conversion of characters in the FLAG column of the photometry
             output table (decimal->binary->letters).
 __________

 .USAGE      run PIPE_EXE:(IDENT) (invoked from fors_photo_flags.prg)

 .INPUT      IN_A     : name of photometry table
             IN_B     : name of flag column
             INPUTI(1): numbers of rows to process (from beginning)

 .OUTPUT     table from IN_A: column FLAGS is converted

 .RETURN     Q1:  0: successful return
                 -1: a warning message occured
                  1: an error occured
 __________

 .ALGORITHM  Decimal->Binary + Binary->Letter with reversed presentation.

 .COMMENT    Special programme for photometry - not to be used generally.

 .ENVIRON    MIDAS 98NOV

 .LANGUAGE   C (ANSI)

 .AUTHOR     N.Rainer  DPG/DMD/ESO

 .VERSION    1.0  98/11/09  Creation
                  98/12/15  Release
             1.1  99/01/14  Release
             1.2  99/03/15  Release
             1.3  99/09/23  Release

----------------------------------------------------------------------------*/


/*  PROTOTYPES  :
    **********/

#ifdef __STDC__  /* ANSI */
  static void   error(void);
  static float  nint( float x);
#else  /* K&R */
  float  x;
  static void   error();
  static float  nint();
#endif


/*  INCLUDED FILES  :
    **************/

#include <midas_def.h>
/* ONLY FOR DEBUGGING WITH `cflow': */
/* #include  "/home/dfs1rp1/dmd_df/nrainer/LAN/MIDAS/C/CFLOW/midas_def.h" */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


/*  DEFINITIONS OF CONSTANTS AND MACROS  :
    ***********************************/

#define       PROID      "photo_flags.c"
#define       FLAGS      "CMSTA-DO\0"  /* 8 characters + \0 */


/*  GLOBAL VARIABLES  :
    ****************/

static int  messlevel;


/****************************************************************************/


#ifdef __STDC__  /* ANSI */
  void main(int argc, char *argv[])
#else  /* K&R */
  void main(argc, argv)
  int  argc;
  char *argv[];
#endif
{

/*  LOCAL VARIABLES  :
    ***************/

char          filetblorig[65], namecolorig[17];
char          **out2, **outlett, LEGEND[9];
char          tmps[160], tmps1[10], tmps2[10], tmps3[10];

float         *in10, *in10_0;

register int  flagloop, i;
int           r, nval, unit, null, info;
int           warn = 0;
int           maxrow, idtblorig, colno, COLS1=9, COLS2=10, elem, reverse, sign;


SCSPRO("photo_flags");


if (argc > 1 && *argv[1] == '-' && tolower(*(argv[1]+1)) == 'h') {
  SCTPUT("      ******************************************************************");
  SCTPUT("      IN_A     : name of photometry table");
  SCTPUT(" ");
  SCTPUT("      IN_B     : name of flag column");
  SCTPUT(" ");
  SCTPUT("      INPUTI(1): numbers of rows to process (from beginning)");
  SCTPUT("      ******************************************************************");
  SCSEPI();
}


/*  INPUT  :
    *****/

r = SCKRDI("MESS_LEVEL",1,1,&nval,&messlevel,&unit,&null);

r = SCKGETC("IN_A",1,64,&nval,filetblorig);
r = SCKGETC("IN_B",1,16,&nval,namecolorig);
r = SCKRDI("INPUTI",1,1,&nval,&maxrow,&unit,&null);


/*  READING FLAG COLUMN  :
    *******************/

messout(messlevel,4,PROID,"      reading flag column");

r = TCTOPN(filetblorig,F_IO_MODE,&idtblorig);

r = TCLSER(idtblorig,namecolorig,&colno);

in10 = (float *) malloc ((unsigned int)(maxrow*sizeof(float)));
if (in10 == 0) {
  SCTPUT(" ");
  messout(messlevel,1,PROID,"Allocation not possible !");
  sprintf(&tmps[0],"(%s,%s - decimal input)",filetblorig,namecolorig);
  messout(messlevel,1,PROID,tmps);
  error();
}

out2 = (char **) malloc ((unsigned int)(maxrow*sizeof(char *)));
if (out2 == 0) {
  SCTPUT(" ");
  messout(messlevel,1,PROID,"Allocation not possible !");
  sprintf(&tmps[0],"(%s,%s - binary storage)",filetblorig,namecolorig);
  messout(messlevel,1,PROID,tmps);
  error();
}
for(i=0; i<maxrow; i++) {
  out2[i] = (char *) malloc ((unsigned int)(COLS1*sizeof(char)));
  if (out2[i] == 0) {
    SCTPUT(" ");
    messout(messlevel,1,PROID,"Allocation not possible !");
    sprintf(&tmps[0],"(%s,%s - binary storage)",filetblorig,namecolorig);
    messout(messlevel,1,PROID,tmps);
    error();
  }
}

outlett = (char **) malloc ((unsigned int)(maxrow*sizeof(char *)));
if (outlett == 0) {
  SCTPUT(" ");
  messout(messlevel,1,PROID,"Allocation not possible !");
  sprintf(&tmps[0],"(%s,%s - character output)",filetblorig,namecolorig);
  messout(messlevel,1,PROID,tmps);
  error();
}
for(i=0; i<maxrow; i++) {
  outlett[i] = (char *) malloc ((unsigned int)(COLS2*sizeof(char)));
  if (outlett == 0) {
    SCTPUT(" ");
    messout(messlevel,1,PROID,"Allocation not possible !");
    sprintf(&tmps[0],"(%s,%s - character output)",filetblorig,namecolorig);
    messout(messlevel,1,PROID,tmps);
    error();
  }
}

in10_0 = in10;
for (flagloop=1; flagloop<maxrow+1; flagloop++,in10++)
  r = TCERDR(idtblorig,flagloop,colno,in10,&null);
in10 = in10_0;


/**/


/***************************
 *                         *
 *  ACTUAL COMPUTATIONS  : *
 *                         *
 ***************************/

messout(messlevel,4,PROID,"      converting flag column");

strcpy(LEGEND,FLAGS);
in10_0 = in10;
for (flagloop=0; flagloop<maxrow; flagloop++,in10++) {

  elem = 8;
  while (elem--) {                          /* CONVERSION DECIMAL -> BINARY */

    *in10 = nint(*in10-0.49)/2.;  /* "0.5"->"0.49" for 0 */

    if (nint(*in10)-(*in10) != 0.)
      out2[flagloop][elem] = '1';
    else
      out2[flagloop][elem] = '0';
  }
  out2[flagloop][8] = '\0';

  for (sign=7,reverse=0; sign>-1; sign--,reverse++) {  /* CONVERSION BINARY */
                                                       /* -> LETTERS AND    */
    if (out2[flagloop][sign] == '1')                   /* REVERSE DISPLAY   */
      outlett[flagloop][reverse] = LEGEND[reverse];    /* OF LETTERS        */
    else
      outlett[flagloop][reverse] = '-';
  }
  outlett[flagloop][8] = '\0';

  strcpy(tmps1,"  \0");
  strncpy(tmps2,&outlett[flagloop][0],5);
  tmps2[5] = '\0';
  strncpy(tmps3,&outlett[flagloop][6],2);
  tmps3[2] = '\0';
  strcat(tmps1,tmps2);
  strcat(tmps1,tmps3);
  strcpy(outlett[flagloop],tmps1);

}
in10 = in10_0;


/*  REWRITING FLAG COLUMN  :
    *********************/

messout(messlevel,4,PROID,"      rewriting flag column");

for (flagloop=1; flagloop<maxrow+1; flagloop++)
  r = TCEWRC(idtblorig,flagloop,colno,outlett[flagloop-1]);


free(in10);
for(i=0; i<maxrow; i++) free(out2[i]);
free(out2);
for(i=0; i<maxrow; i++) free(outlett[i]);
free(outlett);


r = TCTCLO(idtblorig);


if (warn == 0)
  r = SCKWRC("Q1",1,"0",1,60,&unit);
else
  r = SCKWRC("Q1",1,"-1",1,60,&unit);
r = SCSEPI();


} /* void main(int argc, char *argv[]) */


/****************************************************************************/

#ifdef __STDC__  /* ANSI */
  static void error(void)
#else  /* K&R */
  static void error()
#endif
{
  int  r, unit;

  r = SCKWRC("Q1",1,"1",1,60,&unit);
  r = SCSEPI();
}

/*----------------------------------------------------------------------------*/

#ifdef __STDC__  /* ANSI */
  static float nint(float x)
#else  /* K&R */
  static float nint(x)
  float x;
#endif
{
  float y;

  if (x >= 0)
    y = (float) floor((double)(x+0.5));
  else
    y = (float) ceil((double)(x-0.5));

  return(y);
}

/******************************************************************************/
