/*  $Id: fors_replcol.c,v 1.4 1998/12/10 14:51:17 swolf Exp $    */
/*
  IN_A         name of image file to be processed
  INPUTI       number of columns to calculate mean and each column index
  OUTPUTI      number of output column (= replaced column)
*/


#include <string.h>
#include <stdio.h>
#include <tblsys.h>
#include <tblerr.h>
#include <tbldef.h>
#include <macrogen.h>
#include <atype.h>
#include <midas_def.h>
#include <math.h>

#define TRUE 1
#define FALSE 0
#define BELL  0x7
#define NLEN  160
#define MLEN  255

int   SI, SF, SU;                       /* size of int, size of float */
int   Mlev;                             /* message level MESS_LEVEL */

int   Stat=0;                           /* status of program 0=ok, -1 error */
int   Id1;                              /* file id of IMAGE frame */
int   NIx, NIy;                         /* dimensions of INPUT frame */
int   NI;                               /* total number of pixels in Ima */
int   Cols[20], Colo;                   /* column numbers */
int   Datform;                          /* data format of BDF image file */
float *Im1f, **MIm1f;                   /* image frame, matrix represent. */
unsigned short *Im1u, **MIm1u;          /* image frame, matrix represent. */

float **map2mat();                      /* map a matrix on a float vector */
unsigned short **usmap2mat();           /* map a matrix on a unsigned vector */

char  Mes[MLEN];                        /* message buffer */
char  Ima1name[NLEN] = "image.bdf";     /* filename of image frame */

char  Prgnam[]  = "replcol";
char  Version[] = "1998.10.23-SW";

char *malloc();

main()

{
  int uni;
  int i, j, k;


  SCSPRO("replcol");
  
  SF = sizeof(float);
  SI = sizeof(int);
  SU = sizeof(unsigned short);

  get_params();                                      /* get MIDAS parameters */
  replace(Cols,Colo);
  save();

}


/* === interface to MIDAS ================================================== */

int get_params()
{
  char   *ptr;
  int    i, j, len;
  int    uni, null;
  int    nx, ny;
  int    npix[2];
  int    finfo[4];

  SCKGETC("IN_A",1,NLEN,&len,Ima1name);           /* get name of image frame */
  if (Ima1name[0] == '?'){
    i=0;
    sprintf(Mes,"PROGRAM: replcol -- %s",Version);
    SCTPUT(Mes);
    SCSEPI();
    exit(0);
  }
  SCKRDI("MESS_LEVEL",1,1,&len,&Mlev,&uni,&null); /* message level */
  SCKRDI("OUTPUTI",1,1,&len,&Colo,&uni,&null);    /* output column */
  SCKRDI("INPUTI",1,20,&len,Cols,&uni,&null);     /* column numbers */
  if (Cols[0]>19) warn(-1);
  /* Cols[0] holds number of columns to be averaged */
  /* Cols[1]...Cols[ Cols[0] ] columns to be averaged */




  SCFINF(Ima1name,4,&finfo[0]);                /* get file information */
  Datform = finfo[1];                          /* data format */
  SCFOPN(Ima1name,Datform,0,F_IMA_TYPE,&Id1);  /* open BDF-image file */
  SCDRDI(Id1,"NPIX",1,2,&len,npix,&uni,&null);
  NIx = npix[0];                               /* x-size of complete frame */
  NIy = npix[1];                               /* y-size of complete frame */
  NI  = npix[0]*npix[1];                       /* total number of pixels */

  switch(Datform)
    {
    case D_R4_FORMAT:
      Im1f = (float*) malloc((size_t) NI*SF);
      if (Im1f == NULL) no_memory("Im1f",NI*SF);
      ptr = (char*) Im1f;
      SCFGET(Id1,1,NI,&len,ptr);               /* read the whole frame */

      MIm1f = map2mat(Im1f,NIx,NIy,1,1);       /* map data for matrix access */
      break;
    case D_UI2_FORMAT:
      Im1u = (unsigned short*) malloc((size_t) NI*SU);
      if (Im1u == NULL) no_memory("Im1u",NI*SU);
      ptr = (char*) Im1u;
      SCFGET(Id1,1,NI,&len,ptr);               /* read the whole frame */

      MIm1u = usmap2mat(Im1u,NIx,NIy,1,1);     /* map data for matrix access */
      break;
    default:
      warn(-4);                                /* un-supported format, exit */
    }
}


int save()
{
  char *ptr;

  switch(Datform)
    {
    case D_R4_FORMAT:
      ptr = (char*) Im1f;
      break;
    case D_UI2_FORMAT:
      ptr = (char*) Im1u;
      break;
    default:
      warn(-4);                                /* un-supported format, exit */
    }

  Stat = SCFPUT(Id1,1,NI,ptr);
  if (Stat != 0) warn(-3);

  SCFCLO(Id1);
  SCSEPI();
}


/* === subroutines doing the main job ====================================== */

int replace(cols,colo)
int *cols, colo;
{
  sprintf(Mes,"Replace column %d by average of %d columns",colo,cols[0]);
  messout(Mlev,4,Prgnam,Mes);


  switch(Datform)
    {
    case D_R4_FORMAT:
      calcmean_f(cols,colo);
      break;
    case D_UI2_FORMAT:
      calcmean_u(cols,colo);
      break;
    default:
      warn(-4);                                /* un-supported format, exit */
    }
}



int calcmean_f(cols,colo)
int *cols, colo;
{
  int i, j, k, ncol, num;
  double *sum, *dvector();

  ncol = cols[0];
  num = ncol;

  sum = dvector(1,NIy);
  for (j=1;j<=NIy;j++){
    for (sum[j]=0.0,i=1;i<=ncol;i++){
      k = cols[i];
      if ((k>=1) && (k<=NIx)) sum[j] += MIm1f[j][k];
      else if (j==1){
	  warn(k);
	  num--;                             /* holds valid number of colums */
      }
    }
  }
  
  if (num>0)                                 /* valid columns ? */
    for (j=1;j<=NIy;j++){
      MIm1f[j][colo] = (float) sum[j]/num;
    }
  else
    warn(-2);
}


int calcmean_u(cols,colo)
int *cols, colo;
{
  int i, j, k, ncol, num;
  long *sum, *lvector();

  ncol = cols[0];
  num = ncol;

  sum = lvector(1,NIy);
  for (j=1;j<=NIy;j++){
    for (sum[j]=0,i=1;i<=ncol;i++){
      k = cols[i];
      if ((k>=1) && (k<=NIx)) sum[j] += MIm1u[j][k];
      else if (j==1){
	  warn(k);
	  num--;                             /* holds valid number of colums */
      }
    }
  }
  
  if (num>0)                                 /* valid columns ? */
    for (j=1;j<=NIy;j++){
      MIm1u[j][colo] = (float) sum[j]/num;
    }
  else
    warn(-2);
}


/* === utilities =========================================================== */
int no_memory(svar,num)
int   num;
char  *svar;
{
  int nbad, uni;

  nbad = 0;
  sprintf(Mes,"Cannot allocate memory for %s [%d]!\n",svar,num);
  messout(Mlev,1,Prgnam,Mes);
  SCETER(-1,"*** memory-error");

  SCSEPI();
  exit(1);
}


int warn(i)
int i;
{
  if (i>0){
    sprintf(Mes,"Wrong image column number: %d; skip.",i);
    messout(Mlev,2,Prgnam,Mes);
  }else{
    switch(-i)
      {
      case 1: strncpy(Mes,"Maximum number of image columns: 19",MLEN);
	messout(Mlev,1,Prgnam,Mes);
	SCETER(-2,"*** maximum-image-column-error");
	SCSEPI();
	exit(1);

      case 2: strncpy(Mes,"No valid image column found !",MLEN);
	messout(Mlev,1,Prgnam,Mes);
	SCETER(-3,"*** valid-image-column-error");
	SCSEPI();
	exit(1);
	
      case 3: strncpy(Mes,"Image could not be saved !",MLEN);
	messout(Mlev,1,Prgnam,Mes);
	SCETER(-4,"*** save-image-error");
	SCSEPI();
	exit(1);
	
      case 4: sprintf(Mes,"Dataformat %d not supported !",Datform);
	messout(Mlev,1,Prgnam,Mes);
	SCETER(-5,"*** dataformat-error");
	SCSEPI();
	exit(1);
      }
  }
}
