/* === read table column =================================================== */
/* INPUT:                                                                    */
/*  tid      table ID                                                        */
/*  icol     table column (index) to be read (start at 0)                    */
/*  nrow     number of rows to be read                                       */
/*  dat      address of data buffer to be filled                             */
/*  coff     column offset (only used if buffer stores more than one column) */
/* REMARK:                                                                   */
/*  Use negative icol in order to indicate the use of a data-vector, i.e.    */
/*  only ONE column will be stored in the data-buffer. The selected table    */
/*  column will be |icol|, coff usually will be 0.                           */
/* ========================================================================= */
int read_tablec(tid,icol,nrow,dat,coff)
int tid, icol, nrow;
float *dat;
int   *coff;
{
  int  i, j, k, ic, null;

  i = icol;
  ic = i;
  if (icol < 0) { i = 0; ic = -icol; }

  for (j=0,k=coff[i];j<nrow;j++,k++)
    TCERDR(tid,j+1,ic+1,&dat[k]);
}


/* === read table ========================================================== */
int read_table(tid,ncol,nrow,dat,coff)
int tid, ncol, nrow;
float *dat;
int   *coff;
{
  int  i, j, k, null;
  float x;
  for (i=0,k=0;i<ncol;coff[++i]=k)
    for (j=0;j<nrow;j++,k++)
      TCERDR(tid,j+1,i+1,&dat[k],&null);
return;
}



/* === write table column ================================================== */
int write_tablec(tid,icol,nrow,dat,coff)
int tid, icol, nrow;
float *dat;
int   *coff;
{
  int  j, k, null;
  for (j=0,k=coff[icol];j<nrow;j++,k++)
    TCEWRR(tid,j+1,icol+1,&dat[k]);
}



/* === write table ========================================================= */
int write_table(tid,ncol,nrow,dat,coff)
int tid, ncol, nrow;
float *dat;
int   *coff;
{
  int  i, j, k, null;
  for (i=0,k=0;i<ncol;i++);
    for (j=0,k=coff[i];j<nrow;j++,k++)
      TCEWRR(tid,j+1,i+1,&dat[k]);
}



/* === copy table column ONLY in memory! =================================== */
int copy_column(src, dest, num)
float *src, *dest;
int   num;
{
  int  j;
  for (j=0;j<num;j++) dest[j] = src[j];
}



/* === show table ========================================================== */
int show_table(tid,ncol,nrow,dat,coff)
int tid, ncol, nrow;
float *dat;
int   *coff;
{
  int  i, j, k, null;
  for (j=0;j<nrow;j++,k++,printf("\n"))
    for (i=0,k=0;i<ncol;i++)
      printf("%f ",dat[j+coff[i]]);
}



int check_table(tid,icol,nrow,dat,coff,listp,num)
int tid, icol, nrow;
float *dat;
int   *coff, *listp, *num;
{
  int  j, k, null;
  for (j=0,k=0;j<nrow;j++)
    if (dat[j+coff[icol]] == 0)
      {
	listp[j] = j+1;
	k++;
      }
  *num = k;
}
