/* $Id: matrix.c,v 1.8 1998/12/22 17:36:37 swolf Exp $    */

#include <string.h>
#include <stdio.h>
#include <math.h>

#define MIN(A,B) ((A)<(B)?(A):(B))      /* minimum function */
#define MAX(A,B) ((A)>(B)?(A):(B))      /* maximum function */



int nomemory(svar,num)
int   num;
char  *svar;
{
  fprintf(stderr,"Cannot allocate memory for %s [%d]!\n",svar,num);
  exit(1);

}


unsigned int *uvector(nl,nh)
int nl,nh;
{
	unsigned int *v;

	v=(unsigned int *)malloc((unsigned) (nh-nl+1)*sizeof(unsigned int));
	if (!v) nrerror("allocation failure in uvector()");
	return v-nl;
}



long *lvector(nl,nh)
int nl,nh;
{
	long *v;

	v=(long *)malloc((unsigned) (nh-nl+1)*sizeof(long));
	if (!v) nrerror("allocation failure in lvector()");
	return v-nl;
}



int **i_matrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
  int i,j,**m,nsize;

  m = NULL;
  nsize = (nrh-nrl+1)*sizeof(int*);
  m=(int **)malloc((size_t) nsize);
  /* printf("i_matrix: 0x%x\n",m); */
  if (!m) nomemory("m",nsize);

  m -= nrl;
  nsize = (nch-ncl+1)*sizeof(int);
  for(i=nrl;i<=nrh;i++) {
    m[i]=(int *)malloc((size_t) nsize);
    if (!m[i]) nomemory("m[i]",nsize);
    m[i] -= ncl;
    for (j=ncl;j<=nch;j++) m[i][j] = 0;
  }
  return m;
}


int **i_rmatrix(m,nrl,nrh,ncl,nch,nrho)
int nrl,nrh,ncl,nch,nrho,**m;
{
  int i,j,nsize;

  /*  show_mat(m,nrl,nrho,ncl,nch); */

  nsize = (nrh-nrl+1)*sizeof(int*);
  m=(int **)realloc(m, (unsigned) nsize);
  if (!m) nomemory("m",nsize);
  m -= nrl;

  nsize = (nch-ncl+1)*sizeof(int);
  for(i=nrl;i<=nrho;i++) {
    m[i]=(int *)realloc(m[i],(unsigned) nsize);
    if (!m[i]) nomemory("m[i]",nsize);
    m[i] -= ncl;
  }

  for(i=nrho+1;i<=nrh;i++) {
    m[i]=(int *)malloc((size_t) nsize);
    if (!m[i]) nomemory("m[i]",nsize);
    m[i] -= ncl;
    for (j=ncl;j<=nch;j++) m[i][j] = 0;
  }

  /*  show_mat(m,nrl,nrh,ncl,nch); */
  return m;
}


float **f_matrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
  int i,j,nsize;
  float **m;

  m = NULL;
  nsize = (nrh-nrl+1)*sizeof(float*);
  m=(float **)malloc((size_t) nsize);
  /* printf("i_matrix: 0x%x\n",m); */
  if (!m) nomemory("m",nsize);

  m -= nrl;
  nsize = (nch-ncl+1)*sizeof(float);
  for(i=nrl;i<=nrh;i++) {
    m[i]=(float *)malloc((size_t) nsize);
    if (!m[i]) nomemory("m[i]",nsize);
    m[i] -= ncl;
    for (j=ncl;j<=nch;j++) m[i][j] = 0;
  }
  return m;
}


double **d_matrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
  int i,j,nsize;
  double **m;

  m = NULL;
  nsize = (nrh-nrl+1)*sizeof(double*);
  m=(double **)malloc((size_t) nsize);
  /* printf("i_matrix: 0x%x\n",m); */
  if (!m) nomemory("m",nsize);

  m -= nrl;
  nsize = (nch-ncl+1)*sizeof(double);
  for(i=nrl;i<=nrh;i++) {
    m[i]=(double *)malloc((size_t) nsize);
    if (!m[i]) nomemory("m[i]",nsize);
    m[i] -= ncl;
    for (j=ncl;j<=nch;j++) m[i][j] = 0;
  }
  return m;
}


/* ------------------------------------------------------------------------- */
int **imap2mat(vec,nx,ny,xoff,yoff)
int *vec;
int   nx, ny, xoff, yoff;
/* index of first element: mat[yoff][xoff]        */
/* element access: (x,y) = mat[y][x]              */
/*                             |  |column counter */
/*                             |row counter       */
{
  int i, j;
  int **mat;

  mat=(int **)malloc((size_t) ny*sizeof(int*));
  if (!mat) nomemory("mat",ny);
  mat -= yoff;

  for(i=0,j=yoff;j<ny+yoff;j++) {
    mat[j] = &vec[i];
    mat[j] -= xoff;
    i += nx;
  }
  return mat;  
}


unsigned int **umap2mat(vec,nx,ny,xoff,yoff)
unsigned int *vec;
int   nx, ny, xoff, yoff;
/* index of first element: mat[yoff][xoff]        */
/* element access: (x,y) = mat[y][x]              */
/*                             |  |column counter */
/*                             |row counter       */
{
  int i, j;
  unsigned int **mat;

  mat=(unsigned int **)malloc((size_t) ny*sizeof(unsigned int*));
  if (!mat) nomemory("mat",ny);
  mat -= yoff;

  for(i=0,j=yoff;j<ny+yoff;j++) {
    mat[j] = &vec[i];
    mat[j] -= xoff;
    i += nx;
  }
  return mat;  
}


short **smap2mat(vec,nx,ny,xoff,yoff)
short *vec;
int   nx, ny, xoff, yoff;
/* index of first element: mat[yoff][xoff]        */
/* element access: (x,y) = mat[y][x]              */
/*                             |  |column counter */
/*                             |row counter       */
{
  int i, j;
  short **mat;

  mat=(short **)malloc((size_t) ny*sizeof(short*));
  if (!mat) nomemory("mat",ny);
  mat -= yoff;

  for(i=0,j=yoff;j<ny+yoff;j++) {
    mat[j] = &vec[i];
    mat[j] -= xoff;
    i += nx;
  }
  return mat;  
}


unsigned short **usmap2mat(vec,nx,ny,xoff,yoff)
unsigned short *vec;
int   nx, ny, xoff, yoff;
/* index of first element: mat[yoff][xoff]        */
/* element access: (x,y) = mat[y][x]              */
/*                             |  |column counter */
/*                             |row counter       */
{
  int i, j;
  unsigned short **mat;

  mat=(unsigned short **)malloc((size_t) ny*sizeof(unsigned short*));
  if (!mat) nomemory("mat",ny);
  mat -= yoff;

  for(i=0,j=yoff;j<ny+yoff;j++) {
    mat[j] = &vec[i];
    mat[j] -= xoff;
    i += nx;
  }
  return mat;  
}


unsigned char **ucmap2mat(vec,nx,ny,xoff,yoff)
unsigned char *vec;
int   nx, ny, xoff, yoff;
/* index of first element: mat[yoff][xoff]        */
/* element access: (x,y) = mat[y][x]              */
/*                             |  |column counter */
/*                             |row counter       */
{
  int i, j;
  unsigned char **mat;

  mat=(unsigned char **)malloc((size_t) ny*sizeof(unsigned char*));
  if (!mat) nomemory("mat",ny);
  mat -= yoff;

  for(i=0,j=yoff;j<ny+yoff;j++) {
    mat[j] = &vec[i];
    mat[j] -= xoff;
    i += nx;
  }
  return mat;  
}


char **cmap2mat(vec,nx,ny,xoff,yoff)
char *vec;
int   nx, ny, xoff, yoff;
/* index of first element: mat[yoff][xoff]        */
/* element access: (x,y) = mat[y][x]              */
/*                             |  |column counter */
/*                             |row counter       */
{
  int i, j;
  char **mat;

  mat=(char **)malloc((size_t) ny*sizeof(char*));
  if (!mat) nomemory("mat",ny);
  mat -= yoff;

  for(i=0,j=yoff;j<ny+yoff;j++) {
    mat[j] = &vec[i];
    mat[j] -= xoff;
    i += nx;
  }
  return mat;  
}


float **map2mat(vec,nx,ny,xoff,yoff)
float *vec;
int   nx, ny, xoff, yoff;
/* index of first element: mat[yoff][xoff]        */
/* element access: (x,y) = mat[y][x]              */
/*                             |  |column counter */
/*                             |row counter       */
{
  int i, j;
  float **mat;

  mat=(float **)malloc((size_t) ny*sizeof(float*));
  if (!mat) nomemory("mat",ny);
  mat -= yoff;

  for(i=0,j=yoff;j<ny+yoff;j++) {
    mat[j] = &vec[i];
    mat[j] -= xoff;
    i += nx;
  }
  return mat;  
}


double **dmap2mat(vec,nx,ny,xoff,yoff)
double *vec;
int   nx, ny, xoff, yoff;
/* index of first element: mat[yoff][xoff]        */
/* element access: (x,y) = mat[y][x]              */
/*                             |  |column counter */
/*                             |row counter       */
{
  int i, j;
  double **mat;

  mat=(double **)malloc((size_t) ny*sizeof(double*));
  if (!mat) nomemory("mat",ny);
  mat -= yoff;

  for(i=0,j=yoff;j<ny+yoff;j++) {
    mat[j] = &vec[i];
    mat[j] -= xoff;
    i += nx;
  }
  return mat;  
}


/* ------------------------------------------------------------------------- */
int free_d_matrix(mat,nx,ny,xoff,yoff)
double **mat;
int nx, ny, xoff, yoff;
{
  int j;

  for(j=ny+yoff-1;j>=yoff;j--) free((char*) (mat[j]+xoff));
  free((char*) (mat+yoff));
}


int free_f_matrix(mat,nx,ny,xoff,yoff)
float **mat;
int nx, ny, xoff, yoff;
{
  int j;

  for(j=ny+yoff-1;j>=yoff;j--) free((char*) (mat[j]+xoff));
  free((char*) (mat+yoff));
}


int free_i_matrix(mat,nx,ny,xoff,yoff)
int **mat;
int nx, ny, xoff, yoff;
{
  int j;

  for(j=ny+yoff-1;j>=yoff;j--) free((char*) (mat[j]+xoff));
  free((char*) (mat+yoff));
}


int free_u_matrix(mat,nx,ny,xoff,yoff)
unsigned int **mat;
int nx, ny, xoff, yoff;
{
  int j;

  for(j=ny+yoff-1;j>=yoff;j--) free((char*) (mat[j]+xoff));
  free((char*) (mat+yoff));
}


int free_us_matrix(mat,nx,ny,xoff,yoff)
unsigned short **mat;
int nx, ny, xoff, yoff;
{
  int j;

  for(j=ny+yoff-1;j>=yoff;j--) free((char*) (mat[j]+xoff));
  free((char*) (mat+yoff));
}


/* ------------------------------------------------------------------------- */
int show_mat(map,ml,mh,nl,nh)
int **map, ml, mh, nl, nh;
{
  int i,j;

  for (j=nl;j<=nh;j++,printf("\n"))
    for (i=ml;i<=mh;i++) printf("%d ",map[i][j]);
  printf("\n");
}


int show_fmat(map,ml,mh,nl,nh)
float **map;
int  ml, mh, nl, nh;
{
  int i,j;

  for (j=nl;j<=nh;j++,printf("\n"))
    for (i=ml;i<=mh;i++) printf("%f ",map[j][i]);
  printf("\n");
}


int show_dmat(map,ml,mh,nl,nh)
double **map;
int  ml, mh, nl, nh;
{
  int i,j;

  for (j=nl;j<=nh;j++,printf("\n"))
    for (i=ml;i<=mh;i++) printf("%5.1f ",map[j][i]);
  printf("\n");
}


float f_matvecmulrc(a,b,n,k,l,ioff)  /* vector multiplication row*column */
float **a, **b;
int   n, k, l, ioff;
/*
  a(k):    k-ter Zeilenvektor in Matrix a
  b(l):    l-ter Spaltenvektor in Matrix b

  c(k,l) = a(k)*b(l)
  Element(k,l) = Zeile(k) * Spalte(l)

  n: Dimension von Zeile und Spalte
*/
{
  int i;
  float sum;

  sum = 0.0;
  for (i=0+ioff;i<n+ioff;i++) sum += a[k][i] * b[i][l];
  return(sum);
}


float f_matvecmulcr(a,b,n,k,l,ioff)  /* vector multiplication column*row */
float **a, **b;
int   n, k, l, ioff;
/*
  a(k):    k-ter Spaltenvektor
  b(l):    l-ter Zeilenvektor
  c(k,l) = a(k)*b(l)
  Element(k,l) = Spalte(k) * Zeile(l)

  n: Dimension von Spalte und Zeile
*/
{
  int i;
  float sum;

  sum = 0.0;
  for (i=0+ioff;i<n+ioff;i++) sum += a[i][k] * b[l][i];
  return(sum);
}



float **f_matmulrr(a,b,na,ma,nb,mb,nc,mc,ioff)  /* * */
float **a, **b;
int   na, ma, nb, mb, *nc, *mc, ioff;
/*
  na: Dimension von Spalte in Matrix a (= Anzahl Zeilen)
  ma: Dimension von Zeile in Matrix a  (= Anzahl Spalten)
  nb: Dimension von Spalte in Matrix b (= Anzahl Zeilen)
  mb: Dimension von Zeile in Matrix b  (= Anzahl Spalten)

  Spalte(a) * Spalte(b) = Element(c) <=>  Zeile(a) * Zeile(b) = Element(c)

  nc: Dimension von Spalte in Matrix b (= Anzahl Zeilen)      = MIN(na,nb)
  mc: Dimension von Zeile in Matrix b  (= Anzahl Spalten)     = MIN(ma,mb)
*/
{
  int k, l;
  int n, m;
  float **res, sum;

  *nc = MIN(na,nb);
  *mc = MIN(ma,mb);
  n = *nc;
  m = *mc;

  res = f_matrix(0,n-1,0,m-1);

  for (l=0+ioff;l<n+ioff;l++)
    for (k=0+ioff;k<m+ioff;k++){
      res[l][k] = a[l][k] * b[l][k];  /* Zeile(a)*Zeile(b) */
    }
  return(res);    
}


float **f_matmulrc(a,b,na,ma,nb,mb,ioff)  /* row*column ## */
float **a, **b;
int   na, ma, nb, mb, ioff;
/*
  na: Dimension von Spalte in Matrix a (= Anzahl Zeilen)
  ma: Dimension von Zeile in Matrix a  (= Anzahl Spalten)
  nb: Dimension von Spalte in Matrix b (= Anzahl Zeilen)
  mb: Dimension von Zeile in Matrix b  (= Anzahl Spalten)

  Zeile(a) * Spalte(b) = Element(c)

  ==> nb == ma
  nc: Dimension von Spalte in Matrix b (= Anzahl Zeilen)      = na
  mc: Dimension von Zeile in Matrix b  (= Anzahl Spalten)     = mb
*/
{
  int k, l;
  float **res, sum;

  if (nb != ma) return((float**)-1);  /* wrong dimensions */

  res = f_matrix(0+ioff,na-1+ioff,0+ioff,mb-1+ioff);

  for (l=0+ioff;l<na+ioff;l++)
    for (k=0+ioff;k<mb+ioff;k++){
      res[l][k] = f_matvecmulrc(a,b,ma,k,l,ioff);
    }

  return(res);    
}


float **f_matmulcr(a,b,na,ma,nb,mb,ioff)  /* column*row # */
float **a, **b;
int   na, ma, nb, mb, ioff;
/*
  na: Dimension von Spalte in Matrix a (= Anzahl Zeilen)
  ma: Dimension von Zeile in Matrix a  (= Anzahl Spalten)
  nb: Dimension von Spalte in Matrix b (= Anzahl Zeilen)
  mb: Dimension von Zeile in Matrix b  (= Anzahl Spalten)

  Spalte(a) * Zeile(b) = Element(c)

  ==> mb == na
  nc: Dimension von Spalte in Matrix b (= Anzahl Zeilen)      = ma
  mc: Dimension von Zeile in Matrix b  (= Anzahl Spalten)     = nb
*/
{
  int k, l;
  float **res, sum;

  if (mb != na) return((float**)-1);  /* wrong dimensions */

  res = f_matrix(0+ioff,nb-1+ioff,0+ioff,ma-1+ioff);

  for (l=0+ioff;l<nb+ioff;l++)
    for (k=0+ioff;k<ma+ioff;k++){
      res[l][k] = f_matvecmulcr(a,b,na,k,l,ioff);
    }

  return(res);    
}


int f_transpose(a,n,ioff)
float **a;
int   n, ioff;
/* 
   a: quadratic matrix
   n: number of rows and columns

   exchange rows and columns
*/
{
  int i, j;

  for (j=0+ioff;j<n+ioff;j++)
    for (i=0+ioff;i<n+ioff;i++)
      a[j][i] = a[i][j];
}


double d_matvecmulrc(a,b,n,k,l,ioff)  /* vector multiplication row*column */
double **a, **b;
int   n, k, l, ioff;
/*
  a(k):    k-ter Zeilenvektor in Matrix a
  b(l):    l-ter Spaltenvektor in Matrix b

  c(k,l) = a(k)*b(l)
  Element(k,l) = Zeile(k) * Spalte(l)

  n: Dimension von Zeile und Spalte
*/
{
  int i;
  double sum;

  sum = 0.0;
  for (i=0+ioff;i<n+ioff;i++) sum += a[k][i] * b[i][l];
  return(sum);
}


double d_matvecmulcr(a,b,n,k,l,ioff)  /* vector multiplication column*row */
double **a, **b;
int   n, k, l, ioff;
/*
  a(k):    k-ter Spaltenvektor
  b(l):    l-ter Zeilenvektor
  c(k,l) = a(k)*b(l)
  Element(k,l) = Spalte(k) * Zeile(l)

  n: Dimension von Spalte und Zeile
*/
{
  int i;
  double sum;

  sum = 0.0;
  for (i=0+ioff;i<n+ioff;i++) sum += a[i][k] * b[l][i];
  return(sum);
}



double **d_matmulrr(a,b,na,ma,nb,mb,nc,mc,ioff)  /* * */
double **a, **b;
int   na, ma, nb, mb, *nc, *mc;
/*
  na: Dimension von Spalte in Matrix a (= Anzahl Zeilen)
  ma: Dimension von Zeile in Matrix a  (= Anzahl Spalten)
  nb: Dimension von Spalte in Matrix b (= Anzahl Zeilen)
  mb: Dimension von Zeile in Matrix b  (= Anzahl Spalten)

  Spalte(a) * Spalte(b) = Element(c) <=>  Zeile(a) * Zeile(b) = Element(c)

  nc: Dimension von Spalte in Matrix b (= Anzahl Zeilen)      = MIN(na,nb)
  mc: Dimension von Zeile in Matrix b  (= Anzahl Spalten)     = MIN(ma,mb)
*/
{
  int k, l;
  int n, m;
  double **res, sum;

  *nc = MIN(na,nb);
  *mc = MIN(ma,mb);
  n = *nc;
  m = *mc;

  res = d_matrix(0+ioff,n-1+ioff,0+ioff,m-1+ioff);

  for (l=0+ioff;l<n+ioff;l++)
    for (k=0+ioff;k<m+ioff;k++){
      res[l][k] = a[l][k] * b[l][k];  /* Zeile(a)*Zeile(b) */
    }
  return(res);    
}


double **d_matmulrc(a,b,na,ma,nb,mb,ioff)  /* row*column ## */
double **a, **b;
int   na, ma, nb, mb, ioff;
/*
  na: Dimension von Spalte in Matrix a (= Anzahl Zeilen)
  ma: Dimension von Zeile in Matrix a  (= Anzahl Spalten)
  nb: Dimension von Spalte in Matrix b (= Anzahl Zeilen)
  mb: Dimension von Zeile in Matrix b  (= Anzahl Spalten)

  Zeile(a) * Spalte(b) = Element(c)

  ==> nb == ma
  nc: Dimension von Spalte in Matrix b (= Anzahl Zeilen)      = na
  mc: Dimension von Zeile in Matrix b  (= Anzahl Spalten)     = mb
*/
{
  int k, l;
  double **res, sum;

  if (nb != ma) return((double**)-1);  /* wrong dimensions */

  res = d_matrix(0+ioff,na-1+ioff,0+ioff,mb-1+ioff);

  for (l=0+ioff;l<na+ioff;l++)
    for (k=0+ioff;k<mb+ioff;k++){
      res[l][k] = d_matvecmulrc(a,b,ma,k,l,ioff);
    }

  return(res);    
}


double **d_matmulcr(a,b,na,ma,nb,mb,ioff)  /* column*row # */
double **a, **b;
int   na, ma, nb, mb, ioff;
/*
  na: Dimension von Spalte in Matrix a (= Anzahl Zeilen)
  ma: Dimension von Zeile in Matrix a  (= Anzahl Spalten)
  nb: Dimension von Spalte in Matrix b (= Anzahl Zeilen)
  mb: Dimension von Zeile in Matrix b  (= Anzahl Spalten)

  Spalte(a) * Zeile(b) = Element(c)

  ==> mb == na
  nc: Dimension von Spalte in Matrix b (= Anzahl Zeilen)      = ma
  mc: Dimension von Zeile in Matrix b  (= Anzahl Spalten)     = nb
*/
{
  int k, l;
  double **res, sum;

  if (mb != na) return((double**)-1);  /* wrong dimensions */

  res = d_matrix(0+ioff,nb-1+ioff,0+ioff,ma-1+ioff);

  for (l=0+ioff;l<nb+ioff;l++)
    for (k=0+ioff;k<ma+ioff;k++){
      res[l][k] = d_matvecmulcr(a,b,na,k,l,ioff);
    }


  return(res);    
}


int d_transpose(a,n,ioff)
double **a;
int   n, ioff;
/* 
   a: quadratic matrix
   n: number of rows and columns

   exchange rows and columns
*/
{
  int i, j;

  for (j=0+ioff;j<n+ioff;j++)
    for (i=0+ioff;i<n+ioff;i++)
      a[j][i] = a[i][j];
}
