#include <string.h>
#include <stdio.h>
#include <math.h>


float f_abs(x)
float x;
{
  if (x<0) return(-x);
  return(x);
}


double calcmean(img,ncl,nch,nrl,nrh,median,sigmed,kappa,sig,nout)
/*
  Determine the mean of a subarea [ncl,..,nch],[nrl,..,nrh] in the float
  matrix img. The parameters ncl,nch,nrl,nrh represent column and row indices
  of the matrix, respectively.
  As soon as sigmed nonzero and kappa positive, a kappa-sigma clipping with
  respect to median will be done: only values within
                    [median-kappa*sigmed, median+kappa*sigmed]

  will be used to determine the mean of the subarea.
  Finally, the mean, sigma of the mean, and the number of outliers will be
  returned as return value, or parameter sig and nout, respectively.
 */
double *sig;
float  **img;
float  median, sigmed;
int    nrl, nrh, ncl, nch, kappa;
int    *nout;
{
/* $Id: statistics.c,v 1.3 2000/07/11 16:36:13 nrainer Exp $    */

  int i, j, num, cnt;
  float thresh, f_abs();
  double sum1, sum2, mean, vars;

  if ((sigmed == 0.0) || (kappa<0)) thresh = 1.0e34;
  else thresh = kappa * sigmed;

  num = 0;
  sum1 = 0.0;
  sum2 = 0.0;
  cnt  = 0;                             /* count outlayers */
  for (j=nrl;j<=nrh;j++)                /* y-direction: loop through rows */
    for (i=ncl;i<=nch;i++,num++){       /* x-direction: loop through columns */
      if (f_abs(img[j][i] - median) < thresh){ /* kappa sigma clipping */
	sum1 += img[j][i];
	sum2 += img[j][i] * img[j][i];
      }else
	cnt++;
    }
  mean = sum1/num;
  vars = abs((sum2 - sum1*sum1/num)/(num-1.0));    /* variance of a sample */
  *sig = sqrt(vars);
  *nout = cnt;

  return(mean);
}


float calcmedian(img,sig,ncl,nch,nrl,nrh,mcol,medsigmed)
float  *img, *sig;
float  *medsigmed;
int    ncl, nch, nrl, nrh, mcol;
{
  long i, j, k, n;
  long nr, nc;
  long n2, n2p, sortmat();
  float med, *fptr;

  nr = nrh-nrl+1;
  nc = nch-ncl+1;
  n = nc*nr;

  fptr = img;
  fptr--;           /* sort routine works with arrays starting at index 1 ! */
  n2p = sortmat(ncl,nch,nrl,nrh,mcol,fptr);
  med = (n % 2 ? fptr[n2p] : 0.5*(fptr[n2p-1] + fptr[n2p]));

  if (sig != NULL){
    k = (nrl-1)*mcol + ncl - 1;
    for (j=nrl;j<=nrh;j++){               /* y-direction: loop through rows */
      for (i=ncl;i<=nch;i++){          /* x-direction: loop through columns */
	k = (j-1)*mcol + i - 1;
	sig[k] = img[k]-med;
	sig[k] *= sig[k];                        /* [image(i,j) - median]^2 */
      }
    }

    fptr = sig;
    fptr--;         /* sort routine works with arrays starting at index 1 ! */
    n2p = sortmat(ncl,nch,nrl,nrh,mcol,fptr);
    *medsigmed = (n % 2 ? fptr[n2p] : 0.5*(fptr[n2p-1] + fptr[n2p]));
    *medsigmed = sqrt(*medsigmed);
  }

  return(med);
}
	 

