! $Id: fors_ech_cal_wave.prg,v 1.10 1999/11/25 11:24:24 sbogun Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_ech_cal_wave.prg,v $
!
! .KEYWORDS   Dispersion coefficients
!
! .PURPOSE    Pipeline wrapper and frame classification for
!             fors_echwave.prg:
!             Determine dispersion coefficients on a lamp frame. For it
!              - extract orders on the frame,
!              - search emission lines on extracted frame,
!              - use existing 1st guess disp.coeffs. for comparison of line
!                positions and laboratory wavelengths for line identification
!                and calculate final disp. coeffs.
!             The extracted orders, found lines, identified wavelengths,
!             residuals (identified minus calculated wavelengths) and
!             pixel sizes can be displayed.
! __________
!
! .USAGE      @p fors_ech_cal_wave in ref [out] [thresh] [degree] [test]
!
! .INPUT      in     (P1): single input frame (ascii list with single frame);
!                          possible frame type:
!                            lamp frame (LAMP_ECH);
!                              default: none
!
!             ref    (P2): super recipe mode: "+" (reference frames are included
!                          in P1),
!                          single recipe mode: set of reference frames (ascii
!                          list);
!                          needed frames:
!                            order detection table
!                            (ORDER_POS_ECH),
!                            table with 1st guess dispersion coefficients
!                            (DISP_COEFF_ECH),
!                            table with laboratory wavelengths
!                            (MASTER_LINECAT_ECH);
!                          also needed frame type:
!                            table with purging information (DESC_PURGE_TABLE);
!                              default: none
!
!             out    (P3): single recipe mode: basename for ascii list with
!                          output frame,
!                          super recipe mode: again set of input frames and
!                          existing image catalogue (".cat"), separated by a
!                          comma;
!                            table with final dispersion coefficients;
!                              default: `dispcoef.tfits' (single mode)
!
!             thresh (P4): detecting threshold above(!) local background
!                          [counts,>0.];
!                            default: 10.
!
!             degree (P5): degree for fitting both x-pix. and absol. orders;
!                            default: 3
!
!             test   (P6): flag for testing calculated disperson coefficients
!                          via rebinning and merging: <y>es,<n>o;
!                            default: <y>
!
! .OUTPUT     "dispcoef.tfits": table with final dispersion coefficients
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  - order extraction:   see "fors_extorder.prg"
!             - line search:        see "fors_echwave_sea.prg"
!             - final disp.coef.:   see "fors_echwave_dis.prg"
!
! .COMMENT    -
!
! .ENVIRON    FORS context
!
! .LANGUAGE   MIDAS 99NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.4  99/10/20  Creation
!                  99/10/21  Release
!
!------------------------------------------------------------------------------
!
!
crossref in ref out thresh degree test
!
!
define/local MAXDEGREE/i/1/1 6  ! technically fixed to 6 - DON'T CHANGE !!!
!
!
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
define/local PROID/c/1/40 "ech_cal_wave"
!
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      in     (P1): single input frame (ascii list with single frame);"
  w/o "                   possible frame type:"
  w/o "                     lamp frame (LAMP_ECH);"
  w/o "                       default: none"
  w/o " "
  w/o "      ref    (P2): super recipe mode: "+" (reference frames are included"
  w/o "                   in P1),"
  w/o "                   single recipe mode: set of reference frames (ascii"
  w/o "                   list);"
  w/o "                   needed frames:"
  w/o "                     order detection table"
  w/o "                     (ORDER_POS_ECH),"
  w/o "                     table with 1st guess dispersion coefficients"
  w/o "                     (DISP_COEFF_ECH),"
  w/o "                     table with laboratory wavelengths"
  w/o "                     (MASTER_LINECAT_ECH);"
  w/o "                   also needed frame type:"
  w/o "                     table with purging information (DESC_PURGE_TABLE);"
  w/o "                       default: none"
  w/o " "
  w/o "      out    (P3): single recipe mode: basename for ascii list with"
  w/o "                   output frame,"
  w/o "                   super recipe mode: again set of input frames and"
  w/o "                   existing image catalogue (".cat"), separated by a"
  w/o "                   comma;"
  w/o "                     table with final dispersion coefficients;"
  w/o "                       default: `dispcoef.tfits' (single mode)"
  w/o " "
  w/o "      thresh (P4): detecting threshold above(!) local background"
  w/o "                   [counts,>0.];"
  w/o "                     default: 10."
  w/o " "
  w/o "      degree (P5): degree for fitting both x-pix. and absol. orders;"
  w/o "                     default: 3"
  w/o " "
  w/o "      test   (P6): flag for testing calculated disperson coefficients"
  w/o "                   via rebinning and merging: <y>es,<n>o;"
  w/o "                     default: <y>"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          PREPARATIONS FOR RECIPE :
!          ***********************
!
        mess/out 3 {PROID} "Compute dispersion coefficients ..."
!
@p fors_paragraph 1 1  ! -------- INITIALIZATION --------
!
log/recipe {PROID} in
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(in):
!------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter name of input frame: "
define/local inpipe_frm/c/1/76 "{p1}"
!
if m$exist(inpipe_frm) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered list with single input frame"
        mess/out 1 {PROID} "is not present or can not be opened !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
!
define/local filecode/i/1/2 0 all
open/file {inpipe_frm} read filecode
if filecode(1) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open entered list !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
read/file {filecode(1)} outputc
if filecode(2) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered list is empty !"
        mess/out 1 {PROID} "({inpipe_frm})"
  close/file {filecode(1)}
  goto error
endif
close/file {filecode(1)}
!
!
!P2(ref):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter name of reference frame set: "
define/local inpipe_ref/c/1/76 "{p2}"
!
! No tests like for P1 - in super recipes reference frames are included in P1 .
!
!
!P3(out):
!-------
define/parameter p3 {PROID} ? -
  "        Enter basename of out.list(single)/exist.cat.".cat"(super): "
define/local outpipe/c/1/76 "{p3}"
!
!
!P4(thresh):
!----------
define/parameter p4 10. n -
  "        Enter detect.thresh. [def.:10.]: "
define/local level/r/1/1 {p4}
!
! Test of detect. thresh. after creation
! of extracted frame in "fors_echwave_sea.prg".
!
!
!P5(degree):
!----------
define/parameter p5 3 n -
  "        Enter fit degree for x-pixels and abs.orders: "
define/local x_ord_deg/i/1/1 {p5}
!
if x_ord_deg .gt. MAXDEGREE then
        write/out " "
        mess/out 1 {PROID} "Fit degree for x-pixels and abs.orders too high !"
        mess/out 1 {PROID} "({x_ord_deg} ; degree<={MAXDEGREE})"
  goto error
endif
!
! Comparison of degree with number of found lines after creation
! of line table in "fors_echwave_dis.prg".
!
!
!P6(test):
!--------
define/parameter p6 y ? -
  "        Enter flag for rebinning / merging [def.:y|n]: "
define/local test/c/1/1 "{p6}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Selecting master line catalogue":
!
  define/local wavcatsel/c/1/76 " " all
!
!     "Identifying input frames":
!
  define/local catal/i/1/1 0
  define/local topicima/c/1/76 " " all
  define/local id/c/1/25 " " all
  !
  define/local lampID/c/1/25 " " all
  define/local ordID/c/1/25 " " all
  define/local dicoID/c/1/25 " " all
  define/local waveID/c/1/25 " " all
  !
  define/local lampima/c/1/76 " " all
  define/local ordtbl/c/1/76 " " all
  define/local guetbl/c/1/76 " " all
  define/local wavcat/c/1/76 " " all
  !
  define/local inlamp/i/1/1 0
  define/local reford/i/1/1 0
  define/local refdico/i/1/1 0
  define/local refwave/i/1/1 0
  !
  define/local insdesc/c/1/60 " " all
!
!     "Determining line width":
!
  define/local descloop/i/1/1 0
  define/local moswid/r/1/1 0.
  define/local linwid/r/1/1 0.
!
!     "Calling of procedure":
!
  define/local disptbl/c/1/76 " " all
!
!
!
!          LOADING OF GLOBAL KEYS (SINGLE RECIPE MODE) ,
!          CONVERSION FITS->BDF/TBL ,
!          CLASSIFICATION OF INPUT FRAME :
!          *********************************************
!
        mess/out 4 {PROID} "Getting input frame"
!
initialize/pipeline {inpipe_frm} {inpipe_ref} -
  inpipe_good.cat ? ? ECH inpipe_bad.cat ?
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Pipeline initialization failed !"
  goto error
endif
-delete inpipe_bad.cat
!
initialize/longlog {inpipe_frm} {inpipe_ref} 2 -
                   "line width,detect.thresh.: {p4}" -
                   "fit degree for x-pixels and abs.orders: {p5}"
!
!
!          IDENTIFICATION VIA DESCRIPTORS ,
!          MAPPING OF FRAMES TO PARAMETERS  :
!          ********************************
!
        mess/out 4 {PROID} "Identifying input and reference frames"
!
lampID = "{WLC_ID}"
ordID = "{ORDER_ID}"
dicoID = "{DISPREL_ID}"
waveID = "{LINCAT_ID}"
!
create/icat {PROID}.cat null {FORS_IDSTRNG} >Null  ! only for finish/longlog
inlamp = 0
reford = 0
refdico = 0
refwave = 0
!
ident_loop:
  store/frame topicima inpipe_good.cat 1 end_ident_loop
  id = "{{topicima},{fors_idstrng}}"
  !
  if id .eq. "{lampID}" then
    lampima = topicima
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    inlamp = inlamp+1
  elseif id .eq. "{ordID}" then
    ordtbl = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    reford = reford+1
  elseif id .eq. "{dicoID}" then
    guetbl = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refdico = refdico+1
  elseif id .eq. "{waveID}" then
    wavcat = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refwave = refwave+1
  endif
  !
  goto ident_loop
end_ident_loop:
-delete inpipe_good.cat
!
if inlamp .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching lamp frame present !"
  goto error
elseif inlamp .gt. 1 then
        write/out " "
        mess/out 1 {PROID} "More than one lamp frame present !"
  goto error
endif
if reford .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching order detection table present !"
  goto error
elseif reford .gt. 1 then
        write/out " "
        mess/out 1 {PROID} "More than one order detection table present !"
  goto error
endif
if refdico .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching table w. 1.guess disp.coef. present !"
  goto error
elseif refdico .gt. 1 then
        write/out " "
        mess/out 1 {PROID} "More than one table w. 1.guess disp.coef. present !"
  goto error
endif
if refwave .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching tbl. w. lab. wavelengths present !"
  goto error
elseif refwave .gt. 1 then
        write/out " "
        mess/out 1 {PROID} "More than one tbl. w. lab. wavelengths present !"
  goto error
endif
!
insdesc = "{{lampima},{h_instrume}}"
!
!
!          CHECKING NUMBER OF PORTS FOR READOUT :
!          ************************************
!
if fors_nout .gt. 1 then
        write/out " "
        mess/out 1 {PROID} "No 4-port readout possible for Echelle !"
        mess/out 1 {PROID} "({lampima})"
  goto error
endif
!
!
!          REMOVING OVERSCAN SECTIONS :
!          **************************
!
@p fors_paragraph 2 1  ! -------- PROCESSING --------
!
        mess/out 4 {PROID} "Cutting overscan regions from lamp frame"
        mess/out 4 {PROID} "  {lampima}"
!
!*************************
@p fors_trim {lampima} + +
!*************************
if {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Cutting of overscan areas failed !"
  goto error
endif
!
!
!          DETERMINING APPROX. LINE WIDTH :
!          ******************************
!
outputi = 0
do descloop = 1 FORS_MAXSLITLET
  if m$existd(lampima,"{h_moswid({descloop})}") .eq. 1 then
    if {{lampima},{h_moswid({descloop})}} .gt. 0. then
      moswid = {{lampima},{h_moswid({descloop})}}
      outputi = 1
      goto found_width  ! use 1st found non-zero descriptor
    endif
  endif
enddo
found_width:
if outputi .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No descriptor w. non-zero MOS slit width present !"
        mess/out 1 {PROID} "({lampima},{kbase_mosi}i{ksuff_moswid})"
  goto error
endif
if m$existd(lampima,h_pixscale) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Descriptor w. pixel scale [arcsec/pix] is missing !"
        mess/out 1 {PROID} "({lampima},{h_pixscale})"
  goto error
endif
!
linwid = {moswid}/{{lampima},{h_pixscale}}  ! in pixels
!
!
!          SELECTING GRISMS AND LAMPS
!          IN MASTER LINE CATALOGUE :
!          **************************
!
!*************************************
@p fors_echwave_sel {lampima} {wavcat}
!*************************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Selection of master line catalogue failed !"
  goto error
endif
wavcatsel = q2
!
!
!          *********************************
!          *                               *
!          *  CALLING OF MAIN PROCEDURE :  *
!          *                               *
!          *********************************
!
        mess/out 4 {PROID} "Applying procedure "fors_echwave" :"
!
!******************************************************************************
@p fors_echwave {lampima} {ordtbl} -
                {linwid},{level} auto {guetbl},{wavcatsel} {x_ord_deg} {test} p
!******************************************************************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Computation of dispersion coefficients failed !"
  goto error
endif
!
disptbl = q2
!
create/icat outprog.cat null >Null
add/icat outprog.cat {disptbl} >Null
-delete dirfile.ascii
!
!
!          UPDATING DESCRIPTORS OF OUT- AND INPUT FRAMES :
!          *********************************************
!
@p fors_paragraph 3 1  ! -------- TERMINATION --------
!
        mess/out 4 {PROID} "Classifying output table"
        mess/out 4 {PROID} "({disptbl})"
!
classify/product {disptbl} {DISPREL_ID} {d_pre} {d_det} OK
write/descriptor {disptbl} {h_instrume}/c/1/60 {insdesc}
write/descriptor {disptbl} {FORS_IDSTRNG}/c/1/60 {DISPREL_ID} !f.su.rec.
write/descriptor {disptbl} {h_proqcsts}/c/1/9 "CHECKED"
write/dhelp {disptbl} {h_proqcsts} "{i_proqcsts}"
longlog/out 1 "     Table with dispersion coefficients: {disptbl}"
!
        mess/out 4 {PROID} "Endorsing QC status in input image"
!
write/descriptor {lampima} {h_proqcsts}/c/1/9 "CHECKED"
write/dhelp {lampima} {h_proqcsts} "{i_proqcsts}"
!
shortlog/out 1 "Dispersion coefficients calculated"
!
!
!          UPDATING OF OUTPUT CATALOGUE (SUPER RECIPE MODE) ,
!          CONVERSION BDF/TBL->FITS/TFITS (SINGLE MODE) :
!          **************************************************
!
        mess/out 4 {PROID} "Releasing output table"
!
finish/pipeline outprog.cat {outpipe} + {disptbl}
!
finish/longlog {PROID}.cat outprog.cat {disptbl} 2 {outpipe}
!
-delete {PROID}.cat
-delete outprog.cat
!
!
!          RESETTINGS AFTER RECIPE :
!          ***********************
!
set/format
!
log/recipe {PROID} out
!
@p fors_paragraph 4 1  ! ================
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" then
      assign/graphic g,3 >Null
      modify/graphic icon >Null
    endif
  endif
  finish/longlog ? -1 ? 2
  write/out " "
  return 1
!
!
!------------------------------------------------------------------------------
