! $Id: fors_gen_cal_ccdlin.prg,v 1.78 2000/02/22 13:00:12 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_gen_cal_ccdlin.prg,v $
!
! .KEYWORDS   Detector:  linearity, transfer
!
! .PURPOSE    Pipeline wrapper and frame classification for
!             fors_ccdlin.prg:
!             Compute CCD linearity (relationship between illumination and
!             output of amplifiers) and transfer values (electrons to ADU
!             conversion factor and readout noise) for 1-port readout.
! __________
!
! .USAGE      @p fors_gen_cal_ccdlin in ref [out]
!
! .INPUT      in   (P1): set of input frames (ascii list);
!                        needed frames:
!                          at least 3 pairs of screen flatfield frames (per pair
!                          same exposure time) with exposure levels from above
!                          bias to nearly saturation (CCDLIN_<all_modes> or
!                          SCREEN_FLAT_<all_modes>) (1-port readout);
!                            default: none
!
!             ref  (P2): super recipe mode: "+" (reference frames are included
!                        in P1),
!                        single recipe mode: set of reference frames (ascii
!                        list);
!                        needed frame:
!                          master bias frame (MASTER_BIAS),
!                        optional:
!                          bad pixel table (BAD_PIXEL_TABLE);
!                        also needed frame type:
!                          table with purging information (DESC_PURGE_TABLE);
!                            default: none
!
!             out  (P3): single recipe mode: basename for ascii list with
!                        output frame,
!                        super recipe mode: again set of input frames and
!                        existing image catalogue (".cat"), separated by a
!                        comma;
!                        table for input set:
!                          linearity statistics / conversion factor
!                          (LIN_GAIN_TABLE);
!                            default: `PROID' defined in procedure (single mode)
!
! .OUTPUT     "<PROID>_lin.tfits":  table with linearity statistics
!                                   and conversion factor
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    -
!
! .ENVIRON    FORS context
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/09/18  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref in ref out
!
!
define/local MINPAIR/i/1/1 3 ? +lower_levels  ! >= 3 !!!
outputi(1) = MINPAIR*2
define/local MINFRAM/i/1/1 {outputi(1)}
!
define/local EXP_LOW/r/1/1 5.
define/local SHUT_INT/c/1/6 "-2.,2."
!
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
define/local PROID/c/1/40 "gen_cal_ccdlin"
!
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      in   (P1): set of input frames (ascii list);"
  w/o "                 needed frames:"
  w/o "                   at least 3 pairs of screen flatfield frames (per pair"
  w/o "                   same exposure time) with exposure levels from above"
  w/o "                   bias to nearly saturation (CCDLIN_<all_modes> or"
  w/o "                   SCREEN_FLAT_<all_modes>) (1-port readout);"
  w/o "                     default: none"
  w/o " "
  w/o "      ref  (P2): super recipe mode: "+" (reference frames are included"
  w/o "                 in P1),"
  w/o "                 single recipe mode: set of reference frames (ascii"
  w/o "                 list);"
  w/o "                 needed frame:"
  w/o "                   master bias frame (MASTER_BIAS),"
  w/o "                 optional:"
  w/o "                   bad pixel table (BAD_PIXEL_TABLE);"
  w/o "                 also needed frame type:"
  w/o "                   table with purging information (DESC_PURGE_TABLE);"
  w/o "                     default: none"
  w/o " "
  w/o "      out  (P3): single recipe mode: basename for ascii list with"
  w/o "                 output frame,"
  w/o "                 super recipe mode: again set of input frames and"
  w/o "                 existing image catalogue (".cat"), separated by a"
  w/o "                 comma;"
  w/o "                 table for input set:"
  w/o "                   linearity statistics / conversion factor"
  w/o "                   (LIN_GAIN_TABLE);"
  w/o "                     default: `PROID' defined in procedure (single mode)"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          PREPARATIONS FOR RECIPE :
!          ***********************
!
log/recipe {PROID} in
!
        mess/out 3 {PROID} "Compute CCD linearity and conversion factor ..."
!
@p fors_paragraph 1 1  ! -------- INITIALIZATION --------
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(in):
!------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter name of input frame set: "
define/local inpipe_frm/c/1/76 "{p1}"
!
if m$exist(inpipe_frm) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered list is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
!
define/local filecode/i/1/2 0 all
open/file {inpipe_frm} read filecode
if filecode(1) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open entered list !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
read/file {filecode(1)} outputc
if filecode(2) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered list is empty !"
        mess/out 1 {PROID} "({inpipe_frm})"
  close/file {filecode(1)}
  goto error
endif
close/file {filecode(1)}
!
!
!P2(ref):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter name of reference frame set: "
define/local inpipe_ref/c/1/76 "{p2}"
!
! No tests like for P1 - in super recipes reference frame is included in P1 .
!
!
!P3(out):
!-------
define/parameter p3 {PROID} ? -
  "        Enter basename of out.list(single)/exist.cat.".cat"(super): "
define/local outpipe/c/1/76 "{p3}"
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Identifying input frames":
!
  define/local catal/i/1/1 0
  define/local topicima/c/1/76 " " all
  define/local id/c/1/25 " " all
  !
  define/local flatID/c/1/25 " " all
  define/local linID/c/1/25 " " all
  define/local biasID/c/1/25 " " all
  define/local bpxID/c/1/25 " " all
  !
  define/local master/c/1/76 " " all
  define/local bpxtable/c/1/76 " " all
  !
  define/local in/i/1/1 0
  define/local refmas/i/1/1 0
  define/local refbpx/i/1/1 0
  !
  define/local insdesc/c/1/20 " " all
!
!     "Checking ... conversion":
!
  define/local lastbutone/i/1/1 0
  define/local portloop/i/1/1 0
  define/local cf1/c/1/60 " " all
  define/local cf2/c/1/60 " " all
!
!     "Calling of procedure":
!
  define/local psc/i/1/1 0
  define/local osc/i/1/1 0
!
!
!          LOADING OF GLOBAL KEYS (SINGLE RECIPE MODE) ,
!          CONVERSION FITS->BDF/TBL ,
!          CLASSIFICATION OF INPUT FRAMES :
!          *********************************************
!
        mess/out 4 {PROID} "Getting input frames"
!
initialize/pipeline {inpipe_frm} {inpipe_ref} -
  inpipe_good.cat ? ? + inpipe_bad.cat ?
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Pipeline initialization failed !"
  goto error
endif
-delete inpipe_bad.cat
!
! Initialize product header:
init/prohead {inpipe_frm} {inpipe_ref} -
  2,"Compute CCD linearity and conversion factor"
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Product header initialization failed !"
  goto error
endif
!
initialize/longlog {inpipe_frm} {inpipe_ref} 2
!
!
!          IDENTIFICATION VIA DESCRIPTORS ,
!          MAPPING OF FRAMES TO PARAMETERS  :
!          ********************************
!
        mess/out 4 {PROID} "Identifying input and reference frames"
!
flatID = "{FLAT_ID}"
linID = "{CCDLIN_ID}"
biasID = "{MBIAS_ID}"
bpxID = "{BPIXT_ID}"
!
create/icat {PROID}.cat null {FORS_IDSTRNG} >Null
in = 0
refmas = 0
refbpx = 0
!
ident_loop:
  store/frame topicima inpipe_good.cat 1 end_ident_loop
  id = "{{topicima},{fors_idstrng}}"
  !
  if id .eq. "{flatID}" .or. id .eq. "{linID}" then
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    in = in+1
  elseif id .eq. "{biasID}" then
    master = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refmas = 1
  elseif id .eq. "{bpxID}" then
    bpxtable = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refbpx = 1
  endif
  !
  goto ident_loop
end_ident_loop:
-delete inpipe_good.cat
!
if in .lt. MINFRAM then
        write/out " "
        mess/out 1 {PROID} "Too few (<{MINFRAM}) input screen-flatf."
        mess/out 1 {PROID} "frames present !"
  goto error
endif
!
if refmas .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No master bias frame present !"
  goto error
endif
!
!if refbpx .eq. 0 then
!        write/out " "
!        mess/out 1 {PROID} "No bad pixel table present !"
!  goto error
!endif
!
insdesc = "{{master},INSTRUME}"
!
!
!          CHECKING EQUALITY OF CONVERSION
!          FACTORS FOR MULTIPLE PORT READOUT  :
!          *********************************
!
if fors_nout .gt. 1 then
  !
        mess/out 4 {PROID} "Checking conversion factors in all ports"
  !
  catal = 0
  conv_loop:
    store/frame topicima {PROID}.cat 1 end_conv_loop
    lastbutone = fors_nout-1
    do portloop = 1 lastbutone
      cf1 = h_confac({fors_outi({portloop})})
      outputi(1) = portloop+1
      cf2 = h_confac({fors_outi({outputi(1)})})
      if "{{topicima},{cf2}}" .ne. "{{topicima},{cf1}}" then
        write/out " "
        mess/out 1 {PROID} "Ports have different conversion factors !"
        mess/out 1 {PROID} "({topicima})"
        goto error
      endif
    enddo
    goto conv_loop
  end_conv_loop:
  !
endif
!
!
!          REMOVING BAD PIXELS :
!          *******************
!
@p fors_paragraph 2 1  ! -------- PROCESSING --------
!
if refbpx .ne. 0 then
  !
        mess/out 4 {PROID} "Removing bad pixels from input frames"
  !
  !***************************************
  @p fors_corr_bpxl {bpxtable} {PROID}.cat
  !***************************************
  !
endif
!
!
!          DETERMINING OVERSCAN SECTIONS :
!          *****************************
!
!*******************************
@p fors_get_imsec #1_{PROID}.cat
!*******************************
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Determination of overscan sections failed !"
  goto error
endif
psc = outputi(5)
osc = outputi(6)
!
!
!          *********************************
!          *                               *
!          *  CALLING OF MAIN PROCEDURE :  *
!          *                               *
!          *********************************
!
if fors_nout .ne. 1 then
        mess/out 2 {PROID} "Frames were read out with multiple ports -"
        mess/out 2 {PROID} "average statistics of whole frame will be used !"
  warn = 1
endif
!
        mess/out 4 {PROID} "Applying procedure  "fors_ccdlin" :"
!
!*******************************************************************
@p fors_ccdlin {PROID} {master} {psc},{osc} {EXP_LOW} {SHUT_INT} p n
!*******************************************************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Linearity determination failed !"
  goto error
endif
!
create/icat outprog.cat null >Null
add/icat outprog.cat {PROID}_lin.tbl >Null
-delete dirfile.ascii
!
!
!          UPDATING DESCRIPTORS OF OUTPUT FRAMES :
!          *************************************
!
@p fors_paragraph 3 1  ! -------- TERMINATION --------
!
        mess/out 4 {PROID} "Classifying output table ({PROID}_lin.tbl)"
!
classify/product {PROID}_lin.tbl {CCDLINT_ID} {d_pre} {d_empty} OK y
write/descriptor {PROID}_lin.tbl {h_instrume}/c/1/20 {insdesc}
write/descriptor {PROID}_lin.tbl {FORS_IDSTRNG}/c/1/60 {CCDLINT_ID} !f.su.rec.
longlog/out 1 "     Linearity table: {PROID}_lin.tbl"
shortlog/out 1 "Computed CCD linearity"
!
!
!          UPDATING OF OUTPUT CATALOGUE (SUPER RECIPE MODE) ,
!          CONVERSION BDF/TBL->FITS/TFITS (SINGLE MODE) :
!          **************************************************
!
        mess/out 4 {PROID} "Releasing output table"
!
! Create product header:
finish/prohead outprog.cat 2
!
finish/pipeline outprog.cat {outpipe} + {PROID}_lin.tbl
!
finish/longlog {PROID}.cat outprog.cat {PROID}_lin.tbl 2 {outpipe}
!
-delete {PROID}.cat
-delete outprog.cat
!
!
!          RESETTINGS AFTER RECIPE :
!          ***********************
!
set/format
!
log/recipe {PROID} out
!
@p fors_paragraph 4 1  ! ================
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  write/out " "
  return 1
!
!
!-------------------------------------------------------------------------------
