! $Id: fors_gen_qc0_overexpo.prg,v 1.149 2000/04/06 15:38:46 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_gen_qc0_overexpo.prg,v $
!
! .KEYWORDS   Pixel values, simple statistics
!
! .PURPOSE    Pipeline wrapper and frame classification for
!             fors_overexpo.prg:
!               Check if the raw images contain overexposed pixels and write
!               their number into a table per input frame. The count threshold
!               for overexposed pixels is taken from the global keyword
!               fors_maxlin. For each input image with at least one overexposed
!               pixel an output image is created as mask with the overexposed
!               pixels. This mask is intended to be used after the basic reduc-
!               tions (devision by flatfield etc. and devision by exposure time
!               before photometry).
!               For SUPER RECIPE MODE frames which have more than <P4> percent
!               overexposed pixels are removed from the input catalogue.
!               The images can be displayed with the overexposed pixels marked
!               blue.
! __________
!
! .USAGE      @p fors_gen_qc0_overexpo in ref [out] [maxper]
!
! .INPUT      in    (P1): set of input frames (ascii list);
!                         possible frame types:
!                           screen flatfields (SCREEN_FLAT_<all_modes>),
!                           sky flatfields (SKY_FLAT_IMG),
!                           standard star images (STANDARD_<all_modes>),
!                           science images (SCIENCE_<all_modes>),
!                           wavelength calibration frames (LAMP_LSS/MOS);
!                             default: none
!
!             ref   (P2): super recipe mode: "+" (reference frame is
!                         included in P1),
!                         single recipe mode: set of reference frame (ascii
!                         list);
!                         needed frame type:
!                           table with purging information (DESC_PURGE_TABLE);
!                             default: none
!
!             out   (P3): single recipe mode: basename for ascii list with out-
!                         put frame,
!                         super recipe mode: again set of input frames and
!                         existing image catalogue (".cat"), separated by a
!                         comma;
!                         frame for each overexposed input image:
!                           mask images with overexposed pixels (OVEREXPO_MASK),
!                         table for each input image:
!                           exposure statistics (OVEREXPO_TABLE);
!                             default: `PROID' defined in proc. (single mode)
!
!             maxper(P4): percentage (of number of all pixels) for minimal
!                         forbidden number of overexposed pixels - in super
!                         recipe mode frames with at least `maxper' overexposed
!                         pixels are removed from the input catalogue;
!                             default: 3.00
!
! .OUTPUT     Q2 :  total number of frames with less than <P4>% overexposed
!                   pixels; if Q2 is "0", Q1 is set to "1" (error).
!
!             "*_oex.tfits":  tables with numbers of overexposed pixels
!
!             "*_oem.fits" :  images as masks with overexposed pixels
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Viewing: creation of a special colour lookup table and per image
!             creation of a special intensity transfer table - for belittled
!             representation the maximum values including omitted columns/rows
!             are indicated.
!             Search / Mask creation: C programme.
!
! .COMMENT    -
!
! .ENVIRON    FORS context
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    0.9  98/05/18  Creation
!                  98/07/15  Release
!             1.0  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref in ref out maxper
!
!
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
define/local PROID/c/1/40 "gen_qc0_overexpo"
!
if fors_maxlin .le. 65534. then
  define/local FINDVAL/i/1/1 {fors_maxlin}
  define/local MASKVAL/i/1/1 65535
else
        write/out " "
        mess/out 1 {PROID} "Global key fors_maxlin={fors_maxlin} >65534. !"
  goto error
endif
!
define/local warn/i/1/1 0
write/keyword q1 0
write/keyword q2 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      in    (P1): set of input frames (ascii list);"
  w/o "                  possible frame types:"
  w/o "                  screen flatfields (SCREEN_FLAT_<all_modes>),"
  w/o "                  sky flatfields (SKY_FLAT_IMG),"
  w/o "                  standard star images (STANDARD_<all_modes>),"
  w/o "                  science images (SCIENCE_<all_modes>),"
  w/o "                  wavelength calibration frames (LAMP_LSS/MOS);"
  w/o "                      default: none"
  w/o " "
  w/o "      ref   (P2): super recipe mode: "+" (reference frame is"
  w/o "                  included in P1),"
  w/o "                  single recipe mode: set of reference frame (ascii"
  w/o "                  list);"
  w/o "                  needed frame type:"
  w/o "                    table with purging information (DESC_PURGE_TABLE);"
  w/o "                      default: none"
  w/o " "
  w/o "      out   (P3): single recipe mode: basename for ascii list with out-"
  w/o "                  put frame,"
  w/o "                  super recipe mode: again set of input frames and"
  w/o "                  existing image catalogue (".cat"), separated by a"
  w/o "                  comma;"
  w/o "                  frame for each overexposed input image:"
  w/o "                    mask images with overexposed pixels (OVEREXPO_MASK),"
  w/o "                  table for each input image:"
  w/o "                    exposure statistics (OVEREXPO_TABLE);"
  w/o "                      default: `PROID' defined in proc. (single mode)"
  w/o " "
  w/o "      maxper(P4): percentage (of number of all pixels) for minimal"
  w/o "                  forbidden number of overexposed pixels - in super"
  w/o "                  recipe mode frames with at least `maxper' overexposed"
  w/o "                  pixels are removed from the input catalogue;"
  w/o "                      default: 3.00"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          PREPARATIONS FOR RECIPE :
!          ***********************
!
log/recipe {PROID} in
!
        mess/out 3 {PROID} "Check image(s) for overexposed pixels ..."
!
@p fors_paragraph 1 1  ! -------- INITIALIZATION --------
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(in):
!------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter name of input frame set: "
define/local inpipe_frm/c/1/76 "{p1}"
!
if m$exist(inpipe_frm) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered list is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
!
define/local filecode/i/1/2 0 all
open/file {inpipe_frm} read filecode
if {filecode(1)} .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open entered list !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
read/file {filecode(1)} outputc
if {filecode(2)} .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered list is empty !"
        mess/out 1 {PROID} "({inpipe_frm})"
  close/file {inpipe_frm}
  goto error
endif
close/file {filecode(1)}
!
!
!P2(ref):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter name of reference frame set: "
define/local inpipe_ref/c/1/76 "{p2}"
!
! No tests like for P1 - in super recipes reference frame is included in P1 .
!
!
!P3(out):
!-------
define/parameter p3 {PROID} ? -
  "        Enter basename of out.list(single)/exist.cat.".cat"(super): "
define/local outpipe/c/1/76 "{p3}"
!
!
!P4(maxper):
!----------
define/parameter p4 3. n -
  "        Enter percent. (of #allPIX.) for max. allowed #oexPIX. [def.:3.]: "
define/local maxper/r/1/1 {p4}
!
if maxper .lt. 0. .or. maxper .gt. 100. then
        write/out " "
        mess/out 1 {PROID} "Impossible percentage !"
    goto error
endif
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Identifying input frames":
!
  define/local catal/i/1/1 0
  define/local topicima/c/1/76 " " all
  define/local id/c/1/25 " " all
  !
  define/local bpxrawID/c/1/25 " " all
  define/local flatID/c/1/25 " " all
  define/local skyID/c/1/25 " " all
  define/local stdID/c/1/25 " " all
  define/local sciID/c/1/25 " " all
  define/local lampID/c/1/25 " " all
  !
  define/local in/i/1/1 0
  !
  define/local insdesc/c/1/60 " " all
!
!     "Calling of procedure":
!
  define/local notremoved/i/1/1 0
  define/local badprog_cat/c/1/76 " " all
  define/local outprog_cat/c/1/76 " " all
!
!     "Cutting pre-/overscan":
!
  define/local maskname/c/1/76 " " all
!
!     "Updating descriptors":
!
  define/local char/i/1/1 0
  define/local topictab/c/1/76 " " all
  define/local topicmsk/c/1/76 " " all
  define/local topicin/c/1/76 " " all
  define/local percent/r/1/1 0.
  define/local ovextable/c/1/76 " " all
!
!
!          LOADING OF GLOBAL KEYS (SINGLE RECIPE MODE) ,
!          CONVERSION FITS->BDF/TBL ,
!          CLASSIFICATION OF INPUT FRAMES :
!          *********************************************
!
        mess/out 4 {PROID} "Getting input frames"
!
initialize/pipeline {inpipe_frm} {inpipe_ref} -
  inpipe_good.cat ? ? + inpipe_bad.cat ?
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Pipeline initialization failed !"
  goto error
endif
-delete inpipe_bad.cat
!
! Initialize product header:
init/prohead {inpipe_frm} {inpipe_ref} -
  2,"Check image(s) for overexposed pixels" maxper,{maxper}
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Product header initialization failed !"
  goto error
endif
!
set/format T6.2,
initialize/longlog {inpipe_frm} {inpipe_ref} 2 "max.overexp.pix.: {maxper}%"
set/format {FMTDEF}
!
!
!          IDENTIFICATION VIA DESCRIPTORS ,
!          MAPPING OF FRAMES TO PARAMETERS  :
!          ********************************
!
        mess/out 4 {PROID} "Identifying input and reference frames"
!
bpxrawID = "{BPIXF_ID}"
flatID = "{FLAT_ID}"
skyID = "{SKY_ID}"   ! only IMG
stdID = "{STD_ID}"
sciID = "{SCIENCE_ID}"
lampID = "{WLC_ID}"  ! only LSS/MOS
!
create/icat {PROID}.cat null {FORS_IDSTRNG} >Null
in = 0
!
ident_loop:
  store/frame topicima inpipe_good.cat 1 end_ident_loop
  id = "{{topicima},{fors_idstrng}}"
  !
  if id .eq. "{bpxrawID}" then
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    in = 1
  elseif id .eq. "{flatID}" then
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    in = 1
  elseif id .eq. "{stdID}" then
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    in = 1
  elseif id .eq. "{sciID}" then
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    in = 1
  elseif "{FORS_INSMODE}" .eq. "LSS" .or. "{FORS_INSMODE}" .eq. "MOS" then
    if id .eq. "{lampID}" then
      add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
      in = 1
    endif
  elseif "{FORS_INSMODE}" .eq. "IMG" then
    if id .eq. "{skyID}" then
      add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
      in = 1
    endif
  endif
  !
  goto ident_loop
end_ident_loop:
-delete inpipe_good.cat
!
if in .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching input frame present !"
        mess/out 1 {PROID} "(screenf., skyf., std.star, science,"
        mess/out 1 {PROID} " or wavelen.cal. frame)"
  goto error
endif
!
insdesc = "{#1_{PROID}.cat,{h_instrume}}"
!
!
!          *********************************
!          *                               *
!          *  CALLING OF MAIN PROCEDURE :  *
!          *                               *
!          *********************************
!
@p fors_paragraph 2 1  ! -------- PROCESSING --------
!
        mess/out 4 {PROID} "Applying procedure "fors_overexpo" :"
!
!*******************************************************************
@p fors_overexpo {PROID} {FINDVAL},{MASKVAL} {maxper} {h_ovexmask} p
!*******************************************************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Overexposure check failed !"
  goto error
endif
!
notremoved = {q2}  ! for return (q2)
!
outputi = m$parse(q3,"cats")
if outputi(1) .ne. 2 then
        write/out " "
        mess/out 1 {PROID} "Wrong number of elements in Q3 !"
  goto error
endif
!
badprog_cat = cats01
if pipeline(6) .eq. 1 then  ! ONLY FOR SUPERRECIPES :
  !
  ! FRAMES WHICH HAVE MORE THAN <P3>% OVEREXPOSED
  ! PIXELS ARE REMOVED FROM THE INPUT CATALOGUE :
  !
  show/icat {badprog_cat} >Null
  if outputi(1) .gt. 0 subtract/icat {inpipe_frm} {badprog_cat} >Null
  !
endif
-delete {badprog_cat}
!
outprog_cat = cats02
!
!
!          CUTTING PRE-/OVERSCAN FROM OUTPUT MASK IMAGES :
!          *********************************************
!
catal = 0
scan_loop:
  store/frame topicima {PROID}.cat 1 end_scan_loop
  !
  if m$existd(topicima,h_ovexmask) .eq. 1 then  ! else no overexposed pixels
    !
    maskname = "{{topicima},{h_ovexmask}}"
    if m$exist(maskname) .eq. 1 then
      !
        mess/out 4 {PROID} "Cutting overscan regions from mask frame"
        mess/out 4 {PROID} "  {maskname}"
      !
      !**************************
      @p fors_trim {maskname} + +
      !**************************
      if {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Cutting of overscan areas failed !"
        goto error
      endif
      !
    else
      !
        write/out " "
        mess/out 1 {PROID} "Descriptor w. overexpo. mask present -"
        mess/out 1 {PROID} "({topicima},{h_ovexmask})"
        mess/out 1 {PROID} "but mask frame not !"
        mess/out 1 {PROID} "({maskname})"
      goto error
      !
    endif
    !
  endif
  !
  goto scan_loop
end_scan_loop:
!
!
!          UPDATING DESCRIPTORS OF OUT- AND INPUT FRAMES :
!          *********************************************
!
@p fors_paragraph 3 1  ! -------- TERMINATION --------
!
        mess/out 4 {PROID} "Classifying output table(s) and image(s)"
        mess/out 4 {PROID} "(*_oex.tbl, *_oem.bdf) and endorsing"
        mess/out 4 {PROID} "QC status and value in input image(s)"
!
catal = 0
desc_loop:
  store/frame topicima {outprog_cat} 1 end_desc_loop
  !
  if m$index(topicima,"_oex.tbl") .gt. 1 then
    topictab = topicima
      char = m$indexb(topicima,"_oex.tbl")-1
    topicmsk = topicima(1:{char})//"_oem.bdf"
    topicin = topicima(1:{char})//".bdf"
  else
    goto desc_loop
  endif
  !
    percent = {{topictab},:OEX_PCEN,1}
  !
  if m$exist(topicmsk) .ne. 0 then
    classify/product {topicmsk} {OVEXPM_ID} {d_qcp} {d_empty} OK
    write/descriptor {topicmsk} {h_instrume}/c/1/60 {insdesc}
    write/descriptor {topicmsk} {FORS_IDSTRNG}/c/1/60 {OVEXPM_ID} !f.su.rec.
    write/descriptor {topicmsk} {h_proqcsts}/c/1/9 "CHECKED"
    write/dhelp {topicmsk} {h_proqcsts} "{i_proqcsts}"
    write/descriptor {topicmsk} {h_proqcovex}/r/1/1 {percent}
    write/dhelp {topicmsk} {h_proqcovex} "{i_proqcovex}"
  endif
  !
  classify/product {topictab} {OVEXPT_ID} {d_qcp} {d_empty} OK
  write/descriptor {topictab} {h_instrume}/c/1/60 {insdesc}
  write/descriptor {topictab} {FORS_IDSTRNG}/c/1/60 {OVEXPT_ID} !f.su.rec.
  write/descriptor {topictab} {h_proqcsts}/c/1/9 "CHECKED"
  write/dhelp {topictab} {h_proqcsts} "{i_proqcsts}"
  write/descriptor {topictab} {h_proqcovex}/r/1/1 {percent}
  write/dhelp {topictab} {h_proqcovex} "{i_proqcovex}"
  ovextable = topictab
  !
  write/descriptor {topicin} {h_proqcsts}/c/1/9 "CHECKED"
  write/dhelp {topicin} {h_proqcsts} "{i_proqcsts}"
  write/descriptor {topicin} {h_proqcovex}/r/1/1 {percent}
  write/dhelp {topicin} {h_proqcovex} "{i_proqcovex}"
  !
  goto desc_loop
end_desc_loop:
!
shortlog/out 1 "Frame(s) checked for overexposure"
!
!
!          UPDATING OF OUTPUT CATALOGUE (SUPER RECIPE MODE) ,
!          CONVERSION BDF/TBL->FITS/TFITS (SINGLE MODE) :
!          **************************************************
!
        mess/out 4 {PROID} "Releasing output table(s) and image(s)"
!
! Create product header:
finish/prohead {outprog_cat} 2
!
finish/pipeline {outprog_cat} {outpipe} + {ovextable}
!
finish/longlog {PROID}.cat {outprog_cat} {ovextable} 2 {outpipe}
!
-delete {PROID}.cat
-delete {outprog_cat}
!
!
!          RESETTINGS AFTER RECIPE :
!          ***********************
!
log/recipe {PROID} out
!
@p fors_paragraph 4 1  ! ================
!
!
if notremoved .eq. 0. then
        write/out " "
        set/format F5.2,
        mess/out 1 {PROID} -
          "All frames have more than {maxper}% overexp. pixels !"
  goto error
endif
!
set/format
!
if m$existk("nice_crea") .eq. 1 then
  if nice_crea .eq. "y" then
    display/lut off
    clear/display
    -delete ovex.itt
    -delete ovex.lut
    load/lut heat
    load/itt ramp
  endif
endif
!
if warn .eq. 0 then
  return 0 {notremoved}
else
  return -1 {notremoved}
endif
!
error:
  set/format
  if m$existk("nice_crea") .eq. 1 then
    if nice_crea .eq. "y" then
      display/lut off
      clear/display
      -delete ovex.itt
      -delete ovex.lut
      load/lut heat
      load/itt ramp
    endif
  endif
  finish/longlog ? -1 ? 2
  write/out " "
  return 1 -99
!
!
!-------------------------------------------------------------------------------
