! $Id: fors_gen_qc1_chkcal.prg,v 1.169 2000/02/22 12:58:52 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_gen_qc1_chkcal.prg,v $
!
! .KEYWORDS   Statistics
!
! .PURPOSE    Pipeline wrapper and frame classification for
!             fors_chkcal.prg:
!             Determine differences between (reduced) testframes and a master
!             frame by the use of statistical tests. Outliers ("cosmics") are
!             removed within the tests.
!             The images and statistical plots can be displayed.
! __________
!
! .USAGE      @p fors_gen_qc1_chkcal in ref [out] type [thresh]
!
! .INPUT      in     (P1): set of input frames (ascii list);
!                          possible frame types:
!                          1) reduced bias frames (REDUCED_BIAS),
!                          2) reduced dark frames (REDUCED_DARK),
!                          3) reduced sky flatfields (REDUCED_SKY_FLAT_IMG),
!                          4) reduced screen fflds. (REDUCED_SCREEN_FLAT_IMG),
!                          5) normalized screen fflds. (NORM_FLAT_[LSS|MOS]),
!                          6) reduced night fflds. (SCIENCE_REDUCED_IMG);
!                              default: none
!
!             ref    (P2): super recipe mode: "+" (reference frames are
!                          included in P1),
!                          single recipe mode: set of reference frames (ascii
!                          list);
!                          for each reduced frame (P1) also the original version
!                          must be present:
!                          1) bias frames (BIAS),
!                          2) dark frames (DARK),
!                          3) sky flatfields (SKY_FLAT_IMG),
!                          4) screen flatfields (SCREEN_FLAT_IMG),
!                          5) screen flatfields (SCREEN_FLAT_[LSS|MOS]),
!                          6) science frames (SCIENCE_IMG);
!                          furthermore master frame of same type is needed:
!                          1) master bias frame (MASTER_BIAS),
!                          2) master dark frame (MASTER_DARK),
!                          3) master sky flatfield (MASTER_SKY_FLAT_IMG),
!                          4) master screen flatfield (MASTER_SCREEN_FLAT_IMG),
!                          5) normalized master screen ffld. (MASTER_NORM_FLAT_
!                                                             [LSS|MOS]),
!                          6) master night ffld. (MASTER_NIGHT_FLAT_IMG);
!                          also needed frame type:
!                            table with purging information (DESC_PURGE_TABLE);
!                              default: none
!
!             out    (P3): single recipe mode: basename for ascii list with
!                          output frame,
!                          super recipe mode: again set of input frames and
!                          existing image catalogue (".cat"), separated by a
!                          comma;
!                          table for input set:
!                            image statistics (CHKCAL_TABLE);
!                              default: `PROID' defined in proc. (single mode)
!
!             type   (P4): number for type of input frames:
!                          <1>:bias, <2>:dark, <3>:skyflat, <4>:screenflat,
!                          <5>:normflat, <6>:nightflat;
!                            default: none
!
!             thresh (P5): 7 float numbers with following thresholds, separated
!                          by commas:
!                           1. median+FACTOR*mediansigma -> outlier limit,
!                           2. max. allowed PERCENTAGE of outliers,
!                           3. |MEAN| limit for binned difference frame,
!                           4. SLOPE(Poisson-Fit): x=sqrt(cnt.level) y=std.dev.,
!                           5. ZERO POINT(Poisson-Fit),
!                           6. STD.DEV. limit for binned difference frame,
!                           7. STD.DEV. limit for quadrants of diff.frame;
!                          If one of the numbers in 2./3./6./7. is "-1.",
!                          the concerning limit test is not applied;
!                          limit 7. is only used for imaging.
!                            default:
!                              none
!                            examples:
!                              bias/dark      : 7.,0.5,10.,0.034,-0.13,0.75,0.4
!                              sky  flat.(IMG): 7.,1.0,70.,0.034,-0.13,10.,40.
!                              scr. flat.(IMG): 7.,0.5,10.,0.034,-0.13,2.,2.
!                              norm.flat.(LSS): 7.,0.5,10.,0.034,-0.13,4.,-1.
!                              nightflat.(IMG): 7.,1.0,10.,0.034,-0.13,35,10.
!
! .OUTPUT     "<PROID>_chk.tfits":  table with statistics
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Comparisons between
!             each binned/outlier-cleaned difference frame "(master-scalefactor*
!               testframe)", with the following scalefactors: for bias none, for
!               darks exp.time(master)/exp.time(testframe), for flats median(ma-
!               ster)/median(testframe),
!             and pre-defined, empirically received thresholds:
!               - mean (64x64 bins)
!               - standard deviation (64x64 bins)
!               - standard deviation (2x2 bins = 4 quadrants)
!               - percentage of outliers
!               The thresholds are:
!                1. a FACTOR for calculating the outlier limit: (median value of
!                   the frame + this FACTOR * the median sigma of the frame)
!                   yields the limit for detecting outliers;
!                2. maximal allowed PERCENTAGE of outliers;
!                3. limit for the ABSOLUTE value of the MEAN of the binned
!                   difference frame - difference as explained above;
!                4. the SLOPE calculated by a `Poisson fit': the abscissa is the
!                   square root of the count level of every possible raw frame
!                   type, the ordinate is the standard deviation of the counts;
!                5. the ZERO POINT (standard deviation when count level is zero)
!                   of the `Poisson fit' from 4. - can be negative;
!                6. limit for the STANDARD DEVIATION of the binned (64x64)
!                   difference frame;
!                7. limit for the STANDARD DEVIATION of the MEANS of the
!                   quadrants of the difference frame;
!             If one of the numbers in 2./3./6./7. is "-1.",
!             the concerning limit test is not applied;
!             limit 7. is only used for imaging.
!             If at least one of the tests notices that the input image is bad,
!             it is rejected.
!
! .COMMENT    -
!
! .ENVIRON    FORS context
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    0.9  98/04/10  Creation
!                  98/07/15  Release
!             1.0  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref in ref out type thresh
!
!
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
define/local PROID/c/1/40 "gen_qc1_chkcal"
!
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "     in     (P1): set of input frames (ascii list);"
  w/o "                   possible frame types:"
  w/o "                   1) reduced bias frames (REDUCED_BIAS),"
  w/o "                   2) reduced dark frames (REDUCED_DARK),"
  w/o "                   3) reduced sky flatfields (REDUCED_SKY_FLAT_IMG),"
  w/o "                   4) reduced screen fflds. (REDUCED_SCREEN_FLAT_IMG),"
  w/o "                   5) normalized screen fflds. (NORM_FLAT_[LSS|MOS]),"
  w/o "                   6) reduced night fflds. (SCIENCE_REDUCED_IMG);"
  w/o "                       default: none"
  w/o " "
  w/o "      ref    (P2): super recipe mode: "+" (reference frames are"
  w/o "                   included in P1),"
  w/o "                   single recipe mode: set of reference frames (ascii"
  w/o "                   list);"
  w/o "                   for each reduced frame (P1) also the original version"
  w/o "                   must be present:"
  w/o "                   1) bias frames (BIAS),"
  w/o "                   2) dark frames (DARK),"
  w/o "                   3) sky flatfields (SKY_FLAT_IMG),"
  w/o "                   4) screen flatfields (SCREEN_FLAT_IMG),"
  w/o "                   5) screen flatfields (SCREEN_FLAT_[LSS|MOS]),"
  w/o "                   6) science frames (SCIENCE_IMG);"
  w/o "                   furthermore master frame of same type is needed:"
  w/o "                   1) master bias frame (MASTER_BIAS),"
  w/o "                   2) master dark frame (MASTER_DARK),"
  w/o "                   3) master sky flatfield (MASTER_SKY_FLAT_IMG),"
  w/o "                   4) master screen flatfield (MASTER_SCREEN_FLAT_IMG),"
  w/o "                   5) normalized master screen ffld. (MASTER_NORM_FLAT_"
  w/o "                                                      [LSS|MOS]),"
  w/o "                   6) master night ffld. (MASTER_NIGHT_FLAT_IMG);"
  w/o "                   also needed frame type:"
  w/o "                     table with purging information (DESC_PURGE_TABLE);"
  w/o "                       default: none"
  w/o " "
  w/o "      out    (P3): single recipe mode: basename for ascii list with"
  w/o "                   output frame,"
  w/o "                   super recipe mode: again set of input frames and"
  w/o "                   existing image catalogue (".cat"), separated by a"
  w/o "                   comma;"
  w/o "                   table for input set:"
  w/o "                     image statistics (CHKCAL_TABLE);"
  w/o "                       default: `PROID' defined in proc. (single mode)"
  w/o " "
  w/o "      type   (P4): number for type of input frames:"
  w/o "                   <1>:bias, <2>:dark, <3>:screenflat, <4>:skyflat,"
  w/o "                   <5>:nightflat, <6>:normflat;"
  w/o "                       default: none"
  w/o " "
  w/o "      thresh (P5): 7 float numbers with following thresholds, separated"
  w/o "                   by commas:"
  w/o "                    1. median+FACTOR*mediansigma -> outlier limit,"
  w/o "                    2. max. allowed PERCENTAGE of outliers,"
  w/o "                    3. |MEAN| limit for binned difference frame,"
  w/o "                    4. SLOPE(Poisson-Fit): x=sqrt(cnt.level) y=std.dev.,"
  w/o "                    5. ZERO POINT(Poisson-Fit),"
  w/o "                    6. STD.DEV. limit for binned difference frame,"
  w/o "                    7. STD.DEV. limit for quadrants of diff.frame;"
  w/o "                   If one of the numbers in 2./3./6./7. is "-1.","
  w/o "                   the concerning limit test is not applied;"
  w/o "                   limit 7. is only used for imaging."
  w/o "                     default:"
  w/o "                       none"
  w/o "                     examples:"
  w/o "                       bias/dark      : 7.,0.5,10.,0.034,-0.13,0.75,0.4"
  w/o "                       sky  flat.(IMG): 7.,1.0,70.,0.034,-0.13,10.,40."
  w/o "                       scr. flat.(IMG): 7.,0.5,10.,0.034,-0.13,2.,2."
  w/o "                       norm.flat.(LSS): 7.,0.5,10.,0.034,-0.13,4.,-1."
  w/o "                       nightflat.(IMG): 7.,1.0,10.,0.034,-0.13,35,10."
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          PREPARATIONS FOR RECIPE :
!          ***********************
!
log/recipe {PROID} in
!
        mess/out 3 {PROID} "Compare testframes with master frame ..."
!
@p fors_paragraph 1 1  ! -------- INITIALIZATION --------
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(in):
!------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter name of input frame set: "
define/local inpipe_frm/c/1/76 "{p1}"
!
if m$exist(inpipe_frm) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered list is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
!
define/local filecode/i/1/2 0 all
open/file {inpipe_frm} read filecode
if filecode(1) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open entered list !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
read/file {filecode(1)} outputc
if filecode(2) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered list is empty !"
        mess/out 1 {PROID} "({inpipe_frm})"
  close/file {filecode(1)}
  goto error
endif
close/file {filecode(1)}
!
!
!P2(ref):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter name of reference frame set: "
define/local inpipe_ref/c/1/76 "{p2}"
!
! No tests like for P1 - in super recipes reference frame is included in P1 .
!
!
!P3(out):
!-------
define/parameter p3 {PROID} ? -
  "        Enter basename of out.list(single)/exist.cat.".cat"(super): "
define/local outpipe/c/1/76 "{p3}"
!
!
!P4(type):
!--------
if p4 .eq. "?" then
        write/out " "
        mess/out 1 {PROID} "No type switch has been entered (P4) !"
  goto error
endif
define/parameter p4 ? n -
  "        Enter type switch [def:none|<1>-<6>:(see header)]:"
define/local type/i/1/1 {p4}
if type .lt. 1 .or. type .gt. 6 then
        write/out " "
        mess/out 1 {PROID} "Impossible type switch (P4) !"
  goto error
endif
!
!
!P5(thresh):
!----------
outputi = m$parse(p5,"limit")
if p5 .eq. "?" then
        write/out " "
        mess/out 1 {PROID} "No thresholds have been entered (P5) - 7 needed !"
  goto error
elseif outputi .ne. 7 then
        write/out " "
        mess/out 1 {PROID} -
          "Wrong number of thresh.(P5): {outputi} entered - 7 needed !"
  goto error
endif
!
define/parameter p5 ? ? -
  "        Enter 7 thresholds (see procedure header) : "
! only for initialize/prohead and initialize/longlog:
define/local thresh/c/1/80 "{p5}"
! actual input:                         ! ALLOWED :
define/local tf_cosm64/r/1/1 {limit01}  ! >0.
define/local tf_outp64/r/1/1 {limit02}  ! 0.-100. or -1.
define/local tf_mea64/r/1/1 {limit03}   ! >0.     or -1.
define/local tf_slope/r/1/1 {limit04}   ! >0.
define/local tf_zero/r/1/1 {limit05}    ! all
define/local tf_sig64/r/1/1 {limit06}   ! >0.     or -1.
define/local tf_sig2/r/1/1 {limit07}    ! >0.     or -1.
!
define/local badthr/i/1/1 0
if tf_cosm64 .le. 0. badthr = 1
if tf_outp64 .lt. 0. .or. tf_outp64 .gt. 100. then
   if tf_outp64 .ne. -1. badthr = 1
endif
if tf_mea64 .le. 0. .and. tf_mea64 .ne. -1. badthr = 1
if tf_slope .le. 0. badthr = 1
if tf_sig64 .le. 0. .and. tf_sig64 .ne. -1. badthr = 1
if tf_sig2 .le. 0. .and. tf_sig2 .ne. -1. badthr = 1
if badthr .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "At least one threshold (P5) is out of range !"
  goto error
endif
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Identifying input frames":
!
  !define/local found/i/1/1 0
  define/local catal/i/1/1 0
  define/local topicima/c/1/76 " " all
  define/local id/c/1/25 " " all
  !
  define/local oriframeID/c/1/25 " " all
  define/local redframeID/c/1/25 " " all
  define/local masterID/c/1/25 " " all
  !
  define/local master/c/1/76 " " all
  !
  define/local ori/i/1/1 0
  define/local in/i/1/1 0
  define/local ref/i/1/1 0
  !
  define/local insdesc/c/1/60 " " all
!
!     "Calculation of mean ...":
!
  define/local psc/i/1/1 0
  define/local osc/i/1/1 0
  define/local binfact/i/1/1 0
  define/local subxstart/i/1/1 0
  define/local subxend/i/1/1 0
  define/local loopcnt/i/1/1 0
!
!     "Calling of procedure ...":
!
  define/local thrstrng/c/1/80 " " all
!
!
!          LOADING OF GLOBAL KEYS (SINGLE RECIPE MODE) ,
!          CONVERSION FITS->BDF/TBL ,
!          CLASSIFICATION OF INPUT FRAMES :
!          *********************************************
!
        mess/out 4 {PROID} "Getting input frames"
!
initialize/pipeline {inpipe_frm} {inpipe_ref} -
  inpipe_good.cat ? ? + inpipe_bad.cat ?
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Pipeline initialization failed !"
  goto error
endif
-delete inpipe_bad.cat
!
! Initialize product header:
init/prohead {inpipe_frm} {inpipe_ref} -
  2,"Compare testframes with master frame" type,{type} thresh,{thresh}
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Product header initialization failed !"
  goto error
endif
!
initialize/longlog {inpipe_frm} {inpipe_ref} 2 -
                   "frame type: {type}" -
                   "thresholds: {thresh}"
!
!
!          IDENTIFICATION VIA DESCRIPTORS ,
!          MAPPING OF FRAMES TO PARAMETERS  :
!          ********************************
!
        mess/out 4 {PROID} "Identifying input (incl.orig.) and reference frames"
!
!found = 0
branch type 1,2,3,4,5,6 biasID,darkID,skyflatID,scrflatID,normflatID,nightflatID
  !autoID:  ! first found master frame determines type of input frames
  !  store/frame topicima inpipe_good.cat 1 end_branchID
  !  id = "{{topicima},{fors_idstrng}}"
  !  if id .eq. "{MBIAS_ID}" then
  !    goto biasID
  !  elseif id .eq. "{MDARK_ID}" then
  !    goto darkID
  !  elseif id .eq. "{MSKY_ID}" then
  !    goto skyflatID
  !  elseif id .eq. "{MFLAT_ID}" then
  !    goto scrflatID
  !  elseif id .eq. "{MNFLAT_ID}" then
  !    goto normflatID
  !  elseif id .eq. "{MNIGHT_ID}" then
  !    goto nightflatID
  !  endif
  !  goto autoID
  biasID:
    oriframeID = "{BIAS_ID}"
    redframeID = "{RBIAS_ID}"
    masterID = "{MBIAS_ID}"
  !  found = 1  ! needed for autoID above
    goto end_branchID
  darkID:
    oriframeID = "{DARK_ID}"
    redframeID = "{RDARK_ID}"
    masterID = "{MDARK_ID}"
  !  found = 1  ! needed for autoID above
    goto end_branchID
  skyflatID:
    oriframeID = "{SKY_ID}"
    redframeID = "{RSKY_ID}"
    masterID = "{MSKY_ID}"
  !  found = 1  ! needed for autoID above
    goto end_branchID
  scrflatID:
    oriframeID = "{FLAT_ID}"
    redframeID = "{RFLAT_ID}"
    masterID = "{MFLAT_ID}"
  !  found = 1  ! needed for autoID above
    goto end_branchID
  normflatID:
    oriframeID = "{FLAT_ID}"
    redframeID = "{NFLAT_ID}"
    masterID = "{MNFLAT_ID}"
  !  found = 1  ! needed for autoID above
    goto end_branchID
  nightflatID:
    oriframeID = "{SCIENCE_ID}"
    redframeID = "{RSCIENCE_ID}"
    masterID = "{MNIGHT_ID}"
  !  found = 1  ! needed for autoID above
    goto end_branchID
end_branchID:
!if found .eq. 0 goto noref
!
create/icat origima.cat null >Null
create/icat {PROID}.cat null {FORS_IDSTRNG} >Null
ori = 0
in = 0
ref = 0
!
catal = 0
ident_loop:
  store/frame topicima inpipe_good.cat 1 end_ident_loop
  id = "{{topicima},{fors_idstrng}}"
  !
  if id .eq. "{oriframeID}" then
    add/icat origima.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => original"
    ori = ori+1
  elseif id .eq. "{redframeID}" then
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => reduced"
    in = in+1
  elseif id .eq. "{masterID}" then
    master = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    ref = 1
  endif
  !
  goto ident_loop
end_ident_loop:
-delete inpipe_good.cat
!
if ori .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching original frame present !"
        mess/out 1 {PROID} "(new bias/dark/skyf./scrf./sci.)"
  goto error
endif
!
if in .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching reduced frame present !"
        mess/out 1 {PROID} "(reduced bias/dark/skyf./scrf./nightf. or normf.)"
  goto error
endif
!
if ori .ne. in then
        write/out " "
        mess/out 1 {PROID} "Numbers of original and reduced"
        mess/out 1 {PROID} "frames are not equal !"
  goto error
endif
!
if ref .eq. 0 then
!noref:
        write/out " "
        mess/out 1 {PROID} "No matching reference frame present !"
        mess/out 1 {PROID} "(master bias/dark/skyf./scrf./normf./nightf.)"
  goto error
endif
!
insdesc = "{#1_{PROID}.cat,{h_instrume}}"
!
!
!          CALCULATION OF MEAN VALUES
!          OF ORIGINAL FRAMES  :
!          **************************
!
@p fors_paragraph 2 1  ! -------- PROCESSING --------
!
        mess/out 3 {PROID} "Calculating statistics of original frames"
!
!*******************************
@p fors_get_imsec #1_{PROID}.cat
!*******************************
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Determination of overscan sections failed !"
  goto error
endif
psc = outputi(5)
osc = outputi(6)
binfact = outputi(9)  ! for "Calling of procedure"
subxstart = psc+1
subxend = {{master},NPIX(1)}-osc
!
show/icat origima.cat n
define/local orimean/r/1/{outputi(1)} 0. all +lower_levels
!
catal = 0
orig_loop:
  store/frame topicima origima.cat 1 end_orig_loop
  loopcnt = loopcnt+1
  !
  statistics/image {topicima} [@{subxstart},<:@{subxend},>] ? ? sn >Null
  orimean({loopcnt}) = outputr(3)
  !
  goto orig_loop
end_orig_loop:
!
-delete origima.cat
!
!
!          *********************************
!          *                               *
!          *  CALLING OF MAIN PROCEDURE :  *
!          *                               *
!          *********************************
!
set/format F7.3,
!
thrstrng = "{tf_cosm64},{tf_outp64},{tf_mea64},{tf_slope},{tf_zero},{tf_sig64}"
!
if FORS_INSMODE .eq. "IMG" then  ! IMAGING
 thrstrng = thrstrng // ",{tf_sig2}"
else                             ! SPECTROSCOPY
 thrstrng = thrstrng // ",-1."
endif
!
set/format {FMTDEF}
!
define/local MEDDESC/c/1/30 "{h_promedi}" ? +lower_levels
!
        mess/out 4 {PROID} "Applying procedure  "fors_chkcal" :"
!
!***************************************************************************
@p fors_chkcal {PROID},orimean,{binfact} {master} {type} p ? {psc},{osc} ? -
   {thrstrng}
!***************************************************************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Statistical tests on calibration frames failed !"
  goto error
endif
!
create/icat outprog.cat null >Null
add/icat outprog.cat {PROID}_chk.tbl >Null
-delete dirfile.ascii
!
!
!          UPDATING DESCRIPTORS OF OUT- AND INPUT FRAMES :
!          *********************************************
!
@p fors_paragraph 3 1  ! -------- TERMINATION --------
!
        mess/out 4 {PROID} "Classifying output table ({PROID}_chk.tbl)"
!
classify/product {PROID}_chk.tbl {CHKCALT_ID} {d_qcp} {d_empty} OK y
write/descriptor {PROID}_chk.tbl {h_instrume}/c/1/60 {insdesc}
write/descriptor {PROID}_chk.tbl {FORS_IDSTRNG}/c/1/60 {CHKCALT_ID} !f.su.rec.
write/descriptor {PROID}_chk.tbl {h_proqcsts}/c/1/9 "CHECKED"
write/dhelp {PROID}_chk.tbl {h_proqcsts} "{i_proqcsts}"
!
        mess/out 4 {PROID} "Endorsing QC status in input image(s)"
!
catal = 0
desc_loop_in:
  store/frame topicima {PROID}.cat 1 end_desc_loop_in
  !
  write/descriptor {topicima} {h_proqcsts}/c/1/9 "CHECKED"
  write/dhelp {topicima} {h_proqcsts} "{i_proqcsts}"
  !
  goto desc_loop_in
end_desc_loop_in:
!
!
!          UPDATING OF OUTPUT CATALOGUE (SUPER RECIPE MODE) ,
!          CONVERSION BDF/TBL->FITS/TFITS (SINGLE MODE) :
!          **************************************************
!
        mess/out 4 {PROID} "Releasing output table"
!
! Create product header:
finish/prohead outprog.cat 2
!
finish/pipeline outprog.cat {outpipe} + {PROID}_chk.tbl
!
finish/longlog {PROID}.cat outprog.cat {PROID}_chk.tbl 2 {outpipe}
!
-delete {PROID}.cat
-delete outprog.cat
!
!
!          RESETTINGS AFTER RECIPE :
!          ***********************
!
set/format
!
log/recipe {PROID} out
!
@p fors_paragraph 4 1  ! ================
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  finish/longlog ? -1 ? 2
  write/out " "
  return 1
!
!
!-------------------------------------------------------------------------------
