! $Id: fors_gen_qc1_tstmaster.prg,v 1.74 2000/02/22 12:58:52 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_gen_qc1_tstmaster.prg,v $
!
! .KEYWORDS   Statistics
!
! .PURPOSE    Pipeline wrapper and frame classification for
!             fors_tstmaster.prg:
!             Determine differences between (a) newly created master-frame(s)
!             and a certified one by the use of statistical tests.
!             The images and statistical plots can be displayed.
! __________
!
! .USAGE      @p fors_gen_qc1_tstmaster in ref [out] [type] [thresh]
!
! .INPUT      in     (P1): set of NEW created master frames (ascii list);
!                          possible frame types:
!                            master bias frames (MASTER_BIAS),
!                            master dark frames (MASTER_DARK),
!                            master screen flatfields (MASTER_SCREEN_FLAT_IMG),
!                            master sky    flatfields (MASTER_SKY_FLAT_IMG),
!                            super  sky    flatfields (SUPER_SKY_FLAT_IMG),
!                            master night  flatfields (MASTER_NIGHT_FLAT_IMG),
!                            super  night  flatfields (SUPER_NIGHT_FLAT_IMG),
!                            normalized master screen fflds. (MASTER_NORM_FLAT_
!                                                             [LSS|MOS]);
!                              default: none
!
!             ref    (P2): super recipe mode: "+" (reference frame is
!                          included in P1),
!                          single recipe mode: set of reference frame (ascii
!                          list);
!                          needed frame: CERTIFIED master frame of same type
!                          as given in P1 - the distinction from NEW master
!                          frames is done via the descriptors ESO.PRO.QC.CERTIF;
!                          also needed frame type:
!                            table with purging information (DESC_PURGE_TABLE);
!                              default: none
!
!             out    (P3): single recipe mode: basename for ascii list with
!                          output frame,
!                          super recipe mode: again set of input frames and
!                          existing image catalogue (".cat"), separated by a
!                          comma;
!                          table for input set:
!                            master statistics (TSTMASTER_TABLE);
!                              default: `PROID' defined in proc. (single mode)
!
!             type   (P4): ID-string of frames to be tested (superrecipes);
!                          <+>:type will be determined from first found
!                          CERTIFIED master frame (of current obs. mode) in P2;
!                              default: +
!
!             thresh (P5): median+FACTOR*mediansigma -> outlier limit;
!                              default: 7.
!
!
! .OUTPUT     "<PROID>_tst.tfits":  table with statistics
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Comparisons between
!             each binned difference frame "(certif.master-scalefactor*
!               new_master)", with the following scalefactors: for bias none,
!               for darks exp.time(certif.master)/exp.time(new_master), for
!               flats median(certif.master)/median(new_master),
!             and ideal zero values:
!               - mean (64x64 bins)
!               - standard deviation (64x64 bins)
!               - standard deviation (2x2 bins = 4 quadrants)
!             A FACTOR for calculating the outlier limit is needed: (median
!               value of the frame + this FACTOR * the median sigma of the
!               frame) yields the limit for detecting outliers;"
!
! .COMMENT    -
!
! .ENVIRON    FORS context
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/03  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref in ref out type thresh
!
!
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
define/local PROID/c/1/40 "gen_qc1_tstmaster"
!
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      in     (P1): set of NEW created master frames (ascii list);"
  w/o "                   possible frame types:"
  w/o "                     master bias frames (MASTER_BIAS),"
  w/o "                     master dark frames (MASTER_DARK),"
  w/o "                     master screen flatfields (MASTER_SCREEN_FLAT_IMG)"
  w/o "                     master sky    flatfields (MASTER_SKY_FLAT_IMG),"
  w/o "                     super  sky    flatfields (SUPER_SKY_FLAT_IMG),"
  w/o "                     master night  flatfields (MASTER_NIGHT_FLAT_IMG),"
  w/o "                     super  night  flatfields (SUPER_NIGHT_FLAT_IMG),"
  w/o "                     normalized master screen fflds. (MASTER_NORM_FLAT_"
  w/o "                                                      [LSS|MOS]);"
  w/o "                       default: none"
  w/o " "
  w/o "      ref    (P2): super recipe mode: "+" (reference frames are"
  w/o "                   included in P1),"
  w/o "                   single recipe mode: set of reference frames (ascii"
  w/o "                   list);"
  w/o "                   needed frame: CERTIFIED master frame of same type"
  w/o "                   as given in P1 - the distinction from NEW master"
  w/o "                   frames is done via the descriptors ESO.PRO.QC.CERTIF;"
  w/o "                   also needed frame type:"
  w/o "                     table with purging information (DESC_PURGE_TABLE);"
  w/o "                       default: none"
  w/o " "
  w/o "      out    (P3): single recipe mode: basename for ascii list with"
  w/o "                   output frame,"
  w/o "                   super recipe mode: again set of input frames and"
  w/o "                   existing image catalogue (".cat"), separated by a"
  w/o "                   comma;"
  w/o "                   table for input set:"
  w/o "                     master statistics (TSTMASTER_TABLE);"
  w/o "                       default: `PROID' defined in proc. (single mode)"
  w/o " "
  w/o "      type   (P4): ID-string of frames to be tested (superrecipes);"
  w/o "                   <+>:type will be determined from first found"
  w/o "                   CERTIFIED master frame (of current obs. mode) in P2;"
  w/o "                       default: +"
  w/o " "
  w/o "      thresh (P5): median+FACTOR*mediansigma -> outlier limit;"
  w/o "                       default: 7."
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          PREPARATIONS FOR RECIPE :
!          ***********************
!
log/recipe {PROID} in
!
        mess/out 3 {PROID} "Compare new master frame(s) with existing one"
!
@p fors_paragraph 1 1  ! -------- INITIALIZATION --------
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(in):
!------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter name of input frame set: "
define/local inpipe_frm/c/1/76 "{p1}"
!
if m$exist(inpipe_frm) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered list is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
!
define/local filecode/i/1/2 0 all
open/file {inpipe_frm} read filecode
if filecode(1) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open entered list !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
read/file {filecode(1)} outputc
if filecode(2) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered list is empty !"
        mess/out 1 {PROID} "({inpipe_frm})"
  close/file {filecode(1)}
  goto error
endif
close/file {filecode(1)}
!
!
!P2(ref):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter name of reference frame set: "
define/local inpipe_ref/c/1/76 "{p2}"
!
! No tests like for P1 - in super recipes reference frame is included in P1 .
!
!
!P3(out):
!-------
define/parameter p3 {PROID} ? -
  "        Enter basename of out.list(single)/exist.cat.".cat"(super): "
define/local outpipe/c/1/76 "{p3}"
!
!
!P4(type):
!--------
define/parameter p4 + ? -
  "        Enter frame ID-string [def:<+>:auto|(see header)]:"
define/local type/c/1/25 "{p4}"
!
!
!P5(thresh):
!----------
define/parameter p5 7. n -
  "        Enter threshold factor for outliers : "
define/local thresh/r/1/1 {p5}
if thresh .le. 0 then
        write/out " "
        mess/out 1 {PROID} "Impossible thresh.fact. for outliers !"
  goto error
endif
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Getting input frames":
!
  define/local par4/c/1/60 " " all
!
!     "Identifying input frames":
!
  define/local found/i/1/1 0
  define/local catal/i/1/1 0
  define/local topicima/c/1/76 " " all
  define/local id/c/1/25 " " all
  !
  define/local mID/c/1/25 " " all
  define/local typidx/i/1/1 0
  !
  define/local cert/c/1/76 " " all
  !
  define/local in/i/1/1 0
  define/local ref/i/1/1 0
  !
  define/local insdesc/c/1/60 " " all
  !
  define/local psc/i/1/1 0
  define/local osc/i/1/1 0
  define/local binfact/i/1/1 0
!
!
!          LOADING OF GLOBAL KEYS (SINGLE RECIPE MODE) ,
!          CONVERSION FITS->BDF/TBL ,
!          CLASSIFICATION OF INPUT FRAMES :
!          *********************************************
!
        mess/out 4 {PROID} "Getting input frames"
!
initialize/pipeline {inpipe_frm} {inpipe_ref} -
  inpipe_good.cat ? ? + inpipe_bad.cat ?
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Pipeline initialization failed !"
  goto error
endif
-delete inpipe_bad.cat
!
if type .ne. "+" then
  par4 = type
else
  par4 = "from first found CERTIFIED master frame"
endif
!
! Initialize product header:
set/format F5.2,
init/prohead {inpipe_frm} {inpipe_ref} -
  2,"Compare new master frame(s) with existing one" -
  type,"{par4}" thresh,{thresh}
set/format {FMTDEF}
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Product header initialization failed !"
  goto error
endif
!
set/format T6.2,
initialize/longlog {inpipe_frm} {inpipe_ref} 2 -
                   "frame type: {par4}" -
                   "thresholds: {thresh}"
set/format {FMTDEF}
!
!
!          IDENTIFICATION VIA DESCRIPTORS ,
!          MAPPING OF FRAMES TO PARAMETERS  :
!          ********************************
!
        mess/out 4 {PROID} "Identifying input and reference frames"
!
found = 0
if type .eq. "+" then
  autoID:  ! First found certified master frame (of current
           ! observation mode) determines type of input frames !
    store/frame topicima inpipe_good.cat 1 end_autoID
    if m$existd(topicima,h_proqccert) .gt. 0 then
      !
      if m$index("{{topicima},{h_proqccert}}","CERTIFIED") .gt. 0 then
        id = "{{topicima},{fors_idstrng}}"
        if id .eq. "{MBIAS_ID}" found = 1
        if id .eq. "{MDARK_ID}" found = 2
        if id .eq. "{MFLAT_ID}" found = 3
        if id .eq. "{MSKY_ID}" found = 3
        if id .eq. "{SSKY_ID}" found = 3
        if id .eq. "{MNIGHT_ID}" found = 3
        if id .eq. "{SNIGHT_ID}" found = 3
        if id .eq. "{MNFLAT_ID}" found = 3
        if found .ne. 0 then
          mID = id
          goto end_autoID
        endif
      endif
      !
    endif
    goto autoID
  end_autoID:
  if found .eq. 0 goto noref
else
  mID = type
  if mID .eq. "{MBIAS_ID}" found = 1
  if mID .eq. "{MDARK_ID}" found = 2
  if mID .eq. "{MFLAT_ID}" found = 3
  if mID .eq. "{MSKY_ID}" found = 3
  if mID .eq. "{SSKY_ID}" found = 3
  if mID .eq. "{MNIGHT_ID}" found = 3
  if mID .eq. "{SNIGHT_ID}" found = 3
  if mID .eq. "{MNFLAT_ID}" found = 3
  if found .eq. 0 then
        write/out " "
        mess/out 2 {PROID} "Entered frame ID not supported !"
    warn = 1
    goto do_nothing
  endif
endif
!
typidx = found  ! needed for calling of main procedure
!
create/icat {PROID}.cat null {FORS_IDSTRNG} >Null
in = 0
ref = 0
!
catal = 0
ident_loop:
  store/frame topicima inpipe_good.cat 1 end_ident_loop
  id = "{{topicima},{fors_idstrng}}"
  !
  if id .eq. "{mID}" then
    if m$existd(topicima,h_proqccert) .gt. 0 then
      !
      if m$index("{{topicima},{h_proqccert}}","CERTIFIED") .gt. 0 then
        cert = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
        ref = 1
      else
        add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
        in = in+1
      endif
      !
    else
        add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
        in = in+1
    endif
  endif
  !
  goto ident_loop
end_ident_loop:
-delete inpipe_good.cat
!
if in .eq. 0 then
        write/out " "
        mess/out 2 {PROID} "No matching input frame present !"
        mess/out 2 {PROID} "(new master or super frame)"
  warn = 1
  goto do_nothing
endif
!
if ref .eq. 0 then
noref:
        write/out " "
        mess/out 2 {PROID} "No matching reference frame present !"
        mess/out 2 {PROID} "(certified master or super frame)"
  warn = 1
  goto do_nothing
endif
!
insdesc = "{#1_{PROID}.cat,{h_instrume}}"
!
!
!          DETERMINING OVERSCAN SECTIONS :
!          *****************************
!
@p fors_paragraph 2 1  ! -------- PROCESSING --------
!
!*******************************
@p fors_get_imsec #1_{PROID}.cat
!*******************************
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Determination of overscan sections failed !"
  goto error
endif
psc = outputi(5)
osc = outputi(6)
binfact = outputi(9)
!
!
!          *********************************
!          *                               *
!          *  CALLING OF MAIN PROCEDURE :  *
!          *                               *
!          *********************************
!
define/local MEDDESC/c/1/30 "{h_promedi}" ? +lower_levels
!
        mess/out 4 {PROID} "Applying procedure  "fors_tstmaster" :"
!
!*****************************************************************************
@p fors_tstmaster {PROID},{binfact} {cert} {typidx} p ? {psc},{osc} ? {thresh}
!*****************************************************************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Statistical tests on master frames failed !"
  goto error
endif
!
create/icat outprog.cat null >Null
add/icat outprog.cat {PROID}_tst.tbl >Null
-delete dirfile.ascii
!
!
!          UPDATING DESCRIPTORS OF OUT- AND INPUT FRAMES :
!          *********************************************
!
@p fors_paragraph 3 1  ! -------- TERMINATION --------
!
        mess/out 4 {PROID} "Classifying output table ({PROID}_tst.tbl)"
!
classify/product {PROID}_tst.tbl {TMASTERT_ID} {d_qcp} {d_empty} OK y
write/descriptor {PROID}_tst.tbl {h_instrume}/c/1/60 {insdesc}
write/descriptor {PROID}_tst.tbl {FORS_IDSTRNG}/c/1/60 {TMASTERT_ID} !f.su.rec.
write/descriptor {PROID}_tst.tbl {h_proqcsts}/c/1/9 "CHECKED"
write/dhelp {PROID}_tst.tbl {h_proqcsts} "{i_proqcsts}"
longlog/out 1 "     Table with statistics: {PROID}_tst.tbl"
!
        mess/out 4 {PROID} "Endorsing QC status in input image(s)"
!
catal = 0
desc_loop_in:
  store/frame topicima {PROID}.cat 1 end_desc_loop_in
  !
  write/descriptor {topicima} {h_proqcsts}/c/1/9 "CHECKED"
  write/dhelp {topicima} {h_proqcsts} "{i_proqcsts}"
  !
  goto desc_loop_in
end_desc_loop_in:
!
shortlog/out 1 "Determ. diff. betw. new and certif. master frm."
!
!
!          UPDATING OF OUTPUT CATALOGUE (SUPER RECIPE MODE) ,
!          CONVERSION BDF/TBL->FITS/TFITS (SINGLE MODE) :
!          **************************************************
!
        mess/out 4 {PROID} "Releasing output table"
!
! Create product header:
finish/prohead outprog.cat 2
!
finish/pipeline outprog.cat {outpipe} + {PROID}_tst.tbl
!
finish/longlog {PROID}.cat outprog.cat {PROID}_tst.tbl 2 {outpipe}
!
-delete {PROID}.cat
-delete outprog.cat
!
!
!          RESETTINGS AFTER RECIPE :
!          ***********************
!
set/format
!
log/recipe "{PROID}" out
!
@p fors_paragraph 4 1  ! ================
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  finish/longlog ? -1 ? 2
  write/out " "
  return 1
!
do_nothing:
  set/format
  finish/longlog ? -1 ? 2
  write/out " "
  log/recipe "{PROID}" out
  @p fors_paragraph 4 1  ! ================
  return -1
!
!
!-------------------------------------------------------------------------------
