! $Id: fors_gen_tec_ccdcte.prg,v 1.57 2000/02/22 12:58:52 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_gen_tec_ccdcte.prg,v $
!
! .KEYWORDS   Detector: transfer
!
! .PURPOSE    Pipeline wrapper and frame classification for
!             fors_ccdcte.prg:
!             Determine horizontal CCD charge transfer efficiency
!             (vertical not possible because of nonexistent overscan pixels).
! __________
!
! .USAGE      @p fors_gen_tec_ccdcte in ref [out]
!
! .INPUT      in   (P1): set of input frames (ascii list);
!                        needed frames:
!                          at least one RAW high level screen flatfield frame
!                          WITH OVERSCAN AREA (SCREEN_FLAT_IMG);
!                            default: none
!
!             ref  (P2): super recipe mode: "+" (reference frame is included
!                        in P1),
!                        single recipe mode: set of reference frame (ascii
!                        list);
!                        optional frame:
!                          bad pixel table (BAD_PIXEL_TABLE);
!                        also needed frame type:
!                          table with purging information (DESC_PURGE_TABLE);
!                            default: none
!
!             out  (P3): single recipe mode: basename for ascii list with
!                        output frame,
!                        super recipe mode: again set of input frames and
!                        existing image catalogue (".cat"), separated by a
!                        comma;
!                        table for input set:
!                          charge transfer efficiencies (CCDCTE_TABLE);
!                            default: `PROID' defined in procedure (single mode)
!
! .OUTPUT     "<PROID>_cte.tfits":  table with charge transfer efficiencies
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Extended Pixel Edge Response test (Janesick 1987)
!
! .COMMENT    -
!
! .ENVIRON    FORS context
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/28  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref in ref out
!
!
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
define/local PROID/c/1/40 "gen_tec_ccdcte"
!
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      in   (P1): set of input frames (ascii list);"
  w/o "                 needed frames:"
  w/o "                   at least one RAW high level screen flatfield frame"
  w/o "                   WITH OVERSCAN AREA (SCREEN_FLAT_IMG);"
  w/o "                     default: none"
  w/o " "
  w/o "      ref  (P2): super recipe mode: "+" (reference frame is included"
  w/o "                 in P1),"
  w/o "                 single recipe mode: set of reference frame (ascii"
  w/o "                 list);"
  w/o "                 needed frame:"
  w/o "                   bad pixel table (BAD_PIXEL_TABLE);"
  w/o "                 also needed frame type:"
  w/o "                   table with purging information (DESC_PURGE_TABLE);"
  w/o "                     default: none"
  w/o " "
  w/o "      out  (P3): single recipe mode: basename for ascii list with"
  w/o "                 output frame,"
  w/o "                 super recipe mode: again set of input frames and"
  w/o "                 existing image catalogue (".cat"), separated by a"
  w/o "                 comma;"
  w/o "                 table for input set:"
  w/o "                   charge transfer efficiencies (CCDCTE_TABLE);"
  w/o "                     default: `PROID' defined in procedure (single mode)"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          PREPARATIONS FOR RECIPE :
!          ***********************
!
log/recipe {PROID} in
!
        mess/out 3 {PROID} "Compute horizon. CCD charge transfer efficiency ..."
!
@p fors_paragraph 1 1  ! -------- INITIALIZATION --------
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(in):
!------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter name of input frame set: "
define/local inpipe_frm/c/1/76 "{p1}"
!
if m$exist(inpipe_frm) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered list is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
!
define/local filecode/i/1/2 0 all
open/file {inpipe_frm} read filecode
if filecode(1) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open entered list !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
read/file {filecode(1)} outputc
if filecode(2) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered list is empty !"
        mess/out 1 {PROID} "({inpipe_frm})"
  close/file {filecode(1)}
  goto error
endif
close/file {filecode(1)}
!
!
!P2(ref):
!-------
define/parameter p2 + ? -
  "        Enter name of reference frame set: "
define/local inpipe_ref/c/1/76 "{p2}"
!
! No tests like for P1 - in super recipes reference frame is included in P1 ;
! furthermore bad pixel table is only an optional reference frame.
!
!
!P3(out):
!-------
define/parameter p3 {PROID} ? -
  "        Enter basename of out.list(single)/exist.cat.".cat"(super): "
define/local outpipe/c/1/76 "{p3}"
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Identifying input frames":
!
  define/local catal/i/1/1 0
  define/local topicima/c/1/76 " " all
  define/local id/c/1/25 " " all
  !
  define/local flatID/c/1/25 " " all
  define/local cteID/c/1/25 " " all
  define/local bpxID/c/1/25 " " all
  !
  define/local bpxtable/c/1/76 " " all
  !
  define/local in/i/1/1 0
  define/local ref/i/1/1 0
  !
  define/local insdesc/c/1/60 " " all
!
!     "Calling of procedure":
!
  define/local psc/i/1/1 0
  define/local osc/i/1/1 0
!
!
!          LOADING OF GLOBAL KEYS (SINGLE RECIPE MODE) ,
!          CONVERSION FITS->BDF/TBL ,
!          CLASSIFICATION OF INPUT FRAMES :
!          *********************************************
!
        mess/out 4 {PROID} "Getting input frames"
!
initialize/pipeline {inpipe_frm} {inpipe_ref} -
  inpipe_good.cat ? ? + inpipe_bad.cat ?
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Pipeline initialization failed !"
  goto error
endif
-delete inpipe_bad.cat
!
! Initialize product header:
init/prohead {inpipe_frm} {inpipe_ref} -
  2,"Compute CCD charge transfer efficiency"
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Product header initialization failed !"
  goto error
endif
!
initialize/longlog {inpipe_frm} {inpipe_ref} 2
!
!
!          IDENTIFICATION VIA DESCRIPTORS ,
!          MAPPING OF FRAMES TO PARAMETERS  :
!          ********************************
!
        mess/out 4 {PROID} "Identifying input frames"
!
flatID = "{FLAT_ID}"
cteID = "{CCDCTE_ID}"
bpxID = "{BPIXT_ID}"
!
create/icat {PROID}.cat null {FORS_IDSTRNG} >Null
in = 0
ref = 0
!
ident_loop:
  store/frame topicima inpipe_good.cat 1 end_ident_loop
  id = "{{topicima},{fors_idstrng}}"
  !
  if id .eq. "{flatID}" .or. id .eq. "{cteID}" then
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    in = 1
  elseif id .eq. "{bpxID}" then
    bpxtable = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    ref = 1
  endif
  !
  goto ident_loop
end_ident_loop:
-delete inpipe_good.cat
!
if in .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching input screen-flatf. frame present !"
  goto error
endif
!
!if ref .eq. 0 then
!        write/out " "
!        mess/out 1 {PROID} "No bad pixel table present !"
!  goto error
!endif
!
insdesc = "{#1_{PROID}.cat,{h_instrume}}"
!
!
!          REMOVING BAD PIXELS :
!          *******************
!
@p fors_paragraph 2 1  ! -------- PROCESSING --------
!
if ref .ne. 0 then
  !
        mess/out 4 {PROID} "Removing bad pixels from input frames"
  !
  !***************************************
  @p fors_corr_bpxl {bpxtable} {PROID}.cat
  !***************************************
  !
endif
!
!
!          DETERMINING OVERSCAN SECTIONS :
!          *****************************
!
!*******************************
@p fors_get_imsec #1_{PROID}.cat
!*******************************
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Determination of overscan sections failed !"
  goto error
endif
psc = outputi(5)
osc = outputi(6)
!
if osc .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Needed overscan area not present !"
  goto error
endif
!
!
!          *********************************
!          *                               *
!          *  CALLING OF MAIN PROCEDURE :  *
!          *                               *
!          *********************************
!
        mess/out 4 {PROID} "Applying procedure  "fors_ccdcte" :"
!
!***********************************
@p fors_ccdcte {PROID} {psc},{osc} p
!***********************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Comput. of charge trans. effic. failed !"
  goto error
endif
!
create/icat outprog.cat null >Null
add/icat outprog.cat {PROID}_cte.tbl >Null
-delete dirfile.ascii
!
!
!          UPDATING DESCRIPTORS OF OUTPUT FRAMES :
!          *************************************
!
@p fors_paragraph 3 1  ! -------- TERMINATION --------
!
        mess/out 4 {PROID} "Classifying output table ({PROID}_cte.tbl)"
!
classify/product {PROID}_cte.tbl {CCDCTET_ID} {d_pre} {d_empty} OK y
write/descriptor {PROID}_cte.tbl {h_instrume}/c/1/60 {insdesc}
write/descriptor {PROID}_cte.tbl {FORS_IDSTRNG}/c/1/60 {CCDCTET_ID} !f.su.rec.
longlog/out 1 "     CT efficiency table: {PROID}_cte.tbl"
shortlog/out 1 "Computed horizontal charge transfer efficiency"
!
!
!          UPDATING OF OUTPUT CATALOGUE (SUPER RECIPE MODE) ,
!          CONVERSION BDF/TBL->FITS/TFITS (SINGLE MODE) :
!          **************************************************
!
        mess/out 4 {PROID} "Releasing output table"
!
! Create product header:
finish/prohead outprog.cat 2
!
finish/pipeline outprog.cat {outpipe} + {PROID}_cte.tbl
!
finish/longlog {PROID}.cat outprog.cat {PROID}_cte.tbl 2 {outpipe}
!
-delete {PROID}.cat
-delete outprog.cat
!
!
!          RESETTINGS AFTER RECIPE :
!          ***********************
!
set/format
!
log/recipe {PROID} out
!
@p fors_paragraph 4 1  ! ================
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  finish/longlog ? -1 ? 2
  write/out " "
  return 1
!
!
!-------------------------------------------------------------------------------
