! $Id: fors_gen_tec_focus.prg,v 1.52 2001/04/10 09:49:23 sbogun Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_gen_tec_focus.prg,v $
!
! .KEYWORDS   Focal alignment
!
! .PURPOSE    Pipeline wrapper and frame classification for
!             fors_focus.prg:
!             Determines best focus position of a set of focus frames
!             by calculating smallest FWHM of the mapped pinhole.
! __________
!
! .USAGE      @p fors_gen_tec_focus in ref [out]
!
! .INPUT      in   (P1): set of input frames (ascii list);
!                        needed frames:
!                          at least 5 focus images (FOCUS_IMG);
!                            default: none
!
!             ref  (P2): super recipe mode: "+" (reference frames are included
!                        in P1),
!                        single recipe mode: set of reference frames (ascii
!                        list);
!                        needed frames:
!                          table with SExtractor input parameters
!                          (PHOTOMETRY_INPUT_PAR),
!                          table with SExtractor convolution mask
!                          (PHOTOMETRY_CONVOL_MASK),
!                          table with SExtractor output parameters
!                          (PHOTOMETRY_OUTPUT_PAR),
!                          table with neuronal network weights
!                          (PHOTOMETRY_NEURON_PAR);
!                        optional:
!                          bad pixel table (BAD_PIXEL_TABLE);
!                        also needed frame type:
!                          table with purging information (DESC_PURGE_TABLE);
!                            default: none
!
!             out  (P3): single recipe mode: basename for ascii list with
!                        output frame,
!                        super recipe mode: again set of input frames and
!                        existing image catalogue (".cat"), separated by a
!                        comma;
!                        table for input set:
!                          focal statistics (FOCUS_TABLE);
!                            default: `PROID' defined in procedure (single mode)
!
! .OUTPUT     "<P1>_foc.tfits": table with (smallest) FWHM(s) and (best) focal
!                               position(s).
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  SExtractor v2.0.15 - see corresponding documents.
!
! .COMMENT    The ascii files with input parameters and output parameters (P2)
!             must NOT be changed; otherwise "fors_focus.prg" must be al-
!             tered partially (see comment in header of fors_focus.prg) !!!
!
! .ENVIRON    FORS context
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/10/19  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref in ref out
!
!
define/local MINOBJ/i/1/1 5 ? +lower_levels ! MUST BE >= 3 !!!
!
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
define/local PROID/c/1/40 "gen_tec_focus"
!
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      in   (P1): set of input frames (ascii list);"
  w/o "                 needed frames:"
  w/o "                   at least 5 focus images (FOCUS_IMG);"
  w/o "                     default: none"
  w/o " "
  w/o "      ref  (P2): super recipe mode: "+" (reference frames are included
  w/o "                 in P1),
  w/o "                 single recipe mode: set of reference frames (ascii
  w/o "                 list);
  w/o "                 needed frames:
  w/o "                   table with SExtractor input parameters
  w/o "                   (PHOTOMETRY_INPUT_PAR),
  w/o "                   table with SExtractor convolution mask
  w/o "                   (PHOTOMETRY_CONVOL_MASK),
  w/o "                   table with SExtractor output parameters
  w/o "                   (PHOTOMETRY_OUTPUT_PAR),
  w/o "                   table with SExtractor output parameters"
  w/o "                   (PHOTOMETRY_NEURON_PAR);"
  w/o "                 optional:"
  w/o "                   bad pixel table (BAD_PIXEL_TABLE);
  w/o "                 also needed frame type:"
  w/o "                   table with purging information (DESC_PURGE_TABLE);"
  w/o "                   default: none"
  w/o " "
  w/o "      out  (P3): single recipe mode: basename for ascii list with"
  w/o "                 output frame,"
  w/o "                 super recipe mode: again set of input frames and"
  w/o "                 existing image catalogue (".cat"), separated by a"
  w/o "                 comma;"
  w/o "                 table for input set:"
  w/o "                   focal statistics (FOCUS_TABLE);"
  w/o "                     default: `PROID' defined in procedure (single mode)"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          PREPARATIONS FOR RECIPE :
!          ***********************
!
log/recipe {PROID} in
!
        mess/out 3 {PROID} "Compute best focus position ..."
!
@p fors_paragraph 1 1  ! -------- INITIALIZATION --------
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(in):
!------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter name of input frame set: "
define/local inpipe_frm/c/1/76 "{p1}"
!
if m$exist(inpipe_frm) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered list is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
!
define/local filecode/i/1/2 0 all
open/file {inpipe_frm} read filecode
if filecode(1) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open entered list !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
read/file {filecode(1)} outputc
if filecode(2) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered list is empty !"
        mess/out 1 {PROID} "({inpipe_frm})"
  close/file {filecode(1)}
  goto error
endif
close/file {filecode(1)}
!
!
!P2(ref):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter name of reference frame set: "
define/local inpipe_ref/c/1/76 "{p2}"
!
! No tests like for P1 - in super recipes reference frames are included in P1 .
!
!
!P3(out):
!-------
define/parameter p3 {PROID} ? -
  "        Enter basename of out.list(single)/exist.cat.".cat"(super): "
define/local outpipe/c/1/76 "{p3}"
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Identifying input frames":
!
  define/local catal/i/1/1 0
  define/local topicima/c/1/76 " " all
  define/local id/c/1/25 " " all
  !
  define/local focID/c/1/25 " " all
  define/local sexiID/c/1/25 " " all
  define/local sexcID/c/1/25 " " all
  define/local sexoID/c/1/25 " " all
  define/local sexnID/c/1/25 " " all
  define/local bpxID/c/1/25 " " all
  !
  define/local inpartbl/c/1/76 " " all
  define/local convoltbl/c/1/76 " " all
  define/local outpartbl/c/1/76 " " all
  define/local neupartbl/c/1/76 " " all
  define/local bpxtable/c/1/76 " " all
  !
  define/local in/i/1/1 0
  define/local refsex/i/1/1 0
  define/local refbpx/i/1/1 0
  !
  define/local insdesc/c/1/60 " " all
!
!     "Conversion of ... tables ...":
!
  define/local totrowsin/i/1/1 0
  define/local row/i/1/1 0
  define/local totrowscon/i/1/1 0
  define/local totrowsout/i/1/1 0
!
!     "Calling of procedure":
!
  define/local scl/c/1/60 " " all
  define/local foc/c/1/60 " " all
!
!
!          LOADING OF GLOBAL KEYS (SINGLE RECIPE MODE) ,
!          CONVERSION FITS->BDF/TBL ,
!          CLASSIFICATION OF INPUT FRAMES :
!          *********************************************
!
        mess/out 4 {PROID} "Getting input frames"
!
initialize/pipeline {inpipe_frm} {inpipe_ref} -
  inpipe_good.cat ? ? + inpipe_bad.cat ?
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Pipeline initialization failed !"
  goto error
endif
-delete inpipe_bad.cat
!
! Initialize product header:
init/prohead {inpipe_frm} {inpipe_ref} 2,"Compute best focus position"
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Product header initialization failed !"
  goto error
endif
!
initialize/longlog {inpipe_frm} {inpipe_ref} 2
!
!
!          IDENTIFICATION VIA DESCRIPTORS ,
!          MAPPING OF FRAMES TO PARAMETERS  :
!          ********************************
!
        mess/out 4 {PROID} "Identifying input and reference frames"
!
focID = "{FOCUS_ID}"
sexiID = "{SEXINPAR_ID}"
sexcID = "{SEXCONVOL_ID}"
sexoID = "{SEXOUTPAR_ID}"
sexnID = "{SEXNEUPAR_ID}"
bpxID = "{BPIXT_ID}"
!
create/icat {PROID}.cat null {FORS_IDSTRNG} >Null
in = 0
refsex = 0
refbpx = 0
!
ident_loop:
  store/frame topicima inpipe_good.cat 1 end_ident_loop
  id = "{{topicima},{fors_idstrng}}"
  !
  if id .eq. "{focID}" then
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    insdesc = "{{topicima},{h_instrume}}"
    in = in+1
  elseif id .eq. "{sexiID}" then
    inpartbl = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refsex = refsex+1
  elseif id .eq. "{sexcID}" then
    convoltbl = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refsex = refsex+1
  elseif id .eq. "{sexoID}" then
    outpartbl = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refsex = refsex+1
  elseif id .eq. "{sexnID}" then
    neupartbl = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refsex = refsex+1
  elseif id .eq. "{bpxID}" then
    bpxtable = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refbpx = 1
  endif
  !
  goto ident_loop
end_ident_loop:
-delete inpipe_good.cat
!
if in .lt. MINOBJ then
        write/out " "
        mess/out 1 {PROID} "Too few (<{MINOBJ}) input focus frames present !"
  goto error
endif
!
if refsex .lt. 4 then
        write/out " "
        mess/out 1 {PROID} "Not enough reference frames present :"
  !
  if m$len(inpartbl) .eq. 0 then
        mess/out 1 {PROID} "No reference file with sextractor"
        mess/out 1 {PROID} "input parameters present !"
  endif
  !
  if m$len(convoltbl) .eq. 0 then
        mess/out 1 {PROID} "No reference file with sextractor"
        mess/out 1 {PROID} "convolution mask present !"
  endif
  !
  if m$len(outpartbl) .eq. 0 then
        mess/out 1 {PROID} "No reference file with sextractor"
        mess/out 1 {PROID} "output parameters present !"
  endif
  !
  if m$len(neupartbl) .eq. 0 then
        mess/out 1 {PROID} "No reference file with sextractor"
        mess/out 1 {PROID} "neuronal network weights present !"
  endif
  !
  goto error
endif
!
!if refbpx .eq. 0 then
!        write/out " "
!        mess/out 1 {PROID} "No bad pixel table present !"
!  goto error
!endif
!
!
!          CONVERSION OF REFERENCE TABLES
!          TO ASCII  :
!          ******************************
!
        mess/out 4 {PROID} "Converting reference tables to ascii"
!
!***************************
@p fors_tbl2ascii {inpartbl}
!***************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Table conversion failed !"
        mess/out 1 {PROID} "({inpartbl})"
  goto error
endif
if q2 .ne. "sex__inpar.dat" -rename {q2} sex__inpar.dat  ! def.f. `fors_focus'
!
!****************************
@p fors_tbl2ascii {convoltbl}
!****************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Table conversion failed !"
        mess/out 1 {PROID} "({convoltbl})"
  goto error
endif
if q2 .ne. "sex_convol.dat" -rename {q2} sex_convol.dat
!
!****************************
@p fors_tbl2ascii {outpartbl}
!****************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Table conversion failed !"
        mess/out 1 {PROID} "({outpartbl})"
  goto error
endif
if q2 .ne. "sex_outpar.dat" -rename {q2} sex_outpar.dat
!
!****************************
@p fors_tbl2ascii {neupartbl}
!****************************
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Table conversion failed !"
        mess/out 1 {PROID} "({neupartbl})"
  goto error
endif
if q2 .ne. "sex_neupar.dat" -rename {q2} sex_neupar.dat
!
!
!          REMOVING BAD PIXELS :
!          *******************
!
@p fors_paragraph 2 1  ! -------- PROCESSING --------
!
if refbpx .ne. 0 then
  !
        mess/out 4 {PROID} "Removing bad pixels from input frames"
  !
  !***************************************
  @p fors_corr_bpxl {bpxtable} {PROID}.cat
  !***************************************
  !
endif
!
!
!          *********************************
!          *                               *
!          *  CALLING OF MAIN PROCEDURE :  *
!          *                               *
!          *********************************
!
scl = h_pixscale
foc = h_focpos
!
        mess/out 4 {PROID} "Applying procedure  "fors_focus" :"
!
!************************************
@p fors_focus {PROID} ? {scl},{foc} p
!************************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Focus determination failed !"
  goto error
endif
!
create/icat outprog.cat null >Null
add/icat outprog.cat {PROID}_foc.tbl >Null
-delete dirfile.ascii
!
!
!          UPDATING DESCRIPTORS OF OUTPUT FRAMES :
!          *************************************
!
@p fors_paragraph 3 1  ! -------- TERMINATION --------
!
        mess/out 4 {PROID} "Classifying output table ({PROID}_foc.tbl)"
!
classify/product {PROID}_foc.tbl {FOCT_ID} {d_pre} {d_empty} OK y
write/descriptor {PROID}_foc.tbl {h_instrume}/c/1/60 {insdesc}
write/descriptor {PROID}_foc.tbl {FORS_IDSTRNG}/c/1/60 {FOCT_ID} !f.su.rec.
longlog/out 1 "     Focus table: {PROID}_foc.tbl"
shortlog/out 1 "Determined best focus position"
!
!
!          UPDATING OF OUTPUT CATALOGUE (SUPER RECIPE MODE) ,
!          CONVERSION BDF/TBL->FITS/TFITS (SINGLE MODE) :
!          **************************************************
!
        mess/out 4 {PROID} "Releasing output table"
!
! Create product header:
finish/prohead outprog.cat 2
!
finish/pipeline outprog.cat {outpipe} + {PROID}_foc.tbl
!
finish/longlog {PROID}.cat outprog.cat {PROID}_foc.tbl 2 {outpipe}
!
-delete {PROID}.cat
-delete outprog.cat
!
!
!          RESETTINGS AFTER RECIPE :
!          ***********************
!
set/format
!
log/recipe {PROID} out
!
@p fors_paragraph 4 1  ! ================
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  finish/longlog ? -1 ? 2
  write/out " "
  return 1
!
!
!-------------------------------------------------------------------------------
