! $Id: fors_img_cal_align.prg,v 1.57 2000/12/01 16:25:02 sbogun Exp $   
!+++ img_cal_align ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         $RCSfile: fors_img_cal_align.prg,v $
!
!.KEYWORDS      Get reference brightnesses of a list of standard stars
!
!.USAGE         @p fors_img_cal_align sof_in sof_ref out accuracy
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!                             contains a MIDAS table standard star measurements
!               sof_ref (P2): set of reference frames (ascii list)
!                             contains a MIDAS table with reference std. star
!                             measurements, the zero point table and the
!                             extinction coeffincient table
!               accur   (P4): pointing accuracy of the telescope
!
!.OUTPUT        out     (P3): path + basename of output file:
!                             Extended copy of the REFERENCE table: the
!                             measurment, sky, seeing, and airmass columns of
!                             the INPUT table are copied to the refer.table.
!
!.RETURN        Q1:  0: successful return
!                    otherwise: a successful return failed
!
!.DEFAULT       P3:     stdali
!               P4:     4.0
!
!.PURPOSE       Get reference brightnesses of a list of standard stars
!
!.ALGORITHM     Compare the coordinates of the standard star measurements with
!               the ones of the reference std.star catalog. Only coinciding
!               entries will be copied to the OUTPUT table. Additionally, the
!               magnitudes, sky-background, seeing, and airmass from the INPUT
!               table will be stored in the OUTPUT table.
!               This table may finally be passed to the calcoeff recipe.
!                 The zero point and extincition tables of the set of reference
!               frames will be used to make a first guess of the measured mag-
!               nitudes in order to be able to identify the standard star after
!               multiple detections. If the zero point table or the extinction
!               table is missing default values will be used which had been
!               derived from La Silla Observatory.
!
!.ENVIRON       FORS, CCDRED context
!               use global keyword: PIPELINE
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.9 1998/03/09  Creation
!               1.0 1998/10/28  Output improved, comments
!                   1998/10/29  classify/product included
!                               main product specified: aligned photom. table
!                   1998/12/09  fors_paragraph implemented
!                   1999/01/07  shortlog implemented.
!               1.2 1999/02/26  NICE_HCOPY implemented.
!                               CHCK_MAGS improved: divide by (S2N*NPIX)^2
!                               (S2N may be -999 in case it could not be
!                                determined through fors_photometry!
!                                S2N: signal/noise, NPIX: num_of_pixel/obj.)
!                   1999/03/10  shortlog close shifted to FINISH/PIPELINE
!               1.3 1999/04/28  longlog implemented (SB).
!                   1999/09/03  Product header generation implemented (SB).
!                   1999/09/08  Error with "c0/c1" correc., outp. improved (NR).
!                   1999/09/09  Longlog entry added, output corrected (NR).
!                   1999/09/09  1st plot/ali offset used for 2nd plot/ali (NR).
!
!.COMMENT       Used UNIX command (NR):
!               $ /bin/echo (as long as mess/out is not yet in monitor)
!
!------------------------------------------------------------------------------

define/local line/c/1/79 "-" ALL
write/out {line}
LOG/REC fors_img_cal_align IN
!
! Check parameters
!
define/param p1 ? c "Enter name of ASCII filelist with std-*-measurements:"
define/param p2 ? c "Enter name of ASCII filelist with reference measurements:"
define/param p3 stdali c "Enter base-name of result table (aligned table):"
define/param p4 4.0 n "Enter pointing accuracy (default=18.0 arcsec):"
!
! constants
!
define/local DR/d/1/1 {p4}                              !accuracy in arcsec
define/local OFF/i/1/1 3                                !skip last {OFF} rows
define/local UBVRI/c/1/5 UBVRI ? +lower_level           !allowed filters
define/local extcoef/r/1/5 0.43,0.26,0.13,0.06,0.07 ? +lower_levels
define/local zeropoint/r/1/5 -23.56,-26.15,-26.47,-26.33,-25.54 ? +lower_levels
                                                        !UT1/FORS1 P63 / 1999


!
! local variable declaration
!
define/local i1/i/1/1 1
define/local i2/i/1/1 1
define/local j1/i/1/1 1
define/local k1/i/1/1 1
define/local l1/i/1/1 1
define/local c0/r/1/1 0.
define/local c1/r/1/1 0.
define/local catal/i/1/1 1
define/local num/i/1/1 0
define/local ncol/i/1/1 0
define/local nrow/i/1/1 0
define/local ncol1/i/1/1 0
define/local nval/i/1/1 0
define/local nstar/i/1/1 0
define/local mid$selidx/i/1/5000 0 ALL +lower_levels
define/local igood/i/1/5000 0 ALL
define/local ngood/i/1/1 0
define/local nlit/i/1/1 0
define/local RAin/d/1/1 0.0
define/local DEin/d/1/1 0.0
define/local sDEi/d/1/1 0.0
define/local cDEi/d/1/1 0.0
define/local mindec/d/1/1 0.0
define/local maxdec/d/1/1 0.0
define/local procid/c/1/20 img_cal_align ? +lower_levels
define/local obj/c/1/100 " " ALL
define/local tout/c/1/100 " " ALL +lower_levels
define/local tmea/c/1/100 " " ALL
define/local rmea/c/1/100 " " ALL
define/local tlit/c/1/100 " " ALL
define/local cmagx/c/1/10 " " ALL
define/local refcol/c/1/1 " " ? +lower_levels
define/local iramin/r/1/1 0
define/local rblanks/c/1/52 " " all
define/local c_recm/c/1/20 {c_rect}_mea
define/local c_decm/c/1/20 {c_decl}_mea
rblanks(51:52) = "\r"
define/local clean/c/1/79 " " all
clean(79:79) = "\\\"

mess/out 3 {procid} -
  "Recipe IMG_CAL_ALIGN will align standard star\nobservations with reference catalog."
@p fors_paragraph 1


INITIALIZE/PIPELINE {P1} {P2} alignI.cat + + IMG reject.cat alignO.cat
if {Q1} .gt. 0 return {Q1}

init/prohead {p1} {p2} 2,"Align standard star obs. with reference catalog" -
   "accur,{P4}"
if {Q1} .gt. 0 return 1

init/longlog {p1} {p2} 2 "pointing accuracy: {p4}"
if {Q1} .gt. 0 return 1

@p fors_paragraph 2
search/icat alignI.cat {PHOTT_ID} >Null
if m$len(out_a) .ne. 0 then
  tmea = out_a
else
  mess/out 1 {procid} "Photometry table is missing!"
  return -1
endif

search/icat alignI.cat {STDFLX_ID} >Null
if m$len(out_a) .ne. 0 then
  tlit = out_a
else
  mess/out 1 {procid} "Standard star reference catalog is missing!"
  return -2
endif

check/filter {tmea}                                     !get filter used
if outputi(1) .ne. 1 return -6                          !no valid filter found
refcol = outputc(1:1)
i1 = m$index(UBVRI,refcol)                              !check for filter type
if i1 .le. 0 then                                       !only: UBVRI allowed
  mess/out 1 {procid} "Only {UBVRI} filter are allowed !"
  return -7
endif

search/icat alignI.cat {ZEROPOINT_ID} >Null
if m$len(out_a) .ne. 0 then
  !
  if m$existc(out_a,":{c_izerop}_{refcol}") .le. 0 then
    mess/out 2 {procid} -
      "Col. in ext.coef.tbl. missing: {c_izerop}_{refcol} -\nusing default value !"
    c0 = 0.                                      ! 0. only as flag for below ...
  else
    c0 = m$value({out_a},:{c_izerop}_{refcol},1)
  endif
  !
else
  c0 = 0.                                        ! 0. only as flag for below ...
  append/out "Table with zero point is missing -\n"
  append/out "using default value for better identifications !"
  mess/out 2 {procid}
endif

search/icat alignI.cat {EXTCOEFF_ID} >Null
if m$len(out_a) .ne. 0 then
  !
  if m$existc(out_a,":{c_iextco}_{refcol}") .le. 0 then
    mess/out 2 {procid} -
      "Col. in ext.coef.tbl. missing: {c_iextco}_{refcol} -\nusing default value !"
    c1 = 0.                                      ! 0. only as flag for below ...
  else
    c1 = m$value({out_a},:{c_iextco}_{refcol},1)
  endif
  !
else
  c1 = 0.                                        ! 0. only as flag for below ...
  append/out "Table with extinction coefficient is missing -\n"
  append/out "using default value for better identifications !"
  mess/out 2 {procid}
endif

if m$existc(tmea,":{c_rect}") .le. 0 then
  mess/out 1 {procid} "Table column missing: {c_rect}\nAlignment impossible !"
  return -3
endif
if m$existc(tmea,":{c_decl}") .le. 0 then
  mess/out 1 {procid} "Table column missing: {c_decl}\nAlignment impossible !"
  return -4
endif




tout = "stdali.tbl"
append/out "INPUT:  measurements: {tmea}\n"
append/out "INPUT:  reference:    {tlit}\n"
append/out "OUTPUT: alignment:    {tout}"
mess/out 3 {procid}

rmea = "r{tmea}"
-copy {tmea} {rmea}
name/column {rmea} :{c_rect} :{c_rect}_mea
name/column {rmea} :{c_decl} :{c_decl}_mea

show/table {rmea} >Null
nrow  = outputi(2)                                      !number of rows
nstar = nrow - OFF                                      !number of measurements
if nstar .le. 0 then
  mess/out 1 {procid} "Empty measurement-table !"
  return -5
endif

comput/table {tlit} :OBJECT = collapse(:OBJECT)         !to remove spaces
show/table {tlit} >Null
nlit  = outputi(2)                                      !number of rows
if nlit .le. 0 then
  mess/out 1 {procid} "Empty standard star catalog !"
  return -5
endif

create/colu {rmea} :DIST "arcsec" F9.2 R*8             !distance of standard star
create/colu {rmea} :ZEROPOINT "mag" F9.4 R*8           !night zero point
compute/table {rmea} :DIST = 0.0
compute/table {rmea} :ZEROPOINT = 0.0

!
! Create an output table having the same header as the
! input table (measurements).:
! - Initialize (header),
! - Copy columns of reference,
! - Merge with columns of measurements.
! --> Empty table (no rows).
@p fors_crea_tab {tout} {rmea}  "Created by img_cal_align" 1
@p fors_crea_tab {tout} {tlit}  "Created by img_cal_align" 3
ncol = {Q1}
ncol1 = ncol + 1
@p fors_crea_tab {tout} {rmea}  "Created by img_cal_align" 3
nval = {Q1} - ncol                                      !output table created:
                                                        !. same descriptors as
                                                        !  measurements
                                                        !. same columns as in
                                                        !  mea. and ref.
                                                        !
!check/filter {rmea}                                     !get filter used
!if outputi(1) .ne. 1 return -6                          !no valid filter found
!refcol = outputc(1:1)
!i1 = m$index(UBVRI,refcol)                              !check for filter type
!if i1 .le. 0 then                                       !only: UBVRI allowed
!  mess/out 1 {procid} "Only {UBVRI} filter are allowed !"
!  return -7
!endif

@p fors_niceclear G                                     !Clear graphics
plot/align {tmea} {refcol} 1 N 2 >Null                  !plot measurements
if q2 .ne. "?" then
  iramin = {q2}
else
  iramin = 0
endif

set/format F9.4 I1
mess/out 3 {procid} "Number of measured standard stars [{refcol}]: {nstar}"
mess/out 3 {procid} "Search for coincidences ..."
mess/out 3 {procid} "  assumed pointing accuracy: {DR} arcsec"
DELETE/row {tout} @1..{nrow}                            !delete ALL rows


! Output to longlog
set/format i1
longlog/out 1 "     Table with found objects:         {rmea}"
longlog/out 1 "     (contains {nstar} objects)"
longlog/out 1 "     searched for matches in standard catalog: {tlit}"
longlog/out 1 " "
set/format

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
set/format I1
outputi = m$len("{nlit}")
set/format I{outputi(1)}
set/format F15.10
statistics/table {rmea} :{c_decm} >Null
mindec = outputr(1) - DR / 3600.
maxdec = outputr(2) + DR / 3600.


select/table {tlit} :{c_decl}.gt.{mindec} .and. :{c_decl}.lt.{maxdec} >Null
ngood = outputi
if ngood .eq. 0 goto NOSTAR

do i1 = 1 ngood
  igood({i1}) = mid$selidx({i1})
enddo
select/table {tlit} all

do i2 = 1 ngood 1                                       !loop through measurem.
  i1 = igood({i2})
  !
  !! mess/out too slow (if not yet in monitor) !
  !mess/out -3 {procid} "  catalog star {i1} / {nlit}\r"
  $ /bin/echo -
    "INFO    \0133{procid}\0135:      catalog star {i1} / {nlit}\r\c"
  !
  DEin = m$value({tlit},:{c_decl},{i1})                 !(RA,DEC)
  RAin = m$value({tlit},:{c_rect},{i1})                 !input coordinates
  cDEi = m$cos( DEin )                                  !circle centered on
  sDEi = m$sin( DEin )                                  !get reference inside
                                                        !meas. with radius=DR

! We have to use compute/table although select/table could be a more direct way
! as select/table has accuracy problems as soon as the mathematical expression
! gets too long.
!
  compute/table {rmea} :DIST = {sDEi}*SIN(:{c_decm}) + {cDEi}*COS(:{c_decm})*COS({RAin}-:{c_recm}), 1.0)
  compute/table {rmea} :DIST = ABS(ACOS(MIN(:DIST, 1.0))) * 3600.
  select/table {rmea} :DIST.le.{DR} >Null
  j1 = {Q1}                                             !index of first detect.
  num = outputi(1)                                      !total number of detec.
  if num .gt. 1 then                                    !there are more than
    do l1 = 1 {num}                                     !will be checked later!
      j1 = mid$selidx({l1})
      @p fors_copyrow {tlit} {tout} {i1} {k1} 1 1 {ncol} !SAVE coincidence
      @p fors_copyrow {rmea} {tout} {j1} {k1} 1 {ncol1} {nval}
      k1 = k1 + 1
    enddo
  else if num .eq. 1 then                               !Coincidence found
    @p fors_copyrow {tlit} {tout} {i1} {k1} 1 1 {ncol}  !SAVE coincidence
    @p fors_copyrow {rmea} {tout} {j1} {k1} 1 {ncol1} {nval}
    k1 = k1 + 1
  endif
  !
  select/table {rmea} ALL
enddo                                                   !all measurem. checked
!
!! mess/out too slow (if not yet in monitor) !
!mess/out -3 {procid} "{rblanks}"
$ /bin/echo "{clean}\r\c"
  set/format I1
!

NOSTAR:
k1 = k1 - 1                                             !number of detected
                                                        !"standard stars".
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

if k1 .gt. 1 then
  @p fors_img_cal_align,CHCK_MAGS {tout} {refcol} {c1} {c0}
  k1 = {Q1}                                             !number of identified
endif                                                   !standard stars.


@p fors_paragraph 3

if k1 .gt. 0 then                                       !display result
  if m$existc(tout,":COLOR") .gt. 0 delete/column {tout} :COLOR
  select/table {tout} :{c_rect}.ne.0.0 .AND. :{c_decl}.ne.0.0 >Null
  !
  if mess_level .ge. 3 then
    mess/out 3 {procid} " "
    mess/out 3 {procid} "Identified objects:"
    write/out " "
    read/table {tout} :OBJECT,:MAG_{refcol},:{refcol},:AIRMASS
    write/out " "
  endif
  !
  @p fors_img_cal_align,LONGLOG
  if {q1} .ne. 0 return 1
  !
  plot/align {tout} {refcol} 2 {NICE_HCOPY} 2 + {iramin}
else
  mess/out 1 {procid} "No standard star could be identified !"
  longlog/out 1 "     No standard star could be identified !" >Null
  return -8
endif

-delete {rmea}
add/icat alignO.cat {tout} >Null
classify/product {tout} {PHOTTALI_ID} {d_pre} {d_det} OK
copy/dd {tout} {h_procatg} {tout} {fors_idstrng}
shortlog/out 1 "Alignment with standard star catalog [{refcol}] done."

! Create product header
finish/prohead alignO.cat 2

FINISH/PIPELINE alignO.cat {P3} + {tout}
finish/longlog alignI.cat alignO.cat {tout} 2 {p3}

LOG/REC fors_img_cal_align OUT
@p fors_paragraph 4

return 0



!=== CHCK_MAGS ================================================================

ENTRY CHCK_MAGS
define/param p1 ? c "Enter name of aligned standard star table: "
define/param p2 ? c "Enter reference colour: "
define/param p3 ? n "Enter extinction coefficient: "
define/param p4 ? n "Enter extinction zero point: "

define/local i1/i/1/1 0
define/local j1/i/1/1 0
define/local k1/i/1/1 0
define/local num/i/1/1 0
define/local ncol/i/1/1 0
define/local nrow/i/1/1 0
!define/local mid$selidx/i/1/2 0 all +lower_levels
define/local name/c/1/80 " " all

define/local c0/r/1/1 {p4}
define/local c1/r/1/1 {p3}
define/local eps/r/1/1 0.00001
define/local ra/r/1/1 0.0
define/local dec/r/1/1 0.0

mess/out 3 img_cal_align "Check coincidences ..."
if c1 .eq. 0. .or. c0 .eq. 0. then
  i1 = m$index(UBVRI,p2)
  c1 = extcoef({i1})                            !default extinction coeff.
  c0 = zeropoint({i1})                          !default zero point
endif

set/format F6.3
mess/out 3 img_cal_align "  slope: {c1}  zero_point: {c0}"
set/format

select/table {p1} all >Null
show/table {p1} >Null
ncol = outputi(1)                               !number of columns in table
nrow = outputi(2)                               !number of rows in input table
-copy {p1} wrk.tbl                              !copy
create/column wrk.tbl :DIFF "mag" Z12.5 R*8
create/column wrk.tbl :C_DIFF "mag" A20 C*20
delete/row {p1} @1..{nrow}                      !delete all rows,
                                                !will be filled again.


compute/table wrk.tbl -
  :ZEROPOINT = :{c_mag}_{p2} - :{p2}
compute/table wrk.tbl -
  :DIFF = ABS(:{c_mag}_{p2}-{c1}*:{c_airm}-{c0} - :{p2})
compute/table wrk.tbl :C_DIFF = tochar(:DIFF)
sort/table wrk.tbl :OBJECT,:C_DIFF              !sort on object names AND
                                                !mag-difference. Thus, the
                                                !following select/table will
                                                !always return the 'correct'
                                                !coincidence in its first
                                                !selected row -- it will have
                                                !the minimum mag-difference.

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
set/format I1 F15.6
do i1 = 1 {nrow} 1
  name = "{wrk.tbl,:OBJECT,{i1}}"
  select/table wrk.tbl :OBJECT.eq."{name}" >Null
!  ra = m$value(wrk.tbl,:RA,{i1})
!  dec = m$value(wrk.tbl,:DEC,{i1})
!  select/table wrk.tbl -
!    ABS(:RA-{ra}).lt.{eps}.AND.ABS(:DEC-{dec}).lt.{eps} >Null
  i1 = i1 + outputi(1)-1
  j1 = j1 + 1
  k1 = {Q1}                                   !index of first match
  @p fors_copyrow wrk.tbl {p1} {k1} {j1} 1 1 {ncol}
enddo
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

-delete wrk.tbl
set/format
return {nrow}                           !number of identified standard stars.



!=== LONGLOG ==================================================================
! (added 99/09/09 NR)
!
ENTRY LONGLOG
!
  define/local tmpcode/i/1/2 0 all
  define/local lastrow/i/1/1 0
  define/local txtline/i/1/1 0
  define/local oneline/c/1/47 " " all
  define/local minus/c/1/47 " " all
  !
  assign/print file tmp.txt
  print/table {tout} :OBJECT,:MAG_{refcol},:{refcol},:AIRMASS 47
  assign/print terminal
  !
  open/file tmp.txt read tmpcode
  if tmpcode(1) .eq. -1 then
        write/out " "
        mess/out 1 {procid} "Unable to open tmp.txt !"
    return 1
  endif
  show/table {tout} >Null
  lastrow = outputi(2)+3
  do txtline = 1 lastrow
    read/file {tmpcode(1)} oneline
    if txtline .ge. 2 then
      if txtline .eq. 3 minus = oneline(11:)
      longlog/out 1 "     {oneline(11:)}"
    endif
  enddo
  close/file {tmpcode(1)}
  longlog/out 1 "     {minus}"
  -delete tmp.txt
  !
  longlog/out 1 "     Table with identified std. stars: {tout}"
  return 0
!
!-------------------------------------------------------------------------------
