! $Id: fors_img_obs_calflux.prg,v 1.49 2000/03/13 11:27:16 nrainer Exp $   
!+++ FORS_IMG_OBS_CALFLUX +++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_img_obs_calflux.prg,v $
!
!.KEYWORDS      Flux calibration of a science frame and photometry table
!
!.USAGE         @p fors_img_obs_calflux sof_in sof_ref out
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!                             contains a reduced science frame or
!                             a reduced sky-subtracted science frame
!                             and a photometry table, optionally
!               sof_ref (P2): set of reference frames (ascii list)
!                             contains an extinction coefficient table
!                             of the same filter as the science frame.
!
!.OUTPUT        out     (P3): path + basename of output files:
!                             flux calibrated frame and flux calibrated
!                             photometry table
!
!.RETURN        Q1:  0: successful return
!                    otherwise: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Flux calibration of a science frame and photometry table.
!
!.ALGORITHM     Perform a sky subtraction and extinction correction with the
!               entire science frame using the following transformation:
!
!                 Flux_cal = (Flux_reduc - Flux_sky) * 10^[0.4 * (k*X + zp)]
!
!               Flux_reduc: bias, dark and flatfield corrected science frame
!               Flux_sky:   sky background frame
!               k:          extinction coefficient
!               zp:         zero point (extraterrestrial magnitude)
!               X:          airmass
!               In case a sky subtracted frame has been specified, the sky
!               subtraction will be skipped.
!               All these parameters and the sky-flux (Flux_sky) will be passed
!               by the input parameters: extinction table and sky frame,
!               respectively.
!               The same procedure will be made for a photometry table, if
!               specified within the input set of frames. No sky subtraction
!               has to be done as the values stored in the MAG_x column are
!               already sky subtracted. Filter-check of image and photometry
!               table is not necessary as the dataorganizer already does this
!               job!
!               Set of calibration frames should contain:
!                 (1) name of sky background frame (output of photometry.prg)
!                 (2) name of extinction table     (output of extcoeff.prg)
!
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.9 1998/03/09  Creation
!               1.0 1998/10/28  Output improved, comments
!                   1998/10/29  classify/product included
!                               main product specified: flux calibrated image
!                   1998/11/17  civil date calculation in case O_TIME(1)<1998.0
!                               check for valid airmass e [1.0,10.0]
!                   1998/12/09  fors_paragraph implemented
!                   1999/01/07  shortlog implemented
!                   1999/03/10  shortlog close shifted to FINISH/PIPELINE
!                   1999/03/11  bug fixed QC-keyword (divide by pix-area)
!               1.3 1999/09/03  Product header generation implemented (SB).
!.COMMENT
!------------------------------------------------------------------------------

define/local FAILPLUS/r/1/1 -90000.  ! -99999.9 is "FAILNO"
define/local line/c/1/79 "-" ALL
write/out {line}
LOG/REC fors_img_obs_calflux IN
!
! Check parameters
!
define/param p1 ? c "Enter name of ASCII filelist with reduced science frames:"
define/param p2 + c "Enter name of ASCII filelist with photom.&extinc. table:"
define/param p3 ? c "Enter name of FLUX CALIBRATED OUTPUT frame:"
define/param p4 + c "Enter constant sky value: "
!
! local variable declaration
!
define/local procid/c/1/16 img_obs_calflux
define/local SKYSUBTR/i/1/1 0           !sky subtraction flag
define/local BAD/r/1/1 -99999.9
!
define/local airm/r/1/1 0.0
define/local koeff/r/1/1 0.0
define/local zerop/r/1/1 0.0
define/local magcorr/r/1/1 0.0
define/local smea/r/1/1 0.0
define/local ssig/r/1/1 0.0
define/local smed/r/1/1 0.0
define/local limg/r/1/1 0.0
define/local fac/r/1/1 0.0
define/local lon/r/1/1 0.0
define/local lat/r/1/1 0.0
define/local area/r/1/1 0.0             !pixel area in arcsec^2
define/local frm/c/1/100 " " ALL        !may be deleted (s. below)
define/local ifrm/c/1/100 " " ALL
define/local phot/c/1/100 " " ALL
define/local skyb/c/1/100 " " ALL
define/local extt/c/1/100 " " ALL
define/local mag_bess/c/1/25 " " ALL
define/local c_magx/c/1/25 " " ALL
define/local refcol/c/1/1 " " ALL
define/local limmag/r/1/1 0.

define/local tmpwarn/c/1/80 " " ALL
tmpwarn = "AT THE MOMENT IGNORED IN ORDER TO TEST ALGORITHM!"


mess/out 3 {procid} -
  "Recipe IMG_OBS_CALFLUX will calibrate science frame\nand photometry table."
@p fors_paragraph 1


INITIALIZE/PIPELINE {P1} {P2} calfluxI.cat CCDRED + + reject.cat result.cat
if {Q1} .gt. 0 return {Q1}
!
init/prohead {p1} {p2} 2,"Calibration of science frame and photometry table"
if {Q1} .gt. 0 return {Q1}

initialize/longlog {p1} {p2} 2



@p fors_paragraph 2
                                                !check first for a sky
search/icat calfluxI {SKYSUB_ID} >Null          !subtracted science frame
if m$len(out_a) .ne. 0 then

  ifrm = out_a
  if nice_crea .eq. "y" then                    !display input data
    !
    @p fors_niceclear
    !
    !*******************************************
    @p fors_nicedisp {ifrm} ? 7. 0,0 ? ? 1.,3. n
    !*******************************************
    if {q1} .eq. 1 then
        write/out " "
        mess/out 1 {procid} "Displaying of frame failed !"
        mess/out 1 {procid} "({ifrm})"
      return 1
    endif
    !
        mess/out 4 {procid} "  loading image into display 1"
    assign/display d,1 >Null
    load/image {ifrm} ? {outputi(1)},avg c,c
    label/display "REDUCED, NORM., SKYSUB." {nice_dlab}
    !
  endif
  mess/out 3 {procid} "Science frame:\t{ifrm}"
  SKYSUBTR = 1                                  !skysubtraction already done.

else                                            !not found
                                                !-->  search for reduced
  search/icat calfluxI {RSCIENCE_ID} >Null      !science frame
  if m$len(out_a) .ne. 0 then

    ifrm = out_a
    if nice_crea .eq. "y" then                  !display input data
      !
      @p fors_niceclear
      !
      !*******************************************
      @p fors_nicedisp {ifrm} ? 7. 0,0 ? ? 1.,3. n
      !*******************************************
      if {q1} .eq. 1 then
        write/out " "
        mess/out 1 {procid} "Displaying of frame failed !"
        mess/out 1 {procid} "({ifrm})"
        return 1
      endif
      !
        mess/out 4 {procid} "  loading image into display 1"
      assign/display d,1 >Null
      load/image {ifrm} ? {outputi(1)},avg c,c
      label/display "REDUCED, NORM., SKYSUB." {nice_dlab}
      !
    endif
    mess/out 3 {procid} "Science frame:\t{ifrm}"
                                                !sky background image only
    if P4(1:1) .eq. "+" then                    !necessary as soon as an
      search/icat calfluxI {SKYBACK_ID} >Null   !science frame has been
      if m$len(out_a) .ne. 0 then               !specified
        skyb = out_a
        mess/out 3 {procid} "Sky background:\t{skyb}"
      else
        mess/out 1 {procid} "Sky background frame is missing!"
        return -2
      endif
    else
      skyb = P4
    endif
  else
    mess/out 2 {procid} "NO science frame found [{RSCIENCE_ID}]!"
  endif
  SKYSUBTR = 0

endif


search/icat calfluxI {PHOTT_ID} >Null           !photometry-table
if m$len(out_a) .ne. 0 then
  phot = out_a
  mess/out 3 {procid} "Photometry table:\t{phot}"
else
  mess/out 2 {procid} "No aligned photometry table found !"
endif

if m$len(ifrm) .eq. 0 .AND. m$len(phot) .eq. 0 then
  mess/out 1 {procid} -
    "Both, reduced SCIENCE frame and PHOTOMETRY TABLE are missing!"
  return -1
endif


search/icat calfluxI {ATMEXT_ID} >Null          !atmospheric extinction table
if m$len(out_a) .ne. 0 then
  extt = out_a
  mess/out 3 {procid} "Extinction table:\t{extt}"
  if m$existc("{extt}",":{c_iextco}") .gt. 0 then
    koeff = {{extt},:{c_iextco},1}
  else
    mess/out 1 {procid} -
      "Column :{c_iextco} of table {extt} does not exist !"
    return -11
  endif
  if m$existc("{extt}",":{c_izerop}") .gt. 0 then
    zerop = {{extt},:{c_izerop},1}
  else
    mess/out 1 {procid} -
      "Column :{c_izerop} of table {extt} does not exist !"
    return -12
  endif
else
  mess/out 1 {procid} "Extinction coefficients table is missing!"
  return -3
endif




if m$len(ifrm) .gt. 0 then              !do the calib. of the science frame
  GET/AIRM {ifrm}                       !determine airmass
  if {Q1} .ne. 0 then
    mess/out 1 {procid} "Unable to determine airmass !"
    return -13
  endif
  airm = outputr(1)

  if airm .GE. 1.0 .AND. airm .LE. 10.0 then
    !
    magcorr = koeff * airm + zerop
    fac = 10.**(0.4 * magcorr)
    if SKYSUBTR .eq. 1 then             !sky subtraction already done
      mess/out 3 {procid} -
        "Do the flux calibration:\nout = {ifrm}*{fac} ..."
      compute/image calflux = {ifrm} * {fac}
      smea = m$value({ifrm},{h_proqcsmea})      !relative sky mean
      ssig = m$value({ifrm},{h_proqcssig})      !relative sky sigma
      smed = m$value({ifrm},{h_proqcsmed})      !relative sky median
      limg = m$value({ifrm},{h_proqclimg})      !limiting magnitude
    else
      mess/out 3 {procid} -
        "Do the flux calibration:\nout = (in - {skyb})*{fac} ..."
      compute/image calflux = ({ifrm} - {skyb}) * {fac}
      smea = m$value({skyb},{h_proqcsmea})      !relative sky mean
      ssig = m$value({skyb},{h_proqcssig})      !relative sky sigma
      smed = m$value({skyb},{h_proqcsmed})      !relative sky median
      limg = m$value({skyb},{h_proqclimg})      !limiting magnitude
    endif
    calflux.bdf,{h_proqcsmea} = 0.0
    calflux.bdf,{h_proqcssig} = 0.0
    calflux.bdf,{h_proqcsmed} = 0.0
    calflux.bdf,{h_proqclimg} = -99999.9
    area = m$value({ifrm},{h_pixscale})
    area = area * area                          !pixelarea in arcsec^2
    if m$tstno(skyb) .eq. 1 then                !sky specified as constant (P4)
      calflux.bdf,{h_proqcsmea} = {P4}
      calflux.bdf,{h_proqcssig} = 0.0
      calflux.bdf,{h_proqcsmed} = {P4}
    else                                        !sky frame used.
      calflux.bdf,{h_proqcsmed} = smed
      calflux.bdf,{h_proqcsmea} = smea
      calflux.bdf,{h_proqcssig} = ssig
      calflux.bdf,{h_proqclimg} = limg
      if smed .gt. BAD calflux.bdf,{h_proqcsmed} = smed - magcorr
      if smea .gt. BAD calflux.bdf,{h_proqcsmea} = smea - magcorr
      if limg .gt. bad calflux.bdf,{h_proqclimg} = limg - magcorr
    endif

    mess/out 3 {procid} "Calibrated science frame `calflux.bdf` created."
    classify/product calflux.bdf {FSCIENCE_ID} {d_red} {d_pho},{d_det} OK ? y
    copy/dd calflux.bdf {h_procatg} calflux.bdf {fors_idstrng}
    copy/dd {extt} {h_proqcextc} calflux.bdf {h_proqcextc}/r/1/1
    copy/dd {extt} {h_proqczpoi} calflux.bdf {h_proqczpoi}/r/1/1
    !
    longlog/out 1 "     Calibrated image:     calflux.bdf"
    !
    add/icat result.cat calflux.bdf >Null
    get/outname result.cat {P3} calflux.bdf     !output name (without path)
    if nice_crea .eq. "y" then
      !
      !************************************************
      @p fors_nicedisp calflux.bdf ? 7. 0,0 ? ? 1.,3. n
      !************************************************
      if {q1} .eq. 1 then
        write/out " "
        mess/out 1 {procid} "Displaying of frame failed !"
        mess/out 1 {procid} "(calflux.bdf)"
        return 1
      endif
      !
        mess/out 4 {procid} "  loading image into display 2"
      assign/display d,2 >Null
      load/image calflux.bdf ? {outputi(1)},avg c,c
      label/display "FLUX CALIBRATED" {nice_dlab}
      !
    endif
    !
    shortlog/out 1 "Flux calibration of science image done."
    !
  else
    mess/out 1 {procid} "Bad airmass. No flux calibration possible !"
  endif
endif



if m$len(phot) .gt. 0 then              !do the calib. of the photometry table
  mess/out 3 {procid} -
    "Do the flux calibration of the photometry table ..."
  check/filter {phot}                   !only to get filter
  refcol = outputc(1:1)
  c_magx = "{c_mag}_{refcol}"
  mag_bess = "{refcol}_BESS"
  -copy {phot} calflux.tbl
  create/colu calflux :{mag_bess} "rel.MAG" F9.4 R*4
  compute/tab calflux :{mag_bess} = :{c_magx} - {koeff} * :{c_airm} - {zerop}
  airm = m$value(calflux,:{c_airm},1)
  magcorr = koeff * airm + zerop
  smed = m$value(calflux.tbl,{h_proqcsmed})
  smea = m$value(calflux.tbl,{h_proqcsmea})
  limg = m$value(calflux.tbl,{h_proqclimg})
  if smed .gt. BAD calflux.tbl,{h_proqcsmed} = smed - magcorr
  if smea .gt. BAD calflux.tbl,{h_proqcsmea} = smea - magcorr
  if limg .gt. BAD calflux.tbl,{h_proqclimg} = limg - magcorr


  mess/out 3 {procid} "Calibrated photometry table `calflux.tbl` created."
  classify/product calflux.tbl {PHOTTCAL_ID} {d_red} {d_pho},{d_det} OK
  copy/dd calflux.tbl {h_procatg} calflux.tbl {fors_idstrng}
  copy/dd {extt} {h_proqcextc} calflux.tbl {h_proqcextc}/r/1/1
  copy/dd {extt} {h_proqczpoi} calflux.tbl {h_proqczpoi}/r/1/1
  !
  longlog/out 1 "     Calib. photom. table: calflux.tbl"
  show/table calflux.tbl >Null
  limmag = {calflux.tbl,:{mag_bess},@{outputi(2)}}
  longlog/out 1 " "
  if limmag .gt. FAILPLUS then
    set/format T7.3,
    mess/out 3 {procid} "limiting magnitude, calib. = {limmag}"
    longlog/out 1 "     limit. magnitude, calib. = {limmag}"
    set/format
  else
    mess/out 3 {procid} "No calculation of limiting magnitude possible !"
    longlog/out 1 "     Limiting magnitude could not be determined !"
  endif
  !
  add/icat result.cat calflux.tbl >Null
  !
  shortlog/out 1 "Flux calibration of photometry table done."
endif


@p fors_paragraph 3
mess/out 3 {procid} "Flux calibration done."

! Create product header
finish/prohead result.cat 2

FINISH/PIPELINE result.cat {P3} + calflux.bdf
!
finish/longlog calfluxI.cat result.cat calflux.bdf 2 {p3}
!
-delete calfluxI.cat
-delete result.cat

LOG/REC fors_img_obs_calflux OUT
@p fors_paragraph 4

return 0
