! $Id: fors_img_qc0_statistics.prg,v 1.59 2001/03/13 21:01:00 nrainer Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_img_qc0_statistics.prg,v $
!
! .KEYWORDS   Simple statistics
!
! .PURPOSE    Pipeline wrapper and frame classification for
!             fors_statistics.prg:
!             Determine statistics of a science or standard star frame.
!             The histogrammes can be displayed.
! __________
!
! .USAGE      @p fors_img_qc0_statistics in ref [out]
!
! .INPUT      in   (P1): set of input frames (ascii list);
!                        possible frame types:
!                          standard star images (STANDARD_<all_modes>),
!                          science images (SCIENCE_<all_modes>);
!                            default: none
!
!             ref  (P2): super recipe mode: "+" (reference frames are included
!                        in P1),
!                        single recipe mode: set of reference frames (ascii
!                        list);
!                        needed frame:
!                          master bias frame (MASTER_BIAS),
!                        also needed frame type:
!                          table with purging information (DESC_PURGE_TABLE);
!                            default: none
!
!             out  (P3): single recipe mode: basename for ascii list with
!                        output frame,
!                        super recipe mode: again set of input frames and
!                        existing image catalogue (".cat"), separated by a
!                        comma;
!                        table for input set:
!                          science/standard statistics (STATISTICS_TABLE);
!                            default: `PROID' defined in procedure (single mode)
!
!             scale (P4):method for scaling the master bias frame with a stan-
!                        dard or science frame as used in "fors_normbias.prg";
!                          default: none
!                          possible values: MEA(N) | MED(IAN) | OVE(RSCAN) | NO
!
! .OUTPUT     "<PROID>_sta.tfits":  table with statistics
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  Statistics on whole frame and quadrants.
!
! .COMMENT    -
!
! .ENVIRON    FORS context
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.0  98/09/23  Creation
!                  98/12/15  Release
!             1.1  99/01/14  Release
!             1.2  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref in ref out scale
!
!
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
define/local PROID/c/1/40 "img_qc0_statistics"
!
define/local warn/i/1/1 0
write/keyword q1 0
!
!
if p1 .eq. "??" then
  w/o "      ******************************************************************"
  w/o "      in   (P1): set of input frames (ascii list);"
  w/o "                 possible frame types:"
  w/o "                   standard star images (STANDARD_<all_modes>),"
  w/o "                   science images (SCIENCE_<all_modes>);"
  w/o "                     default: none"
  w/o " "
  w/o "      ref  (P2): super recipe mode: "+" (reference frames are included"
  w/o "                 in P1),"
  w/o "                 single recipe mode: set of reference frames (ascii"
  w/o "                 list);"
  w/o "                 needed frames:"
  w/o "                   master bias frame (MASTER_BIAS),"
  w/o "                 optional:"
  w/o "                   bad pixel table (BAD_PIXEL_TABLE);"
  w/o "                 also needed frame type:"
  w/o "                   table with purging information (DESC_PURGE_TABLE);"
  w/o "                     default: none"
  w/o " "
  w/o "      out  (P3): single recipe mode: basename for ascii list with"
  w/o "                 output frame,"
  w/o "                 super recipe mode: again set of input frames and"
  w/o "                 existing image catalogue (".cat"), separated by a"
  w/o "                 comma;"
  w/o "                 table for input set:"
  w/o "                   science/standard statistics (STATISTICS_TABLE);"
  w/o "                     default: `PROID' defined in procedure (single mode)"
  w/o " "
  w/o "      scale(P4): method for scaling the master bias frame with a stan-"
  w/o "                 dard or science frame as used in "fors_normbias.prg";"
  w/o "                   default: none"
  w/o "                   possible values: MEA(N) | MED(IAN) | OVE(RSCAN) | NO"
  w/o "      ******************************************************************"
  return/exit
endif
!
!
!          PREPARATIONS FOR RECIPE :
!          ***********************
!
log/recipe {PROID} in
!
        mess/out 3 {PROID} "Determine statistics of science/std. frame(s) ..."
!
@p fors_paragraph 1 1  ! -------- INITIALIZATION --------
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(in):
!------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter name of input frame set: "
define/local inpipe_frm/c/1/76 "{p1}"
!
if m$exist(inpipe_frm) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered list is not present"
        mess/out 1 {PROID} "or can not be opened !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
!
define/local filecode/i/1/2 0 all
open/file {inpipe_frm} read filecode
if {filecode(1)} .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open entered list !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
read/file {filecode(1)} outputc
if {filecode(2)} .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered list is empty !"
        mess/out 1 {PROID} "({inpipe_frm})"
  close/file {inpipe_frm}
  goto error
endif
close/file {filecode(1)}
!
!
!P2(ref):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter name of reference frame set: "
define/local inpipe_ref/c/1/76 "{p2}"
!
! No tests like for P1 - in super recipes reference frame is included in P1 .
!
!
!P3(out):
!-------
define/parameter p3 {PROID} ? -
  "        Enter basename of out.list(single)/exist.cat.".cat"(super): "
define/local outpipe/c/1/76 "{p3}"
!
!
!P4(scale):
!---------
if p4 .eq. "?" mess/out 2 {PROID} " "
define/parameter p4 ? ? -
  "        Enter scaling method: "
define/local scale/c/1/3 "{p4}"
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Identifying input frames":
!
  define/local catal/i/1/1 0
  define/local topicima/c/1/76 " " all
  define/local id/c/1/25 " " all
  !
  define/local stdID/c/1/25 " " all
  define/local sciID/c/1/25 " " all
  define/local mbiasID/c/1/25 " " all
  !
  define/local science/c/1/76 " " all
  define/local mbias/c/1/76 " " all
  define/local char/i/1/1 0
  define/local n_mbias/c/1/76 " " all
  define/local s_mbias/c/1/76 " " all
  !
  define/local in/i/1/1 0
  define/local refmas/i/1/1 0
  !
  define/local insdesc/c/1/60 " " all
  !
  define/local psc/i/1/1 0
  define/local osc/i/1/1 0
!
!
!          LOADING OF GLOBAL KEYS (SINGLE RECIPE MODE) ,
!          CONVERSION FITS->BDF/TBL ,
!          CLASSIFICATION OF INPUT FRAMES :
!          *********************************************
!
        mess/out 4 {PROID} "Getting input frames"
!
initialize/pipeline {inpipe_frm} {inpipe_ref} -
  inpipe_good.cat ? ? + inpipe_bad.cat ?
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Pipeline initialization failed !"
  goto error
endif
-delete inpipe_bad.cat
!
! Initialize product header:
init/prohead {inpipe_frm} {inpipe_ref} -
  2,"Determine statistics of science/std.star frame(s)" 
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Product header initialization failed !"
  goto error
endif
!
initialize/longlog {inpipe_frm} {inpipe_ref} 2
!
!
!          IDENTIFICATION VIA DESCRIPTORS ,
!          MAPPING OF FRAMES TO PARAMETERS  :
!          ********************************
!
        mess/out 4 {PROID} "Identifying input and reference frames"
!
stdID = "{STD_ID}"
sciID = "{SCIENCE_ID}"
mbiasID = "{MBIAS_ID}"
!
create/icat {PROID}.cat null {FORS_IDSTRNG} >Null
in = 0
refmas = 0
!
ident_loop:
  store/frame topicima inpipe_good.cat 1 end_ident_loop
  id = "{{topicima},{fors_idstrng}}"
  !
  if id .eq. "{stdID}" then
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    in = 1
  elseif id .eq. "{sciID}" then
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    in = 1
  elseif id .eq. "{mbiasID}" then
    mbias = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refmas = 1
  endif
  !
  goto ident_loop
end_ident_loop:
-delete inpipe_good.cat
!
if in .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching input science or"
        mess/out 1 {PROID} "standard star frame present !"
  goto error
endif
!
if refmas .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No master bias frame present !"
  goto error
endif
!
insdesc = "{#1_{PROID}.cat,{h_instrume}}"
!
!
!          DETERMINING OVERSCAN SECTIONS :
!          *****************************
!
@p fors_paragraph 2 1  ! -------- PROCESSING --------
!
!*******************************
@p fors_get_imsec #1_{PROID}.cat
!*******************************
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Determination of overscan sections failed !"
  goto error
endif
psc = outputi(5)
osc = outputi(6)
!
!
!          SCALING MASTER BIAS :
!          *******************
!
if scale .ne. "NO" then
  !
        mess/out 4 {PROID} "Scaling master bias with method <{scale}>"
  !
  show/icat {PROID}.cat n
  if outputi(1) .gt. 1 then
        write/out " "
        mess/out 1 {PROID} "More than one input frame currently not supported !"
    goto error
  endif
  !
    catal = 0
    store/frame science {PROID}.cat
  compute/ovbias {science} med
  if {q1} .eq. -1 then
        mess/out 2 {PROID} "No overscan in input frame - no scaling of bias !"
    scale = "NO"
    goto skip_scale
  elseif {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Bias calculation from overscan failed !"
    goto error
  endif
  !
    char = m$indexb(mbias,".")-1
    n_mbias = mbias(1:{char}) // "_norm.bdf"
  normalize/mbias {mbias} {n_mbias} {scale}
  if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Normalizing of master bias frame failed !"
    goto error
  endif
  !
    s_mbias = mbias(1:{char}) // "_scal.bdf"
  scale/mbias {science} {n_mbias} {s_mbias}
  if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Scaling of master bias frame failed !"
    goto error
  endif
  !
endif
!
skip_scale:
if scale .eq. "NO" s_mbias = mbias
!
!
!          *********************************
!          *                               *
!          *  CALLING OF MAIN PROCEDURE :  *
!          *                               *
!          *********************************
!
        mess/out 4 {PROID} "Applying procedure "fors_statistics" :"
!
!*****************************************************
@p fors_statistics {PROID} {s_mbias} p ? {psc},{osc} ?
!*****************************************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Statistical tests on science/std. frames failed !"
  goto error
endif
!
create/icat outprog.cat null >Null
add/icat outprog.cat {PROID}_sta.tbl >Null
-delete dirfile.ascii
!
!
!          UPDATING DESCRIPTORS OF OUT- AND INPUT FRAMES :
!          *********************************************
!
@p fors_paragraph 3 1  ! -------- TERMINATION --------
!
        mess/out 4 {PROID} "Classifying output table ({PROID}_sta.tbl)"
!
classify/product {PROID}_sta.tbl {STATISTT_ID} {d_qcp} {d_empty} OK y
write/descriptor {PROID}_sta.tbl {h_instrume}/c/1/60 {insdesc}
write/descriptor {PROID}_sta.tbl {FORS_IDSTRNG}/c/1/60 {STATISTT_ID} !f.su.rec.
write/descriptor {PROID}_sta.tbl {h_proqcsts}/c/1/9 "CHECKED"
write/dhelp {PROID}_sta.tbl {h_proqcsts} "{i_proqcsts}"
longlog/out 1 "     Statistics table: {PROID}_sta.tbl"
!
        mess/out 4 {PROID} "Endorsing QC status in input image(s)"
!
catal = 0
desc_loop_in:
  store/frame topicima {PROID}.cat 1 end_desc_loop_in
  !
  write/descriptor {topicima} {h_proqcsts}/c/1/9 "CHECKED"
  write/dhelp {topicima} {h_proqcsts} "{i_proqcsts}"
  !
  goto desc_loop_in
end_desc_loop_in:
!
shortlog/out 1 "Simple statistics calculated"
!
!
!          UPDATING OF OUTPUT CATALOGUE (SUPER RECIPE MODE) ,
!          CONVERSION BDF/TBL->FITS/TFITS (SINGLE MODE) :
!          **************************************************
!
        mess/out 4 {PROID} "Releasing output table"
!
! Create product header:
finish/prohead outprog.cat 2
!
finish/pipeline outprog.cat {outpipe} + {PROID}_sta.tbl
!
finish/longlog {PROID}.cat outprog.cat {PROID}_sta.tbl 2 {outpipe}
!
-delete {PROID}.cat
-delete outprog.cat
!
!
!          RESETTINGS AFTER RECIPE :
!          ***********************
!
set/format
!
log/recipe {PROID} out
!
@p fors_paragraph 4 1  ! ================
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  finish/longlog ? -1 ? 2
  write/out " "
  return 1
!
!
!-------------------------------------------------------------------------------
