! $Id: fors_lss_cal_response.prg,v 1.20 2000/09/26 16:26:06 nrainer Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: fors_lss_cal_response.prg,v $
!
!.KEYWORDS    LSS/MOS instrumental response
!
!.USAGE       @p fors_lss_cal_response  frm_in frm_ref frm_out mode area
!
!.INPUT       frm_in      (P1)    Set of raw input frames to be processed
!                                 (relative flux spectrum of standard star(s)).
!             frm_ref     (P2)    Set of reference frames (reference flux
!                                 table, atmospheric extinction coefficients).
!             mode        (P4)    Indicator of observing mode (LSS, MOS)
!             area        (P5)    Telescope effective area (area taking
!                                 occultation effects into account)
!
!.OUTPUT      frm_out     (P3)    Set of output frames (system response
!                                 table(s)) 
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The input set of frames consists of a table of extracted
!             standard star spectra. The extracted spectra of each standard
!             stars is then corrected for atmospheric extinction effects and
!             divided by the reference flux. The result is a table of response
!             values for the combination telescope and instrument. The
!             response values are given at the wavelengths provided by the
!             reference spectrum. 
!
!.ENVIRON     FORS context, XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!             1.3  1999/09/03:  Product header generation implemented (SB).
!
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

define/param p1 frm_in.set   c  "Enter input frame set:"
define/param p2 frm_ref.set  c  "Enter reference frame set:"
define/param p3 frm_out      c  "Enter output frame set:"
define/param p4 LSS          c  "Enter observing mode:"
define/param p5 49.0         c  "Enter telescope effective area:"


! Procedure identification
define/local procID/c/1/80      "{p4}CalResponse"


! Recipe identification
define/local recID/c/1/12       "CAL_RESPONSE"


! Local constants
define/local dwin/i/1/2         1,2
define/local gwin/i/1/2         1,2
define/local stdspec/c/1/20     "fors_stdspec.bdf"
define/local refspec/c/1/20     "fors_refspec.tbl"
define/local refspec_b/c/1/20   "fors_refspec_b.tbl"
define/local rspbase/c/1/20     "fors_rsp"
define/local rspsuffix/c/1/20   ".tbl"



! Local variables
define/local i/i/1/1            0
define/local pos/i/1/1          0
define/local catal/i/1/1        0
define/local ctrl/i/1/3         0             all
define/local nstd/i/1/1         0
define/local nrow/i/1/1         0
define/local nwave/i/1/1        0
define/local effarea/r/1/1      0.
define/local airmass/r/1/1      0.
define/local wstart/d/1/1       0.
define/local wstep/d/1/1        0.
define/local wend/d/1/1         0.
define/local obsmod/c/1/3       " "           all
define/local setup/c/1/20       " "           all
define/local cat_accept/c/1/20  "accept.cat"
define/local cat_reject/c/1/20  "reject.cat"
define/local cat_result/c/1/20  "result.cat"
define/local cat_entry/c/1/512  " "           all
define/local min_respo/r/1/1    0.

! Unfortunately needed for FINISH/LONGLOG
define/local resptab_name/c/1/32 " "



! Connect to pipeline
log/rec "{p4}_{recID}" IN
mess/out 3 {procID} "Recipe {p4}_{recID}: Instrument response computation."


! Process parameters
obsmod = m$upper(p4)
setup = "{obsmod}pipe"

if m$tstno(p5) .eq. 1 then
  effarea = {p5}
else
  mess/out 1 {procID} "Invalid telescope effective area!"
  clean/pipeline xmos,spec
  return 1
endif


! Startup context, initialize environment and prepare input 
! frames for processing
@p fors_paragraph 1

initialize/pipeline {p1} {p2} {cat_accept} xmos {setup} {obsmod} -
                    {cat_reject} {cat_result}
if {q1} .ne. 0 then
  mess/out 1 {procID} "Pipeline initialization failed!"
  clean/pipeline xmos,spec
  return 1
endif
resptab_name = "{resptab}"


! Initialize product header
init/prohead {p1} {p2} 2,"Instrument response computation" -
   obsmode,{p4} area,{p5}
if {q1} .ne. 0 then
  mess/out 1 {procID} "Product header initialization failed!"
  clean/pipeline xmos,spec
  return 1
endif


! Initialize longlog
init/longlog {p1} {p2} 2 "obsmode: {p4}" "area: {p5}"
if {q1} .ne. 0 then
  mess/out 1 {procID} "Longlog initialization failed!"
  clean/pipeline xmos,spec
  return 1
endif


@p fors_paragraph 2


! Update session keywords
read_cat_entry:

store/frame cat_entry {cat_accept} 1 exit_loop
fors_id = m$value({cat_entry},{FORS_IDSTRNG})
fors_id = m$upper(fors_id) 

if fors_id .eq. ASTD_ID then
  extobjec = cat_entry
  ctrl(1) = 1
elseif fors_id .eq. ATMEXT_ID then
  atmext = cat_entry
  ctrl(2) = 1
elseif fors_id .eq. STDFLX_ID then
  refflux = cat_entry
  ctrl(3) = 1
endif

goto read_cat_entry


exit_loop:

if ctrl(1) .eq. 0 then
  mess/out 1 {procID} "No standard star spectrum table found!"
  clean/pipeline xmos,spec
  return 1
else if ctrl(2) .eq. 0 then
  mess/out 1 {procID} "No atmospheric extinction table present!"
  clean/pipeline xmos,spec
  return 1
else if ctrl(3) .eq. 0 then
  mess/out 1 {procID} "No reference flux table present!"
  clean/pipeline xmos,spec
  return 1
endif


! Get number of standard stars in input table
nstd = m$value({extobjec},tblcontr(4))


! Calculate airmass for observed standard stars
get/airmass {extobjec}
if {q1} .eq. 0 then
  airmass = outputr(1)
else
  mess/out 1 {procID} "Airmass computation failed!"
  clean/pipeline xmos,spec
  return 1
endif


! Correct observed spectra for atmospheric extinction and compute
! wavelength dependent system response for all identified objects
do i = 1 nstd

  set/format i1
  mess/out 3 {procID} "computing response(wavelength) for obj. {i} / {nstd}"

  prepare/standard {extobjec} {i} {refflux} {effarea} {stdspec} {refspec}
  if {q1} .ne. 0 then
    mess/out 1 {procID} "System response computation failed!"
    clean/pipeline xmos,spec
    return 1
  endif

  if nstd .gt. 1 then
    set/format i4
    resptab = rspbase // "{i}" // rspsuffix
    set/format
  endif

  extinction/mos {stdspec} {atmobjec} {atmext} :{c_sextco} {airmass}


! Rebin reference spectrum to a step size twice the step size of the
! observed standard star spectrum to avoid problems with the current
! version of integrate/mos

  wstep = 2. * m$value({stdspec},step(1))

  nrow = m$value({refspec},tblcontr(4))

  wstart = m$value({refspec},:{c_fxwave},1) - 0.5 * -
           m$value({refspec},:{c_fxbwidth},1)
  wend = m$value({refspec},:{c_fxwave},{nrow}) + 0.5 * -
         m$value({refspec},:{c_fxbwidth},{nrow})
  nwave = (wend - wstart) / wstep

! Calculate central wavelength of first bin in the new grid
  wstart = wstart + 0.5 * wstep

  rebin/table {refspec} :{c_fxwave},{c_fxflux},{c_fxbwidth} {refspec_b} -
              {wstart},{wstep},{nwave} pixel


  integrate/mos {atmobjec} {refspec_b} {resptab}

!*** Added (NR) ***
! Flagging response values which are less than (max.resp.*FORS_MINRESPFAC) -
! instead of check/response below:
  create/column {resptab} :MIN " " I3 I*4
  statistics/table {resptab} :{c_rsratio} >Null
  min_respo = outputr(2)*FORS_MINRESPFAC
  !
  set/format F6.4
  mess/out 3 {procID} -
    "  min. allowed response: {min_respo} ({FORS_MINRESPFAC}*max.)" 
  set/format
  !
  select/table {resptab} :{c_rsratio}.lt.{min_respo} >Null
  copy/table {resptab} respo_lo.tbl
  compute/table respo_lo.tbl :MIN = 0
  !
  select/table {resptab} :{c_rsratio}.ge.{min_respo} >Null
  copy/table {resptab} respo_hi.tbl
  compute/table respo_hi.tbl :MIN = 1
  !
  select/table {resptab} seq.eq.0 >Null
  copy/table {resptab} respo_all.tbl
  merge/table respo_lo.tbl respo_hi.tbl respo_all.tbl
  -rename respo_all.tbl {resptab}
  -delete respo_lo.tbl respo_hi.tbl
!******************

! Copy all but the standard descriptors to the response table
  copy/dd {extobjec} *,3 {resptab}


  if nice_gids({gwin(1)}) .eq. 1 then
    view/response {resptab} {extobjec} {i} {gwin(1)}
    if {q1} .ne. 0 then
      mess/out 2 {procID} "Problems showing computed instrument response!"
    endif
    !*** Added (NR) ***
      set/graphic colour=2
    overplot/line 1 {plrgrap(1)},{min_respo} {plrgrap(2)},{min_respo}
      @p fors_nicegrap
      if {q1} .eq. 1 then
        mess/out 2 {procID} "Label of min. allowed response failed !"
        clean/pipeline xmos,spec
        return 1
      endif
      set/format F6.4
      inputi = m$index(nice_glabw," ")-1
      inputc = nice_glabw(1:{inputi})
    label/graphic "Min.allow.resp.: {min_respo}={FORS_MINRESPFAC}*max." -
      {inputc} ? 0.7 1
      set/format
      set/graphic
    !******************
  endif


! Check if response is out of range
! check/response {resptab}
! if {q1} .ne. 0 then
!   mess/out 1 {procID} -
!       "Computed response exceeds limits. Allowed range is [0.,1.]"
!   clean/pipeline xmos,spec
!   return 1
! endif


! Prepare output catalog
  classify/product {resptab} {RESP_ID} {d_pre} {d_geo},{d_det} OK
  write/desc {resptab} {FORS_IDSTRNG}/c/1/20 {RESP_ID}

  add/icat {cat_result} {resptab} >Null
enddo
@p fors_paragraph 3


! Update internal log file
shortlog/out 1 "Instrument response computed."


! Create product header
finish/prohead {cat_result} 2


! Shutdown pipeline
finish/pipeline {cat_result} {p3} xmos,spec {resptab}
if {q1} .ne. 0 then
  mess/out 1 {procID} "Error leaving pipeline environment!"
  finish/longlog {cat_accept} -1 ? 1 {p3}
  clean/pipeline xmos,spec
  return 1
endif


! Close longlog
finish/longlog {cat_accept} {cat_result} {resptab_name} 2 {p3}


! Disconnect from pipeline
log/rec "{p4}_{recID}" OUT

@p fors_paragraph 4

return 0
