! $Id: fors_lss_qc1_resol.prg,v 1.23 2001/03/09 11:48:45 sbogun Exp $   
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
! .COPYRIGHT  (C) 1999  European Southern Observatory (ESO)
!
! .IDENT       $RCSfile: fors_lss_qc1_resol.prg,v $
!
! .KEYWORDS   Spectroscopical quality, wavelength calibration
!
! .PURPOSE    Pipeline wrapper and frame classification for
!             fors_resol.prg:
!             Compute the spectral resolution of lamp emission lines in each
!             slit, and that per bin perpendicular to the dispersion direction.
! __________
!
! .USAGE      @p fors_lss_qc1_resol in ref [out] [mod]
!
! .INPUT      in   (P1): input frames (ascii list);
!                        possible frame type (m(o)d(e) = [LSS|MOS]):
!                          reduced lamp image (REDUCED_LAMP_<md>);
!                        further needed frames:
!                          table with identified lines (LINEPOS_<md>),
!                          table with dispersion solution (DISP_COEFF_<md>);
!                            default: none
!
!             ref  (P2): super recipe mode: "+" (reference frames are included
!                        in P1),
!                        single recipe mode: set of reference frames (ascii
!                        list);
!                        needed frames:
!                          table w. posit. of all slits (SLIT_POSITIONS_<md>),
!                          table f. XMOS context w.y-bin.val. (DRS_SETUP_<md>),
!                        also needed frame type:
!                          table with purging information (DESC_PURGE_TABLE);
!                            default: none
!
!             out  (P3): single recipe mode: basename for ascii list with
!                        output frames,
!                        super recipe mode: again input frame and
!                        existing image catalogue (".cat"), separated by a
!                        comma;
!                          table with spectral resolutions (RESOL_TABLE);
!                            default: `PROID' defined in procedure (single mode)
!
!             mod  (P4): flag for observing mode <md>;
!                          default: LSS
!
!
! .OUTPUT     "<P1>_resol.tfits":  1 SLIT       ,
!              (`d':detect. pix.   2 SLT_STRT d ,
!               = world coords.)   3 SLT_END  d : slit positions
!              (`A':angstrom)     -------------
!              (`n':do NOT change) 4 X        d ,
!                                  5 Y        d ,
!                                  6 WAVEC    A : line positions
!                                  7 REJECT     : "1" = wave calib. O.K.
!                                 -------------
!                               n  8 XSTART   d ,
!                               n  9 XEND     d ,
!                               n 10 YSTART   d ,
!                               n 11 YEND     d : bin positions for fitting
!                                 12 XCEN     d : fit results: line centre ,
!                                 13 STATUS     :              "0" = fit O.K. ,
!                                 14 FWHM     d :              FWHM (per bin)
!                                 -------------
!                                 15 STEP   A/d : dispe.(1.ord.) f.FWHM convers.
!                                 16 D_WAVEC  A : convert.FWHM as "delta_lambda"
!                                 17 SPECRES    : SPECTRAL RESOLUTION (per bin)
!
!               Hint for columns 8-11 (bin marks):
!
!                 With
!                   `select/table <P1>_resol.tbl :REJECT.eq.1.and.:STATUS.eq.0'
!                   `draw/rectangle <P1>_resol.tbl f n green,0'
!                 the used bins can be marked onto the displayed <P1> frame.
!
!               Legend for column 13 (bin qual.flags):
!
!                 <0 : estimation failed (during Gauss fit)
!                  0 : O.K.
!                  1 : ambiguous sources
!                  2 : no source
!                  3 : iteration failed
!                 >3 : routine error
!
!               Formula for column 17 (spectral resolution):
!
!                 SPECRES = WAVEC / D_WAVEC , with D_WAVEC = FWHM * STEP
!
! .RETURN     Q1:  0: successful return
!                 -1: a warning message occured
!                  1: an error occured
! __________
!
! .ALGORITHM  -
!
! .COMMENT    -
!
! .ENVIRON    FORS context, XMOS context
!
! .LANGUAGE   MIDAS 98NOV
!
! .AUTHOR     N.Rainer  DPG/DMD/ESO
!
! .VERSION    1.2  99/03/11  Creation
!                  99/03/15  Release
!             1.3  99/09/23  Release
!
!-------------------------------------------------------------------------------
!
!
crossref in ref out mod
!
!
define/local XFITFACT/r/1/1 3. ? +lower_levels
!                           ! multiplier for fitting slit width
!
define/local FMTDEF/c/1/17 "I1 E14.7,E23.15"
!
define/local warn/i/1/1 0
write/keyword q1 0
!
!
!P4(mod):
!-------
define/parameter p4 lss ? -
  "        Enter flag for observing mode  [def:lss|mos]: "
define/local insmod/c/1/10 "{p4}"
!
inputc = m$lower(insmod)
define/local PROID/c/1/40 "{inputc}_qc1_resol"
!
if insmod .ne. "lss" .and. insmod .ne. "mos" then
        write/out " "
        mess/out 1 {PROID} "Entered observing mode unknown !"
        mess/out 1 {PROID} "({insmod})"
  goto error
endif
!
!
if p1 .eq. "??" then
  set/format I1
  w/o "      ******************************************************************"
  w/o "      in   (P1): input frames (ascii list);"
  w/o "                 possible frame type (m(o)d(e) = [LSS|MOS]):"
  w/o "                   reduced lamp image (REDUCED_LAMP_<md>);"
  w/o "                 further needed frames:"
  w/o "                   table with identified lines (LINEPOS_<md>),"
  w/o "                   table with dispersion solution (DISP_COEFF_<md>);"
  w/o "                     default: none"
  w/o " "
  w/o "      ref  (P2): super recipe mode: "+" (reference frames are included"
  w/o "                 in P1),"
  w/o "                 single recipe mode: set of reference frames (ascii"
  w/o "                 list);"
  w/o "                 needed frames:"
  w/o "                   table w. posit. of all slits (SLIT_POSITIONS_<md>),"
  w/o "                   table f. XMOS context w.y-bin.val. (DRS_SETUP_<md>),"
  w/o "                 also needed frame type:"
  w/o "                   table with purging information (DESC_PURGE_TABLE);"
  w/o "                     default: none"
  w/o " "
  w/o "      out  (P3): single recipe mode: basename for ascii list with"
  w/o "                 output frames,"
  w/o "                 super recipe mode: again input frame and"
  w/o "                 existing image catalogue (".cat"), separated by a"
  w/o "                 comma;"
  w/o "                   table with spectral resolutions (RESOL_TABLE);"
  w/o "                     default: `PROID' defined in procedure (single mode)"
  w/o " "
  w/o "      mod  (P4): flag for observing mode <md>;"
  w/o "                   default: LSS"
  w/o "      ******************************************************************"
  set/format
  return/exit
endif
!
!
!          PREPARATIONS FOR RECIPE :
!          ***********************
!
log/recipe {PROID} in
!
        mess/out 3 {PROID} "Compute spectral resolutions ..."
!
@p fors_paragraph 1 1  ! -------- INITIALIZATION --------
!
set/format {FMTDEF}
!
!
!          INPUT :
!          *****
!
!P1(in):
!------
if p1 .eq. "?" mess/out 2 {PROID} " "
define/parameter p1 ? ? -
  "        Enter name of input frame: "
define/local inpipe_frm/c/1/76 "{p1}"
!
if m$exist(inpipe_frm) .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "Entered list with single input frame"
        mess/out 1 {PROID} "is not present or can not be opened !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
!
define/local filecode/i/1/2 0 all
open/file {inpipe_frm} read filecode
if filecode(1) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Unable to open entered list !"
        mess/out 1 {PROID} "({inpipe_frm})"
  goto error
endif
read/file {filecode(1)} outputc
if filecode(2) .eq. -1 then
        write/out " "
        mess/out 1 {PROID} "Entered list is empty !"
        mess/out 1 {PROID} "({inpipe_frm})"
  close/file {filecode(1)}
  goto error
endif
close/file {filecode(1)}
!
!
!P2(ref):
!-------
if p2 .eq. "?" mess/out 2 {PROID} " "
define/parameter p2 ? ? -
  "        Enter name of reference frame set: "
define/local inpipe_ref/c/1/76 "{p2}"
!
! No tests like for P1 - in super recipes reference frame is included in P1 .
!
!
!P3(out):
!-------
define/parameter p3 {PROID} ? -
  "        Enter basename of out.list(single)/exist.cat.".cat"(super): "
define/local outpipe/c/1/76 "{p3}"
!
!
!
!          DEFINITION OF VARIABLES :
!          ***********************
!
!     "Getting input frames":
!
  define/local obsmod/c/1/10 " " all
  define/local setup/c/1/14 " " all
!
!     "Identifying input frames":
!
  define/local catal/i/1/1 0
  define/local topicima/c/1/76 " " all
  define/local id/c/1/25 " " all
  !
  define/local lampID/c/1/25 " " all
  define/local slitID/c/1/25 " " all
  define/local lineID/c/1/25 " " all
  define/local dicoID/c/1/25 " " all
  !
  define/local lampima/c/1/76 " " all
  define/local slittblori/c/1/76 " " all
  define/local linetbl/c/1/76 " " all
  define/local dcoeftbl/c/1/76 " " all
  !
  define/local inlamp/i/1/1 0
  define/local refslit/i/1/1 0
  define/local refline/i/1/1 0
  define/local refdico/i/1/1 0
  !
  define/local insdesc/c/1/60 " " all
!
!     "Selecting slits":
!
  define/local char/i/1/1 0
  define/local slittbl/c/1/76 " " all
!
!     "Overscan sections":
!
  define/local psc/i/1/1 0
  define/local osc/i/1/1 0
!
!     "Calling of procedure":
!
  define/local slitcol/c/1/100 " " all
  define/local linecol/c/1/100 " " all
  define/local dcoefdesc/c/1/100 " " all
  define/local slwid/c/1/72 " " all
  define/local resoltbl/c/1/76 " " all
!
!
!
!          LOADING OF GLOBAL KEYS (SINGLE RECIPE MODE) ,
!          CONVERSION FITS->BDF/TBL ,
!          CLASSIFICATION OF INPUT FRAMES :
!          *********************************************
!
        mess/out 4 {PROID} "Getting input frames"
!
obsmod = m$upper(insmod)
setup = obsmod//"pipe"
!
initialize/pipeline {inpipe_frm} {inpipe_ref} -
  inpipe_good.cat xmos {setup} {obsmod} inpipe_bad.cat ?
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Pipeline initialization failed !"
  goto error
endif
-delete inpipe_bad.cat
!
! Initialize product header:
init/prohead {inpipe_frm} {inpipe_ref} -
  2,"Compute spectral resolution" obsmode,{obsmod}
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Product header initialization failed !"
  goto error
endif
!
initialize/longlog {inpipe_frm} {inpipe_ref} 2 "obs.mode: {obsmod}"
!
!
!          IDENTIFICATION VIA DESCRIPTORS ,
!          MAPPING OF FRAMES TO PARAMETERS  :
!          ********************************
!
        mess/out 4 {PROID} "Identifying input and reference frames"
!
lampID = WLC_ID
slitID = SLITS_ID
lineID = LINPOS_ID
dicoID = DISPREL_ID
!
create/icat {PROID}.cat null {FORS_IDSTRNG} >Null  ! only for finish/longlog
inlamp = 0
refslit = 0
refline = 0
refdico = 0
!
ident_loop:
  store/frame topicima inpipe_good.cat 1 end_ident_loop
  id = "{{topicima},{fors_idstrng}}"
  !
  if id .eq. "{lampID}" then
    lampima = topicima
    add/icat {PROID}.cat {topicima} >Null
        mess/out 4 {PROID} "{topicima} : {id} => input"
    inlamp = inlamp+1
  elseif id .eq. "{slitID}" then
    slittblori = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refslit = refslit+1
  elseif id .eq. "{lineID}" then
    linetbl = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refline = refline+1
  elseif id .eq. "{dicoID}" then
    dcoeftbl = topicima
        mess/out 4 {PROID} "{topicima} : {id} => reference"
    refdico = refdico+1
  endif
  !
  goto ident_loop
end_ident_loop:
-delete inpipe_good.cat
!
if inlamp .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching input lamp frame present !"
  goto error
elseif inlamp .gt. 1 then
        write/out " "
        mess/out 1 {PROID} "More than one input lamp frame present !"
  goto error
endif
!
if refslit .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching table w. posit. of all slits present !"
  goto error
elseif refslit .gt. 1 then
        write/out " "
        mess/out 1 {PROID} "More than one tbl. w. posit. of all slits present !"
  goto error
endif
!
if refline .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching table with identified lines present !"
  goto error
elseif refline .gt. 1 then
        write/out " "
        mess/out 1 {PROID} "More than one tbl. with identified lines present !"
  goto error
endif
!
if refdico .eq. 0 then
        write/out " "
        mess/out 1 {PROID} "No matching table w. dispersion solution present !"
  goto error
elseif refdico .gt. 1 then
        write/out " "
        mess/out 1 {PROID} "More than one tbl. w. dispersion solution present !"
  goto error
endif
!
insdesc = "{{lampima},{h_instrume}}"
!
!
!          SELECTING USED SLITS IN TABLE
!          WITH POSITIONS OF ALL :
!          *****************************
!
@p fors_paragraph 2 1  ! -------- PROCESSING --------
!
        mess/out 4 {PROID} "Selecting used slits"
!
!***********************************************************
@p fors_defpositions {lampima} {slittblori} {FORS_INSMODE} +
!***********************************************************
if {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Selection of used slits failed !"
  goto error
endif
!
! COPY TABLE:
  char = m$indexb(lampima,".")-1
  slittbl = lampima(1:{char})//"_slits.tbl"
if slittblori .ne. slittbl -copy {slittblori} {slittbl}
!
!
!          DETERMINING OVERSCAN SECTIONS :
!          *****************************
!
!**************************
@p fors_get_imsec {lampima}
!**************************
if {q1} .ne. 0 then
        write/out " "
        mess/out 1 {PROID} "Determination of overscan sections failed !"
  goto error
endif
psc = outputi(5)
osc = outputi(6)
!
!
!          *********************************
!          *                               *
!          *  CALLING OF MAIN PROCEDURE :  *
!          *                               *
!          *********************************
!
slitcol = "{c_slsno},{c_slystart},{c_slyend}"
linecol = "{c_ltslit},{c_ltxpos},{c_ltypos},{c_ltwavec},{c_ltreject}"
dcoefdesc = "{h_dicolamb},{h_dicodpix}"
! ybin: char. key with 2 elements (separated by a comma) from xmos-context,
!       1st elem.: bin-to-bin stepsize, 2nd elem.: half binsize,
!       both perpendicular to dispersion direction
!
        mess/out 4 {PROID} "Applying procedure  "fors_resol" :"
!
if insmod .eq. "lss" then
  slwid = "lss," // h_slwidth
elseif insmod .eq. "mos" then
  slwid = "mos," // kbase_mosi // "," // ksuff_moswid
endif
!
!******************************************************************
@p fors_resol {lampima} {slittbl},{slitcol} {linetbl},{linecol} -
   {dcoeftbl},{dcoefdesc} {ybin} {slwid},{h_pixscale} p {psc},{osc}
!******************************************************************
!
if {q1} .eq. -1 then
  warn = 1
elseif {q1} .eq. 1 then
        write/out " "
        mess/out 1 {PROID} "Computation of spectral resolution failed !"
  goto error
endif
!
  char = m$indexb(lampima,".bdf")-1
  resoltbl = lampima(1:{char})//"_resol.tbl"
!
create/icat outprog.cat null >Null
add/icat outprog.cat {resoltbl} >Null
-delete dirfile.ascii
!
!
!          UPDATING DESCRIPTORS OF OUT- AND INPUT FRAMES :
!          *********************************************
!
@p fors_paragraph 3 1  ! -------- TERMINATION --------
!
        mess/out 4 {PROID} "Classifying output table"
        mess/out 4 {PROID} "({resoltbl})"
!
classify/product {resoltbl} {RESOLT_ID} {d_qcp} {d_det} OK
write/descriptor {resoltbl} {h_instrume}/c/1/60 {insdesc}
write/descriptor {resoltbl} {FORS_IDSTRNG}/c/1/60 {RESOLT_ID} !f.su.rec.
write/descriptor {resoltbl} {h_proqcsts}/c/1/9 "CHECKED"
write/dhelp {resoltbl} {h_proqcsts} "{i_proqcsts}"
longlog/out 1 "     Table with spectral resolutions: {resoltbl}"
!
        mess/out 4 {PROID} "Endorsing QC status in input image"
!
write/descriptor {lampima} {h_proqcsts}/c/1/9 "CHECKED"
write/dhelp {lampima} {h_proqcsts} "{i_proqcsts}"
!
shortlog/out 1 "Spectral resolutions computed."
!
!
!          UPDATING OF OUTPUT CATALOGUE (SUPER RECIPE MODE) ,
!          CONVERSION BDF/TBL->FITS/TFITS (SINGLE MODE) :
!          **************************************************
!
        mess/out 4 {PROID} "Releasing output table"
!
! Create product header:
finish/prohead outprog.cat 2
!
finish/pipeline outprog.cat {outpipe} xmos,spec {resoltbl}
!
finish/longlog {PROID}.cat outprog.cat {resoltbl} 2 {outpipe}
!
-delete {PROID}.cat
-delete outprog.cat
!
!
!          RESETTINGS AFTER RECIPE :
!          ***********************
!
set/format
!
log/recipe {PROID} out
!
@p fors_paragraph 4 1  ! ================
!
!
if warn .eq. 0 then
  return 0
else
  return -1
endif
!
error:
  set/format
  finish/longlog ? -1 ? 2
  write/out " "
  return 1
!
!
!-------------------------------------------------------------------------------
