! $Id: fors_s_gen_badpix.prg,v 1.15 2000/04/12 10:27:48 sbogun Exp $   
!+++ FORS_S_GEN_BADPIX +++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT       $RCSfile: fors_s_gen_badpix.prg,v $
!
!.KEYWORDS      SUPER RCIPE: bad pixel table and mask creation
!
!.USAGE         @p fors_s_gen_badpix sof_in out low high minnum noise expfact
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!                               list of names of screen flats
!               sof_ref (P2): set of reference frame (ascii list)
!               low     (P4): low level to identify bad pixels
!               high    (P5): high level to identify bad pixels
!               minnum  (P6): minimum number of bad pixels def. a bad column
!               noise   (P7): maximum allowed noise level
!               expfact (P8): expand factor to get best bad pixel areas
!
!.OUTPUT        out     (P3): path + basename of output files
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT       P3:     badpix  (output basename)
!               P4:     0.53    (low level)
!               P5:     1.89    (high level)
!               P6:     10      (minimum number)
!               P7:     0.02    (noise)
!               P8:     0.05    (expandfactor)
!
!.PURPOSE       Create a bad pixel mask from a set of flatfield measurements
!
!.ALGORITHM     Calls single recipes in the following order:
!               - look for overexposed pixels   (QC)
!               - determine bad pixel on CCD frame
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1998/12/21  Creation
!                   1999/03/10  shortlog close shifted to FINISH/PIPELINE
!                   1999/04/28  longlog implemented (SB).
!               1.3 1999/09/03  Product header generation implemented (SB).
!
!.COMMENT       SUPER-RECIPES:
!               All output frames will have the following naming-convention:
!
!                    {P3}_{n}.fits              while {n} counts starting at 0
!
!               They will be finally saved as soon as FinishPipeline will be
!               called at the end of the SUPER-RECIPE. Until then only
!               temporary BDF-versions of the outputs will exist in the local
!               directory. All outputs have to be protocolled adding these
!               BDF-files to the result catalog created by the first
!               InitPipeline call.
!
!---------------------------------------------------------------------------
!
define/param P1 ? c "Enter name of ASCII filelist with science frames:"
define/param P2 ? c "Enter name of ASCII filelist with reference frame:"
define/param P3 ? c "Enter basename of output frames:"
define/param P4 0.53 number     !low cut
define/param P5 1.89 number     !high cut
define/param P6 10   number     !min. number of pixels defining a bad col.
define/param P7 0.02 number     !max. noise
define/param P8 0.03 number     !expand factor

define/local part/i/1/1 0
define/local procname/c/1/25 "s_gen_badpix"
define/local line/c/1/79 "-" ALL
define/local badpixtab/c/1/80 " " ALL
define/local errmsg/c*80/1/6 " " ALL

errmsg(1) = "Error in INIT/PIPELINE occured !"
errmsg(2) = "Error in fors_gen_qc0_overexpo occured !"
errmsg(3) = "Error in GET/BADPIX occured !"
errmsg(4) = "Error while identifying main product !"
errmsg(5) = "Error in FINISH/PIPELINE !"
errmsg(6) = "No good frames left after overexposure check !"



LOG/REC {procname} IN SUPER

mess/out 3 {procname} "START creation of bad pixel table and mask ..."
@p fors_paragraph 1 0


part = 1
INIT/PIPELINE {P1} {P2} XSFinput.cat CCDRED + IMG XSFreject.cat XSFresult.cat
if {Q1} .gt. 0 goto ABORT
!
!
! now all input frames and reference frames are stored in the image catalog:
!    XSFinput.cat
! thus P3 may be ignored (+) in the following recipe-calls.
! (P3 usually refers to the set of reference frames)
!

init/prohead {p1} {p2} 1,"Badpixel mask and table generation" -
   "low_cut,{P4}" "high_cut,{P5}" "min_num,{P6}" "max_nois,{P7}" -
   "expfact,{P8}"
if {Q1} .gt. 0 goto ABORT

init/longlog {p1} {p2} 1 "low level: {P4}" "high level: {P5}" "minnum: {P6}" -
                      "noise: {P7}" "expfact: {P8}"
if {Q1} .gt. 0 goto ABORT


!------------------------------------------------------------------------------
@p fors_paragraph 2 0

part = 2
mess/out 3 {procname} "Check for overexposed pixels ..."
write/out {line}
@p fors_gen_qc0_overexpo XSFinput.cat + XSFinput.cat,XSFresult.cat
if {Q2} .eq. 0 mess/out 1 {procname} "{errmsg(6)}"
if {Q1} .gt. 0 goto ABORT

!------------------------------------------------------------------------------
!Create bad pixel mask and table
!
part = 3
write/out
mess/out 3 {procname} "Create bad pixel mask and table ..."
@p fors_gen_cal_badpix -
     XSFinput.cat + XSFinput.cat,XSFresult.cat {P4} {P5} {P6} {P7} {P8}
if {Q1} .ne. 0 goto ABORT


!------------------------------------------------------------------------------
!write FITS files to specified output directory
!
part = 4
@p fors_paragraph 3 0
search/icat XSFresult {BPIXT_ID} >Null
if m$len(out_a) .gt. 0 then
  badpixtab = out_a
else
  mess/out 1 {procname} "No result frame ({BPIXT_ID}) found !"
  goto ABORT
endif


part = 5
write/out
finish/prohead XSFresult.cat 1
FINISH/PIPELINE XSFresult.cat {P3} CCDRED {badpixtab}
if {Q1} .gt. 0 goto ABORT
finish/longlog XSFinput.cat XSFresult.cat {badpixtab} 1 {P3}


LOG/REC {procname} OUT SUPER
@p fors_paragraph 4 0

return 0


ABORT:
finish/longlog XSFinput.cat -1 ? 1 {P4}
mess/out 1 {procname} "{errmsg({part})}"
write/error -{part}
return -{part}
