! $Id: fors_s_img_stdstar.prg,v 1.53 2001/06/28 10:25:01 sbogun Exp $   
!+++ FORS_S_IMG_STDSTAR ++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         $RCSfile: fors_s_img_stdstar.prg,v $
!
!.KEYWORDS      SUPER RCIPE: Complete standard star frame pipline processing
!
!.USAGE         @p (IDENT) sof_in sof_ref out bias dark accur
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!                               name of standard star frame
!               sof_ref (P2): set of reference frames (ascii list)
!                               Badpixel-table, Bias, (Dark), Flat,
!                               Standard star reference catalog (table)
!                               Photometry Input parameter table (*)
!                               Photometry Output parameter table (*)
!                               Photometry Convolution mask (*)
!                               Photometry Neuronal network weights (*)
!                               (*): used by sextractor in photometry recipe.
!               bias    (P4):   Bias constant to be used instead of master.
!               dark    (P5):   Dark constant to be used instead of master.
!               scale   (P6): "MED": the bias correction will be done using the 
!                               scaled master bias. The scaling is done as
!                               follows: 
!                               Each of the sections of the master bias is 
!                               divided by its median and multiplied by the
!                               median of the corresponding pre-/overscan 
!                               region of the input frame.
!                             "OVE": like "MED", but each of the sections of the
!                               master bias is divided by the median of its
!                               corresponding pre-/overscan region and
!                               multiplied by the median of the pre-/overscan
!                               region of the input frame.
!                             "NO":  The master bias will not be scaled.
!               accur   (P7): Pointing accuarcy of the telescope.
!
!.OUTPUT        out     (P3): path + basename of output files:
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT       p4 0
!               p5 0
!               p6 OVE
!               p7 18.0
!
!.PURPOSE       Performs the whole pipeline processing of a standard star
!               frame.
!               It starts with two quality control procedures, overexposed
!               pixel checking and bias determination through the overscan
!               regions of the science frame.
!               Passing these quality control procedures, the science frame
!               gets reduced in the standard way (bad pixel correction and
!               bias-, dark-subtraction as well as flatfield division). Now
!               the standard star frame is prepared in order to locate the
!               objects and to align them with a reference standard star
!               catalog.
!               Finally the extinction coefficients may be determined
!               by a robust linear least squares fit to the difference
!               of magnitudes between observed and referenced standard star
!               luminosities. At the moment, this last step has to be done
!               BY HAND as the DO does not support linking of a series of
!               reduction blocks.
!
!.ALGORITHM     Calls single recipes in the following order:
!               - look for overexposed pixels   (QC)
!               - determine bias from overscan  (QC)
!               - reduce science frame
!               - extract sky and subtract from science frame
!               - find objects in the frame
!
!.ENVIRON       FORS, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.9 1998/07/09  Creation
!               1.0 1998/12/03  main product defined: ALIGNED PHOTOMETRY TABLE
!                               parameter handling implemented
!                   1998/12/08  paragraphs implemented
!                   1998/12/10  error handler implemented (write/error)
!                   1999/01/07  shortlog implemented.
!               1.2 1999/02/03  update write/error
!                   1999/03/10  shortlog close shifted to FINISH/PIPELINE
!               1.3 1999/04/28  longlog implemented (SB).
!                   1999/09/03  Product header generation implemented (SB).
!
!.COMMENT       SUPER-RECIPES:
!               All output frames will have the following naming-convention:
!
!                    {P3}_{n}.fits              while {n} counts starting at 0
!
!               They will be finally saved as soon as FinishPipeline will be
!               called at the end of the SUPER-RECIPE. Until then only
!               temporary BDF-versions of the outputs will exist in the local
!               directory. All outputs have to be protocolled adding these
!               BDF-files to the result catalog created by the first
!               InitPipeline call.
!---------------------------------------------------------------------------
define/param p1 ? c "Enter name of ASCII filelist with science frames:"
define/param p2 ? c "Enter name of ASCII filelist with reference frames:"
define/param p3 ? c "Enter name of output-basename (+path):"
define/param p4 0 n "Enter bias constant to be used instead of master:"
define/param p5 0 n "Enter dark constant to be used instead of master:"
define/param p6 OVE c "Use scaled master bias (MED/OVE/NO):"
define/param p7 18.0 n "Enter pointing accuarcy of telescope:"

define/local part/i/1/1         0
define/local procname/c/1/25    "s_img_stdstar"
define/local line/c/1/79        "-"             ALL
define/local aliphottab/c/1/80  " "             ALL
define/local errmsg/c*80/1/12   " "             ALL
define/local cat_entry/c/1/128  " "             ALL
define/local frame/c/1/128      " "             ALL
define/local catal/i/1/1        0
define/local ctrl/i/1/1         0
define/local filter/c/1/32      " "             ALL
define/local inst/c/1/32        " "             ALL
define/local filterlist/c/1/128    " "          ALL

! UBVRI set for FORS1:
define/local filterlist1/c/1/128   "U_BESS,B_BESS,V_BESS,R_BESS,I_BESS,"
! UBVRI set for FORS2:
define/local filterlist2/c/1/128   "U_SPECIAL,B_BESS,V_BESS,R_SPECIAL,I_BESS,"

errmsg(1) = "Error in INIT/PIPELINE occured !"
errmsg(2) = "Error in fors_gen_qc0_overexpo occured !"
errmsg(3) = "Error in fors_gen_qc1_statistics occured !"
errmsg(4) = "Error in CHECK/OVERSCAN occured !"
errmsg(5) = "Error in OVERSCAN/FRAMES occured !"
errmsg(6) = "Error in REDUCE/IMAGE occured !"
errmsg(7) = "Error in fors_img_obs_photometry occured !"
errmsg(8) = "Error in ALIGN/STANDARD occured !"
errmsg(9) = "Error while identifying main product !"
errmsg(10) = "Error in FINISH/PIPELINE !"
errmsg(11) = "No good frames left after overexposure check !"
errmsg(12) = "Filter check failed !"



LOG/REC {procname} IN SUPER

mess/out 3 {procname} "START standard star processing ..."
@p fors_paragraph 1 0


part = 1
INIT/PIPELINE {P1} {P2} XSFinput.cat CCDRED + IMG XSFreject.cat XSFresult.cat
if {Q1} .gt. 0 goto ABORT
!
!
!now all input frames and reference frames are stored in the image catalog:
!   XSFinput.cat
!thus P2 may be ignored (+) in the following recipe-calls.
!(P2 usually refers to the set of reference frames)
!

init/prohead {p1} {p2} 1,"Standard star image reduction" -
             "bias,{p4}" "dark,{p5}" "scale,{p6}" "pointacc,{p7}"
if {Q1} .gt. 0 goto ABORT

init/longlog {p1} {p2} 1 "bias: {p4}" "dark: {p5}" "scale: {p6}" -
             "pointing accuracy: {P7}"
if {Q1} .gt. 0 goto ABORT

@p fors_paragraph 2 0
!------------------------------------------------------------------------------
!Quit if filter is not U/B/V/R/I
part = 12
read_cat_entry:
  store/frame cat_entry XSFinput.cat 1 exit_loop
  out_a = m$value({cat_entry},{FORS_IDSTRNG})
  out_a = m$upper(out_a) 

  if out_a .eq. STD_ID then
    ctrl = 1
    frame = cat_entry
  endif

  goto read_cat_entry

exit_loop:
if ctrl .eq. 1 then
  if m$existd(frame,h_filti(1)) .eq. 1 then
    filter = "{{frame},{h_filti(1)}}" // ","
    inst = "{{frame},{h_instrume}}"
    if inst .eq. "FORS1" then
      filterlist = filterlist1
    else if inst .eq. "FORS2" then
      filterlist = filterlist2
    else
      mess/out 1 {procname} "Instrument is neither FORS1 nor FORS2!"
      goto ABORT
    endif
    outputi = m$index(filterlist,filter)
    if outputi(1) .eq. 0 then
      mess/out 1 {procname} "Filter is not in U/B/V/R/I. Stopping reduction!"
      goto ABORT
    endif
  else
    mess/out 1 {procname} "Filter is not in U/B/V/R/I. Stopping reduction!"
    goto ABORT
  endif
else
  mess/out 1 {procname} "No standard star image present in frame list!"
  goto ABORT
endif

!------------------------------------------------------------------------------
!Check overexposed pixels
!
part = 2
write/out
mess/out 3 {procname} "Check for overexposed pixels ..."
write/out {line}
@p fors_gen_qc0_overexpo XSFinput.cat + XSFinput.cat,XSFresult.cat
if {Q2} .eq. 0 mess/out 1 {procname} "{errmsg(11)}"
if {Q1} .gt. 0 goto ABORT

!------------------------------------------------------------------------------
!Perform statistic tests
!
part = 3
write/out
mess/out 3 {procname} "Perform statistic tests (img_qc0_statistics)..."
write/out {line}
@p fors_img_qc0_statistics XSFinput.cat + XSFinput.cat,XSFresult.cat {p6}
if {Q1} .gt. 0 goto ABORT
!!
!------------------------------------------------------------------------------
!determine overscan
!
!! part = 4
!! write/out
!! !mess/out 2 {procname} "OVERSCAN: disabled"
!! mess/out 3 {procname} "Determine bias from overscan ..."
!! check/overscan XSFinput.cat + XSFinput.cat,XSFresult.cat
!! if {Q1} .gt. 0 goto ABORT
!!
!------------------------------------------------------------------------------
!do the reduction
!
part = 6
write/out
mess/out 3 {procname} "Reduce standard star frame ..."
reduce/image XSFinput.cat + XSFinput.cat,XSFresult.cat {P4} {P5} 10 {P6}
if {Q1} .ne. 0 goto ABORT

!------------------------------------------------------------------------------
!do the photometry
!
part = 7
write/out
mess/out 3 {procname} "Determine photometric values ..."
write/out {line}
@p fors_img_obs_photometry XSFinput.cat + XSFinput.cat,XSFresult.cat
if {Q1} .gt. 0 goto ABORT


!------------------------------------------------------------------------------
!standard star alignment
!
part = 8
write/out
mess/out 3 {procname} "Standard star catalog alignment ..."
align/standard XSFinput.cat + XSFinput.cat,XSFresult.cat {P7}
if {Q1} .ne. 0 goto ABORT
!
if nice_crea .eq. "y" then
  assign/display d,3 >Null  ! image still loaded from photometry recipe
  clear/channel overlay
  label/display "FOUND STANDARD STARS" {nice_dlab}
  draw/circle stdali.tbl p f green >Null
endif

!------------------------------------------------------------------------------
!write FITS files to specified output directory
!
part = 9
@p fors_paragraph 3 0
search/icat XSFresult {PHOTTALI_ID} >Null
if m$len(out_a) .gt. 0 then
  aliphottab = out_a
  show/table {aliphottab} >Null
  if outputi(2) .gt. 0 .AND. nice_crea .eq. "y" -
    draw/elli {aliphottab} P N BLUE,0 >Null
  append/out "Standard star processing has finished so far!\n"
  append/out "The determination of the EXTINCTION-COEFFICIENTS\n"
  append/out "has to be done manually using the recipe:\nfors_img_cal_calcoeff."
  mess/out 3 {procname}
else
  mess/out 1 {procname} "No result frame ({PHOTTALI_ID}) found !"
  goto ABORT
endif


part = 10
finish/prohead XSFresult.cat 1
FINISH/PIPELINE XSFresult.cat {P3} CCDRED  {aliphottab}
if {Q1} .gt. 0 goto ABORT
finish/longlog XSFinput.cat XSFresult.cat {aliphottab} 1 {p3}


LOG/REC {procname} OUT SUPER
@p fors_paragraph 4 0
return 0


ABORT:
finish/longlog XSFinput.cat -1 ? 1 {p3}
mess/out 1 {procname} "{errmsg({part})}"
write/error {part},APP,NO "{errmsg({part})}"
