dnl ------------------------------------------------------------------------
dnl Get host type dependent compiler configuration for sextractor
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(SEXTRACTOR_CHECK_COMPILER,
[AC_ARG_WITH(machine,
[  --with-machine=MACHINE  configure sextractor for building on MACHINE],
sext_host="$withval", sext_host=auto)

AC_REQUIRE([AC_CANONICAL_HOST])dnl
AC_REQUIRE([AC_PROG_CC])

AC_MSG_CHECKING([for sextractor target host configuration])
AC_CACHE_VAL(ac_cv_host_sextractor,
[
if test "$sext_host" = auto; then
  ac_sext_host=""
  case "${host}" in
    *ibm-aix*)    ac_sext_host="ibm";;
    *dec-osf*)    ac_sext_host="alpha";;
    *dec-ultrix*) ac_sext_host="ultrix";;
    *hp-hpux*)    ac_sext_host="hpux";;
    *solaris*)    ac_sext_host="solaris";;
    *sunos*)      ac_sext_host="sunos";;
    *sgi-irix*)   ac_sext_host="sgi";;
    *linux-gnu*)  ac_sext_host="linuxpc";;
    *)
      AC_MSG_ERROR([Unsupported host type $host])
      ;;
  esac
else
  ac_sext_host=$sext_host
fi

ac_cv_host_sextractor="ac_sext_host=$ac_sext_host"
])

eval "$ac_cv_host_sextractor"
AC_MSG_RESULT($ac_sext_host)

dnl Check extra compiler flags and add them to CFLAGS
case "${ac_sext_host}" in
  ibm)
    CPPFLAGS="$CPPFLAGS -DIBM_AIX"
    if test "$GCC" != yes; then
      AC_CHECK_C_COMPILER_FLAG(O, [CFLAGS="$CFLAGS -O"])
    fi
    ;;
  alpha)
    CPPFLAGS="$CPPFLAGS -DDEC_ALPHA"
    if test "$GCC" != yes; then
      AC_CHECK_C_COMPILER_FLAG(O, [CFLAGS="$CFLAGS -O"])
    fi
    ;;
  ultrix)
    CPPFLAGS="$CPPFLAGS -DBSWAP"
    if test "$GCC" != yes; then
      AC_CHECK_C_COMPILER_FLAG(O, [CFLAGS="$CFLAGS -O"])
    fi
    ;;
  hpux)
    if test "$GCC" != yes; then
      CPPFLAGS="$CPPFLAGS -DHP_UX"
    fi
    ;;
  solaris)
    CPPFLAGS="$CPPFLAGS -DSUN_SOLARIS"
    if test "$GCC" != yes; then
      AC_CHECK_C_COMPILER_FLAG(fast, [CFLAGS="$CFLAGS -fast"])
      AC_CHECK_C_COMPILER_FLAG(xO5, [CFLAGS="$CFLAGS -xO5"])
    fi
    ;;
  sunos)
    CPPFLAGS="$CPPFLAGS -DSUN_OS"
    ;;
  sgi)
    ;;
  linuxpc)
    CPPFLAGS="$CPPFLAGS -DPC_LINUX"
    if test "$GCC" = yes; then
      AC_CHECK_C_COMPILER_FLAG(funroll-loops, [CFLAGS="$CFLAGS -funroll-loops"])
    fi
    ;;
  *)
    if test "$sext_host" != auto; then
      AC_MSG_ERROR([Unsupported host type $sext_host])
    fi
    ;;
esac

SEXMACHINE="$ac_sext_host"
AC_SUBST(SEXMACHINE)
])

dnl ------------------------------------------------------------------------
dnl Check if the C compiler supports the specified flag
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(AC_CHECK_C_COMPILER_FLAG,
[
AC_REQUIRE([AC_PROG_CC])
AC_MSG_CHECKING(whether $CC supports -$1)
flag_name=`echo $1 | sed 'y%.=/+-%___p_%'`
AC_CACHE_VAL(ac_cv_prog_cc_$flag_name,
[
echo 'int main() { return 0; }' >conftest.c
eval "ac_cv_prog_cc_$flag_name=no"
if test -z "`$CC -$1 -c conftest.c 2>&1`"; then
  if test -z "`$CC -$1 -o conftest conftest.o 2>&1`"; then
    eval "ac_cv_prog_cc_$flag_name=yes"
  fi
fi
rm -f conftest*
])
if eval "test \"`echo '$ac_cv_prog_cc_'$flag_name`\" = yes"; then
 AC_MSG_RESULT(yes)
 :
 $2
else
 AC_MSG_RESULT(no)
 :
 $3
fi
])

dnl ------------------------------------------------------------------------
dnl Check for ar
dnl ------------------------------------------------------------------------
dnl
AC_DEFUN(AC_PROG_AR,
[
AC_PROVIDE([AC_PROG_AR])

AC_CHECK_PROG(AR, ar, ar, NONE)
if test "$AR" = NONE; then
  AC_MSG_ERROR([Cannot find \'ar\'])
fi
])

