! $Id: mosextinc.prg,v 1.5 1999/11/24 09:48:45 sbogun Exp $ (ESO-DMO)  
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.COPYRIGHT   (C) 1998 European Southern Observatory
!
!.IDENT       $RCSfile: mosextinc.prg,v $
!
!.AUTHORS     Pascal Ballester (ESO/Garching) 
!             Cristian Levin   (ESO/La Silla)
!
!.KEYWORDS    Spectroscopy, Long-Slit, MOS
!
!.USAGE       EXTINCTION/MOS input output exttab [column] [scale]
!
!.INPUT       input     Image to be correct for extinction
!             exttab    Table with the wavelength dependence of
!                       reddening or the extinction
!             column    Column reference (label) for the extinction table
!             scale     scaling factor:
!                         a) E(B-V) if interstellar reddening     
!                         b) AIRMASS if atmospheric extinction
!                       
!.OUTPUT                
!             output    image to hold the corrected data
!
!.PURPOSE     Generates the extinction spectrum from the extinction
!             table and the airmass.
!             
!.VERSION     1.0  Creation  18-Nov-1998
!-------------------------------------------------------

crossref input output exttab column scale

define/param p1  ?         i    "Enter input  image:"
define/param p2  ?         c    "Enter output image:"
define/param p3  {atmext}  tab  "Extinction table:"
define/param p4  :EXTCOEF  c    "Column label:"
define/param p5  +         c    "Scale factor:"
define/maxpar 5


! Local constants
define/local tmp0/c/1/20  "ext_tmp0.bdf"
define/local tmp1/c/1/20  "ext_tmp1.bdf"



! Local variables
define/local scale/r/1/1      1.
define/local rawfrm/c/1/100   {p1}
define/local calfrm/c/1/100   {p2}
define/local column/c/1/20    " "   all



! Process parameters
set/mos atmext={p3}
if m$index(p4,":") .lt. 1 then
  write/out "Error: Invalid column label {p4}!"
  return 1
else
  column = "{p4}"
endif

! If not user supplied use default value
if m$tstno(p5) .eq. 0 then
  write/out "Warning: Using default airmass value {scale}.
else
  scale = {p5}
endif  


! Create image from extinction coefficients table
create/image {tmp0} 1,{{rawfrm},npix(1)} {{rawfrm},start(1)},{{rawfrm},step(1)}
convert/table {tmp1} = {atmext} #1 {column} {tmp0} spline


! Expand image if necessary
if {{rawfrm},naxis} .gt. 1 then
  grow/image {tmp0} = {tmp1} -
             {{rawfrm},start(2)},{{rawfrm},step(2)},{{rawfrm},npix(2)}
  -rename {tmp0} {tmp1}
endif

! Do the job
compute/dimage {calfrm} = {rawfrm} * exp10(0.4 * {tmp1} * {scale})   
copy/dd {rawfrm} *,3 {calfrm}

! Cleanup
delete/image {tmp0} no
delete/image {tmp1} no

return 0
