! $Id: mosnflat.prg,v 1.3 1999/07/12 13:30:27 rpalsa Exp $   
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       $RCSfile: mosnflat.prg,v $
!
!.KEYWORDS    Flat field normalization
!
!.USAGE       normalize/flat  flat slits nflat method ffwin fford chat
!
!.INPUT       flat  	  (P1)   Name of flat field frame
!             slits  	  (P2)   Name of slit position table
!             method      (P4)   List of smoothing methods.
!             ffwin       (P5)   Window width for smoothing along Y axis.
!             fford       (P6)   Window width for smoothing along X axis or
!                                Order of polynomial fit.
!             chat        (P7)   Level of output.
!                                (0/1: output disabled/enabled)
!
!.OUTPUT      nflat       (P3)   Name of normalized flat field frame.
!             cat_reject  (P4)   Catalog name containing rejected frames
!
!.RETURNS     The recipe returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status.
!
!.PURPOSE     The procedure normalizes the input flat field frame within the
!   	      slit boundaries using the specified method. The method is given
!             by a comma separated list of smoothing methods for the Y axis
!             and the X axis. For the Y axis the methods AVERAGE, SMOOTH and
!             MEDIAN are available and for the X axis the possible methods are
!             POLYNOM, SMOOTH and MEDIAN. The combination MEDIAN or SMOOTH
!             along the Y axis together with POLYNOM for the X direction is
!             not supported!.
!             Additional output may be enabled by setting CHAT to a number 
!             other than zero. The default is no output, i.e. CHAT = 0.
!
!.ENVIRON     XMOS context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1999/01/04:  Creation
!
!------------------------------------------------------------------------------

crossref flat slits nflat method ffwin fford chat

define/param p1 {flat}     image   "Enter input flat field:"
define/param p2 {mos}      table   "Enter slit position table:"
define/param p3 {normflat} image   "Enter normalized flat field:"
define/param p4 {normmet}  ?       "Smoothing method:"
define/param p5 {fford}    number  "Enter smoothing parameters:"
define/param p6 {ffwin}    number  "Averaging window width:"
define/param p7 0          number  "Enter debug level:"

define/local ffrm/c/1/100  {p1}
define/local stbl/c/1/100  {p2}
define/local nfrm/c/1/100  {p3}
define/local meth/c/1/20   {p4}
define/local ffpar/i/1/2   {p5},{p6}
define/local chat/i/1/1    {p7}

run PIPE_EXE:mosnflat
if {q1} .ne. 0 then
  return 1
endif

return 0
