! $Id: mosrspfit.prg,v 1.5 1999/11/24 09:48:54 sbogun Exp $ (ESO-DMD)  
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++
!.COPYRIGHT   (C) 1999 European Southern Observatory
!.IDENT       $RCSfile: mosrspfit.prg,v $
!.AUTHORS     Pascal Ballester (ESO/Garching) 
!             Cristian Levin   (ESO/La Silla)
!.KEYWORDS    Spectroscopy, Long-Slit, MOS
!.PURPOSE
!               Fitting for generating the response curve.
!               Command: RESPONSE/LONG
!.VERSION     1.0  Package Creation  17-MAR-1993
!             1.1  Adapted for MOS package 12-JAN-1999
!-------------------------------------------------------

crossref table image mode ftype fpar reference

define/param p1 {resptab}   c "Enter input response table name:"
define/param p2 {response}  c "Enter output response image name:"
define/param p3 RATIO       c "Enter response type (RATIO,COLOR):"
define/param p4 {rspftyp}   c "Enter fit type:"
define/param p5 {rspfpar}   c "Enter order of fit and smoothing factor:"
define/param p6 +           c "Enter reference frame:"
define/maxpar 6

define/local pos/i/1/1      0
define/local order/i/1/1    0
define/local nwaves/i/1/1   0
define/local smooth/r/1/1   0.
define/local wstart/r/1/1   0.
define/local wstep/r/1/1    0.
define/local mode/c/1/1     " "
define/local type/c/1/1     " "
define/local column/c/1/20  " "



! Process parameters
verify/mos {p1} tab
pos = m$index(p1,".tbl")
if pos .le. 0 then
  resptab = "{p1}.tbl"
else
  resptab = "{p1}"
endif

pos = m$index(p2,".bdf")
if pos .le. 0 then
  response = "{p2}.bdf"
else
  response = "{p2}"
endif

mode = m$upper(p3(1:1))
if mode .eq. "C" then
  column = ":COLOUR"
else
  column = ":RATIO"
endif

type = m$upper(p4(1:1))

if type .eq. "P" then
  if m$tstno(p5) .eq. 1 then
    order = {p5}
    smooth = 0.
  else
    write/out "Error: Invalid fit degree!"
    return 1
  endif
else if type .eq. "S" then
  pos = m$index(p5,",") - 1
  if pos .lt. 0 then
    write/out "Error: Invalid form of fit parameter string!"
    return 1
  else
    order = {p5(1:{pos})}
    pos = pos + 2
    smooth = {p5({pos}:)}
  endif
else
  write/out "Error: Unknown fit type requested!"
  return 1
endif


! Create reference image
if p6(1:3) .eq. "+" then
  wstart = m$value({resptab},rsprange(1))
  wstep = m$value({resptab},rsprange(2))
  nwaves = m$value({resptab},rsprange(3))
else
  wstart = m$value({p6},start(1))
  wstep = m$value({p6},step(1))
  nwaves = m$value({p6},npix(1))
endif

create/image &f 1,{nwaves} {wstart},{wstep}

! Fit response
if type .eq. "P" then
  convert/table &r = {resptab} :wave {column} &f poly {order}
else
  convert/table &r = {resptab} :wave {column} &f spline {smooth},{order}
endif

if mode .eq. "C" then
  compute/image {response} = 10.**(-0.4 * &r)
else
 -copy middummr.bdf {response}
endif


! Update descriptor CUNIT
{response},cunit(1:16) = "RATIO"
{response},cunit(17:32) = "ANGSTROM"
{response},cunit(33:48) = " "


! Cleanup
-delete middummf.bdf
-delete middummr.bdf

return 0
