/* $Id: mossky.c,v 1.6 2000/06/27 10:38:12 nrainer Exp $ (ESO-DMD)   */
/* fit sky with median or polynomial                 */
/*                   Otmar Stahl, Nov.28, 1994       */


#include <tbldef.h>
#include <midas_def.h>
#include <ok.h>
#include <math.h>

#include <proto_fors.h>

#define MEDIAN 0
#define POLYNOMIAL 1




#ifdef __STDC__
void sky_slit (float inframe[], float outframe[], int npix[], int order, 
	    int position[], int npos, int lower, int upper, double dstart[], 
	    double dstep[], float cosmic[], int method);
void copy_frame(float[],float[],int);
void median_sky(float *xin, float *yin,int nin, float *xout, float *yout, 
		int nout,int winsize);
#else
void sky_slit ();
void copy_frame();
void median_sky();
#endif

int debug = 0;

main ()

{
  int parm[5],method;
  int i, iav, order;
  int nax, kun, knul, tid, imno1, imno2;
  int col, ncol, nslits, acol, arow, nsort, select;
  int npix[2], null[3], slit[100], upper[100], lower[100];
  int sky_upper, sky_lower;
  int icol[3],ii,npos,k;
  
  char  inframe[60], mos_table[60], win_table[60], outframe[60], mode[11];
  float *inimage, *outimage, cosmic[3];
  char unit[64], inst[72], line[80];

  float moscol[3];

  int *position;

  double dstart[2], dstep[2];

  /*         get into Midas and get parameters             */

  SCSPRO ("mossky");

  strcpy(unit,""), strcpy(inst,"");

  SCKGETC ("IN_A", 1, 60, &iav, inframe);
  SCKGETC ("IN_B", 1, 60, &iav, mos_table);
  SCKGETC ("IN_C", 1, 60, &iav, win_table);
  SCKGETC ("OUT_A", 1, 60, &iav, outframe);
  SCKGETC ("INPUTC", 1 , 10, &iav, mode);
  SCKRDI ("INPUTI", 1, 4, &iav, parm, &kun, &knul);
  SCKRDR ("INPUTR", 1, 3, &iav, cosmic, &kun, &knul);

  order = parm[0];
  
  SCTPUT ("\n ----------------------- ");
  sprintf (line, "Input image:         %s ", inframe);
  SCTPUT (line);
  sprintf (line, "Input table:         %s ", win_table);
  SCTPUT (line);
  sprintf (line, "Output image:        %s\n ", outframe);
  SCTPUT (line);
  SCTPUT ("input parameters:\n ");

  method = MEDIAN;
  sprintf(line,"Fitting method: median");

  if (!strncmp(mode,"POL",3) || !strncmp(mode,"pol",3))
    {
      method = POLYNOMIAL;
      sprintf(line,"Fitting method: polynomial");
    }	

  SCTPUT (line);
  sprintf (line, "order of fit:        %i", order);
  SCTPUT (line);

/*          map input file                              */

  SCIGET (inframe, D_R4_FORMAT, F_I_MODE, F_IMA_TYPE,
	  2, &nax, npix, dstart, dstep, inst, unit, (char **)&inimage, &imno1);

  SCIPUT (outframe, D_R4_FORMAT, F_O_MODE, F_IMA_TYPE,
	  nax, npix, dstart, dstep, inst, unit, (char **)&outimage, &imno2);

  copy_frame(inimage,outimage,npix[0]*npix[1]);

  position = ivector(0,npix[1]);

/*-------read table MOS------------------------------------------------*/

  TCTOPN (mos_table, F_I_MODE, &tid);
  TCIGET (tid, &col, &ncol, &nsort, &acol, &arow);
 
  TCCSER (tid, ":slit", &icol[0]);
  TCCSER (tid, ":ystart", &icol[1]);
  TCCSER (tid, ":yend", &icol[2]);
  
/* 1.step-----------read table MOS------------------------------------*/
  /*copy selected slitlets to slit[0,..,nslits-1] , */
  nslits = 0;
  for (i = 1; i <= ncol; i++)
    {
      TCSGET(tid, i, &select);
      if (select) {
        TCRRDR (tid, i, 3, icol, moscol, null);
        slit[nslits] = (int) moscol[0];
        lower[nslits] =  NINT((moscol[1]-dstart[1])/dstep[1]);
        upper[nslits] =  NINT((moscol[2]-dstart[1])/dstep[1]);
	if (debug > 2) {
	  sprintf (line, "lower = %d upper = %d \n",
		   lower[nslits],upper[nslits]); 
	  SCTPUT(line);
	}
        nslits++;
      }
    }

  TCTCLO (tid);

/*           read table WINDOWS                           */

  TCTOPN (win_table, F_I_MODE, &tid);
  TCIGET (tid, &col, &ncol, &nsort, &acol, &arow);
 
  TCCSER (tid, ":Sky_Slit", &icol[0]);
  TCCSER (tid, ":Sky_Strt", &icol[1]);
  TCCSER (tid, ":Sky_End", &icol[2]);

/* 2.step-----------loop through slitlets, search sky windows-------------*/

  SCTPUT ("\n ----------------------- ");
  SCTPUT (" slit: rows for sky:");

  /* for all slitlets*/
  for (ii = 0; ii < nslits; ii++) {
    npos = 0;
    /*for all selected skypositions*/
    for (i = 1; i <= ncol; i++) {
      TCSGET(tid, i, &select);
      if (select) {
	TCRRDR (tid, i, 3, icol, moscol, null);
	if ((int) (moscol[0]) == slit[ii] &&  ! null[0] ) {
	  sky_lower =  NINT((moscol[1]-dstart[1])/dstep[1]);
	  sky_upper =  NINT((moscol[2]-dstart[1])/dstep[1]);
	  if (sky_lower < lower[ii]) sky_lower = lower[ii];
	  if (sky_upper > upper[ii]) sky_upper = upper[ii];
	  
	  /* 3.step----------produce array of sky positions----------------*/
	  for (k=sky_lower; k<=sky_upper;k++) {
	    position[npos] =  k;
	    if (debug > 2) {
	      sprintf (line, "%d\n",position[npos]); 
	      SCTPUT(line);
	    }
	    npos++;
	  }
	}
      }
    }

    sprintf(line, "   %4d    %4d \n",slit[ii],npos);
    SCTPUT(line);

    sky_slit (inimage, outimage, npix, order, position, npos, lower[ii],
	   upper[ii], dstart, dstep, cosmic, method);
    
    
  }

/*          epilogue ...                 */


  TCTCLO (tid);


  SCSEPI ();
  return (1);
}


#ifdef __STDC__
void
sky_slit (float inframe[], float outframe[], int npix[], int order, 
       int position[], int npos, int lower, int upper, double dstart[], 
       double dstep[], float cosmic[], int method)
#else
void
sky_slit (inframe, outframe, npix, order, position, npos, lower,upper,dstart,
       dstep,cosmic, method)
  float inframe[],outframe[],cosmic[];
  int npix[],order,npos,lower,upper,method,position[];
  double dstart[], dstep[];
#endif
     
{
  int i, j, k, jj,nsel;

  char line[80];
  float med,limit;
  double *A;
  /* new - dlfit not lfit*/
  float *tmp, *yin, *yout, *xin, *xout, *xbuff;

  if (debug > 1)
    SCTPUT("sky_slit");

  /*      arrays for intermediate data                                */

  A = dvector(1, order);
  
  tmp = vector(0,(upper-lower));
  yin = vector(0,(upper-lower));
  xin = vector(0,(upper-lower));
  yout = vector(0,(upper-lower));
  xout = vector(0,(upper-lower));

  for (j = lower; j <= upper; j++) 
    xout[j-lower] = (float) j;

  /*for all pixel - x-axis*/
  for (k = 0; k < npix[0]; k++) 
    {
      /*for all pixel within sky-position --  y-axis */
      for (j = 0; j < npos; j++)
	{
	  jj = position[j] * npix[0] + k;
	  yin[j] = inframe[jj];
	  xin[j] = (float) position[j];
	  if (debug > 3) {
	    sprintf (line, "xin = %7.1f yin = %7.1f jj = %4d position %4d\n",
		     xin[j],yin[j],jj,position[j]); 
	    SCTPUT(line);
	  }
	}


      if (method == POLYNOMIAL)
	{
	  /* cosmic cleaning & polynom fit */
	  if ( (int) cosmic[2] > 0)
	    {
	      
	      med = heap_median(npos,yin);
	      
	      limit = cosmic[2] * (float) sqrt( fabs(med/cosmic[1] + 
						  SQR(cosmic[0]/cosmic[1])));
	      
	      nsel = 0;
	      for (j = 0; j < npos; j++)
		{
		  if (yin[j] < med + limit && yin[j] > med - limit) 
		    {
		      tmp[nsel] = yin[j];
		      xin[nsel] = xin[j];  
		      nsel++;
		    }
		}
	  
	      /*              polynomial fit - after cleaning */
	      if (nsel > order)
		fit_poly(tmp, xin, nsel, yout, xout, ((upper-lower)+1), 
			 order, A);
	      else if (nsel > 1)
		{
		  med = heap_median(npos,yin);
		  for (j = 0; j < ((upper-lower)+1); j++)
		    yout[j] = med;
		}
	    }
	  else
	    {
	      xbuff = fvector(0,(upper-lower));
	      for (j = 0; j < npos; j++)
		xbuff[j] = (float) position[j];
	      /*              polynomial fit - no clean function */
	      fit_poly(yin, xbuff, npos, yout, xout, ((upper-lower)+1),
		       order, A);
	      free_fvector(xbuff, 0, (upper-lower));
	    }
	}
      else
	{
	  xbuff = fvector(0,(upper-lower));
	  for (j = 0; j < npos; j++)
	    xbuff[j] = (float) position[j];
	  median_sky(xbuff,yin,npos,xout,yout,(upper-lower)+1,order);
	  free_fvector(xbuff, 0, (upper-lower));
	  /*
	  med = heap_median(npos,yin);
	  for (j = 0; j < ((upper-lower)+1); j++)
	    yout[j] = med;
	  */
	}

      /*                  prepare output frame           */
      for (j = lower; j <= upper; j++)
	{
	  jj = j  * npix[0] + k;
	  outframe[jj] = yout[j-lower];
	}
    }

  free_vector(tmp, 0,(upper-lower));
  free_vector(yin, 0,(upper-lower));
  free_vector(xin, 0,(upper-lower));
  free_vector(yout,0,(upper-lower));
  free_vector(xout,0,(upper-lower));

  free_dvector(A,1,order);
}


#ifdef __STDC__
void copy_frame(float inframe[], float outframe[], int npix)
#else
void copy_frame(inframe, outframe, npix)
float inframe[],outframe[];
int npix;
#endif
{
int j;

  /* init outframe to inframe                                */

  for (j = 0; j < npix; j++)
    {
	  outframe[j] = inframe[j];
    }
}


/* linear interpolation */
#ifdef __STDC__
int     
interpol_lin(float x1[], float y1[], int n1, float x2[], float y2[], int n2)
#else
int     
interpol_lin(x1, y1, n1, x2, y2, n2)
int n1,n2;
float x1[],y1[],x2[],y2[];
#endif
{
  int i,j=0;
  float *buff;


  for (i = 0 ; i < n2 ; i++)
    {
      if (NINT(x1[j] - x2[i]) == 0)
	{
	  y2[i] = y1[j];
	  j++;
	}
      else if (j > 0 && x1[j] > x2[i])
	{
	  y2[i] = (y1[j-1] * (x2[i] - x1[j-1]) + y1[j] * (x1[j] - x2[i])) /
	    ((x2[i] - x1[j-1]) + (x1[j] - x2[i]));
	}
      else
	{
	  y2[i] = y1[0];
	}
    }


  return 0;
}


#ifdef __STDC__
void
median_sky(float *xin, float *yin,int nin, float *xout, float *yout, int nout,
	   int winsize)
#else
void 
median_sky(xin,yin,nin,xout,yout,nout,winsize)
int nin, nout, winsize;
float *xin,*yin,*xout,*yout;
#endif
{
  int i, rad;
  float med,*buff;

  buff = fvector(0,nin);

  if (nin <= winsize)
    {
      med = heap_median(nin,yin);
      for (i = 0; i < nout; i++)
	yout[i] = med;
      return;
    }

  rad = (int) (winsize / 2);
  winsize = 2 * rad + 1;

  med = heap_median(winsize,yin);
  for( i = 0 ; i < rad; i++)
    {
      buff[i] = med;
    }
  med = heap_median(winsize,&yin[nin - (2*rad)]);
  for( i = (nin - rad) - 1; i < nin ; i++)
    {
      buff[i] = med;
    }
  for (i = rad ; i < (nin - rad) - 1 ; i++)
    {
      buff[i] = heap_median(winsize,&yin[i - rad]);
    }

  interpol_lin(xin,buff,nin,xout,yout,nout);

  /*
  for (i = 0 ; i < nin ; i++)
    printf("%4.0f %4.0f %4.0f %4.0f %d\n",xin[i],buff[i],xout[i],yout[i],i);
  */

  free_fvector(buff,0,nin);

  return;
}


